; This code was written by Semantic Microsystems, Inc.
; which has placed it in the public domain.

; Pre-lease version of the file ":Examples:fonts.sch"
; from MacScheme+Toolsmith


; Simple font hacking.
;
; Suppose w is a text edit window created by the Toolsmith, f is a
; font number of a font (see chap7.data), and n is a font size.  Then:
;
;     (set-font w f) will change the window to use font f
;
;     (set-fontsize w n) will change the window to use font size n
;
; Example:
;
;    (let ((w (make-window 'text)))
;      (set-font w monaco)
;      (set-fontsize w 9))
;
; If this code doesn't work for you, check to make sure that the font
; you want, in the size you want, has been installed in the System
; file you are using.

(define chicago 0)
(define geneva 3)
(define monaco 4)

(mac-type FontInfo
  (record
    (ascent integer)
    (descent integer)
    (widMax integer)
    (leading integer)))

(define textfont (make-trap "a887" 1))
(define textsize (make-trap "a88a" 1))

(define getfontinfo (make-trap "a88b" 1))

(define (set-font w font)
  (call-without-interrupts
   (lambda ()
     (if (not (w 'closed?))
         (let ((texth ((w 'editor) 'texthandle)))
           (if texth
               (let ((a (w 'editor 'selstart))
                     (b (w 'editor 'selend)))
                 (poke.handle.word texth
                                   (%offset TERec txFont)
                                   font)
                 ((w 'editor 'set-textstring)
                  (w 'editor 'textstring))
                 (w 'editor 'set-selection a b)
                 (w 'editor 'scroller 'show (w 'editor) a))))))))

(define (set-fontsize w n)
  (call-without-interrupts
   (lambda ()
     (if (not (w 'closed?))
         (let ((info (newptr (%sizeof FontInfo)))
               (texth ((w 'editor) 'texthandle))
               (a (w 'editor 'selstart))
               (b (w 'editor 'selend)))
           (if texth
               (begin
                (poke.handle.word texth (%offset TERec txSize) n)
                (SetPort (w 'windowptr))
                (textfont (peek.handle.word texth
                                            (%offset TERec txFont)))
                (textsize n)
                (GetFontInfo info)
                (poke.handle.word texth
                                  (%offset TERec lineHeight)
                                  (+ (peek.ptr.word
                                      info (%offset FontInfo ascent))
                                     (peek.ptr.word
                                      info (%offset FontInfo descent))
                                     (peek.ptr.word
                                      info (%offset FontInfo leading))))
                (poke.handle.word texth
                                  (%offset TERec fontAscent)
                                  (peek.ptr.word
                                   info (%offset FontInfo ascent)))
                ((w 'editor 'set-textstring)
                 (w 'editor 'textstring))
                (w 'editor 'set-selection a b)
                (w 'editor 'scroller 'show (w 'editor) a)))
           (disposptr info))))))
