{------------------------------------------------------------------------------
FILE Color Life.p
	Copyright Steve Sheets, 1987
	All rights reserved.

NAME
	Color Life

DESCRIPTION
	A small Mac // Color Application written by Steve Sheets.
	It plays the original Life game. It also demonstrates some of the
    new commands of Color Quickdraw.

------------------------------------------------------------------------------}

PROGRAM ColorLife;

USES	Memtypes, Quickdraw, OSIntf, ToolIntf, PackIntf, PickerIntf;

{Constants: various Resource IDs, File Type and Creator Signitures,
	Maximum recorded Age of Cell, Horizontal/Vertical size of Dish
	(containing cells), Size of cells, Size of other windows and other
	size information.}

CONST	appleID		=	256;
		fileID		= 	257;
		editID		=	258;
		actionID	=	259;
		showID		=	260;
		popupID		=	261;
		
		strID		=	256;
		
		sorryID		=	256;
		
		aboutID		=	256;
		colorID		=	257;
		
		FileType	=	'life';
		CreatorType	=	'life';

		MaxAge		=	7;
		
		VEdge		=	50;
		HEdge		=	70;
		Big			=	8;
		VLife		=	Big*VEdge;
		HLife		=	Big*HEdge;
		
		VQuick		=	300;
		HQuick		=	450;
		QOff		=	10;
		
		colorSize	=	300;
		
		StepSize	=	30;
		HStep		=	20;
		VStep		=	10;
		HAbout		=	StepSize*HStep;	
		VAbout		=	StepSize*VStep;
	
{Types: Age range, Dish info (containing all cells & age), Dish Colors
	(containing RGB colors), Dish Color Patterns (RGB info converted
    to a Color Pattern).}

TYPE	Age			=	0..MaxAge;

		Dish			=	PACKED ARRAY[0..HEdge+1,0..VEdge+1] OF Age;
		
		DishColor	=	PACKED ARRAY[0..MaxAge] of RGBColor;
		DishPixMap	=	PACKED ARRAY[0..MaxAge] of PixPatHandle;

{Variables: Menus (including PopupMenu), Windows, RGB Colors, Color 
	Patterns, large Rect (used for drag & updates), Done Flag and Current
    & 0ld Dishes.}

VAR		appleMenu,fileMenu,editMenu,actionMenu,
			showMenu,popupMenu:						MenuHandle;
		lifeWindow,aboutWindow,
			colorWindow,quickWindow:				WindowPtr;
		myColors:									DishColor;
		myPixMap:									DishPixMap;
		bigRect:									Rect;
		doneFlag:	 								BOOLEAN;
		OldDish,CurDish:							Dish;
		CWhite:										RGBColor;

{Called if Color Quickdraw does not exists to explain why the program
	can not be run.}

PROCEDURE DoSorry;
VAR	n:integer;
BEGIN
	n:=Alert(sorryID,nil);
END;

{Given Integer value (corrsponding to a string stored in STR# resource),
	draw that string, centered on point h,v in c color.}

PROCEDURE DrawCenter(N,h,v,c:integer);
VAR S:str255;
BEGIN
	RGBForeColor(myColors[c]);
	GetIndString(S,strID,N);
	MoveTo(h-(StringWidth(S) DIV 2),v);
	DrawString(S);
END;

{Given a Dish, Clear all the cells (set to 0).}

PROCEDURE ClearDish(VAR J:Dish);
VAR h,v:integer;
BEGIN
	FOR h:=0 to HEdge+1 DO
		FORv:=0 to VEdge+1 DO
			J[h,v]:=0;
END;

{Given a Color Window Port, set the Pen and Background color Patterns
	(stored in myPicMap in position 0 & 1, respectively).}

PROCEDURE SetPortColors(W:windowptr);
BEGIN
	SetPort(W);
	BackPixPat(myPixMap[0]);
	PenPixPat(myPixMap[1]);
END;

{Given a Color Window Port, calls SetPortColors to set Pen and
	Background color Patterns and forces on update by invalidating a big 
	Rectangle.}

PROCEDURE ForceUpdate(W:windowptr);
BEGIN
	SetPortColors(W);
	InvalRect(bigRect);
END;

{Given a Color Window Port, saves the current Port, calls UpdateOne 
	and then restore the current port.}

PROCEDURE UpdateOne(W:windowptr);
VAR tempPort:grafptr;
BEGIN
	GetPort(tempPort);
	ForceUpdate(W);
	SetPort(tempPort);
END;

{Saves the current Port, calls ForceUpdate for all the windows to update
	them all and then restore the current port.}

PROCEDURE UpdateAll;
VAR tempPort:grafptr;
BEGIN
	GetPort(tempPort);
	ForceUpdate(aboutWindow);
	ForceUpdate(colorWindow);
	ForceUpdate(quickWindow);
	ForceUpdate(lifeWindow);
	SetPort(tempPort);
END;

{Given a Color Window Port, if it is not in front (or invisible), Shows it
	and Select it (make it the top window).}

PROCEDURE ShowIt(W:windowptr);
BEGIN
	IF FrontWindow<>W THEN BEGIN
		ShowWindow(W);
		SelectWindow(W);
	END;
END;

{Loads a new Dish and Dish Colors from disk.  If successful, remakes the
	Dishes Color Patterns and updates all the windows (using UpdateAll).}

PROCEDURE DoLoad;
VAR	tempReply: SFReply;
	tempType: SFTypeList;
	tempP: point;
	Ref,n: INTEGER;
	tempE: OSErr;
	tempLong:longint;
BEGIN
	tempP.v:=40;
	tempP.h:=40;
	tempType[0]:=FileType;
	SFGetFile(tempP,'',nil,1,tempType,nil,tempReply);
	Ref:=0;
	IF tempReply.good THEN BEGIN
		tempE:=FSOpen(tempReply.fname,tempReply.vrefnum,Ref);
		IF tempE=noErr THEN BEGIN
			tempLong:=SIZEOF(Dish);
			tempE:=FSRead(Ref,tempLong,@CurDish);
			IF tempE=noErr THEN BEGIN
				tempLong:=SIZEOF(DishColor);
				tempE:=FSRead(Ref,tempLong,@myColors);
				IF tempE=noErr THEN BEGIN
					tempE:=FSClose(Ref);
					Ref:=0;
					FOR n:=0 to MaxAge
						DO MakeRGBPat(myPixMap[n],myColors[n]);
					UpdateAll;
				END;
			END;
		END;
		IF Ref<>0
			THEN tempE:=FSClose(Ref);
	END;
END;

{Save the current Dish and Dish Colors to disk.}

PROCEDURE DoSave;
VAR	tempReply: SFReply;
	tempP: point;
	Ref: INTEGER;
	tempE: OSErr;
	tempLong:longint;
BEGIN
	tempP.v:=40;
	tempP.h:=40;
	SFPutFile(tempP,'','',nil,tempReply);
	IF tempReply.good THEN BEGIN
		tempE:=FSDelete(tempReply.fname,tempReply.vrefnum);
		IF (tempE=noErr) OR (tempE=fnfErr) THEN BEGIN
			tempE:=Create(tempReply.fname,tempReply.vrefnum,CreatorType,FileType);
			IF tempE=noErr THEN BEGIN
				tempE:=FSOpen(tempReply.fname,tempReply.vrefnum,Ref);
				IF tempE=noErr THEN BEGIN
					tempLong:=SIZEOF(Dish);
					tempE:=FSWrite(Ref,tempLong,@CurDish);
					IF tempE=noErr THEN BEGIN
						tempLong:=SIZEOF(DishColor);
						tempE:=FSWrite(Ref,tempLong,@myColors);
						IF tempE=noErr THEN BEGIN
							tempE:=FSClose(Ref);
							Ref:=0;
						END;
					END;
				END;
			END;
		END;
		IF Ref<>0
			THEN tempE:=FSClose(Ref);
	END;
END;
			
{Given a cells horiztonal and vertical position (in cell position, not
	screen position), return the screen position Rectangle that contains
    that cell.}

PROCEDURE MakeRect(h,v:integer;VAR R:rect);
VAR	n:integer;
BEGIN
	n:=(h*Big);
	R.right:=n;
	R.left:=n-big;
	n:=(v*Big);
	R.bottom:=n;
	R.top:=n-big;
END;

{Given a cells horiztonal and vertical position (in cell position, not
	screen position) and the correct screen rectangle holding that cell,
    draws the Cell in the correct pattern.  It will only erase the cell
    if this call was not done by an update event (update events erase 
    everything).}

PROCEDURE DrawRect(h,v:INTEGER;R:rect;UpEvent:BOOLEAN);
BEGIN
	IF CurDish[h,v]=1
		THEN PaintRect(R)
	ELSE IF (CurDish[h,v]>1) AND (CurDish[h,v]<=MaxAge)
		THEN FillCRect(R,myPixMap[CurDish[h,v]])
	ELSE IF NOT UpEvent
		THEN EraseRect(R);
END;

{Given a point in the window where a mouse down occured (global
	coordinates), flips the cell at that point (erasing one that there
    or making on in an empty space).  Then loops for every cell the
    mouse pass over (drawing or erasing cells) until the button is
    released.}

PROCEDURE DoLifeClick(Where:point);
VAR h,v:integer;
	tempPort:grafptr;
	tempR:rect;
	empty:BOOLEAN;
	
	PROCEDURE CalcWhere;
	BEGIN
		h:=(Where.h DIV big)+1;
		v:=(Where.v DIV big)+1;
	END;
	
	PROCEDURE DoDraw;
	BEGIN
		MakeRect(h,v,tempR);
		DrawRect(h,v,tempR,FALSE);
	END;
	
	PROCEDURE DoClick;
	BEGIN
		IF (h>0) AND (h<=HEdge) AND (v>0) AND (v<=VEdge) THEN BEGIN
			IF Empty THEN BEGIN
				IF CurDish[h,v]=0 THEN BEGIN
			  		CurDish[h,v]:=1;
			  		DoDraw;
				END;
			END
			ELSE BEGIN
				IF CurDish[h,v]<>0 THEN BEGIN
			 		CurDish[h,v]:=0;
			 		DoDraw;
				END;
			END;
		END;
	END;
	
BEGIN
	GetPort(tempPort);
	SetPort(lifeWindow);
	
	GlobalToLocal(Where);
	CalcWhere;
	empty:=(CurDish[h,v]=0);
	DoClick;
	repeat
		GetMouse(where);
		CalcWhere;
		DoClick;
	until not Button;
	
	SetPort(tempPort);
END;

{Given a mouse down point in the Color Window, display a pop up menu
	listing the RGB colors.  If a color is selected, call the Color
    Picker (GetColor) to select a new color.  If a color is picked,
    recalculate the Color Pattern (for that color) and Update all the
    windows (using UpdateAll).}

PROCEDURE DoColorClick(Where:point);
VAR	tempLong:LONGINT;
	M,I:INTEGER;
	tempP:point;
	tempS:str255;
BEGIN
	tempLong:=PopUpMenuSelect(popupMenu,Where.v,Where.h,1);
	M:=HiWord(tempLong);
	I:=LoWord(tempLong)-1;
	IF (M=popupID) and (0<=I) and (I<=MaxAge) THEN BEGIN
		tempP.v:=0;
		tempP.h:=0;
		GetIndString(tempS,strID,3);
		IF GetColor(tempP,tempS,myColors[I],myColors[I]) THEN BEGIN
			MakeRGBPat(myPixMap[i],myColors[i]);
			UpdateAll;
		END;
	END;
END;

{Draws all the cells in the Life window.}

PROCEDURE DoLifeDraw;
VAR h,v:integer;
	tempR:rect;
BEGIN
	EraseRect(bigRect);
	FOR h:=1 to HEdge DO
		FORv:=1 to VEdge DO BEGIN
			MakeRect(h,v,tempR);
			DrawRect(h,v,tempR,TRUE);
		END;
END;

{Draws all the colors in the Color window.}

PROCEDURE DoColorDraw;
VAR	tempR:rect;
	n,start,finish:INTEGER;
BEGIN
	SetRect(tempR,0,0,colorSize,colorSize);
	EraseRect(tempR);
	FOR n:=1 to MaxAge DO BEGIN
		start:=(360*(n-1)) DIV MaxAge;
		finish:=(360*n) DIV MaxAge;
		IF n=1 
			THEN PaintArc(tempR,start,finish-start)
			ELSE FillCArc(tempR,start,finish-start,myPixMap[n]);
	END;
END;

{Draws the About Window (information about the program and
	a spirograph type of desgin).}

PROCEDURE DoAboutDraw;
VAR	h,v,n:INTEGER;
BEGIN
	EraseRect(bigRect);
	
	TextSize(24);
	DrawCenter(4,HAbout DIV 2,120,7);
	DrawCenter(5,HAbout DIV 2,150,6);
	DrawCenter(6,HAbout DIV 2,180,5);
	
	FOR n:=1 TO StepSize-1 DO BEGIN
		h:=n*HStep;
		v:=n*VStep;
	
		PenPixPat(myPixMap[4]);
		MoveTo(0,v);
		LineTo(HAbout-h,0);
		
		PenPixPat(myPixMap[3]);
		MoveTo(h,0);
		LineTo(HAbout,v);
		
		PenPixPat(myPixMap[2]);
		LineTo(HAbout,v);
		LineTo(HAbout-h,VAbout);
			
		PenPixPat(myPixMap[1]);
		MoveTo(h,VAbout);
		LineTo(0,v);
	END;
END;

{Draws the Quickdraw Window that displays the major Color Quickdraw
	commands (similar to the Lisa version).}

PROCEDURE DoQuickDraw;
VAR	tempRect:Rect;
	tempPat:Pattern;
	tempPoly:PolyHandle;
	tempRgn:RgnHandle;
BEGIN
{Erase Background}
	EraseRect(bigRect);
	
{Draws Squares}
	PenPixPat(myPixMap[1]);
	MoveTo(0,150);
	LineTo(450,150);

	MoveTo(150,0);
	LineTo(150,300);
	MoveTo(300,0);
	LineTo(300,300);
	
	
{Draws Titles}
	DrawCenter(7,75,20,2);
	DrawCenter(8,225,20,3);
	DrawCenter(9,375,20,4);
	DrawCenter(10,75,170,5);
	DrawCenter(11,225,170,6);
	DrawCenter(12,375,170,7);
	
{Draw Rectangles}
	SetRect(tempRect,15,25,95,105);
	RGBForeColor(myColors[1]);
	RGBBackColor(CWhite);
	PenPat(Black);
	FrameRect(tempRect);
	
	OffSetRect(tempRect,QOff,QOff);
	RGBForeColor(myColors[2]);
	GetIndPattern(tempPat,0,31);
	PenPat(tempPat);
	PaintRect(tempRect);
	
	OffSetRect(tempRect,QOff,QOff);
	PenPixPat(myPixMap[3]);
	PaintRect(tempRect);
	
	OffSetRect(tempRect,QOff,QOff);
	RGBForeColor(myColors[4]);
	RGBBackColor(myColors[5]);
	GetIndPattern(tempPat,0,32);
	PenPat(tempPat);
	PaintRect(tempRect);
	
	OffSetRect(tempRect,QOff,QOff);
	FillCRect(tempRect,myPixMap[6]);

{Draw Ovals}
	SetRect(tempRect,165,25,245,105);
	RGBForeColor(myColors[7]);
	RGBBackColor(CWhite);
	PenPat(Black);
	FrameOval(tempRect);
	
	OffSetRect(tempRect,QOff,QOff);
	RGBForeColor(myColors[1]);
	GetIndPattern(tempPat,0,33);
	PenPat(tempPat);
	PaintOval(tempRect);
	
	OffSetRect(tempRect,QOff,QOff);
	PenPixPat(myPixMap[2]);
	PaintOval(tempRect);
	
	OffSetRect(tempRect,QOff,QOff);
	RGBForeColor(myColors[3]);
	RGBBackColor(myColors[4]);
	GetIndPattern(tempPat,0,34);
	PenPat(tempPat);
	PaintOval(tempRect);
	
	OffSetRect(tempRect,QOff,QOff);
	FillCOval(tempRect,myPixMap[5]);

{Draw Round Rectangles}
	SetRect(tempRect,315,25,395,105);
	RGBForeColor(myColors[6]);
	RGBBackColor(CWhite);
	PenPat(Black);
	FrameRoundRect(tempRect,20,20);
	
	OffSetRect(tempRect,QOff,QOff);
	RGBForeColor(myColors[7]);
	GetIndPattern(tempPat,0,35);
	PenPat(tempPat);
	PaintRoundRect(tempRect,20,20);
	
	OffSetRect(tempRect,QOff,QOff);
	PenPixPat(myPixMap[1]);
	PaintRoundRect(tempRect,20,20);
	
	OffSetRect(tempRect,QOff,QOff);
	RGBForeColor(myColors[2]);
	RGBBackColor(myColors[3]);
	GetIndPattern(tempPat,0,36);
	PenPat(tempPat);
	PaintRoundRect(tempRect,20,20);
	
	OffSetRect(tempRect,QOff,QOff);
	FillCRoundRect(tempRect,20,20,myPixMap[4]);

{Draw Polygons}
	tempPoly:=OpenPoly;
	SetRect(tempRect,15,175,95,255);
	MoveTo(95,175);
	LineTo(65,215);
	LineTo(95,255);
	LineTo(15,255);
	LineTo(15,215);
	LineTo(55,175);
	LineTo(95,175);
	ClosePoly;
	
	RGBForeColor(myColors[5]);
	RGBBackColor(CWhite);
	PenPat(Black);
	FramePoly(tempPoly);
	
	OffSetPoly(tempPoly,QOff,QOff);
	RGBForeColor(myColors[6]);
	GetIndPattern(tempPat,0,37);
	PenPat(tempPat);
	PaintPoly(tempPoly);
	
	OffSetPoly(tempPoly,QOff,QOff);
	PenPixPat(myPixMap[7]);
	PaintPoly(tempPoly);
	
	OffSetPoly(tempPoly,QOff,QOff);
	RGBForeColor(myColors[1]);
	RGBBackColor(myColors[2]);
	GetIndPattern(tempPat,0,38);
	PenPat(tempPat);
	PaintPoly(tempPoly);
	
	OffSetPoly(tempPoly,QOff,QOff);
	FillCPoly(tempPoly,myPixMap[3]);
	
	KillPoly(tempPoly);

{Draw Arcs}
	SetRect(tempRect,165,175,265,275);
	RGBForeColor(myColors[4]);
	RGBBackColor(CWhite);
	PenPat(Black);
	FrameArc(tempRect,198,72);
	
	RGBForeColor(myColors[5]);
	GetIndPattern(tempPat,0,15);
	PenPat(tempPat);
	PaintArc(tempRect,126,72);
	
	PenPixPat(myPixMap[6]);
	PaintArc(tempRect,270,72);
	
	RGBForeColor(myColors[7]);
	RGBBackColor(myColors[1]);
	GetIndPattern(tempPat,0,16);
	PenPat(tempPat);
	PaintArc(tempRect,342,72);
	
	OffSetRect(tempRect,20,0);
	FillCArc(tempRect,54,72,myPixMap[2]);

{Draw Regions}
	tempRgn:=NewRgn;
	OpenRgn;
	SetRect(tempRect,315,175,395,255);
	FrameRoundRect(tempRect,20,20);
	InsetRect(tempRect,10,10);
	FrameOval(tempRect);
	CloseRgn(tempRgn);
	
	RGBForeColor(myColors[3]);
	RGBBackColor(CWhite);
	PenPat(Black);
	FrameRgn(tempRgn);
	
	OffSetRgn(tempRgn,QOff,QOff);
	RGBForeColor(myColors[4]);
	GetIndPattern(tempPat,0,17);
	PenPat(tempPat);
	PaintRgn(tempRgn);
	
	OffSetRgn(tempRgn,QOff,QOff);
	PenPixPat(myPixMap[5]);
	PaintRgn(tempRgn);
	
	OffSetRgn(tempRgn,QOff,QOff);
	RGBForeColor(myColors[6]);
	RGBBackColor(myColors[7]);
	GetIndPattern(tempPat,0,18);
	PenPat(tempPat);
	PaintRgn(tempRgn);
	
	OffSetRgn(tempRgn,QOff,QOff);
	FillCRgn(tempRgn,myPixMap[1]);
	
	DisposeRgn(tempRgn);
END;

{Steps through a generation of growth/death of the Dish.  Stores the
	old Graf port, then sets the Life window as the new one.  Copies the
	current dish into the old dish, then checks each cells one by one.
    Calculates the number of cells around it, and decides deaths, lifes 
    and births.  If the new value does not match the old value
    (ie. change), redraw that window.}

PROCEDURE DoStep;
VAR	n,h,v,c:INTEGER;
	tempPort:grafptr;
	tempR:rect;
BEGIN
	GetPort(tempPort);
	SetPort(lifeWindow);
	BlockMove(@CurDish,@OldDish,SIZEOF(Dish));
	
	FOR h:=1 to HEdge DO
		FORv:=1 to VEdge DO BEGIN
			n:=0;
			IF OldDish[h-1,v-1]<>0 THEN n:=n+1;
			IF OldDish[h,v-1]<>0 THEN n:=n+1;
			IF OldDish[h+1,v-1]<>0 THEN n:=n+1;
			IF OldDish[h-1,v]<>0 THEN n:=n+1;
			IF OldDish[h+1,v]<>0 THEN n:=n+1;
			IF OldDish[h-1,v+1]<>0 THEN n:=n+1;
			IF OldDish[h,v+1]<>0 THEN n:=n+1;
			IF OldDish[h+1,v+1]<>0 THEN n:=n+1;
			c:=OldDish[h,v];
			IF (c=0) AND (n=3)
				THEN CurDish[h,v]:=1
			ELSE IF (c<>0) AND ((n=3) or (n=2)) THEN BEGIN
				IF c<MaxAge
					THEN CurDish[h,v]:=c+1;
			END
				ELSE CurDish[h,v]:=0;
				
			IF CurDish[h,v]<>c THEN BEGIN
				MakeRect(h,v,tempR);
				DrawRect(h,v,tempR,FALSE);
			END;
		END;
		
	SetPort(tempPort);
END;

{Standard Menu Command command procedures.  Takes card of Desk Accessories,
	About (show About Window), New (clear jar and update Life window), 
    Load/Save (call DoLoad/DoSave), Quit (mark Done Flag), Action Menus 
    (Step, Step 10 Times or Step until a Mouse down) and Show Menus 
    (show one of the 3 other windows).}

PROCEDURE DoCommand(mResult: LONGINT);
VAR
	theItem: 	INTEGER;
	theMenu: 	INTEGER;
	name: 		Str255;
	temp: 		INTEGER;
	tempBool:	BOOLEAN;
	tempEvent:	EventRecord;
BEGIN
	theItem := LoWord(mResult);
	theMenu := HiWord(mResult);

	CASE theMenu OF

		appleID: IF (theItem = 1)
			THEN ShowIt(aboutWindow)
		ELSE BEGIN
			GetItem(appleMenu, theItem, name);
			temp := OpenDeskAcc(name);
		END;
			
		fileID: CASE theItem OF
			1:	BEGIN
				ClearDish(CurDish);
				UpdateOne(lifeWindow);
			END;
			2:	DoLoad;
			3:	DoSave;
			5:	doneFlag := TRUE;
			OTHERWISE
		END;

		editID: tempBool:=SystemEdit(theItem - 1);
		
		actionID: CASE theItem OF
			1: DoStep;
			2: FOR temp:=1 TO 10
					DO DoStep;
			3: WHILE not GetNextEvent(mDownMask,tempEvent)
					DO DoStep;
			OTHERWISE
		END;
			
		showID: CASE theItem of
			1: ShowIt(lifeWindow);
			2: ShowIt(colorWindow);
			3: ShowIt(quickWindow);
			OTHERWISE
		END;
			
		OTHERWISE
	 END;
	 HiliteMenu(0);
END;

{Standard Init calls to set Macintosh up (regardless if it is a Mac //
	or not).}

PROCEDURE DoInit;
BEGIN
	InitGraf(@thePort);
	InitFonts;
	FlushEvents(everyEvent, 0);
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(NIL);
	InitCursor;
END;

{Checks if Color Quickdraw Exists on this machine.}

FUNCTION ColorQDExists:boolean;
CONST	ROM85Loc	=	$28E;
		TwoHighMask	=	$C000;
TYPE	WordPtr		=	^INTEGER;
BEGIN
	ColorQDExists:=(BitAnd(WordPtr(ROM85Loc)^,TwoHighMask)=0);
END;

{Given a h and v size, calculates a Rectangle that is centered on
	the screen.}

PROCEDURE CenterRect(h,v:INTEGER;VAR R:rect);
VAR	Hoff,Voff:INTEGER;
BEGIN
	Hoff:=(ScreenBits.bounds.right-h) DIV 2;
	VOff:=(ScreenBits.bounds.bottom-v) DIV 2;
	IF VOff<35
		THEN VOff:=35;
	SetRect(R,Hoff,VOff,Hoff+h,Voff+v);
END;

{Given RGB values (3 integers) and pos (which , places the values in
	the Dish Color in the correct position.}

PROCEDURE SetMyColors(i,r,g,b:integer);
BEGIN
	myColors[i].red:=r;
	myColors[i].green:=g;
	myColors[i].blue:=b;
END;

{Does the setup for this program (only if there is Color Quickdraw).
	Makes the Menus, clears the Dishes, Sets the Colors, make the Color
	Patterns and sets them, Make the windows, and inits in other variable
	that need to be set.}

PROCEDURE DoSetup;
VAR tempS:str255;
	tempR:rect;
	n:INTEGER;
BEGIN
	appleMenu := GetMenu(appleID);
	AddResMenu(appleMenu, 'DRVR');
	InsertMenu(appleMenu, 0);
	
	fileMenu := GetMenu(fileID);
	InsertMenu(fileMenu, 0);

	editMenu := GetMenu(editID);
	InsertMenu(editMenu, 0);

	actionMenu := GetMenu(actionID);
	InsertMenu(actionMenu, 0);

	showMenu := GetMenu(showID);
	InsertMenu(showMenu, 0);

	popupMenu := GetMenu(popupID);
	InsertMenu(popupMenu,-1);

	DrawMenuBar;

	ClearDish(OldDish);
	ClearDish(CurDish);
	
	SetMyColors(0,-1,-1,-1);
	SetMyColors(1,0,-1,-1);
	SetMyColors(2,0,-1,0);
	SetMyColors(3,-1,-1,0);
	SetMyColors(4,-1,0,0);
	SetMyColors(5,-1,0,-1);
	SetMyColors(6,0,0,-1);
	SetMyColors(7,0,0,0);
	CWhite.red:=-1;
	CWhite.green:=-1;
	CWhite.blue:=-1;

	FOR n:=0 to MaxAge DO BEGIN
		myPixMap[n]:=NewPixPat;
		MakeRGBPat(myPixMap[n],myColors[n])
	END;

	aboutWindow:=GetNewCWindow(aboutID,nil,POINTER(-1));
	SetPortColors(aboutWindow);

	colorWindow:=GetNewCWindow(colorID,nil,POINTER(-1));
	SetPortColors(colorWindow);
	
	GetIndString(tempS,strID,2);
	CenterRect(HQuick,VQuick,tempR);
	quickWindow:=NewCWindow(nil,tempR,tempS,FALSE,noGrowDocProc,POINTER(-1),TRUE,0);
	SetPortColors(quickWindow);

	GetIndString(tempS,strID,1);
	CenterRect(HLife,VLife,tempR);
	lifeWindow:=NewCWindow(nil,tempR,tempS,TRUE,noGrowDocProc,POINTER(-1),TRUE,0);
	SetPortColors(lifeWindow);

	SetRect(bigRect, -32000, -32000, 32000, 32000);
	doneFlag := FALSE;
END;

{Standard Main Progrom Loop, loops til done flag is marked.  Handles drags,
	go Aways, keys, and menu the same way.  Updates and mouse downs in
    windows, call the specific routine to handle those events.}

PROCEDURE MainLoop;
var	myWindow:	WindowPtr;
	myChar:	 	CHAR;
	myEvent:	EventRecord;
	myPort:		grafptr;
BEGIN
	REPEAT
		SystemTask;

		IF GetNextEvent(everyEvent, myEvent)
			THEN CASE myEvent.what OF

		  		mouseDown: CASE FindWindow(myEvent.where, myWindow) OF
			  		inSysWindow: SystemClick(myEvent, myWindow);
					inMenuBar: DoCommand(MenuSelect(myEvent.where));
			  		inDrag:	DragWindow(myWindow, myEvent.where, bigRect);
					inGoAway: IF TrackGoAway(myWindow,myEvent.where)
						THEN HideWindow(myWindow);
					inContent: IF myWindow <> frontWindow
						THEN SelectWindow(myWindow)
					ELSE IF myWindow=lifeWindow
						THEN DoLifeClick(myEvent.where)
					ELSE IF myWindow=colorWindow
						THEN DoColorClick(myEvent.where);
				END;

		  		keyDown, autoKey: BEGIN
					myChar := CHR(BAnd(myEvent.message, charCodeMask));
					IF BAnd(myEvent.modifiers, cmdKey) <> 0
						THEN DoCommand(MenuKey(myChar));
				END;


				updateEvt: BEGIN
					GetPort(myPort);
					myWindow:=WindowPtr(myEvent.message);
					BeginUpdate(myWindow);
					SetPort(myWindow);
					IF myWindow=lifeWindow
						THEN DoLifeDraw
					ELSE IF myWindow=colorWindow
						THEN DoColorDraw
					ELSE IF myWindow=aboutWindow
						THEN DoAboutDraw
					ELSE IF myWindow=quickWindow
						THEN DoQuickDraw;
					EndUpdate(WindowPtr(myEvent.message));
					SetPort(myPort);
				END;

				OTHERWISE
		END;
	UNTIL doneFlag;
END;

{Main Program.  First calls DoInit to do the standard inits.  Then checks
	if Color Quickdraw Exists.  If so, does Setup and Mainloop.  If not,
    call DoSorry to explain.}
 BEGIN
	DoInit;
	IF ColorQDExists then BEGIN
		DoSetup;
		MainLoop;
	END
	ELSE DoSorry;
END.
