        integer function txtdmp(fcn, text)
* This subroutine allows immediate access to the printer.
* It performs 3 different functions:
*
*       error = txtdmp(0, <character expression>) -
*                       Open the printer for immediate
*                       print.  The <character expression> is
*                       ignored.
*       error = txtdmp(1, <character expression>) -
*                       Output the <character expression> to
*                       the printer.
*       error = txtdmp(2 <character expression>) -
*                       Close the printer for immediate access.
*                       The <character expression> is
*                       ignored.
*
* The function result is the first non-zero error code returned
* by the ROM, or zero if there was none.
* If you want to start a new line, you must explicitly print a
* carriage return/line feed pair (char(13)//char(10)).
* This routine has not been tested on the LaserWriter, and may
* not function on it.
* This function can be mixed with the use of the spooled print
* (unit 6 or spool.sub) output feature.
* The close function should be executed,
* however, before spooling a file (unit 6 files are output on exit
* from the program).  This function can only output character
* variables; it cannot be used for formatted output.  For immediate
* formatted output, use txtdmp in conjunction with the internal file
* feature of FORTRAN.
* For speed, this function and
* toolbx.sub should be linked into the application.
*
* 02 Feb 86                                                     EWG
* 12 Mar 86	Sent to Compuserve.				EWG


        implicit none
        integer toolbx

        include file.inc
        include misc.inc
        include params.inc

        integer fcn
        character*(*) text

* Define the functions.
        integer popen, pprint, pclose
        parameter(popen = 0, pprint = 1, pclose = 2)

* Constant print driver reference number.
        integer iprdrvrref
        parameter (iprdrvrref = -3)

* Eqivalence 3 long words (integer*4) over the first 12 elements of the byte array
* csparam (in params.inc) for convenience in setting up the printer port.
        integer*4 prcont4(3)            ! Print protocol control words.
        equivalence (csparam(1),prcont4(1))

* Eqivalence 3 words (integer*2) over the first 6 elements of the byte array
* csparam (in params.inc) for convenience in setting up the printer port.
        integer*2 prcont2(3)            ! Print protocol control words.
        equivalence (csparam(1),prcont2(1))

        character*7 filename            ! Used to build print driver name.

        integer err                     ! Error code for file calls.
        integer i                       ! scratch
        integer textsize                ! Length of text string.
        integer textptr                 ! Pointer to the text string.

        integer paramptr                ! Pointer to the parameter block.

* Clear the file parameter block.
        do (i = 1, 80)
          params(i) = 0
        repeat

        paramptr = toolbx(PTR, params)  ! Pointer to parameter block.

* Perform the requested function.
        select case (fcn)

        case (popen)
* Open the printer port for output.
          filename = char(6) // '.PRINT'
          ionameptr = toolbx(PTR, filename)
          err = toolbx(PBOPEN, paramptr)

* Reset the printer.
          if (err = 0) then
            cscode = 7
            prcont2(1) = 1
            prcont2(2) = 0
            err = toolbx(PBCONTROL, paramptr)
          endif
        case (pprint)

* Set up a pointer to the text and the text size in the parameter block.
          textsize = len(trim(text))
          textptr = toolbx(PTR, text)

          iorefnum = iprdrvrref         ! Constant reference number.
          prcont4(1) = textptr
          prcont4(2) = textsize

* Print the text.
          cscode = 5
          err = toolbx(PBCONTROL, paramptr)

        case (pclose)
* Close the printer port.
        iorefnum = iprdrvrref           ! Constant reference number.
        err = toolbx(PBCLOSE, paramptr)

        end select

        txtdmp = err                    ! Return the error code.

        return
        end
