/*.h2 TEXT input file driver (Acta)*/
/*.fo 1987 Maitreya Design  code may be freely used in Acta format drivers */
/*
Read TEXT documents into Acta

Written by David Dunham
*/
#define	_DRIVER		/* Avoid QuickDraw globals */
#include	<quickdraw.h>
#include	<toolutils.h>
#include	<errors.h>
#include	<fonts.h>
#include	<memory.h>
#include	<files.h>
#include	<resources.h>

typedef int				word;				/* Synonyms for VAX-hackers */
typedef unsigned char	byte;

#define TAB		'\t'
#define CR		'\r'

#define NIL		(0L)
#define TRUE	(-1)
#define FALSE	0

#define TEXTtype	1
#define PICTtype	2

struct BLOCK_ATT {			/* Attribute portion of a topic */
	word	bType;			/* TEXTtype, PICTtype,  */
	word	bAttributes;	/* 0 */
	word	bFont;
	word	bStyle;
	word	bSize;
	byte	bColor;			/* Triangle color: index to COL# resource, 0 = black */
	byte	filler;			/* 0 */
	byte	firstLine;		/* Show first line only? 0xFF or 0 */
	byte	expanded;		/* Subtopics visible? 0xFF or 0 */
	word	hidden;			/* Is block hidden (by Collapse)? count of hide depth */
};
typedef struct BLOCK_ATT	BLOCK_ATT;

/*
READ_TEXT  Read generic TEXT file to Acta outline
			infil : refNum of input file
			resfil : refNum of input files resource fork
			att : default topic attributes
			add_topic : pointer to routine to call
*/
OSErr read_text(infil,resfil,att,add_topic)
word infil, resfil; BLOCK_ATT *att; ProcPtr add_topic; {
	register char	**text;					/* Handle for accumulating text */
	register OSErr	error;
	long			count;					/* Used in FS calls */
	register word	level;					/* Topics level */
	word			i_count, last_count, spaces;
	word			last_level;				/* Level of previous topic */
	word			space_indent;			/* Number of spaces equal to a tab */
	char			c;
	long			size;					/* Length of text */
	Handle			h;
	word			oldResFile;

	oldResFile = CurResFile();				/* Remember previous top resource file */
	UseResFile(resfil);						/* Just look in document */
	h = GetIndResource('EFNT',1);			/* The MDS Edit/QUED/miniWRITER font */
	if (h != NIL) {							/* There is such a resource */
		att->bSize = **(word **)h;
		GetFNum(*(char **)h + 2,&att->bFont);
	}
	UseResFile(oldResFile);					/* Restore */
	level = -1;								/* Indicate weve never parsed a topic */
	last_level = 0;
	space_indent = 5;						/* SHOULD be a resource. 5 for MORE 1.0 */
	last_count = i_count = spaces = 0;
	text = NIL;
	att->bType = TEXTtype;					/* Everything is text! */
	while (TRUE) {							/* Keep going until EOF */
											/* Will this die with empty file? */
		count = 1L;							/* Read a single character */
		error = FSRead(infil,&count,&c);
		if (error) {						/* Probably hit EOF before RETURN */
			if (error == eofErr) return(noErr);	/* Finished with the file */
			return(error);					/* Uh-oh */
		}
		if (c == TAB) {
			i_count++;						/* Increment indent count */
			spaces = 0;						/* Start count over */
		} else if (c == ' ') {
			spaces++;
			if (spaces == space_indent) {	/* A pseudo-tab */
				i_count++;					/* Increment indent count */
				spaces = 0;					/* Start count over */
			}
		} else {							/* Not space or tab */
			if (level == -1)				/* Very first topic */
				level = 1;
			  else
				level = level + i_count - last_count;
			if (level < 1)
				level = 1;					/* Minimum indent */
			if (level > last_level + 1)
				level = last_level + 1;		/* Only go one level deeper */
			if (level == 1) {				/* Left-margin topic? */
				att->expanded = TRUE;
				att->hidden = 0;
	  		  } else {						/* Subtopics arent expanded */
				att->expanded = FALSE;
				att->hidden = level - 2;	/* Number of collapsed levels above */
			}
			text = NIL;						/* No text so far */
			/* Read until end of line or end of file */
			while (c != CR && error == noErr) {
				if (text == NIL) {			/* First character? */
					text = NewHandle(1L);
					**text = c;				/* Copy it in */
				} else
					PtrAndHand(&c,text,1L);	/* Add to the end */
				count = 1L;					/* Read a single character */
				error = FSRead(infil,&count,&c);
			}
			if (error != noErr && error != eofErr)
				return(error);				/* Uh-oh */
			if (text == NIL) {				/* Empty topic */
				(*add_topic)(NIL,0L,level,att,NIL);
			  } else {
				size = GetHandleSize(text);
				HLock(text);				/* Keep dereference valid */
				(*add_topic)(*text,size,level,att,NIL);
				DisposHandle(text);			/* Free the memory */
				text = NIL;					/* Mark it as empty */
			}
			if (error == eofErr) {
				return(noErr);				/* Finished with the file */
			}
			last_count = i_count;			/* Remember input files indent */
			i_count = spaces = 0;
			last_level = level;				/* Remember topics level */
		} /* end if */
	} /* end while */
}
