!!S gaussj
	subroutine gaussj (ierr,mp,m,b,np,n,a)
c
c 	linear equation solution by Gauss-Jordan elimination.
c	A is an input matrix of N*N elements, stored in an array
c	of physical dimensions NP*NP. B is an input matrix of 
c	N*M containing the M right hand side vectors, stored 
c	in an array of physical dimensions NP*MP. On output, A
c	is replaced by its matrix inverse, and B is replaced by
c	the corresponding set of solution vectors.
c
c	from: Press/Flannery/Teukolsky/Vetterling, Numerical Recipes, 
c	Cambridge University Press, Cambridge, UK 1986.
c
c	JL \ added IERR for return of error status:
c			IERR=0	no error
c			IERR=-1	singular matrix
c		 parameters are in inverse order wrt original definition
c		 so that Mach2 can push them on the stack in the original
c		 order.
c
	integer nmax
	parameter (nmax=50)
	
	integer*4 n,np,m,mp
	real*4 a(np,np),b(np,mp)
	
	integer*4 ipiv(nmax),indxr(nmax),indxc(nmax)
	integer*4 i,j,k,l,ll,irow,icol
	real*4 big,dum,pivinv
	
	do i=1,n
		ipiv(i) = 0
	end do
	do i=1,n
		big=0.
		do j=1,n
			if (ipiv(j) .ne. 1) then
				do k=1,n
					if (ipiv(k).eq.0) then
						if(abs(a(j,k)) .ge. big) then
							big = abs(a(j,k))
							irow=j
							icol=k
						end if
					else if (ipiv(k).gt.1) then
						ierr=-1
						return
					end if
				end do
			end if
		end do
		ipiv(icol)=ipiv(icol)+1
		
		if(irow.ne.icol) then
			do l=1,n
				dum=a(irow,l)
				a(irow,l)=a(icol,l)
				a(icol,l) = dum
			end do
			do l=1,m
				dum=b(irow,l)
				b(irow,l)=b(icol,l)
				b(icol,l)=dum
			end do
		end if
		
		indxr(i) = irow
		indxc(i) = icol
		if (a(icol,icol).eq.0.) then
			ierr=-1
			return
		end if
		pivinv=1./a(icol,icol)
		a(icol,icol)=1.
		
		do l=1,n
			a(icol,l)=a(icol,l)*pivinv
		end do
		do l=1,m
			b(icol,l)=b(icol,l)*pivinv
		end do
		do ll=1,n
			if (ll.ne.icol) then
				dum=a(ll,icol)
				a(ll,icol)=0.
				do l=1,n
					a(ll,l)=a(ll,l)-a(icol,l)*dum
				end do
				do l=1,m
					b(ll,l)=b(ll,l)-b(icol,l)*dum
				end do
			end if
		end do
	end do
	
	do l=n,1,-1
		if(indxr(l).ne.indxc(l)) then
			do k=1,n
				dum=a(k,indxr(l))
				a(k,indxr(l))=a(k,indxc(l))
				a(k,indxc(l))=dum
			end do
		end if
	end do
	
	ierr=0
	return
	end
		