(*******************************************************************
	
	Corners.p
	
	Shell program for four-cornered windows.
	
	(c) 1988, by Clifford Story & Attic Software
	
*******************************************************************)

program Corners;
	
(******************************************************************)
	
	uses memtypes, quickdraw, osintf, toolintf, Common, Patches;
	
(*******************************************************************
	
	Program constants:
		
*******************************************************************)

	const

		applenum				=	1001;
			aboutitem		=	1;
			atticitem		=	2;

		filenum				=	1002;
			newitem			=	1;
			closeitem		=	2;
			quititem			=	4;

		editnum				=	1003;
			undoitem			=	1;
			cutitem			=	3;
			copyitem			=	4;
			pasteitem		=	5;
			clearitem		=	6;
		
		windownum			=	1001;
		scrollnum			=	1001;
		
		hoffset				=	32;
		voffset				=	20;
		
(*******************************************************************
	
	Program types:
		
*******************************************************************)
	
	type
		
		wrecord				=	record
			window			:	WindowPtr;
			zoomrect			:	Rect;
			hscroll			:	ControlHandle;
			vscroll			:	ControlHandle;
			disprect			:	Rect;
		end;
		wpointer				=	^wrecord;
		whandle				=	^wpointer;
	
(*******************************************************************
	
	Program variables:
		
*******************************************************************)
		
	var
	
		APPLEMENU			:	MenuHandle;
		FILEMENU				:	MenuHandle;
		EDITMENU 			:	MenuHandle;

		MOUSEREGION			:	RgnHandle;
		SCREENREGION		:	RgnHandle;
		
		MENUHEIGHT			:	integer;
		DRAGRECT				:	Rect;
		GROWRECT				:	Rect;
		SCREENRECT			:	Rect;
		
		COLUMNS				:	integer;
		ROWS					:	integer;
		WINDOWCOUNT			:	integer;
		
		DONE					:	logical;
		JEVENT				:	logical;
		MAINEVENT			:	EventRecord;
		
(******************************************************************)
	
	procedure _datainit; external;
	
(******************************************************************)
	
	{$R-}
	{$SC+}
		
(******************************************************************)
	
	procedure panic;
	
		begin

			ExitToShell;

		end;
		
(******************************************************************)
	
	procedure initmac;
	
		begin

			MaxApplZone;
			InitGraf(@thePort);
			InitFonts;
			InitWindows;
			InitCursor;
			InitMenus;
			TEInit;
			InitDialogs(@panic);
			
			UnloadSeg(@_datainit);
			
		end;
		
(******************************************************************)
			
	procedure setupmenus;
			
		begin
		
			APPLEMENU := GetMenu(applenum);
			AddResMenu(APPLEMENU, 'DRVR');
			InsertMenu(APPLEMENU, 0);
			
			FILEMENU := GetMenu(filenum);
			InsertMenu(FILEMENU, 0);
			
			EDITMENU := GetMenu(editnum);
			InsertMenu(EDITMENU, 0);
			
			DrawMenuBar;
			
		end;
		
(*******************************************************************
	
	initglobals
	-----------
	
	Here's where I initialize all the global variables.  DRAGRECT is
	an argument to DragWindow; GROWRECT is an argument to
	growthewindow (in the Patches unit); SCREENRECT is used in
	zooming (it's the portrect of a full-screen window); and ROWS and
	COLUMNS are used to stack the windows on the screen.
		
*******************************************************************)
	
	procedure initglobals;
	
		var
			index				:	integer;
			theshort			:	shortpointer;
	
		begin

			for index := 1 to 10 do
				MoreMasters;
			
			if BitTst(Ptr(rom85), 0) then begin
				MENUHEIGHT := 20;
				JEVENT := false;
			end else begin
				theshort := shortpointer(mbarheight);
				MENUHEIGHT := theshort^;
				JEVENT := (NGetTrapAddress($A860, ToolTrap)
								<> NGetTrapAddress($A89F, ToolTrap));
			end;
			
			with screenBits.bounds do begin
				SetRect(DRAGRECT, left + 5, top + MENUHEIGHT + 5,
								right - 5, bottom - 25);
				SetRect(GROWRECT, 160, 100, right - left - 10,
								bottom - top - MENUHEIGHT - 10);
				SetRect(SCREENRECT, left + 5, top + MENUHEIGHT + 25,
								right - 5, bottom - 5);
				COLUMNS := 1 + ((right - left - 330) div hoffset);
				ROWS := 1 + ((bottom - top - MENUHEIGHT - 230) div voffset);
				WINDOWCOUNT := 0;
			end;
			
			DONE := false;
			
		end;

(*******************************************************************
	
	clickapplemenu
	--------------
	
	This may not be new to you but it is to me!  Instead of using an
	alert for the About... box, I'm using a picture.  First I open
	a new GrafPort, then draw the picture in its center.  Note that
	I must re-draw the windows after the user dismisses the screen,
	using PaintBehind.
		
*******************************************************************)
	
	procedure clickapplemenu(theitem : integer);
	
		var
			itemname			:	Str255;
			savedport		:	GrafPtr;
			dummy				:	integer;
			newport			:	GrafPort;
			thepicture		:	PicHandle;
			therect			:	Rect;

		begin
		
			if theitem > 3 then begin
				GetItem(APPLEMENU, theitem, itemname);
				GetPort(savedport);
				dummy := OpenDeskAcc(itemname);
				SetPort(savedport);
			end else if theitem < 3 then begin
				
				InitCursor;
				GetPort(savedport);
				OpenPort(@newport);
				SetPort(@newport);
				
				thepicture := PicHandle(GetResource('PICT',
								1000 + theitem));
				with thepicture^^.picFrame do
					SetRect(therect, 0, 0, right - left, bottom - top);
				with screenBits.bounds, therect.botright do
					OffsetRect(therect, (right - left - h) div 2,
									(bottom - top - v) div 3);
				DrawPicture(thepicture, therect);
				
				repeat until Button;
				
				ClosePort(@newport);
				EnableItem(EDITMENU, 0);
				DrawMenuBar;
				PaintBehind(WindowPeek(FrontWindow),
								RgnHandle(longpointer(grayrgn)^));
				
				SetPort(savedport);
				FlushEvents(everyEvent, 0);
			
			end;
			
		end;
		
(*******************************************************************
	
	placewindow
	-----------
	
	This routine cost me $40.00 (it's the only thing I got out of
	Macintosh Revealed).
		
*******************************************************************)

	procedure placewindow(thewindow : WindowPtr);
	
		var
			left				:	integer;
			top				:	integer;
	
		begin
			
			left := 5 + hoffset * (WINDOWCOUNT mod COLUMNS);
			top := 5 + MENUHEIGHT
							+ voffset * (1 + (WINDOWCOUNT mod ROWS));
			
			MoveWindow(thewindow, left, top, true);
			
		end;
		
(******************************************************************)

	procedure wsize(thewindow : whandle);
	
		var
			thewidth			:	integer;
			theheight		:	integer;
	
		begin
			
			HLock(Handle(thewindow));
			with thewindow^^ do begin
				
				thewidth := window^.portRect.right;
				theheight := window^.portRect.bottom;
			
				HideControl(hscroll);
				HideControl(vscroll);
					
					MoveControl(vscroll, thewidth - 15, -1);
					SizeControl(vscroll, 16, theheight - 13);
					
					MoveControl(hscroll, 15, theheight - 15);
					SizeControl(hscroll, thewidth - 29, 16);
			
					disprect := window^.portRect;
					InsetRect(disprect, 10, 10);
					OffsetRect(disprect, -8, -8);
						
				ShowControl(hscroll);
				ShowControl(vscroll);
				
			end;
			HUnlock(Handle(thewindow));
			
		end;

(******************************************************************)

	procedure donew;
		
		label
			100;
	
		var
			thehandle		:	whandle;
			thewindow		:	WindowPtr;
	
		begin
			
			thehandle := whandle(NewHandle(sizeof(wrecord)));
			if thehandle = nil then
				goto 100;
			
			HLock(Handle(thehandle));
			with thehandle^^ do begin
				
				window := GetNewWindow(windownum, nil, WindowPtr(-1));
				if window = nil then begin
					DisposHandle(Handle(thehandle));
					goto 100;
				end;
				SetWRefCon(window, long(thehandle));
				
				zoomrect := SCREENRECT;
				
				hscroll := GetNewControl(scrollnum, window);
				if hscroll = nil then begin
					DisposeWindow(window);
					DisposHandle(Handle(thehandle));
					goto 100;
				end;
				
				vscroll := GetNewControl(scrollnum, window);
				if vscroll = nil then begin
					DisposeControl(hscroll);
					DisposeWindow(window);
					DisposHandle(Handle(thehandle));
					goto 100;
				end;
				
				placewindow(window);
				wsize(thehandle);
				ShowWindow(window);
				
				WINDOWCOUNT := WINDOWCOUNT + 1;
			
			end;
			HUnlock(Handle(thehandle));
			
100:	end;
		
(******************************************************************)

	procedure doclose(thepeek : WindowPeek);
		
		var
			thehandle			:	whandle;
	
		begin
			
			if thepeek^.windowkind < 0 then
				CloseDeskAcc(thepeek^.windowkind)
			else begin
				thehandle := whandle(thepeek^.refCon);
				DisposeControl(thehandle^^.hscroll);
				DisposeControl(thehandle^^.vscroll);
				DisposeWindow(WindowPtr(thepeek));
				DisposHandle(Handle(thehandle));
				WINDOWCOUNT := WINDOWCOUNT - 1;
			end;
			
		end;
		
(******************************************************************)

	procedure clickfilemenu(itemchoice : integer);
	
		var
			dummy				:	logical;
	
		begin
			
			case itemchoice of
				newitem		:	donew;
				closeitem	:	doclose(WindowPeek(FrontWindow));
				quititem		:	DONE := true;
			end;
			
		end;
		
(******************************************************************)

	procedure checkmenu(thewindow : WindowPeek);
	
		begin
			
			DisableItem(EDITMENU, 0);
			
			if thewindow = nil then
				DisableItem(FILEMENU, closeitem)
			else begin
				if thewindow^.windowkind <> userKind then
					EnableItem(EDITMENU, 0);
				EnableItem(FILEMENU, closeitem);
			end;
			
		end;
		
(******************************************************************)

	procedure clickinmenu;
			
		var
			choice			:	long;
			
		begin
			
			checkmenu(WindowPeek(FrontWindow));
			choice := MenuSelect(MAINEVENT.where);
			
			case HiWord(choice) of
				applenum		:	clickapplemenu(LoWord(choice));
				filenum		:	clickfilemenu(LoWord(choice));
				editnum		:	if not SystemEdit(LoWord(choice) - 1) then;
			end;

			HiliteMenu(0);
		
		end;
		
(*******************************************************************
	
	clickingrow
	-----------
	
	This is just like the usual grow routine, except that I'm calling
	the Patches routines growthewindow and sizethewindow.  These
	behave like GrowWindow and SizeWindow, but in the four-cornered
	context.
		
*******************************************************************)
	
	procedure clickingrow(thewindow : WindowPtr);
		
		var
			newrect			:	Rect;
	
		begin
			
			if growthewindow(thewindow, MAINEVENT.where,
							GROWRECT, newrect) then begin
				sizethewindow(thewindow, newrect);
				wsize(whandle(GetWRefCon(thewindow)));
			end;

		end;
		
(******************************************************************)
	
	procedure clickingoaway(thewindow : WindowPtr);
	
		begin
			
			if TrackGoAway(thewindow, MAINEVENT.where) then
				doclose(WindowPeek(thewindow));

		end;
		
(*******************************************************************
	
	clickinzoom
	-----------
	
	Custom zooming - this will work on any ROM.  trackthebox
	performs exactly like TrackBox; I wrote it so I could track
	clicks in the zoom box regardless of ROM version.  The actual
	zooming uses sizethewindow, which I already have for growing
	windows.
	
	Each window has a field called zoomrect.  If I'm zooming to
	full screen, I save the old portRect in zoomrect first.  If I'm
	already at full screen and I'm zooming back, I use the save
	zoomrect as the target portRect.  Note, though, that zoomrect
	must be in global coordinates.
		
*******************************************************************)
	
	procedure clickinzoom(thewindow : WindowPtr);
		
		var
			thehandle		:	whandle;
			therect			:	Rect;
	
		begin
			
			if trackthebox(thewindow) then begin
				
				thehandle := whandle(GetWRefCon(thewindow));
				therect := thewindow^.portRect;
				EraseRect(therect);
				
				LocalToGlobal(therect.topleft);
				LocalToGlobal(therect.botright);
				
				if EqualRect(therect, SCREENRECT) then
					therect := thehandle^^.zoomrect
				else begin
					thehandle^^.zoomrect := therect;
					therect := SCREENRECT;
				end;
				
				sizethewindow(thewindow, therect);
				wsize(thehandle);
			
			end;
			
		end;
		
(******************************************************************)
	
	procedure aclick;
		
		var
			location			:	integer;
			thewindow		:	WindowPtr;
			
		begin
			
			location := FindWindow(MAINEVENT.where, thewindow);

			case location of
				inDesk		:	SysBeep(1);
				inMenuBar	:	clickinmenu;
				inSysWindow	:	SystemClick(MAINEVENT, thewindow);
				inContent	:	SelectWindow(thewindow);
				inDrag		:	DragWindow(thewindow, MAINEVENT.where, DRAGRECT);
				inGrow		:	clickingrow(thewindow);
				inGoAway		:	clickingoaway(thewindow);
				inZoomIn		:	clickinzoom(thewindow);
			end;
		
		end;
		
(******************************************************************)
	
	procedure akey;
		
		var
			charcode			:	integer;
			choice			:	long;
	
		begin
			
			if BitAnd(MAINEVENT.modifiers, cmdKey) <> 0 then begin
	
				charcode := BitAnd(MAINEVENT.message, charCodeMask);
				checkmenu(WindowPeek(FrontWindow));
				choice := MenuKey(chr(charcode));
				
				if choice <> 0 then begin
				
					case HiWord(choice) of
						applenum		:	clickapplemenu(LoWord(choice));
						filenum		:	clickfilemenu(LoWord(choice));
						editnum		:	if not SystemEdit(LoWord(choice) - 1) then;
					end;
					
					HiliteMenu(0);
					
				end;
				
			end;

		end;
		
(******************************************************************)
	
	procedure anactivate(thewindow : WindowPtr);
	
		begin
			
			SetPort(thewindow);
			DrawGrowIcon(thewindow);

		end;
		
(******************************************************************)
	
	procedure anupdate(thewindow : WindowPtr);
		
		var
			savedport		:	GrafPtr;
	
		begin
			
			GetPort(savedport);
			SetPort(thewindow);
			
			BeginUpdate(thewindow);
				
				ClipRect(thewindow^.portRect);
				EraseRect(thewindow^.portRect);
				DrawGrowIcon(thewindow);
				DrawControls(thewindow);
			
			EndUpdate(thewindow);
			
			SetPort(savedport);

		end;
		
(******************************************************************)
	
	procedure mainloop;
		
		var
			dummy				:	logical;
	
		begin
			
			repeat
				
				if JEVENT then
					dummy := waitnextevent(everyEvent, MAINEVENT,
									GetCaretTime, nil)
				else begin
					SystemTask;
					dummy := GetNextEvent(everyEvent, MAINEVENT);
				end;
				
				if dummy then begin
					case MAINEVENT.what of
						mouseDown		:	aclick;
						keyDown			:	akey;
						autoKey			:	akey;
						activateEvt		:	anactivate(WindowPtr(MAINEVENT.message));
						updateEvt		:	anupdate(WindowPtr(MAINEVENT.message));
					end;
				end;
			
			until DONE;

		end;
		
(******************************************************************)
	
	procedure shutdown;
		
		var
			thelong			:	longpointer;
			thepeek			:	WindowPeek;
		
		begin
			
			thelong := longpointer(windowlist);
			thepeek := WindowPeek(thelong^);
			
			while thepeek <> nil do begin
				doclose(thepeek);
				thepeek := thepeek^.nextwindow;
			end;

		end;
		
(******************************************************************)
	
	begin
		
		initmac;
		setupmenus;
		initglobals;
		
		mainloop;
		
		shutdown;
		
	end.
		
(******************************************************************)
