(*******************************************************************
	
	Patches.p
	
	ROM replacements for four-cornered windows.
	
	(c) 1988, by Clifford Story & Attic Software
	
*******************************************************************)

unit Patches;
	
(******************************************************************)
	
	interface
	
(******************************************************************)
	
	uses memtypes, quickdraw, osintf, toolintf, Common;
	
(******************************************************************)

	function growthewindow(theWindow : WindowPtr; startpoint : Point;
					sizerect : Rect; var newportrect : Rect) : logical;

	procedure sizethewindow(theWindow : WindowPtr; newrect : Rect);

	function trackthebox(theWindow : WindowPtr) : logical;
		
(******************************************************************)
	
	implementation
	
(******************************************************************)
	
	{$R-}
	{$SC+}
	
(*******************************************************************
	
	callWDEF
	--------
	The inline code here is:
		
		movea.l		(SP)+,A0
		jsr			A0
	
	That is, it pops the address argument into register A0, and
	then calls the routine at that address.
	
*******************************************************************)
	
	function callWDEF(variation : integer; thewindow : WindowPtr;
					message : integer; param : Ptr;
					address : Ptr) : long; inline $205F, $4E90;

(*******************************************************************
	
	growthewindow
	-------------
	This routine is an analog of the Toolbox trap GrowWindow; it
	follows the mouse with a gray outline of the window's new size.
	
	I'm going to draw in the Window Manager port, and I'm going to
	set the clipping to full screen (the low-memory global grayrgn
	contains a handle to the region consisting of everything but the
	menu bar).  Since I don't want to upset the Window Manager, I'll
	save the old clip region first, so I can restore it at the end.
	The pen mode makes drawing its own inverse.  This routine draws
	by calling the WDEF with a wGrow message.  All this is in the
	Window Manager section of Inside Mac; I'm just approaching it
	from the other side.
	
	The mouse is down in one corner of the window.  The diagonally
	opposite window is fixed for this operation; I call it the
	pivot.  The mouse is probably not on the corner exactly; I use
	offset to correct the mouse position.  And this corrected
	position will be corrected further to bring it within limitrect
	if it strays outside.
	
	From here on, it's simple.  Draw the first outline; then, so
	long as the mouse stays down, repeatedly erase it (by re-drawing)
	and draw the new outline.  When the mouse is released, erase the
	outline and return the new portRect.
	
*******************************************************************)

	function growthewindow(thewindow : WindowPtr; startpoint : Point;
					sizerect : Rect; var newportrect : Rect) : logical;
		
		var
			savedport		:	GrafPtr;
			drawport			:	GrafPtr;
			savedclip		:	RgnHandle;
			thepointer		:	longpointer;
			WDEF				:	Handle;
			newrect			:	Rect;
			west				:	logical;
			north				:	logical;
			pivot				:	Point;
			offset			:	Point;
			limitrect		:	Rect;
			menuheight		:	integer;
			theshort			:	shortpointer;
			dummy				:	long;
			oldrect			:	Rect;
			newpoint			:	Point;
	
		begin
			
			GetPort(savedport);
			GetWMgrPort(drawport);
			SetPort(drawport);
			
			savedclip := NewRgn;
			GetClip(savedclip);
			
			thepointer := longpointer(grayrgn);
			SetClip(RgnHandle(thepointer^));
			
			PenMode(notPatXor);
			PenPat(gray);
			
			WDEF := WindowPeek(thewindow)^.windowdefproc;
			HLock(WDEF);
			
			newrect := thewindow^.portRect;
			with thewindow^.portBits.bounds do
				OffsetRect(newrect, - left, - top);
			
			with newrect do begin
				
				if startpoint.v <= ((top + bottom) div 2) then begin
					pivot.v := bottom;
					offset.v := top - startpoint.v;
					limitrect.top := bottom - sizerect.bottom;
					limitrect.bottom := bottom - sizerect.top;
				end else begin
					pivot.v := top;
					offset.v := bottom - startpoint.v;
					limitrect.top := top + sizerect.top;
					limitrect.bottom := top + sizerect.bottom;
				end;
				
				if startpoint.h <= ((left + right) div 2) then begin
					pivot.h := right;
					offset.h := left - startpoint.h;
					limitrect.left := right - sizerect.right;
					limitrect.right := right - sizerect.left;
				end else begin
					pivot.h := left;
					offset.h := right - startpoint.h;
					limitrect.left := left + sizerect.left;
					limitrect.right := left + sizerect.right;
				end;
				
			end;
			
			if BitTst(Ptr(rom85), 0) then
				menuheight := 38
			else begin
				theshort := shortpointer(mbarheight);
				menuheight := theshort^ + 18;
			end;
			if limitrect.top < menuheight then
				limitrect.top := menuheight;
			
			newpoint := startpoint;
			AddPt(offset, newpoint);
			
			newpoint := Point(PinRect(limitrect, newpoint));
			Pt2Rect(pivot, newpoint, newrect);
			
			dummy := callWDEF(0, thewindow, wGrow, @newrect, WDEF^);
			oldrect := newrect;
			
			while StillDown do begin
				
				GetMouse(newpoint);
				AddPt(offset, newpoint);
				
				newpoint := Point(PinRect(limitrect, newpoint));
				Pt2Rect(pivot, newpoint, newrect);
				
				if not EqualRect(newrect, oldrect) then begin
					dummy := callWDEF(0, thewindow, wGrow, @oldrect, WDEF^);
					dummy := callWDEF(0, thewindow, wGrow, @newrect, WDEF^);
					oldrect := newrect;
				end;
				
			end;
			
			dummy := callWDEF(0, thewindow, wGrow, @oldrect, WDEF^);
			
			HUnlock(WDEF);
			
			PenNormal;
			SetClip(savedclip);
			DisposeRgn(savedclip);
			SetPort(thewindow);
			
			oldrect := thewindow^.portRect;
			LocalToGlobal(oldrect.topleft);
			LocalToGlobal(oldrect.botright);
			
			if EqualRect(oldrect, newrect) then
				growthewindow := false
			else begin
				newportrect := newrect;
				growthewindow := true;
			end;
			
			SetPort(savedport);
			
		end;
		
(*******************************************************************
	
	sizethewindow
	-------------
	This routine is an analog of the Toolbox trap SizeWindow.  It
	just precedes SizeWindow with MovePortTo, which moves the
	window without updating the screen.  After this, I update the
	Window Manager port's clipping, so it will draw the window
	correctly.
	
*******************************************************************)

	procedure sizethewindow(thewindow : WindowPtr; newrect : Rect);
		
		var
			drawport			:	GrafPtr;
		
		begin
			
			EraseRect(thewindow^.portRect);
			
			with newrect do begin
				MovePortTo(left, top);
				SizeWindow(thewindow, right - left, bottom - top, false);
			end;
			
			InvalRect(thewindow^.portRect);
			
			GetWMgrPort(drawport);
			SetPort(drawport);
			SetClip(WindowPeek(thewindow)^.strucrgn);
			
			SetPort(thewindow);
			ClipAbove(WindowPeek(thewindow));
			
		end;
		
(*******************************************************************
	
	trackthebox
	-----------
	This routine is a duplicate of TrackBox; I would use TrackBox
	but it isn't available in the old ROM.
	
	The idea is to highlight the zoom box, and then track the mouse
	while the button remains down.  The zoom box should be
	highlighted when the mouse is in it, and unhighlighted when it
	is not.  When the button is finally released, the routine returns
	true if the mouse is in the zoom box, and false if it is not.
	
*******************************************************************)

	function trackthebox(thewindow : WindowPtr) : logical;
		
		var
			highlight		:	logical;
			savedport		:	GrafPtr;
			drawport			:	GrafPtr;
			thepointer		:	longpointer;
			WDEF				:	Handle;
			thepoint			:	Point;
			thepart			:	long;
		
		begin
			
			GetPort(savedport);
			GetWMgrPort(drawport);
			SetPort(drawport);
			
			WDEF := WindowPeek(thewindow)^.windowdefproc;
			HLock(WDEF);
			
			thepart := callWDEF(0, thewindow,
							wDraw, Ptr(wInZoomIn), WDEF^);
			highlight := true;
			
			while StillDown do begin
				
				GetMouse(thepoint);
				thepart := callWDEF(0, thewindow,
								wHit, Ptr(thepoint), WDEF^);
				
				if not highlight and (thepart = wInZoomIn) then begin
					thepart := callWDEF(0, thewindow,
									wDraw, Ptr(wInZoomIn), WDEF^);
					highlight := true;
				end else if highlight and (thepart <> wInZoomIn) then begin
					thepart := callWDEF(0, thewindow,
									wDraw, Ptr(wInZoomIn), WDEF^);
					highlight := false;
				end;
				
			end;
			
			if highlight then
				thepart := callWDEF(0, thewindow,
								wDraw, Ptr(wInZoomIn), WDEF^);
			
			HUnlock(WDEF);
			
			SetPort(savedport);
			
			trackthebox := highlight;
			
		end;
		
(******************************************************************)
	
	end.
		
(******************************************************************)
