(*******************************************************************
	
	WDEF.p
	
	WDEF for four-cornered window.
	
	(c) 1988, by Clifford Story & Attic Software
	
*******************************************************************)

unit WDEF;

(******************************************************************)
	
	interface
	
(******************************************************************)
	
	uses memtypes, quickdraw, osintf, toolintf, Common;
	
(******************************************************************)
	
	function windowdef(variation : integer; thewindow : WindowPeek;
					message : integer; param : long) : long;
	
(******************************************************************)
	
	implementation
	
(******************************************************************)
	
	{$R-}
	{$SC+}
	
(*******************************************************************
	
	datahandle
	----------
	This is horribly inefficient; each window saves the data for
	each BitMap in its datahandle.  So each window duplicates this
	information (which must also be in the code, or where would it
	come from?).  I've got an assembler version of this WDEF that
	eliminates all this duplicate data; you could do it in Pascal
	by moving the StuffHex's out of the init routine and into the
	draw routine.
	
*******************************************************************)
	
	type
	
		rectrecord			=	record
			dragrect			:	Rect;
			closerect		:	Rect;
			zoomrect			:	Rect;
			NWmap				:	BitMap;
			NWdata			:	array [1..28] of integer;
			NEmap				:	BitMap;
			NEdata			:	array [1..28] of integer;
			SWmap				:	BitMap;
			SWdata			:	array [1..8] of integer;
			SEmap				:	BitMap;
			SEdata			:	array [1..8] of integer;
			erasedata		:	array [1..8] of integer;
			closemap			:	BitMap;
			closedata		:	array [1..9] of integer;
			zoommap			:	BitMap;
			zoomdata			:	array [1..9] of integer;
		end;
		rectpointer			=	^rectrecord;
		recthandle			=	^rectpointer;
		
		QDrecord				=	record
			randseed			:	long;
			screenbits		:	bitmap;
			arrow				:	cursor;
			dkgray			:	pattern;
			ltgray			:	pattern;
			gray				:	pattern;
			black				:	pattern;
			white				:	pattern;
			theport			:	grafptr;
		end;
		QDpointer			=	^QDrecord;
	
(******************************************************************)
	
	procedure wdodraw(thewindow : WindowPeek; param : long); forward;
	function wdohit(thewindow : WindowPeek; thepoint : Point) : long; forward;
	procedure wdocalc(thewindow : WindowPeek); forward;
	procedure wdonew(thewindow : WindowPeek); forward;
	procedure wdodispose(thewindow : WindowPeek); forward;
	procedure wdogrow(therect : rectpointer); forward;
	procedure wdoicon(thewindow : WindowPtr); forward;
	
(******************************************************************)
	
	function windowdef(variation : integer; thewindow : WindowPeek;
					message : integer; param : long) : long;
	
		begin
			
			windowdef := 0;
			
			case message of
				wDraw				:	wdodraw(thewindow, param);
				wHit				:	windowdef := wdohit(thewindow, Point(param));
				wCalcRgns		:	wdocalc(thewindow);
				wNew				:	wdonew(thewindow);
				wDispose			:	wdodispose(thewindow);
				wGrow				:	wdogrow(rectpointer(param));
				wDrawGIcon		:	wdoicon(WindowPtr(thewindow));
			end;

		end;
		
(*******************************************************************
	
	QDglobals
	---------
	A WDEF can't use globals, hence cannot reference the Quickdraw
	globals directly.  This routine returns a pointer to the
	Quickdraw globals, which the WDEF can use instead.
	
*******************************************************************)

	function QDglobals : QDpointer;
		
		var
			thepointer		:	longpointer;
		
		begin
			
			thepointer := longpointer(currenta5);
			thepointer := longpointer(thepointer^);
			QDglobals := QDpointer(long(thepointer^)
							- sizeof(QDrecord) + sizeof(grafptr));
		
		end;
	
(*******************************************************************
	
	wdodraw
	-------
	This routine draws the window.  There are two special cases:  if
	the low word of the param argument is wInGoAway, then this is
	a call for close box highlighting; if it is wInZoomIn, then this
	is a call for zoom box highlighting.
	
	I draw the various parts of the window in order.  First, I
	recalculate the various regions.  Then I draw the window's
	outline and shadow.  Next is the title bar:  outline the title
	bar, then erase its interior.  Draw the title (truncated, if
	necessary, by the clipping), then the corners with CopyBits,
	and finally the horizontal lines between the corners and the
	title.  Last of all, draw the lower corners.  Note that the
	corner brackets are drawn only if the window is highlighted.
	
*******************************************************************)
	
	procedure wdodraw(thewindow : WindowPeek; param : long);
		
		var
			therect			:	Rect;
			width				:	integer;
			limitrect		:	Rect;
			savedclip		:	RgnHandle;
			height			:	integer;
			index				:	integer;
		
		begin
			
			with thewindow^, QDglobals^.thePort^ do begin
				
				if visible then begin
					
					HLock(dataHandle);
					with recthandle(dataHandle)^^ do begin
						
						if LoWord(param) = wInGoAway then begin
							closemap.baseAddr := @closedata;
							CopyBits(closemap, portBits, closemap.bounds,
											closemap.bounds, srcXor, nil);
						end else if LoWord(param) = wInZoomIn then begin
							zoommap.baseAddr := @zoomdata;
							CopyBits(zoommap, portBits, zoommap.bounds,
											zoommap.bounds, srcXor, nil);
						end else if param = 0 then begin
							
							wdocalc(thewindow);
							
							FrameRgn(strucrgn);
							therect := strucrgn^^.rgnBBox;
							with therect do begin
								MoveTo(left, bottom - 2);
								LineTo(right - 2, bottom - 2);
								LineTo(right - 2, top);
							end;
							
							therect := dragrect;
							InsetRect(therect, 1, 1);
							EraseRect(therect);
							FrameRect(dragrect);
							
							with dragrect do
								width := (right + left - titlewidth) div 2;
							if width < NWmap.bounds.right then
								width := NWmap.bounds.right + 3;
							
							limitrect.left := NWmap.bounds.right;
							limitrect.top := NWmap.bounds.top - 2;
							limitrect.right := NEmap.bounds.left - 3;
							limitrect.bottom := NWmap.bounds.bottom + 2;
							
							savedclip := NewRgn;
							GetClip(savedclip);
							ClipRect(limitrect);
							SectRgn(savedclip, clipRgn, clipRgn);
							
							MoveTo(width, dragrect.bottom - 5);
							DrawString(titlehandle^^);
							
							SetClip(savedclip);
							DisposeRgn(savedclip);
	
							if hilited then begin
								
								NWmap.baseAddr := @NWdata;
								CopyBits(NWmap, portBits, NWmap.bounds,
												NWmap.bounds, srcCopy, nil);
								
								NEmap.baseAddr := @NEdata;
								CopyBits(NEmap, portBits, NEmap.bounds,
												NEmap.bounds, srcCopy, nil);
								
								if (width - 7) > NWmap.bounds.right then begin
									height := closerect.top;
									for index := 1 to 6 do begin
										MoveTo(NWmap.bounds.right, height);
										LineTo(width - 7, height);
										Move(titlewidth + 13, 0);
										LineTo(NEmap.bounds.left - 1, height);
										height := height + 2;
									end;
								end;
							
							end;
							
							if hilited then begin
								SEmap.baseAddr := @SEdata;
								SWmap.baseAddr := @SWdata;
							end else begin
								SWmap.baseAddr := @erasedata;
								SEmap.baseAddr := @erasedata;
							end;
							
							CopyBits(SWmap, portBits, SWmap.bounds,
											SWmap.bounds, srcCopy, nil);
							CopyBits(SEmap, portBits, SEmap.bounds,
											SEmap.bounds, srcCopy, nil);
						
						end;
					
					end;
					HUnlock(dataHandle);
				
				end;
			
			end;

		end;
		
(*******************************************************************
	
	wdohit
	------
	This routine hit-tests a point, and returns the window part that
	the point lies in.  The first section tests for the grow brackets;
	if the point is near the top, and also near the left edge, then
	it's in the top-left bracket, ect.  Note that unhighlighted
	windows don't have grow brackets.  The rest is just testing for
	specific rectangles.
	
*******************************************************************)
	
	function wdohit(thewindow : WindowPeek; thepoint : Point) : long;
		
		label
			100, 200;
		
		var
			therect			:	Rect;
		
		begin
			
			wdocalc(thewindow);
			
			with thewindow^ do begin
				
				HLock(dataHandle);
				with recthandle(dataHandle)^^ do begin
					
					wdohit := wNoHit;
					
					if hilited then begin
						therect := strucrgn^^.rgnBBox;
						if (thepoint.v > (therect.top + 8))
										and (thepoint.v < (therect.bottom - 8)) then
							goto 100;
						if (thepoint.h > (therect.left + 8))
										and (thepoint.h < (therect.right - 8)) then
							goto 100;
						wdohit := wInGrow;
						goto 200;
					end;
					
			100:	if PtInRect(thepoint, closerect) and hilited and goAwayFlag then
						wdohit := wInGoAway
					else if PtInRect(thepoint, zoomrect) and hilited then
						wdohit := wInZoomIn
					else if PtInRect(thepoint, contrgn^^.rgnBBox) then
						wdohit := wInContent
					else if PtInRect(thepoint, dragrect) then
						wdohit := wInDrag;
				
				end;
				HUnlock(dataHandle);
			
			end;

200:	end;
		
(*******************************************************************
	
	wdocalc
	-------
	This routine calculates all the regions and rectangles in the
	window.  Note that these are all in local coordinates, and that
	I assume that the window's portBits.bounds.topleft is the point
	(0, 0).
	
*******************************************************************)
	
	procedure wdocalc(thewindow : WindowPeek);
		
		var
			therect			:	Rect;
	
		begin
			
			with thewindow^ do begin
				
				therect := port.portRect;
				with port.portBits.bounds do
					OffsetRect(therect, - left, - top);
				
				OpenRgn;
					MoveTo(therect.left + 1, therect.top);
					LineTo(therect.right - 1, therect.top);
					LineTo(therect.right - 1, therect.bottom - 9);
					LineTo(therect.right - 9, therect.bottom - 9);
					LineTo(therect.right - 9, therect.bottom - 1);
					LineTo(therect.left + 9, therect.bottom - 1);
					LineTo(therect.left + 9, therect.bottom - 9);
					LineTo(therect.left + 1, therect.bottom - 9);
					LineTo(therect.left + 1, therect.top);
				CloseRgn(contrgn);
			
				with recthandle(dataHandle)^^, therect do begin
					SetRect(dragrect, left, top - 19, right, top);
					SetRect(closerect, left + 12, top - 15, left + 23, top - 4);
					SetRect(zoomrect, right - 23, top - 15, right - 12, top - 4);
					SetRect(NWmap.bounds, left + 1, top - 18, left + 33, top - 4);
					SetRect(NEmap.bounds, right - 33, top - 18, right - 1, top - 4);
					SetRect(SWmap.bounds, left + 1, bottom - 9, left + 9, bottom - 1);
					SetRect(SEmap.bounds, right - 9, bottom - 9, right - 1, bottom - 1);
					SetRect(closemap.bounds, left + 13, top - 14, left + 22, top - 5);
					SetRect(zoommap.bounds, right - 22, top - 14, right - 13, top - 5);
				end;
				
				therect.top := therect.top - 19;
				OpenRgn;
					MoveTo(therect.left, therect.top);
					LineTo(therect.right, therect.top);
					LineTo(therect.right, therect.top + 1);
					LineTo(therect.right + 1, therect.top + 1);
					LineTo(therect.right + 1, therect.bottom + 1);
					LineTo(therect.left + 1, therect.bottom + 1);
					LineTo(therect.left + 1, therect.bottom);
					LineTo(therect.left, therect.bottom);
					LineTo(therect.left, therect.top);
				CloseRgn(strucrgn);
			
			end;

		end;
		
(*******************************************************************
	
	wdonew
	------
	This routine is called by GetNewWindow, so the WDEF can set up
	its private data, which is just what I do here.  All those BitMaps
	were carefully calculated with screen shots and fat bits, and at
	a severe cost to my eyesight!
	
*******************************************************************)
	
	procedure wdonew(thewindow : WindowPeek);
		
		begin
			
			with thewindow^ do begin
				
				dataHandle := NewHandle(sizeof(rectrecord));
				HLock(dataHandle);
				with recthandle(dataHandle)^^ do begin
					
					with NWmap do begin
						baseAddr := @NWdata;
						if goAwayFlag then
							StuffHex(baseAddr, '0100000001000000010000001F5FFDFF1010040017D005FF10100400F7D005FF001004007FD005FF001004007FD005FF001004007FDFFDFF')
						else
							StuffHex(baseAddr, '0100000001000000010000001F7FFFFF1000000017FFFFFF10000000F7FFFFFF000000007FFFFFFF000000007FFFFFFF000000007FFFFFFF');
						rowBytes := 4;
					end;
					
					with NEmap do begin
						baseAddr := @NEdata;
						StuffHex(baseAddr, '000000800000008000000080FFBFFAF800208808FFA08BE800208808FFA08BEF00208800FFBF8BFE00200800FFA00BFE00200800FFBFFBFE');
						rowBytes := 4;
					end;
					
					with SWmap do begin
						baseAddr := @SWdata;
						StuffHex(baseAddr, 'F0001000100010001F00010001000100');
						rowBytes := 2;
					end;
					
					with SEmap do begin
						baseAddr := @SEdata;
						StuffHex(baseAddr, '0F00080008000800F800800080008000');
						rowBytes := 2;
					end;
					
					StuffHex(@erasedata, '00000000000000000000000000000000');
					
					with closemap do begin
						baseAddr := @closedata;
						StuffHex(baseAddr, '080049002A000000E38000002A0049000800');
						rowBytes := 2;
					end;
					
					with zoommap do begin
						baseAddr := @zoomdata;
						StuffHex(baseAddr, '0C004D002E000400E780FC002A0049000800');
						rowBytes := 2;
					end;
				
				end;
				HUnlock(dataHandle);
			
			end;
			
			wdocalc(thewindow);
			
		end;
		
(*******************************************************************
	
	wdodispose
	----------
	This routine is called by DisposeWindow so the WDEF can clean up
	its private data.
	
*******************************************************************)
	
	procedure wdodispose(thewindow : WindowPeek);
	
		begin
			
			DisposHandle(thewindow^.dataHandle);

		end;
		
(*******************************************************************
	
	wdogrow
	-------
	This routine draws a gray facsimile of the window, using the
	rectangle it is passed as the portRect.  It is called by
	GrowWindow (in this program, by growthewindow).
	
*******************************************************************)
	
	procedure wdogrow(therect : rectpointer);
		
		begin
			
			with therect^ do begin
				
				MoveTo(dragrect.left, dragrect.top - 18);
				LineTo(dragrect.right, dragrect.top - 18);
				LineTo(dragrect.right, dragrect.bottom);
				LineTo(dragrect.left, dragrect.bottom);
				LineTo(dragrect.left, dragrect.top - 18);
				
				MoveTo(dragrect.left, dragrect.top);
				LineTo(dragrect.right, dragrect.top);
			
			end;

		end;
		
(*******************************************************************
	
	wdoicon
	-------
	This routine is supposed to draw the grow icon.  This window
	doesn't have a grow icon, so it only draws the outline for the
	scroll bars.
	
*******************************************************************)
	
	procedure wdoicon(thewindow : WindowPtr);
		
		var
			savedport		:	GrafPtr;
	
		begin
			
			GetPort(savedport);
			SetPort(thewindow);
			
			with thewindow^.portRect do begin
				MoveTo(left, bottom - 15);
				LineTo(right, bottom - 15);
				MoveTo(right - 15, top);
				LineTo(right - 15, bottom);
			end;
			
			SetPort(savedport);

		end;
		
(******************************************************************)
		
	end.
		
(******************************************************************)
