(****************************)
(*	file: 	ADSPListen.p		*)
(* 													*)
(* Perform the following: 	*)
(* (1) Listen for incoming	*)
(* 	calls on the connection	*)
(*	listener								*)
(* (2) Check to see if any	*)
(* 	outgoing calls have been*)
(* 	answered								*)
(* (3) Check connections 		*)
(*	for incoming data				*)
(* (4) Check connections for*)
(*	attention messages			*)
(*													*)
(* In:											*)
(* params[1] =	name of 		*)
(* message to call hypercard*)
(* with on incoming data		*)
(*													*)
(* Defaults:								*)
(* container = HyperADSPData*)
(* message 	= none.					*)
(* 													*)
(* -----------------------	*)
(*	All Rights Reserved			*)
(*													*)
(*													*)
(* ------------------------	*)
(* By:		Donald Koscheka		*)
(* Date:	2-Mar-89					*)
(*		All Rights Reserved		*)
(*													*)
(* ------------------------	*)
(****************************)

(*****************************
							Build Sequence

pascal -o "{hpo}"ADSPListen.p.o "{adsp}"ADSPListen.p
link -m ENTRYPOINT  -rt XCMD=1303 -sn Main=ADSPListen
	ADSPListen.p.o
	"{libraries}"Interface.o 
	-o YourStackNameHere
		 
******************************)


{$R-}
{$S ADSPListen}
UNIT Koscheka;

(****************************)
				INTERFACE 
(****************************)


Uses MemTypes, QuickDraw, OSIntf, ToolIntf, PackIntf, HyperXCmd, AppleTalk, ADSP, adspxcmd;


Procedure EntryPoint( paramPtr : XCmdPtr );



(****************************)
				IMPLEMENTATION
(****************************)
TYPE Str31 = String[31];


PROCEDURE ADSPListen( paramPtr: XCmdPtr ); FORWARD;

Procedure EntryPoint( paramPtr : XCmdPtr );
	Begin
		ADSPListen( paramPtr );
	End;
	
	
Procedure ADSPListen( paramPtr : XCmdPtr );
VAR
	myFlags	: SignedByte;	{ flags from incoming unsolicited message	}
	adsp		: ADSPPtr;		{ pointer to ADSP Control block						}
	error		: OSErr;		
	err			: OSErr;
	cn,nn		: cbPtr;			{ pointers to established conections			}
	nbp			: NBPPtr;			{ pointer to the memory  NBP interface		}
	mybPtr	: Ptr;				{ used for getting and setting flags			}
	
	
{$I XCmdGlue.inc }
{$I XCMDADSP.inc }	

PROCEDURE RespondToListen;
(********************************)
(* If the connection listener		*)
(* has completed, then someone	*)
(* is trying to call. If we can	*)
(* initialize a connection, 		*)
(* acknowledge receipt of  call.*)
(********************************)
VAR
	err		: Integer;
	conn	: CBPtr;
	tH		: Handle;
	
BEGIN
	IF  adsp^.pb.ioResult = noErr THEN 
		BEGIN 		{*** We Have an incoming call.		***}
							{*** Create a connection end to 	***}
							{*** the caller and set the				***}
							{*** state to active indicating 	***}
							{*** the connection is accepted		***}
			conn := DSPCreate(adsp^.pb.remoteAddress,
							ATPBSIZE,ATPBSIZE,RETRY,INTERVAL,NIL );
			
			IF conn <> NIL THEN
			WITH conn^.dspPB DO
			BEGIN		{*** accept the connection request***}
				ioCRefNum 			:= adsp^.dspRefNum;
				ioCompletion		:= NIL;
				csCode					:= dspOpen;
				ocMode					:= ocAccept;
				ccbRefNum				:= conn^.ccbRef;
				remoteCID				:= adsp^.pb.remoteCID;
				localCID				:= 0;
				remoteAddress			:= adsp^.pb.remoteAddress;	
				filterAddress.aNet		:= 0;
				filterAddress.aNode		:= 0;
				filterAddress.aSocket := 0;
				RecvSeq					:= 0;
				attnRecvSeq			:= 0;
				SendSeq					:= adsp^.pb.SendSeq;
				attnSendSeq			:= adsp^.pb.attnSendSeq;
				SendWindow			:= adsp^.pb.SendWindow;
				ocInterval			:= INTERVAL;
				ocMaximum				:= RETRY;
				err := PBControl( @conn^.dspPB, ASYNC );
				
				{*** Call back Hypercard to inform 	***}
				{***it of an incoming call.					***}
						
				conn^.remName := NBPGetName( adsp^.pb.remoteAddress );
			END;
					
			{*** Reset connection listener to	***}
			{*** wait for more calls					***}
			err := PBControl( @adsp^.pb, ASYNC );
		END
END;


PROCEDURE ReadADSPData( conn:CBPtr; cnt:Integer; callMe:Handle );
(********************************)
(* Poll the connection to 			*)
(* determine whether any				*)
(* data is pending.  If so, 		*)
(* read it and pass it  back to *)
(* hypercard via the callback 	*)
(* mechanism.										*)
(*															*)
(* IN:													*)
(* conn : pointer to connection *)
(* cnt	: num. of bytes to read	*)
(* callMe	: callback method			*)
(********************************)
VAR
	Tempb		: dspParamBlock;
	err			: Integer;
	temp		: LongInt;
	inSize	: LongInt;
	globName: Str255;	
	tp			: Ptr;
	tH			: Handle;
	
BEGIN
	globName:= 'HyperADSPData';
	
	WITH conn^ DO
	BEGIN	{*** read the data ***}
		IF inBuf = NIL THEN
		BEGIN
			inBuf := NewHandle( LongInt(cnt) );
			MoveHHi( inBuf );
			HLock( inBuf );
			tp	:= inBuf^;
		END
		ELSE			{*** inBuf already allocated, add to end of it	***}
		BEGIN
			HUnlock( inBuf );
			inSize 	:= GetHandleSize(inBuf);
			temp 	:= inSize;
			inSize 	:= inSize + LongInt( cnt );
			SetHandleSize( inBuf, inSize );
			MoveHHi( inBuf );
			HLock( inBuf );
			temp 	:= temp + Ord4( inBuf^ );
			tp		:= Pointer(temp);
		END;
	
		WITH Tempb DO	{*** now read the data ***}
		BEGIN
			ioCRefNum 	:= adsp^.dspRefNum;
			ccbRefNum 	:= conn^.ccbRef;
			csCode 		:= dspRead;
			reqCount	:= cnt;
			dataPtr 	:= tp;			
		END;
		err := PBControl( @Tempb, SYNC );
		HUnlock( inBuf );

		IF err=noErr THEN
		BEGIN
			inSize := GetHandleSize( inBuf );
			inSize := inSize - (cnt-Tempb.actCount);
			SetHandleSize( inBuf, inSize );
		END;
		
		IF (err=noErr) AND (Tempb.eom=1) THEN
		BEGIN
			IF inBuf <> NIL THEN {*** Call back with data	***}
			BEGIN
				IF remName <> NIL THEN
					SetGlobal( 'ADSPCaller', remName );

				tp   	:= Pointer( Ord4( inBuf^ ) + LongInt(inSize) );
				tp^		:= 0;
				
				SetGlobal( globName, inbuf );
				DisposHandle( inbuf );
				inbuf := NIL;
			END;
			
			IF callMe <> NIL THEN {*** Set remote-end name	***}
			BEGIN
				HLock( callMe );
				ZeroToPas( Pointer( callMe^ ), globName );
				HUnlock( callMe );
				SendCardMessage(  globName );
			END;
		END;
	END;{*** read the data ***}
END;





Function CheckMessages( cb : cbPtr ): OSErr;
(************************************)
(* check this connection for:				*)
(* (1) Completed Incoming Messages	*)
(* (2) Completed Outgoing Messages	*)
(************************************)
VAR
	myErr	: OSErr;
	tpb		: dspParamBlock;
BEGIN
	WITH tpb DO
	BEGIN
		csCode 		:= dspStatus;
		ioCRefNum	:= adsp^.dspRefNum;
		ccbRefNum	:= cn^.ccbRef;
	END;
	myErr := PBControl( @tpb, SYNC );
			
	IF myErr = noErr THEN
	BEGIN
		{*** (3) Check for pending incoming data	***}
		IF tpb.recvQPending <> 0  THEN
			ReadADSPData( cn, tpb.recvQPending, paramPtr^.params[1] );

		{*** (4) See if any write operations have completed	***}
		IF (tpb.SendQPending = 0) AND (cn^.OutBuf <> NIL ) THEN
		BEGIN
			HUnlock( cn^.outBuf );
			DisposHandle( cn^.outBuf );
			cn^.outBuf := NIL;
		END;
	END; { myErr <> noErr }
	CheckMessages := myErr;
END;


(**********************************)
(* 					ADSPListen						*)
(**********************************)

BEGIN
	error	:= noErr;
	adsp := ADSPPtr(RetrieveData( 'GLOBALDSPDATA' ));

	IF adsp <> NIL THEN
	BEGIN
		adsp^.checkPoint := RECEIVING;
		cn := adsp^.ends;

		{*** (2) Monitor status on all connections	***}
		WHILE cn <> NIL DO WITH cn^ DO 
		BEGIN
			nn := next;
			err := noErr;
			
			CASE ccb.state OF { connection states } 
				sClosed:	err := DSPHangUp( cn );
							
				sOpen:
				BEGIN
					IF mode = NOP THEN {*** connection is opening 	***}
						BEGIN	{*** inform Hypercard of connection	***}
							SetGlobal( 'ADSPCaller', remName );
							SendCardMessage( 'ConnectionMade' );
							mode := EST;
						END
					ELSE
					BEGIN 		{*** Check an established connection ***}
						myBPtr := @cn^.ccb.userFlags;
						
						IF BAND(myBPtr^, eClosed)=eClosed THEN 
							err := DSPHangUp( cn )
						ELSE
						BEGIN
							err := CheckMessages( cn );
							
							myBPtr^ := 0;		{ clear user flags}
						END;
					END;
				END;
			END; { case of connection states }
			
			IF err <> noErr THEN error := err;
			cn := nn;
		END;
		RespondToListen;		{*** check for opening calls ***}
		adsp^.checkPoint := CLOSE_OK;
	END; {*** IF adsp <> NIL ***}

	paramPtr^.returnValue := PasToZero( NumToStr( LongInt(err) ) );;
END;

end.




