{ Kirk Austin, 4/9/88 }
{This shows how to use a dialog box in a MIDI program}

PROGRAM ShellExample;

	USES
		ROM85, ColorQuickDraw, ColorMenuMgr, LSPMIDI;

{ Global Constants }
	CONST
		Null = '';

		AppleMenuID = 1;
		FileMenuID = 2;
		EditMenuID = 3;
		MIDIMenuID = 4;
		PopMenuID = 206;	{Item number plus the value in the Dialog's RefCon}

		AboutID = 200;
		MIDIDialog = 201;
		MIDIDialogRefCon = 200;

	{Items in our dialog box}
		OKOutline = 8;
		AEditText = 3;
		ArrowETCtl = 4;
		iPopPrompt = 5;	{the Prompt staticText}
		iPopUp = 6;		{the Pop-up userItem}
		ModemCheckBox = 9;
		PrinterCheckBox = 10;

{ Global Types }
	TYPE
		MIDIPrgData = RECORD
				MIDIChan : integer;
				MIDIProg : integer;
				ModemActive : boolean;
				PrinterActive : boolean;
			END;
		MPDPtr = ^MIDIPrgData;
		MPDHdle = ^MPDPtr;

{ Global Variables }

	VAR
		myMenus : ARRAY[AppleMenuID..MIDIMenuID] OF MenuHandle;
		Done : Boolean;			{ true when user selects quit}
		TextCursor : CursHandle; {handle to the text entry cursor}
		ClockCursor : CursHandle; {handle to the waiting watch cursor}
		PopMenuHdle : MenuHandle;
		TheMPDPtr : MPDPtr;
		TheMPDHdle : MPDHdle;
		TheResHdle : handle;
		TheResRefNum : integer;

		TheType : integer;
		TheHandle : Handle;
		TheRect : Rect;


{ }
	PROCEDURE ShowAbout;
		VAR
			theDlog : DialogPtr;
			oldPort : GrafPtr;
	BEGIN
		GetPort(oldPort);
		theDlog := GetNewDialog(AboutID, NIL, Pointer(-1));
		SetPort(theDlog);
		DrawDialog(theDlog);
		WHILE NOT Button DO
			SystemTask;
		DisposDialog(theDlog);
		SetPort(oldPort);
	END;

	PROCEDURE LaunchIt (mode : integer;
									VAR fName : Str255);
{The compiler has just pushed a word for the mode, and a pointer to the string}
	INLINE
		$204F, 	{movea.l  a7,a0		; (a0) is a ptr to string, 4(a0) is mode}
		$A9F2;	{_Launch}


	PROCEDURE DoXfer;
		VAR
			where : Point;
			reply : SFReply;
			vRef : integer;
			thefName : Str255;
			textType : SFTypeList;
	BEGIN
		where.h := 80;
		where.v := 55;
		textType[0] := 'APPL';
		SFGetFile(where, Null, NIL, 1, textType, NIL, reply);
		WITH reply DO
			IF NOT good THEN
				thefName := Null
			ELSE
				BEGIN
					thefName := fName;
					vRef := vRefNum
				END;
		IF thefName <> Null THEN
			BEGIN
				Done := true;
				IF SetVol(NIL, vRef) = noErr THEN
					LaunchIt(0, thefName)
			END
	END;

	PROCEDURE DrawOKOutline (theDialog : DialogPtr;
									theItem : INTEGER);
		VAR
			savePen : PenState;
	BEGIN
		GetPenState(savePen); {save the old pen state}
		GetDItem(TheDialog, TheItem, TheType, TheHandle, TheRect); {get the items rect}
		PenSize(3, 3); {make the pen fatter}
		InsetRect(TheRect, -4, -4);
		FrameRoundRect(TheRect, 16, 16); {draw the ring}
		SetPenState(savePen); {restore the pen state}
	END; {DrawOKOutline}

{The DrawPopUp procedure was made to be as general as possible}
{The main thing to remember is that it expects the PopUpMenuID to}
{be TheItem + TheRefCon of the Dialog.  Also, the Prompt item number}
{must be 1 less than the Pop Up Menu item number}
	PROCEDURE DrawPopUp (TheDialog : DialogPtr;
									TheItem : integer);
		CONST
	{constants for positioning the default item within its box}
			leftSlop = 13;			{leave this much space on left of title}
			rightSlop = 5;			{  this much on right}
			botSlop = 5;			{  this much below baseline}
		VAR
			TheType : integer;
			TheHandle : handle;
			r : Rect;
			TheString : Str255;
			newWid, newLen, wid : INTEGER;
			TheMenuHdle : MenuHandle;
			TheMenuItem : integer;
			i : integer;
			TheChar : char;
			TheRefCon : integer;
			MenuItemsCount : integer;
	BEGIN
	{Get the menu that is associated with this Dialog Item (TheItem + TheRefCon)}
		TheRefCon := LoWord(GetWRefCon(WindowPtr(TheDialog)));
		TheMenuHdle := MenuHandle(GetResource('MENU', TheItem + TheRefCon));
	{Now, figure out which menu item is the current selection by scanning for a check mark}
		MenuItemsCount := CountMItems(TheMenuHdle);
		TheMenuItem := 0;
		i := 1;
		REPEAT
			GetItemMark(TheMenuHdle, i, TheChar);
			IF TheChar = char(CheckMark) THEN
				TheMenuItem := i;
			i := i + 1;
		UNTIL (TheMenuItem <> 0) OR (i = MenuItemsCount + 1);
		IF TheMenuItem = 0 THEN
			BEGIN
				SetItemMark(TheMenuHdle, 1, CHR(checkMark)); {check the first item}
				TheMenuItem := 1;
			END;
		GetItem(TheMenuHdle, TheMenuItem, TheString); {get currently-selected item}
		GetDItem(TheDialog, TheItem, TheType, TheHandle, r);	{set up the rectangle}
		WITH r DO
			BEGIN
				InsetRect(r, -1, -1); {make it a little bigger}

		{Make sure the title fits. Truncate it and add an ellipses ()}
		{if it doesnt (by the way,  is option-semicolon)}
				wid := (right - left) - (leftSlop + rightSlop); {available string area}
				newWid := StringWidth(TheString); {get current width}
				IF newWid > wid THEN
					BEGIN {doesn't fit - truncate it}
						newLen := LENGTH(TheString); {current length in characters}
						wid := wid - CharWidth(''); {subtract width of ellipses}

						REPEAT {until it fits (or we run out of characters)}
				{drop the last character and its width}
							newWid := newWid - CharWidth(TheString[newLen]);
							newLen := PRED(newLen);
						UNTIL (newWid <= wid) OR (LENGTH(TheString) = 0);

			{add the ellipses character}
						newLen := SUCC(newLen); {one more char}
						TheString[newLen] := ''; {its the ellipses}
						TheString[0] := CHR(newLen); {fix the length}
					END;

		{draw the box and its drop shadow}
				FrameRect(r);
				MoveTo(right, top + 2);
				LineTo(right, bottom);
				LineTo(left + 2, bottom);

		{draw the string}
				MoveTo(left + LeftSlop, bottom - BotSlop);
				DrawString(TheString);
			END;
	END; {DrawPopUp}

	FUNCTION PUMenuAction (TheDialog : DialogPtr;
									PopUpItem : integer) : integer;
		VAR
			popLoc : Point;
			newChoice : INTEGER;
			chosen, ignoreLong : LongInt;
			TheString : Str255;
			TheItem : integer;
			TheType : integer;
			TheHandle : handle;
			PromptRect : rect;
			PopUpRect : rect;
			TheMenuHdle : MenuHandle;
			TheRefCon : integer;
			TheMenuID : integer;
			TheMenuItem : integer;
			i : integer;
			TheChar : char;
			MenuItemsCount : integer;

	BEGIN
		PUMenuAction := 0;
	{Get the menu that is associated with this Dialog Item (PopUpItem + TheRefCon)}
		TheRefCon := LoWord(GetWRefCon(WindowPtr(TheDialog)));
		TheMenuID := PopUpItem + TheRefCon;
		TheMenuHdle := MenuHandle(GetResource('MENU', PopUpItem + TheRefCon));
	{Now, figure out which menu item is the current selection by scanning for a check mark}
		MenuItemsCount := CountMItems(TheMenuHdle);
		TheMenuItem := 0;
		i := 1;
		REPEAT
			GetItemMark(TheMenuHdle, i, TheChar);
			IF TheChar = char(CheckMark) THEN
				TheMenuItem := i;
			i := i + 1;
		UNTIL (TheMenuItem <> 0) OR (i = MenuItemsCount + 1);
	{Call PopUpMenuSelect and let the user drag around. Note that the}
	{(top,left) parameters to PopUpMenuSelect are our items, converted to}
	{global coordinates.}
		GetDItem(TheDialog, PopUpItem - 1, TheType, TheHandle, PromptRect);
		GetDItem(TheDialog, PopUpItem, TheType, TheHandle, PopUpRect);
		InvertRect(PromptRect); {hilight the prompt}
		InsertMenu(TheMenuHdle, -1); {insert our menu in the menu list}
		PopLoc := PopUpRect.TopLeft; {copy our items topleft}
		LocalToGlobal(PopLoc); {convert back to global coords}
		CalcMenuSize(TheMenuHdle); {Work around Menu Mgr bug}
		WITH popLoc DO
			chosen := PopUpMenuSelect(TheMenuHdle, v, h, TheMenuItem);
		InvertRect(PromptRect); {unhilight the prompt}
		DeleteMenu(TheMenuID); {remove our menu from the menu list}
	{Was something chosen?}
		IF chosen <> 0 THEN
			BEGIN {yep, something was chosen}
				newChoice := LoWord(chosen); {get the chosen item number}
				IF newChoice <> TheMenuItem THEN
					BEGIN
			{the user chose an item other than the current one}
						SetItemMark(TheMenuHdle, TheMenuItem, ' '); {unmark the old choice}
						SetItemMark(TheMenuHdle, newChoice, CHR(checkMark)); {mark the new choice}
						PUMenuAction := newChoice;
			{Draw the new title}
						EraseRect(PopUpRect);
						DrawPopUp(theDialog, iPopUp);
					END; {if this choice was not the current choice}
			END; {if something was chosen}
	END;{of PUMenuAction}

{The EditText item number must be 1 less than the ArrowUserItem item number}
	PROCEDURE DrawArrowETCtl (TheDialog : DialogPtr;
									TheItem : integer);
		VAR
			theType : Integer;			{ the type of dlog item		}
			theHandle : Handle;			{ Handle to the item		}
			theRect : Rect;				{ rect which encloses the item	}
			Height : Integer;
			HalfHeight : integer;
			ArrowRect : rect;

	BEGIN
		GetDItem(TheDialog, TheItem, theType, theHandle, theRect); {get handle to control}
		FrameRect(TheRect);
		InsetRect(TheRect, 2, 2);
		TheRect.right := TheRect.right - 10;
		FrameRect(TheRect);

		GetDItem(TheDialog, TheItem, theType, theHandle, theRect); {get handle to control}
		Height := TheRect.bottom - TheRect.top;
		HalfHeight := Height DIV 2;
		HalfHeight := TheRect.bottom - HalfHeight;
		TheRect.left := TheRect.right - 11;

		EraseRect(TheRect);
		FrameRect(TheRect);

		MoveTo(TheRect.left, HalfHeight);		{draw bold center line}
		LineTo(TheRect.right - 1, HalfHeight);
		MoveTo(TheRect.left, HalfHeight - 1);
		LineTo(TheRect.right - 1, HalfHeight - 1);

		ArrowRect.top := TheRect.top + 4;	{draw up arrow}
		ArrowRect.bottom := HalfHeight - 2;
		ArrowRect.left := TheRect.left + 3;
		ArrowRect.right := TheRect.right - 3;
		FillRect(ArrowRect, black);
		MoveTo(ArrowRect.left - 1, ArrowRect.top + 1);
		LineTo(ArrowRect.right, ArrowRect.top + 1);
		MoveTo(ArrowRect.left + 1, ArrowRect.top - 1);
		LineTo(ArrowRect.right - 2, ArrowRect.top - 1);
		MoveTo(ArrowRect.left + 2, ArrowRect.top - 2);
		LineTo(ArrowRect.left + 2, ArrowRect.top - 2);

		ArrowRect.top := HalfHeight + 2;
		ArrowRect.bottom := TheRect.bottom - 4;
		ArrowRect.left := TheRect.left + 3;
		ArrowRect.right := TheRect.right - 3;
		FillRect(ArrowRect, black);
		MoveTo(ArrowRect.left - 1, ArrowRect.bottom - 2);
		LineTo(ArrowRect.right, ArrowRect.bottom - 2);
		MoveTo(ArrowRect.left + 1, ArrowRect.bottom);
		LineTo(ArrowRect.right - 2, ArrowRect.bottom);
		MoveTo(ArrowRect.left + 2, ArrowRect.bottom + 1);
		LineTo(ArrowRect.left + 2, ArrowRect.bottom + 1);
	END;

	FUNCTION ArrowCtlAction (TheDialog : DialogPtr;
									TheItem : integer;
									LimitLo : integer;
									LimitHi : integer) : integer;
		VAR
			theType : Integer;
			theHandle : Handle;
			theRect : Rect;
			TheNum : longint;
			TheString : Str255;
			Height : Integer;
			HalfHeight : integer;
			ArrowRect : rect;
			ThePoint : point;
			Inverted : boolean;
			HowLong : longint;
			TickResult : longint;
			UpArrowRect : rect;
			DnArrowRect : rect;
			TheTEHandle : handle;
			GoingUp : boolean;

		PROCEDURE BtnDelay (ticks : integer);
			VAR
				dummy : longint;
				i : integer;
		BEGIN
			i := ticks;
			IF ticks = 0 THEN
				i := 1;
			REPEAT
				Delay(1, dummy);
				i := i - 1;
			UNTIL (i = 0) OR NOT button;
		END;

	BEGIN
		GetDItem(TheDialog, TheItem - 1, theType, theHandle, theRect); {get handle to control}
		TheTEHandle := TheHandle;
		GetIText(TheTEHandle, TheString);
		StringToNum(TheString, TheNum);
		ArrowCtlAction := loword(TheNum);
		GetDItem(TheDialog, TheItem, theType, theHandle, theRect); {get handle to control}
		Height := TheRect.bottom - TheRect.top;
		HalfHeight := Height DIV 2;
		HalfHeight := TheRect.bottom - HalfHeight;
		TheRect.left := TheRect.right - 11;
		UpArrowRect := TheRect;
		UpArrowRect.bottom := HalfHeight;
		DnArrowRect := TheRect;
		DnArrowRect.top := HalfHeight;
		Inverted := false;
		HowLong := 22;

		GetMouse(ThePoint);
		IF (PtInRect(ThePoint, TheRect)) AND Stilldown THEN {we need to hilite an arrow}
			BEGIN
				IF PtInRect(ThePoint, UpArrowRect) THEN
					BEGIN
						ArrowRect := UpArrowRect;
						GoingUp := true;
					END
				ELSE
					BEGIN
						ArrowRect := DnArrowRect;
						GoingUp := false;
					END;
				REPEAT
					GetMouse(ThePoint);
					IF NOT PtInRect(ThePoint, ArrowRect) THEN
						BEGIN
							IF inverted THEN
								BEGIN
									TheRect := ArrowRect;
									InsetRect(TheRect, 1, 1);
									InvertRect(TheRect);
									Inverted := false;
								END;
						END
					ELSE
						BEGIN
							IF NOT Inverted THEN
								BEGIN
									TheRect := ArrowRect;
									InsetRect(TheRect, 1, 1);
									InvertRect(TheRect);
									Inverted := true;
								END;
							GetIText(TheTEHandle, TheString);
							StringToNum(TheString, TheNum);
							IF GoingUp THEN
								BEGIN
									IF TheNum <> LimitHi THEN
										BEGIN
											TheNum := TheNum + 1;
											IF TheNum > LimitHi THEN
												TheNum := LimitHi;
											ArrowCtlAction := loword(TheNum);
											NumToString(TheNum, TheString);
											SetIText(TheTEHandle, TheString);
											SelIText(TheDialog, TheItem - 1, 0, 32767);
										END;
								END
							ELSE
								BEGIN
									IF TheNum <> LimitLo THEN
										BEGIN
											TheNum := TheNum - 1;
											IF TheNum < LimitLo THEN
												TheNum := LimitLo;
											ArrowCtlAction := loword(TheNum);
											NumToString(TheNum, TheString);
											SetIText(TheTEHandle, TheString);
											SelIText(TheDialog, TheItem - 1, 0, 32767);
										END;
								END;
							BtnDelay(HowLong);
							IF HowLong > 3 THEN
								HowLong := HowLong - 2;
						END;
				UNTIL NOT StillDown;
				DrawArrowETCtl(TheDialog, TheItem);
			END;
	END;	{ArrowCtlAction}

	FUNCTION MyDialogFilter (theDialog : DialogPtr;
									VAR theEvent : EventRecord;
									VAR item : integer) : Boolean;
{ This function is called by ModalDialog for every event that occurs }
{ while it is in control. It is used to "filter" the events so you   }
{ can do special things when certain events occur.   It is used to }
{ change the cursor to an I beam when editing text.  The routine }
{ also handles the keyboard entry; limiting text input to numbers }
{ and making return and enter the same as clicking the OK button }

		CONST
			CrCode = 13; {ASCII code for RETURN}
			EnterCode = 3;  {ASCII ccode for ENTER}
			BsCode = 8; {ASCII code for Back Space}
			TabCode = 9;  {ASCII ccode for Tab}
		VAR
			mouseLocation : point;      {will hold coordinates of mouse location}
			TheHandle : Handle;          {used for dummy purpose here}
			TheType : Integer;            {used for dummy purpose here}
			TheRect : Rect;				{used for dummy purpose here}
			TextBox1 : Rect;			{defines area to test for cursor change}
			TheString : Str255;
			TheNum : longint;
			TheDlogRecPtr : DialogPeek;
			TheItem : integer;
			TheChoice : integer;
			TheMenuItem : integer;
			i : integer;
			TheChar : char;
			TheMenuHdle : menuHandle;
			ignorelong : longint;
	BEGIN
		MyDialogFilter := false;  {let modalDialog also handle the event}
		GetDItem(theDialog, AEditText, TheType, TheHandle, TextBox1); {chg cur in area}

		CASE theEvent.what OF
			nullEvent :   {nothing happening so chk if cursor needs to change}
				BEGIN
					GetMouse(mouseLocation);
					IF PtinRect(mouseLocation, TextBox1) THEN
						SetCursor(TextCursor^^)
					ELSE
						SetCursor(arrow);
					GetIText(TheHandle, TheString);	{see if someone typed in an invalid number}
					StringToNum(TheString, TheNum);
					IF TheNum > 128 THEN
						BEGIN
							SysBeep(10);
							TheNum := 128;
							NumToString(TheNum, TheString);
							SetIText(TheHandle, TheString);
							SelIText(TheDialog, AEditText, 0, 32767);	{hilite the editable text}
						END;
					IF TheNum < 1 THEN
						BEGIN
							SysBeep(10);
							TheNum := 1;
							NumToString(TheNum, TheString);
							SetIText(TheHandle, TheString);
							SelIText(TheDialog, AEditText, 0, 32767);	{hilite the editable text}
						END;
				END;

			mouseDown : 
				BEGIN {"Click!"}
					mouseLocation := theEvent.where; {copy the mouse position}
					GlobalToLocal(mouseLocation); {convert it to local coordinates}

			{Was the click in a user item?}
					IF (FindDItem(theDialog, mouseLocation) + 1) = 4 THEN
						BEGIN
							TheMPDPtr^.MIDIProg := ArrowCtlAction(TheDialog, 4, 1, 128);
						END; {if clicked in ArrowEditTextCtl userItem}

					IF (FindDItem(theDialog, mouseLocation) + 1) = iPopUp THEN
						BEGIN {Clicked in the pop-up box}
							TheChoice := PUMenuAction(TheDialog, iPopUp);
							IF TheChoice <> 0 THEN
								BEGIN
									MyDialogFilter := TRUE; {dialog is over}
									TheMPDPtr^.MIDIChan := TheChoice;
									item := iPopUp; {have ModalDialog return that the user changed items}
								END;
						END; {if clicked in our userItem}
				END; {mousedown case}

			keyDown, autokey :    {to follow std. procedure, chk if RETURN or ENTER was pressed}
				BEGIN
					IF (theEvent.message MOD 256) IN [crCode, enterCode] THEN
						BEGIN
							GetDItem(theDialog, 1, TheType, TheHandle, TheRect);
							HiLiteControl(ControlHandle(TheHandle), 1);			{hilite the OK button}
							Delay(3, ignoreLong);
							HiliteControl(ControlHandle(TheHandle), 0);
							MyDialogFilter := true;
							Item := 1;
						END
					ELSE IF (theEvent.message MOD 256) IN [bsCode, tabCode] THEN
						BEGIN
						END
					ELSE IF (Char(theEvent.message MOD 256) >= '0') AND (Char(theEvent.message MOD 256) <= '9') THEN
						BEGIN
							TheDlogRecPtr := DialogPeek(theDialog);
							TheItem := TheDlogRecPtr^.editField + 1;	{ find out which EditText Item we are in }
							GetDItem(theDialog, TheItem, TheType, TheHandle, TheRect);
							GetIText(TheHandle, TheString);
							IF (Length(TheString) > 2) THEN
								SetIText(TheHandle, Null);		{ set it to Null if there are more than 3 characters }
						END
					ELSE
						MyDialogFilter := true;
				END;
			OTHERWISE
				;
		END; {of the CASE statment}
	END;

	PROCEDURE SendMIDI;
		VAR
			dummy : longint;
	BEGIN
		IF TheMPDPtr^.ModemActive THEN
			BEGIN
				InitSCCA;
				TXMIDIA(TheMPDPtr^.MIDIChan + 191);
				TXMIDIA(TheMPDPtr^.MIDIProg - 1);
				Delay(1, dummy);
				ResetSCCA;
			END;
		IF TheMPDPtr^.PrinterActive THEN
			BEGIN
				InitSCCB;
				TXMIDIB(TheMPDPtr^.MIDIChan + 191);
				TXMIDIB(TheMPDPtr^.MIDIProg - 1);
				Delay(1, dummy);
				ResetSCCB;
			END;
	END;	{of SendMIDI}

{ }
	PROCEDURE ProcessMenu (codeWord : Longint);	{ handle menu selections}
		VAR
			menuNum : Integer;
			itemNum : Integer;
			NameHolder : str255;
			dummy : Integer;
			yuck : boolean;
			oldPort : GrafPtr;
			aDialog : DialogPtr;
			ItemHit : integer;
			TheItemHandle : handle;
			TheItemType : integer;  {type of the selected item}
			TheItemRect : rect;     {bounding box of the selected item}
			TheNum : longint;
			TheString : Str255;

	BEGIN
		IF codeWord <> 0 THEN	{ nothing was selected}
			BEGIN
				menuNum := HiWord(codeWord);
				itemNum := LoWord(codeWord);
				CASE menuNum OF { the different menus}
					AppleMenuID : 
						BEGIN
							IF itemNum < 3 THEN
								BEGIN
									ShowAbout;
								END
							ELSE
								BEGIN
									GetItem(myMenus[AppleMenuID], itemNum, NameHolder);
									dummy := OpenDeskAcc(NameHolder);
								END;
						END;
					FileMenuID : 
						BEGIN
							CASE ItemNum OF
								1 : 
									BEGIN
										DoXfer;
									END;
								2 : 
									BEGIN
										Done := true;
									END;
							END;
						END;
					EditMenuID : 
						BEGIN
							yuck := SystemEdit(itemNum - 1);
						END;
					MIDIMenuID : 
						BEGIN
							GetPort(oldPort);
				{Get a menu}
							PopMenuHdle := GetMenu(PopMenuID); {Create a menu (its title is ignored)}
							SetItemMark(PopMenuHdle, TheMPDPtr^.MIDIChan, CHR(checkMark)); {check it}
							aDialog := GetNewDialog(MIDIDialog, NIL, WindowPtr(-1));
							SetPort(aDialog);
							SetWRefCon(WindowPtr(aDialog), MIDIDialogRefCon);
			{set the defaults}
							GetDItem(aDialog, AEditText, theType, TheHandle, TheRect);
							TheNum := TheMPDPtr^.MIDIProg;
							NumToString(TheNum, TheString);
							SetIText(TheHandle, TheString);
							GetDItem(aDialog, ModemCheckBox, theType, TheHandle, TheRect);
							IF TheMPDPtr^.ModemActive THEN
								SetCtlValue(ControlHandle(TheHandle), 1);
							GetDItem(aDialog, PrinterCheckBox, theType, TheHandle, TheRect);
							IF TheMPDPtr^.PrinterActive THEN
								SetCtlValue(ControlHandle(TheHandle), 1);
				{Find out where our UserItems are, and set their item handles to be}
				{a pointer to the drawing procedures}
							GetDItem(aDialog, ArrowETCtl, theType, TheHandle, TheRect);
							SetDItem(aDialog, ArrowETCtl, theType, @DrawArrowETCtl, TheRect);
							GetDItem(aDialog, iPopUp, theType, TheHandle, TheRect);
							SetDItem(aDialog, iPopUp, theType, @DrawPopUp, TheRect);
							GetDItem(aDialog, OKOutline, theType, TheHandle, TheRect);
							SetDItem(aDialog, OKOutline, theType, @DrawOKOutline, TheRect);
							SelIText(aDialog, 3, 0, 32767);	{hilite the editable text}
							ShowWindow(WindowPtr(aDialog));	{show the window}
							REPEAT
								ModalDialog(@MyDialogFilter, ItemHit);     {will process all events while dialog is up}
								CASE itemHit OF

									1 : 
										BEGIN {this is the Send Button item}
											SendMIDI;
										END;

									2 : 
										BEGIN {this is the Done Button item}
											DisposDialog(aDialog);
											SetPort(oldPort);
										END;

									iPopUp : 
										BEGIN {this is the PopUpMenu item}
											SysBeep(1);
										END;

									ModemCheckBox : 
										BEGIN {this is the Modem Check Box item}
											GetDItem(aDialog, ModemCheckBox, theType, TheHandle, TheRect);
											IF TheMPDPtr^.ModemActive THEN
												BEGIN
													TheMPDPtr^.ModemActive := false;
													SetCtlValue(ControlHandle(TheHandle), 0);
												END
											ELSE
												BEGIN
													TheMPDPtr^.ModemActive := true;
													SetCtlValue(ControlHandle(TheHandle), 1);
												END;
										END;

									PrinterCheckBox : 
										BEGIN {this is the Printer Check Box item}
											GetDItem(aDialog, PrinterCheckBox, theType, TheHandle, TheRect);
											IF TheMPDPtr^.PrinterActive THEN
												BEGIN
													TheMPDPtr^.PrinterActive := false;
													SetCtlValue(ControlHandle(TheHandle), 0);
												END
											ELSE
												BEGIN
													TheMPDPtr^.PrinterActive := true;
													SetCtlValue(ControlHandle(TheHandle), 1);
												END;
										END;

									OTHERWISE
										;
								END; {case of item hit}
							UNTIL ItemHit = 2;	{We're done with the Dialog}
						END;
				END;
				HiliteMenu(0);
			END;
	END;

{ }
	PROCEDURE DealWithMouseDowns (theEvent : EventRecord);
		VAR
			location : Integer;
			windowPointedTo : WindowPtr;
			mouseLoc : point;
			windowLoc : integer;
			VandH : Longint;
			Height : Integer;
			Width : Integer;
	BEGIN
		mouseLoc := theEvent.where;
		windowLoc := FindWindow(mouseLoc, windowPointedTo);
		CASE windowLoc OF
			inMenuBar : 
				BEGIN
					ProcessMenu(MenuSelect(mouseLoc));
				END;
			inSysWindow : 
				BEGIN
					SystemClick(theEvent, windowPointedTo);
				END;
			inContent : 
				BEGIN
					IF windowPointedTo <> FrontWindow THEN
						BEGIN
							SelectWindow(windowPointedTo);
						END;
				END;
			OTHERWISE
				BEGIN
				END;
		END;
	END;

{ }
	PROCEDURE DealWithKeyDowns (theEvent : EventRecord);
		TYPE
			Trick = PACKED RECORD
					CASE boolean OF
						true : (
								long : Longint
						);
						false : (
								chr3, chr2, chr1, chr0 : char
						)
				END;
		VAR
			CharCode : char;
			TrickVar : Trick;
	BEGIN
		TrickVar.long := theEvent.message;
		CharCode := TrickVar.chr0;
		IF BitAnd(theEvent.modifiers, CmdKey) = CmdKey THEN {check for a menu selection}
			BEGIN
				ProcessMenu(MenuKey(CharCode));
			END
	END;

{ }
	PROCEDURE DealWithActivates (theEvent : EventRecord);
		VAR
			TargetWindow : WindowPtr;

	BEGIN
		TargetWindow := WindowPtr(theEvent.message);
		IF Odd(theEvent.modifiers) THEN
			BEGIN
				SetPort(TargetWindow);
			END
		ELSE
			BEGIN

			END;
	END;

{ }
	PROCEDURE DealWithUpdates (theEvent : EventRecord);
		VAR
			UpDateWindow : WindowPtr;
			tempPort : WindowPtr;

	BEGIN
		UpDateWindow := WindowPtr(theEvent.message);
		GetPort(tempPort);
		SetPort(UpDateWindow);
		BeginUpDate(UpDateWindow);

		EndUpDate(UpDateWindow);
		SetPort(tempPort);
	END;

{ }
	PROCEDURE MainEventLoop;
		VAR
			Event : EventRecord;
			ProcessIt : boolean;
			x : byte;
	BEGIN
		REPEAT
			SystemTask;
			ProcessIt := GetNextEvent(everyEvent, Event); { get the next event in queue}
			IF ProcessIt THEN
				BEGIN
					CASE Event.what OF
						mouseDown : 
							DealWithMouseDowns(Event);
						AutoKey : 
							DealWithKeyDowns(Event);
						KeyDown : 
							DealWithKeyDowns(Event);
						ActivateEvt : 
							DealWithActivates(Event);
						UpdateEvt : 
							DealWithUpdates(Event);
						OTHERWISE
							BEGIN
							END;
					END;
				END;
		UNTIL Done;
	END;

{ }
	PROCEDURE MakeMenus;		{ get the menus & display them}
		VAR
			index : Integer;
	BEGIN
		FOR index := AppleMenuID TO MIDIMenuID DO
			BEGIN
				myMenus[index] := GetMenu(index);
				InsertMenu(myMenus[index], 0);
			END;
		AddResMenu(myMenus[AppleMenuID], 'DRVR');
		DrawMenuBar;
	END;

{ Program Starts Here }
BEGIN
	Done := false;
	FlushEvents(everyEvent, 0);

   { initialize routines go here }
   {get the cursors we use and lock them down - no clutter}
	ClockCursor := GetCursor(watchCursor);
	TextCursor := GetCursor(iBeamCursor);
	HLock(Handle(ClockCursor));
	HLock(Handle(TextCursor));

	MakeMenus;

	TheResHdle := GetResource('MIDI', 128);
	HLock(TheResHdle);
	TheMPDHdle := MPDHdle(TheResHdle);
	TheMPDPtr := TheMPDHdle^;

	InitCursor;
	MainEventLoop;

	ChangedResource(TheResHdle);
	TheResRefNum := HomeResFile(TheResHdle);
	UpdateResFile(TheResRefNum);
	ReleaseResource(TheResHdle);
END.