/*------------------------------------------------------------------------------

NAME
	 Pict2Rgn -- convert PICT resources to Quickdraw region format.

SYNOPSIS
	 Pict2Rgn [file] [-v]

	file	= any resource file containing PICT resources
	-v		= verbose flag
	
DESCRIPTION
	 Converts PICT resources within a specified file to Quickdraw
	 region format.  The id's of the generated regions will be the
	 same as the source PICT's.  The region resource type is 'RGN '.
	 Does not check to see if the RGN resources are already there.

AUTHOR
	 Written Ted Cohn, March 1988.
	 Copyright Ted Cohn, 1988.

------------------------------------------------------------------------------*/

#include <types.h>
#include <stdio.h>
#include <resources.h>
#include <quickdraw.h>
#include <memory.h>

pascal RgnHandle MakeRgn(srcMap)
	BitMap *srcMap;
	extern;

#define RgnType 'RGN '

main(argc, argv)
int		argc;
char	*argv[];
{
	Rect		frame;
	GrafPort	gp;
	short		index;
	short		refNum;
	short		pictCount;
	Handle		pictHandle;
	short		rowWords;
	BitMap		srcMap;
	short		theID;
	ResType		theType;
	Handle		theRgn;
	Str255		theName;
	short		verbose;

	/* Must initialize Quickdraw for the conversion process later */
	InitGraf(&qd.thePort);
	
	printf("\n");
	if (argc<2) {
		fprintf(stderr,"\nUsage:  Pict2Rgn <resource file>");
		fprintf(stderr,"\n  -Converts all PICT resources to Quickdraw region format.");
		fprintf(stderr,"\n  -ResType is 'RGN ', ID's correspond to PICT ID's.");
		fprintf(stderr,"\n  -Written by Ted Cohn, March, 1988.\n");
		exit(1);
	}

	if ((refNum = OpenResFile(argv[1])) == -1) {
		fprintf(stderr, "\nCan't open the resource file %s.\n", argv[1]);
		exit(1);
	}
	
	verbose = (argc>2);

	/* Find out how many pictures we have to convert */
	if ((pictCount = Count1Resources('PICT')) == 0) {
		fprintf(stderr, "\nThere are no PICTs in this file to convert.\n");
		CloseResFile(refNum);
		exit(1);
	}
	
	if (verbose)
		 fprintf(stderr, "PICT Count = %d\n", pictCount);

	/* Now convert each PICT to RGN format */
	for (index = 1; index <= pictCount; index++) {	
		
		/* Get a picture and lock it down */
		HLock(pictHandle = Get1IndResource('PICT', index));
		
		/* Must get the picture's ID */
		GetResInfo(pictHandle, &theID, &theType, &theName);
		
		/* Get the picture's frame rectangle */
		frame = *((Rect *) (*pictHandle + 2));
		
		/* Calculate size of srcMap we need to allocate */
		rowWords = (frame.right-frame.left-1)/16+1;
		srcMap.baseAddr = NewPtr((long) ((frame.bottom-frame.top)*rowWords*2));
		srcMap.rowBytes = rowWords*2;
		srcMap.bounds = frame;
		
		/* Open a temporary port which we can draw into */
		OpenPort(&gp);
		SetPortBits(&srcMap);
		RectRgn(gp.clipRgn, &frame);
		RectRgn(gp.visRgn, &frame);
		gp.portRect = frame;
		DrawPicture((PicHandle) pictHandle, &frame);
		ClosePort(&gp);
		
		/* Now convert the picture from a srcMap to a region */
		theRgn = MakeRgn(&srcMap);
		ReleaseResource(pictHandle);
		DisposPtr(srcMap.baseAddr);
		
		if (verbose)
			fprintf(stderr, "Converted (%d) PICT ID = %d\n", index, theID);
		
		/* Add the new region to the file */
		AddResource(theRgn, RgnType, theID, &theName);
		WriteResource(theRgn);
		ReleaseResource(theRgn);
	}
	/* All done, so close the resource file */
	CloseResFile(refNum);
	exit(0);
}
