#include <String.h>
#include <Stream.h>
#include <StdLib.h>

void error (char* p)
{
	cerr << p << endl;
	exit(1);
}

class vector 
{
    float* v;
    int sz;
	friend matrix;
	
public:
    vector(int);
    vector(vector&); 
	~vector();
	int size () { return sz; }
    void set_size(int);
    float& operator[](int);
    float elem(int i) { return v[i]; }
	vector& operator= (vector&); // assignment
	float operator* (vector&); // scalar product
	vector& operator+ (vector&); // elementwise addition
	vector& operator- (vector&); // and subtraction
	void print(); 

};

vector::vector (int size)
{
	cout << "constructing vector object" << endl;
	if (size<=0) 
		{ v = 0; return; }
	sz = size;
	v = new float[sz];
}

vector::vector (vector& a)
{
	cout << "copying vector object" << endl;
	sz = a.sz;
	v = new float[sz];
	for (register int i=0 ; i<sz ; i++)
		v[i] = a.v[i];
}

vector::~vector ()
{
	cout << "deleting vector object" << endl;
	delete v;
}

float&
vector::operator[] (int i)
{
	if (i<0 || i>= sz) 
		error("class vector: index out of range");
	return v[i]; 
}

vector&
vector::operator= (vector& a)
{
	if (a.sz != sz)
		error("class vector: size mismatch in =");
	for (register int i=0 ; i<sz ; i++)
		v[i] = a.v[i];
	return a;
}

vector&
vector::operator+ (vector& a)
{
	if (sz != a.sz)
		error("class vector: size mismatch in +");
	vector& c = *new vector(sz);
// DON'T USE vector c(sz); 
// this would be destroyed on exiting the method
	for (register int i=0 ; i<sz ; i++)
		c.v[i] = v[i]+a.v[i];
	return c;
}	

vector&
vector::operator- (vector& a)
{
	if (sz != a.sz)
		error("class vector: size mismatch in +");
	vector& c = *new vector(sz);
	for (register int i=0 ; i<sz ; i++)
		c.v[i] = v[i]-a.v[i];
	return c;
}	

float
vector::operator* (vector& a)
{
	if (sz != a.sz)
		error("class vector: size mismatch in *");
	float sum = 0;
	for (register int i=0 ; i<sz ; i++)
		sum = sum + v[i]*a.v[i];
	return sum;
}	

void
vector::print()
{
	for (int i=0; i<sz-1; i++)
		cout << v[i] << " | ";
	cout << v[sz-1] << endl;
}

class matrix
{
    float* mv;
	int rows, cols;
	 
public:
	matrix(int,int); 
	matrix(matrix&);
	~matrix();
	int rowsize () { return rows; }
	int colsize () { return cols; }
	void set_size(int,int);
	float& operator()(int,int);
	float elem(int i, int j) { return mv[i*cols + j]; }
	matrix& operator= (matrix&); // assignment
	matrix& operator+ (matrix&); // elementwise addition
	matrix& operator- (matrix&); // elementwise subtraction	
	matrix& operator* (matrix&); // matrix product
	vector& operator* (vector&); // matrix*vector
	void print();
};

matrix::matrix (int rowsz, int colsz)
{
	cout << "constructing matrix object" << endl;
	if (rowsz <=0 || colsz <=0) 
		{ mv = 0; return; }
	rows = rowsz;
	cols = colsz;
	mv = new float[rows*cols];
}

matrix::matrix (matrix& a)
{
	cout << "copying matrix object" << endl;
	rows = a.rows; cols = a.cols;
	int size = rows*cols;
	mv = new float[size];
	for (register int i=0 ; i<size ; i++)
		mv[i] = a.mv[i];
}


matrix::~matrix ()
{
	cout << "deleting matrix object" << endl;
	delete mv;
}

float&
matrix::operator() (int i, int j) 
{ 
	if (i<0 || i>= rows) 
		error("class matrix: column index out of range");	
	if (j<0 || j>= cols) 
		error("class matrix: row index out of range");	
	return mv[i*cols + j]; 
}

matrix&
matrix::operator= (matrix& a)
{
	if ((a.rows != rows) || (a.cols != cols))
		error("class matrix: size mismatch in assignment");
	for (register int i=0 ; i<rows*cols ; i++)
		mv[i] = a.mv[i];
	return a;
}

matrix&
matrix::operator+ (matrix& a)
{
	if ((rows != a.rows) || (cols != a.cols))
		error("class matrix: size mismatch in +");
	matrix& c = *new matrix(rows,cols);
	for (register int i=0 ; i<rows*cols ; i++)
		c.mv[i] = mv[i]+a.mv[i];
	return c;
}	

matrix&
matrix::operator- (matrix& a)
{
	if ((rows != a.rows) || (cols != a.cols))
		error("class matrix: size mismatch in +");
	matrix& c = *new matrix(rows,cols);
	for (register int i=0 ; i<rows*cols ; i++)
		c.mv[i] = mv[i]-a.mv[i];
	return c;
}	

matrix&
matrix::operator* (matrix& a)
{
	if (cols != a.rows)
		error("class matrix: size mismatch in *");
	matrix& c = *new matrix(rows,a.cols);
	for (register int i=0 ; i<rows; i++)
		for (register int j=0 ; j<a.cols; j++)
		{
			register float sum = 0;
			for (register int k=0 ; k<cols; k++)
				sum = sum + elem(i,k)*a.elem(k,j); // elem is faster
			c(i,j) = sum;
		}
	return c;
}	

vector&
matrix::operator* (vector& a)
{
	if (cols != a.sz)
		error("class matrix: size mismatch in *vector");
	vector& c = *new vector(rows);
	for (register int i=0 ; i<rows; i++)
		{
			register float sum = 0;
			for (register int j=0 ; j<cols; j++)
				sum = sum + elem(i,j)*a.v[j]; 
			c[i] = sum;
		}
	return c;
}	

void
matrix::print()
{
	for (int i=0; i<rows; i++)
	{
		for (int j=0; j<cols-1; j++)
			cout << elem(i,j) << " | ";
		cout << elem(i,cols-1) << endl;
	}
}	


int
main(int argc, char* argv[])
{
	cout << "Start main program..." << endl;
	
	int i,j;

	cout << "Defining A and C" << endl;	
	matrix a(3,5);
	matrix c(5,3);
	
	for (i=0; i<3; i++)
		for(j=0; j<5; j++)
		{
			a(i,j) = i + (1.0*j)/10;
			c(j,i) = a(i,j) - 2;
		}

	cout << "Assigning B" << endl;	
	matrix b = a;
	
	cout << "Elements of A:" << endl;
	a.print(); 
	
	cout << "Elements of B:" << endl;
	b.print();
	
	cout << "Elements of C:" << endl;
	c.print();
	
//	cout << "Defining X" << endl;	
//	matrix x(3,3);
	
	cout << "Multiplying A*C into X" << endl;	
	matrix& x = a*c;
	
	cout << "Elements of A*C:" << endl;
	x.print();
	
	
	cout << "Defining D" << endl;	
	vector d(3);
	for (i=0; i<3; i++)
		d[i] = i*2.75;

	cout << "Defining E" << endl;	
	vector e = d;
	
	cout << "Multiplying C*E into F" << endl;	
	vector& f = c*e;
	
	cout << "Elements of D, E, F:" << endl;
	d.print(); e.print(); f.print(); 
	
	a(1,4) = 432.056;
	b = a;
	
	a.print(); b.print();

	delete &x; delete &f;	
	
	cout << "Exiting" << endl;
	return 0;
}