/* File ResBTMPEd.c 
 *
 * ResEdit editor for 'BTMP' resources.
 * by Mike Scanlin  12 June 1989
 *
 * portions of this code are from the example
 * source ResXXXXEd.c which is provided by 
 * and copyrighted by Apple Computer, Inc.
 */

#include	<types.h>
#include	<memory.h>
#include	<menus.h>
#include	<resources.h>
#include	<Packages.h>
#include	"ResEd.h"
#include	"ResIDs.h"

#define	DFLT_FAT_BIT_SIZE	8		/* must be [4..9] */
#define	DFLT_BTMP_WIDTH		20
#define DFLT_BTMP_HEIGHT	10
#define	DFLT_ROWBYTES			((((DFLT_BTMP_WIDTH-1)>>4)+1)<<1)
#define	DFLT_BTMP_SIZE		(DFLT_BTMP_HEIGHT*DFLT_ROWBYTES)
#define kLockStateFlag		0x80

/* data structure for resource type 'BTMP' */
typedef struct BTMP {
	Ptr						baseAddr;
	short					rowBytes;
	Rect					bounds;
	short					bitImage[];
} BTMP, *BTMPPtr, **BTMPHndl;

/* our editor's window's refCon has a handle to this */
typedef struct rBTMPRec {
	ParentHandle	father;
	Str64					name;
	WindowPtr			windPtr;
	Boolean				rebuild;
	BTMPHndl			hBTMP;
	short					fatBitSize;
} rBTMPRec, *rBTMPRecPtr, **rBTMPHndl;


/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

pascal void PickBirth(ResType t, ParentHandle dad)
/* PickBirth is not used for editors */
{
	/* the following is to prevent compiler warnings 
	 * about unused parameters */
	#pragma	unused (t, dad)
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void CalcWindowSize(BTMPHndl theBitMapHndl,
	short fatBitSize, short *wWidth, short *wHeight)
{	
	short			bitMapWidth, bitMapHeight;
	BTMPPtr		theBitMapPtr;
	
	theBitMapPtr = *theBitMapHndl;
	bitMapWidth = DFLT_BTMP_WIDTH;
	bitMapHeight = DFLT_BTMP_HEIGHT;
	/* if we're not being called for the default bitmap, 
	 * then use the actual size */
	if (GetHandleSize((Handle)theBitMapHndl) != 0) {
		bitMapWidth = theBitMapPtr->bounds.right - 
			theBitMapPtr->bounds.left;
		bitMapHeight = theBitMapPtr->bounds.bottom - 
			theBitMapPtr->bounds.top;
	}
	
	*wWidth = fatBitSize * bitMapWidth + 1;
	*wHeight = (fatBitSize + 1) * bitMapHeight + 4;
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

void GetNameAndTitle(StringPtr windowTitle,
	StringPtr windowName, Handle theBitMapHndl)
{	
	strcpy(windowTitle, "\pBTMP");
	SetETitle(theBitMapHndl, windowTitle);
	strncpy(windowName, windowTitle, *windowTitle + 1);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

pascal void EditBirth(Handle theBitMapHndl, 
	ParentHandle dad)
{
	rBTMPHndl			myBTMP;
	rBTMPRecPtr		pMyBTMP;
	WindowPtr			myWindow;
	Str255				windowTitle, newName;
	short					windowWidth, windowHeight;

	GetNameAndTitle(&windowTitle, &newName, theBitMapHndl);
	CalcWindowSize((BTMPHndl)theBitMapHndl, 
		DFLT_FAT_BIT_SIZE, &windowWidth, &windowHeight);

	myWindow = EditorWindSetup(false, windowWidth, 
		windowHeight, &windowTitle, &newName, true, dad);
	
	/* if we got a new window (i.e. no errors), 
	 * then start up the editor */
	if (myWindow != NULL) {
		/* if theBitMapHndl is empty, then user did 'New' */
		if ((GetHandleSize(theBitMapHndl)) == 0) { 
			BTMPPtr		theBitMapPtr;

			/* allocate some space for the default bitmap */
			FixHand(sizeof(BTMP) + DFLT_BTMP_SIZE, theBitMapHndl);
			/* Note: FixHand has initialized the structure 
			 * with zeros */
			theBitMapPtr = *(BTMPHndl)theBitMapHndl;
			theBitMapPtr->rowBytes = DFLT_ROWBYTES;
			SetRect(&theBitMapPtr->bounds, 0, 0, 
				DFLT_BTMP_WIDTH, DFLT_BTMP_HEIGHT);
		}

		/* create the instance record used to communicate 
		 * with ResEdit and initialize fields */
		myBTMP = (rBTMPHndl)NewHandle(sizeof(rBTMPRec));

		pMyBTMP = *myBTMP;
		pMyBTMP->windPtr = myWindow;
		pMyBTMP->father = dad;
		BlockMove(&newName, &pMyBTMP->name, newName[0] + 1);
		pMyBTMP->hBTMP = (BTMPHndl)theBitMapHndl;
		pMyBTMP->fatBitSize = DFLT_FAT_BIT_SIZE;
		((WindowPeek)myWindow)->refCon = (long)myBTMP;
		((WindowPeek)myWindow)->windowKind = ResEdID();
	}
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void PlotPBitMap(Rect *r, BTMPPtr theBitMapPtr,
  short mode)
/* Analagous to PlotIcon except that you pass a pointer 
 * instead of a handle. Mode specifies the transfer
 * mode to use which can be any of the source transfer
 * modes. This routine assumes that the handle to the 
 * bitMap is locked. */
{
	GrafPtr				port;
	
	GetPort(&port);
	CopyBits((BitMap *)theBitMapPtr, &port->portBits, 
		&theBitMapPtr->bounds, r, mode, NULL);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

char LockBitMap(BTMPHndl theBitMapHndl)
/* LockBitMap() locks the BTMP specified by 
 * theBitMapHndl and sets the baseAddr field to point
 * to the bitImage field of the BTMP. The function's
 * return value is the original value of the byte that
 * contains the flags of the master pointer for the
 * given handle (as returned by HGetState()).
 * Warning: If the handle is not locked on entry then
 * the OS routine MoveHHi() is called before locking
 * the handle. Therefore, this routine may move or
 * purge blocks in the heap.
 * m_o 07.09.89 */
{
	char				origState;
	BTMPPtr			theBitMapPtr;

	origState = HGetState((Handle)theBitMapHndl);
	if (!(origState & kLockStateFlag)) {
		MoveHHi((Handle)theBitMapHndl);
		HLock((Handle)theBitMapHndl);
	}
	theBitMapPtr = *theBitMapHndl;
	theBitMapPtr->baseAddr = (Ptr)&theBitMapPtr->bitImage;
	return(origState);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void PlotBitMap(Rect *r, BTMPHndl theBitMapHndl,
  short mode)
/* Analagous to PlotIcon but it works with BTMPs, 
 * not ICONs. Mode specifies the transfer mode to
 * use which can be any of the source transfer
 * modes. */
{
	char					origState;
	
	origState = LockBitMap(theBitMapHndl);	
	PlotPBitMap(r, *theBitMapHndl, mode);
	(*theBitMapHndl)->baseAddr = NULL;
	HSetState((Handle)theBitMapHndl, origState);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void PlotCenteredBitMap(BTMPHndl theBitMapHndl)
/* Custom to this editor. This plots the current BTMP 
 * in the top center of the editing window. */
{
	GrafPtr				port;
	Rect					theRect;
	BTMPPtr				theBitMapPtr;
	
	theBitMapPtr = *theBitMapHndl;
	theRect = theBitMapPtr->bounds;
	GetPort(&port);
	OffsetRect(&theRect, ((port->portRect.right - 
		port->portRect.left - (theBitMapPtr->bounds.right - 
		theBitMapPtr->bounds.left)) >> 1) - 
		theBitMapPtr->bounds.left, 
		1 - theBitMapPtr->bounds.top);
	PlotBitMap(&theRect, theBitMapHndl, srcCopy);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void DrawBTMPWindow(rBTMPHndl myBTMP)
{
	rBTMPRecPtr		pMyBTMP;
	BTMPPtr				theBitMapPtr;
	Rect					theRect;
	Rect					*r;
	short					i, j, nRows, nCols, fatBitSize, 
								currentWord, nPixels;
	short					*bitImagePtr;
	
	BubbleUp((Handle)myBTMP);
	HLock((Handle)myBTMP);
	pMyBTMP = *myBTMP;
	
	BubbleUp((Handle)pMyBTMP->hBTMP);
	HLock((Handle)pMyBTMP->hBTMP);
	theBitMapPtr = *pMyBTMP->hBTMP;
	
	/* draw the actual size bitmap */
	PlotCenteredBitMap(pMyBTMP->hBTMP);
	
	/* draw the line above the fatBits */
	nRows = theBitMapPtr->bounds.bottom - 
		theBitMapPtr->bounds.top;
	MoveTo(0, i = nRows + 2);
	LineTo((pMyBTMP->windPtr)->portRect.right, i);

	/* draw the fat bits */
	fatBitSize = pMyBTMP->fatBitSize;
	nCols = theBitMapPtr->bounds.right - 
		theBitMapPtr->bounds.left;
	r = &theRect;
	r->bottom = (r->top = nRows + 4) + fatBitSize - 1;
	bitImagePtr = &theBitMapPtr->bitImage;
	
	for (i = 0; i < nRows; i++) {
		r->left = 1;
		r->right = fatBitSize;
		nPixels = 0;
		
		for (j = 0; j < nCols; j++) {
			if (nPixels == 0) {
				currentWord = *bitImagePtr;
				bitImagePtr++;
				nPixels = 16;
			}
			
			if (currentWord < 0)
				PaintRect(r);
			else
				EraseRect(r);
				
			currentWord <<= 1;
			nPixels--;
			
			r->left += fatBitSize;
			r->right += fatBitSize;
		}
		
		r->top += fatBitSize;
		r->bottom += fatBitSize;
	}

	HUnlock((Handle)myBTMP);
	HUnlock((Handle)pMyBTMP->hBTMP);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static Boolean GetABit(BTMPPtr theBitMapPtr, short gridH, 
	short gridV)
{
	short					*bitImagePtr;
	short					mask;
	
	bitImagePtr = &theBitMapPtr->bitImage + 
		((theBitMapPtr->rowBytes >> 1) * gridV + (gridH >> 4));
	mask = (1 << (15 - (gridH % 16)));
	return(!!(*bitImagePtr & mask));
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static Boolean FlipABit(BTMPHndl theBitMapHndl, 
	short fatBitSize, short gridH, short gridV)
{
	Rect					theRect;
	Rect					*r;
	short					*bitImagePtr;
	short					mask;
	BTMPPtr				theBitMapPtr;
	
	theBitMapPtr = *theBitMapHndl;
	
	/* flip the bit in the bitMap */
	bitImagePtr = &theBitMapPtr->bitImage + 
		((theBitMapPtr->rowBytes >> 1) * gridV + (gridH >> 4));
	*bitImagePtr ^= (mask = (1 << (15 - (gridH % 16))));
	
	/* invert the corresponding fatBit */
	r = &theRect;
	r->bottom = (r->top = 4 + theBitMapPtr->bounds.bottom - 
		theBitMapPtr->bounds.top + gridV * fatBitSize) + 
		fatBitSize - 1;
	r->right = (r->left = gridH * fatBitSize + 1) + 
		fatBitSize - 1;
	InvertRect(r);

	PlotCenteredBitMap(theBitMapHndl);

	return(!!(*bitImagePtr & mask));
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void DoClick(Point mousePoint, rBTMPHndl myBTMP)
{
	rBTMPRecPtr		pMyBTMP;
	BTMPPtr				theBitMapPtr;
	short					gridH, gridV, h, fatBitSize;
	Boolean				state = 0, firstTime, atLeastOne;

	BubbleUp((Handle)myBTMP);
	HLock((Handle)myBTMP);
	pMyBTMP = *myBTMP;
	
	BubbleUp((Handle)pMyBTMP->hBTMP);
	HLock((Handle)pMyBTMP->hBTMP);
	theBitMapPtr = *pMyBTMP->hBTMP;
	
	atLeastOne = false;
	firstTime = true;
	fatBitSize = pMyBTMP->fatBitSize;
	
	while (StillDown() && (firstTime || atLeastOne)) {
		gridH = (mousePoint.h - 1) / fatBitSize;
		mousePoint.v -= (h = (theBitMapPtr->bounds.bottom - 
			theBitMapPtr->bounds.top) + 4);
		gridV = (mousePoint.v - 1) / fatBitSize;
		
		if ((mousePoint.v >= 0) && (gridV >= 0) && (gridV < h) &&
			(gridH >= 0) && (gridH < theBitMapPtr->bounds.right - 
			theBitMapPtr->bounds.left) && (firstTime || 
			(!firstTime && (GetABit(theBitMapPtr, gridH, gridV) !=
			state)))) {
				state = FlipABit(pMyBTMP->hBTMP, fatBitSize, gridH, 
					gridV);
				atLeastOne = true;
		}
		
		firstTime = false;
		GetMouse(&mousePoint);
	}
	
	if (atLeastOne)
		SetResChanged((Handle)pMyBTMP->hBTMP);

	HUnlock((Handle)myBTMP);
	HUnlock((Handle)pMyBTMP->hBTMP);	
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void SetRadioButton(DialogPtr theDialog, 
	short minItemNum, short maxItemNum, short currentItemNum)
/* unsets a range of radio buttons except for item number 
 * currentItemNum which it sets (if it is within the range 
 * minItemNum to maxItemNum). */
{
	short					i, itemType;
	Handle				item;
	Rect					box;
	
	for (i = minItemNum; i <= maxItemNum; i++) {
		GetDItem(theDialog, i, &itemType, &item, &box);
		SetCtlValue((ControlHandle)item, i == currentItemNum);
	}
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void UpdateWindow(rBTMPHndl myBTMP)
{
	rBTMPRecPtr		pMyBTMP;
	short			windowWidth, windowHeight;
	WindowPtr		w;
	
	pMyBTMP = *myBTMP;
	CalcWindowSize(pMyBTMP->hBTMP, pMyBTMP->fatBitSize, 
		&windowWidth, &windowHeight);
	w = pMyBTMP->windPtr;
	SizeWindow(w, windowWidth, windowHeight, false);
	InvalRect(&w->portRect);
	EraseRect(&w->portRect);
	DrawBTMPWindow(myBTMP);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

#define	OK_BUTTON						1
#define	LOWEST_ITEM_NO			3
#define	HIGHEST_ITEM_NO			8
#define	FIRST_PT_SIZE				4
#define	DIFF_BASE_TO_LOWEST	(FIRST_PT_SIZE - LOWEST_ITEM_NO)

static void ResizeFatBits(rBTMPHndl myBTMP)
{
	short					saveRefNum, fatBitSize, oldFatBitSize, 
								itemHit;
	DialogPtr			theDialog;
	
	saveRefNum = CurrentRes();
	UseResFile(ResEditRes());
	
	theDialog = GetNewDialog(FAT_BIT_SIZE_DLOG, NULL, 
		(WindowPtr)-1);

	oldFatBitSize = fatBitSize = (*myBTMP)->fatBitSize;

	SetRadioButton(theDialog, LOWEST_ITEM_NO, HIGHEST_ITEM_NO,
		fatBitSize - DIFF_BASE_TO_LOWEST);

	do {
		ModalDialog(NULL, &itemHit);
		if ((itemHit >= LOWEST_ITEM_NO) &&
			(itemHit <= HIGHEST_ITEM_NO) && 
		  (itemHit != (fatBitSize - DIFF_BASE_TO_LOWEST))) {
				fatBitSize = itemHit + DIFF_BASE_TO_LOWEST;
				SetRadioButton(theDialog, LOWEST_ITEM_NO, 
					HIGHEST_ITEM_NO, fatBitSize - 
					DIFF_BASE_TO_LOWEST);
		}
	} while (itemHit != OK_BUTTON);

	DisposDialog(theDialog);

	UseResFile(saveRefNum);

	if (fatBitSize != oldFatBitSize) {
		(*myBTMP)->fatBitSize = fatBitSize;
		UpdateWindow(myBTMP);
	}
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void SetDItemText(DialogPtr theDialog, short itemNo, 
	short value)
{
	Str255				theString;
	short					itemType;
	Handle				item;
	Rect					box;

	GetDItem(theDialog, itemNo, &itemType, &item, &box);
	NumToString(value, &theString);
	SetIText(item, &theString);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static short GetDItemText(DialogPtr theDialog, 
	short itemNo)
{
	Str255			theString;
	short			itemType;
	long			value;
	Handle			item;
	Rect			box;
	
	GetDItem(theDialog, itemNo, &itemType, &item, &box);
	GetIText(item, &theString);
	StringToNum(&theString, &value);
	return(value);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void ResizeBTMP(BTMPHndl theBitMapHndl, 
	Rect *newBounds)
{
	BTMPPtr			theBitMapPtr, pOldtheBitMap;
	long			sizeOfBitMap;
	short			*bitImagePtr;
	short			i, j;
	Rect			intersectRect, copyOftheBitMapBounds;
	
	/* make a copy of the old bitMap */
	pOldtheBitMap = (BTMPPtr)NewPtr(sizeOfBitMap = 
		GetHandleSize((Handle)theBitMapHndl));
	theBitMapPtr = *theBitMapHndl;
	BlockMove((Ptr)theBitMapPtr, (Ptr)pOldtheBitMap, 
		sizeOfBitMap);
	
	/* set up the new bitMap */
	theBitMapPtr->bounds = *newBounds;
	theBitMapPtr->rowBytes = (((newBounds->right - 
		newBounds->left - 1) >> 4) + 1) << 1;
	HUnlock((Handle)theBitMapHndl);
	sizeOfBitMap = theBitMapPtr->rowBytes * 
		(newBounds->bottom - newBounds->top);
	FixHand(sizeof(BTMP) + sizeOfBitMap, 	
		(Handle)theBitMapHndl);
	BubbleUp((Handle)theBitMapHndl);
	HLock((Handle)theBitMapHndl);
	theBitMapPtr = *theBitMapHndl;
	
	/* clear the bitImage of the new bitMap */
	sizeOfBitMap >>= 1;
	bitImagePtr = &theBitMapPtr->bitImage;
	for (i = 0; i < sizeOfBitMap; i++)
		*bitImagePtr++ = 0;
	
	/* copy this old bits over to the new bitMap */
	copyOftheBitMapBounds = theBitMapPtr->bounds;
	i = pOldtheBitMap->bounds.left - 
		theBitMapPtr->bounds.left;
	j = pOldtheBitMap->bounds.top - theBitMapPtr->bounds.top;
	OffsetRect(&copyOftheBitMapBounds, i, j);
	SectRect(&copyOftheBitMapBounds, &pOldtheBitMap->bounds, 
		&intersectRect);
	theBitMapPtr->baseAddr = (Ptr)&theBitMapPtr->bitImage;
	pOldtheBitMap->baseAddr = (Ptr)&pOldtheBitMap->bitImage;
	copyOftheBitMapBounds = intersectRect;
	OffsetRect(&copyOftheBitMapBounds, -i, -j);
	CopyBits((BitMap *)pOldtheBitMap, (BitMap *)theBitMapPtr, 
		&intersectRect, &copyOftheBitMapBounds, srcCopy, NULL);	
	theBitMapPtr->baseAddr = NULL;

	DisposPtr((Ptr)pOldtheBitMap);
	
	SetResChanged((Handle)theBitMapHndl);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

#define	BOUNDS_TOP_ITEM			2
#define	BOUNDS_LEFT_ITEM		3
#define	BOUNDS_BOTTOM_ITEM	4
#define	BOUNDS_RIGHT_ITEM		5

static void ChangeBounds(rBTMPHndl myBTMP)
{
	BTMPHndl			theBitMapHndl;
	BTMPPtr				theBitMapPtr;
	short					saveRefNum, itemHit, i, *iPtr;
	DialogPtr			theDialog;
	Rect					newBounds;

	BubbleUp((Handle)myBTMP);
	HLock((Handle)myBTMP);
	theBitMapHndl = (*myBTMP)->hBTMP;
	
	BubbleUp((Handle)theBitMapHndl);
	HLock((Handle)theBitMapHndl);
	theBitMapPtr = *theBitMapHndl;

	saveRefNum = CurrentRes();
	UseResFile(ResEditRes());

	theDialog = GetNewDialog(CHANGE_BOUNDS_DLOG, NULL, 
		(WindowPtr)-1);

	iPtr = &theBitMapPtr->bounds.top;
	for (i = BOUNDS_TOP_ITEM; i <= BOUNDS_RIGHT_ITEM; i++)
		SetDItemText(theDialog, i, *iPtr++);

	do {
		ModalDialog(NULL, &itemHit);
	} while (itemHit != OK_BUTTON);
	
	iPtr = &newBounds.top;
	for (i = BOUNDS_TOP_ITEM; i <= BOUNDS_RIGHT_ITEM; i++)
		*iPtr++ = GetDItemText(theDialog, i);
		
	DisposDialog(theDialog);
	UseResFile(saveRefNum);
	
	if (theBitMapPtr->bounds != newBounds) {
		ResizeBTMP(theBitMapHndl, &newBounds);
		UpdateWindow(myBTMP);
	}
	
	HUnlock((Handle)myBTMP);
	HUnlock((Handle)theBitMapHndl);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void FillBitMap(short state, rBTMPHndl myBTMP)
{
	BTMPPtr 	theBitMapPtr;
	short			i, sizeOfBitMap;
	short			*bitImagePtr;
	
	theBitMapPtr = *(*myBTMP)->hBTMP;
	bitImagePtr = &theBitMapPtr->bitImage;
	sizeOfBitMap = ((theBitMapPtr->bounds.bottom - 
		theBitMapPtr->bounds.top)*theBitMapPtr->rowBytes) >> 1;
	
	for (i = 0; i < sizeOfBitMap; i++) {
		switch (state) {
		case FILL_WHITE_ITEM:
			*bitImagePtr++ = 0;
			break;
		case FILL_BLACK_ITEM:
			*bitImagePtr++ = -1;
			break;
		case INVERT_ITEM:
			*bitImagePtr++ ^= 0xFFFF;
			break;
		}
	}
	
	InvalRect(&(*myBTMP)->windPtr->portRect);
	SetResChanged((Handle)(*myBTMP)->hBTMP);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

#define	UP		1
#define	DOWN	2
#define	LEFT	3
#define	RIGHT	4

static void Nudge(short nudgeDir, short dh, short dv, 
	Rect *sRect, rBTMPHndl myBTMP)
{
	rBTMPRecPtr		pMyBTMP;
	BTMPPtr				theBitMapPtr;
	Rect					destRect;
	RgnHandle			trashRgn;
	short					i, fatBitSize, rowWords, mask;
	short					*bitImagePtr;
	
	BubbleUp((Handle)myBTMP);
	HLock((Handle)myBTMP);
	pMyBTMP = *myBTMP;
	
	BubbleUp((Handle)pMyBTMP->hBTMP);
	HLock((Handle)pMyBTMP->hBTMP);
	theBitMapPtr = *pMyBTMP->hBTMP;
	
	/* copy the pixel map onto itself (offset by 1 pixel 
	 * in 1 direction) */
	destRect = theBitMapPtr->bounds;
	OffsetRect(&destRect, dh, dv);
	theBitMapPtr->baseAddr = (Ptr)&theBitMapPtr->bitImage;
	CopyBits((BitMap *)theBitMapPtr, (BitMap *)theBitMapPtr,
		&theBitMapPtr->bounds, &destRect, srcCopy, NULL);
	theBitMapPtr->baseAddr = NULL;

	/* scroll the screen to agree with the new bitMap */
	sRect->top += theBitMapPtr->bounds.bottom - 
		theBitMapPtr->bounds.top + 3;
	ScrollRect(sRect, dh * (fatBitSize = pMyBTMP->fatBitSize),
		dv * fatBitSize, trashRgn = NewRgn());
	DisposeRgn(trashRgn);

	/* mask off one row or column of bits */
	bitImagePtr = &theBitMapPtr->bitImage;
	mask = 0x7FFF;
	rowWords = theBitMapPtr->rowBytes >> 1;

	switch (nudgeDir) {
	case UP:
		bitImagePtr += ((theBitMapPtr->bounds.bottom - 
			theBitMapPtr->bounds.top - 1) *
			theBitMapPtr->rowBytes) >> 1;
	case DOWN:
		for (i = 0; i < rowWords; i++)
			*bitImagePtr++ = 0;
		break;
	case LEFT:
		bitImagePtr += rowWords - 1;
		mask = 0xFFFF ^ (1 << (15 - 
			((theBitMapPtr->bounds.right -
			theBitMapPtr->bounds.left - 1) % 16)));
	case RIGHT:
		for (i = 0; i < (theBitMapPtr->bounds.bottom -
			theBitMapPtr->bounds.top); i++) {
				*bitImagePtr &= mask;
				bitImagePtr += rowWords;
		}
		break;
	}
	
	PlotCenteredBitMap(pMyBTMP->hBTMP);
	
	SetResChanged((Handle)pMyBTMP->hBTMP);

	HUnlock((Handle)pMyBTMP->hBTMP);
	HUnlock((Handle)myBTMP);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

static void DoClose(rBTMPHndl myBTMP)
{
	CloseWindow((*myBTMP)->windPtr);
	WindReturn((*myBTMP)->windPtr);
	SetTheCursor(arrowCursor);
	
	DeleteMenu(BTMP_MENU_ID);
	DrawMenuBar();
	
	/* Release the resource if we were launched from 
	 * a picker. */
	if ((*(*myBTMP)->father)->name[1] != editorNameChr)
		ReleaseResource((Handle)(*myBTMP)->hBTMP);

	DisposHandle((Handle)myBTMP);
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

pascal void DoMenu(short menu, short item, 
	rBTMPHndl myBTMP)
{
	short					saveRefNum;
	Handle				hTemp;
	Rect					sRect;

	BubbleUp((Handle)myBTMP);
	HLock((Handle)myBTMP);
	
	SetPort((*myBTMP)->windPtr);
	
	switch (menu) {
	case fileMenu:
		switch (item) {
		case closeItem:
			DoClose(myBTMP);
			return;	
			
		case saveItem:
			PassMenu(fileMenu, saveItem, (ParentHandle)myBTMP);
			break;
			
		case revertItem:
			if (NeedToRevert((*myBTMP)->windPtr, 
				(Handle)(*myBTMP)->hBTMP)) {
					InvalRect(&(*myBTMP)->windPtr->portRect);
					
					saveRefNum = CurrentRes();
					UseResFile(HomeResFile((Handle)(*myBTMP)->hBTMP));
					
					if (!RevertResource((Handle)(*myBTMP)->hBTMP)) {
						hTemp = (Handle)(*myBTMP)->hBTMP;
	
						/* Make sure that the picker list is rebuilt 
						 * to remove this item. */
						(*(*myBTMP)->father)->rebuild = true;
						
						DoClose(myBTMP);
						RemoveResource(hTemp);
						return;
					}
	
					UseResFile(saveRefNum);
			}
			break;
		
		case getInfoItem:
			ShowInfo((Handle)(*myBTMP)->hBTMP,
				(ParentHandle)myBTMP);
			break;
			
		default:
			break;
		}
		
		break;
	
	case BTMP_MENU_ID:
		sRect = (*myBTMP)->windPtr->portRect;	
		
		switch (item) {
		case CHANGE_BOUNDS_ITEM:
			ChangeBounds(myBTMP);
			break;
			
		case FATBIT_SIZE_ITEM:
			ResizeFatBits(myBTMP);
			break;
			
		case FILL_WHITE_ITEM:
		case FILL_BLACK_ITEM:
		case INVERT_ITEM:
			FillBitMap(item, myBTMP);
			break;
			
		case NUDGE_UP_ITEM:		
			Nudge(UP, 0, -1, &sRect, myBTMP);
			break;
			
		case NUDGE_DOWN_ITEM:
			Nudge(DOWN, 0, 1, &sRect, myBTMP);
			break;
			
		case NUDGE_LEFT_ITEM:
			Nudge(LEFT, -1, 0, &sRect, myBTMP);
			break;
			
		case NUDGE_RIGHT_ITEM:
			Nudge(RIGHT, 1, 0, &sRect, myBTMP);
			break;
			
		default:
			break;
		}
		
		break;
		
	default:
		break;
	}
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

pascal void DoEvent(EventRecord *evt, rBTMPHndl myBTMP)
{
	Point		mousePoint;
	short		saveRefNum;
	
	SetPort((*myBTMP)->windPtr);
	
	switch (evt->what) {
	case mouseDown:
		mousePoint = evt->where;
		GlobalToLocal(&mousePoint);
		DoClick(mousePoint, myBTMP);
		break;
		
	case activateEvt: 
		AbleMenu(fileMenu, fileTop);
		if (evt->modifiers & activeFlag) {
			saveRefNum = CurrentRes();
			UseResFile(HomeResFile((Handle)(*myBTMP)->hBTMP));
			InsertMenu(GetMenu(BTMP_MENU_ID), 0);
			UseResFile(saveRefNum);
		}
		else
			DeleteMenu(BTMP_MENU_ID);
		DrawMenuBar();
		break;

	case updateEvt:
		DrawBTMPWindow(myBTMP);
		break;

	default:
		break;
	}
}

/* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

pascal void DoInfoUpdate(short oldID, short newID, 
	rBTMPHndl myBTMP)
{
	Str255	windowTitle, windowName;

	/* Since our ID has changed, we need to change our 
	 * window title */
	GetNameAndTitle (&windowTitle, &windowName, 
		(Handle)(*myBTMP)->hBTMP);
	GetWindowTitle (&windowTitle, &windowName, true, 
		(*myBTMP)->father);
	BlockMove(&windowName,&(*myBTMP)->name,windowName[0] + 1);
	SetWTitle((*myBTMP)->windPtr, &windowTitle);

	/* Now, let our father object know that our ID has been 
	 * changed (set flag to rebuild the picker list) */
	(*(*myBTMP)->father)->rebuild = true;
	CallInfoUpdate(oldID, newID, 
		(*(*myBTMP)->father)->wind->refCon,
		(*(*myBTMP)->father)->wind->windowKind);
}
