/*------------------------------------------------------------------------------
#	MacTutorApp
#
#	A rudimentary application skeleton
#	J. Langowski / MacTutor 1989
#------------------------------------------------------------------------------*/
#include <Types.h>
#include <QuickDraw.h>
#include <Fonts.h>
#include <Events.h>
#include <OSEvents.h>
#include <Controls.h>
#include <Windows.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <Desk.h>
#include <Scrap.h>
#include <ToolUtils.h>
#include <Memory.h>
#include <SegLoad.h>
#include <Files.h>
#include <OSUtils.h>
#include <Traps.h>
#include <StdLib.h>

#include "TDocument.h"
#include "TApplication.h"
#include "DocListApp.h"
#include "DisplList.h"
#include "ListDoc.h"

// Methods for our application class
TDocListApp::TDocListApp(void)
{
	Handle	menuBar;

	// read menus into menu bar
	menuBar = GetNewMBar(rMenuBar);
	// install menus
	SetMenuBar(menuBar);
	DisposHandle(menuBar);
	// add DA names to Apple menu
	AddResMenu(GetMHandle(mApple), 'DRVR');
	DrawMenuBar();

	// create empty mouse region
	fMouseRgn = NewRgn();
	// create a single empty document
	DoNew();
}

// Tell TApplication class how much heap we need
long TDocListApp::HeapNeeded(void)
{
	return (kMinSize * 1024);
}

// Calculate a sleep value for WaitNextEvent. This takes into account the things
// that DoIdle does with idle time.

unsigned long TDocListApp::SleepVal(void)
{
	unsigned long sleep;
	const long kSleepTime = 0x7fffffff;	// a very large positive number

	sleep = kSleepTime;				// default value for sleep
	if ((!fInBackground))
	{
		  sleep = GetCaretTime();	// A reasonable time interval for MenuClocks, etc.
	}
	return sleep;
}

void TDocListApp::AdjustMenus(void)
{
	WindowPtr	frontmost;
	MenuHandle	menu;
	Boolean undo,cutCopyClear,paste;

	TListDoc* fCurListDoc = (TListDoc*) fCurDoc;

	frontmost = FrontWindow();

	menu = GetMHandle(mFile);
	if ( fDocList->NumDocs() < kMaxOpenDocuments )
	  EnableItem(menu, iNew);			// New is enabled when we can open more documents 
	else DisableItem(menu, iNew);
	if ( frontmost != (WindowPtr) nil )	// Close is enabled when there is a window to close 
	  EnableItem(menu, iClose);
	else DisableItem(menu, iClose);

	undo = false;
	cutCopyClear = false;
	paste = false;
	
	if ( fCurListDoc == nil )
	  {
		undo = true;				// all editing is enabled for DA windows 
		cutCopyClear = true;
		paste = true;
	  }
	  
	menu = GetMHandle(mEdit);
	if ( undo )
		EnableItem(menu, iUndo);
	else
		DisableItem(menu, iUndo);
	
	if ( cutCopyClear )
	  {
		EnableItem(menu, iCut);
		EnableItem(menu, iCopy);
		EnableItem(menu, iClear);
	  } 
	else
	  {
		DisableItem(menu, iCut);
		DisableItem(menu, iCopy);
		DisableItem(menu, iClear);
	  }
	  
	if ( paste )
		EnableItem(menu, iPaste);
	else
		DisableItem(menu, iPaste);
		
	menu = GetMHandle(myMenu);
	EnableItem(menu, item1);
	EnableItem(menu, item2);
	EnableItem(menu, item3);
	if (fCurListDoc->fItem1Set)
		SetItem(GetMHandle(myMenu),item1,"\pRoundRect-");
	else
		SetItem(GetMHandle(myMenu),item1,"\pRoundRect+");
	if (fCurListDoc->fItem2Set)
		SetItem(GetMHandle(myMenu),item2,"\pOval-");
	else
		SetItem(GetMHandle(myMenu),item2,"\pOval+");
	if (fCurListDoc->fItem3Set)
		SetItem(GetMHandle(myMenu),item3,"\pRect-");
	else
		SetItem(GetMHandle(myMenu),item3,"\pRect+");
} // AdjustMenus


void TDocListApp::DoMenuCommand(short menuID, short menuItem)
{
	short		itemHit;
	Str255		daName;
	short		daRefNum;
	WindowPtr	window;
	TListDoc* fCurListDoc = (TListDoc*) fCurDoc;

	window = FrontWindow();
	switch ( menuID )
	  {
		case mApple:
			switch ( menuItem )
			  {
				case iAbout:		// About box
					itemHit = Alert(rAboutAlert, nil);
					break;
				default:			// DAs etc.
					GetItem(GetMHandle(mApple), menuItem, daName);
					daRefNum = OpenDeskAcc(daName);
					break;
			  }
			break;
		case mFile:
			switch ( menuItem )
			  {
				case iNew:
					DoNew();
					break;
				case iClose:
					if (fCurListDoc != nil)
					  {
						fDocList->RemoveDoc(fCurListDoc);
						delete fCurListDoc;
					  }
					else CloseDeskAcc(((WindowPeek) fWhichWindow)->windowKind);
					break;
				case iQuit:
					Terminate();
					break;
			  }
			break;
		case mEdit:					// call SystemEdit for DA editing & MultiFinder 
			if ( !SystemEdit(menuItem-1) )
			  {
				switch ( menuItem )
				  {
					case iCut: break;
					case iCopy: break;
					case iPaste: break;
					case iClear: break;
				   }
			  }
			break;
		case myMenu:
			if (fCurListDoc != nil) 
			{
				switch ( menuItem )
				  {
					case item1:
						if (fCurListDoc->fItem1Set)
							fCurListDoc->fObjList->RemoveObj(fCurListDoc->obj1);
						else
							fCurListDoc->fObjList->AddObj(fCurListDoc->obj1);
						fCurListDoc->fItem1Set = !fCurListDoc->fItem1Set;
						break;
					case item2:
						if (fCurListDoc->fItem2Set)
							fCurListDoc->fObjList->RemoveObj(fCurListDoc->obj2);
						else
							fCurListDoc->fObjList->AddObj(fCurListDoc->obj2);
						fCurListDoc->fItem2Set = !fCurListDoc->fItem2Set;
						break;
					case item3: 
						if (fCurListDoc->fItem3Set)
							fCurListDoc->fObjList->RemoveObj(fCurListDoc->obj3);
						else
							fCurListDoc->fObjList->AddObj(fCurListDoc->obj3);
						fCurListDoc->fItem3Set = !fCurListDoc->fItem3Set;
						break;
					}

			InvalRect(&(window->portRect));
			fCurListDoc->DoUpdate();
			}
			break;
	  }
	HiliteMenu(0);
} // DoMenuCommand

void TDocListApp::DoNew(void)
{
	TListDoc* tMyDoc;
	
	tMyDoc = new TListDoc(rDocWindow);
	if (tMyDoc != nil) fDocList->AddDoc(tMyDoc);
} // DoNew

void TDocListApp::Terminate(void)
{
	ExitLoop();
} // Terminate

TDocListApp *gTheApplication;

int main(void)
{
	gTheApplication = new TDocListApp;
	if (gTheApplication == nil)	 return 0;
	gTheApplication->EventLoop();
	return 0;
}

