/************************************/
/* File: EventPatch.c				*/
/* 									*/
/* Patch to get next event			*/
/* to allow us to intercept events	*/
/* from Hypercard.					*/
/* 									*/
/* check to see if we get the event	*/
/* if not, pass it back on,			*/
/* otherwise, handle the event and 	*/
/* return a null event to Hypercard	*/
/*									*/
/* The real getnextevent will unbias*/
/* the stack parameters so we need 	*/
/* to save them off before we call	*/
/* the real trap					*/
/*									*/
/* The Boolean result of GetNextEvt	*/
/* will be at 0(A7) immediately 	*/
/* after calling the real trap		*/
/* 									*/
/* If we took the event, then all we*/
/* need do is set the result of 	*/
/* GetNextEvent to false.			*/
/* --------------------------------	*/
/* 1989 Donald Koscheka			*/
/* All Rights Reserved				*/
/************************************/

#include	<SetUpA4.h>
#include	<MacTypes.h>
#include	<OSUtil.h>
#include	<ControlMgr.h>
#include 	<HyperXCmd.h>
#include	<HyperUtils.h>
#include	"Control XFCN.h"

/*** NOTE: you can have no stack frames in the 		***/
/*** intercept since this is a stack based trap		***/
/*** That is to say, the stack is already biased	***/
/*** correctly for the trap -- any local variables 	***/
/*** must be declared as statics below, no locals	***/
/*** are allowed in main.							***/



static	void			*returnAddress;
static	EventRecord		*theEvent;
static	short			evtMask;
static	short			haveEvt;
static	WindowPtr		whichWindow;
static	short			windoPart;
static	void			*oldA4;
static	ProcPtr			oldTrap;
static	Point			thePoint;
static	short			controlPart;
static	ControlHandle	whichControl;
static	wdGlobalsHand	windH;

void main(){
	asm{
		Move.l	A4, D0						; we need to save the old A4
		Move.l	A0, A4						; pointer to our globals.
		Move.l	D0, oldA4					; note that oldA4 is referenced off A4
		Move.l	(A7)+, returnAddress		; save the real return address...
		Move.l	(A7), theEvent				; save the event record pointer
		Move.w	4(A7), evtMask				; and the event mask
	}

	windH = (wdGlobalsHand)GetSystemResource( CONTROL_INFO, CONTROL_PATCH );
	oldTrap = (*windH)->oldTrap;
	
	asm{									; left the stack intact for the call
		Move.l	oldTrap, A0
		Jsr		(A0)						
		Move.b	(A7), haveEvt				; did we get an event?
	}
	
	whichWindow = FrontWindow();
	
	if( windH && haveEvt && (((WindowPeek)whichWindow)->windowKind != dialogKind)  ){
		switch( theEvent->what ){
			case mouseDown:
					
				windoPart = FindWindow( theEvent->where, &whichWindow );
				
				if( windoPart == inContent ){
					thePoint = theEvent->where;
					GlobalToLocal( &thePoint );
					
					if( controlPart = FindControl( thePoint, whichWindow, &whichControl )){
					
						/*** need to temporarily unpatch GetNextEvent 	***/
						/*** for the mouse tracking stuff because some	***/
						/*** tracking calls also call getNextEvent!!!	***/
						
						NSetTrapAddress( (*windH)->oldTrap, EVENT_TRAP, ToolTrap);
	
						controlPart = TrackControl( whichControl, thePoint, NIL );
						
						/*** now re-install the patch	***/
						NSetTrapAddress( *((*windH)->evtProc), EVENT_TRAP, ToolTrap);
					}

				}	/*** if mouse down in our window ***/
				break;
			
			case updateEvt: 
				whichWindow = (WindowPtr)theEvent->message;
				DrawControls( whichWindow );
				break;
				
			default: 
				break;
		} /* switch theEvent->what */
	} /* if haveEvt */
	
	asm{	
		Move.b	haveEvt, (A7)
		Move.l	returnAddress, A0
		Move.l	oldA4, A4
		Jmp		(A0)
	}
}


