/*.h2 Aesthete */
/*.fo 1988 David Dunham */
/*
Test the DetachResource patch
*/

#define global
#include "aesthete.h"

#define	IAZGlobal	((long *)0x33c)

typedef struct RGBColor {
	short	red;	/* magnitude of red component */
	short	green;	/* magnitude of green component */
	short	blue;	/* magnitude of blue component */
} RGBColor;

#define appleMenu	1
#define fileMenu	2
#define editMenu	3
#define LAST_MENU	3

CursHandle		watch;
MenuHandle		menus[LAST_MENU];
word			myRef;						/* refNum of my resource file */
long			oldTrap;
long			oldIAZ;						/* Save old IAZNotify */

/* Prototypes not in aesthete.h */
void			about(void);
void			close_all(void);
void			do_command(unsigned long result);
void			initialize(void);
void			install_patch(void);
void			kill_patch(void);
void			notify(void);
void			setup_menus(void);
void			show_string(word h, word v, word rsrc);

/*
MAIN
*/
void main() {
	WindowPtr	whichWindow;
	word		code;
	char		c;
	Point		pt;
	EventRecord	event;

	oldIAZ = *IAZGlobal;
	*IAZGlobal = (long)notify;				/* Install our routine */
	initialize();
	InitCursor();							/* Back to arrow */	
	while (TRUE) {							/* Main event loop	*/
		SystemTask();
		if (GetNextEvent(everyEvent,&event))
		switch(event.what) {
		  case mouseDown:
			code = FindWindow(event.where,&whichWindow);
			switch (code) {
			  case inMenuBar:
				do_command(MenuSelect(event.where));
				break;
			  case inSysWindow:				/* Cursor in system window */
				SystemClick(&event,whichWindow);
				break;
			} /* end switch code */
			break;
		  case keyDown:
		  case autoKey:
			c = event.message & 255;
			if (event.modifiers & 256)
				do_command(MenuKey(c));
		  	break;
		} /* end switch */
	} /* end while */
}

/*
ABOUT - Show info about this program
*/
void about() {
	register WindowPtr	window;
	WindowRecord		wbuf;
	char				bufr[80];
	EventRecord			event;
	Rect				bounds;
	RGBColor			colour;
	Handle				icon;
	
	window = GetNewWindow(128,&wbuf,-1L);
	SetPort(window);
	colour.red = 39321;
	colour.green = 26214;
	colour.blue = 0;
	RGBForeColor(&colour);
	TextFace(0);
	TextFont(newYork);
	TextSize(12);
	show_string(140,85,3);
	icon = GetResource('ICN#',128);
	SetRect(&bounds,10,10,42,42);
	PlotIcon(&bounds,icon);
	DetachResource(icon);
	SetRect(&bounds,110,10,142,42);
	PlotIcon(&bounds,icon);
	SetRect(&bounds,210,10,226,26);
	PlotIcon(&bounds,icon);
	while (TRUE) {
		SystemTask();					/* Keep clock ticking */
		if (GetNextEvent(mDownMask | keyDownMask,&event)) break;
	}
	CloseWindow(window);
}

/*
CLOSE_ALL - Close all desk accessories
*/
void close_all() {
	register WindowPeek	w, v;

	w = (WindowPeek)FrontWindow();
	while (w != NIL) {					/* Look at each window */
		v = w->nextWindow;				/* Remember next */
		InitCursor();					/* Normal cursor for MockWrite */
		CloseDeskAcc(w->windowKind);
		SetCursor(*watch);
		w = v;							/* Look at next one */
	}
}

/*
DO_COMMAND - Do something from the menu bar
*/
void do_command(result) unsigned long result; {
	register word		menu, item, i;
	char				name[64];

	menu = result >> 16;
	item = result;
	switch (menu) {
	  case appleMenu:
		if (item == 1) {				/* About */
			about();
			break;
		}
		GetItem(menus[appleMenu - 1],item,name);
		OpenDeskAcc(name);
		break;
	  case fileMenu:
		switch (item) {
		  case 4:						/* Quit */
			SetCursor(*watch);
			close_all();
			ExitToShell();				/* IAZNotify restores original _DetachResource */
		}
		break;
	  case editMenu:
		SystemEdit(item - 1);
		break;
	}
	HiliteMenu(0);						/* Make sure menu bar isnt highlighted	*/
}

/*
INITIALIZE
*/
void initialize() {
	InitGraf(&thePort);
	watch = GetCursor(4);				/* Get watch from resource */
	SetCursor(*watch);
	FlushEvents(everyEvent,0);
	InitFonts();
	InitWindows();
	InitDialogs(0L);					/* No disaster function */
	TEInit();
	MaxApplZone();						/* Pre-grow */
	setup_menus();
	install_patch();
}

/*
INSTALL_PATCH  Install patch (in Application Heap for now)
*/
void install_patch() {
register word		saveRef, first, numTypes;
Handle				sicn, creators;
OSErr				error;

asm {
	bra		@around
	dc.l	0						; for oldTrap
	dc.l	0						; for creators handle
	dc.l	0						; for sicn handle
#define oCreators	-8
#define oSicn		-4
myPatch:
	lea		@myPatch,A0
	move.l	oCreators(A0),-(SP)
	move.l	oSicn(A0),-(SP)
	move.l	12(SP),A0				; original parameter to _DetachResource
	move.l	A0,-(SP)
	jsr		aesthetize				; aesthetize(rsrc,sicn,creators)
	add.l	#12,SP
	move.l	oldTrap,A0
	jmp		(A0)					; pass the call to original _DetachResource
	_Debugger						; should never get here!
around:
	move.w	#0xA992,D0
	_GetTrapAddress
	move.l	A0,oldTrap				; save original trap address
	lea		@myPatch,A0
	move.w	#0xA992,D0
	_SetTrapAddress					; install our patch
}
	sicn = double_SICN();
	creators = GetResource('SIG#',0);
	DetachResource(creators);
asm {
	lea		@myPatch,A0
	move.l	sicn,oSicn(A0)			; stash it
	move.l	creators,oCreators(A0)	; stash it
}
}

/*
KILL_PATCH  Unpatch our mods to _DetachResource
*/
void kill_patch () {
asm {
	move.l	oldTrap,A0				; global holds old address
	move.w	#0xA992,D0
	_SetTrapAddress					; everythings back to original state
}
}

/*
NOTIFY - Clean things up if we _ExitToShell from debugger
		This routine gets called from our own _ExitToShell too
*/
void notify() {
	long	trap;
	Handle	h;
	
	*IAZGlobal = oldIAZ;
	kill_patch();					/* Restore original _DetachResource */
}

/*
SETUP_MENUS - Set up menus
*/
void setup_menus() {
	register word	i;
	register Handle	handle;
	long			type;
	word			code;
	char			string[256];

	InitMenus();
	for (i = 0; i < LAST_MENU; ) menus[i] = GetMenu(++i);
	AddResMenu(menus[appleMenu - 1],'DRVR');
	for (i = 0; i < LAST_MENU; i++) {
		InsertMenu(menus[i],0);
	}
	DrawMenuBar();
}

/*
SHOW_STRING
*/
void show_string(h,v,rsrc) word h, v, rsrc; {
	register StringHandle	s;

	MoveTo(h,v);
	s = GetString(rsrc);
	HLock(s);
	DrawString(*s);
	HUnlock(s);
}
