/*****************************************/
/* File HandleDialog.c */
/*****************************************/

/* Prototypes */
	
int HandleDialog(short Dialog_ID);
		
/* Called whenever you want to put up a modal dialog */

pascal Boolean FProc(DialogPtr theDialog, 
			EventRecord *theEvent, 
			int *itemHit);

/* Filter procedure called by ModalDialog to screen dialog events */
	
Point CenterDialogItem(DialogPtr theDialog, int			item_number);

/* Returns the center of the rect of a dialog item, to be called from a filter proc while a modal dialog is active. */

void FlashDialogItem(DialogPtr theDialog, int 			item_number);
		
/* Flashes an item of a dialog, to be called from a filter proc while a modal dialog is active. */

void HandleUpdate(DialogPtr theDialog);

/* Used to handle update events while a modal 
dialog is active, called from a filter proc.*/

/************************************************/
/* This procedure returns the center of a dialog item as a point.  It is designed to be used with buttons, but will work with any type of dialog item.  */

/* Input:  theDialog (a DialogPtr), item number (an integer) */
/* Output: a point */

Point CenterDialogItem(theDialog, item_number)
						DialogPtr 	theDialog;
	int		item_number;

{
	Point 	the_center;			
/* Center of item, to return */
	int		itemType;
/* Returned by GetDItem but not used */
	Handle	theItem;			
/* Returned by GetDItem but not used */
	Rect	the_Rect;			
/* Returned by GetDItem */

GetDItem(theDialog, item_number, &itemType, 		&theItem, &the_Rect);
the_center.h = the_Rect.left + ((the_Rect.right - 	the_Rect.left)/ 2);
the_center.v = the_Rect.top + ((the_Rect.bottom - 	the_Rect.top)/ 2);
	return the_center;
}

/************************************************/
/* This procedure returns the center of the rectangle of the default dialog item as a single point.  */

/* Input:  theDialog (a DialogPtr), item_number (an integer) */
/* Output:  none */

void FlashDialogItem(theDialog, item_number)
						DialogPtr 	theDialog;
	int		item_number;

{
	long	tickScratch;		
/* returned by Delay, unused */
	int	itemType;		
/* Returned by GetDItem but not used */
	Handle	the_item;			
/* Handle to default button */
	Rect	controlRect;		
/* Returned by GetDItem but not used */

GetDItem(theDialog, item_number, &itemType, 	&the_item, &controlRect);
HiliteControl(the_item, 1); 
Delay(6, &tickScratch);
}

/************************************************/
/* The following function handles update events for your dialog box.  It is called whenever the filterProc receives an update event.  You can put whatever you want to be drawn in the dialog in it.  Right now it redraws the rounded rect around the default button (number one). */

/* Constants for drawing roundRect */
#define edgeCurve		16	
#define gapBetween		-4
#define lineSize		3

void HandleUpdate(theDialog)
	DialogPtr theDialog;

{
	Rect	controlRect;	
/* rectangle of the default control.  We
need this to draw the round rect. */
	int		itemType;		
/* Returned by GetDItem but not used */
	Handle	the_item;		
/* ditto */
	Rect	border;			
/* Rect of the thick border */
	
PenSize(lineSize, lineSize);
/* Get control #1's rectangle and grow it a little bit, then call FrameRoundRect to draw it. */
GetDItem(theDialog, (int)1, &itemType, &the_item, 	&controlRect);
border = controlRect;	
InsetRect(&border, gapBetween, gapBetween);	
FrameRoundRect(&border, edgeCurve, edgeCurve);	
}
	
/************************************************/

/* The following filterProc is used to handle extra events during the modal dialog loop, such as update events.  Its original purpose was to keep the default round-rect drawn around the control even after a screen saver has redrawn the screen. It also handles insertion of blank disks and button-keyboard equivalents. 
	
NOTE:  If you have one or more editable text fields within your modal dialog, your key equivalents should use the command key.  If you don't have any editable text fields, your key equivalents can be handled straight.  The names of your buttons should all begin with unique first letters, and if you use command-key entry you should provide a legend of command-key equivalents next to the buttons. */
	
pascal Boolean FProc(theDialog,  theEvent, 			itemHit)
	
	DialogPtr	theDialog;
	EventRecord 	*theEvent;
	int		*itemHit;
{
	long	key;			
/* Holds the key code */
	Point	ctr;			
/* where the mouse click will go */
	int	DIResult;		
/* Returned by DIBadMount */
	EventRecord	diskEvent;		
/* Returned by EventAvail, below */

/* Since ModalDialog doesn't handle bad disk mounts, we have a handler that will allow the user to format new disks with a modal dialog still on the screen. */

if (GetNextEvent(diskMask, &diskEvent) == TRUE)
{
	if (HiWord(diskEvent.message) != 0) 
	{
		diskEvent.where.h = 				((screenBits.bounds.right - 		screenBits.bounds.left)/ 2) - 
		(304 / 2);
		diskEvent.where.v = 				((screenBits.bounds.bottom - 
		screenBits.bounds.top)/ 3) 
		- (104 / 2);
		InitCursor();
		DIResult = 					DIBadMount(diskEvent.where, 
		diskEvent.message);
	}

}	
/* end of if GetNextEvent test for disk events */

switch (theEvent->what)
{
	case (updateEvt):
	HandleUpdate(theDialog);
	return FALSE;
/*Do not return an item hit value.  Returning
FALSE tells modalDialog to handle the update
event.  We have just added our own handling
of the event. */
	break;	
									
	case (keyDown):
	key = theEvent->message & charCodeMask;
	switch(key)
	{	
/* If a key has been pressed, we want to interpret it properly. */
		case 13: /* Return key */
		{	
		FlashDialogItem(theDialog, 1);
	ctr = CenterDialogItem(theDialog, 1);

/* We can do it this way: change the event record to fool ModalDialog.  ModalDialog doesn't flash the button long enough for my taste, so I
flash it some more myself. */
							
		theEvent->what = mouseDown;
		LocalToGlobal (&ctr);
		theEvent->where = ctr;
		theEvent->message = 0;

/* Now we tell ModalDialog to handle the
event.  It doesn't suspect a thing! */
								*itemHit = 1;
		return FALSE;
		break;
	}

		case 3: /* the Enter key */
		{	

/* Or we can do it our own way by flashing the button and telling ModalDialog to exit. */
								FlashDialogItem(theDialog, 1);
		*itemHit = 1;
		return TRUE;
		break;
		}
						
/* These key equivalents do not rely on the use
of the command key.  You would use these in
your modal dialog only if you had no editable
text fields. */

		case 49: /* 1 key */
		{	
		FlashDialogItem(theDialog, 1);
		*itemHit = 1;
		return TRUE;
		break;
		}

		case 50: /* 2 key */
		{	
		FlashDialogItem(theDialog, 2);
		*itemHit = 2;
		return TRUE;
		break;
		}

		case 51: /* 3 key */
		{	
		FlashDialogItem(theDialog, 3);
		*itemHit = 3;
		return TRUE;
		break;
		}
						
/* These key equivalents for buttons use
the command key, and are appropriate even 
for use in a modal dialog with editable text. */
		
		case 102:	/* ASCII F */
		{	
		if (theEvent->modifiers & cmdKey)
			{
		FlashDialogItem(theDialog, 1);
			*itemHit = 1;
			return TRUE;
			break;
			}
		}
							
		case 115:	/* ASCII S */
		{	
		if (theEvent->modifiers & cmdKey)
			{
		FlashDialogItem(theDialog, 2);
			*itemHit = 2;
			return TRUE;
			break;
			}
		}

		case 116:	/* ASCII T */
								{	
		if (theEvent->modifiers & cmdKey)
			{
		FlashDialogItem(theDialog, 3);
			*itemHit = 3;
			return TRUE;
			break;
			}
		}
	
		default:
		{
		return FALSE;
/* Do nothing if another key is chosen. */
		break;
		}
	}	/* end of key code switch */
		
case (mouseDown):
						
/* You can insert your own mouse click
handlers here.  ModalDialog takes care of mouse events, but you can do do special processing.  For example, ModalDialog does nothing if a mouse click
occurs inside a modal dialog but outside of a control, but you could do something.  */
							
	return FALSE;
	break;
case (mouseUp):
		return FALSE;
		break;
default:	
		return FALSE;
		break;		
/* We don't handle any other types of events, so
these get sent to ModalDialog unchanged. */		
	}	/* end of switch */
} /* end of filterproc function */

/************************************************/
/* HandleDialog is a function to draw and dispose of a simple modal dialog.  It tells ModalDialog to use a filter proc to screen events for the dialog. 

Input: short Dialog_ID - the resource ID
Output: item number of the control that was hit to
exit ModalDialog. */

int HandleDialog(dialog_ID)
		short	dialog_ID;		
{			
	int	itemHit;		
/* returned by ModalDialog */
	DialogPtr	theDialog;		
/* The dialog we will work with */
	GrafPtr	oldWindow;		
/* Saves the previous window */		

GetPort(&oldWindow);			
/* save current grafPort */
SetDAFont(0);					
/* use system font */
/* First, get the dialog from the resource and prepare to execute ModalDialog */
theDialog = GetNewDialog(dialog_ID, (Ptr)0, 		(WindowPtr)-1);
if (ResError() != noErr)
/* Called if GetNewDialog returns an error.  You can put your own error handler here.  In this example we are assuming that the dialog is available in the application resource fork */
	{
	SysBeep(40);
	ExitToShell();
	}
	
ShowWindow(theDialog);
SetPort(theDialog);
FlushEvents(everyEvent, 0);
	
/* Modal dialog call should wait until event in
active control */
		
ModalDialog((ProcPtr)FProc, &itemHit);
DisposDialog(theDialog);
FlushEvents (everyEvent, 0);
SetPort(oldWindow);			
/* restore drawing state */
return itemHit;
}			

