unit DemoDialog;
interface
	uses
		KeyBoard, Memory, ListUtilities, Dialogs, MenuUtilities, DemoConstants;
	procedure DemoDialog (Data: DataHandle;
									var error: boolean);
implementation
	procedure FillDataList (Data: DataHandle;
									ListGlobal: ListGlobalHandle;
									var error: boolean);
		var
			i: integer;
			theCell: cell;
	begin
		if not error then
			for i := 0 to DataRecords(Data, error) - 1 do
				begin
					theCell.v := i;
					theCell.h := 0;
					LSetCellString(data^^[i].name, theCell, ListGlobal^^.List);
					theCell.h := 1;
					LSetCellString(data^^[i].formula, theCell, ListGlobal^^.List);
					theCell.h := 2;
					LSetCellNumber(data^^[i].weight, 2, 0, theCell, ListGlobal^^.List);
					theCell.h := 3;
					LSetCellNumber(data^^[i].reference, 0, 0, theCell, ListGlobal^^.List);
				end;
	end;
{***************************}
	procedure SaveDataList (Data: DataHandle;
									ListGlobal: ListGlobalHandle;
									var error: boolean);
		var
			i: integer;
			theCell: cell;
	begin
		if not error then
			begin
				DisposeCell(ListGlobal);{just in case a cell was currently active}
				for i := 0 to DataRecords(Data, error) - 1 do
					begin
						theCell.v := i;
						theCell.h := 0;
						data^^[i].name := LGetCellString(theCell, ListGlobal^^.List);
						theCell.h := 1;
						data^^[i].formula := LGetCellString(theCell, ListGlobal^^.List);
						theCell.h := 2;
						data^^[i].weight := LGetCellNumber(theCell, ListGlobal^^.List);
						theCell.h := 3;
						data^^[i].reference := round(LGetCellNumber(theCell, ListGlobal^^.List));
					end;
			end;
	end;
{***************************}
	procedure SortDataList (Data: DataHandle;
									ListGlobal: ListGlobalHandle;
									SortType: integer;
									var error: boolean);
		var
			i, n, loop: integer;
			changed, switch: boolean;
			temp: DataRecord;
	begin
		if not error then
			begin
				SaveDataList(Data, ListGlobal, error);
{do a simple sort routine}
				n := DataRecords(Data, error);
				loop := 1;
				repeat
					changed := false;
					for i := loop to n - 1 do
						begin{do}
							case SortType of
								bSortName: 
									switch := (1 = IUCompString(Data^^[i - 1].name, Data^^[i].name));
								bSortFormula: 
									switch := (1 = IUCompString(Data^^[i - 1].formula, Data^^[i].formula));
								bSortWeight: 
									switch := (Data^^[i - 1].weight > Data^^[i].weight);
								bSortReference: 
									switch := (Data^^[i - 1].reference > Data^^[i].reference);
							end;{case}
							if switch then
								begin
									changed := true;
									Temp := Data^^[i - 1];
									Data^^[i - 1] := Data^^[i];
									Data^^[i] := Temp;
								end;
						end;{do}
					if loop = n - 1 then
						changed := false;
				until not changed;


				FillDataList(Data, ListGlobal, error);
				SetUpCell2(ListGlobal^^.selCell, ListGlobal);
			end;
	end;
{***************************}
	procedure DemoDialog;
		var
			Dlg: DialogPtr;
			i, itemHit: integer;

			ListGlobal: ListGlobalHandle;
			rows: integer;

			lastPopUpItem, EditItem: integer;
			MenuID: integer;

			oldData: DataHandle;

			LastEditText: str255;
	begin
{save the old data in case the user cancels the dialog}
		oldData := Data;
		myHandToHand(handle(oldData), error);

		if not error then
			begin {error check}
				mySetupDialog(128, Dlg, error);
{initialize the popup menu item}
				SetUpPopUp(Dlg, uDemoPopUp, GetMenu2(mDemo));
				SetUpPopUp(Dlg, uEditPopUp, GetMenu2(mEditPopUp));
				lastPopUpItem := 1;
				DrawPopUp(Dlg, uDemoPopUp, GetMenu2(mDemo), lastPopUpItem);
				DrawPopUp(Dlg, uEditPopUp, GetMenu2(mEditPopUp), 1);

				rows := DataRecords(data, error);
				SetUpListItem(Dlg, nListItem, ListGlobal, cols, rows);
				ListGlobal^^.number[0] := false;
				ListGlobal^^.number[1] := false;
				ListGlobal^^.integer[3] := true;

				FillDataList(Data, ListGlobal, error);

{now, handle events for the ModalDialog}
				repeat
					ModalDialog(@ListFilterProc, itemHit);
					case itemHit of
						bDelete: 
							DeleteRow(ListGlobal);
						bInsert: 
							InsertRow(ListGlobal);
						bAppend: 
							AppendRow(ListGlobal);
						bSortName, bSortFormula, bSortWeight, bSortReference: 
							SortDataList(data, ListGlobal, itemHit, error);
						eSampleEditItem: 
							CheckNumberField(Dlg, eSampleEditItem, LastEditText, false);
						uDemoPopUp: 
							begin
								TrackPopUp(Dlg, uDemoPopUp, sDemoPopUp, GetMenu2(mDemo), MenuID, lastPopUpItem);
								DrawPopUp(Dlg, uDemoPopUp, GetMenu2(mDemo), lastPopUpItem);
							end;
						uEditPopUp: 
							begin
								EditItem := 1;
								TrackPopUp(Dlg, uEditPopUp, sEditPopUp, GetMenu2(mEditPopUp), MenuID, EditItem);
								DrawPopUp(Dlg, uEditPopUp, GetMenu2(mEditPopUp), 1);
								if MenuID > 0 then
									case EditItem of
										iCut: 
											HandleListKey(ListGlobal, xKey, '0', CmdKey);
										iCopy: 
											HandleListKey(ListGlobal, cKey, '0', CmdKey);
										iPaste: 
											HandleListKey(ListGlobal, vKey, '0', CmdKey);
										iClear: 
											HandleListKey(ListGlobal, DummyClearKey, '0', CmdKey);
										iOptionPaste: 
											HandleListKey(ListGlobal, DummyOptionPasteKey, '0', CmdKey);
									end;
							end;
					end;{case}
				until itemHit <= 3;

				SaveDataList(data, ListGlobal, error);
				if itemHit = bCancel then
					begin{cancelled}
						DisposData(data, error);
						Data := oldData;
					end{cancelled}
				else{not cancelled}
					DisposData(oldData, error);

				DisposDialog(Dlg);
			end;{error check}
	end;
end.