PROGRAM SlotVBLTaskDemo;


USES
	MemTypes, QuickDraw, OSIntf, ToolIntf;


{ Declare our external vbl task }
PROCEDURE MyVBLTask; EXTERNAL;


CONST
	F	=	False;
	T	=	True;

	kBallSpeed	=	4;


TYPE
	XVBLRec	=	RECORD
		xVBLTask	:	VBLTask;
		xDoTask	:	INTEGER;
	END;


VAR
	gDone	:	BOOLEAN;

	gVideoInfoRec	:	DefVideoRec;

	gEvt	:	EventRecord;

	gFontInfo	:	FontInfo;

	gBoxDir		:	INTEGER;
	gBoxRect	:	Rect;
	gBoxColor	:	RGBColor;
	gBoxString	:	Str255;

	gDragRect,
	gRect	:	Rect;
	gMainWindow	:	WindowPtr;
	gWRec	:	WindowRecord;

	gXVBLRec	:	XVBLRec;


{ Proc to run if a crash occurs }
PROCEDURE Crash;
BEGIN
	ExitToShell;
END;


{ Proc that does MacInits }
PROCEDURE MacInits;
BEGIN
	MaxApplZone;

	MoreMasters;
	MoreMasters;
	MoreMasters;
	MoreMasters;

	InitGraf (@thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs (@Crash);
	InitCursor;
END;


{ Proc that sets up our window }
PROCEDURE SetUpWindows;
BEGIN
	gRect := ScreenBits.bounds;
	WITH gRect DO BEGIN
		left := left + 20;
		right := left + 400;
		top := top + 40;
		bottom := bottom - 20;
	END;

	gMainWindow := NewCWindow (@gWRec, gRect,
		'SlotVBLTaskDemo - Click in this window to Exit',
		T, noGrowDocProc, WindowPtr (-1), F, 0);
	SetPort (gMainWindow);
END;


{ Proc that sets up the box for our ball }
PROCEDURE SetUpBox;
VAR
	width	:	INTEGER;
BEGIN
	gBoxDir := kBallSpeed;

	gBoxString := 'I love you Manu';

	width := StringWidth (gBoxString);

	GetFontInfo (gFontInfo);

	WITH gBoxRect DO BEGIN
		top := 10;
		bottom := top + gFontInfo.ascent + gFontInfo.descent;
		left := ((gRect.right - gRect.left) DIV 2) - (width DIV 2);
		right := left + width;
	END;

	WITH gBoxColor DO BEGIN
		red := -1;
		green := 0;
		blue := 0;
	END;

	RGBForeColor (gBoxColor);
END;


{ Proc that inits our globals }
PROCEDURE GlobalInits;
BEGIN
	SetUpWindows;

	SetUpBox;

	gDragRect := ScreenBits.bounds;
	InsetRect (gDragRect, 4, 4);

	gDone := F;
END;


{ Proc that cleans up before we exit }
PROCEDURE CleanUps;
BEGIN
	CloseWindow (gMainWindow);
END;


{ Func that makes sure we run in the currect environment }
FUNCTION EnvironmentOK : BOOLEAN;
VAR
	err	:	OSErr;
	theWorld	:	SysEnvRec;
BEGIN
	EnvironmentOK := F; { Assume env is bad }

	err := SysEnvirons (curSysEnvVers, theWorld);

	IF (err = noErr) THEN BEGIN
		IF (theWorld.machineType >= envMacII) THEN
			EnvironmentOK := T;
	END; { IF }
END;


{ Func that sets up our vbl task }
FUNCTION VBLTaskSetUp : BOOLEAN;
VAR
	err	:	OSErr;
BEGIN
	VBLTaskSetUp := F; { Assume we fail }

	GetVideoDefault (@gVideoInfoRec);

	WITH gXVBLRec DO BEGIN
		xDoTask := 0;

		WITH xVBLTask DO BEGIN
			qType := ORD (vType);
			vblAddr := @MyVBLTask;
			vblCount := 1;
			vblPhase := 0;
		END;

		err := SlotVInstall (@xVBLTask, gVideoInfoRec.sdSlot);
		IF (err = noErr) THEN
			VBLTaskSetUp := T; { We succeeded! }
	END;
END;


{ Proc that removes our vbl task }
PROCEDURE RemoveVBLTask;
VAR
	err	:	OSErr;
BEGIN
	err := SlotVRemove (@gXVBLRec.xVBLTask, gVideoInfoRec.sdSlot);
END;


{ Proc that draws our window }
PROCEDURE DrawStuff;
VAR
	oldPort	:	GrafPtr;
BEGIN
	GetPort (oldPort);

	SetPort (gMainWindow);

	EraseRect (gBoxRect);

	WITH gBoxRect DO BEGIN
		top := top + gBoxDir;
		bottom := bottom + gBoxDir;
	END;

	IF (gBoxRect.bottom >= gWRec.port.portRect.bottom) THEN
		gBoxDir := -kBallSpeed
	ELSE BEGIN
		IF (gBoxRect.top <= gWRec.port.portRect.top) THEN
			gBoxDir := kBallSpeed;
	END;

	MoveTo (gBoxRect.left, gBoxRect.bottom - gFontInfo.descent);
	DrawString (gBoxString);

	SetPort (oldPort);
END;


{ Proc that handles mouse downs }
PROCEDURE DoMouseDown;
VAR
	result	:	INTEGER;
	whichWindow	:	WindowPtr;
BEGIN
	result := FindWindow (gEvt.where, whichWindow);

	CASE result OF
		inContent:
			gDone := T;
		inDrag:
			DragWindow (whichWindow, gEvt.where, gDragRect);
		OTHERWISE
	END;
END;


{ Main }
BEGIN
	MacInits;

	IF (EnvironmentOK) THEN BEGIN
		GlobalInits;

		IF (VBLTaskSetUp) THEN BEGIN

			REPEAT
				IF NOT (WaitNextEvent (mDownMask, gEvt, 0, NIL)) THEN BEGIN
					IF (gXVBLRec.xDoTask <> 0) THEN BEGIN
						DrawStuff;
						gXVBLRec.xDoTask := 0;
					END;
				END
				ELSE
					DoMouseDown;
			UNTIL (gDone);

			RemoveVBLTask;
		END;

		CleanUps;
	END;

	ExitToShell; { Back to the Finder! (or MultiFinder) }
END.
