{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ USavedMe.inc1.p
    Copyright  1989 Fred J. Condo Jr.
    All rights reserved. }

CONST
    autoSaving          = TRUE;

    {$S AOpen}

PROCEDURE TAutoSaveDocument.IAutoSaveDocument(itsFileType, itsCreator: OSType; usesDataFork,
                                              usesRsrcFork: BOOLEAN; keepsDataOpen,
                                              keepsRsrcOpen: BOOLEAN);

    BEGIN
    {$IFC qDebug}
    IF gIntenseDebugging THEN
        WriteLn('TAutoSaveDocument.IAutoSaveDocument');
    {$ENDC}
    IDocument(itsFileType, itsCreator, usesDataFork, usesRsrcFork, keepsDataOpen, keepsRsrcOpen);
    { Initial conditions are... }
    fSaving := NOT autoSaving;
    fRespectIdleTime := FALSE;
    fHasEverBeenSet := FALSE; fLastSave := 0;           { never saved }
    fDesiredInterval := MAXLONGINT;                     { never want it saved. (This is 1 yr., 1
                                                         mo., 19 days) }
    fAutoSaveWindow := NIL                              { no dialog exists }
    END;

{$S AClose}

PROCEDURE TAutoSaveDocument.Free; OVERRIDE;

    BEGIN
{ If this object is being freed and its dialog is
  still open (which can happen because it is a
        modeless dialog), then close the dialog. }

    {$IFC qDebug}
    IF gIntenseDebugging THEN
        WriteLn('TAutoSaveDocument.Free');
    {$ENDC}

    IF fAutoSaveWindow <> NIL THEN
        fAutoSaveWindow.Close;
    INHERITED Free
    END;

{$S ARes}

FUNCTION TAutoSaveDocument.DoIdle(Phase: IdlePhase): BOOLEAN; OVERRIDE;

    VAR
        dontCare:           TCommand;
        currentTicks:       LongInt;

    BEGIN
    DoIdle := FALSE;                                    { I don't free myself }
    currentTicks := TickCount;

    {$IFC qDebug}
    IF gIntenseDebugging THEN
        WriteLn(fTitle^^, ' has idled');
    IF fSaving & (currentTicks - fLastSave >= fDesiredInterval) THEN
        WriteLn('Enough time has elapsed to save.');
    {$ENDC}

    IF fSaving & (fChangeCount > 0) & (currentTicks - fLastSave >= fDesiredInterval) THEN
        BEGIN
        dontCare := DoMenuCommand(cSave);
        fLastSave := currentTicks
        END
    END;

{$S ARes}

PROCEDURE TAutoSaveDocument.DoSetupMenus; OVERRIDE;

    BEGIN
    {$IFC qDebug}
    WriteLn('AutoSaveDocument.DoSetupMenus:');
    {$ENDC}

    INHERITED DoSetupMenus;
    EnableCheck(cToggleAutoSave, fHasEverBeenSet, fSaving);
    Enable(cSetAutoSave, fSaveExists)                   { Allow auto-saving only after the file has
                                                         a name }
    END;

{$S ARes}

FUNCTION TAutoSaveDocument.DoMenuCommand(aCmdNumber: CmdNumber): TCommand; OVERRIDE;

    VAR
        anAutoSaveDialogView: TAutoSaveDialogView;

    BEGIN

    {$IFC qDebug}
    IF gIntenseDebugging THEN
        WriteLn('TAutoSaveDocument.DoMenuCommand:', aCmdNumber);
    {$ENDC}

    DoMenuCommand := gNoChanges;
    CASE aCmdNumber OF

        cToggleAutoSave:
            BEGIN
            fSaving := NOT fSaving;
            IF NOT fSaving & NOT fRespectIdleTime THEN
                fIdleFreq := MAXLONGINT
            END;

        cSetAutoSave:
            IF fAutoSaveWindow = NIL                    { Open a dialog only if one is not already
                                                         open }
               THEN
                PoseAutoSaveDialog
            ELSE
                fAutoSaveWindow.Select;

        cSave, cSaveAs:
            BEGIN
 { Intercept cSave and cSaveAs commands to make
      sure that the document name given in the
            AutoSave Dialog is still correct. If there is
            no dialog open, skip it. }
            Save(aCmdNumber, ((aCmdNumber = cSaveAs) | (NOT fSaveExists)) { Put up SFPutFile if Save
                                                                           As } , FALSE);
            IF (fAutoSaveWindow <> NIL) THEN
                BEGIN
                anAutoSaveDialogView := TAutoSaveDialogView(fAutoSaveWindow.FindSubView('DLOG'));
                FailNIL(anAutoSaveDialogView);
                anAutoSaveDialogView.UpdateDocumentName
                END
            END;                                        { cSave }
        OTHERWISE
            DoMenuCommand := INHERITED DoMenuCommand(aCmdNumber)
    END                                                 { Case }
    END;                                                { DoMenuCommand }

{$S ANonRes}

PROCEDURE TAutoSaveDocument.PoseAutoSaveDialog;
{ This method creates and opens a modeless dialog }

    VAR
        theDialog:          TAutoSaveDialogView;
        theTimeText:        TNumberText;
        theWordMinutes:     TStaticText;
        minutesStr:         Str255;
        whichStr:           1..2;

    BEGIN

    {$IFC qDebug}
    IF gIntenseDebugging THEN
        WriteLn('TAutoSaveDocument.PoseAutoSaveDialog');
    {$ENDC}

    fAutoSaveWindow                                     { This field will allow the auto-save object
                                                         to keep track of its dialog }
      := NewTemplateWindow(kAutoSaveDialogID, NIL);
    FailNIL(fAutoSaveWindow);
    theDialog := TAutoSaveDialogView(fAutoSaveWindow.FindSubView('DLOG'));
    FailNIL(theDialog);

        { Now give the TAutoSaveDialogView a
                                  reference to the auto-save object so
                                        that it can update the appropriate
                                        fields when it is dismissed. }
    theDialog.fAutoSaveDocument := SELF;
    theDialog.UpdateDocumentName;

    { Now display the proper desired interval, if
                  one's been previously set, or the default;
      and determine whether to use the singular or
                        plural of "minute." }
    theTimeText := TNumberText(theDialog.FindSubView('time'));
    FailNIL(theTimeText);
    IF fHasEverBeenSet THEN
        theTimeText.SetValue(fDesiredInterval DIV 3600, FALSE {no redraw; not open yet} );
    IF theTimeText.GetValue = 1 THEN
        whichStr := 1
    ELSE
        whichStr := 2;
    theWordMinutes := TStaticText(theDialog.FindSubView('mnut'));
    FailNIL(theWordMinutes);
    GetIndString(minutesStr, kMinutesID, whichStr);
    theWordMinutes.SetText(minutesStr, FALSE);
    fAutoSaveWindow.Open
    END;

{$S AFields}

PROCEDURE TAutoSaveDocument.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr;
                                                       fieldType: Integer)); OVERRIDE;

    BEGIN
    DoToField('TAutoSaveDocument', NIL, bClass);
    DoToField('fSaving', @fSaving, bBoolean);
    DoToField('fHasEverBeenSet', @fHasEverBeenSet, bBoolean);
    DoToField('fRespectIdleTime', @fRespectIdleTime, bBoolean);
    DoToField('fLastSave', @fLastSave, bLongInt);
    DoToField('fDesiredInterval', @fDesiredInterval, bLongInt);
    DoToField('fAutoSaveWindow', @fAutoSaveWindow, bObject);
    INHERITED Fields(DoToField)
    END;

{$S ANonRes}

PROCEDURE TAutoSaveDialogView.DismissDialog(dismisser: IDType; flashDismisser: BOOLEAN); OVERRIDE;

    BEGIN

    {$IFC qDebug}
    IF gIntenseDebugging THEN
        WriteLn('TAutoSaveDialogView.DismissDialog');
    {$ENDC}

    IF NOT fDismissed AND DeselectCurrentEditText THEN
        BEGIN
        INHERITED DismissDialog(dismisser, flashDismisser);
        IF dismisser = 'ok  ' THEN
            BEGIN
            fAutoSaveDocument.fSaving := autoSaving;
            fAutoSaveDocument.fHasEverBeenSet := TRUE;
            fAutoSaveDocument.fDesiredInterval := 60 * 60 * { The interval is specified in minutes
                                                             by the user. We want ticks. }
                                                  TNumberText(FindSubView('time')).GetValue;
            IF fAutoSaveDocument.fRespectIdleTime THEN
                BEGIN
                IF fAutoSaveDocument.fDesiredInterval < fAutoSaveDocument.fIdleFreq THEN
                    fAutoSaveDocument.fIdleFreq := MAX(1, fAutoSaveDocument.fDesiredInterval DIV 2)
                                { This  way, we don't stomp on a
          co-handler's idle frequency. }
                END
            ELSE
                fAutoSaveDocument.fIdleFreq := fAutoSaveDocument.fDesiredInterval DIV 2
            END;
        fAutoSaveDocument.fAutoSaveWindow := NIL;       { Tell the document there's no dialog to
                                                         close }
        TWindow(fSuperView).Close
        END
    END;

{$S ANonRes}

PROCEDURE TAutoSaveDialogView.UpdateDocumentName;

    VAR
        theDocName:         TStaticText;
        theDocTitle:        Str255;

    BEGIN

    {$IFC qDebug}
    IF gIntenseDebugging THEN
        WriteLn('TAutoSaveDialogView.UpdateDocumentName');
    {$ENDC}

{ Get the name of the document so the dialog can
  display it for the user }
    theDocName := TStaticText(FindSubView('docn'));
    FailNIL(theDocName);
    theDocTitle := fAutoSaveDocument.fTitle^^;
    theDocName.SetText(theDocTitle, TRUE                {redraw} )
    END;

{$S AClose}

PROCEDURE TAutoSaveDialogView.Free; OVERRIDE;
{ This is overridden to inform the document that
  it has no dialog available. This is done so
        that, if the user clicks the goAway of the
        dialog, the document is informed. }

    BEGIN

    {$IFC qDebug}
    IF gIntenseDebugging THEN
        WriteLn('TAutoSaveDialogView.Free');
    {$ENDC}

    fAutoSaveDocument.fAutoSaveWindow := NIL;
    INHERITED Free
    END;

{$S AClose}

PROCEDURE TAutoSaveDialogView.Close; OVERRIDE;

    BEGIN
    fDismissed := TRUE;                                 { Pretend we've dismissed with the Cancel
                                                         button, }
    fDismisser := 'cncl';                               { but without doing any real dismissing,
                                                         because objects may be freed. }
    INHERITED Close
    END;

{$S AFields}

PROCEDURE TAutoSaveDialogView.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr;
                                                         fieldType: Integer)); OVERRIDE;

    BEGIN
    DoToField('TAutoSaveDialogView', NIL, bClass);
    DoToField('fAutoSaveDocument', @fAutoSaveDocument, bObject);
    INHERITED Fields(DoToField)
    END;

{$S AInit}

PROCEDURE InitUSavedMe;                                 { This just makes sure our dialog view
                                                         classes don't get stripped. }

    VAR
        g:                  BOOLEAN;

    BEGIN

    {$IFC qDebug}
    IF gIntenseDebugging THEN
        WriteLn('InitUSavedMe');
    {$ENDC}

    IF gDeadStripSuppression THEN
        g := Member(TObject(NIL), TAutoSaveDialogView)
    END;
