;File:   Volume-Fkey.Asm
;-------------------------------------------------
;An FKEY that lets you change the speakers volume
; 1989 by John Holder for MacTutor
;Thu May 4, 1989
;-------------------------------------------------
;The purpose of this Fkey is to show how to set-up & use
;an item list in memory for a dialog (instead of using a 
;DITL resource) and to show how to check and set the Macs
;speaker volume

	INCLUDE		Traps.D
	Include 	ToolEqu.D	; Use ToolBox equates
	Include		SysEqu.D	; Use System equates

;some useful macros
MACRO	MakePointer	HowManyBytes,StorageRegister =
	MOVE.L		#{HowManyBytes},D0
	_NewPtr,CLEAR
	MOVE.L		A0,{StorageRegister}
	|

MACRO	MakeHandle	HowManyBytes,StorageRegister =
	MOVE.L		#{HowManyBytes},D0
	_NewHandle,CLEAR
	MOVE.L		A0,{StorageRegister}
	|

MACRO	SaveRegs	=
	MOVEM.L		A0-A4/D0-D7,-(SP)
	|

MACRO	RestoreRegs	=
	MOVEM.L		(SP)+,A0-A4/D0-D7
	|

MACRO	beep	=
	MOVE.W		#10,-(SP)
	_SysBeep
	|


;-------------------------------------------------
;All equates go here
;-------------------------------------------------
;these are offsets into our global storage.
;(a non-relocatable block we created pointed to by A4)
;-------------------------------------------------
EventBlock	equ	0
what 		equ	0	; Event number
message		equ	2	; Additional information
when		equ	6	; Time event was posted 
where		equ	10	; Mouse coordinates
modify		equ	14	; State of keys and button

TheWindow	equ	16	; var used by _DialogSelect

windowpointer	equ	20	; Dialog ptr

ItemListHandle	equ	24	; Item List handle

ItemHit		equ	28	; word

CurrentVol	equ	30	; current Volume (byte)
padding		equ	31	; next byte (unused)

DispRect	equ	32
ItemNumber	equ	40
ItemHandle	equ	42
ItemType	equ	46

MemNeeded	equ	100	;how much memory needed 					;for globals



mDownMask	equ	2
keyDownMask	equ	8

true		equ	$0100
false		equ	0
nil		equ	0


;Buttons used in dialog
QuitButton	equ	1
InfoButton	equ	2

ZeroBut		equ	3
OneBut		equ	4
TwoBut		equ	5
ThreeBut	equ	6
FourBut		equ	7
FiveBut		equ	8
SixBut		equ	9
SevenBut	equ	10

BytesInItemList	equ	300	;how many bytes we want
				;allocated for the dialog's 					;item list

SysParam	equ	$1F8	;location of copy of Parameter
				;RAM (PRAM) settings (20 bytes)
				;used by _WriteParam

SPVolCtl	equ	$208	;location of the speaker vol.
				;setting in the copy of PRAM
				;settings

VIA		equ	$1D4	;contains address of the
				;VIA buffer
vBufA		EQU  	$1E00	;Offset to Buffer A (of the
				; VIA)  (for the sound chip)


;-------------------------------------------------
;The start of the Fkey
;-------------------------------------------------
JumpIt
	BRA.S	Start	;Jump the FKEY header info

	DC.W		0		
	DC.B		'FKEY'	
	DC.W		7	;(FKEYs res id#)
	DC.W		0	

Start

	SaveRegs

	;save current port
	SUBQ		#4,SP
	MOVE.L		SP,-(SP)
	_GetPort

	_InitCursor

	;allocate storage for our globals
	MakePointer	MemNeeded,A4
	cmp.l		#0,A4		  ;if returned ptr is not
	bne.s		Pointer_Is_Fine	  ;zero it was allocated ok

	;not enough mem if it gets here, so beep & quit
	bsr		Theres_Been_an_Error
	bra		No_Mem_Must_Quit

Pointer_Is_Fine	;will get here if pointer was allocated!

	;set up storage for our dialogs Item List
	MakeHandle	BytesInItemList,ItemListHandle(A4)
	cmp.l		#0,ItemListHandle(A4)	;if returned handle
	bne.s		Handle_Is_Fine    	;is not zero it was 					   ;allocated ok

	;not enough mem if it gets here, so beep,
	;release global storage we just allocated & quit
	bsr		Theres_Been_an_Error
	bra		No_Handle_Jump

Handle_Is_Fine	;will get here if handle was allocated!

	;save current speaker volume level
	;speaker volume level is store in low
	;3 bits of global var sdVolume
	move.b		sdVolume,D0
	and.b		#$07,D0	;clear all but low 3 bits
	move.b		D0,CurrentVol(A4)

	bsr		ShowDialog


;-------------------------------------------------
;The Main loop
;-------------------------------------------------
MainLoop

	;let _ModalDialog handle all events
	clr.l		-(sp)		;filter proc
	pea		ItemHit(A4)	;returns dialog item hit
	_ModalDialog

	;which item was clicked?
	cmp		#QuitButton,ItemHit(A4)
	beq		QuitRoutine

	cmp		#InfoButton,ItemHit(A4)
	beq		DoAbout

	;if it wasn't on of the two buttons
	;check for click in the radio buttons

	;if ItemHit(A4) is < the first radio 
	;button or > the last one we don't
	;handle it
	cmp		#ZeroBut,ItemHit(A4)
	blt		MainLoop

	cmp		#SevenBut,ItemHit(A4)
	bgt		MainLoop

	move		ItemHit(A4),D7
	bsr		ChangeAndBeep

	bra		MainLoop	;back to the Main loop



;-------------------------------------------------
;Turn on button selected and beep
;-------------------------------------------------
ChangeAndBeep
	saveregs

	;it was a click in a radio button
	;first, unhilite all the radio buttons!
	bsr		Clear_Radio_Buttons

	;now turn on the one that was clicked on!
	move.l		windowpointer(A4),-(SP)
	move		D7,-(sp)
	pea		ItemType(A4)
	pea		ItemHandle(A4)
	pea		DispRect(A4)
	_GetDItem

	move.l		ItemHandle(A4),-(sp)
	move		#1,-(sp)
	_SetCtlValue


	;calculate new volume setting by which button was
	;clicked.
	;button clicked (3-10) - first button (3) = new
	;volume setting
	move		D7,D2
	sub		#ZeroBut,D7
	and.b		#$07,D7	;clears all but low 3 bits


	;set the sdVolume global var to sound 
	;level chosen!
	move		D7,D2	;put new setting into D2

	move.b		sdVolume,D1	;get current value
	and.b		#$F8,D1		;clears low 3 bits (leaves 
					;the rest of the sdVolume
					;bits unchanged)

	or.b		D1,D2		;combine the two

	move.b		D2,sdVolume	;set the global to new value

	;save new speaker volume level
	move.b		sdVolume,D0
	and.b		#$07,D0		;clear all but low 3 bits
	move.b		D0,CurrentVol(A4)


	;set the SPVolCtl (part of Parameter RAM vars kept
	;in low memory globals)
	move		D7,D2		;put new setting into D2

	move.b		SPVolCtl,D1	;get current value
	and.b		#$F8,D1		;clears low 3 bits (leaves 
					;the rest of the SPVolCtl
					;bits unchanged)

	or.b		D1,D2		;combine the two

	move.b		D2,SPVolCtl	;set the global to new value



	;last, set the VIA Buffer A global var
	;to sound level chosen (this is the sound
	;chip buffer A)
	move		D7,D2		;put new setting into D2

	move.l		VIA,A0		;pointer to Vbase
	move.b		vBufA(A0),D1	;get current value

	and.b		#$F8,D1		;clears low 3 bits (leaves 
					;the rest of the
					;bits unchanged)

	or.b		D2,D1		;combine the two

	move.l		VIA,A0		;pointer to Vbase
	move.b		D1,vBufA(A0)	;set the global to new value

	;now change parameter RAM
	;Saves settings in clock chip
	lea		SysParam,A0
	move.l		minusOne,D0
	_WriteParam

	beep				;beep to hear new setting

	restoreregs
	rts



;-------------------------------------------------
;Do about (info button was pushed)
;-------------------------------------------------
DoAbout
	saveregs

	;set to Monaco 9pt
	MOVE.W		#Monaco,-(SP)
	_TextFont
	MOVE.W		#9,-(SP)
	_TextSize

	;use _TextBox to show About info
	pea		AboutTextBegin
	MOVE.L		#AboutTextEnd-AboutTextBegin,-(SP)
	PEA		TextRect
	MOVE.W		#0,-(SP)	;left justification
	_TextBox

	bsr		Wait_For_Event  ;wait for mouse or
					;key click

	;erase About text
	pea		TextRect
	_EraseRect

	;restore normal text attributes for the dialog
	MOVE.W		#0,-(SP)	;0 = systemFont
	_TextFont
	MOVE.W		#12,-(SP)	;12 point
	_TextSize

	;redraw the dialog to show all controls
	move.l		windowpointer(A4),-(sp)
	_DrawDialog

	restoreregs
	BRA		MainLoop



;-------------------------------------------------
;Quit, release all memory and return
;-------------------------------------------------
QuitRoutine

	;This will also dispose of the handle
	;we created for the item list so we dont
	;have to
	MOVE.L		windowpointer(A4),-(SP)
	_DisposDialog

No_Handle_Jump
	move.l		A4,A0
	_DisposPtr

No_Mem_Must_Quit 
	MOVE.L		#$0000FFFF,D0
	_FlushEvents			;flush of all events
	_SetPort			;restore port
	restoreregs
	rts



Theres_Been_an_Error			;will jump here if space
					;couldn't be allocated!
	beep
	Rts



;-------------------------------------------------
;Set up the dialog
;-------------------------------------------------
ShowDialog
	saveregs

	bsr		SetUpItmList

	;create the dialog
	CLR.L		-(SP)		;For ptr
	clr.l		-(SP)		;dialog storage
	pea		WindowRect	;rect of dialog
	clr.l		-(sp)		;title
	MOVE		#true,-(SP)	;vis
	MOVE.W		#dboxproc,-(SP)
	MOVE.L		#-1,-(SP)	;behind wind ptr
	MOVE		#false,-(SP)	;goaway
	clr.l		-(sp)		;refcon
	move.l		ItemListHandle(A4),-(sp) ;handle to item list
	_NewDialog
	MOVE.L		(SP),windowpointer(A4)  ;leave ptr on stack
	_SetPort

	bsr		SetUpControl	;show current vol. level

	restoreregs
	rts



;-------------------------------------------------
;Set up our item list
;-------------------------------------------------
SetUpItmList
	;blockmove our item list into space we
	;allocated for it at the beginning
	
	move.l		ItemListHandle(A4),A0	
	_HLock					;dont want it
						;moving!

	lea		ItmList,A0		;from ptr
	move.l		ItemListHandle(A4),A1
	move.l		(A1),A1
	move.l		#BytesInItemList,D0  	;lgth of data
	_BlockMove

	move.l	ItemListHandle(A4),A0  ;unlock it
	_HUnLock

	rts



;-------------------------------------------------
;Turn on button (depending on the current volume)
;-------------------------------------------------
SetUpControl
;depending on the value of sdVolume, turn on the appropriate
;radio button!
	saveregs

	clr		D6
	move.b		CurrentVol(A4),D6
	add		#ZeroBut,D6	;make volume level
					;correspond to a button
					;in the dialog

	move.l		windowpointer(A4),-(SP)
	move		D6,-(sp)
	pea		ItemType(A4)
	pea		ItemHandle(A4)
	pea		DispRect(A4)
	_GetDItem

	;turn on the button
	move.l		ItemHandle(A4),-(sp)
	move		#1,-(sp)
	_SetCtlValue

	restoreregs
	rts



;-------------------------------------------------
;Wait for key or mouse press
;-------------------------------------------------
Wait_For_Event
	LINK		A6,#-evtBlkSize
Here	
	LEA		-evtBlkSize(A6),A0
	MOVEQ		#mDownMask!KeyDownMask,D0
	_GetOSEvent
	BEQ		M_Down
	BRA		Here		;keep looping until mouse press!
M_Down
	UNLK		A6
	rts




;-------------------------------------------------
;Clear all radio buttons
;-------------------------------------------------
Clear_Radio_Buttons
	;Unhilite all the radio buttons!
	saveregs

	move		#7,D7		;how many times to loop
					;(8 buttons -1 for dbra)
	move		#ZeroBut,D6	;button to clear

Clear_Loop
	move.l		windowpointer(A4),-(SP)
	move		D6,-(sp)
	pea		ItemType(A4)
	pea		ItemHandle(A4)
	pea		DispRect(A4)
	_GetDItem

	move.l		ItemHandle(A4),-(sp)
	move		#0,-(sp)
	_SetCtlValue

	add		#1,D6		;inc. dialog item counter

	dbra		D7,Clear_Loop

	restoreregs
	rts



;-------------------------------------------------
;Constants
;-------------------------------------------------
WindowRect		DC.W	105,145,265,365
TextRect		DC.W	5,5,150,215  ;for About info

;Item list of our dialog (this data
;get copied into a handle
ItmList	dc	10		;number of items in list - 1

	dc.l	0			;place for handle
	dc	130,40,150,95		;item rect
	dc.b	btnCtrl+4		;Item type (Button Ctrl)
	dc.b	4			;length of following data
	dc.b	'Quit'			;Title

	dc.l	0		
	dc	130,125,150,180	
	dc.b	btnCtrl+4	
	dc.b	4		
	dc.b	'Info'		

	dc.l	0		
	dc	30,30,45,70	
	dc.b	radCtrl+4		 ;(Radio Button Ctrl)
	dc.b	4		
	dc.b	'0   '			; 0 + 3 spaces (same with 					;all radio buttons below)

	dc.l	0		
	dc	50,30,65,70	
	dc.b	radCtrl+4	
	dc.b	4		
	dc.b	'1   '		

	dc.l	0		
	dc	70,30,85,70	
	dc.b	radCtrl+4	
	dc.b	4		
	dc.b	'2   '		

	dc.l	0		
	dc	90,30,105,70	
	dc.b	radCtrl+4	
	dc.b	4		
	dc.b	'3   '		

	dc.l	0		
	dc	30,150,45,190	
	dc.b	radCtrl+4	
	dc.b	4		
	dc.b	'4   '		

	dc.l	0		
	dc	50,150,65,190	
	dc.b	radCtrl+4	
	dc.b	4		
	dc.b	'5   '		

	dc.l	0		
	dc	70,150,85,190	
	dc.b	radCtrl+4	
	dc.b	4		
	dc.b	'6   '		

	dc.l	0		
	dc	90,150,105,190	
	dc.b	radCtrl+4	
	dc.b	4		
	dc.b	'7   '		

	dc.l	0		
	dc	5,62,15,170	
	dc.b	statText	
	dc.b	12		
	dc.b	'Volume-Fkey '	; Volume-Fkey + 1 space

EndItmList


AboutTextBegin	
	dc.b	'        Volume-Fkey (1.0)',$0D
	dc.b	'     (c) 1989 by John Holder',$0D
	dc.b	'          for MacTutor',$0D,$0D
	dc.b	'  This Fkey lets you change the speaker volume. '
	dc.b	'Click on a button to set the volume.'
	.ALIGN	2
AboutTextEnd

	END