/* File RezC.CNTL.Cmdo.R */
/* Rez input for the Rezc.CNTL Commando interface */
/* W. Powell 1988 */

/* Rez definitions for Rez-building Commando interfaces */
#include "Types.r"
#include "Cmdo.R"

Resource 'cmdo' (200) {
	{ 285,
	"Rezc.CNTL produces Rez input for a Control resource",
	{
/* This include file has a standard arrangement of items
 *   common to all resources: ID#, attributes, name, 
 * 	 and redirection of output from the Rezc. script		*/
#include "CmdoReztStart.R"

/* Set up parameters for a Rectangle entry area				*/	
	#define RTop 85
	#define RLeft 30
	#define RTit "Control Rectangle"
	#define RDep {1}
/* This include file has a standard arrangement of edit boxes
 *   for input of rectangles								*/
#include "CmdoRect.R"

/* other items particular to this case */	
	/* Item 11 */
	OR {{1}}, NestedDialog {
		4, /* Dialog number */
		{80,250,101,360},
		"Control Type",
		"Define control type, range and value" },
	/* Item 12 */
	OR {{1}}, TextTitle {
		plain,
		{85,365,101,460},
		SystemFont,systemSize,
		"(and setting)" },
	/* Item 13 */
	OR {{1}}, RadioButtons {{
		{109,260,125,350},
			"Visible","-V",NotSet,"Make control visible.",
		{109,360,125,460},
			"Invisible","-I",NotSet,"Make control invisible."
		}},
	/* Item 14 */
	OR {{1}}, RegularEntry {
		"RefCon:",
		{134,270,150,330},
		{134,340,150,440},
		"",keepCase,"-const",
		"User-defined constant.\nHEX long word." },
	/* Item 15 */
	OR {{1}}, RegularEntry {
		"Label:",
		{155,30,171,90},
		{155,100,171,440},
		"",keepCase,"-label",
		"Control title or label." },
	/* Item 16 */
	AND {{6,7,8,9}}, DoItButton {},
	}, 
/* Dialog 2 */
/* Include the standard resource attributes dialog */
#include "CmdoReztAttr.R"

/* Dialog 3 */
/* Include a redirection dialog */
#include "CmdoReztRdir.R"

/* Dialog 4 */
/* Control types and values */
	275, 
	"Define type of control and its range of settings.",
	{
	/* Item 1 */
	/* A dummy for dependency logic */
	NotDependent {}, CheckOption {
		NotSet, {20,1000,36,1000},
		"Title","","Not used" },
	/* Item 2 */
	OR {{1}}, NestedDialog {
		4,
		{20,20,36,80},
		"Button","" },
	/* Item 3 */
	OR {{1}}, CheckOption {
		Set,
		{40,64,56,80},
		"","","" },
	/* Item 4 */
	OR {{1}}, RadioButtons {{
		{60,64,76,80},
		"","",Set,"" }},
	/* Item 5 */
	NotDependent {}, PictOrIcon {
		Picture, 200, 
		{80,20,96,80}  },
	/* Item 6 */ 
	NotDependent {}, RadioButtons {{
		{20,85,36,260},
			"Button","-P",NotSet,
			"Control type is a push button", 
		{40,85,56,260},
			"Check Box","-C",NotSet,
			"Control type is a check box.",
		{60,85,76,260},
			"Radio Button","-R",NotSet,
			"Control type is a radio button.",
		{80,85,96,260},
			"Scroll Bar","-S",NotSet,
			"Control type is a scroll bar."
		}},
	/* Item 7 */
	OR {{(1<<12)+6,(2<<12)+6,(3<<12)+6}}, RadioButtons {{
		{40,300,56,400},
			"System","",NotSet,
			"Draw label in System font.",
		{60,300,76,400},
			"Windows", "-W",NotSet,
			"Draw label in windows font."
		}},
	/* Item 8 */
	NotDependent {}, TextBox {
		gray,
		{30,290,80,410},
		"Label Font" },
	/* Item 9 */
	OR {{(1<<12)+6,(2<<12)+6,(3<<12)+6}}, RadioButtons {{
		{110,335,126,380}, "Off","0",NotSet,
			"Control is initially off (unset).",
		{110,415,126,455},"On","1",NotSet,
			"Control is initially on (set)."
		}},
	/* Item 10 */
	OR {{-9}}, RegularEntry {
		"Setting",
		{139,335,155,395},
		{139,395,155,452},
		"",keepCase,"",
		"Initial value or setting of control." },
	/* Item 11 */
	OR {{-9}}, RegularEntry {
		"Minimum",
		{139,25,155,88},
		{139,93,155,153},
		"",keepCase,"",
		"Minimum of control range." },
	/* Item 12 */
	OR {{-9}}, RegularEntry {
		"Maximum",
		{139,165,155,235},
		{139,240,155,300},
		"",keepCase,"",
		"Maximum of control range." },
	/* Item 13 */
	OR {{9}}, RadioButtons {{
		{20,1000,36,1100},
		"Minimum","0",NotSet,"Not Seen" }},
	/* Item 14 */
	OR {{9}}, RadioButtons {{
		{20,1000,36,1100},
		"Maximum","1",NotSet, "Not Seen" }},
	/* Item 15 */
	NotDependent {}, TextBox {
		gray,
		{100,330,166,460},
		"Setting" },
	/* Item 16 */
	NotDependent {}, TextBox {
		gray, 
		{130, 20,166,460},
		"Range" },
	},
}
}; 

/* Picture of Scroll Bar */
resource 'PICT' (200) {
	251,
	{49, 122, 64, 183},
	$"1101 A000 82A0 008C 0100 0A00 0000 0002"
	$"D002 4009 0000 0000 0000 0000 3100 3100"
	$"7A00 4000 B709 FFFF FFFF FFFF FFFF 3809"
	$"8822 8822 8822 8822 3100 3100 8500 4000"
	$"AC09 FFFF FFFF FFFF FFFF 3809 0000 0000"
	$"0000 0000 3100 3200 9000 3F00 9E09 FFFF"
	$"FFFF FFFF FFFF 38A0 008E 9000 0200 3300"
	$"7800 3F00 8800 3300 7A00 3F00 8600 3300"
	$"7A00 3F00 8600 0100 0001 8002 8004 FC08"
	$"0410 0410 0408 0404 FC02 8001 8000 00A0"
	$"008F A000 8E01 000A 0033 00A9 02D0 0240"
	$"9000 0200 0000 0000 0D00 1000 0000 0000"
	$"0D00 0E00 3300 A900 4000 B700 0100 0001"
	$"8001 403F 2020 1020 0820 0820 103F 2001"
	$"4001 8000 0000 00A0 008F A000 8DA0 0083"
	$"FF"
};

	
