/********************************************************/
/* HyperUtils.c	 										*/
/* 							 							*/
/* A collection of useful 	 							*/
/* routines...				 							*/
/* 							 							*/
/* ----------------------------------------------------	*/
/* 10/23/89 Added Routines to Support HyperAppleTalk	*/
/********************************************************/

#include	<MacTypes.h>
#include	<OSUtil.h>
#include	<MemoryMgr.h>
#include	<FileMgr.h>
#include	<ResourceMgr.h>
#include	<StdFilePkg.h>
#include 	<HyperXCmd.h>
#include 	<HyperUtils.h>


#define		UsingHypercard

#ifdef 		UsingHypercard

				/****************************************************/
				/* 				--- HyperCard UTILITIES ---			*/
				/*													*/
				/****************************************************/

void	paramtoPString( paramPtr, i, str )
	XCmdBlockPtr	paramPtr;
	short			i;
	char			*str;
/************************
* Given an index into the 
* parameter list, convert the data
* in the handle into a pstring
*
* 10.25.89 DK if handle empty, return empty.
************************/
{
	HLock( paramPtr->params[i] );
	ZeroToPas( paramPtr, (char *)*(paramPtr->params[i]), (char *)str );
	HUnlock( paramPtr->params[i] );
}


long 	paramtoNum( paramPtr, i )
	XCmdBlockPtr	paramPtr;
	short			i;
/************************
* Given a handle to an input
* argument in the paramBlk
* return an integer representation
* of the data.
*
* 10.25.89 DK if handle empty, return empty.
************************/
{
	Str31		theStr;
	
	theStr.data[0] = '\0';
	
	if( paramPtr->params[i] ){
		HLock( paramPtr->params[ i ] );
		ZeroToPas( paramPtr, (char *)*(paramPtr->params[ i ]), (char *)&theStr );
		HUnlock( paramPtr->params[ i ] );
	}
	return( StrToLong( paramPtr, (char *)&theStr ) );
}


Handle 	NumToParam( paramPtr, num )
	XCmdBlockPtr	paramPtr;
	long			num;
/************************
* Given a long,  return a
* Handle to a string
* representation of the data.
*
************************/
{
	Str31		theStr;
	
	NumToStr( paramPtr, num, (char *)&theStr );
	
	return( PasToZero( paramPtr, (char *)&theStr ) );
}


#endif

				/****************************************************/
				/* 				--- Generic UTILITIES ---			*/
				/*													*/
				/* Routines that do generic things like allocate	*/
				/* memory in the system heap						*/
				/****************************************************/


void	*sys_alloc( len )
	long	len;
/***************************
* Return a chunk of system heap
* requested (len bytes)
*
***************************/
{	
	asm{
        move.l  len, D0
        NewPtr	SYS
        move.l  A0, D0
      }
}

Handle	NewSysHandle( len )
	long	len;
/***************************
* allocate a handle in the 
* system heap.  
*
* requested (len bytes)
*
* 10.31.89 added this routine to file
***************************/
{	
	asm{
        move.l  len, D0
        NewHandle	SYS
        move.l  A0, D0
      }
}

short	validHandle( h )
	void	*h;
/***************************
* This will be my handle checker
* in the future.  Right now, all 
* it does is check to see if the
* handle is nil.
* 
* In the future, I will use
* the information in the header
* to determine if the handle is
* a valid reference in the heap.
*
* (need to make sure this works
* across heap zones).
***************************/
{
	if( h ) 	
		return( TRUE );
	else
		return( FALSE );
}


void	TrashHandle( h )
	Handle	h;
/***************************
* if the handle passed in is not
* NIL, unlock the handle and 
* dispose its contents
*
* ADD A TEST FOR VALID HANDLE!!!
*
* 10.31.89 sets it purgeable first
***************************/
{
	if( validHandle( h ) ){
		HPurge( h );
		HUnlock( h );
		DisposHandle( h );
	}
}

Handle	GetSystemResource( typ, id )
	ResType	typ;
	short	id;
/***************************
* Load a resource from the system
* resources. Returns nil if handle
* not available.
*
* DEBUG: Return nil if the resource
* is unallocated (size == 0).
*
* 10.31.89 sets it purgeable first
***************************/
{
	short	oldRes	= CurResFile();
	Handle	h		= NIL;

	UseResFile( SYS_RES );
	h = GetResource( typ, id );
	
	if( GetHandleSize( h ) == 0 )
		h = NIL;
		
	UseResFile( oldRes );
	
	return( h );
}


void	RemoveSystemResource( h )
	Handle	h;
/***************************
* Remove the specified resource
* from the system resource file
*
* the handle is deleted from 
* memory and its reference in the
* resource map is removed.
***************************/
{
	short	oldRes = CurResFile();

	if( validHandle( h ) ){
		HPurge( h );
		HUnlock( h );

		UseResFile( SYS_RES );
		RmveResource( h );		
		UpdateResFile( SYS_RES );
		
		UseResFile( oldRes );
		TrashHandle( h );
	}
}


void	AddSystemResource( h, typ, id, name )
	Handle	h;
	ResType	typ;
	short	id;
	char	*name;
/***************************
* Given a handle to something,
* allocate the something as a 
* resource in the system file.
*
* Note that the resource gets
* locked down (this is a special
* routine that is used to allocate
* pseudo-global data.
*
* EVENTUALLY: WANT TO RETURN THE 
* ID SO THAT YOU CAN MAKE SURE YOU
* ALLOCATE A UNIQUE ID EACH TIME!!!
* THEN THE ID CAN BECOME A TASK
* REFERENCE NUMBER!!!
***************************/
{
	short	oldRes = CurResFile();
	short	attributes;
	
	if( validHandle( h ) ){
		MoveHHi( h );
		HLock( h );
		HNoPurge( h );

		UseResFile( SYS_RES );
		AddResource( h, typ, id, name );

		/*** test ***/
		/*** don't want this resource written out ***/
		attributes = GetResAttrs( h );
		SetResAttrs( h, attributes & 0xfffe );

		UseResFile( oldRes );
	}
}




void	AppendCharToHandle( theHand, theChar )
	Handle	theHand;
	char	theChar;
/****************************
*
* Given a valid handle, append
* the character passed in to 
* the end of the handle
* 
* This is a useful way to embed
* \r, \t or \0 into a container
* for use by hypercard.
****************************/
{
	long		hsiz = GetHandleSize( theHand );

 	SetHandleSize( theHand, hsiz + 1 );
	*(*theHand + hsiz) = theChar;
}


void 	appendChar( theStr, theChar )
	char	*theStr;
	char	theChar;
/************************
* append the character passed
* to the end of the string 
************************/
{
	long	len = strlen( theStr );
	char	*theEnd;

	theEnd 		= theStr + len;
	*theEnd++ 	= theChar;
	*theEnd 	= '\0';
}


char 	*CopyAscii( outStr, theChar )
	char	*outStr;
	char	theChar;
/************************
* if the character passed  
* in the input stream is a 
* printing character, append
* it to the output string,
* otherwise, append the '.'
*
* return the update output 
* string.
************************/
{
	if ( theChar >= SPACE  &&  theChar <= 0x0D8  ) 
		*outStr++ = theChar;
	else
		*outStr++ = '.';
		
	return( outStr );
}

	
void CopyStrToHandle( theStr, hand )
	char	*theStr;
	Handle	hand;
/************************
* Copy the input data to the
* output handle.
*
* The input string is a "C"
* string.
************************/
{
	long	cnt 	= strlen( theStr );
	long	oldSize	= GetHandleSize( hand );

	SetHandleSize( hand, oldSize + cnt );
	BlockMove( theStr, *hand + oldSize, cnt );
}


void CopyDataToHandle( theStr, theLen, theHand, delim )
	char	*theStr;
	long	theLen;
	Handle	theHand;
	char	delim;
/************************
* Copy the input data to the
* output handle and append
* the requested character to
* the data.
*
************************/
{
	long	oldSize	= GetHandleSize( theHand );

	SetHandleSize( theHand, oldSize + theLen );
	BlockMove( theStr, *theHand + oldSize, theLen );
	AppendCharToHandle( theHand, delim );
}


short	pstrcmp( s1, s2 )
		char	*s1;
		char	*s2;
/************************
* Compare the two pascal strings
* passed in regardless of case.
*
* returning:
*	FALSE	== don't match
* 	TRUE 	== do match
* 
* We first check to see if 
* both strings are the same
* length.  If not, no sense
* in proceeding.
************************/
{
	short 		i;
	short		test			= TRUE;
	short		len				= (short)*s1;
	register	char	*str1	= s1;
	register	char	*str2	= s2;
	
	for( i = 0; i <= len; i++ )
		if( toUpper( *str1 ) == toUpper( *str2 ) ){
			str1++;
			str2++;
		}
		else{
			test = FALSE;
			break;
		}
	
	return( test );
}





short	pStrToField( str, delim, list )
	char	*str;			
	char	delim;
	Handle	list;
/*********************************
* Given a pascal string, append it to
* the end of the handle passed in list
* which is assumed to be a valid handle
* of length >= 0.
*
* delim is some character to stick on the
* end of the string to delimit it.
* if you want to build a list for presentation
* in a field, pass '\r' as the delimiter
*
* If you are building items pass a comma
* 
* A value of 0 for delim is ignored.  Pass
* 0 when you don't want a delimiter.
*********************************/
{
	long		strlen;		/* length of the input string	*/
	long		oldHSize;	/* size of the input handle		*/
	char		*end;		/* pointer to end of data		*/
	
	if( validHandle( list ) ){
		strlen = (long)str[0];		
		oldHSize = GetHandleSize( list );
		
		SetHandleSize( list, oldHSize + strlen );
		end = *list + oldHSize;
	
		BlockMove( (char *)&str[1] , end, strlen );
		
		AppendCharToHandle( list, delim );
	}
}


void	DataToField( str, len, delim, list )
	char	*str;		
	short	len;	
	char	delim;
	Handle	list;
/*********************************
* copy <len> bytes from the input
* stream to the output handle and
* append char <delim> to the end of 
* output handle.
*
* delim is some character to stick on the
* end of the string to delimit it.
* if you want to build a list for presentation
* in a field, pass '\r' as the delimiter
*
* If you are building items pass a comma
* 
* A value of 0 for delim is ignored.  Pass
* 0 when you don't want a delimiter.
*********************************/
{
	long		oldHSize;	/* size of the input handle		*/
	char		*end;		/* pointer to end of data		*/
	
	if( validHandle( list ) ){
		oldHSize = GetHandleSize( list );
		
		SetHandleSize( list, oldHSize + len );
		end = *list + oldHSize;
	
		BlockMove( str, end, len );
		
		AppendCharToHandle( list, delim );
	}
}



void ClimbTree( child, cpb, fullName )
	long		child;
	CInfoPBPtr	cpb;
	char		*fullName;
/*************************
* Climb the directory tree
* until we reach the root
*
* Allocate the records in the
* heap to keep the stack frame
* as small as necessary.  Too
* large a stack frame can 
* lead to a case of terminal
* terminal.
*
* child is the working directory
* id of the "current folder", vol
* is the volume reference number and
* fullName points to the 
* output string
*************************/
{
	StringPtr	folder_name	= (StringPtr)NewPtr( 256 );
	char		colon[2];
	
	colon[0] = '\1';
	colon[1] = ':';
	
	/*** setting the file directory index to -1	***/
	/*** lets us get information about the		***/
	/*** directory whose id is specified in	the	***/
	/*** ioDrDIrID field of the parameter block	***/
	folder_name[0] = '\0';	
	
	cpb->dirInfo.ioNamePtr 	= (StringPtr)folder_name;				
	cpb->dirInfo.ioDrDirID 	= child; 

	if( PBGetCatInfo( cpb, 0) == noErr ){
		ClimbTree(cpb->dirInfo.ioDrParID,cpb,fullName);

		Concat( (char *)fullName, (char *)folder_name );
		Concat( (char *)fullName, (char *)&colon );
	}
	DisposPtr( folder_name 	);
	
}

void	CenterWindow( wptr )
	WindowPtr	wptr;
/***************************
* Center a window in the current
* screen port.  Note: Does not
* attempt to work with multi-screen
* systems.
*
* This code is inspired by a
* similar routine written by Steve
* Maller in MPW Pascal.  Thanks Steve.
***************************/
{
 short 	hWindSize = wptr->portRect.right - wptr->portRect.left;
 short	vWindSize = wptr->portRect.bottom - wptr->portRect.top;
 short	hSize = wptr->portBits.bounds.right - wptr->portBits.bounds.left;
 short	vSize = wptr->portBits.bounds.bottom - wptr->portBits.bounds.top;
 
 MoveWindow( wptr, 	
 			( hSize - hWindSize ) / 2, 
 			( vSize - vWindSize + 20) / 2,
			false
		);
}

void Concat( str1, str2 )
	char	*str1;
	char	*str2;
/*****************************
* Append string 2 to the end of
* string 1.  Both strings are 
* pascal-format strings.
*
* str1 must be large enough to hold
* the new string and is assumed to 
* be of Type Str255 (a pascal string)
*****************************/
{
	short len1	= *str1;		/*** the number of chars in string 1	***/
	short len2	= *str2++;		/*** the number of chars in string 2	***/
	char  *temp;				/*** string pointer 					***/
	
	if( len1 +len2  > 255 )
		len2 = 255 - len1;
		
	*str1++ += len2 ;			/*** add sizes together to get new size	***/
	
	temp = str1 + len1;			/*** move to the end of string 1		***/
	while( len2 ){
		*temp++ = *str2++;		/*** add a char to temp and move along	***/
		--len2;					/*** until all characters are added		***/
	}

}

void	CopyPStr( pStr1, pStr2 )
	char	*pStr1;
	char	*pStr2;
/****************************
* Copy the contents of pstr1 into
* pstr2.  The strings are assumed 
* to be of type STR255 (length byte
* precedes data 
*
****************************/
{	short 	i;
	char	*tstr;
	
	tstr = pStr2;
	
	for( i = 0; i <= *pStr1; i++ )
		*tstr++ = *pStr1++;
}


short GetFileNameToOpen( typs, typCnt,theName, theWDID )
	SFTypeList	typs;
	short		typCnt;
	char		*theName;
	short		*theWDID;
/*****************************
* Invokes SFOpenFile to query the 
* user for the name of a file to 
* open. 
*
* In: 	List of types of files to
*		filter for (up to 4)
*
* Out:	fileName if picked in theName
*		working directory in theWDID
*		nil otherwise
*		the file's volum ref num.
*
* ( Note that the space for the 
* string must be allocated by the
* caller).
*****************************/
{
	Point		where;
	char		prompt[1];
	SFReply		reply;
	GrafPort	*oldPort;
	WindowPtr	dlogID;
	
	prompt[0] 		= '\0';
	
	/*** Get and put up the standard file	***/
	/*** dialog.  You will only see the	file***/
	/*** types that you filtered for.  If 	***/
	/*** you filtered for no files, then 	***/
	/*** all files will display				***/
	
	GetPort( &oldPort );
	dlogID = GetNewDialog( (short)getDlgID, (Ptr)NIL, (Ptr)UPFRONT );
	
	SetPort( dlogID );
	CenterWindow( dlogID );
	where.h = dlogID->portRect.left;
	where.v = dlogID->portRect.top;
	LocalToGlobal( &where );
	
	SFGetFile( where, prompt, (Ptr)NIL, typCnt, typs, (Ptr)NIL, &reply );
	
	DisposDialog( dlogID );
	SetPort( oldPort );
	
	/*** If the user selected a file, let's ***/
	/*** get the information about it		***/
	
	if (reply.good){
		*theWDID = reply.vRefNum;
		PtoCstr( (char *)&reply.fName );
		strcpy( theName, &reply.fName  );
	}
	return( reply.good );
}




OSErr	CopyFork( inref, outref, siz )
	short	inref;
	short	outref;
	long	siz;
/*****************************
* Given that the caller has opened
* a fork and passed you the names of 
* the input, copy the number of bytes
* from the input fork to the output 
* fork.
*
* The input mark should be set to 
* start of fork.
*
* We use a "semi-smart" algorithm
* to do the copy.  If the entire
* fork can be copied, we try doing 
* that, otherwise, we keep dividing
* the size by two until we get enough
* room to read some data in.
*
******************************/
{
	OSErr	rd_err		= noErr;
	OSErr	wrt_err		= noErr;
	long	rd_len;					/*** actual bytes read in	***/
	Ptr		inbuf;
	
	/*** make sure that the size is even 		***/
	if( siz % 2 )
		++siz;
		
	if( siz > 0 ){
		do{
			inbuf = NewPtr( siz );
			if( !inbuf )
				siz = ( siz >> 1 );
				
		}while( !inbuf );
	
		/*** inbuf is the buffer that we read into	***/
		/*** it 									***/
		if( inbuf ){
			do{
				rd_len = siz;
				rd_err = FSRead( inref, &rd_len, inbuf );
				wrt_err= FSWrite( outref, &rd_len, inbuf );
			}while( !rd_err && !wrt_err );
			
			DisposPtr( inbuf );
		}
	}
	return( wrt_err );
}


OSErr	CopyFile( inFile, inWD, outFile, outWD )
	char	*inFile;
	short	inWD;
	char	*outFile;
	short	outWD;
/*****************************
* (1) Determine the size of the input 
* file. 
*
* (2) Attempt to allocate that 
* much space for the output file.
*
* (3) If allocation successful,
* create the output file.
*
* (4) Once the file is created,
* copy the data fork, the resource
* fork and the finder information
* from the input file.
* 
* The file will be called "copy of..."
* Each time we create the file, first
* see if that name exists, if so, keep
* sticking "copy of" onto the name.
******************************/
{
	OSErr		err;
	OSErr		err2;
	short		inref;
	short		outref;
	long		data_eof	= 0L;
	long		rsrc_eof	= 0L;
	FInfo		fndrinfo;
	short		vref;	/** volume the file is on	***/
	
	
	/*** (2) Determine how big the input file is 	***/
	if( (err = FSOpen( inFile, inWD, &inref )) == noErr){
		err = GetEOF( inref, &data_eof );
		err = FSClose( inref );
	}		
	
	if( (err = OpenRF( inFile, inWD, &inref )) == noErr ){
		err = GetEOF( inref, &rsrc_eof );
		err = FSClose( inref );	
	}
	
	
	/*** (2) Create  output file and allocate space	***/
	if( ( err = GetFInfo( inFile, inWD, &fndrinfo ) ) != noErr )
		return( err );
	
	if( ( err = Create( outFile, inWD, fndrinfo.fdCreator, fndrinfo.fdType )) != noErr )
		return( err );
			
	/*** (3) Try to allocate enough space for both	***/
	/*** forks. Note that if we get enough space. 	***/
		
	if( (err = FSOpen( outFile, outWD, &outref )) != noErr)
		return( err );
	
	if( (err = SetEOF( outref, data_eof  )) != noErr ){
		err2 = FSClose( outref );
		err2 = FSDelete( outFile, outWD );
		return( err );
	}

	err2 	= FSClose( outref );
	err 	= OpenRF( outFile, outWD, &outref );
	
	if( (err = SetEOF( outref, rsrc_eof  )) != noErr ){
		err2 = FSClose( outref );
		err2 = FSDelete( outFile, outWD );
		return( err );
	}
	err2 = FSClose( outref );

	
	/*** (4) Copy the Data fork						***/
	err 	= FSOpen( inFile, inWD, &inref );
	
	if( !err ){
		err2 	= SetFPos( inref, fsFromStart, 0L );
		err		= CopyFork( inref, outref, data_eof );
		err2 	= FSClose( inref );
		err2 	= FSClose( outref );	
	}
	
	if( err ){
		err2 = FSDelete( outFile, outWD );
		return( err );
	}
		
	/*** (5) Now copy the resource fork				***/
	err 	= OpenRF( inFile, inWD, &inref );
	
	if( !err ){
		err2 	= SetFPos( inref, fsFromStart, 0L );
		err 	= OpenRF( outFile, outWD, &outref );
		err		= CopyFork( inref, outref, rsrc_eof );
		err2 	= FSClose( inref );
		err2 	= FSClose( outref );	
	}
	
	if( err ){
		err2 = FSDelete( outFile, outWD );
		return( err );
	}
		
	return( noErr );
}

/*** 10/23/89 ***/

#ifdef 		UsingHypercard

void SaveHandle(paramPtr, containerName, theData)
	XCmdBlockPtr	paramPtr;
	Handle			containerName;
	long 			theData;
/**********************************
* Saves a long integer (handle) away 
* in the Hypercard global container 
* whose name is passed as a parameter. 
* containerName is a Handle containing
* a null terminated string.
*
* IN: 
* containerName	=the name of
*				a Hypercard container
* theData		= the handle to store
*				in this container.
**********************************/
{
	Handle 	theDataHand;
	char	pasStr[256];
	char 	buffHandle[256];

	buffHandle[0] = '\0';
	
	/*** convert the data into a string since	***/
	/*** that is the friendliest form for HC	***/
	LongToStr( paramPtr, theData, (char *)&buffHandle );
	theDataHand = PasToZero( paramPtr, (char *)buffHandle );
	
	/*** convert the name to a string			***/
	HLock( containerName );
	ZeroToPas( paramPtr, *containerName, (char *)&pasStr);

	SetGlobal( paramPtr, pasStr, theDataHand );
	HUnlock( containerName );
}


void PutEmptyIntoGlobal(paramPtr, containerName)
	XCmdBlockPtr	paramPtr;
	Handle			containerName;
/**********************************
* Sets the given global to the empty
* container.
*
* IN: 
* containerName	=the name of
*				a Hypercard container
**********************************/
{
	Handle 	theDataHand;
	char	pasStr[256];
	char 	buffHandle[256];

	buffHandle[0] = '\0';
	
	/*** convert the data into a string since	***/
	/*** that is the friendliest form for HC	***/
	theDataHand = PasToZero( paramPtr, (char *)buffHandle );
	
	/*** convert the name to a string			***/
	HLock( containerName );
	ZeroToPas( paramPtr, (char *)*containerName, (char *)&pasStr);

	SetGlobal( paramPtr, pasStr, theDataHand );
	HUnlock( containerName );
}




long RetrieveHandle(paramPtr, containerName )
	XCmdBlockPtr	paramPtr;
	Handle			containerName;
/**********************************
* Retrieve a handle from the Hypercard
* global pool.  The name of the global 
* container is passed as a parameter
*
* If no data could be extracted from the
* container, we return empty.
**********************************/
{
	Handle		theData;		/*** handle to the data as string	***/
	char		localString[256];
	long		returnHandle = 0L;
	
	if( containerName ){
		HLock( containerName );						
		ZeroToPas( paramPtr, (char *)*containerName, (char *)&localString );
		theData = GetGlobal( paramPtr, (char *)&localString ); 
		HUnlock( containerName );	
			
		if ( theData ){
			MoveHHi( theData );
			HLock( theData );
			ZeroToPas( paramPtr, *theData, (char *)&localString );
			returnHandle = StrToLong( paramPtr, (char *)&localString );
			TrashHandle( theData );
			return( (long)returnHandle );
		}
	}
		
	return( 0L );
}


Handle ErrorReturn( theErr )
	short theErr;
/**********************************
* Given a system error number in theErr
* extract its string from the resource
* fork and returns it to HyperCard
*
* If we can't work out the error string,
* build one using the error number.
**********************************/
{
	Handle	strhand = NIL;
	char	s[256];
	char	num[256];
	long	len = 0;
	
	if(theErr == noErr)
		return( NIL );

	*s = '\0';
	
	strhand = GetResource( 'STR ', theErr );
	if( strhand ){
		/*** copy the string to another handle and save it off ***/
		len = GetHandleSize( strhand );
		BlockMove( *strhand, s, len );
		PtoCstr( (char *)&s );
	}
	else{
		s[0]='E'; s[1]='R'; s[2]='R'; s[3]='O'; s[4]='R'; s[5]=' '; s[6]='\0';
		NumToString( (long)theErr, num );
		sappend( s, num );
		len = (long)slength( s );
	}
	
	if( *s ){
		strhand = NewHandle( len );
		BlockMove( s, *strhand, len );
	}
	return( strhand );
}



/*******************	 	Miscellaneous Support Routines		*******************/

void pStrCopy( inStr,outStr )
	char *inStr, *outStr;
/*********************************
* copy the pascal-style string from
* the input string to the output 
* string.
*********************************/
{
	short i;
	
	for (i = 0; i <= inStr[0]; i++)
		outStr[i] = inStr[i];
}


short slength( str )
	char	*str;
/*********************************
* return the length of a c string
* including the '\0' byte. 
*********************************/
{
	short i = 0;
	
	while( *str++ )
		i++;
		
	return( (i+1) );
}


short strcopy( dst, src )
	char	*dst, *src;
/*********************************
* Copy a "C" string to another
* "C" string. 
*********************************/
{
	while( *dst++ = *src++ );
}


short sappend( dst, src )
	char	 *dst, *src;
/*********************************
* NOTE: the destination
* string must be large enough to hold
* both strings going in!
*
* tacks source onto destination.
*********************************/
{
	while( *dst )
		dst++;
	while( *dst++ = *src++ );
}


short strCMP( s1, s2 )
	char	*s1, *s2;
/*********************************
* compare two c strings regardless
* of case returning:
*
* <0 if s1 < s2
* 0  if s1 == s2
* >0 if s1 > s2
*********************************/
{
	
	for( ; toUpper( *s1 ) == toUpper( *s2 ); s1++, s2++ )
		if( *s1 == '\0' )
			return( 0 );
			
	return( *s1 - *s2 );
}

#endif

