/****
 * CHeapInfo.h
 *
 *
 ****/

#define _H_CHeapInfo			/* include this file only once */

#include <CObject.h>
#include <CBureaucrat.h>
#include <CList.h>

#define MAXBLOCKS		300
#define MAXMASTERS		10

#define FREE			0
#define NONRELOCATABLE	1
#define RELOCATABLE		2

#define cmdRefresh			2000
#define cmdMaxMem			2001
#define cmdCompactMem		2002
#define cmdPurgeMem			2003
#define cmdMoreMasters		2004
#define cmdMaxBlock			2005
#define cmdPurgeSpace		2006

#define cmdNewHandle		3000
#define cmdNewPointer		3001
#define cmdDisposHandle		3002
#define cmdDisposPtr		3003

#define cmdToggleState		4000
#define cmdMoveHHi			4001
#define cmdTogglePurge		4002

#define cmdSystemHeap		6000
#define cmdAppHeap1			6001

enum {BYTE = 5000, PSTRING, CSTRING, WINDOW, CONTROL, REGION, MENU, MENULIST,
		RESMAP, MPTRBLOCK, CODE, STR, GRAFPORT, USERH, USERP};

#define GONE				5500

typedef struct blkHdr
{
 	long	physSize;
 	long	relHdl;
}
blkHdr, *blkHdrPtr;

typedef struct mPtrBlock
{
	long	startAddress;
	long	endAddress;
}
mPtrBlock;

typedef struct heapEntry
{
	long		physSize;		/* physical size of the block		*/
	int			type;			/* 0 = free, 1 = nonrel, 2 = rel	*/
	Ptr			address;		/* block's address					*/
	Handle		mPtrAddress;	/* block's master pointer address	*/
	int			unused;			/* unused bytes						*/
	int			dataType;		/* display format					*/
	int			extra;			/* 									*/
}
heapEntry;

typedef struct heapHdr
{
	THz		zonePtr;			/* pointer to Application's heap zone	*/
	long	totalBlocks;		/* total blocks in the heap				*/
	long	lastUsed;			/* zone trailer block : zone->bkLim		*/
	long	totalBUsed;			/* bytes in heap						*/
	long	totalBFree;			/* free bytes : zone->zcbFree			*/
	long	totalFree;			/* free objects							*/
	long	totalRel;			/* relocatable objects					*/
	long	totalNonRel;		/* nonrelocatable objects				*/
	long	totalKnown;			/* identified blocks					*/
	long	totalFreeBytes;		/* bytes in free objects				*/
	long	totalRelBytes;		/* bytes in relocatable objects			*/
	long	totalNonRelBytes;	/* bytes in nonrelocatable objects		*/
	long	lineSelected;		/* display line that has been selected	*/
	SignedByte	selState;		/* status flag byte for master pointer	*/
}
heapHdr;

struct CHeapInfo : CBureaucrat
{
	heapEntry	heapList[MAXBLOCKS];
	heapHdr		heapHeader;
	mPtrBlock	mList[MAXMASTERS];
	int			totalMasters;
	int			currentZone;
	CList		*itsHandles;
	CList		*itsPointers;
	
	void		IHeapInfo (CBureaucrat *aSupervisor, int whichHeap);
	void		Dispose (void);
	
	void		SetHeapZone (int zone);
	int			GetHeapZone (void);
	
	heapHdr		GetHeader (void);
	heapEntry	GetBlock (long entryNo);
	Boolean		FindHandle (Handle address, long *lineNo);
	Boolean		FindPointer (Ptr address, long *lineNo);
	Boolean		FindMPtrBlock (long address, long *lineNo);
	Boolean		HandleInHeap (Handle address);
	
	long		AddMPtrBlock (long address);
	long		GetTotalMasters (void);
	
	long		GetLineSelected (void);
	void		SetLineSelected (long lineNo);
	
	void		WriteDataType (long lineNo);
	
	void		Refresh (Boolean tellSupervisor);
	void		DoCommand (long theCommand);
	void		UpdateMenus (void);
};
	