#include "CDataPane.h"
#include "CHeapInfo.h"
#include <DPMUtilities.h>

extern CHeapInfo	*gHeap;

int					lineHeight;
int					lineCount;
FontInfo			info;

/****************************************/
/*		IDataPane						*/
/****************************************/

void CDataPane::IDataPane (CView *anEnclosure, CBureaucrat *aSupervisor,
							short aWidth, short aHeight,
							short aHEncl, short aVEncl,
							SizingOption aHSizing, SizingOption aVSizing)
{
	CPanorama::IPanorama (anEnclosure, aSupervisor, aWidth, aHeight,
							aHEncl, aVEncl, aHSizing, aVSizing);
							
}											/* IDataPane */

/****************************************/
/*		SetPaneSize						*/
/****************************************/

void CDataPane::SetPaneSize ()
{
	Rect			bounds;
	long			currentLine;
	heapEntry		block;
	long			nLines;
	Point			p;
	
	this->GetBounds (&bounds);
	
	currentLine = gHeap->GetLineSelected ();
	if (currentLine == 0)
	{
		bounds.bottom = 1;
		this->SetBounds (&bounds);
		return;
	}
	
	block = gHeap->GetBlock (currentLine);
	switch (block.dataType)
	{
		case BYTE:
		case REGION:
		case CODE:
			nLines = (block.physSize - 9) / 16 + 1;
			break;
			
		case WINDOW:
			nLines = 20;
			break;
			
		case GRAFPORT:
			nLines = 25;
			break;
			
		case MENULIST:
			nLines = 15;
			break;
			
		case MENU:
			nLines = 30;
			break;
			
		case RESMAP:
			nLines = 1000;
			break;
			
		case STR:
			nLines = 30;
			break;
			
		case MPTRBLOCK:
			nLines = 64;		/* could change */
			break;
		
		default:
			nLines = 1;
			break;
	}
			
	if (bounds.bottom != nLines)
	{
		bounds.bottom = nLines;
		this->SetBounds (&bounds);
	}
	
	p.v = 0;
	p.h = 0;
	this->ScrollTo (p, FALSE);
}											/* SetPaneSize */

/****************************************/
/*		Draw							*/
/****************************************/

void CDataPane::Draw (Rect *area)
{
	GrafPtr			currentPort;
	int				saveFont;
	heapHdr			heapHeader;
	heapEntry		block;
	long			currentHeapLine;
	SignedByte		currentState;
	Rect			bounds;
	
	currentHeapLine = gHeap->GetLineSelected ();
	
	if (currentHeapLine == 0)
		return;
		
	heapHeader = gHeap->GetHeader ();
	block = gHeap->GetBlock (currentHeapLine);
	
	if (block.dataType == GONE)		/* sometimes leaves the pane half drawn */
		return;
	
	GetPort (&currentPort);
	saveFont = currentPort->txFont;
	
	TextFont (monaco);
	TextSize (9);
	
	GetFontInfo (&info);
	lineHeight = info.ascent + info.descent + info.leading;
	lineCount = info.ascent + info.leading;
	
	if (block.type == RELOCATABLE)		/* lock it */
	{
		currentState = HGetState (block.mPtrAddress);
		HLock (block.mPtrAddress);
	}
	
	switch (block.dataType)
	{
		case WINDOW:
			WriteWindow (block);
			break;
		
		case GRAFPORT:
			WriteGrafPort (block);
			break;
			
		case MPTRBLOCK:
			WriteMPtrBlock (block, area);
			break;
			
		case CONTROL:
			WriteControl (block);
			break;
			
		case PSTRING:
			WritePString (block);
			break;
			
		case CSTRING:
			WriteCString (block);
			break;
			
		case MENU:
			WriteMenu (block);
			break;
			
		case MENULIST:
			WriteMenuList (block);
			break;
			
		case RESMAP:
			WriteResMap (block, area);
			break;
			
		case STR:
			WriteStringList (block, area);
			break;
		
		default:
			WriteByte (block, area);
			break;
	}
	TextFont (saveFont);
	
	if (block.type == RELOCATABLE)
		HSetState (block.mPtrAddress, currentState);
		
}											/* Draw */

/****************************************/
/*		WriteWindow						*/
/****************************************/

void CDataPane::WriteWindow (heapEntry block)
{
	WindowPeek		wPeek;
	char			*p;
	int				length;
	ControlHandle	cHdl;
	int				h1 = 100;
	int				h2 = 280;
	Rect			*rect1;
	
	wPeek = (WindowPeek) block.address;

	Write (h1, lineCount, "windowKind :", (long) wPeek->windowKind, 8);
/*
 * draw the window's title
*/
	Write (h2, lineCount, "title :", 0, 0);
	Move (6, 0);
	p = (char*) *(wPeek->titleHandle);
	length = *p++;
	DrawText (p, 0, length);
	
	lineCount += lineHeight;
	
	Write (h1, lineCount, "titleWidth :", (long) wPeek->titleWidth, 8);
	
	Write (h2, lineCount, "titleHandle :", 0, 0);
	WriteHandle ((Handle)wPeek->titleHandle, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "visible :", 0, 0);
	WriteBoolean (wPeek->visible, 5, 0);
	
	Write (h2, lineCount, "strucRgn :", 0, 0);
	WriteHandle ((Handle)wPeek->strucRgn, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "hilited :", 0, 0);
	WriteBoolean (wPeek->hilited, 5, 0);
	
	Write (h2, lineCount, "contRgn :", 0, 0);
	WriteHandle ((Handle)wPeek->contRgn, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "goAwayFlag :", 0, 0);
	WriteBoolean (wPeek->goAwayFlag, 5, 0);
	
	Write (h2, lineCount, "updateRgn :", 0, 0);
	WriteHandle ((Handle)wPeek->updateRgn, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "zoom box :", 0, 0);
	WriteBoolean (wPeek->spareFlag, 5, 0);
		
	Write (h2, lineCount, "dataHandle :", 0, 0);
	WriteHandle ((Handle)wPeek->dataHandle, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "nextWindow :", 0, 0);
	WriteAddress ((Ptr)wPeek->nextWindow, 5, 0);
		
	Write (h2, lineCount, "windowPic :", 0, 0);
	WriteHandle ((Handle)wPeek->windowPic, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "userState :", 0, 0);
	
	if (wPeek->spareFlag)
	{
		HLock (wPeek->dataHandle);
		rect1 = *(Rect**)(wPeek->dataHandle);
		WriteRect (rect1, 5, 0);
	}
		
	Write (h2, lineCount, "wDefProc :", 0, 0);
	WriteHandle ((Handle)((long)wPeek->windowDefProc & 0x00FFFFFF), 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "stdState :", 0, 0);
	
	if (wPeek->spareFlag)
	{
		++rect1;
		WriteRect (rect1, 5, 0);
		HUnlock (wPeek->dataHandle);
	}
		
	Write (h2, lineCount, "RefCon :", 0, 0);
	WriteHandle ((Handle)wPeek->refCon, 5, 0);
	lineCount += lineHeight;
	
	cHdl = wPeek->controlList;
	while (cHdl != 0L)
	{
		Write (h1, lineCount, "Control :", 0, 0);
		WriteHandle ((Handle) cHdl, 5, 10);
		
		p = (char*) (*cHdl)->contrlTitle;
		length = *p++;
		DrawText (p, 0, length);
		
		cHdl = (*cHdl)->nextControl;
		lineCount += lineHeight;
	}
}											/* WriteWindow */

/****************************************/
/*		WriteGrafPort					*/
/****************************************/

void CDataPane::WriteGrafPort (heapEntry block)
{
	GrafPtr			thePort;
	int				h1 = 100;
	int				h2 = 250;
	
	thePort = (GrafPtr) block.address;
	
	Write (h1, lineCount, "device :", thePort->device, 3);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "portBits :", 0, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "portRect :", 0, 0);
	WriteRect (&thePort->portRect, 6, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "visRgn :", 0, 0);
	WriteHandle ((Handle)thePort->visRgn, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "clipRgn :", 0, 0);
	WriteHandle ((Handle)thePort->clipRgn, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "bkPat :", 0, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "fillPat :", 0, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "pnLoc :", 0, 0);
	WritePoint (thePort->pnLoc, 6, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "pnSize :", 0, 0);
	WritePoint (thePort->pnSize, 6, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "pnMode :", thePort->pnMode, 5);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "pnPat :", 0, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "pnVis :", thePort->pnVis, 5);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "txFont :", thePort->txFont, 5);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "txFace :", 0, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "txMode :", thePort->txMode, 5);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "txSize :", thePort->txSize, 5);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "spExtra :", 0, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "fgColor :", thePort->fgColor, 5);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "bkColor :", thePort->bkColor, 5);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "colrBit :", thePort->colrBit, 5);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "patStretch :", thePort->patStretch, 5);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "picSave :", 0, 0);
	WriteHandle ((Handle)thePort->picSave, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "rgnSave :", 0, 0);
	WriteHandle ((Handle)thePort->rgnSave, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "polySave :", 0, 0);
	WriteHandle ((Handle)thePort->polySave, 5, 0);
	lineCount += lineHeight;
	
	Write (h1, lineCount, "grafProcs :", 0, 0);
	lineCount += lineHeight;
}											/* WriteGrafPort */

/****************************************/
/*		WriteControl					*/
/****************************************/

void CDataPane::WriteControl (heapEntry block)
{
	ControlHandle	cHdl;
	int				h = 100;
	char			*p;
	int				length;
	WindowPeek		wPeek;
	
	cHdl = (ControlHandle) block.mPtrAddress;
	Write (h, lineHeight * 1, "Owner :", (long) (*cHdl)->contrlOwner, 8);
/*
 * draw the owning window's title
*/
	Move (10, 0);
	wPeek = (WindowPeek) (*cHdl)->contrlOwner;
	p = (char*) *(wPeek->titleHandle);
	length = *p++;
	DrawText (p, 0, length);
	DrawText (" window", 0, 7);
	
	Write (h, lineHeight * 2, "Min value :", (*cHdl)->contrlMin, 8);
	Write (h, lineHeight * 3, "Max value :", (*cHdl)->contrlMax, 8);
	Write (h, lineHeight * 4, "Current value :", (*cHdl)->contrlValue, 8);
	Write (h, lineHeight * 5, "Title :", 0, 0);
/*
 * draw the control's title
*/
	Move (5, 0);
	p = (char*) (*cHdl)->contrlTitle;
	length = *p++;
	DrawText (p, 0, length);
}											/* WriteControl */

/****************************************/
/*		WriteResMap						*/
/****************************************/

#define		TAB1		100
#define		TAB2		300
#define		TAB3		20

void CDataPane::WriteResMap (heapEntry block, Rect *area)
{
	int				length;
	char			*p;
	int				i, j;
	Rect			drawRect;
	Rect			tempRect;
	ResourceMap			*resMap;
	ResourceTypeList	*resList;
	ResourceRefList		*refList;

	resMap = (ResourceMap*)StripAddress (*block.mPtrAddress);
/*
 * display resource map header
*/
	SetRect (&drawRect, 0, 0, width, lineHeight * 3);
	if (SectRect (&drawRect, area, &tempRect))
	{
		Write (TAB1, lineCount, "next resMap :", 0, 0);
		WriteHandle ((Handle)resMap->nextResMap, 5, 0);
		lineCount += lineHeight;
		
		Write (TAB1, lineCount, "file Ref :", resMap->fileRefNum, 8);
		lineCount += lineHeight;
		
		Write (TAB1, lineCount, "attributes :", resMap->attrs, 8);
		
		lineCount = info.ascent + info.leading;
		Write (TAB2, lineCount, "type offset :", resMap->typeListOffset, 8);
		lineCount += lineHeight;
		
		Write (TAB2, lineCount, "res offset :", resMap->nameListOffset, 8);
		lineCount += lineHeight;
		
		resList = (ResourceTypeList*)((char*) resMap + resMap->typeListOffset);
		Write (TAB2, lineCount, "resTypes :", resList->totalTypes + 1, 8);
		lineCount += lineHeight * 2;
	}
	else
	{
		resList = (ResourceTypeList*)((char*) resMap + resMap->typeListOffset);
		lineCount += lineHeight * 4;
	}
/*
 * display type list
*/
	
	SetRect (&drawRect, 0, lineCount - lineHeight, width, lineCount);
		
	for (i = 0; i <= resList->totalTypes; i++)
	{
		MoveTo (TAB3, lineCount);				/* resource type */
		DrawText (&resList->typeDef[i].resourceType, 0, 4);
		
		refList = (ResourceRefList*)((char*)resMap + resMap->typeListOffset
							+ resList->typeDef[i].refListOffset);
		
		for (j = 0; j <= resList->typeDef[i].totalResources; j++)
		{
			if (SectRect (&drawRect, area, &tempRect))
			{
				Write (TAB3 + 40, lineCount, ":", refList->resourceID, 8);
				
				if (refList->nameOffset != -1)
				{
					Move (120, 0);
					p = (char*)resMap + resMap->nameListOffset +
								refList->nameOffset;
					length = *p++;
					DrawText (p, 0, length);				/* resource name */
				}
				
				if (refList->dataHandle != 0)
				{
					MoveTo (TAB3 + 100, lineCount);
					WriteHandle ((Handle)refList->dataHandle, 0, 0);
				}
			}
				
			lineCount += lineHeight;
			OffsetRect (&drawRect, 0, lineHeight);
			++refList;
		}
		if (drawRect.top > area->bottom)
			break;
	}
}											/* WriteResMap */

/****************************************/
/*		WriteMenu						*/
/****************************************/

void CDataPane::WriteMenu (heapEntry block)
{
	int				h1 = 100;
	int				h2 = 200;
	char			*p;
	int				length;
	MenuInfo		*menu;
	int				i;
	
	menu = (MenuInfo *) StripAddress (*block.mPtrAddress);
	
	Write (h1, lineCount, "Menu title :", 0, 0);
	
	p = (char*) menu->menuData;
	Move (5, 0);
	length = *p++;
	DrawText (p, 0, length);
	p += length;
	lineCount += lineHeight;
	
	Write (h1, lineCount, "menuID :", menu->menuID, 4);
	lineCount += lineHeight;
	Write (h1, lineCount, "menuWidth :", menu->menuWidth, 4);
	lineCount += lineHeight;
	Write (h1, lineCount, "menuHeight :", menu->menuHeight, 4);
	lineCount += lineHeight;
	Write (h1, lineCount, "menuProc :", (long) menu->menuProc, 8);
	lineCount += lineHeight;
	Write (h1, lineCount, "enableFlags :", menu->enableFlags, 8);
	lineCount += lineHeight;
	
	lineCount = info.ascent + info.leading;
	while (*p != 0)
	{
		MoveTo (h2, lineCount);
		length = *p++;
		DrawText (p, 0, length);
		p += length;
		
		for (i = 0; i < 4; i++)
		{
			MoveTo (h2 + 100 + i * 10, lineCount);
			DrawChar (*p++);
		}
		lineCount += lineHeight;
	}
}

/****************************************/
/*		WriteMenuList					*/
/****************************************/

void CDataPane::WriteMenuList (heapEntry block)
{
	int				h = 100;
	char			*p;
	int				length;
	int				*intP;
	long			*longP;
	int				totMenus;
	int				i;
	MenuInfo		*menu;
	
	intP = (int *) StripAddress (*block.mPtrAddress);
	totMenus = *intP / 6;
	
	Write (h, lineCount, "lastMenu :", (long) *intP, 8);
	lineCount += lineHeight;
	
	++intP;
	Write (h, lineCount, "lastRight :", (long) *intP, 8);
	lineCount += lineHeight;
	
	++intP;
	Write (h, lineCount, "mbResID :", (long) *intP, 8);
	
	lineCount = info.ascent + info.leading;
	
	++intP;
	for (i = 1; i <= totMenus; i++)
	{
		longP = (long*) intP;
		
		MoveTo (180, lineCount);
		WriteHandle ((Handle)*longP, 0, 20);
					
		menu = (MenuInfo *) StripAddress (*(long*)*longP);
	
		p = (char*) menu->menuData;
		length = *p++;
		DrawText (p, 0, length);
		
		lineCount += lineHeight;
		intP += 3;
	}
}											/* WriteMenuList */

/****************************************/
/*		WritePString					*/
/****************************************/

void CDataPane::WritePString (heapEntry block)
{
	char			*p;
	int				length;
	FontInfo		info;
	
	Write (100, lineCount, "Pascal string :", 0, 0);
	Move (5, 0);
	
	if (block.type == RELOCATABLE)
		p = (char*) StripAddress (*block.mPtrAddress);
	else
		p = (char*) block.address;
	
	length = *p++;
	DrawText (p, 0, length);
}											/* WritePString */

/****************************************/
/*		WriteCString					*/
/****************************************/

void CDataPane::WriteCString (heapEntry block)
{
	char			*p;
	char			*q;
	int				length = 0;
	
	Write (100, lineCount, "C string :", 0, 0);
	Move (5, 0);
	
	if (block.type == RELOCATABLE)
		p = (char*) StripAddress (*block.mPtrAddress);
	else
		p = (char*) block.address;
	
	q = p;
	while (*q++ != 0)
		++length;
		
	DrawText (p, 0, length);
}											/* WriteCString */

/****************************************/
/*		WriteStringList					*/
/****************************************/

void CDataPane::WriteStringList (heapEntry block, Rect *area)
{
	char			*p;
	int				length;
	int				totalStrings;
	int				i;
	
	Write (100, lineCount, "String list :", 0, 0);
	
	p = (char*) StripAddress (*block.mPtrAddress);
	totalStrings = *(int*)p;
	p += 2;
	
	for (i = 1; i <= totalStrings; i++)
	{
		MoveTo (110, lineHeight * (i - 1) + info.ascent + info.leading);
		length = *p++;
		DrawText (p, 0, length);
		p += length;
	}
}											/* WriteStringList */

/****************************************/
/*		WriteMPtrBlock					*/
/****************************************/

void CDataPane::WriteMPtrBlock (heapEntry block, Rect *area)
{
	long			*mPtr;
	int				i;
	int				firstLine;
	int				lastLine;
	Str255			text;
	long			lineNo;
	heapEntry		block2;
	THz				theZone;
	long			endAddress = (long)block.address + block.physSize;
	
	firstLine = area->top / lineHeight;
	lastLine = area->bottom / lineHeight + 1;
	
	mPtr = (long*)block.address;
	mPtr += firstLine;
	lineCount = firstLine * lineHeight + info.ascent + info.leading;
	
	for (i = firstLine; i < lastLine; i++)
	{
		NumToString (i + 1, text);
		MoveTo (30 - StringWidth (text), lineCount);
		DrawString (text);
		
		Move (20, 0);
		WriteHandle ((Handle) mPtr, 0, 0);
		
		if (gHeap->FindHandle ((Handle) mPtr, &lineNo))
		{
			NumToString (lineNo, text);
			Move (35 - StringWidth (text), 0);
			DrawString (text);
			block2 = gHeap->GetBlock (lineNo);
			
			if (block2.dataType != BYTE)
			{
				Move (15, 0);
				gHeap->WriteDataType (lineNo);
			}
		}
		else
		{
			Move (15,0);
			if (*mPtr < (long)block.address || *mPtr > endAddress)
				if (gHeap->FindMPtrBlock ((long)*mPtr, &lineNo))
				{
					DrawString ("\plink to line ");
					NumToString (lineNo, text);
					DrawString (text);
				}
				else
					DrawString ("\pdangling");
		}
		
		++mPtr;
		lineCount += lineHeight;
	}
}											/* WriteMPtrBlock */

/****************************************/
/*		WriteByte						*/
/****************************************/

void CDataPane::WriteByte (heapEntry block, Rect *area)
{
	int				totLines;
	char			*lineStart;
	char			*endData;
	long			firstLine;
	long			lastLine;
	register int 	i;
	long			v;
	char			display[128];
	int				totBytes;
	int				maxLines;
	Str255			text;
	
	maxLines = (height - 1) / lineHeight + 1;

	if (block.type == RELOCATABLE)
		lineStart = (char*) StripAddress (*block.mPtrAddress);
	else
		lineStart = (char*) block.address;
		
	endData = lineStart + block.physSize - 8;
	
	totLines = ((endData - lineStart) - 1) / 16 + 1;
	
	if (totLines > maxLines)
		totLines = maxLines;
/*
 * Calculate the visible lines
*/
	firstLine = area->top / lineHeight;

	lastLine = firstLine + totLines;
	lineStart += (firstLine * 16);
	totBytes = 16;

	for (i = firstLine; i < lastLine && lineStart < endData; i++)
	{
		v = i * lineHeight + info.ascent;
		if (endData - lineStart < 16)
			totBytes = endData - lineStart;
		
		MoveTo (10, v);
		FillLine (lineStart, display, totBytes);
		DrawString (display);
			
		lineStart += 16;
	}
}											/* WriteByte */
