#include <CBartender.h>
#include <CError.h>
#include <DPMUtilities.h>
#include <Constants.h>
#include "HeapConstants.h"
#include "CHeapInfo.h"
#include "CHeapListerDoc.h"
 
extern CHeapInfo	*gHeap;
extern CursHandle	gWatchCursor;
extern CBartender	*gBartender;

/****************************************/
/*		IHeapInfo						*/
/****************************************/

void CHeapInfo::IHeapInfo (CBureaucrat *aSupervisor, int whichHeap)
{
	CBureaucrat::IBureaucrat (aSupervisor);
	
	itsHandles = new (CList);
	itsHandles->ICluster ();
	itsPointers = new (CList);
	itsPointers->ICluster ();
	
	SetHeapZone (whichHeap);
}

/****************************************/
/*		Dispose							*/
/****************************************/

void CHeapInfo::Dispose ()
{
	long			totItems;
	Handle			p;
	Ptr				q;
	
	totItems = itsHandles->GetNumItems ();
	while (totItems > 0)
	{
		p = (Handle)itsHandles->LastItem ();
		itsHandles->Remove ((CObject*)p);
		DisposHandle (p);
		--totItems;
	}
	itsHandles->Dispose ();
	
	totItems = itsPointers->GetNumItems ();
	while (totItems > 0)
	{
		q = (Ptr)itsPointers->LastItem ();
		itsPointers->Remove ((CObject*)q);
		DisposPtr (q);
		--totItems;
	}
	itsPointers->Dispose ();
		
	inherited::Dispose();
}

/****************************************/
/*		SetHeapZone						*/
/****************************************/

void CHeapInfo::SetHeapZone (int zone)
{
	currentZone = zone;
	Refresh (TRUE);
}

/****************************************/
/*		GetHeapZone						*/
/****************************************/

int CHeapInfo::GetHeapZone ()
{
	return (currentZone);
}

/****************************************/
/*		Refresh							*/
/****************************************/

void CHeapInfo::Refresh (Boolean tellSupervisor)
{
	blkHdr			*hdrPtr;
	unsigned char	tagByte;
	long			size;
	long			blockAddr;
	long			blockHdl;
	int				unused;
	THz				theZone;
	long			totalBlocks;
	WindowPeek		wPeek;
	long			i, j, k;
	ControlHandle	cHdl;
	int				*intP;
	long			*longP;
	int				totMenus;
	long			trueHandle;
	long			*dummyPtr;
	int				max;
	long			blockAddress;
	ResourceMap		**resMap;
	ResourceTypeList	*resList;
	ResourceRefList		*refList;
	int				theDataType;
	
	SetCursor (*gWatchCursor);
	
	heapHeader.totalBlocks = 0;
	heapHeader.totalBUsed = 0;
	heapHeader.totalFree = 0;
	heapHeader.totalRel = 0;
	heapHeader.totalNonRel = 0;
	heapHeader.totalKnown = 0;
	heapHeader.totalFreeBytes = 0;
	heapHeader.totalRelBytes = 0;
	heapHeader.totalNonRelBytes = 0;
	
	totalMasters = 0;
	
	SetLineSelected (0);			/* line numbering starts from 1 */
	
	if (currentZone == 0)
		heapHeader.zonePtr = SystemZone ();		/* System heap */
	else
		heapHeader.zonePtr = GetZone ();		/* Application heap */
	
	theZone = heapHeader.zonePtr;
	
	hdrPtr = (blkHdr*) &theZone->heapData;
	heapHeader.totalBFree = theZone->zcbFree;
	heapHeader.lastUsed = (long) theZone->bkLim;
	
	while (hdrPtr < (blkHdr*) theZone->bkLim)
	{
		tagByte = hdrPtr->physSize >> 24;
		unused = tagByte & 0x0F;
		tagByte >>= 6;
		
		size = hdrPtr->physSize & 0x00FFFFFF;
		blockAddr = (long) (Ptr) hdrPtr + 8;
		theDataType = BYTE;
		
		switch (tagByte)
		{
			case RELOCATABLE:
				blockHdl = (long) (Ptr) theZone + hdrPtr->relHdl;
				++heapHeader.totalRel;
				heapHeader.totalRelBytes += size;
				if (itsHandles->Includes ((CObject*)blockHdl))
					theDataType = USERH;
				break;
			case NONRELOCATABLE:
				blockHdl = 0;
				++heapHeader.totalNonRel;
				heapHeader.totalNonRelBytes += size;
				if (itsPointers->Includes ((CObject*)blockAddr))
					theDataType = USERP;
				break;
			case FREE:
				blockHdl = 0;
				++heapHeader.totalFree;
				heapHeader.totalFreeBytes += size;
				break;
			default:
				break;
		}
		
		totalBlocks = heapHeader.totalBlocks;
		
		if (totalBlocks < MAXBLOCKS)
		{
			heapList[totalBlocks].type = tagByte;
			heapList[totalBlocks].physSize = size;
			heapList[totalBlocks].address = (Ptr) blockAddr;
			heapList[totalBlocks].mPtrAddress = (Handle) blockHdl;
			heapList[totalBlocks].unused = unused;
			heapList[totalBlocks].dataType = theDataType;
			heapList[totalBlocks].extra = 0;
			
			if (theDataType != BYTE)
				++heapHeader.totalKnown;
		}
		
		heapHeader.totalBUsed += size;
		++heapHeader.totalBlocks;
		
		hdrPtr = (blkHdr*) ((Ptr) hdrPtr + size);
	}
/*
 * examine the window list
*/
	wPeek = (WindowPeek) FrontWindow ();
	while (wPeek != 0L)
	{
		if (FindPointer ((Ptr)wPeek, &i))
		{
			heapList[i - 1].dataType = WINDOW;
			++heapHeader.totalKnown;
				
			if (FindHandle ((Handle)wPeek->titleHandle, &i))
			{
				heapList[i - 1].dataType = PSTRING;
				++heapHeader.totalKnown;
			}
				
			if (FindHandle ((Handle)wPeek->strucRgn, &i))
			{
				heapList[i - 1].dataType = REGION;
				++heapHeader.totalKnown;
			}
				
			if (FindHandle ((Handle)wPeek->contRgn, &i))
			{
				heapList[i - 1].dataType = REGION;
				++heapHeader.totalKnown;
			}
				
			if (FindHandle ((Handle)wPeek->updateRgn, &i))
			{
				heapList[i - 1].dataType = REGION;
				++heapHeader.totalKnown;
			}
			
			cHdl = wPeek->controlList;
			while (cHdl != 0L)
			{
				if (FindHandle ((Handle)cHdl, &i))
				{
					heapList[i - 1].dataType = CONTROL;
					++heapHeader.totalKnown;
				}
				cHdl = (*cHdl)->nextControl;
			}
		}
		wPeek = wPeek->nextWindow;
	}
/*
 * examine the menu list
*/
	if (FindHandle (MenuList, &i))
	{
		heapList[i - 1].dataType = MENULIST;
		++heapHeader.totalKnown;
	}
	
	intP = (int*) *MenuList;
	totMenus = *intP / 6;
	
	for (i = 1; i <= totMenus; i++)
	{
		intP += 3;
		longP = (long *) intP;
		
		if (FindHandle ((Handle) *longP, &j))
		{
			heapList[j - 1].dataType = MENU;
			++heapHeader.totalKnown;
		}
	}
/*
 * examine the resource list
*/
	resMap = (ResourceMap**)TopMapHndl;
	
	do
	{
		if (HandleInHeap ((Handle)resMap))
		{
			if (FindHandle ((Handle) resMap, &j))
			{
				heapList[j - 1].dataType = RESMAP;
				++heapHeader.totalKnown;
				resList = (ResourceTypeList*)((char*)*resMap +
							(*resMap)->typeListOffset);
				for (i = 0; i < resList->totalTypes; i++)
				{
					switch (resList->typeDef[i].resourceType)
					{
						case 'CODE':
						case 'STR#':
						case 'STR ':
							refList = (ResourceRefList*)((char*)*resMap
								+ (*resMap)->typeListOffset
								+ resList->typeDef[i].refListOffset);
			
							for (j = 0; j <= resList->typeDef[i].totalResources; j++)
							{
								if (FindHandle ((Handle) refList->dataHandle, &k))
								{
									switch (resList->typeDef[i].resourceType)
									{
										case 'CODE':
											heapList[k - 1].dataType = CODE;
											heapList[k - 1].extra = refList->resourceID;
											break;
											
										case 'STR#':
											heapList[k - 1].dataType = STR;
											break;
											
										case 'STR ':
											heapList[k - 1].dataType = PSTRING;
											break;
										
										default:
											break;
									}
									++heapHeader.totalKnown;
								}
								++refList;
							}
							break;
						
						default:
							break;
					}
				}
			}
		}
		resMap = (ResourceMap**)(*resMap)->nextResMap;
	} while (resMap != 0L);
/*
 * Check out all the handles. First, follow the linked list of
 * available master pointers, creating a list of master pointer blocks
 * with one or more free slots
*/
	longP = (long*)theZone->hFstFree;
	while (longP != 0L)
	{
		AddMPtrBlock ((long)longP);	/* register its master ptr block */
		longP = (long*)*longP;
	}
/*
 * ...then check each allocated handle in the heap.  This will give us a
 * complete list of all master pointer blocks, including those that are
 * completely used
*/
	max = (heapHeader.totalBlocks >= MAXBLOCKS ? MAXBLOCKS
								: heapHeader.totalBlocks);
	
	for (i = 0; i < max; i++)
		if (heapList[i].type == RELOCATABLE)
			AddMPtrBlock ((long)heapList[i].mPtrAddress);
/*
 * Now loop through this new list and mark each entry as a master pointer block
*/
	for (i = 0; i < totalMasters; i++)
	{
		if (FindPointer ((Ptr)mList[i].startAddress, &j))
		{
			heapList[j - 1].dataType = MPTRBLOCK;
			++heapHeader.totalKnown;
		}
	}
/*
 * tell our supervisor that we've rebuilt the heap
*/
	if (tellSupervisor)
		itsSupervisor->DoCommand (cmdHeapRefreshed);
}

/****************************************/
/*		AddMPtrBlock					*/
/****************************************/
/*
* Input  : the address of a master pointer (i.e. a handle)
*
* Output : the address of the master pointer block in which the
*          master pointer lives, or NULL if it wasn't found
*/

long CHeapInfo::AddMPtrBlock (long address)
{
	int			i;
	int			max;
/*
 *	check our list of known master pointer blocks
*/
	for (i = 0; i < totalMasters; i++)
	{
		if (address >= mList[i].startAddress &&
				address <= mList[i].endAddress)
			return (mList[i].startAddress);			/* success */
	}
/*
 *	not known - search the heap and add it to the list if we find it
*/
	max = (heapHeader.totalBlocks >= MAXBLOCKS ? MAXBLOCKS
								: heapHeader.totalBlocks);
	
	for (i = 0; i < max; i++)
	{
		if (heapList[i].type == NONRELOCATABLE)
		{
			if (address >= (long)heapList[i].address
					&& address <= (long)heapList[i].address + heapList[i].physSize)
			{
				mList[totalMasters].startAddress = (long)heapList[i].address;
				mList[totalMasters].endAddress = (long)heapList[i].address
										 + heapList[i].physSize;
				++totalMasters;
				return ((long)heapList[i].address);		/* success */
			}
		}
	}
	return (0L);		/* failure - not in our heap */
}

/****************************************/
/*		FindMPtrBlock					*/
/****************************************/
/*
* Input  : the address of a master pointer (i.e. a handle)
*
* Output : the line number of the master pointer block in which the
*          master pointer lives, or FALSE if it wasn't found
*/

Boolean CHeapInfo::FindMPtrBlock (long address, long *lineNo)
{
	int			i;
	
	*lineNo = 0;
/*
 *	check our list of known master pointer blocks
*/
	for (i = 0; i < totalMasters; i++)
	{
		if (address >= mList[i].startAddress &&
				address < mList[i].endAddress)
		{
			FindPointer ((Ptr) mList[i].startAddress, lineNo);
			return (TRUE);
		}
	}
	return (FALSE);
}

/****************************************/
/*		FindHandle						*/
/****************************************/

Boolean CHeapInfo::FindHandle (Handle address, long *lineNo)
{
	long		j;
	long		*blockAddress;
	long		max;
	
	*lineNo = 0;
	if (address == 0L)
		return (FALSE);
	
	max = (heapHeader.totalBlocks >= MAXBLOCKS ? MAXBLOCKS
								: heapHeader.totalBlocks);
	
	for (j = 0; j < max; j++)
	{
		blockAddress = (long*)StripAddress (*heapList[j].mPtrAddress);

		if (blockAddress == (long*)StripAddress (*address))
		{
			*lineNo = j + 1;		/* visible line number */
			return (TRUE);
		}
	}
	return (FALSE);
}

/****************************************/
/*		FindPointer						*/
/****************************************/

Boolean CHeapInfo::FindPointer (Ptr address, long *lineNo)
{
	long		j;
	long		hi, lo;
	
	if (address == 0L)
		return (FALSE);
	
	hi = (heapHeader.totalBlocks >= MAXBLOCKS ? MAXBLOCKS - 1
								: heapHeader.totalBlocks - 1);
	lo = 0;
	
	do
	{
		j = (hi + lo) / 2;
		if (heapList[j].address == address)
		{
			*lineNo = j + 1;		/* visible line number */
			return (TRUE);
		}
		else if (heapList[j].address > address)
			hi = j - 1;
		else
			lo = j + 1;
	} while (hi >= lo);
	
	return (FALSE);
}

/****************************************/
/*		HandleInHeap					*/
/****************************************/
/*
* Input  : a handle
*
* Output : TRUE if the handle's master pointer lives within the current
*          heap, or FALSE if it doesn't
*/

Boolean CHeapInfo::HandleInHeap (Handle address)
{
	if ((address > (Handle) heapHeader.zonePtr)
					&& (address < (Handle)heapHeader.lastUsed))
		return (TRUE);
	else
		return (FALSE);
}

/****************************************/
/*		WriteDataType					*/
/****************************************/

void CHeapInfo::WriteDataType (long lineNo)
{
	Str255			text;
	
	switch (heapList[lineNo - 1].dataType)
	{
		case BYTE:
			DrawString ("\pByte");
			break;
			
		case REGION:
			DrawString ("\pRegion");
			break;
			
		case WINDOW:
			DrawString ("\pWindow");
			break;
			
		case GRAFPORT:
			DrawString ("\pGrafPort");
			break;
			
		case MENULIST:
			DrawString ("\pMenu list");
			break;
			
		case MENU:
			DrawString ("\pMenu");
			break;
			
		case RESMAP:
			DrawString ("\pResource map");
			break;
			
		case CONTROL:
			DrawString ("\pControl record");
			break;
			
		case PSTRING:
			DrawString ("\pPascal string");
			break;
			
		case CSTRING:
			DrawString ("\pC string");
			break;
			
		case MPTRBLOCK:
			DrawString ("\pMaster ptr block");
			break;
			
		case CODE:
			DrawString ("\pCode ");
			NumToString (heapList[lineNo - 1].extra, text);
			DrawString (text);
			break;
			
		case STR:
			DrawString ("\pString List");
			break;
			
		case USERH:
			DrawString ("\pUser handle");
			break;
			
		case USERP:
			DrawString ("\pUser pointer");
			break;
			
		case GONE:
			DrawString ("\pReclaimed");
			break;
		
		default:
			DrawString ("\pUnknown");
			break;
	}
}

/****************************************/
/*		GetTotalMasters					*/
/****************************************/

long CHeapInfo::GetTotalMasters ()
{
	return (totalMasters);
}

/****************************************/
/*		GetHeader						*/
/****************************************/

heapHdr CHeapInfo::GetHeader ()
{
	return (heapHeader);
}

/****************************************/
/*		GetBlock						*/
/****************************************/

heapEntry CHeapInfo::GetBlock (long blockNo)
{
	long			lineNo;
	Size			ptrSize;
	
	if (heapList[--blockNo].dataType != GONE)
	{
		if (heapList[blockNo].type == RELOCATABLE)
		{
			if (FindMPtrBlock ((long)*heapList[blockNo].mPtrAddress, &lineNo))
			{
				heapList[blockNo].dataType = GONE;
				itsSupervisor->DoCommand (cmdRefreshHeapLine);
			}
		}
		if (heapList[blockNo].type == NONRELOCATABLE)
		{
			ptrSize = GetPtrSize (heapList[blockNo].address);
			if (ptrSize == 0)
			{
				heapList[blockNo].dataType = GONE;
				itsSupervisor->DoCommand (cmdRefreshHeapLine);
			}
		}
	}
	return (heapList[blockNo]);
}

/****************************************/
/*		GetLineSelected					*/
/****************************************/

long CHeapInfo::GetLineSelected (void)
{
	return (heapHeader.lineSelected);
}

/****************************************/
/*		SetLineSelected					*/
/****************************************/

void CHeapInfo::SetLineSelected (long selLine)
{
	heapHeader.lineSelected = selLine;
}

/****************************************/
/*		DoCommand						*/
/****************************************/

void CHeapInfo::DoCommand (long theCommand)
{
	Size			size;
	Size			grow;
	long			contig;
	Handle			p;
	Ptr				q;
	char			text1[10];
	char			text2[10];
	long			lineNo;
	long			currentLine = heapHeader.lineSelected - 1;
	unsigned char	mPtrFlags;
	
	switch (theCommand) {
	
		case cmdRefresh:
					this->Refresh (TRUE);
					break;
	
		case cmdMaxMem:
					size = MaxMem (&grow);
					this->Refresh (TRUE);
					NumToString (size, text1);
					NumToString (grow, text2);
					ParamText ("\pLargest ", text1, "\p. Can grow ", text2);
					InitCursor ();
					Alert (ALRTgeneral, NULL);
					break;
	
		case cmdCompactMem:
					size = CompactMem (99999);
					this->Refresh (TRUE);
					NumToString (size, text1);
					ParamText ("\pLargest ", text1, "\p", "\p");
					InitCursor ();
					Alert (ALRTgeneral, NULL);
					break;
	
		case cmdPurgeMem:
					PurgeMem (99999);
					this->Refresh (TRUE);
					break;
		
		case cmdMoreMasters:
					MoreMasters ();
					this->Refresh (TRUE);
					break;
		
		case cmdMaxBlock:
					size = MaxBlock ();
					NumToString (size, text1);
					ParamText ("\pLargest ", text1, "\p", "\p");
					InitCursor ();
					Alert (ALRTgeneral, NULL);
					break;
		
		case cmdPurgeSpace:
					PurgeSpace (&size, &contig);
					NumToString (size, text1);
					NumToString (contig, text2);
					ParamText ("\pTotal free space ", text1, "\p. Largest ", text2);
					InitCursor ();
					Alert (ALRTgeneral, NULL);
					break;
		
		case cmdNewHandle:
					p = NewHandleClear (HANDLESIZE);
					CheckAllocation (p);
					itsHandles->Add ((CObject*)p);
					HLock (p);
					StuffHex (*p, "\p44656E6973204D6F6C6F6E79");	/* me */
					HUnlock (p);
					this->Refresh (FALSE);
					if (FindHandle (p, &lineNo))
						heapHeader.lineSelected = lineNo;
					itsSupervisor->DoCommand (cmdHeapRefreshed);
					break;
		
		case cmdNewPointer:
					q = NewPtrClear (PTRSIZE);
					CheckAllocation (q);
					itsPointers->Add ((CObject*)q);
					StuffHex (q, "\p44656E6973204D6F6C6F6E79");
					this->Refresh (FALSE);
					if (FindPointer (q, &lineNo))
						heapHeader.lineSelected = lineNo;
					itsSupervisor->DoCommand (cmdHeapRefreshed);
					break;
		
		case cmdDisposHandle:
					p = heapList[currentLine].mPtrAddress;
					itsHandles->Remove ((CObject*)p);
					DisposHandle (p);
					heapList[currentLine].dataType = GONE;
					itsSupervisor->DoCommand (cmdRefreshHeapLine);
					itsSupervisor->DoCommand (cmdRefreshData);
					break;
		
		case cmdDisposPtr:
					q = heapList[currentLine].address;
					itsPointers->Remove ((CObject*)q);
					DisposPtr (q);
					heapList[currentLine].dataType = GONE;
					itsSupervisor->DoCommand (cmdRefreshHeapLine);
					itsSupervisor->DoCommand (cmdRefreshData);
					break;
		
		case cmdToggleState:
					p = heapList[currentLine].mPtrAddress;
					mPtrFlags = HGetState (heapList[currentLine].mPtrAddress);
					if (mPtrFlags & 0x80)		/* locked */
						HUnlock (p);
					else
						HLock (p);
					itsSupervisor->DoCommand (cmdRefreshHeapLine);
					break;
		
		case cmdMoveHHi:
					p = heapList[currentLine].mPtrAddress;
					MoveHHi (p);
					this->Refresh (TRUE);
					if (FindHandle (p, &lineNo))
						heapHeader.lineSelected = lineNo;
					break;
		
		case cmdTogglePurge:
					p = heapList[currentLine].mPtrAddress;
					mPtrFlags = HGetState (heapList[currentLine].mPtrAddress);
					if (mPtrFlags & 0x40)		/* purgeable */
						HNoPurge (p);
					else
						HPurge (p);
					itsSupervisor->DoCommand (cmdRefreshHeapLine);
					break;
	
		case BYTE:			case PSTRING:		case CSTRING:
		case WINDOW:		case CONTROL:		case REGION:
		case MENU:			case MENULIST:		case RESMAP:
		case MPTRBLOCK:		case CODE:			case STR:
		case GRAFPORT:		case USERH:			case USERP:
					heapList[heapHeader.lineSelected - 1].dataType = theCommand;
					itsSupervisor->DoCommand (cmdRefreshData);
					itsSupervisor->DoCommand (cmdRefreshHeapLine);
					break;
					
		case cmdSystemHeap:
					if (currentZone != 0)
						this->SetHeapZone (0);
					break;
					
		case cmdAppHeap1:
					if (currentZone != 1)
						this->SetHeapZone (1);
					break;
	
		default:	itsSupervisor->DoCommand (theCommand);
					break;
	}
}

/****************************************/
/*		UpdateMenus						*/
/****************************************/

void CHeapInfo::UpdateMenus ()
{
	unsigned char	mPtrFlags;
	long			currentLine = heapHeader.lineSelected - 1;
	Str255			text;
	
	inherited::UpdateMenus ();
	
	gBartender->EnableCmd (cmdRefresh);
	
	if (gHeap->GetHeapZone () > 0)
	{
		gBartender->EnableCmd (cmdMaxMem);
		gBartender->EnableCmd (cmdCompactMem);
		gBartender->EnableCmd (cmdPurgeMem);
		gBartender->EnableCmd (cmdMoreMasters);
		gBartender->EnableCmd (cmdMaxBlock);
		gBartender->EnableCmd (cmdPurgeSpace);
	
		gBartender->EnableCmd (cmdNewHandle);
		gBartender->EnableCmd (cmdNewPointer);
		
		if (currentLine >= 0)
		{
			if (heapList[currentLine].type == RELOCATABLE)
			{
				mPtrFlags = HGetState (heapList[currentLine].mPtrAddress);
				if (mPtrFlags & 0x80)		/* locked */
				{
					GetIndString (text, STRcommon, UNLOCKtext);
					gBartender->SetCmdText (cmdToggleState, text);
				}
				else
				{
					gBartender->EnableCmd (cmdMoveHHi);
					GetIndString (text, STRcommon, LOCKtext);
					gBartender->SetCmdText (cmdToggleState, text);
				}
				if (mPtrFlags & 0x40)		/* purgeable */
				{
					GetIndString (text, STRcommon, HNOPURGEtext);
					gBartender->SetCmdText (cmdTogglePurge, text);
				}
				else
				{
					GetIndString (text, STRcommon, HPURGEtext);
					gBartender->SetCmdText (cmdTogglePurge, text);
				}
				gBartender->EnableCmd (cmdTogglePurge);
				gBartender->EnableCmd (cmdToggleState);
				gBartender->EnableCmd (cmdDisposHandle);
			}
			if (heapList[currentLine].type == NONRELOCATABLE)
				gBartender->EnableCmd (cmdDisposPtr);
		}
	}
	
	if (currentLine > 0 && heapList[currentLine].dataType != GONE)
	{
		gBartender->EnableCmd (BYTE);
		gBartender->EnableCmd (PSTRING);
		gBartender->EnableCmd (CSTRING);
		gBartender->EnableCmd (WINDOW);
		gBartender->EnableCmd (CONTROL);
		gBartender->EnableCmd (REGION);
		gBartender->EnableCmd (MENU);
		gBartender->EnableCmd (MENULIST);
		gBartender->EnableCmd (RESMAP);
		gBartender->EnableCmd (MPTRBLOCK);
		gBartender->EnableCmd (CODE);
		gBartender->EnableCmd (STR);
		gBartender->EnableCmd (GRAFPORT);
		gBartender->EnableCmd (USERH);
		gBartender->EnableCmd (USERP);
		
		gBartender->CheckMarkCmd (
						heapList[heapHeader.lineSelected - 1].dataType, TRUE);
	}
	gBartender->EnableCmd (cmdSystemHeap);
	gBartender->EnableCmd (cmdAppHeap1);
	gBartender->CheckMarkCmd (currentZone + cmdSystemHeap, TRUE);
}
