#include <Global.h>
#include <Commands.h>
#include <CApplication.h>
#include <CBartender.h>
#include <CBorder.h>
#include <CDecorator.h>
#include <CDesktop.h>
#include <CError.h>
#include <CPanorama.h>
#include <CScrollPane.h>
#include <TBUtilities.h>
#include "CHeapListerDoc.h"
#include "CHeapListerPane.h"
#include "CHeaderPane.h"
#include "HeapConstants.h"


#define	WINDHeapLister		500		/* Resource ID for WIND template */

extern	CApplication *gApplication;	/* The application */
extern	CBartender	*gBartender;	/* The menu handling object */
extern	CDecorator	*gDecorator;	/* Window dressing object	*/
extern	CDesktop	*gDesktop;		/* The enclosure for all windows */
extern	CBureaucrat	*gGopher;		/* The current boss in the chain of command */
extern	OSType		gSignature;		/* The application's signature */
extern	CError		*gError;		/* The global error handler */
extern	CHeapInfo	*gHeap;			/* The global heap info object */

/****************************************/
/*		IHeapListerDoc					*/
/****************************************/

void CHeapListerDoc::IHeapListerDoc (CBureaucrat *aSupervisor, Boolean printable)

{
	CDocument::IDocument(aSupervisor, printable);
}

/****************************************/
/*		Dispose							*/
/****************************************/

void CHeapListerDoc::Dispose ()

{
	gHeap->Dispose ();
	inherited::Dispose();
}

/****************************************/
/*		DoCommand						*/
/****************************************/

void CHeapListerDoc::DoCommand (long theCommand)

{
	Size		size;
	long		currentLine;
	Point		p;
	
	switch (theCommand) {
		
		case cmdRefreshData:				/* new line selected */
					itsDataPane->SetPaneSize ();
					itsDataPane->Refresh ();
					break;
		
		case cmdRefreshHeapLine:			/* sent by the heap object */
					currentLine = gHeap->GetLineSelected ();
					itsHeapListerPane->DrawLine (currentLine);
					break;
		
		case cmdHeapRefreshed:				/* sent by the heap object */
					if (itsWindow != NULL)
					{
						itsHeaderPane->Refresh ();
						currentLine = gHeap->GetLineSelected ();
						itsHeapListerPane->SetPaneSize ();
						if (currentLine > 0)
							itsHeapListerPane->ScrollToSelection ();
						itsHeapListerPane->Refresh ();
						itsDataPane->SetPaneSize ();
						itsDataPane->Refresh ();
					}
					break;
	
		default:	inherited::DoCommand(theCommand);
					break;
	}
}

/****************************************/
/*		UpdateMenus						*/
/****************************************/

void CHeapListerDoc::UpdateMenus ()
{
	inherited::UpdateMenus ();
	
	gBartender->DisableCmd (cmdSaveAs);
}

/****************************************/
/*		NewFile							*/
/****************************************/

void CHeapListerDoc::NewFile (void)

{
	FontInfo		info;
	CHeapListerPane	*theMainPane;
	CDataPane		*theDataPane;
	CScrollPane		*theScrollPane;
	CBorder			*theBorder;
	GrafPtr			currentPort;
	int				saveTextSize;
	int				saveFont;
	Rect			wAperture;
	int				i;
	heapHdr			header;
	long			totLines;
	
	gHeap = new (CHeapInfo);
	gHeap->IHeapInfo (this, 1);
	gGopher = gHeap;
	
	header = gHeap->GetHeader ();
	totLines = header.totalBlocks;
	if (totLines > MAXBLOCKS)
		totLines = MAXBLOCKS;
/*
 * build the window
*/
	itsWindow = new (CWindow);
	itsWindow->IWindow (WINDHeapLister, FALSE, gDesktop, this);
	itsWindow->GetAperture (&wAperture);

/*
 * build the header pane
*/
	
	itsHeaderPane = new (CHeaderPane);
	itsHeaderPane->IHeaderPane (itsWindow, this, 50, 42, 0, 0,
									sizELASTIC, sizFIXEDTOP);
	itsHeaderPane->FitToEnclFrame (TRUE, FALSE);

/*
 * build the scrolling heap pane
*/
	
	theScrollPane = new (CScrollPane);
	theScrollPane->IScrollPane (itsWindow, this,
									100, LINEHEIGHT * LINESINMAINPANE + 1,
									0, 42,
									sizELASTIC, sizFIXEDTOP,
									FALSE, TRUE, FALSE);
	theScrollPane->FitToEnclFrame (TRUE, FALSE);

	theMainPane = new (CHeapListerPane);
	itsMainPane = theMainPane;
	itsHeapListerPane = theMainPane;
	itsGopher = gHeap;

	theMainPane->IHeapListerPane (theScrollPane, this, 0, totLines, 0, 0,
									sizELASTIC, sizELASTIC);
	theMainPane->FitToEnclosure (TRUE, TRUE);
	theMainPane->SetScales (1, LINEHEIGHT);
	
	theScrollPane->InstallPanorama (theMainPane);

/*
 * build the border
*/

	theBorder = new (CBorder);
	theBorder->IBorder (itsWindow, this, 10, 2, 0,
								LINEHEIGHT * LINESINMAINPANE + 43,
								sizELASTIC, sizFIXEDTOP);
	theBorder->FitToEnclosure (TRUE, FALSE);

/*
 * build the scrolling data pane
*/

	theScrollPane = new (CScrollPane);
	theScrollPane->IScrollPane (itsWindow, this,
					0, wAperture.bottom - (LINEHEIGHT * LINESINMAINPANE + 45),
					0, LINEHEIGHT * LINESINMAINPANE + 45,
					sizELASTIC, sizELASTIC,
					FALSE, TRUE, TRUE);
	theScrollPane->FitToEnclFrame (TRUE, FALSE);
	
	theDataPane = new (CDataPane);
	theDataPane->IDataPane (theScrollPane, this, 0, 0, 0, 0,
									sizELASTIC, sizELASTIC);
	itsDataPane = theDataPane;
	theDataPane->SetScales (1, LINEHEIGHT);
	theScrollPane->SetSteps (1, 1);
	theDataPane->FitToEnclosure (TRUE, TRUE);
	
	theScrollPane->InstallPanorama (theDataPane);
	
	itsWindow->Select ();
}
