#include "CHeapListerPane.h"
#include "CHeapInfo.h"
#include "CHeapListerDoc.h"
#include "HeapConstants.h"
#include "DPMUtilities.h"

extern CHeapInfo	*gHeap;
extern short		gClicks;

/****************************************/
/*		IHeapListerPane					*/
/****************************************/

void CHeapListerPane::IHeapListerPane(CView *anEnclosure, CBureaucrat *aSupervisor,
							short aWidth, short aHeight,
							short aHEncl, short aVEncl,
							SizingOption aHSizing, SizingOption aVSizing)
{
	CPanorama::IPanorama(anEnclosure, aSupervisor, aWidth, aHeight,
							aHEncl, aVEncl, aHSizing, aVSizing);
	
	wantsClicks = TRUE;
}

/****************************************/
/*		Draw							*/
/****************************************/

void CHeapListerPane::Draw (Rect *area)
{
	Str255			text;
	THz				zonePtr;
	blkHdr			*hdrPtr;
	int				h1 = 5;
	int				h2 = 180;
	int				h3 = 310;
	int				h4 = 380;
	int				v = 50;
	heapEntry		block;
	long			trueHandle;
	int				firstLine;
	int				lastLine;
	long			currentLine;
	heapHdr			heapHeader;
	GrafPtr			currentPort;
	unsigned char	mPtrFlags;
	long			selectedLine;
	Rect			inverseRect;
	FontInfo		info;
	char			display[10];
	
	TextFont (monaco);
	TextSize (9);
	GetFontInfo (&info);
	
	heapHeader = gHeap->GetHeader ();
/*
 * Print each visible line
*/
	firstLine = (area->top) / LINEHEIGHT - 1;
	if (firstLine < 1)
		firstLine = 1;
	
	lastLine = (area->bottom) / LINEHEIGHT + 1;
	if (lastLine > heapHeader.totalBlocks)
		lastLine = heapHeader.totalBlocks;
	if (lastLine > MAXBLOCKS)
		lastLine = MAXBLOCKS;
	
	selectedLine = gHeap->GetLineSelected ();
	
	for (currentLine = firstLine; currentLine <= lastLine; ++currentLine)
	{
		block = gHeap->GetBlock (currentLine);
		
		v = (currentLine - 1) * LINEHEIGHT + info.ascent - 1;
		
		NumToString (currentLine, text);
		MoveTo (h1 + 25 - StringWidth (text), v);
		DrawString (text);
		
		NumToHex6 ((long)block.address, display);
		MoveTo (h1 + 40, v);
		DrawString (display);
		
		MoveTo (h1 + 90, v);
		switch (block.type)
		{
			case FREE:
				DrawString ("\pFree");
				break;
			case NONRELOCATABLE:
				DrawString ("\pNonrel");
				MoveTo (h1 - 2, v);
				DrawChar ('*');
				break;
			case RELOCATABLE:
				DrawString ("\pRel");
				Move (30, 0);
				mPtrFlags = HGetState (block.mPtrAddress);
				
				if (mPtrFlags & 0x80)		/* locked */
					DrawChar ('L');
				else
					DrawChar ('l');
				
				if (mPtrFlags & 0x40)		/* purgeable */
					DrawChar ('P');
				else
					DrawChar ('p');
				
				if (mPtrFlags & 0x20)		/* resource */
					DrawChar ('R');
				else
					DrawChar ('r');
				
				if (mPtrFlags & 0x80)		/* locked */
				{
					MoveTo (h1 - 2, v);
					DrawChar ('*');
				}
				break;
			default:
				SysBeep (8);
				break;
		}
		
		NumToString (block.unused, text);
		MoveTo (h1 + 165, v);
		DrawString (text);
		
		NumToString (block.physSize - 8, text);
		MoveTo (h1 + 175, v);
		Move (40 - StringWidth (text), 0);
		DrawString (text);
		
		if (block.type == RELOCATABLE)
		{
			MoveTo (h1 + 225, v);
			if (WriteHandle (block.mPtrAddress, 0, 0) != (Handle)block.address)
				DrawChar ('+');
		}
		
		if (block.dataType != BYTE)
		{
			MoveTo (h1 + 330, v);
			gHeap->WriteDataType (currentLine);
		}
		
		if (currentLine == selectedLine)
		{
			SetRect (&inverseRect, 0, (currentLine - 1) * LINEHEIGHT,
							width, currentLine * LINEHEIGHT - 1);
			InvertRect (&inverseRect);
		}
	}
}

/****************************************/
/*		DrawLine						*/
/****************************************/

void CHeapListerPane::DrawLine (long lineNo)
{
	Rect			area;
	FontInfo		info;
	
	SetRect (&area, 0, (lineNo - 1) * LINEHEIGHT,
						width, lineNo * LINEHEIGHT - 1);
	inherited::RefreshRect (&area);
}

/****************************************/
/*		SetPaneSize						*/
/****************************************/

void CHeapListerPane::SetPaneSize ()
{
	Rect			bounds;
	heapHdr			header;
	long			totLines;
	Point			p;
	
	header = gHeap->GetHeader ();
	totLines = header.totalBlocks;
	
	if (totLines > MAXBLOCKS)
		totLines = MAXBLOCKS;
		
	this->GetPosition (&p);
	
	if (p.v > totLines - LINEHEIGHT)
	{
		p.v = totLines - LINESINMAINPANE;
		this->ScrollTo (p, FALSE);
	}
	
	this->GetBounds (&bounds);
	
	if (bounds.bottom != totLines)
	{
		bounds.bottom = totLines;
		this->SetBounds (&bounds);
	}
}

/****************************************/
/*		ScrollToSelection				*/
/****************************************/

void CHeapListerPane::ScrollToSelection ()
{
	Point			p;
	long			currentLine;
	
	currentLine = gHeap->GetLineSelected ();
	if (currentLine == 0)
		return;
	
	p.h = 0;
	p.v = currentLine - (LINESINMAINPANE + 1) / 2;
	
	if (p.v > bounds.bottom - LINESINMAINPANE)
		p.v = bounds.bottom - LINESINMAINPANE;
	else if (p.v < 0)
		p.v = 0;
	
	this->ScrollTo (p, FALSE);
}

/****************************************/
/*		DoClick							*/
/****************************************/

void CHeapListerPane::DoClick (Point hitPt, short modifierKeys, long when)

{
	long			currentLine;
	Rect			inverseRect;
	long			originalLine;
	long			newLine;
	Point			mouse;
	Point			panePosition;
	int				lineHeight;
	FontInfo		info;
	GrafPtr			currentPort;
	int				saveTextSize;
	heapHdr			heapHeader;
	short			paneHeight, paneWidth;
	
	GetPort (&currentPort);
	saveTextSize = currentPort->txSize;
	
	TextSize (9);
	GetFontInfo (&info);
	lineHeight = info.ascent + info.descent + info.leading;
	TextSize (saveTextSize);
	
	heapHeader = gHeap->GetHeader ();
	
	originalLine = currentLine = gHeap->GetLineSelected ();
	
	while (StillDown ())
	{
		GetMouse (&mouse);
		AutoScroll (mouse);
		
		newLine = mouse.v / lineHeight + 1;
		if (newLine < 0)
			newLine = 0;
		if (newLine > heapHeader.totalBlocks)
			newLine = 0;
			
		if (newLine != currentLine)
		{
			if (currentLine > 0)
			{
				SetRect (&inverseRect, 0, (currentLine - 1) * lineHeight,
								width, currentLine * lineHeight - 1);
				InvertRect (&inverseRect);
			}
			SetRect (&inverseRect, 0, (newLine - 1) * lineHeight,
							width, newLine * lineHeight - 1);
			InvertRect (&inverseRect);
			
			gHeap->SetLineSelected (newLine);	/* wrong - should be inside */
			currentLine = newLine;				/* the next IF statement	*/
		}
	}
	if (currentLine != originalLine)
	{
		itsSupervisor->DoCommand (cmdRefreshData);
	}
}
