/*------ Double FloatMatrix -------------------------------------------------*/

#include "Neural Network.h"
#include <math.h>

AllotDTypeMatrix(matrix,r,c)	/* setup space and dimensions for knot matrix	*/
	DTypeMatrix	* matrix;	/* address of matrix record	*/
	int			r,c;
{
	matrix->rows = r;
	matrix->cols = c;
	matrix->cells = (DataType **) NewHandle( r*c*sizeof(DataType) );
	if(!matrix->cells) HandleOutOfMem();
}

WriteMatrixToFile(jac,matrix)
	FILE * jac;
	DTypeMatrix * matrix;
{
	int	i,j;
	DataType * cell;
	
	HLock(matrix->cells);	
	
	cell = *matrix->cells;
	for(i=0; i<matrix->rows; i++)	
	{						
		for(j=0; j<matrix->cols; j++)  
			fprintf(jac,"%.5lf	",*(cell + (i*matrix->cols + j)) );
		fprintf(jac,"\n");
	}
	HUnlock(matrix->cells);	
}	

DisplayDTypeMatrix(matrix)
	DTypeMatrix 	*matrix;	/* address of matrix record	*/
{
	int	i,j;
	DataType * cell;
	
	HLock(matrix->cells);	
	
	cell = *matrix->cells;
	for(i=0; i<matrix->rows; i++)	
	{						
		for(j=0; j<matrix->cols; j++)  
			printf("(%.5le)  ",*(cell + (i*matrix->cols + j)) );
		printf("\n");
	}
	HUnlock(matrix->cells);	
}	

ClearDTypeMatrix(matrix)
	DTypeMatrix 	*matrix;	/* address of matrix record	*/
{
	int	i,j;
	DataType * cell;
	
	HLock(matrix->cells);
	
	cell = *matrix->cells;
	for(i=0; i<matrix->rows; i++)	
	{						
		for(j=0; j<matrix->cols; j++) 
			*(cell + (i*matrix->cols + j)) = 0.0;
	}
	HUnlock(matrix->cells);
}

/*---------------------- 
	postmultiply matrix by vector, result in prod 
	no checking for conformability
*/
Matrix_by_Vec(matrix,vector,prod)	
	DTypeMatrix	*	matrix;	/* address of matrix record	*/
	DTypeVector	*	vector;	/* address of vector record	*/
	DTypeVector	*	prod;	/* address of vector record to hold answer	*/
{
	register int	i;
	register int	j;
	register int	R;			/* number of rows in matrix	*/
	register int	C;			/* number of columns in matrix */
	register DataType temp;		/* ? this uses a floating pt register on 68881 */
	register DataType * y;		/* following declarations use up the 3 address registers */
	register DataType * row;
	register DataType * vec;

	R = matrix->rows;
	C = matrix->cols;
	vec = *vector->cells;
	y = *prod->cells;
	for(i=0; i<R; y++, i++)
	{	row = *matrix->cells + i*C;
		temp = (*row)*(*vec);
		for(j=1; j<C; j++) 	
			temp += (*(row+j))*(*(vec+j));
		*y = temp;
	}
}

/*---------------------- 
	postmultiply matrix by diagonal matrix, result in prod 
	no checking for conformability
	diagonal matrix nust be square, stored as a vector
*/
Matrix_by_Diag(matrix,diag,prod)
	DTypeMatrix	*	matrix;	/* address of matrix record	*/
	DTypeVector	*	diag;	/* address of vector record	*/
	DTypeMatrix	*	prod;	/* address of vector record to hold answer	*/
{
	register int	i;
	register int	j;
	register int	R;			/* number of rows in matrix	*/
	register int	C;			/* number of columns in matrix */
	register DataType * y;		/* pointer of cells of product matrix */
	register DataType * row;
	register DataType * vec;

	R = matrix->rows;
	C = matrix->cols;
	vec = *diag->cells;
	row = *matrix->cells;
	y = *prod->cells;
	for(i=0; i<R; i++)
	{	for(j=0; j<C; j++, row++, y++) 	
			*y = (*row)*(*(vec+j));
	}
}

/*---------------------- 
	postmultiply matrix1 by matrix1, result in prod 
	no checking for conformability
*/
Matrix_by_Matrix(matrix1,matrix2,prod)
	DTypeMatrix	*	matrix1;	/* address of matrix record	*/
	DTypeMatrix	*	matrix2;	/* address of matrix record	*/
	DTypeMatrix	*	prod;		/* address of matrix record	*/
{
	register int	i;
	register int	j;
	register int	k;
	register int	C1;			/* number of columns in matrix1 */
	int	R1;						/* number of rows in matrix1	*/
	register int	C2;			/* number of columns in matrix2 */
	register DataType sum;
	register DataType * y;		/* pointer to cells of product matrix */
	register DataType * row;
	register DataType * col;
	
	R1 = matrix1->rows;
	C1 = matrix1->cols;
	C2 = matrix2->cols;
	y = *prod->cells;
	
	for(i=0; i<R1; i++)
	{	for(j=0; j<C2; j++) 	
		{	col = *matrix2->cells + j;		/* position pointer at top of column for matrix2 */
			row = *matrix1->cells + i*C1;	/* position pointer at start of row for matrix1 */
			sum = 0.0;
			for(k=0; k<C1; k++, col += C2, row++)
				sum += (*row)*(*(col));
			*(y + i*C2 +j) = sum;
		}
	}
}