/*------ DataType Vector -------------------------------------------------*/

#include "Neural Network.h"
#include <math.h>

/*--------------------- 
	Compute the L2 norm of a vector
*/
double
L2Norm(vec)
	DTypeVector * vec;
{
	register int i;
	register int N = vec->rows;
	register double norm = 0.0;
	register DataType * cell = *vec->cells;
	
	for(i=0; i<N; i++, cell++)
		norm += pow((double)*cell,2.0);
		
	return(norm);
}

/*---------------------
	Multiply vector 1 by vector 2 and return answer as scaler.
*/
DataType
Vec_by_Vec(vec1,vec2)
	DTypeVector	* vec1;
	DTypeVector	* vec2;
{
	register int i;
	register DataType * cell1;
	register DataType * cell2;
	register DataType ans;
	
	ans = 0.0;
	cell1 = *vec1->cells;
	cell2 = *vec2->cells;
	for(i=0; i<vec1->rows; i++, cell1++, cell2++)
		ans += (*cell2)*(*cell1);
	return(ans);
}
/*---------------------
	Copy vector vec1 to vec2.
*/
CopyDTypeVector(vec1,vec2)
	DTypeVector	* vec1;
	DTypeVector * vec2;
{
	register int i;
	register DataType * cell1;
	register DataType * cell2;
	
	cell1 = *vec1->cells;
	cell2 = *vec2->cells;
	for(i=0; i<vec1->rows; i++, cell1++, cell2++)
		*cell2 = *cell1;
}

AllotDTypeVector(vector,r)	/* setup space and dimensions for vector	*/
	DTypeVector	* vector;	/* address of vector record	*/
	int			r;
{
	vector->rows = r;
	vector->cells = (DataType **) NewHandle( r*sizeof(DataType) );
	if(!vector->cells) HandleOutOfMem();
}

DisplayDTypeVector(vector)		/* clear vector */
	DTypeVector *	vector;	/* address of vector record	*/
{
	int	i;
	DataType * cell;
	
	HLock(vector->cells);	
	
	cell = *vector->cells;
	for(i=0; i<vector->rows; i++)	
	{						
			printf("(%.5le)  ",cell[i]);
	}
	printf("\n");

	HUnlock(vector->cells);	
}	

WriteVectorToFile(file,vector)
	FILE * file;
	DTypeVector * vector;
{
	int	i;
	DataType * cell;
	
	HLock(vector->cells);	
	
	cell = *vector->cells;
	for(i=0; i<vector->rows; i++)	
	{						
			fprintf(file,"	%.5lf	",cell[i]);
	}
	fprintf(file,"\n");

	HUnlock(vector->cells);	
}	

ClearDTypeVector(vector)
	DTypeVector *	vector;	/* address of matrix record	*/
{
	int	i;
	DataType * cell;
	
	HLock(vector->cells);
	
	cell = *vector->cells;
	for(i=0; i<vector->rows; i++)	
	{						
			cell[i] = 0.0;
	}
	HUnlock(vector->cells);
}
