{ ******************************************************** }
{ "rotMiscSubs.p"                                          }
{                                                          }
{ by John A. Love, III [ Washington Apple Pi Users' Group] }
{                                                          }
{ using Symantec's "THINK Lightspeed Pascal", v 3.00       }
{                                                          }
{ ******************************************************** }


UNIT rotMiscSubs;

INTERFACE

	USES
		Palettes, Retrace, Sound, rotInterface, rotGlobals;


	PROCEDURE InitManagers;
	FUNCTION TestForMac2: BOOLEAN;
	FUNCTION TestForColor: INTEGER;
	PROCEDURE LocalGlobal (VAR r: Rect);
	PROCEDURE GlobalLocal (VAR r: Rect);
	FUNCTION WNEisImplemented: BOOLEAN;
	PROCEDURE PlaySound (mySound: Str255);
	PROCEDURE InstallVBLTask (rsrcID: INTEGER);
	PROCEDURE RemoveVBLTask;
	FUNCTION GetMouseMovement (gMouse0: Point): Size;
	FUNCTION DoubleClick: BOOLEAN;




IMPLEMENTATION




{ ==================================== }
{ No further explanation is required : }
{ ==================================== }

	PROCEDURE FatalSystemCrash;

	BEGIN
		ExitToShell;
	END;   { FatalSystemCrash }



	PROCEDURE InitManagers;

	BEGIN

		MoreMasters;
		MoreMasters;
		MoreMasters;
		MoreMasters;
		InitGraf(@thePort);
		InitFonts;
		InitWindows;
		InitMenus;
		TEInit;
		InitDialogs(@FatalSystemCrash);
		;
		FlushEvents(everyEvent, 0);
		InitCursor;

	END;   { InitManagers }



{ ============================================== }
{ Test for a Mac II, or an SE30 for that matter: }
{ ============================================== }

	FUNCTION TestForMac2: BOOLEAN;

		VAR
			theWorld: SysEnvRec;
			error: OSErr;

	BEGIN

		TestForMac2 := FALSE;                    { Assume the old stuff !! }

		error := SysEnvirons(1, theWorld);
		IF error <> 0 THEN
			EXIT(TestForMac2);
		IF theWorld.machineType >= envMacII THEN
			TestForMac2 := TRUE;

	END;  { TestForMac2 }



{ ================================================================================ }
{ Test for the presence of a Mac with Color QuickDraw and a Color Monitor that the }
{ user has set to Color via the Control Panel or using the "Switch-A-Roo" FKEY.    }
{                                                                                  }
{ Return the color depth:                                                          }
{ ================================================================================ }

	FUNCTION TestForColor: INTEGER;

		LABEL
			100;

		VAR
			theWorld: SysEnvRec;
			error: OSErr;

	BEGIN

		TestForColor := 1;                        { Assume NO color := Black-and-White }

		error := SysEnvirons(1, theWorld);
		IF error <> 0 THEN
			EXIT(TestForColor);                     { or, GOTO 100 }
		IF NOT theWorld.hasColorQD THEN
			GOTO 100;
		TestForColor := GetGDevice^^.gdPMap^^.pixelSize;

100:

	END;  { TestForColor }



{ =================== }
{ A short-cut or two: }
{ =================== }

	PROCEDURE LocalGlobal (VAR r: Rect);

	BEGIN
		LocalToGlobal(r.topLeft);
		LocalToGlobal(r.botRight);
	END;   { LocalGlobal }



	PROCEDURE GlobalLocal (VAR r: Rect);

	BEGIN
		GlobalToLocal(r.topLeft);
		GlobalToLocal(r.botRight);
	END;   { GlobalLocal }



{ =================================== }
{ Common to the routines that follow: }
{ =================================== }

	FUNCTION TrapAvailable (myTrapNbr: INTEGER; myTrapType: TrapType): BOOLEAN;

		VAR
			UnimplementedTrapNbr: INTEGER;

	BEGIN

		UnimplementedTrapNbr := LoWord(BXOR(_Unimplemented, $A800));   { LONGINT -> INTEGER }
		TrapAvailable := (NGetTrapAddress(myTrapNbr, myTrapType) <> GetTrapAddress(UnimplementedTrapNbr));

	END;  { TrapAvailable }



{ ============================================== }
{ Check to see if _WaitNextEvent is implemented: }
{ ============================================== }

	FUNCTION WNEisImplemented: BOOLEAN;

		VAR
			WNEtrapNbr: INTEGER;
			theWorld: SysEnvRec;
			discardError: OSErr;

	BEGIN

		WNEtrapNbr := LoWord(BXOR(_WaitNextEvent, $A800));

{ Since _WaitNextEvent and _HFSDispatch have the same trap number ( = $60 ),     }
{ we can call "TrapAvailable" for _WaitNextEvent ONLY if we are on a machine     }
{ that supports separate OS and ToolBox trap tables.  Therefore, we need to      }
{ check for a machineType that is >= 0.  NOTE that even if we get an error       }
{ calling _SysEnvirons, the compiler's glue has filled-in the machineType field: }

		discardError := SysEnvirons(SysEnvironsVersion, theWorld);
		IF theWorld.machineType < 0 THEN
			WNEisImplemented := FALSE
		ELSE
			WNEisImplemented := TrapAvailable(WNEtrapNbr, ToolTrap);

	END;  { WNEisImplemented }



{ =============== }
{ Play it, Sam !! }
{ =============== }

	PROCEDURE PlaySound (mySound: Str255);

		CONST
			_SndPlay = $A805;

		VAR
			SndPlayTrapNbr: INTEGER;
			theWorld: SysEnvRec;
			discardError: OSErr;
			SndPlayIsImplemented: BOOLEAN;
			sndHandle: Handle;

	BEGIN

		SndPlayTrapNbr := LoWord(BXOR(_SndPlay, $A800));
		discardError := SysEnvirons(SysEnvironsVersion, theWorld);
		IF theWorld.machineType < 0 THEN
			SndPlayIsImplemented := FALSE
		ELSE
			SndPlayIsImplemented := TrapAvailable(SndPlayTrapNbr, ToolTrap);
		;
		sndHandle := GetNamedResource('snd ', mySound);
		IF NOT SndPlayIsImplemented | (sndHandle = NIL) THEN
			EXIT(PlaySound);

		discardError := SndPlay(NIL, sndHandle, FALSE);

	END;  { PlaySound }



{ ======================== }
{ My spinning CURSor Task: }
{ ======================== }

	PROCEDURE SpinTheBottle;                    { Cute, huh ?!!? }

		VAR
			globe: cursHandle;
			globeID: INTEGER;

	BEGIN

		PushA5;                                    { Whatever havoc the System has wrought !! }
		GetMyA5;                                   { So I can access my Globals. }

		globeID := CURS_ID0 + nbrGlobe - 1;
		globe := GetCursor(globeID);
		SetCursor(globe^^);

		nbrGlobe := nbrGlobe - 1;                  { Reset stuff for next time ... }
		IF nbrGlobe = 0 THEN
			nbrGlobe := acurHandle(acurHdl)^^.nbrCursors;
		;
		myVBLRec.MyVBL.vblCount := 10;

		PopA5;

	END;   { SpinTheBottle }



{ ============================= }
{ Round-and-around she goes ... }
{ ============================= }

	PROCEDURE InstallVBLTask (rsrcID: INTEGER);

		VAR
			watch: cursHandle;
			ignore: INTEGER;

	BEGIN

		acurHdl := GetResource('acur', rsrcID);
		IF acurHdl = NIL THEN
		BEGIN
			watch := GetCursor(watchCursor);
			SetCursor(watch^^);                   { Reset afterwards by my HandleCursor proc. }
		END   { IF acurHdl = NIL }
		ELSE
		BEGIN

			CURS_ID0 := HiWord(acurHandle(acurHdl)^^.cursorIDs[0]);
			nbrGlobe := acurHandle(acurHdl)^^.nbrCursors;
			;
			WITH myVBLRec, MyVBL DO
			BEGIN
				CurrA5 := longPtr(CurrentA5)^;
				vblAddr := @SpinTheBottle;
				vblCount := 10;                     { Six times every second. }
				qType := ORD(vType);
				vblPhase := 0;
			END;   { WITH }
			;
			ignore := VInstall(@myVBLRec.MyVBL);

		END;   { ELSE }

	END;   { InstallVBLTask }



	PROCEDURE RemoveVBLTask;

		VAR
			ignore: INTEGER;

	BEGIN

		IF acurHdl <> NIL THEN
			ignore := VRemove(@myVBLRec.MyVBL);
		acurHdl := NIL;                      { Mark as gone. }

  { CURSor reset later by "HandleCursor". }
		Sleep := 1;                           { ... so this happens under MultiFinder. }
		;
		stillColorCrsr := FALSE;              { See "HandleCursor". }

	END;   { RemoveVBLTask }



{ ======================================================= }
{ Returns vertical movement in High word and horizontal   }
{ movement in low word, similar to _GrowWindow.           }
{                                                         }
{ Note that the input Point is in GLOBAL coordinates.     }
{ Otherwise, dragging a window will return zero movement. }
{ ======================================================= }

	FUNCTION GetMouseMovement (gMouse0: Point): Size;

		VAR
			mouseLoc: Point;
			mouseDH, mouseDV: INTEGER;
			sizeMove: Size;

	BEGIN

		GetMouse(mouseLoc);
		LocalToGlobal(mouseLoc);            { ... apples with apples }
		mouseDH := mouseLoc.h - gMouse0.h;
		mouseDV := mouseLoc.v - gMouse0.v;
		IF mouseDH < 0 THEN                { Absolute values ... }
			mouseDH := -mouseDH;
		IF mouseDV < 0 THEN
			mouseDV := -mouseDV;
		;
		sizeMove := mouseDV;
		sizeMove := BSL(sizeMove, 16);      { ... into High word. }
		sizeMove := sizeMove + mouseDH;     {  +  the low word.   }
		GetMouseMovement := sizeMove;

	END;   { GetMouseMovement }



{ ================================= }
{ Note that the algorithm I used    }
{ returns FALSE if we are dragging. }
{ ================================= }

	FUNCTION DoubleClick: BOOLEAN;

		VAR
			startTime, endTime, doubleTime: LONGINT;
			mouseLoc0: Point;
			sizeMove: Size;

	BEGIN   { DoubleClick }

		DoubleClick := FALSE;               { Assume Nada !! }
		doubleTime := GetDblTime;

		startTime := TickCount;             { Initialize time & mouse location. }
		endTime := startTime;
		GetMouse(mouseLoc0);
		LocalToGlobal(mouseLoc0);

		WHILE StillDown & ((endTime - startTime) <= doubleTime) DO   { First mouse click. }
			endTime := TickCount;             { Times out if dragging ... }

		sizeMove := GetMouseMovement(mouseLoc0);
		;
		WHILE ((endTime - startTime) <= doubleTime) & (LoWord(sizeMove) <= 5) & (HiWord(sizeMove) <= 5) DO
		BEGIN
			IF Button THEN
			BEGIN
				DoubleClick := TRUE;            { Second time's a charm !! }
				Leave;
			END;   { IF Button }
			;
			endTime := TickCount;
			sizeMove := GetMouseMovement(mouseLoc0);
		END;   { WHILE small delta Time AND small delta movement }

	END;   { DoubleClick }




END.   { UNIT = rotMiscSubs }