{ ******************************************************** }
{ "rotWindowSubs.p"                                        }
{                                                          }
{ by John A. Love, III [ Washington Apple Pi Users' Group] }
{                                                          }
{ using Symantec's "THINK Lightspeed Pascal", v 3.00       }
{                                                          }
{ ******************************************************** }


UNIT rotWindowSubs;

INTERFACE

	USES
		Palettes, rotInterface, rotGlobals, rotMiscSubs, rotScrollSubs;


	FUNCTION GetWindowPartColor (window: WindowPtr; part: INTEGER; VAR color: RGBColor): BOOLEAN;
	PROCEDURE SetWindowPalette (wPtr: WindowPtr; plttID: INTEGER);
	PROCEDURE CalcWindowFrame (window: WindowPtr; VAR r: Rect);
	PROCEDURE DisplayWindow (window: WindowPtr);
	PROCEDURE CloseOurWindow (wPtr: WindowPtr);
	PROCEDURE DoCloseAll;
	PROCEDURE ZoomRect (sourceR, destR: Rect);




IMPLEMENTATION




{ -------------------- }
{ Finder's keepers ... }
{ -------------------- }

	FUNCTION GetWindowPartColor (window: WindowPtr; part: INTEGER; VAR color: RGBColor): BOOLEAN;

		VAR
			auxWindowHdl: AuxWinHndl;
			windowCTab: CTabHandle;

	BEGIN

		GetWindowPartColor := FALSE;                             { Assume NADA !! }

		IF GetAuxWin(window, auxWindowHdl) THEN
		BEGIN
			windowCTab := auxWindowHdl^^.awCTable;
			IF (part < 0) | (part > windowCTab^^.ctSize) THEN     { Color me paranoid !! }
				EXIT(GetWindowPartColor);
			color := windowCTab^^.ctTable[part].rgb;
			GetWindowPartColor := TRUE;
		END;   { IF window has a AuxWinRec }

	END;   { GetWindowPartColor }



{ ----------------------- }
{ ... in living Color !!  }
{ ----------------------- }

	PROCEDURE SetWindowPalette (wPtr: WindowPtr; plttID: INTEGER);

		VAR
			pal: PaletteHandle;

	BEGIN

		IF NOT aMac2 THEN
			EXIT(SetWindowPalette);

		pal := GetNewPalette(plttID);
		IF Handle(pal) <> NIL THEN
		BEGIN
			SetPalette(wPtr, pal, TRUE);
			ActivatePalette(wPtr);
		END;   { IF Handle(pal) <> NIL }

	END;   { SetWindowPalette }



{ --------------------------------------------------------- }
{ CanNOT use the "structRgn" field of the window since this }
{ region handle will be NIL if the window is NOT visible.   }
{ --------------------------------------------------------- }

	PROCEDURE CalcWindowFrame (window: WindowPtr; VAR r: Rect);

		CONST
			frame = 1;
			shadow = 1;
			title = 18;

		VAR
			windDef: INTEGER;

	BEGIN

		windDef := GetWVariant(window);

		r := window^.portRect;
		InsetRect(r, -frame, -frame);
		IF (windDef = 0) OR (windDef > 3) THEN
			r.top := r.top - title;                      { Window has a title bar. }
		IF (windDef = documentProc) OR (windDef = altDBoxProc) OR (windDef = noGrowDocProc) OR (windDef = zoomDocProc) THEN
		BEGIN
			r.bottom := r.bottom + shadow;
			r.right := r.right + shadow;
		END;   { Window has a shadow frame. }

	END;   { CalcWindowFrame }



{ --------------------------------------------------- }
{ Before showing the window, center it on the screen. }
{ --------------------------------------------------- }

	PROCEDURE DisplayWindow (window: WindowPtr);

		CONST
			frame = 1;
    { shadow = 1;     -----     considered within CalcWindowFrame PROC }
			title = 18;

		VAR
			wFrameRect: rect;
			temp: INTEGER;
			Pt: Point;

	BEGIN

		CalcWindowFrame(window, wFrameRect);
		;
		windDef := GetWVariant(window);                { In DoActivate, also. }

		WITH screen DO
			temp := bottom - top;
		WITH wFrameRect DO
			temp := temp - (bottom - top);               { screen height - window height }
  { temp := temp + mBarHt;       --       screen based on GrayRgn, NOT screenBits. }
		temp := temp DIV 2;
		temp := temp + frame;
		IF (windDef = 0) OR (windDef > 3) THEN
			temp := temp + title;                        { Window has a title bar. }
		Pt.v := screen.top + temp;
  { ----- }
		WITH screen DO
			temp := right - left;
		WITH wFrameRect DO
			temp := temp - (right - left);               { screen width - window width }
		Pt.h := screen.left + temp DIV 2;

		MoveWindow(window, Pt.h, Pt.v, TRUE);
		ShowWindow(window);
		ScrollResize(window);

	END;   { DisplayWindow }



{ ----------------------- }
{ One at a time, folks !! }
{ ----------------------- }

	PROCEDURE CloseOurWindow (wPtr: WindowPtr);

		VAR
			myPic: PicHandle;
			pal: PaletteHandle;
			aux: BOOLEAN;
			auxWind: AuxWinHndl;

	BEGIN

		IF aMac2 THEN
		BEGIN
			pal := GetPalette(wPtr);
			IF pal <> NIL THEN
				DisposePalette(pal);
			aux := GetAuxWin(wPtr, auxWind);
			IF aux THEN
				ReleaseResource(Handle(auxWind));
		END;    { IF aMac2 }

		myPic := GetWindowPic(wPtr);
		IF myPic <> NIL THEN
		BEGIN
			HUnlock(Handle(myPic));
			ReleaseResource(Handle(myPic));
		END;   { IF myPic <> NIL }

		horizControl := ScrollHoriz(wPtr);
		IF horizControl <> NIL THEN
			DisposeControl(horizControl);          { Calls ReleaseResource(Handle(AuxCtlHndl)); }
  { ----- }
		vertControl := ScrollVert(wPtr);
		IF vertControl <> NIL THEN
			DisposeControl(vertControl);
  { ----- }
		DisposeWindow(wPtr);

	END;   { CloseOurWindow }



{ ------------------------------------------------- }
{ DoCloseAll is called from the "Quit" command on   }
{ the main Menu, and when we close our main window. }
{ ------------------------------------------------- }

	PROCEDURE DoCloseAll;

		VAR
			window: WindowPeek;

	BEGIN

		window := WindowPeek(FrontWindow);

		WHILE window <> NIL DO
		BEGIN

			IF window^.windowKind < 0 THEN
				CloseDeskAcc(window^.windowKind)
			ELSE
				CloseOurWindow(WindowPtr(window));

			window := window^.nextWindow;

		END;   { WHILE window <> NIL }

	END;   { DoCloseAll }



	PROCEDURE ZoomRect (sourceR, destR: Rect);     { Global coordinates. }

		CONST
			DragPatternLoc = $A34;
			pixPatID = 128;

		VAR
			oldPort: GrafPtr;
			oldWindow: WindowPtr;
			bigGrafPort: GrafPort;
			bigGrafPtr: GrafPtr;
			bigCGrafPort: CGrafPort;
			bigCGrafPtr: CGrafPtr;
			union, srcRect, dstRect, box: Rect;
			increment, delTop, delLeft, delBottom, delRight, i: INTEGER;
			aux: BOOLEAN;
			frameColor: RGBColor;
			bwDragPattern: Pattern;
			cDragPattern: PixPatHandle;


	BEGIN

		IF EqualRect(sourceR, destR) THEN
			EXIT(ZoomRect);
		;
		UnionRect(sourceR, destR, union);
		IF (NOT EqualRect(union, sourceR)) & (NOT EqualRect(union, destR)) THEN
			EXIT(ZoomRect);                      { One does NOT enclose the other !! }

		GetPort(oldPort);

		oldWindow := FrontWindow;
		IF (oldWindow <> NIL) & (colorDepth > 1) THEN
			aux := GetWindowPartColor(oldWindow, wFrameColor, frameColor);

		IF colorDepth = 1 THEN
		BEGIN
			bigGrafPtr := @bigGrafPort;
			OpenPort(bigGrafPtr);
			SetPort(bigGrafPtr);
		END   { IF colorDepth = 1 }

		ELSE

		BEGIN
			bigCGrafPtr := @bigCGrafPort;
			OpenCPort(bigCGrafPtr);
			SetPort(GrafPtr(bigCGrafPtr));
		END;

		srcRect := sourceR;
		dstRect := destR;
		GlobalLocal(srcRect);
		GlobalLocal(dstRect);

		increment := 5;
		delTop := (dstRect.top - srcRect.top) DIV increment;
		delLeft := (dstRect.left - srcRect.left) DIV increment;
		delBottom := (dstRect.bottom - srcRect.bottom) DIV increment;
		delRight := (dstRect.right - srcRect.right) DIV increment;

		bwDragPattern := PatPtr(DragPatternLoc)^;
		PenPat(bwDragPattern);                    { My default state ... }
		PenMode(patXor);

		IF colorDepth > 1 THEN   { in color }
		BEGIN
			cDragPattern := GetPixPat(pixPatID);
			IF cDragPattern <> NIL THEN
			BEGIN
				IF (oldWindow <> NIL) & aux THEN
					MakeRGBPat(cDragPattern, frameColor);
				PenPixPat(cDragPattern);
			END;   { IF cDragPattern <> NIL }
		END;   { in color }

  { Start AWAY from source AND stop short of destination. }

		box := srcRect;
		;
		FOR i := 1 TO (increment - 1) DO
		BEGIN
			WITH box DO
				SetRect(box, left + delLeft, top + delTop, right + delRight, bottom + delBottom);
			FrameRect(box);
			;
			Delay(15, finalTicks);                 { Hang on, Mac !! }
		END;   { FOR }

		PenNormal;

		IF colorDepth > 1 THEN
		BEGIN
			CloseCPort(bigCGrafPtr);
  {   IF cDragPattern <> NIL THEN     -----     Done by _CloseCPort. }
  {    DisposPixPat(cDragPattern);                                   }
		END
		ELSE   { B&W }
			ClosePort(bigGrafPtr);
		;
		SetPort(oldPort);

	END;   { ZoomRect }




END.   { UNIT = rotWindowSubs }