/************************************/
/* General.c

Contains some general functions */
/************************************/
#include "General.h"

/* Routines are as follows:

	 int MenuBarHeight(void)
	 void PositionRect(small, large, hOption, vOption)
	 OSErr AddDItem(theDITL, theType, theRect, param)
	 SICNHandle GetSICN(sicnID)
	 long CountSICN(theSICN)
	 void PlotSICN(theRect, theSICN, theIndex)
*/

/* routines */
/************************************/
/* int MenuBarHeight(void)
	Gets the current height of the menu bar */
/************************************/
int MenuBarHeight(void)
{
#define  mBarHeightGlobal 0xBAA
int *memPtr;

memPtr = (int *) mBarHeightGlobal;

if (*memPtr < 20)
	return (20);
else
	return(*memPtr);
} /* int MenuBarHeight(void) */

/************************************/
/* void PositionRect(small, large, hOption, vOption)
	takes two rectangles, small and large, and positions
	the smaller one horiz. and vert. according to
	options */
/************************************/
void PositionRect(small, large, hOption, vOption)
Rect *small, *large;
int hOption, vOption;
{
int hOff, vOff;

if (hOption != 0) {
	small->right -= small->left;
	small->left = 0;
	hOff = (large->right - large->left - small->right) / hOption;
	small->left = large->left + hOff;
	small->right += small->left;
	}
	
if (vOption != 0) {
	small->bottom -= small->top;
	small->top = 0;
	vOff = (large->bottom - large->top - small->bottom) / vOption;
	small->top = large->top + vOff;
	small->bottom += small->top;
	}
} /* void PositionRect(small, large, hOption, vOption) */

/************************************/
/* OSErr AddDItem(theDITL, theType, theRect, param)
	adds a dialog item to a dialog item list creating
	it if needed */
/************************************/
OSErr AddDItem(theDITL, theType, theRect, param)
Handle *theDITL;
char theType;
Rect *theRect;
char *param;
{
char *p;
DITLItemPtr d;
OSErr error;
int **itemCount, i;
char baseType, dataLen, dataLenDITL;

/*	1.  Check on first one and get count
	2.  Get size to add onto handle
	3.  Loop to place
	4.  Put data in
*/

/* 1.  Check on first one */
baseType = theType;
if ((unsigned char) baseType >= itemDisable) baseType -= itemDisable;
switch (baseType) {
	case ctrlItem + btnCtrl:
	case ctrlItem + chkCtrl:
	case ctrlItem + radCtrl:
	case statText:
	case editText:
		dataLen = *param;
		/* *(char *) */
		break;
	case ctrlItem + resCtrl:
	case iconItem:
	case picItem:
		dataLen = 2;
		break;
	case userItem:
		dataLen = 0;
		break;
	
	default:
		return(-50);
	}

if (*theDITL == NIL) { /* first item */
	*theDITL = NewHandle(2);
	if ((error = MemError()) != noErr) return(error);
	itemCount = (int **) *theDITL;
	**itemCount = -1;
	} /* first item */
else { /* another item */
	itemCount = (int **) *theDITL;
	}

/* 2.  Get size to add onto handle */
/* Get size to make handle */
dataLenDITL = dataLen;
if (dataLenDITL % 2) ++dataLenDITL;

/* increase handle size */
SetHandleSize(*theDITL, GetHandleSize(*theDITL) + sizeof(DITLItem) + dataLenDITL);
if ((error = MemError()) != noErr) return(error);

/* 3.  Loop to place */
HLock(*theDITL);
p = (char *) (**theDITL) + 2;

for (i=0; i<= (int) (**itemCount); i++) { /* item loop */
	d = (DITLItemPtr) p;
	dataLenDITL = (int) (*d).dataLength;
	if (dataLenDITL % 2) ++dataLenDITL;
	p = p + sizeof(DITLItem) + dataLenDITL;
	} /* item loop */

d = (DITLItemPtr) p;

/* 4.  Put data in */	
if (baseType == userItem)
	(*d).HorP = (long) param;
else
	(*d).HorP = 0L;
(*d).displayRect = *theRect;
(*d).itemType = theType;
(*d).dataLength = (char) dataLen;

p = p + sizeof(DITLItem);
switch (baseType) {
	case ctrlItem + btnCtrl:
	case ctrlItem + chkCtrl:
	case ctrlItem + radCtrl:
	case statText:
	case editText:
		BlockMove(param + 1, p, dataLen);
		break;
	case ctrlItem + resCtrl:
	case iconItem:
	case picItem:
		BlockMove(param + 2, p, dataLen);
		break;
		
	case userItem:
		break;
	
	default:;
	}

**itemCount = (int) (**itemCount) + 1;

HUnlock(*theDITL);
} /* OSErr AddDItem(theDITL, theType, theRect, param) */

/************************************/
/* SICNHandle GetSICN(sicnID)
	Gets a sicn handle */
/************************************/
SICNHandle GetSICN(sicnID)
int sicnID;
{
return ((SICNHandle) GetResource('SICN', sicnID));
} /* SICNHandle GetSICN(sicnID) */

/************************************/
/* long CountSICN(theSICN)
	counts the sicn in a sicn handle */
/************************************/
long CountSICN(theSICN)
SICNHandle theSICN;
{
return ((long) (GetHandleSize((Handle) theSICN) / sizeof(SICN)) );
} /* long CountSICN(theSICN) */

/************************************/
/* void PlotSICN(theRect, theSICN, theIndex)
	plots a small icon */
/************************************/
void PlotSICN(theRect, theSICN, theIndex)
Rect *theRect;
SICNHandle theSICN;
long theIndex;
{
auto char state; /* saves original flags of SICN handle */
auto BitMap srcBits; /* built up around SICN data so we can _CopyBits */

/* check the index for a valid value */
if (((GetHandleSize((Handle)theSICN)) / sizeof(SICN)) > theIndex) {
	/* store the resource's current locked/unlocked condition */
	state = HGetState((Handle) theSICN);
	
	/* lock the resource so it won't move during _CopyBits */
	HLock((Handle) theSICN);
	
	/* set up small icon's bitmap */
	srcBits.baseAddr = (Ptr) (*theSICN)[theIndex];
	srcBits.rowBytes = 2;
	SetRect(&srcBits.bounds, 0, 0, 16, 16);
	
	/* draw the small icon in the current grafport */
	CopyBits(&srcBits, &(thePort->portBits), &srcBits.bounds, theRect, srcCopy, NIL);
	
	/* restore handle's state */
	HSetState((Handle) theSICN, state);
	}
} /* void PlotSICN(theRect, theSICN, theIndex) */