{ --------------------------------------------------------------------------- }
{ Apple Macintosh Developer Technical Support                                 }
{                                                                             }
{ Some of "Skippy White's Famous High Level Off-Screen Map Routines"          }
{                                                                             }
{ "OffscreenSubs.p"                                                           }
{                                                                             }
{ These routines provide a high-level interface to the QuickDraw & Color      }
{ Manager routines which allow the creation and manipulation of off-screen    }
{ bitmaps and pixmaps. They are designed to run on any machine with 128K or   }
{ later ROMs.                                                                 }
{                                                                             }
{ NOTE that I've modified some of Skippy's routines and that, therefore, any  }
{      resultant errors in syntax or logic belong solely to me.               }
{ --------------------------------------------------------------------------- }

UNIT OffscreenSubs;


INTERFACE

	USES
		rotInterface, rotGlobals, rotMiscSubs;


	FUNCTION MakeCleanColorTable (inputColors: CTabHandle; depth: INTEGER; VAR makeTable: CTabHandle): OSErr;
	FUNCTION GetMaxAreaDevice (globalRect: Rect): GDHandle;
	FUNCTION CreateOffScreen (VAR myRect: Rect): OSErr;
	PROCEDURE ToOnScreen;
	PROCEDURE DisposOffScreen;




IMPLEMENTATION




{ ********** }
	FUNCTION MakeCleanColorTable (inputColors: CTabHandle; depth: INTEGER; VAR makeTable: CTabHandle): OSErr;

{ This call takes a handle to a list of colors to be included in a color table. }
{ It returns a table suitable for use for a PixMap, with black and white at the }
{ correct offsets -- white @ the beginning, black @ the end. If there is no     }
{ default color table for the depth in question, it returns NIL for the table.  }

		FUNCTION IsWhite (rgb: RGBColor): BOOLEAN;
		BEGIN
			IsWhite := (rgb.red = $FFFF) & (rgb.green = $FFFF) & (rgb.blue = $FFFF);
		END; { IsWhite }

		FUNCTION IsBlack (rgb: RGBColor): BOOLEAN;
		BEGIN
			IsBlack := (rgb.red = 0) & (rgb.green = 0) & (rgb.blue = 0);
		END; { IsBlack }

		PROCEDURE MakeWhite (rgb: RGBColor);
		BEGIN
			rgb.red := $FFFF;
			rgb.green := $FFFF;
			rgb.blue := $FFFF;
		END; { MakeWhite }

		PROCEDURE MakeBlack (rgb: RGBColor);
		BEGIN
			rgb.red := 0;
			rgb.green := 0;
			rgb.blue := 0;
		END; { MakeBlack }

		VAR
			inputColorIndex, makeTableIndex: INTEGER;
			aColor: RGBColor;


	BEGIN { MakeCleanColorTable }

		makeTable := GetCTable(depth);

		IF makeTable = NIL THEN
			MakeCleanColorTable := memFullErr
		ELSE
			WITH makeTable^^ DO
			BEGIN
				ctSeed := GetCTSeed;   { Get a fresh seed; we'll change the color table. }
				;
				makeTableIndex := 1;   { Leave room for white @ the beginning. }
				inputColorIndex := 0;  { Start with first color. }

				REPEAT

					IF inputColorIndex > inputColors^^.ctSize THEN
						Leave;             { Done with all the input colors. }
					aColor := inputColors^^.ctTable[inputColorIndex].rgb;
					inputColorIndex := inputColorIndex + 1;   { Advance to next color. }
					IF IsWhite(aColor) THEN
						Cycle;             { This one is white, try another. }
					IF IsBlack(aColor) THEN
						Cycle;             { This one is black, try another. }
					ctTable[makeTableIndex].rgb := aColor;   { Use this color. }
					makeTableIndex := makeTableIndex + 1;

      { ... until my Table is full, with leaving room for black @ the end. }
				UNTIL makeTableIndex > ctSize - 1;

				MakeWhite(aColor);
				ctTable[0].rgb := aColor;
				MakeBlack(aColor);
				ctTable[makeTableIndex].rgb := aColor;

				MakeCleanColorTable := noErr;
			END; { WITH makeTable^^ & ELSE makeTable <> NIL }

	END;   { MakeCleanColorTable }



{ ********** }
	FUNCTION GetMaxAreaDevice (globalRect: Rect): GDHandle;

{ Find the greatest overlap device for the given global rectangle.}

		VAR
			area: LONGINT;
			maxArea: LONGINT;
			device: GDHandle;
			intersection: Rect;

	BEGIN

		GetMaxAreaDevice := NIL;
		;
		maxArea := 0;
		;
		device := GetDeviceList;

		WHILE device <> NIL DO
		BEGIN
			IF TestDeviceAttribute(device, screenDevice) THEN
				IF TestDeviceAttribute(device, screenActive) THEN

					IF SectRect(globalRect, device^^.gdRect, intersection) THEN
					BEGIN

						WITH intersection DO
							area := LONGINT(right - left) * LONGINT(bottom - top);

						IF area > maxArea THEN
						BEGIN
							GetMaxAreaDevice := device;
							maxArea := area;
						END;   { IF area > maxArea }

					END;   { IF SectRect ... }

			device := GetNextDevice(device);
		END;   { WHILE device <> NIL }

	END;   { GetMaxAreaDevice }



{ ****************************************** }
{ Just for scrolling & other nifty stuff ... }
{ ****************************************** }


	FUNCTION CreateOffScreen (VAR myRect: Rect): OSErr;
{ Reference: Tech Note #120                             }
{ with special thanks to Jon Zap of MacDTS              }
{                                                       }
{ NOTE: Local window coordinates are input, but local   }
{ screen coordinates are returned for drawing purposes. }

		VAR
			offRowBytes: LONGINT;
			sizeOfOff: LONGINT;
			localRect, globRect: Rect;
			i, maxDepth: INTEGER;
			err: OSErr;

		PROCEDURE ErrorOut (error: OSErr);

		BEGIN
			CreateOffScreen := error;
			EXIT(CreateOffScreen);
		END;   { ErrorOut }


	BEGIN   { CreateOffScreen }

		CreateOffScreen := noErr;

		globRect := myRect;        { We're about to switch the Port to off-screen: }
		LocalGlobal(globRect);

		IF colorDepth = 1 THEN
		BEGIN

			offGrafPtr := @offGrafPort;
			OpenPort(offGrafPtr);
			maxDepth := 1;

		END   { IF colorDepth = 1 }
		ELSE
		BEGIN

    { Figure out how much space we need for our pixel image.  We call    }
    { Skippy White's GetMaxAreaDevice to get the pixel map from that.    }
    { We do this to cover the case where the pixel image that we wish to }
    { CopyBits to spans multiple devices (of possibly different depths). }

			myMaxDevice := GetMaxAreaDevice(globRect);
			IF myMaxDevice = NIL THEN
				ErrorOut(MaxDevError);

    { We need to set theGDevice to the device with the maximum pixel }
    { depth (the one we just found), so the pixel map of the new     }
    { CGrafPort will be copied from one of the proper depth.  Save   }
    { the current GDevice for retrieval later and, then, open the    }
    { new CGrafPort to use for our off-screen drawing.               }

			oldDevice := GetGDevice;
			SetGDevice(myMaxDevice);
			;
			offCGrafPtr := @offCGrafPort;           { Initialize this guy. }
			OpenCPort(offCGrafPtr);
			maxDepth := offCGrafPtr^.portPixMap^^.pixelSize;

		END;   { ELSE: colorDepth > 1 }

  { However, before we do ANYthing more, we should set the         }
  { off-screen's visRgn to the FULL size of the input rect so the  }
  { image stays whole even when the window has been dragged partly }
  { beyond the physical edge(s) of the screen.  Otherwise, the     }
  { visRgn^^.rgnBBox in local coordinates remains equal to         }
  { screenBits.bounds as initialized when _Open(C)Port was called: }

		IF colorDepth > 1 THEN
			RectRgn(offCGrafPort.visRgn, globRect)
		ELSE
			RectRgn(offGrafPort.visRgn, globRect);

  { We are now ready to calculate the size of the pixel image we will need.  }
  { Then we can set the location-specific and size-specific information of   }
  { the pixel map.  Since Color QuickDraw distinguishes between a bitmap and }
  { a pixel map by checking the high bit of rowBytes, we need to set it.     }

		localRect := globRect;
		GlobalLocal(localRect);

		WITH localRect DO
		BEGIN
			offRowBytes := (maxDepth * (right - left) + 15) DIV 16;   { # of words.    }
			IF ODD(offRowBytes) THEN                                 { ... made even. }
				offRowBytes := offRowBytes + 1;
			offRowBytes := offRowBytes * 2;                           { Back to bytes. }
			sizeOfOff := LONGINT(bottom - top) * offRowBytes;
		END;   { WITH }

		myBits := NewClearPtr(sizeOfOff);        { Allocate space for the pixel image.}
		IF MemError <> noErr THEN
			ErrorOut(NewPtrError);

  { NOTE that we're filling in the BitMap/PixMap fields of the new Port }
  { directly, so we do NOT call _SetPortBits or _SetCPortPix later:     }

		IF colorDepth > 1 THEN
		BEGIN
			WITH offCGrafPtr^.portPixMap^^ DO
			BEGIN
				baseAddr := myBits;
				rowBytes := offRowBytes + $8000;     { Remember to be a PixMap.}
				bounds := localRect;
			END;   { WITH }

			offBitMapPtr := BitMapPtr(offCGrafPtr^.portPixMap^);
		END   { IF colorDepth > 1 }

		ELSE   { "Yucky" black-and-white. }

		BEGIN
			WITH offGrafPtr^.portBits DO
			BEGIN
				baseAddr := myBits;
				rowBytes := offRowBytes;
				bounds := localRect;
			END;

			offBitMapPtr := @offGrafPtr^.portBits;
		END;

		IF colorDepth > 1 THEN
		BEGIN

    { Next, we clone the color table of the maxDevice }
    { and put it into our off-screen pixel map.       }

			ourCTHandle := myMaxDevice^^.gdPMap^^.pmTable;
			err := HandToHand(Handle(ourCTHandle));    { Clone it. }
			IF err <> noErr THEN
				ErrorOut(CloneHdlError);

			FOR i := 0 TO ourCTHandle^^.ctSize DO
				ourCTHandle^^.ctTable[i].value := i;

    { The following is required to convert GDevice cluts to Pixmap cluts:     }
    {                                                                         }
    { NOTE: THINK Pascal <v 2.0> re-named the field "ctFlags" to "transIndex" }

			ourCTHandle^^.ctFlags := BAND(ourCTHandle^^.ctFlags, $7FFF);
			ourCTHandle^^.ctSeed := GetCTSeed;
			offCGrafPtr^.portPixMap^^.pmTable := ourCTHandle;    { --> the off-screen map. }

		END;   { IF colorDepth > 1 }

		myRect := localRect;          { Return local screen coordinates. }

	END;   { CreateOffScreen}



{ ******************* }
{ Back to "Square 1": }
{ ******************* }


	PROCEDURE ToOnScreen;

	BEGIN

		IF colorDepth > 1 THEN
		BEGIN
			GetCWMgrPort(onCScreen);
			SetPort(GrafPtr(onCScreen));
			SetGDevice(oldDevice);
			onScreenBitsPtr := BitMapPtr(onCScreen^.portPixMap^);
		END   { IF colorDepth > 1 }

		ELSE

		BEGIN
			GetWMgrPort(onBWScreen);
			SetPort(onBWScreen);
			onScreenBitsPtr := @onBWScreen^.portBits;
		END;   { ELSE = "Yucky" black-and-white }

	END;   { ToOnScreen}



{ **************************** }
{ Out with the new.            }
{                              }
{ Whoops -- I meant the old !! }
{ **************************** }


	PROCEDURE DisposOffScreen;

		LABEL
			100, 200;

	BEGIN

		IF CreateOffScreenError = MaxDevError THEN
			EXIT(DisposOffScreen)
		ELSE IF CreateOffScreenError = NewPtrError THEN
			GOTO 200
		ELSE IF CreateOffScreenError = CloneHdlError THEN
			GOTO 100;

  { noErr ... }

		IF colorDepth > 1 THEN
			DisposHandle(Handle(ourCTHandle));
100:
		DisposPtr(myBits);
200:
		IF colorDepth > 1 THEN
			CloseCPort(offCGrafPtr)
		ELSE
			ClosePort(offGrafPtr);

	END;   { DisposOffScreen }




END. { UNIT = OffscreenSubs }