{ ******************************************************** }
{ "rotScrollSubs.p"                                        }
{                                                          }
{ by John A. Love, III [ Washington Apple Pi Users' Group] }
{                                                          }
{ using Symantec's "THINK Lightspeed Pascal", v 3.00       }
{                                                          }
{ ******************************************************** }


UNIT rotScrollSubs;

INTERFACE

	USES
		Palettes, rotGlobals, rotMiscSubs, OffScreenSubs;


	PROCEDURE ScrollText (myText: QDPtr; box: Rect);
	PROCEDURE Scroll (ctl: ControlHandle; part: INTEGER; Pt: Point);
	FUNCTION DrawMyControl (ctl: ControlHandle): BOOLEAN;
	FUNCTION ScrollHoriz (windPtr: WindowPtr): ControlHandle;
	FUNCTION ScrollVert (windPtr: WindowPtr): ControlHandle;
	PROCEDURE ScrollShow (windPtr: WindowPtr);
	PROCEDURE ScrollHide (windPtr: WindowPtr);
	PROCEDURE InvalidScroll (windPtr: WindowPtr);
	PROCEDURE ValidScroll (windPtr: WindowPtr);
	PROCEDURE ScrollResize (windPtr: WindowPtr);
	PROCEDURE SetMaxCtls (windPtr: WindowPtr);
	PROCEDURE SetCtlsToMin (windPtr: WindowPtr);


{ +++++++++++++++++++++++++++++++++++++++ }
{ Our Globals for the scrolling routines: }
{ +++++++++++++++++++++++++++++++++++++++ }

	VAR
		window: WindowPtr;
		VertOrHoriz: LONGINT;
		oldCtlValue: INTEGER;
		stillThere: INTEGER;
		temp: Rect;




IMPLEMENTATION




{ -------------------------------------------- }
{ Scrolls your text string from right to left. }
{ -------------------------------------------- }

	PROCEDURE ScrollText (myText: QDPtr; box: Rect);

		LABEL
			100, 200, 300, 400, 500, 600;

		VAR
			myPort: GrafPtr;
			buffer: ARRAY[0..255] OF SignedByte;
			textBuf: QDPtr;
			boxWidth: INTEGER;
			leftJustify: BOOLEAN;
			x0, y0: INTEGER;
			widthSpace: INTEGER;
			textLen: SignedByte;
			bufWidth: INTEGER;
			firstChar: INTEGER;
			lastChar: INTEGER;
			charCount: INTEGER;
			finalTicks: LONGINT;


	BEGIN

		GetPort(myPort);
		ClipRect(myPort^.portRect);

		textLen := myText^;
		IF textLen = 0 THEN                   { Null string. }
			EXIT(ScrollText);
		;
		textBuf := QDPtr(ORD4(@buffer));
		textLen := SignedByte(ORD(textLen) + 1);   { Include the length byte. }
		BlockMove(Ptr(myText), Ptr(textBuf), size(textLen));
		;
		buffer[0] := textLen;                  { textLen = textLen + 1 from above. }
		buffer[ORD(textLen)] := SignedByte(SP);    { Add a trailing space. }

		widthSpace := CharWidth(SP);
		WITH box DO
		BEGIN
			boxWidth := right - left;
			y0 := bottom - top;
			IF y0 < 10 THEN                     { NOT tall enough !! }
				EXIT(ScrollText);
			y0 := (y0 - 6) DIV 2;
			y0 := bottom - y0;                   { y0 = bottom - 7 for box.tall = 20 }
		END;
		leftJustify := FALSE;                  { Assume right-justified text. }
		;
		firstChar := 1;                        { Start AFTER length byte. }
		lastChar := 1;
		charCount := 1;


{ Scroll the text: }
100:
		bufWidth := TextWidth(textBuf, firstChar, charCount);

		IF bufWidth > boxWidth THEN           { Text does NOT fit !! }

		BEGIN
			firstChar := firstChar + 1;          { Drop first character and }
			charCount := charCount - 1;		   {   try for fit again.     }
			leftJustify := TRUE;
			GOTO 100;
		END;   { IF bufWidth > boxWidth }

{ it Fits }
200:
		EraseRect(box);
		x0 := box.right - bufWidth;
		IF x0 < box.left THEN                { Needed ONLY for very short strings. }
			leftJustify := TRUE;

		IF leftJustify THEN
			x0 := box.left;
  { y0 := box.bottom - 7; }
		MoveTo(x0, y0);
		DrawText(textBuf, firstChar, charCount);
		lastChar := lastChar + 1;

		IF lastChar <= ORD(textLen) THEN     { Haven't reached end of string. }
			GOTO 400;

		IF leftJustify THEN                  { We've reached the left edge. }
			GOTO 300;

{ Not to left edge yet. }

		bufWidth := bufWidth + widthSpace;
		Delay(10, finalTicks);                { ... otherwise too quick. }
		GOTO 200;

{ At left edge. }
300:
		firstChar := firstChar + 1;
		charCount := charCount - 1;
		IF charCount = 0 THEN                { All characters moved left. }
			GOTO 600;                          { Fini !! }
		GOTO 500;

{ Next character. }
400:
		charCount := charCount + 1;           { Bump the length. }

500:
		Delay(10, finalTicks);                { ... otherwise too quick. }
		GOTO 100;                            { Scroll again. }

{ Fini!! }
600:
		ValidRect(box);                       { No Updates, please ... }

	END;   { ScrollText }



{ ------------------------------------------------------------ }
{ Scroll the PICTure displayed in the window's content region: }
{ ------------------------------------------------------------ }

	PROCEDURE ScrollContents (ctl: ControlHandle; dh, dv: INTEGER);

		VAR
			oldClip: RgnHandle;
			myPic: PicHandle;
			gWindPICTrect, kWindPICTrect, oldScrolledRect, newScrolledRect: Rect;

	BEGIN

  { GetPort(oldPort);   --- we KNOW it's the passed window. }
  { window := ctl^^.contrlOwner;                            }
		oldClip := NewRgn;
		GetClip(oldClip);
		myPic := PicHandle(WindowPeek(window)^.refCon);

		gWindPICTrect := windPICTrect;
		LocalGlobal(gWindPICTrect);
		oldScrolledRect := scrolledFullPICTrect;
		;
		CreateOffScreenError := CreateOffScreen(oldScrolledRect);
		;
		kWindPICTrect := gWindPICTrect;
		WITH screenBits.bounds DO                    { See "DoRotate". }
			OffsetRect(kWindPICTrect, left, top);

		IF CreateOffScreenError = noErr THEN
		BEGIN

			ClipRect(oldScrolledRect);                  { Same ole stuff ... }
			EraseRect(oldScrolledRect);
			DrawPicture(myPic, oldScrolledRect);
			;
			ToOnScreen;
			;
			newScrolledRect := oldScrolledRect;         { Scrolling up & down the avenue. }
			OffsetRect(newScrolledRect, dh, dv);
			OffsetRect(scrolledFullPICTrect, dh, dv);   { ... for next time. }
			;
			BackColor(whiteColor);
			ForeColor(blackColor);
			ClipRect(kWindPICTrect);
			EraseRect(kWindPICTrect);                   { Erase old image. }
			CopyBits(offBitMapPtr^, onScreenBitsPtr^, oldScrolledRect, newScrolledRect, srcCopy, NIL);

		END;   { IF CreateOffScreenError = noErr }

		DisposOffScreen;

		SetPort(window);                              { Re-group ... }
		SetClip(oldClip);
		ValidRect(windPICTrect);                      { NO updates please !! }
		DisposeRgn(oldClip);

	END;   { ScrollContents }



{ ----------------------------------------------------- }
{ Mouse clicked on the line arrows:                     }
{                                                       }
{ NOTE: As a matter of academic principle, avoid speed  }
{ penalities associated with TRAP overhead if possible. }
{ ----------------------------------------------------- }

	PROCEDURE UpActionProc (ctl: ControlHandle; part: INTEGER);

		VAR
			newCtlValue: INTEGER;             { # of lines }
			dv, dh, difference: INTEGER;      { # of pixels }

	BEGIN

		IF part = 0 THEN                   { Mouse moved OUTSIDE the Control !! }
			EXIT(UpActionProc);

		oldCtlValue := ctl^^.contrlValue;
		dv := 12;                           { Vertical, NOT sideways !! }
		dh := 0;

		difference := (oldCtlValue - ctl^^.contrlMin) * 12;
		IF difference = 0 THEN             { Prevent flickering. }
			EXIT(UpActionProc);
		IF difference < dv THEN
			dv := difference;

		newCtlValue := oldCtlValue - 1;     { Decrement one line's worth. }
		temp := ctl^^.contrlRect;           { Just because _SetCtlValue is buggy for     }
		InsetRect(temp, 1, 1);              { a Mac II set to black-and-white.  A Mac II }
		ClipRect(temp);                     { set to color works okay, though ?????      }
		SetCtlValue(ctl, newCtlValue);
		ClipRect(window^.portRect);         { Reset. }

		IF VertOrHoriz = horizScrollID THEN
		BEGIN
			dh := dv;                         { Sideways, NOT vertical !! }
			dv := 0;
		END;   { IF VertOrHoriz = horizScrollID }

		ScrollContents(ctl, dh, dv);

		Delay(8, finalTicks);               { ... otherwise too fast. }

	END;   { UpActionProc }



{ --------------------------------- }
{ Mouse clicked on the line arrows: }
{ --------------------------------- }

	PROCEDURE DownActionProc (ctl: ControlHandle; part: INTEGER);

		VAR
			newCtlValue: INTEGER;             { # of lines }
			dv, dh, difference: INTEGER;      { # of pixels }

	BEGIN

		IF part = 0 THEN                   { Mouse moved OUTSIDE the Control !! }
			EXIT(DownActionProc);

		oldCtlValue := ctl^^.contrlValue;
		dv := 12;                           { Vertical, NOT sideways !! }
		dh := 0;

		difference := (ctl^^.contrlMax - oldCtlValue) * 12;
		IF difference = 0 THEN             { Prevent flickering. }
			EXIT(DownActionProc);
		IF difference < dv THEN
			dv := difference;

		newCtlValue := oldCtlValue + 1;     { Bump it one line's worth. }
		temp := ctl^^.contrlRect;
		InsetRect(temp, 1, 1);
		ClipRect(temp);
		SetCtlValue(ctl, newCtlValue);
		ClipRect(window^.portRect);         { Reset. }

		IF VertOrHoriz = horizScrollID THEN
		BEGIN
			dh := dv;                         { Sideways, NOT vertical !! }
			dv := 0;
		END;   { IF VertOrHoriz = horizScrollID }

		ScrollContents(ctl, -dh, -dv);

		Delay(8, finalTicks);

	END;   { DownActionProc }



{ --------------------------- }
{ Our main Scrolling routine: }
{ --------------------------- }

	PROCEDURE Scroll (ctl: ControlHandle; part: INTEGER; Pt: Point);




{ ------------------------------------------------------------------ }
{ Scroll contents of window to match the pre-set Scroll Bar setting: }
{ ------------------------------------------------------------------ }

		PROCEDURE ScrollToThumbPosition (ctl: ControlHandle);

			VAR
				newCtlValue, dh, dv: INTEGER;

		BEGIN

			dh := 0;
			dv := 0;
			newCtlValue := ctl^^.contrlValue;

			IF VertOrHoriz = horizScrollID THEN
				dh := -(newCtlValue - oldCtlValue) * 12     { Normally, * teLineHite }
			ELSE   { = vertScrollID }
				dv := -(newCtlValue - oldCtlValue) * 12;

			ScrollContents(ctl, dh, dv);

			Delay(8, finalTicks);

		END;   { ScrollToThumbPosition }



{ ----------------------------------------------- }
{ Thumb goes UP; text, PICTure etc. scrolls DOWN: }
{ ----------------------------------------------- }

		PROCEDURE DoPageUp (ctl: ControlHandle);

			VAR
				partControl, newCtlValue, dv, ctlDelta: INTEGER;
				newPoint: Point;

		BEGIN

			WITH window^.portRect DO
			BEGIN
				IF VertOrHoriz = horizScrollID THEN
					dv := right - left - growBoxSize;          { = (right+1-left) - scrollWidth }
				IF VertOrHoriz = vertScrollID THEN
					dv := bottom - top - growBoxSize;          { = (bottom+1-top) - scrollHeight }
			END;   { WITH window^.portRect }

			ctlDelta := dv;
    { Normally divide by teLineHite, a field of the TERecord whose Handle }
    { is usually stored in the wRefCon field of the WindowRecord:         }
			ctlDelta := (ctlDelta DIV 12) - 1;             { Leave one line's-worth showing. }

			WHILE StillDown DO
			BEGIN

				GetMouse(newPoint);
				partControl := TestControl(ctl, newPoint);
				IF partControl = inPageUp THEN               { Still INSIDE Control ... }
				BEGIN

					oldCtlValue := ctl^^.contrlValue;
					newCtlValue := oldCtlValue - ctlDelta;
					temp := ctl^^.contrlRect;
					InsetRect(temp, 1, 1);
					ClipRect(temp);
					SetCtlValue(ctl, newCtlValue);              { Compensates if newCtlValue }
					ClipRect(window^.portRect);                 { overshoots ctlMin.         }
					ScrollToThumbPosition(ctl);

				END;   { IF partControl = inPageUp }

			END;   { WHILE StillDown }

		END;   { DoPageUp }



{ ----------------------------------------------- }
{ Thumb goes DOWN; text, PICTure etc. scrolls UP: }
{ ----------------------------------------------- }

		PROCEDURE DoPageDown (ctl: ControlHandle);

			VAR
				partControl, newCtlValue, dv, ctlDelta: INTEGER;
				newPoint: Point;

		BEGIN

			WITH window^.portRect DO
			BEGIN
				IF VertOrHoriz = horizScrollID THEN
					dv := right - left - growBoxSize
				ELSE   { = vertScrollID }
					dv := bottom - top - growBoxSize;
			END;   { WITH window^.portRect }

			ctlDelta := (dv DIV 12) - 1;

			WHILE StillDown DO
			BEGIN

				GetMouse(newPoint);
				partControl := TestControl(ctl, newPoint);
				IF partControl = inPageDown THEN             { Still INSIDE Control ... }
				BEGIN

					oldCtlValue := ctl^^.contrlValue;
					newCtlValue := oldCtlValue + ctlDelta;
					temp := ctl^^.contrlRect;
					InsetRect(temp, 1, 1);
					ClipRect(temp);
					SetCtlValue(ctl, newCtlValue);              { Compensates if newCtlValue }
					ClipRect(window^.portRect);                 { overshoots ctlMax.         }
					ScrollToThumbPosition(ctl);

				END;   { IF partControl = inPageDown }

			END;   { WHILE StillDown }

		END;   { DoPageDown }



	BEGIN   { Scroll }

		window := ctl^^.contrlOwner;
		SetPort(window);
		VertOrHoriz := ctl^^.contrlRFcon;          { Up/Down or sideways ?? }
		oldCtlValue := GetCtlValue(ctl);

		CASE part OF

			inUpButton: 
				stillThere := TrackControl(ctl, Pt, @UpActionProc);

			inDownButton: 
				stillThere := TrackControl(ctl, Pt, @DownActionProc);

			inPageUp: 
				DoPageUp(ctl);

			inPageDown: 
				DoPageDown(ctl);

			inThumb: 
			BEGIN
				temp := ctl^^.contrlRect;
				InsetRect(temp, 1, 1);
				ClipRect(temp);
				IF TrackControl(ctl, Pt, NIL) <> 0 THEN
					ScrollToThumbPosition(ctl);
				ClipRect(window^.portRect);            { Reset. }
			END;

			OTHERWISE

			BEGIN

			END;   { OTHERWISE }

		END;   { CASE }

	END;   { Scroll }



{ -------------------------- }
{ Does she or doesn't she ?? }
{ -------------------------- }

	FUNCTION DrawMyControl (ctl: ControlHandle): BOOLEAN;

	BEGIN

		IF (ctl <> NIL) & (ctl^^.contrlMax > ctl^^.contrlMin) THEN
			DrawMyControl := TRUE
		ELSE
			DrawMyControl := FALSE;

	END;   { DrawMyControl }



{ -------------------------------- }
{ Retrieve Control Handle, if any: }
{ -------------------------------- }

	FUNCTION ScrollHoriz (windPtr: WindowPtr): ControlHandle;

		VAR
			ourControl: ControlHandle;

	BEGIN

		ourControl := WindowPeek(windPtr)^.controlList;

		WHILE ourControl <> NIL DO
		BEGIN
			IF ourControl^^.contrlRFcon = horizScrollID THEN
				LEAVE;
			ourControl := ourControl^^.nextControl;
		END;   { WHILE ourControl <> NIL }

		ScrollHoriz := ourControl;

	END;   { ScrollHoriz }



	FUNCTION ScrollVert (windPtr: WindowPtr): ControlHandle;

		VAR
			ourControl: ControlHandle;

	BEGIN

		ourControl := WindowPeek(windPtr)^.controlList;

		WHILE ourControl <> NIL DO
		BEGIN
			IF ourControl^^.contrlRFcon = vertScrollID THEN
				LEAVE;
			ourControl := ourControl^^.nextControl;
		END;   { WHILE }

		ScrollVert := ourControl;

	END;   { ScrollVert }



{ ------------------ }
{ Hello, or GoodBye: }
{ ------------------ }

	PROCEDURE ScrollShow (windPtr: WindowPtr);

		VAR
			ctlHndl: ControlHandle;

	BEGIN

		ctlHndl := ScrollVert(windPtr);
		;
		IF DrawMyControl(ctlHndl) THEN          { Tests for NIL ctlHndl. }
			ShowControl(ctlHndl);
  { ----- }
		ctlHndl := ScrollHoriz(windPtr);
		;
		IF DrawMyControl(ctlHndl) THEN
			ShowControl(ctlHndl);

	END;   { ScrollShow }



	PROCEDURE ScrollHide (windPtr: WindowPtr);

		VAR
			ctlHndl: ControlHandle;

	BEGIN

		ctlHndl := ScrollVert(windPtr);
		;
		IF DrawMyControl(ctlHndl) THEN
			HideControl(ctlHndl);
  { ----- }
		ctlHndl := ScrollHoriz(windPtr);
		;
		IF DrawMyControl(ctlHndl) THEN
			HideControl(ctlHndl);

	END;   { ScrollHide }



{ ----------------------------------------------------------------- }
{ Explicitly include the Scroll Bars in the window's Update region. }
{ This Update region will purposely overlap the Grow Box.           }
{ ----------------------------------------------------------------- }

	PROCEDURE InvalidScroll (windPtr: WindowPtr);

		VAR
			updateRect: rect;

	BEGIN

		IF DrawMyControl(ScrollVert(windPtr)) THEN
		BEGIN
			updateRect := windPtr^.portRect;
			InsetRect(updateRect, -1, -1);                    { Include the window frame. }
			updateRect.left := updateRect.right - scrollWidth;
			InvalRect(updateRect);
		END;   { IF }
		;
		IF DrawMyControl(ScrollHoriz(windPtr)) THEN
		BEGIN
			updateRect := windPtr^.portRect;
			InsetRect(updateRect, -1, -1);
			updateRect.top := updateRect.bottom - scrollHeight;
			InvalRect(updateRect);
		END;   { IF }

	END;   { InvalidScroll }



	PROCEDURE ValidScroll (windPtr: WindowPtr);

		VAR
			updateRect: rect;

	BEGIN

		IF DrawMyControl(ScrollVert(windPtr)) THEN
		BEGIN
			updateRect := windPtr^.portRect;
			InsetRect(updateRect, -1, -1);
			updateRect.left := updateRect.right - scrollWidth;
			ValidRect(updateRect);
		END;   { IF }
		;
		IF DrawMyControl(ScrollHoriz(windPtr)) THEN
		BEGIN
			updateRect := windPtr^.portRect;
			InsetRect(updateRect, -1, -1);
			updateRect.top := updateRect.bottom - scrollHeight;
			ValidRect(updateRect);
		END;   { IF }

	END;   { ValidScroll }



	PROCEDURE ScrollResize (windPtr: WindowPtr);

		VAR
			contentRect: Rect;
			ctlHndl: ControlHandle;
			ctlWidth, ctlHeight, ctlTop, ctlLeft: INTEGER;

	BEGIN

		contentRect := windPtr^.portRect;

  { Remember, the portRect does NOT include the window frame, }
  { whereas the Scroll Bar and Grow Box sizes do:             }

		InsetRect(contentRect, -1, -1);
		ClipRect(contentRect);             { Sigh !! }

		ScrollHide(windPtr);               { Hide-and-Go Seek !! }

		ctlHndl := ScrollVert(windPtr);
		IF ctlHndl <> NIL THEN
		BEGIN

			WITH contentRect DO
			BEGIN
				ctlHeight := bottom - top - growBoxSize;
				ctlTop := top;
				ctlLeft := right - scrollWidth;
			END;   { WITH contentRect }

			SizeControl(ctlHndl, scrollWidth, ctlHeight);
			MoveControl(ctlHndl, ctlLeft, ctlTop);

		END;   { IF ctlHndl <> NIL }
  { ----- }
		ctlHndl := ScrollHoriz(windPtr);
		IF ctlHndl <> NIL THEN
		BEGIN

			WITH contentRect DO
			BEGIN
				ctlWidth := right - left - growBoxSize;
				ctlTop := bottom - scrollHeight;
				ctlLeft := left;
			END;   { WITH contentRect }

			SizeControl(ctlHndl, ctlWidth, scrollHeight);
			MoveControl(ctlHndl, ctlLeft, ctlTop);

		END;   { IF ctlHndl <> NIL }

		ScrollShow(windPtr);               { Peek-a-Boo !! }

		ValidScroll(windPtr);

		ClipRect(windPtr^.portRect);       { Reset. }

	END;   { ScrollResize }



{ ------------------------------------------------------- }
{ Set ONLY the maximum value(s) because the attached CNTL }
{ resources(s) specify the minimum -- generally zero.     }
{ ------------------------------------------------------- }

	PROCEDURE SetMaxCtls (windPtr: WindowPtr);

		VAR
			oldClip: RgnHandle;
			ctlHndl: ControlHandle;
			dest, view: INTEGER;

		PROCEDURE SetMaxCtlValue (ctl: ControlHandle; excess: INTEGER);

			VAR
				maxValue: INTEGER;
				temp: Rect;

		BEGIN

			IF excess <= 0 THEN
				maxValue := ctl^^.contrlMin          { Inactivates Control since max = min. }
			ELSE                                  { PICTure taller than window           }
				maxValue := (excess + 11) DIV 12;   { ... by this much [rounded up].       }

			IF maxValue <> ctl^^.contrlMax THEN
			BEGIN
				temp := ctl^^.contrlRect;
				InsetRect(temp, 1, 1);
				ClipRect(temp);
				SetCtlMax(ctl, maxValue);
			END;

		END;   { SetMaxCtlValue }


	BEGIN   { SetMaxCtls }

		oldClip := NewRgn;
		GetClip(oldClip);

		ctlHndl := ScrollVert(windPtr);
		IF ctlHndl <> NIL THEN
		BEGIN

			WITH windPICTrect DO            { ... from my GetPicRects Proc. }
				view := bottom - top;

			WITH fullPICTrect DO            { Ditto. }
				dest := bottom - top;

			SetMaxCtlValue(ctlHndl, dest - view);

		END;   { IF ctlHndl <> NIL }

  { ----- }

		ctlHndl := ScrollHoriz(windPtr);
		IF ctlHndl <> NIL THEN
		BEGIN

			WITH windPICTrect DO
				view := right - left;

			WITH fullPICTrect DO
				dest := right - left;

			SetMaxCtlValue(ctlHndl, dest - view);

		END;   { IF ctlHndl <> NIL }

		SetClip(oldClip);
		DisposeRgn(oldClip);

	END;   { SetMaxCtls }



{ ---------------------------------- }
{ Called by the "HandleMouse" PROC: }
{ ---------------------------------- }

	PROCEDURE SetCtlsToMin (windPtr: WindowPtr);

		VAR
			oldClip: RgnHandle;

		PROCEDURE SetCtlValueToMin (ctl: ControlHandle);

			VAR
				temp: Rect;

		BEGIN

			IF ctl = NIL THEN
				EXIT(SetCtlValueToMin);

			temp := ctl^^.contrlRect;
			InsetRect(temp, 1, 1);               { See comments in "UpActionProc". }
			ClipRect(temp);
			SetCtlValue(ctl, GetCtlMin(ctl));

		END;   { SetCtlValueToMin }


	BEGIN   { SetCtlsToMin }

		oldClip := NewRgn;
		GetClip(oldClip);
		;
		SetCtlValueToMin(ScrollHoriz(windPtr));
		SetCtlValueToMin(ScrollVert(windPtr));
		;
		SetClip(oldClip);
		DisposeRgn(oldClip);

	END;   { SetCtlsToMin }




END.   { UNIT = rotScrollSubs }