/*****
 * CRemoveLFApp.c
 *
 *	Application methods for a typical application.
 *
 *****/
#include "oops.h"
#include "messageDefines.h"
#include "CRemoveLFApp.h"
#include "CRemoveLFDoc.h"
#include "CmyFileManager.h"
#include "CmyDataFile.h"
#include "CTextFilter.h"

extern	OSType	gSignature;

static char readBuffer[512];
/*..........................................................*/
void CRemoveLFApp::Run(void)
/*
	Name: Wade Maxfield
	Date: November 25, 1989
	Notes:
	Run
	
	we need to do our thing here, not the normal built in
	code work
	Modification History:
*/
{
CmyFileManager *FileManager;
SFReply sourceReply;
SFReply destinationReply;
short returnMessage;
CmyDataFile *sourceDataFile;
CmyDataFile *destinationDataFile;
long bytesRead;
CTextFilter *TextFilter;

	FileManager = new(CmyFileManager);
	FileManager->ImyFileManager();
	
/* ask for the source file */
	returnMessage = FileManager->GetExistingFile(&sourceReply);

	if ( returnMessage == NAK )
		return; /* this application is finished. */
		
/* ask for the destination file. (this creates it also) */
	returnMessage = FileManager->GetNewFile(&destinationReply,'TEXT');

	if ( returnMessage == NAK )
		return; /* this application is finished. */

	returnMessage = FileManager->OpenFile(&sourceDataFile,&sourceReply);
	
	if ( returnMessage == NAK )
		return;
		
	returnMessage = FileManager->OpenFile(&destinationDataFile,&destinationReply);

	if ( returnMessage == NAK )
		return;
	
	/* we are ready for the new text filter object */
	TextFilter = new(CTextFilter);
	TextFilter->ITextFilter(); /* initialize it */
	
	do
		{		
		returnMessage = sourceDataFile->ReadLine(readBuffer,512L,&bytesRead);
		
		if ( returnMessage == NAK )
			break;
		
		/* strip the line feeds */
		TextFilter->StripLineFeeds(readBuffer,&bytesRead);
				
		returnMessage = destinationDataFile->WriteSome(readBuffer,bytesRead);
		} while(TRUE);


	FileManager->CloseFile(&sourceDataFile);
	
	FileManager->CloseFile(&destinationDataFile);
	
	TextFilter->Dispose(); /* get rid of the text filter object */
}



/***
 * IRemoveLFApp
 *
 *	Initialize the application. Your initialization method should
 *	at least call the inherited method. If your application class
 *	defines its own instance variables or global variables, this
 *	is a good place to initialize them.
 *
 ***/

void CRemoveLFApp::IRemoveLFApp(void)

{
	CApplication::IApplication(4, 20480L, 2048L);
}



/***
 * SetUpFileParameters
 *
 *	In this routine, you specify the kinds of files your
 *	application opens.
 *
 *
 ***/

void CRemoveLFApp::SetUpFileParameters(void)

{
	inherited::SetUpFileParameters();	/* Be sure to call the default method */

		/**
		 **	sfNumTypes is the number of file types
		 **	your application knows about.
		 **	sfFileTypes[] is an array of file types.
		 **	You can define up to 4 file types in
		 **	sfFileTypes[].
		 **
		 **/

	sfNumTypes = 1;
	sfFileTypes[0] = 'TEXT';

		/**
		 **	Although it's not an instance variable,
		 **	this method is a good place to set the
		 **	gSignature global variable. Set this global
		 **	to your application's signature. You'll use it
		 **	to create a file (see CFile::CreateNew()).
		 **
		 **/

	gSignature = '????';
}



/***
 * DoCommand
 *
 *	Your application will probably handle its own commands.
 *	Remember, the command numbers from 1-1023 are reserved.
 *	The file Commands.h contains all the reserved commands.
 *
 *	Be sure to call the default method, so you can get
 *	the default behvior for standard commands.
 *
 ***/
void CRemoveLFApp::DoCommand(long theCommand)

{
	switch (theCommand) {
	
		/* Your commands go here */
	
		default:	inherited::DoCommand(theCommand);
					break;
	}
}

/***
 * Exit
 *
 *	Chances are you won't need this method.
 *	This is the last chance your application gets to clean up
 *	things like temporary files.
 *
 ***/
 
void CRemoveLFApp::Exit()

{
	/* your exit handler here */
}


/***
 * CreateDocument
 *
 *	The user chose New from the File menu.
 *	In this method, you need to create a document and send it
 *	a NewFile() message.
 *
 ***/

void CRemoveLFApp::CreateDocument()

{
	CRemoveLFDoc	*theDocument;
	
	theDocument = new(CRemoveLFDoc);
		
		/**
		 **	Send your document an initialization
		 **	message. The first argument is the
		 **	supervisor (the application). The second
		 **	argument is TRUE if the document is printable.
		 **
		 **/
	
	theDocument->IRemoveLFDoc(this, TRUE);

		/**
		 **	Send the document a NewFile() message.
		 **	The document will open a window, and
		 **	set up the heart of the application.
		 **
		 **/
	theDocument->NewFile();
}

/***
 * OpenDocument
 *
 *	The user chose Open from the File menu.
 *	In this method you need to create a document
 *	and send it an OpenFile() message.
 *
 *	The macSFReply is a good SFReply record that contains
 *	the name and vRefNum of the file the user chose to
 *	open.
 *
 ***/
 
void CRemoveLFApp::OpenDocument(SFReply *macSFReply)

{
	CRemoveLFDoc	*theDocument;
	
	theDocument = new(CRemoveLFDoc);
		
		/**
		 **	Send your document an initialization
		 **	message. The first argument is the
		 **	supervisor (the application). The second
		 **	argument is TRUE if the document is printable.
		 **
		 **/
	
	theDocument->IRemoveLFDoc(this, TRUE);

		/**
		 **	Send the document an OpenFile() message.
		 **	The document will open a window, open
		 **	the file specified in the macSFReply record,
		 **	and display it in its window.
		 **
		 **/
	theDocument->OpenFile(macSFReply);
}