/********************************/
/* File: PlaySndFrag.c			*/
/*								*/
/********************************/

#define		UsingHypercard

#include 	<SoundMgr.h>
#include	<SetupA4.h>
#include 	<HyperXCmd.h>

/*
 
	usage: playSndFrag fileName, soundName, start ,stop
 
 	project requires the following files or libraries:
 		ANSI-A4		-- the A4 based standard "C" libraries
 		HyperXLib	-- the xcmd callback glue
 		MacTraps	-- what it is
 		PlaySnd.c	-- this file
*/
 
 
void   	putResult( XCmdPtr pp, char *msg);
void 	PlayRunSound( Handle sndHdl, long start, long stop );
long 	paramtoNum( XCmdPtr pp, short i );
void	paramtoPString( XCmdPtr pp, short i, char *str );
Handle	strToParam( char	*str );


#ifndef	NIL
	#define	NIL	(void *)0L
#endif

 
 
   
   
   
 
pascal void main( XCmdPtr pp )
/******************************************
* The main entry point 
******************************************/
{
	short 			resFile;			/*** resource file holding the 'snd '	***/
	short			saveResFile;		/*** previous resource file id			***/
	Handle  		sndHdl;				/*** handle to the sound resource		***/
	long 			start;				/*** start sample in sound				***/
	long			stop;				/*** end sample in sound				***/
	char			*temp[256];			/*** for converting to pascal strings	***/
   
 
	pp->returnValue = NIL;				/* empty return means OK 		*/

	if (pp->paramCount == 1){
		if ( **(pp->params[0]) == '!' ){
			pp->returnValue = strToParam("Play Sound XCMD, version 1.0, Donald Koscheka, 1990");
			return;
		}
		
		if ( **(pp->params[0]) == '?' ){
			pp->returnValue = strToParam("PlaySound FILE, 'snd ' rsrc id [,start][,stop]");
			return;
		}
	}


	paramtoPString( pp, 0, (char *)&temp );		
	start	= paramtoNum( pp, 2 );				
	stop 	= paramtoNum( pp, 3 );
  
	saveResFile = CurResFile(); 				/* save the current res file id */
	resFile = OpenResFile( temp );
  
	if (resFile == -1) {
		putResult(	pp, "can't open resource file" );
		return;
	}
 
	UseResFile( resFile );						/* in case it was already open	*/

	paramtoPString( pp, 1, (char *)&temp );

	sndHdl = Get1NamedResource( 'snd ', &temp );	/* getting the resource ok		*/
 	
	if(!sndHdl){
   		putResult( pp, "no such snd" );
		UseResFile( saveResFile );					
   		return;
	}
	
   	BEGINXSOUND( pp, NIL );
	DetachResource( sndHdl );
    PlayRunSound( sndHdl, start, stop );
 	DisposHandle( sndHdl );
    ENDXSOUND( pp );
    
	UseResFile( saveResFile );	
}
 
 
 
void	PlayRunSound( sndHdl, start, stop )
	Handle 	sndHdl;
	long 	start;
	long	stop;
/**********************************
* st[0] determines whether we have a
* format 2 or a format 1 'snd '
* resource.	
*
* currently use the 44K sampler.
* Refer to Inside Macintosh Vol V p.494
* for the format of a format 2 'snd ' resource
**********************************/
{
    short 			*st;
    long 			*lt;
    SndCommand  	sndCmd;
   	long 			idx;
	SndChannelPtr	theChan	= NIL;
	OSErr   		err;
   	
	sndCmd.cmd 	= availableCmd;
	
	if( err = SndControl( sampledSynth, &sndCmd ) ) return;
	if( err = SndNewChannel( &theChan, sampledSynth, initSRate44k, NIL ) ) return; 
 	 
 	HLock( sndHdl );
   	st = (short *)*sndHdl;
	
	sndCmd.cmd 		= bufferCmd;
   	sndCmd.param1	= 0L;
	if( *st == 2){
		idx 	= 6L + ((long)st[2] << 3);	/* offset to start of commands 			*/
		lt 		= *sndHdl + idx;
   		sndCmd.param2 = (long)lt;
   		
   		if( start )
   			lt[0]	= (long)*sndHdl + idx + start;		
   		
   		if( stop )
     		lt[1]	= stop  - start;	
     			
		err = SndDoCommand( theChan, &sndCmd, FALSE );
  		err = SndDisposeChannel( theChan, FALSE );
	}
	
	HUnlock( sndHdl );

}
 
 
 
 
void putResult( XCmdPtr pp , char *msg )
/***************************
*
**************************/
{
   if (pp->returnValue)  			
		DisposHandle(pp->returnValue);	
 
   pp->returnValue = (Handle)NewHandle(1 + strlen(msg) );
   strcpy( *(pp->returnValue), msg);
}
 


Handle	strToParam( char	*str )
/***************************
* Given a pointer to a string,
* copy that string into a handle
* and return the handle.
*
* The input and output strings
* are both null-terminated
*
***************************/
{
	Handle	outH = NIL;
	long	len	= 0;
	
	len = strlen( str );
	if( len )
		if( outH = NewHandle( len ) )
			BlockMove( str, *outH, len + 1 );
	
	return( outH );
}



void	paramtoPString( XCmdPtr pp, short i, char *str )
/************************
* Given an index into the 
* parameter list, convert the data
* in the Block into a pstring
*
* This sequence is so common in XCMDs
* that it makes sense to make it a subr.
* This is a generic routine that you can
* use in any xcmd.
************************/
{
	if( pp->params[i] ){
		HLock( pp->params[i] );
		ZEROTOPAS( pp, (Ptr)*(pp->params[i]), (StringPtr)str );
		HUnlock( pp->params[i] );
	}
	else
		*str = '\0';
}



long 	paramtoNum( XCmdPtr pp, short i )
/************************
* Given a Block to an input
* argument in the paramBlk
* return an integer representation
* of the data.
*
************************/
{
	char		theStr[32];
	
	theStr[0] = '\0';
	
	if( pp->params[i] ){
		HLock( pp->params[ i ] );
		ZEROTOPAS( pp, (char *)*(pp->params[ i ]), theStr );
		HUnlock( pp->params[ i ] );
		return STRTOLONG( pp, theStr );
	}
	return 0L;
}

