#include "MacTypes.h"
#include "X_includes.h"

/* size of a string and a list differ in interp. & compiled code */

#if _INTERPRETER
	typedef Nat2 Index;
#else
	typedef Nat4 Index;
#endif
	
#define LT			-1
#define EQ			0
#define GT			1

/* prototypes for functions in this file */

Nat2 U_binlist_2D_sort( C_list *, C_list ** );
Nat2 U_binlist_2D_find( C_list *, C_string *, C_boolean **, C_integer ** );
void binlist_find( C_list *, C_string *, Bool *, Index * );
Int2 compare_strings( C_string *, C_string * );

#if _INTERPRETER

/* common table for supplied functions */

ProcPtr * ct;

/* main loads primitives into prograph interpreter's primitive table */

void main( table )

ProcPtr * table;				/* common table */

{
	asm 
	{ 
		move.l a4, -(sp) 		/* move A4 on to the stack */
		move.l a0, a4			/* move address of code resource to A4 */
	}
	
	ct = table;					/* set up common table for shared functions */
	
	AddPrimitive( 1, 0x0101, PF_USER, 0, &U_binlist_2D_sort );
	AddPrimitive( 2, 0x0202, PF_USER, 0, &U_binlist_2D_find );


	asm { move.l (sp)+, a4 }	/* move saved value back into A4 */
}

#endif

/* input list of strings, output sorted list of unique strings */

Nat2 U_binlist_2D_sort( inList, outList )

C_list *inList;				/* unsorted list */
C_list **outList;			/* new sorted list */

{
Nat2 inarity;				/* number of inputs */
Nat2 outarity;				/* number of outputs */
register Index size;		/* size of unsorted list */
register Index index;		/* index in unsorted list */
Index sortIndex;			/* index in sorted list */
C_string *curString;		/* current string of inList */
C_list *sortList;			/* the sorted list */
Bool found;					/* TRUE if string found */

#if _INTERPRETER
	GETARITY( inarity, outarity )

	if ( inarity != 1 || outarity != 1 )
		return PRIMERR_ARITY;

	if ( !IsType( (C_object *)inList, C_LIST )  )
		return PRIMERR_TYPE + 1;
#endif

	sortList = MakeC_list( 0 );
	
	for ( index = 0, size = (**inList).length; index < size; index++ )
	{
		curString = (C_string *)((**inList).data[index]);
		
#if _INTERPRETER
		if ( !IsType( (C_object *) curString, C_STRING ) )
		{
			DecUse( sortList );
			return PRIMERR_TYPE + 1;
		}
#endif

		binlist_find( sortList, curString, &found, &sortIndex );
		
		if ( found == FALSE )
			ListInsertSlot( sortList, sortIndex, TRUE, (C_object *)curString );
	}
	
	*outList = sortList;
	return PCF_TRUE;
}


/* primitive to find string in alpha sorted list by matching string */

Nat2 U_binlist_2D_find( input1, input2, output1, output2 )

C_list *input1;			/* sorted list of strings */
C_string *input2;		/* string for key comparison */
C_boolean **output1;	/* boolean is TRUE if found */
C_integer **output2;	/* index found or new position */

{
Nat2 inarity;			/* number of inputs */
Nat2 outarity;			/* number of outputs */
Bool found;				/* TRUE if found */
Index index;			/* index found or new position */

#if _INTERPRETER
	GETARITY( inarity, outarity );
	
	if ( inarity != 2 || outarity != 2 )
		return PRIMERR_ARITY;

	if ( !IsType( (C_object *) input1, C_LIST )  )
		return PRIMERR_TYPE + 1;

	if ( !IsType( (C_object *) input2, C_STRING )  )
		return PRIMERR_TYPE + 2;
#endif
		
	binlist_find( input1, input2, &found, &index );
	*output1 = MakeC_boolean( found );
	*output2 = MakeC_integer( (Nat4)index + 1 );
	return PCF_TRUE;
}

/* find string in alpha sorted table by matching string */

void binlist_find( list, compString, found, index )

C_list *list;				/* sorted list */
C_string *compString;		/* string for key comparison */
Bool *found;				/* TRUE if found */
Index *index;				/* index found or index to put at */

{
register Int4 hit;			/* table hit point */
register Int4 lo;			/* table low bound */
register Int4 hi;			/* table high bound */
register C_string **table;	/* pointer to start of table */
register Bool compare;		/* result of string comparison */

	table = (C_string **)((**list).data);
	hi = (Int4)((**list).length) - 1;
	lo = 0;
	
	while ( lo <= hi )
	{
		hit = ( lo + hi) >> 1;
		compare = compare_strings( compString, table[hit] );
		
		if ( compare == LT )
			hi = hit - 1;
		else if ( compare == GT )
			lo = hit + 1;
		else
		{
			*index = hit;
			*found = TRUE;
			return;
		}
	}
	
	*index = lo;
	*found = FALSE;
}

/* compare_strings - compare two strings. return LT -1, EQ 0, GT 1  */	
					 
Int2 compare_strings( string1, string2 )

C_string *string1;	/* first string */
C_string *string2;	/* second string */

{
register Index ind1;	/* length of 1st string */
register Index ind2;	/* length of 2nd string */
register Nat1 *ptr1;	/* pointer into 1st string */
register Nat1 *ptr2;	/* pointer into 2nd string */

	for( ind1 = (**string1).length, 
		 ind2 = (**string2).length,
		 ptr1 = (**string1).text, 
		 ptr2 = (**string2).text ;
		 ind1 && ind2 ; 
		 ind1--, ind2--, ptr1++, ptr2++ )
	{
		if ( *ptr1 < *ptr2 )
			return LT;
		else if ( *ptr1 > *ptr2 )
			return GT;
	}
	
	if ( ind1 )
		return GT;
	else if ( ind2 )
		return LT;
	else
		return EQ;
}

