{ ----------------------------------------------------------------------------

  AShareNameINIT
  
    Type        INIT
    ID          16056
    Version     1.5
								Wednesday, December 5, 1990 9:53:37 AM
								1990 by Mark Gavin
								All Rights Reserved
              
		Function		AShareNameINIT gets the AppleShare userName from the AppleShare Prep file
								in the system folder and places the username in the chooser.
								
		Revisions		
					1.1		It turns out the AppleShare Prep file saves a name and server history 
								when the user name changes.  The number of entries in the file is
								stored in the first word of the BMLS resource.  Get the last name in
								the list of blocks.
								
					1.2		Logical block size within the resource is also variable according
								to how many server disks are mounted.  The block size is in word
								two of the resouce and points to the next blocksize.  Walk the list
								of blocks to the last block and use that user name.
								
					1.3		It is possible for a user name to be nil.  If nil is in the Chooser
								name, the Responder will ask for the name of the Mac at each launch.
								Set AShareNameINIT not to use a user name which is nil.
								
					1.4		Close resource file on emergency exit.
					
					1.5		Add the deadStrip boolean and IF statement for the About
								string buried in the code segment.

  ---------------------------------------------------------------------------- }
{$Z+} { Identify all routines to the Linker as
				external.  If this is not used then
				ENTRYPOINT must be placed in the
				INTERFACE section }
    
  UNIT _AShareNameINIT;

INTERFACE

	USES
		Types, Memory, ToolUtils,
		OSUtils, Resources, PaslibIntf, Packages;

	TYPE
		intPtr = ^integer;
		{ used to retrieve an integer from the resource }
	

{$R-}		{ Turn Off Range Checking }
{$OV-}	{ Ignore Arithmetic Overflows }

IMPLEMENTATION

	PROCEDURE AShareNameINIT;	FORWARD;


	PROCEDURE ENTRYPOINT;

		BEGIN
			AShareNameINIT;
		END;

{ ---------------------------------- }

  PROCEDURE AShareNameINIT;
	
    VAR
			str								: Str255;
			
			volNamePtr				: StringPtr;
			
			theStringPtr,
			theNamePtr				: Ptr;
			
			{ AppleShare Prep resHdl }
			ASPResHandle,
			chooserStringHdl	: Handle;
			
			ASPHandleSize			: longint;
			
			index,
			blocksize,
			resOffset,
			offsetMultiplier,
			stringLength,
			refNum,
			vRefNum,
			oldResFileRef			: integer;
			
			anError						: OSErr;
			
			deadStrip					: Boolean;

	{ ------------------------------------ }			
	
		PROCEDURE CloseAndExit;
			
			BEGIN	
				CloseResFile(refNum);
				UseResFile(oldResFileRef);
				EXIT(AShareNameINIT);			
			END;
	
	{ ------------------------------------ }			

		BEGIN	
			oldResFileRef := CurResFile;
			
			deadStrip := False;
			IF deadStrip THEN BEGIN
				str := 'AShareNameINIT 1.5 by Mark Gavin';
				str := compdate;
			END;

			{ GetVol returns the name of the
				default volume in volNamePtr and
				its volume reference number in vRefNum. }
				
			volNamePtr := StringPtr(NewPtr(256));
				IF MemError <> noErr THEN
					EXIT(AShareNameINIT);

			anError := GetVol(volNamePtr, vRefNum);
				IF anError <> noErr THEN
					EXIT(AShareNameINIT);

			DisposPtr(Ptr(volNamePtr));
			{ Ptr only needed for GetVol }

			str := 'AppleShare Prep';
			{ StripAddress used per Tech Note 232 - bug in OpenRFPerm }
			refNum := OpenRFPerm(StringPtr(StripAddress(@str))^, vRefNum, fsRdPerm);
				IF ResError <> noErr THEN
					EXIT(AShareNameINIT);	
			
			ASPResHandle := GetResource('BMLS', 2447);
				IF ResError <> noErr THEN
					CloseAndExit;	
			HNoPurge(ASPResHandle);
								
			chooserStringHdl := GetResource('STR ', -16096);
				IF ResError <> noErr THEN
					CloseAndExit;	
			HNoPurge(chooserStringHdl);
			
			ASPHandleSize := GetHandleSize(ASPResHandle);
			IF MemError <> noErr THEN
					CloseAndExit;
					
			{ Make sure there is data in the handle }
			IF ASPHandleSize < 55 THEN
					CloseAndExit;	
			
			offsetMultiplier := intPtr(ASPResHandle^)^;
			IF offsetMultiplier < 1 THEN
					CloseAndExit;	

			blockSize := intPtr(ORD4(ASPResHandle^) + 2)^;
			IF blockSize < 1 THEN
					CloseAndExit;	
		
			resOffset := 2;
			
			FOR index := 1 TO (offsetMultiplier - 1) DO
				BEGIN
					resOffset := resOffset + blockSize;
					blockSize := intPtr(ORD4(ASPResHandle^) + resOffset)^;
					IF blockSize < 1 THEN
							CloseAndExit;
				END;

			resOffset :=  resOffset + 20;
			{ offset to string within block }

			theNamePtr := Ptr(ORD4(ASPResHandle^) + resOffset);
			
			IF theNamePtr^ = 0 THEN
				CloseAndExit;{ exit on nil name }
					
			theStringPtr := chooserStringHdl^;

			BlockMove(theNamePtr, theStringPtr, 32);
	
			ChangedResource(chooserStringHdl);
			WriteResource(chooserStringHdl);

			HPurge(chooserStringHdl);
			HPurge(ASPResHandle);
			
			CloseResFile(refNum);
			
			UseResFile(oldResFileRef);

  	END;
	END.

