; File:  stuff.sch.  (Eval third.)
;
(define print 
  (lambda (x)
    (display x)
    (newline)))
;
(define time-it
  (lambda (expr)
    (display "available space:  ")
    (print (gc))
    (let* ((start-time (time))
           (value (eval expr))
           (end-time (time)))
      (display "time:  ")
      (print (/ (- end-time start-time)
                60))
      value)))
;
; Church numeral factorial.  (Original recipe.)
;
(define com-fact
  (applicative-order-y 
   (lambda (fact)
     (lambda (n)
       (((com-if (com-zero? n))
         (lambda () com-one))
        (lambda () ((com-mul n) 
                    (fact (com-pred n)))))))))
;
(church->number 
 (com-fact (number->compact-church 5)))
;
; Recursionless fact a la pred style. 
;
(define make-fact-tuple 
  (lambda (tuple)
    ((com-cons ((com-mul (com-cdr tuple))
                (com-car tuple)))
     (com-succ (com-cdr tuple)))))
;
(define initial-fact-tuple
  ((com-cons com-one)
   com-one))
;
(define pred-style-fact
  (lambda (n)
    (com-car 
     (((unravel n) 
       make-fact-tuple) 
      initial-fact-tuple))))
;
(church->number 
 (pred-style-fact (number->compact-church 5)))
;
; Recursionless fact a la (fact fact ...) trick.

(define recursionless-fact
  (lambda (f n)
    (if (zero? n)
        1
        (* n (f f (1- n))))))
;
(recursionless-fact recursionless-fact 5)
;
(define recursionless-fact-curried
  (lambda (f)
    (lambda (n)
      (if (zero? n)
          1
          (* n ((f f) (1- n)))))))
;
((recursionless-fact-curried 
  recursionless-fact-curried) 5)
;
; Consumer oriented numerals.
;
(define consumer-succ
  (lambda (x)
    (lambda (y)
      x)))
;
(define consumer-pred
  (lambda (x)
    (x 0)))
;
(define consumer->number
  (lambda (n)
    (if (and (number? n)
             (zero? n))
        0
        (1+ (consumer->number (n 0))))))
;
(define number->consumer 
  (lambda (n)
    (if (zero? n)
        0
        (consumer-succ 
         (number->consumer (1- n))))))
;
(consumer->number (number->consumer 5))
;
; Church arithmetic test cases.
;
(define normalize-church
  (lambda (numeral)
    (((unravel numeral) com-succ) com-zero)))
;
(define cons-stats
  (let ((counter 0))
    (lambda (message)
      (case message
        ((reset-counter)
         (begin (set! counter 0)
                counter))
        ((counter) counter)
        ((get-function)
         (lambda (x)
           (lambda (y)
             (begin
              (set! counter (1+ counter))
              (lambda (selector)
                ((selector x) y))))))
        ((?) '(reset-counter counter get-function ?))
        (else 
         (error "cons-stats:  bad message" message))))))
;
(define unravel-stats
  (let ((counter 0))
    (lambda (message)
      (case message
        ((reset-counter)
         (begin (set! counter 0)))
        ((counter) counter)
        ((get-function)
         (lambda (n)
           (lambda (f)
             (lambda (x)
               (begin (set! counter (1+ counter))
                      ((n f) x))))))
        ((?) '(reset-counter counter get-function ?))
        (else (error "unravel-stats:  bad message" 
                     message))))))
;
(define partial-unravel-stats
  (lambda (message)
    (case message
      ((reset-counter)
       (begin (set! counter 0)))
      ((counter) counter)
      ((get-function)
       (lambda (n)
         (lambda (f)
           (begin (set! counter (1+ counter))
                  (n f)))))
      ((?) '(reset-counter counter get-function ?))
      (else (error "partial-unravel-stats:  bad message" 
                   message)))))
;
(define stats 
  (lambda (expr)
    (begin
     (cons-stats 'reset-counter)
     (unravel-stats 'reset-counter)
     (partial-unravel-stats 'reset-counter)
     (let ((result (time-it expr)))
       (begin 
        (display "conses:  ")
        (print (cons-stats 'counter))
        (display "unravels:  ")
        (print (unravel-stats 'counter))
        (display "partial-unravels:  ")
        (print (partial-unravel-stats 'counter))
        result)))))
;
(define slow-three
  (let* ((one-hundred (number->compact-church 100))
         (ninety-nine (number->compact-church 99))
         (slow-one ((com-sub one-hundred) ninety-nine)))
    ((com-add slow-one)
     ((com-add slow-one)
      slow-one))))
(time-it '(church->number slow-three))
(define normalized-three
  (normalize-church slow-three))
(time-it '(church->number normalized-three))
;
(define two (number->compact-church 2))
(define four (number->compact-church 4))
(define eight (number->compact-church 8))
(define thirty-two (number->compact-church 32))
(define sixty-four (number->compact-church 64))
;
(define try-func ; test functions as computers
  (lambda (func)
    ((func ((func sixty-four) eight))
     ((func ((func thirty-two) two))
      four))))
;
(time-it '(try-func com-pow))
(time-it '(try-func com-mul))
(time-it '(try-func com-add))
(time-it '(try-func com-sub))
(time-it '(try-func com-quo))
;
(set! com-cons (cons-stats 'get-function))
(set! unravel (unravel-stats 'get-function))
(set! partial-unravel 
      (partial-unravel-stats 'get-function))
;
(define two (lambda (f) (lambda (x) (f (f x)))))
(define three (lambda (f) (lambda (x) (f (f (f x))))))
(define four 
  (lambda (f) (lambda (x) (f (f (f (f x)))))))
(define nine 
  (lambda (f) 
    (lambda (x) (f (f (f (f (f (f (f (f (f x))))))))))))
(define twelve 
  (lambda (f)
    (lambda (x)
      (f (f (f (f (f (f (f (f (f (f (f (f x)))))))))))))))
(define sixteen
  (lambda (f)
    (lambda (x)
      (f (f (f (f (f (f (f (f (f (f (f (f (f (f (f (f 
       x)))))))))))))))))))
;
(stats '(church->number (number->church 8))) 
(stats '(church->number normalized-three))
;
(define foo (stats '(com-pred nine)))
(stats '(church->number foo)) ; pred test
(set! foo (stats '((com-pow two) three)))
(stats '(church->number foo)) ; pow test
(set! foo (stats '((com-mul two) four)))
(stats '(church->number foo)) ; mul test
(set! foo (stats '((com-add four) four)))
(stats '(church->number foo)) ; add test
(set! foo (stats '((com-sub twelve) four)))
(stats '(church->number foo)) ; sub test
(set! foo (stats '((com-quo sixteen) two)))
(stats '(church->number foo)) ; quo test
'done