/**************************
OQDKolorize.c
created by Kirk Chase 3/29/91
***************************/
#include "OQDKolorize.h"

#define NIL 0L

#pragma mark Information
/************************************/
/* routines:
	 Ptr NewBitMap(BitMap *bm, Rect *r);
	 void DisposBitMap(BitMap *bm);
	 GrafPtr NewOffScreen(Rect *theBounds);
	 void DisposOffScreen(GrafPtr offPort);
	 void KolorizeMix(Rect *theRect,int *pallete);
	 void Kolorize(Rect *theRect, int theColor);
	 void KolorizeCDEF(ControlHandle theControl, int type,
				int buttonColor, int indicatorColor);
	 void void PaintMixedRect(Rect *theRect, int *pallete);
	*/
/************************************/

/************************************/
/* OQDKolorize Variables */
Pattern oqdMask4[4] = {	0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00,
						0x00, 0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x00, 0xAA,
						0x00, 0x55, 0x00, 0x55, 0x00, 0x55, 0x00, 0x55,
						0x55, 0x00, 0x55, 0x00, 0x55, 0x00, 0x55, 0x00 };
/************************************/

/************************************/
/* Routines */
/************************************/

Ptr NewBitMap(bm, r)
BitMap *bm;
Rect *r;
/************************************/
/* Ptr NewBitMap()
	creates a bitmap */
/************************************/
{ /* NewBitMap() */
bm->rowBytes = ((r->right - r->left + 15) / 16) * 2;
bm->bounds = *r;
bm->baseAddr = NewPtr(bm->rowBytes * (long) (r->right - r->left));
if (MemError() != noErr) return (0L);
else return (bm->baseAddr);
} /* NewBitMap() */

void DisposBitMap(bm)
BitMap *bm;
/************************************/
/* void DisposBitMap()
	disposes of a bitmap */
/************************************/
{ /* DisposBitMap() */
DisposPtr(bm->baseAddr);
SetRect(&bm->bounds,0,0,0,0);
bm->rowBytes=0;
bm->baseAddr=0L;
} /* DisposBitMap() */

GrafPtr NewOffScreen(theBounds)
Rect *theBounds;
/************************************/
/* GrafPtr NewOffScreen()
	creates an offscreen grafport */
/************************************/
{ /* NewOffScreen() */
GrafPtr oldPort, offPort;

/* allocate port memory */
offPort = (GrafPtr) NewPtr(sizeof(GrafPort));
if (MemError() != noErr)
	return (0L);

/* open port and set port variables */
GetPort(&oldPort);
OpenPort(offPort);
offPort->portRect = *theBounds;

/* create bitmap to store port's bits */
if (NewBitMap(&(offPort->portBits), theBounds) == (0L)) {
	ClosePort(offPort);
	SetPort(oldPort);
	DisposPtr((Ptr) offPort);
	return(0L);
	}

/* create necessary regions */
RectRgn(offPort->clipRgn, theBounds);
RectRgn(offPort->visRgn, theBounds);

/* clear bits to white */
EraseRect(theBounds);

/* reset back to original port */
SetPort(oldPort);

/* return offscreen port */
return(offPort);
} /* NewOffScreen() */

void DisposOffScreen(offPort)
GrafPtr offPort;
/************************************/
/* void DisposOffScreen()
	destroys an offscreen grafport */
/************************************/
{ /* DisposOffScreen() */
ClosePort(offPort);
DisposBitMap(&(offPort->portBits));
DisposPtr((Ptr) offPort);
} /* DisposOffScreen() */

void KolorizeMix(theRect, pallete)
Rect *theRect;
int *pallete;
/************************************/
/* void KolorizeMix(theRect, pallete)
	given a rectangle and pallete of  
	4 OQD colors, colorize rectangle */
/************************************/
{
GrafPtr offScreen, oldPort, copyPort;
int i;
PenState oldPen;

/* save old values */
GetPenState(&oldPen);
GetPort(&oldPort);

/* create offscreen ports for kolorizing */
offScreen = NewOffScreen(theRect);
if (offScreen == NIL) return;
copyPort = NewOffScreen(theRect);
if (copyPort == NIL) return;

SetPort(offScreen); /* do work on offscreen */

/* Copy only significant bits (black pixels)
	by using the same bits we wish to copy
	as the mask to the copyPort to work with later */
CopyMask(&(oldPort->portBits), &(oldPort->portBits), &(copyPort->portBits),
	theRect, theRect, theRect);

EraseRect(theRect);
/* Set Proper Pen Mode to clear bits */
for (i=0; i<4; i++) { /* loop through colors/masks */
	CopyMask(&(copyPort->portBits), &(copyPort->portBits), &(offScreen->portBits),
		theRect, theRect, theRect); /* place original copy in off screen */
		
	PenMode(notPatBic);  /* paint it with ith pattern, clearing unneeded bits */
	PenPat(&(oqdMask4[i]));
	PaintRect(theRect);
	
	PenMode(patCopy);
	ForeColor(pallete[i]); /* kolorize with ith color */
	CopyBits (&(offScreen->portBits),&(oldPort->portBits), theRect,theRect, srcOr, 0L);
	ForeColor(blackColor);
	}

/* reset old values */	
SetPort(oldPort);
ForeColor(blackColor);
SetPenState(&oldPen);

/* dispose of memory */
DisposOffScreen(offScreen);
DisposOffScreen(copyPort);
} /* KolorizeMix() */

void Kolorize(theRect, theColor)
Rect *theRect;
int theColor;
/************************************/
/* void Kolorize()
	given a rectangle and an OQDcolor  
	colorize a rectangle */
/************************************/
{
/* set forecolor to color selected */
ForeColor(theColor);

/* kolorize */
CopyMask(&(thePort->portBits), &(thePort->portBits), &(thePort->portBits), 
	theRect, theRect, theRect);
ForeColor(blackColor); /* reset */
} /* Kolorize() */

void KolorizeCDEF(theControl, type, buttonColor, indicatorColor)
ControlHandle theControl;
int type;
int buttonColor, indicatorColor;
/************************************/
/* void KolorizeCDEF()
	given a standard control (button,
	radio, or checkbox determined by
	proc type in type parameter) this
	will colorize the button and the
	indicator */
/************************************/
{
Rect tempRect;
PenState oldPen;

/* get old values and set up normal pen */
GetPenState(&oldPen);
PenNormal();

/* colorize whole control with button color */
ForeColor(buttonColor);
CopyMask(&(thePort->portBits), &(thePort->portBits), &(thePort->portBits), 
	&(**theControl).contrlRect,
	&(**theControl).contrlRect,
	&(**theControl).contrlRect);
ForeColor(blackColor);

/* if indicator color is the same
	as the button color, or it's a button
	or it's not set, then we're done */
if ((buttonColor == indicatorColor) 
		|| (!type)
		|| (!GetCtlValue(theControl))) { /* simple colorizing */
	SetPenState(&oldPen);
	return;
	}

/* not done, need to draw indicator */
/* get initial indicator rect */
tempRect = (**theControl).contrlRect;
tempRect.bottom = tempRect.bottom - ((tempRect.bottom - tempRect.top - 12) / 2);
tempRect.top = tempRect.bottom - 12;
tempRect.left +=2;
tempRect.right = tempRect.left + 12;

ForeColor(indicatorColor);
/* colorize checkbox or radio button */
if (type == checkBoxProc) {
	InsetRect(&tempRect, 1, 1);
	MoveTo(tempRect.left, tempRect.top);
	LineTo(tempRect.right, tempRect.bottom);
	MoveTo(tempRect.left , tempRect.bottom);
	LineTo(tempRect.right, tempRect.top - 1);
	}
else if (type == radioButProc) {
	InsetRect(&tempRect, 3, 3);
	PaintOval(&tempRect);
	}

/* reset old values */
SetPenState(&oldPen);
ForeColor(blackColor);
} /* KolorizeCDEF() */

void PaintMixedRect(theRect, pallete)
Rect *theRect;
int *pallete;
/************************************/
/* void PaintMixedRect()
	given a rectangle and a pallete of
	4 OQD colors, it paints a mixed
	rectangle to give the illusion
	of another color */
/************************************/
{
int i;
PenState oldPen;

/* get old states and set up pen */
GetPenState(&oldPen);
PenNormal();
PenMode(patOr);

/* paint ith pattern with ith color */
for (i=0; i<4; i++) {
	PenPat(&(oqdMask4[i]));
	ForeColor(pallete[i]);
	PaintRect(theRect);
	}

/* reset old values */
ForeColor(blackColor);
SetPenState(&oldPen);
} /* PaintMixedRect() */
