/*
 * Login Dialog.c - Dialog for User Login.
 *
 * Copyright (c) 1988 by Bill Schilit.
 *
 * Edit History:
 *
 *  April 23, 1988      Schilit    Created
 *  May 9, 1988         Schilit    Clean up
 *
 */

/* Includes MacHeaders */

#include "Login Dialog.h"

/* Prototypes */

static int 
TELengthCheck(TEPtr te,char c,int maxLen);

static void
TECpyText(TEHandle teH,Ptr p);

pascal Byte 
LoginFilter(DialogPtr dPtr,
        EventRecord *ePtr,short *iHit);



/*
 * LDialogStg contains the global vars used 
 * by the filter proc and user item procs in
 * our login dialog. A pointer to the 
 * LDialogStg is stored in the window refcon
 * of the dialog window.
 */

typedef struct {
    TEHandle passTeH;
} LDialogStg, *LDialogStgPtr;


/*
 * static Byte 
 *  LoginFilter(DialogPtr theDialog,
 *              EventRecord *theEvent,
 *              int *itemHit);
 *
 * Modal dialog filter to echo bullet 
 * ('\245') instead of the user's password
 * and to limit the number of characters in
 * both the user name and password edit
 * records.
 */

static pascal Byte
LoginFilter(theDialog,theEvent,itemHit)
DialogPtr theDialog;
EventRecord *theEvent;
short *itemHit;
{
    register char c;
    int field,tooBig;
    TEPtr tePtr;
    LDialogStg *ldStg;
    
    /* we're only interested in keyboard 
     * events. If not a key, let modal  
     * process as usual
     */
     
    if (theEvent->what != keyDown && 
        theEvent->what != autoKey)
            return(false);
    
    /* fetch the character from the
     * event message 
     */    

    c = theEvent->message & charCodeMask;
    
    /* Check for CR and convert to OK button.
     * Check for TAB and let it pass.
     */
     
    if (c == CR) {
        *itemHit = OK;
        return(true);
    }
    
    if (c == '\t')
        return(false);

    /* make sure the edit text item is one
     * we are interested in and check to see
     * if the length is not too large.
     */
     
    field = 
      ((DialogPeek) theDialog)->editField+1;
    
    tePtr = 
      *(((DialogPeek) theDialog)->textH);

    /* User is typing in the nameItem -- 
     * our only interest is the size 
     */
    
    if (field == nameItem) {
        tooBig = 
          TELengthCheck(tePtr,c,MAXNAME);

     /* give a beep if too big, and return
      * TRUE to ignore the event.
      */
      
        if (tooBig)
            SysBeep(1);   
        return(tooBig);   
    }
    
    /* If typing into the password, check the
     * size, then diddle the character so 
     * bullet (\245) shows up instead of what
     * the user typed.
     */
     
    if (field == passwdItem) {
        if (TELengthCheck(tePtr,c,MAXPWD)) {
            SysBeep(1);
            return(true);
        }
        
    /* Insert the char into our private
     * password text edit record. First
     * set the text selection so action
     * mimicks exactly what user is 
     * selecting and typing in passwdItem
     * text edit field.
     */
    
        ldStg = (LDialogStg *) 
          GetWRefCon(theDialog);
          
        if (ldStg == 0)
            return(false);
            
        TESetSelect(tePtr->selStart,
                    tePtr->selEnd,
                    ldStg->passTeH);
                    
        TEKey(c,ldStg->passTeH);
    
        /* unless BS or DEL, replace the
         * password character with bullet 
         */

        if (c != DEL && c != BS)
            theEvent->message = '\245' | 
            (theEvent->message & 
              ~charCodeMask);
        return(false);       /* return ok */
    }
    
    return(false);     /* all other items */
    
}    

/*
 * static int 
 * TELengthCheck(tePtr te,char c,int maxLen)
 *
 * Check that adding character c to the text
 * edit te does not cause more than maxLen
 * chars in the text edit item:
 *
 * 1) If delete or backspace then length will
 *    decrease so ok.
 * 2) If a selection range of 1 or more chars 
 *    then same as above.
 * 3) Finally just check the length of the 
 *    edit item. 
 *
 *
 * Returns: FALSE if OK, TRUE if too large.
 *
 */
 
static int
TELengthCheck(te,c,maxLen)
TEPtr te;
char c;
int maxLen;
{

   /* this char a del or bs */
   /*  if so, does not increase */
 
    if (c == DEL || c == BS)
        return(false);       

    /* selected a region? */
    /* if so, then does not increase */
    
    if (te->selStart < 
        te->selEnd)           
        return(false);      
   
   /* else will insert, check length */
      
    if (te->teLength < maxLen) 
        return(false);
    
    return(true);
}

/*
 * static void TECpyText(TEHandle teH,Ptr p)
 *
 * Fetch the text from the text edit handle
 * and store as a pascal string in Ptr p.
 *
 * NB: This only works if the TE text is less
 * than 255 characters (a pascal string 
 * limit) so be careful.
 *
 */
 
static void
TECpyText(teH,p)
TEHandle teH;
Ptr p;
{
    p[0] = (unsigned char) (*teH)->teLength;
    BlockMove(*(*teH)->hText,&p[1],p[0]);
}

/*
 * LoginDialog(char *uName,*uPassword)
 *
 * Perform a login dialog and return the user
 * name and password in uName and uPassword.
 *
 * The dialog has a special filter procedure
 * which echos bullet characters in the
 * password field.
 * 
 * The length of the username and password
 *  are limited to MAXNAME and MAXPWD.
 * 
 * Note: we do not issue ParamText() since 
 * this affects other dialogs on the screen.
 * 
 */
 
LoginDialog(uName,uPasswd)
char *uName,*uPasswd;
{
    DialogPtr d;
    short itemHit;
    Rect aRect;
    int theKind;
    Handle nameHdl,okHdl;
    Boolean okOK = false;
    LDialogStg LDStg;
    

    d = GetNewDialog(LOGIN_DLOG,
                     (Ptr) 0,(Ptr) -1);
                     
    if (d == 0)
        return;
  
    /* make it the current port */          
  
    SetPort(d);         
    
    /* Make an offscreen rect for the text
     * edit to hold the plain text of the
     * entered password.
     * 
     * The dialog edit text for the password
     * will get ҥ for each character typed.
     */
     
    SetRect(&aRect,0,0,1,1);
    OffsetRect(&aRect,
                d->portRect.right,
                d->portRect.bottom);
    
    LDStg.passTeH = TENew(&aRect,&aRect);
    
    /* Set the window data to be a pointer
     * to storage needed by filter procedure.
     */
     
    SetWRefCon(d,(long) &LDStg);
    
    /* Get handles for Name field and 
     * OK Button 
     */
        
    GetDItem(d,nameItem,&theKind,
             &nameHdl,&aRect);
    GetDItem(d,okItem,&theKind,
             &okHdl,&aRect);
    
    ShowWindow(d);
    
    while (!(okOK && itemHit == okItem)) {
        
        /* Set okOK to true if password and
         * name fields both have more than
         * one character. Enable/Disable the
         * OK button accordingly.
         */
         
        GetIText(nameHdl,uName);
    
        okOK = 
          (*LDStg.passTeH)->teLength > 0 &&
           uName[0] > 0;
                
        HiliteControl((ControlHandle) okHdl,
                      okOK ? 0 : 255);
        
        ModalDialog(LoginFilter,&itemHit);
    }
    
    /* Store the password and username
     *  for the caller then clean up.
     */
    
    TECpyText(LDStg.passTeH,uPasswd);
    GetIText(nameHdl,uName);

    DisposDialog(d);    
    TEDispose(LDStg.passTeH);
}


