/*
* Vital Signs, version 1.0. 
*
* A "do everything" desk accessory, written
* for MacTutor magazine by Ted C. Johnson.
* 
* Feel free to use this code for your own
* desk accessories.
*
* Created with Lightspeed C, version 2.15.
*
* Last modification:  Wed, March 9, 1988.
*
* Copyright 1988 by Ted C. Johnson.
* All Rights Reserved.
*/ 
 


#include <MacTypes.h>
#include <DeskMgr.h>
#include <DeviceMgr.h>
#include <DialogMgr.h>
#include <EventMgr.h>
#include <MemoryMgr.h>
#include <MenuMgr.h>
#include <FontMgr.h>
#include <FileMgr.h>
#include <OSUtil.h>
#include <TextEdit.h>
#include <QuickDraw.h>
#include <WindowMgr.h>
#include <ToolboxUtil.h> 
#include <IntlPkg.h>
#include <HFS.h>
#include <pascal.h>



#define	NIL			0L			 

#define YES 		1
#define NO			0

#define FSFCBLen   0x03F6 


/*
*These are the five different message types that
*can be sent to a desk accessory.		
*/

#define	drvrOpen	0 
#define	drvrPrime	1 
#define	drvrControl	2
#define	drvrStatus	3
#define	drvrClose	4



/*These constants used to decide which PICT to draw.*/

#define MAC_PLUS	1
#define BIG_HEART	2
#define SMALL_HEART	3


/*
*The following defines are for the "base id"'s of our
*DA's owned resources.  
*
*Lightspeed C sets our DRVR ID ot 12, so the owned
*resources for this DA start at -16000.  Therefore
*our base ids should start at 0.  However, for the
*sake of clearity, I start with a base id of 1 
*(defining something to be "1" just SEEMS t invite
*less speculation and be less misleading than defining
*it to be "0").  Therefore I use an owned resource ID
*of -15999 in the resource file "Vital Signs proj.rsrc".
*/

/*The 'MENU' resource's base id.*/

#define	OUR_MENU_ID			1	



/*The 'DLOG' and 'DITL' resources' base ids.*/

#define	ABOUT_BOX_DIALOG_ID 1	



/*The four 'PICT' resources' base ids.*/

#define ABOUT_BOX_PICT_ID	1
#define MAC_PLUS_PICT_ID	2
#define BIG_HEART_PICT_ID	3
#define SMALL_HEART_PICT_ID	4	



/*
*Declare our global variables.
*/

MenuHandle		ourMenu;
BitMap			ScreenInfoBitMap;
Str255			OLDDate, OLDDefaultDisk;
Boolean 		UseAnimationFlag, UseSoundFlag;
unsigned long 	OLDFreeSpaceOnDisk, OLDNumFiles, OLDNumFolders;
long 			OLDFreeRAM, OLDLargestBlock;
int				ourDRVR_ID, AlreadyOpen = NO;
int 			HeartBeatTickCounter, MemoryUpdateTickCounter;



/*
*This is where all of the calls to our DA get routed to the proper
*subroutines.  *ipPB --- a pointer to our I/O parameter block.
*			   theDCE -- our DA's device control entry.
*			   n ------- our DA's entry point selector.  It's value 
*		   			     is used to indicate which of the 5 possible 
*					     entry points (open, prime, control, status,
*					     or close) was called.
*/

main(ioPB, theDCE, n)
cntrlParam	*ioPB;	
DCtlPtr 	theDCE;		
int			n;	
{
SysEnvRec	EnvironmentInfo;
OSErr		SysEnvironsErrorMsg;
int 		*hasHFS;
Boolean		hasNewROMs, SysEnvironsMissing;	

/*
*Check for new ROMs.
*This subroutine was gleaned from I.M. IV-97.  The global (low memory)
*system variable FSFCBLen, located at address 0x3F6, is positive if
*the hierarchical version (HFS, Hierarchical File System) of the File
*Manager is active, and -1 if the 64k ROM version (MFS, the Macintosh
*File System) version of the File Manager is running.  See I.M. IV-309
*to see that FSFCBLen is indeed located at 0x3F6.
*/
	hasHFS = (int *)FSFCBLen;
	if (*hasHFS > 0) {
		hasNewROMs = TRUE;
	}
	else {
		hasNewROMs = FALSE;
	}
	
	
	/*
	*Make sure the SysEnvirons() trap is present.
	*/
	
	SysEnvironsErrorMsg = SysEnvirons(1, &EnvironmentInfo);
	
	if (SysEnvironsErrorMsg == envNotPresent) {
		SysEnvironsMissing = TRUE;
	}
	else {
		SysEnvironsMissing = FALSE;
	}
	
	/*
	*Make sure that the data area for our global variables was actually
	*allocated for us.  Give up (without putting up an error message) if 
	*they weren't.  Allocation of our dynamically allocated data succeeded
	*if dCtlStorage != 0.
	*/
	
	if ((theDCE->dCtlStorage == 0) && (n == drvrOpen)) {
		/*
		*If we got an "open" call, but we didn't get any data area,
		*then beep and close the DA.
		*/
		
		SysBeep(3);
		CloseDriver(theDCE->dCtlRefNum);
		return(0);
	}
	else if ((hasNewROMs == FALSE) || (SysEnvironsMissing == TRUE)){
		/*We don't have HFS, so can't run this DA, because it calls some
		 *HFS routines (to display the number of files and folders on the
		 *default volume.  ALso exit if the SysEnvirons() trap is missing.
		 */	
		 
		SysBeep(3);
		CloseDriver(theDCE->dCtlRefNum);
		return(0);
	}
	
	/*
	*Inside Macintosh II-188 explains the meaning of the various
	*flags in the drvrFlags field of the device control entry.  We
	*are setting the dNeedLock flag because want our DA to be locked
	*in memory as soon as it is opened.  We set the dNeedTime flag
	*because we want our DA to be notified so that it can perform a
	*periodic action.  The dNeedGoodBye flag is set because we want
	*our DA to be called before the application heap is reinitialized.
	*/
	
	theDCE->dCtlFlags |= dNeedLock | dNeedTime | dNeedGoodBye;
	theDCE->dCtlDelay = 1;	/*Notify Vital Signs every 1/60 of a second!*/
			
	switch(n) {
	
		case drvrOpen:			
			doOpen(ioPB, theDCE);
			break;
			
		case drvrPrime:
			doPrime(ioPB, theDCE);
			break;
			
		case drvrControl:
			doControl(ioPB, theDCE);
			break;
			
		case drvrStatus:
			doStatus(ioPB, theDCE);
			break;
			
		case drvrClose:
			doClose(ioPB, theDCE);
			break;
		}
	
}


	
/*
*The 5 driver subroutines that every DA must have.  Note that
*I DON'T do any sort of drawing or displaying of text (via
*DrawString(), etc.) inside doOpen().  Why?  Because it doesn't
*show up!  I'm not sure why.  Instead, I draw "static" text and
*pictures inside doEvent() (inside doControl()) in response to an
*activateEvt event.
*/

doOpen(ioPB, theDCE)
cntrlParam 	*ioPB;	
DCtlPtr 	theDCE;		
{
WindowPtr 	ourWindow;
GrafPtr 	oldPort;
Rect 		windowRect;
OSErr		MemoryErrorMessage;

	if (AlreadyOpen == YES) {
		return;
	}
	else {
		AlreadyOpen = YES;
	}	

	SetRect(&windowRect, 20, 45, 270, 205);

	/*
	*This is a bit tricky.  To compute the DRVR ID of a DA, you negate the
	*dCtlRefNum, and then subtract 1.  The DRVR ID number *is not* simply 
	*the value of "theDCE->dCtlRefNum"!!!  See Inside Macintosh II-191.
	*/
	
	ourDRVR_ID = -(theDCE->dCtlRefNum) - 1;
	
	ourWindow = theDCE->dCtlWindow;
	
	MaxApplZone();
	MemoryErrorMessage = MemError();
	
	/*
	*If our DA doesn't already have a window, then it hasn't been opened
	*yet.  Make a window, initialize the global variables, and install
	*our menu.
	*/
	
    if ((ourWindow == NIL) && (MemoryErrorMessage == noErr)) { 
		GetPort(&oldPort);
		
		/*
		*Initialize the global variable ScreenInfoBitMap.  We will
		*use it later (inside ShowMachineAndSystemInfo()) to determine 
		*the resolution of the screen.  We can't use the QuickDraw 
		*global variable "screenBits" because DA's don't have access 
		*to QuickDraw global variables!
		*/
	
		ScreenInfoBitMap = (*oldPort).portBits;
		
    	ourWindow = NewWindow(0, &windowRect, "\pVital Signs", 0, 
    						  rDocProc, -1, 1, 0);
		SetPort(ourWindow);
		
    	((WindowPeek)ourWindow)->windowKind = theDCE->dCtlRefNum;
		theDCE->dCtlWindow = ourWindow;
		
		/*
		*Initialize the global variables.
		*/
		
		PascalStrCpy("\p", OLDDate);
		HeartBeatTickCounter = 0;
		MemoryUpdateTickCounter = 0;
		UseAnimationFlag = YES;
		UseSoundFlag = NO;
		OLDFreeRAM = 0;
		OLDLargestBlock = 0;
		OLDFreeSpaceOnDisk = 0;
		OLDNumFiles = 0;
		OLDNumFolders = 0;

		/*
		*Create the menu and append it to the menu bar.  Check the
		*"Use animated heart" menu item, and uncheck the "Use sound"
		*menu item.
		*/
		
		ourMenu = GetMenu(CalcTheOwnedRsrcID(ourDRVR_ID, OUR_MENU_ID));	
		theDCE->dCtlMenu = CalcTheOwnedRsrcID(ourDRVR_ID, OUR_MENU_ID);
		
		if (ourMenu != NIL) {
			InsertMenu(ourMenu, 0);
			CheckItem(ourMenu, 2, YES);
			CheckItem(ourMenu, 3, NO);
		}
		
		SetPort(oldPort);
	}
	else if ((ourWindow == NIL) && (MemoryErrorMessage != noErr)) {
		CloseDriver(theDCE->dCtlRefNum);
		SysBeep(3);
	}
	
  	return(0);
}
	
	
	
doControl(ioPB, theDCE)
cntrlParam 	*ioPB;	
DCtlPtr 	theDCE;		
{
GrafPtr 	oldPort;
WindowPtr 	ourWindow;
long 		markChar;
int 		MenuItemNumber;


	ourWindow = theDCE->dCtlWindow;
	
	if (validWindow(ourWindow) == YES) {
	
		/*Save the current GrafPort so that we can restore it later.*/
		
		GetPort(&oldPort);
		SetPort(ourWindow);
		
		/*
		*HERE is where you handle events, but this subroutine
		*is NOT in a loop!  Pretty wierd, no?  There is no
		*GetNextEvent() or WaitNextEvent() in any DA!
		*/
		
		switch(ioPB->csCode) {
		
			/*
			*Here is where we handle all of our Desk Accessory events.
			*/
			
			case accEvent:
				doEvent(((EventRecord *) * (long *) &ioPB->csParam), 
					    ourWindow);
				break;
				
			/*
			*The accRun case is  where we actually USE the time that we
			*asked the Finder to give us.
			*/ 
			
			case accRun:
				if (FrontWindow() == ourWindow) {
					doPeriodicStuff(theDCE);
				}
				break;
					
			case accCursor:
				break;
					
			case accMenu:
				MenuItemNumber = ioPB->csParam[1];
					
				/*
				*We handle our menu items here.
				*/
				
				switch(MenuItemNumber) { 
						
					case 1:
						doAboutBox(ourWindow);
						break;
							
					case 2:
						GetItemMark(ourMenu, MenuItemNumber, &markChar);
						
						if (UseAnimationFlag == YES) {
							UseAnimationFlag = NO;
							CheckItem(ourMenu, MenuItemNumber, NO);
							DisableItem(ourMenu, 3);/*Disable the sound
													menu item.*/
						}
						else {
							UseAnimationFlag = YES;
							CheckItem(ourMenu, MenuItemNumber, YES);
							EnableItem(ourMenu, 3);/*Enable the sound 
													menu item.*/
						}
						break;
						
					case 3:
						GetItemMark(ourMenu, MenuItemNumber, &markChar);
						
						if (UseSoundFlag == YES) {
							UseSoundFlag = NO;
							CheckItem(ourMenu, MenuItemNumber, NO);
						}
						else {
							UseSoundFlag = YES;
							CheckItem(ourMenu, MenuItemNumber, YES);
						}
						break;
						
					case 4:
						ShowMemoryInfo(YES);
						break;

					default:
						break;
				} 
					
				HiliteMenu(0);
				break;
			
			case goodBye: /*We asked for a "goodbye kiss".*/
					doGoodBye();
					break;
					
			case accUndo:
					SysBeep(2);
					break; 
					
			case accCut:
					SysBeep(2); 
					break; 
					
			case accCopy:
					SysBeep(2); 
					break; 
					
			case accPaste:
					SysBeep(2); 
					break; 
					
			case accClear:
					SysBeep(2);
					break; 
		} /*switch(ioPB->csCode)*/
		
		SetPort(oldPort);
		
	}/*if (validWindow(ourWindow) == YES))*/
		
 	return(0);
}

	
		
/*
*Usually DA's don't respond to status or prime calls;
*only "real" device drivers do that.
*/

doStatus(ioPB,dce)
cntrlParam 	*ioPB;	
DCtlPtr 	dce;		
{
	return(0);
}
	
	

doPrime(ioPB, dce)
cntrlParam 	*ioPB;	
DCtlPtr	 	dce;		
{
	return(0);
}	
	
	
	
doClose(ioPB,dce)
cntrlParam 	*ioPB;	
DCtlPtr 	dce;		
{
WindowPtr 	ourWindow;
   
   	ourWindow = dce->dCtlWindow;
   	
	if (validWindow(ourWindow) == YES) {
    	DeleteMenu(CalcTheOwnedRsrcID(ourDRVR_ID, OUR_MENU_ID));
    	dce->dCtlMenu = NIL;
       	DrawMenuBar();
       	DisposeWindow(ourWindow);
       	dce->dCtlWindow = NIL;
	}
	
	return(0);
}



doEvent(theEvent, ourWindow)
EventRecord *theEvent;
WindowPtr 	ourWindow;
{
	switch(theEvent->what) {
	
			case keyDown:
			case autoKey:
				if ((theEvent->modifiers & cmdKey) == YES) {
					switch((char)(theEvent->message)) {
					
					/*We don't do anything with the Edit menu.*/
					
						case 'c':
						case 'C':
						case 'v':
						case 'V':
						case 'x':
						case 'X':
							SysBeep(2);
							break;
							
						default:
							SysBeep(2);
							break;
					}/*switch*/
				}/*if*/
					
			
			case mouseDown:
					GlobalToLocal(&theEvent->where);
					break;
			/*
			*Update events occur when all or part of a window's contents
			*need to be drawn or redrawn, usually as a result of a user's
			*opening, closing, activating, or moving a window.  
			*See Inside Macintosh I-244.
			*/
			
			case updateEvt:
					BeginUpdate(ourWindow);
			
					DrawVitalSignsBox();
					ShowMachineAndSystemInfo();
					ShowMemoryInfo(YES);
					ShowDefaultDiskInfo(YES);
					ShowTimeAndDate(YES);
					DrawOurPicture(MAC_PLUS);

					EndUpdate(ourWindow);
					break;
					
			/*
			*Activate events are generated whenever an inactive window 
			*becomes active or an active window becomes inactive.  They 
			*generally occur in pairs (that is, one window is deactivated 
			*and then another is activated.  See Inside Macintosh I-244.  
			*/
			
			case activateEvt:
					if ((theEvent->modifiers & activeFlag) == YES) {
					
						/*Add our menu title to the menu bar.*/
						
						InsertMenu(ourMenu, 0);	
						DrawMenuBar();
						
						TextFont(geneva);
						TextSize(9);
						
						DrawVitalSignsBox();
						ShowMachineAndSystemInfo();
						ShowMemoryInfo(NO);
						ShowDefaultDiskInfo(NO);
						ShowTimeAndDate(NO);
						DrawOurPicture(MAC_PLUS);
					}
					else {
						  DeleteMenu(CalcTheOwnedRsrcID(ourDRVR_ID,OUR_MENU_ID));
						  DrawMenuBar();
					}
					
					break;
			
			/*We just ignore the other event types.*/
			
			case diskEvt:
			case networkEvt:
			case driverEvt:
			case app1Evt:
			case app2Evt:
			case app3Evt:
			case app4Evt:
			
			default:
				break;
				
	}/*switch(theEvent->what)*/
}
						
						
						
ShowMachineAndSystemInfo()
{
SysEnvRec 		Environment;
StringHandle 	FinderVersionNum;
StringHandle 	SystemVersionNum;
Handle 			tempHandle;
OSErr 			errCode;
char			istring[100];
int 			SystemInteger, SystemFraction;
int 			temp1, temp2;
	
	/*
	*See Technical Note #129 or Inside Macintosh V for info. 
	*about SysEnvirons().  It is just used to get information
	*about the hardware in the Mac we are running on.  I don't
	*need to check the errCode because I already did when I
	*called SysEnvirons() in main().
	*/
	
	errCode = SysEnvirons(1, &Environment);
	
	MoveTo(10, 12);
	DrawString("\pMacintosh model:  ");
		  
	TextFace(bold);/*Write the Mac model type in boldface font.*/
	
	switch(Environment.machineType) {
	
		case envXL:
			DrawString("\pXL (Lisa)");
			break;
			
		case envMac:
			DrawString("\pMac");
			break;
			
		case envMachUnknown:
			DrawString("\punknown");
			break;
			
		case env512KE:
			DrawString("\p512K Enhanced");
			break;
			
		case envMacPlus:
			DrawString("\pPlus");
			break;
			
		case envSE:
			DrawString("\pSE");
			break;
			
		case envMacII:
			DrawString("\pII");
			break;
	}
	
	TextFace(0);/*Restore to plain font.*/
	
	/*
	*The "systemVersion" field of "Environment" is an int which is really
	*two numbers (one in each of the 2 bytes).  The high byte is the 
	*part to the left of the decimal, and the low byte is the fractional
	*part.  The decimal point is implied.  See Technical Note #129, p.3.
	*/
	
	temp1 = temp2 = Environment.systemVersion;
	
	SystemInteger = temp1 >> 8;
	SystemFraction = temp2 && 0x100;
	
	MoveTo(10, 22);
	DrawString("\pSystem Version:  ");
	NumToString(SystemInteger, istring);
	DrawString(istring);
	DrawString("\p.");/*Draw the decimal point.*/
	NumToString(SystemFraction, istring);
	DrawString(istring);
		   
	MoveTo(10, 32);
	DrawString("\pScreen size (pixels): ");
	NumToString(ScreenInfoBitMap.bounds.right, istring);
	DrawString(istring);
	DrawString("\p x ");
	NumToString(ScreenInfoBitMap.bounds.bottom, istring);
	DrawString(istring);
}



ShowMemoryInfo(anUpdateEvent)
Boolean 	anUpdateEvent;
{
PolyHandle 	thePolygon;
Ptr 		myPatternPtr;
Rect 		RAMRect, AreaToShade;
Rect 		largestblockBox,freeRAMBox, RAMpercentBox;
double		doublePercentUsed;
long 		totalRAM, freeRAM, LargestContiguousBlockOfRAM;
int 		intPercentUsed, delta, base;
char 		istring[100]; /*Use this string to hold integers.*/

	/*Gather all the data.*/
	
	totalRAM = (long)TopMem()/1024;
	/*MaxApplZone(); just do this once, in doOpen()*/
	freeRAM = FreeMem();
	freeRAM /= 1024; /*Change it from units of bytes to units of kbytes.*/
	
	/*
	*Ask for 8 Meg of RAM to be allocated.  CompactMem() returns the size 
	*(in bytes) of the largest block it is able to allocate.
	*/
	
	LargestContiguousBlockOfRAM = CompactMem(0x800000)/1024;
	
	/*
	*If this is an update or activate event, the redraw the whole memory
	*display.  Else, just draw it if it has changed from the last time
	*we calculated freeRAM and LargestContiguousBlockOfRAM.
	*/
	
	if (  (anUpdateEvent == YES)	
	    ||((OLDFreeRAM != freeRAM) || (OLDLargestBlock != LargestContiguousBlockOfRAM)))
 	{
 		/*Update our  state variables.*/
 		OLDFreeRAM = freeRAM;
 		OLDLargestBlock = LargestContiguousBlockOfRAM;
 		
 		/*Erase old displays.*/
 		SetRect(&largestblockBox, 75, 60, 155, 70);
 		SetRect(&freeRAMBox, 61, 50, 155, 60);
		SetRect(&RAMpercentBox, 175, 12, 202, 34);
		EraseRect(&largestblockBox);
		EraseRect(&freeRAMBox);
	    EraseRect(&RAMpercentBox);
	  
		NumToString(totalRAM, istring); /*Draw the Total RAM display.*/
		MoveTo(10, 50);
		DrawString("\pTotal RAM: ");
		DrawString(istring);
		DrawString("\pk");
	
		NumToString(freeRAM, istring); /*Draw the Free RAM display.*/
		MoveTo(10, 60);
		DrawString("\pFree RAM:    ");
		DrawString(istring);
		DrawString("\pk");
	
		NumToString(LargestContiguousBlockOfRAM, istring);
		MoveTo(10, 70);
		DrawString("\pLargest block: ");
		DrawString(istring);
		DrawString("\pk");
	
		/*Draw RAM bar graph*/
		MoveTo(175, 10);
		DrawString("\pUsage of:");
		TextFace(bold);
		MoveTo(175, 20);
		DrawString("\pRAM");
		TextFace(0);
	
		/*
		*Calculate the percent of RAM space used, via the formula:
		*
		*	% RAM used = used RAM/total RAM
		*	           = (total RAM - free RAM)/total RAM
	    */
	
	    doublePercentUsed = ((double)totalRAM - (double)freeRAM)
					        /(double)totalRAM;
		
	    intPercentUsed = (int)(doublePercentUsed * 100.0); /*Truncate.*/
	     
		NumToString(intPercentUsed, istring);/*Display the new percent.*/
		MoveTo(175, 30);
		DrawString(istring);
		DrawString("\p%");
	
		SetRect(&RAMRect, 175, 35, 202, 155);/*The dimensions of the RAM bar graph.*/
		CalcAndFillBarGraph(doublePercentUsed, &RAMRect);
	}/*if*/
}



ShowDefaultDiskInfo(anUpdateEvent)
Boolean 		anUpdateEvent;
{
HVolumeParam 	vp;
Rect 			DiskRect;
Rect 			filesBox, foldersBox, freespaceBox;
Rect			capacityBox, nameofdiskBox;
double 			doublePercentUsed;
StringPtr		diskname;
Str255 			vname, dname, DefaultDiskName;
unsigned long 	FreeSpaceOnDisk;
unsigned long 	TotalSpaceOnDisk;
int 			intPercentUsed;
char 			istring[100];

	
	/*Gather the data.*/
	
	vp.ioVolIndex = 1;
	vp.ioNamePtr = diskname;
	vp.ioCompletion = 0L;
	
	PBHGetVInfo(&vp, 0);
	
	TotalSpaceOnDisk = (unsigned int)vp.ioVNmAlBlks * 
						(unsigned long)vp.ioVAlBlkSiz;
						
	TotalSpaceOnDisk = TotalSpaceOnDisk/1024;/*Convert units from bytes to kbytes.*/ 
	
	FreeSpaceOnDisk = (unsigned long)vp.ioVFrBlk * vp.ioVAlBlkSiz;
	FreeSpaceOnDisk = FreeSpaceOnDisk/1024;  /*Convert units from bytes to kbytes.*/ 
	
	/*
	*Calculate the percent of disk space used, via the formula:
	*
	*	% disk used = used disk space/total disk space
	*	            = (total disk space - free disk space)/total disk space
	*/
	
	doublePercentUsed = ((double)TotalSpaceOnDisk - (double)FreeSpaceOnDisk)
					/(double)TotalSpaceOnDisk;
	
	intPercentUsed = (int)(doublePercentUsed * 100.0);/*truncate.  Oh well.*/
	
		if ((anUpdateEvent == YES)  ||
		(OLDNumFiles != vp.ioVFilCnt) ||
		(OLDNumFolders != vp.ioVDirCnt) ||
		(OLDFreeSpaceOnDisk != FreeSpaceOnDisk)) {
		
		/*Update state variables.*/
		OLDNumFiles = vp.ioVFilCnt;
		OLDNumFolders = vp.ioVDirCnt;
		OLDFreeSpaceOnDisk = FreeSpaceOnDisk;
		
		/*Erase old displays.*/
		SetRect(&filesBox, 84, 106, 120, 118);
		SetRect(&foldersBox, 87, 118, 125, 127);
		SetRect(&freespaceBox, 61, 96, 125, 106);
		SetRect(&capacityBox, 53, 87, 125, 96);
		SetRect(&nameofdiskBox, 70, 77, 155, 87);
		EraseRect(&filesBox);
		EraseRect(&foldersBox);
		EraseRect(&freespaceBox);
		EraseRect(&capacityBox);
		EraseRect(&nameofdiskBox);
	
		NumToString(TotalSpaceOnDisk, istring);
	
		MoveTo(10, 85);
		DrawString("\pDefault volume: ");
		DrawString(*(vp.ioNamePtr));
		MoveTo(10, 95);
		DrawString("\pCapacity: ");
		DrawString(istring);				
		DrawString("\pk");
	
		NumToString(FreeSpaceOnDisk, istring);
		MoveTo(10, 105);
		DrawString("\pFree space: ");
		DrawString(istring);
		DrawString("\pk");
		
		MoveTo(10, 115);		
		DrawString("\pNumber of files: ");
		NumToString(vp.ioVFilCnt, istring);
		DrawString(istring);

		MoveTo(10, 125);
		DrawString("\pNumber of folders: ");
		NumToString(vp.ioVDirCnt, istring);
		DrawString(istring);
	
		MoveTo(215, 20);
		TextFace(bold);
		DrawString("\pDisk");
		TextFace(0);
		
		NumToString(intPercentUsed, istring);
		MoveTo(215, 30);	
		DrawString(istring);
		DrawString("\p%");
	
		SetRect(&DiskRect, 218, 35, 243, 155);
		CalcAndFillBarGraph(doublePercentUsed, &DiskRect);
	}
}



ShowTimeAndDate(anUpdateEvent)
Boolean anUpdateEvent;
{
Rect 	timeBox, dateBox;
Str255 	AbbreviatedDate, HoursMinsAndSecs;
long 	SecondsSince1904;
				
	/*Get the data.*/
	
	GetDateTime(&SecondsSince1904);
	IUTimeString(SecondsSince1904, YES, &HoursMinsAndSecs);
	IUDateString(SecondsSince1904, abbrevDate, &AbbreviatedDate);

	/*Erase the old time display, and show the new time.*/
	
	SetRect(&timeBox, 35, 142, 118, 150);
	EraseRect(&timeBox);
	MoveTo(10, 150);
	DrawString("\pTime: ");
	DrawString(HoursMinsAndSecs);
	
	/*
	*ONLY if the date has changed or if this is an update event, erase the 
	*old date display and draw the new date.
	*/
	
	if ((anUpdateEvent == YES) ||
	    (EqualString(OLDDate, AbbreviatedDate, YES, YES) != YES)) {
	    
		PascalStrCpy(AbbreviatedDate, OLDDate);
		SetRect(&dateBox, 36, 132, 120, 143);
		EraseRect(&dateBox);
		MoveTo(10, 140);
		DrawString("\pDate: ");
		DrawString(AbbreviatedDate);	
	}
}



doPeriodicStuff(theDCE)
DCtlPtr 	theDCE;
{
WindowPeek 	GuessWhichWindow;
Str255 		HoursMinsAndSecs, AbbreviatedDate;
long 		SecondsSince1904;

	GuessWhichWindow = (WindowPeek)theDCE->dCtlWindow;
	
	SetPort(GuessWhichWindow);
	
	HeartBeatTickCounter += 1;
	MemoryUpdateTickCounter += 1;
	
	/*Start a new heart beat cycle every 5/6 of a second.*/
	
	if (HeartBeatTickCounter == 50) {
		HeartBeatTickCounter = 0;
	}
	
	/*Update the RAM and default disk display once every 10 seconds.*/
	
	if (MemoryUpdateTickCounter == 360) {
		ShowMemoryInfo(NO);
		ShowDefaultDiskInfo(NO);
		MemoryUpdateTickCounter = 0;
	}
	
   /*Don't play the beep sound unless the heart is also animated.*/

	switch(HeartBeatTickCounter) {
		
		case 1:
				if (UseAnimationFlag == YES) {
					DrawOurPicture(BIG_HEART);
					
					if (UseSoundFlag == YES) {
						SysBeep(1);
					}
				}
				break;
			
		case 45:
				if (UseAnimationFlag == YES) {
					DrawOurPicture(SMALL_HEART);
					
					if (UseSoundFlag == YES) {
						SysBeep(1);
					}
				}
				break;
		
		case 49:/*Update the time display every 5/6 of a second.*/
				ShowTimeAndDate(NO);	
				break;
				
		default:
				break;
			
	}/*switch()*/
}



/*
*doAboutBox() puts up the About Vital Signs box.  It is displayed
*until the user hits the mouse button.
*/

doAboutBox(ourWindow)
WindowPtr 	ourWindow;
{
DialogPtr 	theDialog;
PicHandle 	thePicture;
Handle		DITLHandle, DLOGHandle;
Rect		PictureFrameRect;
int 		OwnedRsrcID;

	/*
	*I.M. I-413 says:  If either the dialog template resource (DLOG) or
	*the dialog item list resurce (DITL) can't be read, the function
	*return result for GetNewDialog() is undefined!!!  Very bad news!!!
	*
	*Use Scott Knaster's tip (p298 of "How to Write Macintosh Software")
	*to do the error checking so that we won't bomb due to a missing 
	*resource file.
	*
	*Use GetResource() to get the DITL and DLOG that make up this dialog,
	*and THEN call GetNewDialog().  If either of the GetResource() calls
	*returns a NIL value, then that resource file is missing, so DON'T do
	*the GetNewDialog().  Just beep (to signal an error), and exit.
	*/
	
	DITLHandle = GetResource('DITL',CalcTheOwnedRsrcID(ourDRVR_ID, 
												ABOUT_BOX_DIALOG_ID));
	DLOGHandle = GetResource('DLOG',CalcTheOwnedRsrcID(ourDRVR_ID, 
												ABOUT_BOX_DIALOG_ID));
												
	if ((DITLHandle == NIL) || (DLOGHandle == NIL)) {
		SysBeep(3);/*Yikes!  Missing resource file!  Bail out!*/
		return;
	}	
	else { /*It's safe to do the GetNewDialog() call.*/		
																		
		theDialog = GetNewDialog(CalcTheOwnedRsrcID(ourDRVR_ID, 
								ABOUT_BOX_DIALOG_ID), 
							 NIL, -1);
 		SetPort(theDialog);
 	
 		OwnedRsrcID = CalcTheOwnedRsrcID(ourDRVR_ID, ABOUT_BOX_PICT_ID);
		thePicture = GetPicture(OwnedRsrcID);
	
		if (thePicture != NIL) {
			MoveHHi(thePicture);
			HLock(thePicture);
			
			/*
			*Get these numbers from the Rect dimensions 
			*you see in ResEdit when you Open the PICT.
			*/
			
			SetRect(&PictureFrameRect, 0, 0, 289, 130);

			DrawDialog(theDialog);
			DrawPicture(thePicture, &PictureFrameRect);

   			do { 
    			;
    		} while (!Button());
    
    		HUnlock(thePicture);
 	
			CloseDialog(theDialog);
			SetPort(ourWindow);
		}
	}
}



doGoodBye(ourWindow)
{
	;
}



/*
*Utility subroutines.
*/

CalcAndFillBarGraph(percentUsed, theRect)
double 		percentUsed;
Rect 		*theRect;
{
Rect 		AreaToShade;
PolyHandle 	thePolygon;
Ptr 		myPatternPtr;
int 		delta, base;
		
	myPatternPtr = NewPtr(sizeof(Pattern));

	/*Just use a solid black pattern.*/
	StuffHex(myPatternPtr, "\pFFFFFFFFFFFFFFFF");

	EraseRect(&(*theRect));
	FrameRect(&(*theRect));
	
	base = theRect->top;
	
	delta = (((int)(   (double)(theRect->bottom - theRect->top)  
				     * (1.0 - percentUsed))));
				     
	SetRect(&AreaToShade, 
			theRect->left,
			base + delta,
			theRect->right,
			theRect->bottom);
	
	/*
	*OpenPoly() returns a handle to a new polygon.  All the following
	*QuickDraw calls define polygon.  ClosePoly() tells QuickDraw when
	*we are done defining it.  We then fill it with a pattern via
	*FillPoly(), and then call KillPoly() to release the memory occupied 
	*by our PolyHandle.
	*/
	
	thePolygon = OpenPoly();
	MoveTo(AreaToShade.left, AreaToShade.top);
	LineTo(AreaToShade.right, AreaToShade.top);
	LineTo(AreaToShade.right, AreaToShade.bottom);
	LineTo(AreaToShade.left, AreaToShade.bottom);
	ClosePoly();
	
	FillPoly(thePolygon, myPatternPtr);
	
	KillPoly(thePolygon);
}



CalcTheOwnedRsrcID(DRVR_ID, base_ID)
int DRVR_ID, base_ID;
{	
int OwnedResourceNumber;

	OwnedResourceNumber = (0xC000 + (DRVR_ID << 5)) + base_ID;
	
	return(OwnedResourceNumber);
}



validWindow(ourWindow)
WindowPtr ourWindow;
{
WindowPtr nextWindow;
  
	if (ourWindow != NIL) {
    	nextWindow = FrontWindow();
    }
    
	while (nextWindow != NIL) {
		if (ourWindow == nextWindow) {
			return(YES);
		}
	    else {
	    	nextWindow = (WindowPtr)(((WindowPeek)nextWindow)->nextWindow);
	    }
	}

    return(NO);
}



/*
*Frame the Vital Signs box, and draw the three
*horizontal crossbars.
*/

DrawVitalSignsBox()
{
Rect VitalSignsBox;
						
	SetRect(&VitalSignsBox, 5, 3, 170, 155);
	FrameRect(&VitalSignsBox);
						
	MoveTo(5, 35);
	LineTo(169, 35);
						
	MoveTo(5, 73);
	LineTo(169, 73);
						
	MoveTo(5, 127);
	LineTo(169, 127);
}
						

	
/*
*Draw one of three PICT's, in the DA's window.
*/

DrawOurPicture(whichPicture)
int 		whichPicture;
{
PicHandle 	OurPicture;
Rect  		OurPicturesRect, toEraseBigHeart;
int 		OurPicturesOwnedRsrcID;

	switch (whichPicture) {
		
		case MAC_PLUS:
			OurPicturesOwnedRsrcID = CalcTheOwnedRsrcID(ourDRVR_ID,
												        MAC_PLUS_PICT_ID);
			/*First, make the Rect the right size.*/
			SetRect(&OurPicturesRect, 0, 0, 50, 57);
	
			/*Now offset it so it gets drawn where we want!*/
			OffsetRect(&OurPicturesRect, 124, 100);
			break;
			
		case BIG_HEART:
			OurPicturesOwnedRsrcID = CalcTheOwnedRsrcID(ourDRVR_ID,
												        BIG_HEART_PICT_ID);
			SetRect(&OurPicturesRect, 0, 0, 31, 23);
			OffsetRect(&OurPicturesRect, 133, 108);
			break;
		
		case SMALL_HEART:
			OurPicturesOwnedRsrcID = CalcTheOwnedRsrcID(ourDRVR_ID,
												        SMALL_HEART_PICT_ID);
			/*We have to erase the big heart first.*/
			SetRect(&toEraseBigHeart, 0, 0, 31, 23);
			OffsetRect(&toEraseBigHeart, 133, 108);
			EraseRect(&toEraseBigHeart);
			
			SetRect(&OurPicturesRect, 0, 0, 31, 15);
			OffsetRect(&OurPicturesRect, 133, 115);
			break;		
			
		}/*switch(whichPicture)*/
		
	OurPicture = GetPicture(OurPicturesOwnedRsrcID);
	
	if (OurPicture != NIL) {
		MoveHHi(OurPicture);
		HLock(OurPicture);
		DrawPicture(OurPicture, &OurPicturesRect);

		HUnlock(OurPicture);
	}
}



/*
*PascalStrCpy() copies a pascal string from p1 to p2.                       
*/

PascalStrCpy(p1, p2)
register char *p1, *p2;
{
register int len;
	
	len = *p2++ = *p1++;
	while (--len >= 0) {
		*p2++ = *p1++;
	}
}