UNIT SSU;

	INTERFACE

		USES MemTypes, OSIntf, ToolIntf, Packages;

		TYPE
			NickNameT = Str63; {Maximum length of an alternate name}

			CanonFileSpec = RECORD
												cf_vRefNum: Integer;
												cf_dirID: LongInt;
												cf_fileName: Str63;
											END;

			ItemInfoHdl = ^ItemInfoPtr;
			ItemInfoPtr = ^ItemInfoRec;
			ItemInfoRec = RECORD
											il_nextItemInfo: ItemInfoHdl; {If NIL,last entry }
											il_privData: LongInt; {User definable }
											il_resourceType: OSType; {Resource OS Type }
											il_resID: Integer; {Resource ID }
											il_fileType: OSType; {File type }
											il_fileSpec: CanonFileSpec; {dirId + vRefNum + fileName}
											il_nickName: NickNameT; {Alternate name }
										END;

			SSUInfoPtr = ^SSUInfoRec;
			SSUInfoRec = RECORD
										 ssu_folderTime: LongInt;
																	{Copy of DirInfoArr.di_time}
										 ssu_firstItemInfo: ItemInfoHdl;
																			{Handle to the first item in the list  }
									 END;

			BuildItemInfoRT = RECORD
													bi_fileType: OSType; {File type. Required }
													bi_crtrType: OSType; {Creator. Can be NIL }
													bi_resType: OSType; {Res type. Can be NIL }
													bi_fileName: StringPtr; {File name. Can be NIL }
													bi_procAddr: ProcPtr; {ID routine. Can be NIL }
												END;

			BuildItemInfoAPT = ^BuildItemInfoAT;
			BuildItemInfoAT = ARRAY [1..4] OF BuildItemInfoRT;

		PROCEDURE InitSSU;

		PROCEDURE SSU_BuildList(biiList: BuildItemInfoAPT;
														numTypes: Integer;
														VAR result: SSUInfoRec;
														killPrivData: ProcPtr);

		PROCEDURE SSU_DisposList(VAR result: SSUInfoRec;
														 killPrivData: ProcPtr);

		FUNCTION SSU_OpenFile(theItemInfoHdl: ItemInfoHdl;
													VAR fileRefNum: Integer;
													permission: Byte;
													openResFork: Boolean): OSErr;

		FUNCTION SSU_CloseFile(VAR fileRefNum: Integer;
													 closeResFork: Boolean): OSErr;

		PROCEDURE BlockClear(thePtr: UNIV Ptr;
												 count: Integer);

		VAR
			gAppWDRefNum: Integer;
			{App working directory	refnum									}
			gSysWDRefNum: Integer;
			{System Folder working directory refnum 	}

	IMPLEMENTATION
		{$S SegSSU} {}

		TYPE
			DirectoryInfoRec = RECORD
													 di_dirModTime: LongInt; {Mod date of dir }
													 di_dirID: LongInt; { Directory ID }
													 di_vRefNum: Integer; { Volume ref number }
												 END;

			DirInfoArrHdl = ^DirInfoArrPtr;
			DirInfoArrPtr = ^DirInfoArr;
			DirInfoArr = RECORD
										 di_time: LongInt;
														{Time in ticks when this structure was built}
										 di_count: Integer; {Number of entries in list }
										 di_list: ARRAY [1..512] OF DirectoryInfoRec;
									 END;

		CONST
			kDirInfoArrHeadSize = SizeOf(LongInt) + SizeOf(Integer);
			kCompanyFName = 'Company Folder';
			eternity = False;

		VAR
			gCompanyFStruct: DirInfoArrHdl;

		PROCEDURE CallIdentifyProc(VAR theName: Str255;
															 theCodeH: Handle;
															 theResID: Integer;
															 VAR thePrivData: LongInt;
															 theProcAddr: ProcPtr);
			INLINE $205F, { MOVE.L (A7)+,A0 }
						 $4E90; { JSR (A0) }

		PROCEDURE CallKillPrivData(VAR privData: LongInt;
															 theProcAddr: ProcPtr);
			INLINE $205F, { MOVE.L (A7)+,A0 }
						 $4E90; { JSR (A0) }

		{Clears a block of memory of a specified size}

		PROCEDURE BlockClear(thePtr: UNIV Ptr;
												 count: Integer); BEGIN

			WHILE count > 0 DO BEGIN
				count := count - 1;
				thePtr^ := 0;
				thePtr := Ptr(LongInt(thePtr) + 1);
			END;
		END;

{CleanDisposHandle works like DisposHandle but has two advantages. 
1) If a handle is NIL, DisposHandle is not called and 
2) If h is part of a dereferenced relocatable block, CleanDisposHandle 
first makes a copy of the handle before calling DisposHandle, avoiding
a possible heap error}
PROCEDURE CleanDisposHandle(VAR h:UNIV Handle);
VAR 	temp	: Handle;
BEGIN
	 IF h <> NIL THEN BEGIN
			temp := h;
			h := NIL;
			DisposHandle(temp);
	 END;
END;

{Identical to FSClose, except that it will not try to close if 
dataForkRefNum is equal to 0 on entry; 
also, dataForkRefNum is set to 0 on exit}

		FUNCTION CleanFSClose(VAR dataForkRefNum: Integer): OSErr;

			VAR
				tempRefNum: Integer; BEGIN

			IF dataForkRefNum <> 0 THEN BEGIN
				tempRefNum := dataForkRefNum;
				dataForkRefNum := 0;
				CleanFSClose := FSClose(tempRefNum);
			END
			ELSE
				CleanFSClose := noErr;
		END;

{Identical to CloseResFile, except that it will not try to close if 
resForkRefNum is equal to 0 on entry; also, resForkRefNum is set to 0 on exit}

		FUNCTION CleanCloseResFile(VAR resForkRefNum: Integer): OSErr;

			VAR
				tempRefNum: Integer; BEGIN

			IF resForkRefNum <> 0 THEN BEGIN
				tempRefNum := resForkRefNum;
				resForkRefNum := 0;
				CloseResFile(tempRefNum);
				CleanCloseResFile := ResError;
			END
			ELSE
				CleanCloseResFile := noErr;
		END;

{Identical to ReleaseResource, except that it will not try to release 
if resHandle is equal to 0 on entry; also, resHandle is set to NIL on exit}

		FUNCTION CleanReleaseResource(VAR resHandle: UNIV Handle): OSErr;

			VAR
				tempH: Handle; BEGIN

			IF resHandle <> NIL THEN BEGIN
				tempH := resHandle;
				resHandle := NIL;
				ReleaseResource(tempH);
				CleanReleaseResource := ResError;
			END
			ELSE
				CleanReleaseResource := noErr;
		END;

{CleanOpenWD opens a working directory so that files contained 
therein can be accessed or searched.
'vRefNum' 	is the working directory reference	number of either 
						the Application folder or the System Folder. 
						Additionally, when recursively searching the 'Company Folder', 
						this can also be the wdRefNum of an enclosed folder.
'dirID' 		working directory reference number of the folder in vRefNum
'mustCloseWD'returns TRUE if we opened the working	directory and FALSE 
						if it was already opened. If 'mustCloseWD' is FALSE, we 
						should not call 'CleanCloseWD' (see comments in CleanCloseWD).
This function returns an operating system error }

		FUNCTION CleanOpenWD(vRefNum: Integer;
												 dirID: LongInt;
												 VAR wdRefNum: Integer;
												 VAR mustCloseWD: Boolean): OSErr;

			VAR
				wdInfo: WDPBRec;
				tempErr: OSErr; BEGIN

			BlockClear(@wdInfo, SizeOf(wdInfo));

{According to the Macintosh Tech notes #77 and #190, only those working 
directories that were created by the app should be closed by the app. 
This routine checks if working directory for the specified volume/dirID 
has been already created by somebody. (perhaps by the Finder
or any other application running under MultiFinder), and returns FALSE 
in mustCloseWD if we didn't open it. }

			REPEAT
				wdInfo.ioVRefNum := vRefNum;
				wdInfo.ioWdIndex := wdInfo.ioWdIndex + 1;
				wdInfo.ioWDProcID := 0;
				wdInfo.ioWDVRefNum := 0;
				tempErr := PBGetWDInfo(@wdInfo, False);
			UNTIL (tempErr <> noErr) | (wdInfo.ioWDDirID = dirID);
			mustCloseWD := tempErr <> noErr;

			IF mustCloseWD THEN BEGIN
				tempErr := OpenWD(vRefNum, dirID, LongInt('ERIK'), wdRefNum);
				IF tempErr <> noErr THEN wdRefNum := 0;
				CleanOpenWD := tempErr;
			END
			ELSE BEGIN
				CleanOpenWD := noErr;
				wdRefNum := wdInfo.ioVRefNum;
			END;
		END;

{CleanCloseWD closes a working directory that we opened. 'wdRefNum' 
is the reference number of the working directory that should be closed 
This function returns an operating system error.
WARNING:		This function must never be called if the variable 
mustCloseWD returned FALSE in call to CleanOpenWD}

		FUNCTION CleanCloseWD(VAR wdRefNum: Integer): OSErr; BEGIN

			IF wdRefNum <> 0 THEN BEGIN
				CleanCloseWD := CloseWD(wdRefNum);
				wdRefNum := 0;
			END
			ELSE
				CleanCloseWD := noErr;
		END;

{SSU_OpenFile opens file specified in ItemInfoHdl.
'theItemInfoHdl'			is a handle to information about aparticular file
'fileRefNum'					returned file reference of the file 'permission' 
											specifies access privileges for the file (Constants 
											may be found in Inside Mac Volume 2 Page 100)
'openResFork' 			 	if TRUE, the res fork should  be opened instead of 
											the data fork
This function returns an operating system error 			}

		FUNCTION SSU_OpenFile(theItemInfoHdl: ItemInfoHdl;
													VAR fileRefNum: Integer;
													permission: Byte;
													openResFork: Boolean): OSErr;

			VAR
				pbs: HParamBlockRec;
				wdRefNum: Integer;
				tempErr: Integer;
				mustCloseWD: Boolean; BEGIN

			fileRefNum := 0;
			IF theItemInfoHdl <> NIL THEN BEGIN
				IF openResFork THEN BEGIN
					IF Length(theItemInfoHdl^^.il_fileSpec.cf_fileName) = 0 THEN BEGIN
						{This resource belongs to program resource fork}
						tempErr := 0;
					END
					ELSE BEGIN
						HLock(Handle(theItemInfoHdl));
						WITH theItemInfoHdl^^ DO BEGIN
							IF il_fileSpec.cf_dirID <> 0 THEN BEGIN
								tempErr := CleanOpenWD(il_fileSpec.cf_vRefNum,
																			 il_fileSpec.cf_dirID, wdRefNum,
																			 mustCloseWD);

								IF tempErr = noErr THEN BEGIN
									fileRefNum := OpenRFPerm(StringPtr(@il_fileSpec.cf_fileName)^,
																					 wdRefNum, permission);
									tempErr := ResError;

									IF tempErr <> noErr THEN BEGIN
										fileRefNum := 0;
										IF mustCloseWD & (CleanCloseWD(wdRefNum) <> noErr) THEN;
									END
									ELSE BEGIN
										IF mustCloseWD THEN tempErr := CleanCloseWD(wdRefNum);
									END;
								END;
							END
							ELSE BEGIN
								fileRefNum := OpenRFPerm(StringPtr(@il_fileSpec.cf_fileName)^,
																				 il_fileSpec.cf_vRefNum, permission);
								tempErr := ResError;
								IF tempErr <> noErr THEN fileRefNum := 0;
							END;
						END;
						HUnLock(Handle(theItemInfoHdl));
					END;
				END
				ELSE BEGIN
					IF Length(theItemInfoHdl^^.il_fileSpec.cf_fileName) > 0 THEN BEGIN
						HLock(Handle(theItemInfoHdl));
						BlockClear(@pbs, SizeOf(pbs));
						WITH theItemInfoHdl^^ DO BEGIN
							pbs.ioNamePtr := @il_fileSpec.cf_fileName;
							pbs.ioVRefNum := il_fileSpec.cf_vRefNum;
							pbs.ioPermssn := permission;
							pbs.ioDirID := il_fileSpec.cf_dirID;
						END;
						tempErr := PBHOpen(@pbs, False);
						HUnLock(Handle(theItemInfoHdl));
						IF tempErr = noErr THEN fileRefNum := pbs.ioRefNum;
					END
					ELSE
						tempErr := memFullErr;
				END;
			END
			ELSE
				tempErr := memFullErr;

			SSU_OpenFile := tempErr;
		END;

{SSU_CloseFile closes file specified by fileRefNum
'fileRefNum'			file ref num of file to be closed
'closeResFork' 		if TRUE, then this routine closes the 
									res fork instead of the data fork
This function returns an operating system error }

		FUNCTION SSU_CloseFile(VAR fileRefNum: Integer;
													 closeResFork: Boolean): OSErr; BEGIN

			IF closeResFork THEN BEGIN
				SSU_CloseFile := CleanCloseResFile(fileRefNum);
			END
			ELSE BEGIN
				SSU_CloseFile := CleanFSClose(fileRefNum);
			END;
		END;

		FUNCTION GetDirModTime(vRefNum: Integer;
													 directoryId: LongInt): LongInt;

			VAR
				myCInfoPBRec: CInfoPBRec; BEGIN

			BlockClear(@myCInfoPBRec, SizeOf(myCInfoPBRec));
			myCInfoPBRec.ioVRefNum := vRefNum;
			myCInfoPBRec.ioFDirIndex := - 1;
			myCInfoPBRec.ioDrDirID := directoryId;

			IF PBGetCatInfo(@myCInfoPBRec, False) = noErr THEN BEGIN
				GetDirModTime := myCInfoPBRec.ioDrMdDat;
			END
			ELSE BEGIN
				{could not get modTime for directory? huh }
				GetDirModTime := Random;
			END;
		END;

		PROCEDURE StuffDirInfo(vRefNum: Integer;
													 dirID: LongInt);

			VAR
				currentInfo: DirectoryInfoRec;
				theTicks: LongInt; BEGIN

			currentInfo.di_dirModTime := GetDirModTime(vRefNum, dirID);
			currentInfo.di_dirId := dirID;
			currentInfo.di_vRefNum := vRefNum;

			IF gCompanyFStruct = NIL THEN BEGIN
				theTicks := TickCount;
				gCompanyFStruct := DirInfoArrHdl(NewHandleClear(kDirInfoArrHeadSize));
				gCompanyFStruct^^.di_time := theTicks;
			END;

			SetHandleSize(Handle(gCompanyFStruct), ((gCompanyFStruct^^.di_count + 1) *
										LongInt(SizeOf(DirectoryInfoRec))) + kDirInfoArrHeadSize);
			WITH gCompanyFStruct^^ DO BEGIN
				di_count := di_count + 1;
				di_list[di_count] := currentInfo;
			END;
		END;

{SearchForDirectory searches for Company Folder in the Working 
Directory specified by wdRefNum
'wdRefNum'	is the working directory reference number of either 
						the Application folder or the System Folder.
This function returns either the Company Folder directory ID, 
or zero if none is found }

		FUNCTION SearchForDirectory(wdRefNum: Integer): LongInt;

			VAR
				i: Integer;
				dirIndex: Integer;
				len1: Integer;
				tempName: Str255;
				searchName: Str255;
				cInfo: CInfoPBRec; BEGIN

			BlockClear(@cInfo, SizeOf(cInfo));

			dirIndex := 1;

			searchName := kCompanyFName;
			len1 := Length(kCompanyFName);

			REPEAT
				cInfo.ioNamePtr := @tempName;
				cInfo.ioDirID := 0;
				cInfo.ioVRefNum := wdRefNum;
				cInfo.ioFDirIndex := dirIndex;

				IF PBGetCatInfo(@cInfo, False) = noErr THEN BEGIN
					IF BAnd(Integer(cInfo.ioFlAttrib), $010) <> 0 THEN BEGIN { We've found
																																			a directory }
						IF (Length(tempName) = len1) THEN
							IF EqualString(tempName, searchName, False, True) THEN BEGIN
								SearchForDirectory := cInfo.ioDirID;
								LEAVE;
							END;
					END;
					dirIndex := dirIndex + 1;
				END
				ELSE BEGIN
					{got error, so it looks like we did not find it}
					SearchForDirectory := 0;
					LEAVE;
				END;
			UNTIL eternity;
		END;

		FUNCTION Min(a, b: Integer): Integer; BEGIN

			IF a < b THEN
				Min := a
			ELSE
				Min := b;
		END;

		PROCEDURE SSU_BuildList(biiList: BuildItemInfoAPT;
														numTypes: Integer;
														VAR result: SSUInfoRec;
														killPrivData: ProcPtr);

			VAR
				i: Integer;
				dummyOSErr: OSErr;
				buildFolderInfo: Boolean;
				cInfo: CInfoPBRec;
				currFileName: Str255;
				theResCount: Integer;
				resHdl: Handle;
				theName: NickNameT;
				theResID: Integer;
				theResType: ResType;
				theResName: STR255;
				thePrivData: LongInt;
				noTypeFlag: Boolean;

			PROCEDURE SearchDirectory(theVRefNum: Integer;
																theDirID: LongInt;
																recursive: Boolean);

				VAR
					dirIndex: Integer;
					jj: Integer;
					refNum: Integer;
					wdRefNum: Integer;
					theResIndex: Integer;
					onlyFiles: Boolean;
					validFile: Boolean;
					mustCloseWD: Boolean;
					pbs: HParamBlockRec; BEGIN

				IF (theDirID = 0) | (CleanOpenWD(theVRefNum, theDirID, wdRefNum,
																				 mustCloseWD) = noErr) THEN BEGIN
					IF theDirID = 0 THEN wdRefNum := theVRefNum;

					IF buildFolderInfo THEN StuffDirInfo(theVRefNum, theDirID);

					dirIndex := 1;
					REPEAT
						cInfo.ioNamePtr := @currFileName;
						cInfo.ioDirID := theDirID;
						cInfo.ioVRefNum := wdRefNum;
						cInfo.ioFDirIndex := dirIndex;

						IF PBGetCatInfo(@cInfo, False) = noErr THEN BEGIN
							IF BAnd(Integer(cInfo.ioFlAttrib), $010) <> 0 THEN BEGIN {We've
																																found a directory}
								IF recursive THEN
									SearchDirectory(theVRefNum, cInfo.ioDirID, True);
							END
							ELSE BEGIN
								FOR jj := 1 TO numTypes DO
									IF LongInt(biiList^[jj].bi_fileType) =
										 LongInt(cInfo.ioFlFndrInfo.fdType) THEN
										LEAVE;

								IF jj <= numTypes THEN BEGIN
									WITH biiList^[jj] DO BEGIN
										onlyFiles := LongInt(bi_resType) = 0;

										validFile := (LongInt(bi_crtrType) = 0) |
																 (LongInt(bi_crtrType) =
																 LongInt(cInfo.ioFlFndrInfo.fdCreator));

										IF validFile THEN
											IF bi_fileName <> NIL THEN
												validFile := (Length(bi_fileName^) =
																		 Length(cInfo.ioNamePtr^)) &
																		 EqualString(bi_fileName^,
																		 cInfo.ioNamePtr^, False, True);
									END;

									refNum := 0;
									IF validFile THEN BEGIN
										IF onlyFiles THEN BEGIN
											IF biiList^[jj].bi_procAddr <> NIL THEN BEGIN
												BlockClear(@pbs, SizeOf(pbs));
												pbs.ioNamePtr := cInfo.ioNamePtr;
												pbs.ioVRefNum := wdRefNum;
												pbs.ioPermssn := fsRdPerm;
												pbs.ioDirID := theDirID;
												validFile := PBHOpen(@pbs, False) = noErr;

												IF validFile THEN refNum := pbs.ioRefNum;
											END;
										END
										ELSE BEGIN
											SetResLoad(False);
											refNum := OpenRFPerm(cInfo.ioNamePtr^, wdRefNum,
																fsRdPerm);
											SetResLoad(True);
											validFile := ResError = noErr;
										END;

										IF validFile THEN BEGIN
											noTypeFlag := LongInt(biiList^[jj].bi_resType) = 0;

											IF noTypeFlag THEN
												theResCount := 1
											ELSE
												theResCount := Count1Resources(biiList^[jj].bi_resType);

											FOR theResIndex := 1 TO theResCount DO BEGIN

												IF noTypeFlag THEN
													resHdl := NIL
												ELSE
													resHdl := Get1IndResource(biiList^[jj].bi_resType,
																										theResIndex);

												IF noTypeFlag | ((resHdl <> NIL) & (ResError =
													 noErr)) THEN BEGIN
													IF noTypeFlag THEN BEGIN
														theResID := 0;
														theResType := ResType(NIL);
														theResName := cInfo.ioNamePtr^;
													END
													ELSE BEGIN
														HLock(resHdl);
														GetResInfo(resHdl, theResID, theResType,
																			 theResName);
														IF Length(theResName) = 0 THEN BEGIN
															NumToString(theResID, theResName);
															theResName := Concat('xxxx resource number ', 
																																theResName);
															BlockMove(@theResType, @theResName[1], 4);
														END;
													END;

													thePrivData := 0;

										{ now let's call identify routine to see if it's available }
													IF biiList^[jj].bi_procAddr <> NIL THEN BEGIN
														IF noTypeFlag THEN
															CallIdentifyProc(theResName, NIL, refNum,
																							 thePrivData,
																							 biiList^[jj].bi_procAddr)
														ELSE
															CallIdentifyProc(theResName, resHdl, theResID,
																							 thePrivData,
																							 biiList^[jj].bi_procAddr);
													END;

													IF NOT noTypeFlag THEN BEGIN
														HUnLock(resHdl);
														IF CleanReleaseResource(resHdl) <> noErr THEN
															theResName := '';
													END;

{This would be a good place to check for duplicate names in the list and 
decide what to do with them}

													IF Length(theResName) > 0 THEN BEGIN
														{ so far this file is valid }
														resHdl := NewHandleClear(SizeOf(ItemInfoRec));
														WITH ItemInfoHdl(resHdl)^^ DO BEGIN
															theName := Copy(theResName, 1,
																							Min(SizeOf(il_nickName) - 1,
																							Length(theResName)));

															il_nickName := theName;
														END;

														WITH ItemInfoHdl(resHdl)^^ DO BEGIN
															il_privData := thePrivData;
															il_fileSpec.cf_fileName := cInfo.ioNamePtr^;
															il_fileSpec.cf_vRefNum := theVRefNum;
															il_fileSpec.cf_dirID := theDirID;
															il_fileType := cInfo.ioFlFndrInfo.fdType;
															il_nextItemInfo := result.ssu_firstItemInfo;
															il_resourceType := biiList^[jj].bi_resType;
															il_resID := theResID;
														END;
														result.ssu_firstItemInfo := ItemInfoHdl(resHdl);
													END;
												END; { no resErr }
											END; { all files in this res file }

											IF onlyFiles THEN
												dummyOSErr := CleanFSClose(refNum)
											ELSE
												dummyOSErr := CleanCloseResFile(refNum);
										END
										ELSE
											refNum := 0;
									END;
								END;
							END;

							dirIndex := dirIndex + 1;
						END
						ELSE BEGIN
							theName := '';
							LEAVE;
						END;
					UNTIL eternity;

					IF (theDirID <> 0) & mustCloseWD & (CleanCloseWD(wdRefNum) <>
						 noErr) THEN BEGIN
						{ process the error }
					END;
				END;
			END;

			VAR
				folderDirID: LongInt;
				folderVRefNum: Integer;
				shouldBuild: Boolean; BEGIN

			{ Let's check if the item list needs to be updated }
			shouldBuild := gCompanyFStruct = NIL;

			IF NOT shouldBuild THEN BEGIN
				HLock(Handle(gCompanyFStruct));
				WITH gCompanyFStruct^^ DO
					FOR i := 1 TO di_count DO
						WITH di_list[i] DO
							IF GetDirModTime(di_vRefNum, di_dirId) <> di_dirModTime THEN BEGIN
								shouldBuild := NOT shouldBuild;
								LEAVE;
							END;
				HUnLock(Handle(gCompanyFStruct));

				IF shouldBuild THEN CleanDisposHandle(gCompanyFStruct);
			END;

			buildFolderInfo := gCompanyFStruct = NIL;

			IF shouldBuild |
				(result.ssu_folderTime <> gCompanyFStruct^^.di_time) THEN BEGIN

				SSU_DisposList(result, killPrivData);

			 {First let's look for the Company Folder inside the Application Folder}
				folderVRefNum := gAppWDRefNum;

				folderDirID := SearchForDirectory(folderVRefNum);
				IF folderDirID = 0 THEN BEGIN
				 {or else let's look for the Company Folder inside the System Folder}
					;
					folderVRefNum := gSysWDRefNum;
					folderDirID := SearchForDirectory(folderVRefNum);
				END;

{If the Company Folder is found in either the Application or System Folder, 
then let's look for the items }

				IF folderDirID <> 0 THEN
					SearchDirectory(folderVRefNum, folderDirID, True);

				result.ssu_folderTime := gCompanyFStruct^^.di_time;
			END;
		END;

		PROCEDURE DisposeItemInfoHdl(VAR theItemInfoHdl: ItemInfoHdl;
																 killPrivData: ProcPtr); BEGIN

			IF theItemInfoHdl <> NIL THEN BEGIN
				HLock(Handle(theItemInfoHdl));
				DisposeItemInfoHdl(theItemInfoHdl^^.il_nextItemInfo, killPrivData);
				IF killPrivData <> NIL THEN
					CallKillPrivData(theItemInfoHdl^^.il_privData, killPrivData);
				HUnLock(Handle(theItemInfoHdl));
				CleanDisposHandle(theItemInfoHdl);
			END;
		END;

		PROCEDURE SSU_DisposList(VAR result: SSUInfoRec;
														 killPrivData: ProcPtr); BEGIN

			result.ssu_folderTime := 0;
			DisposeItemInfoHdl(result.ssu_firstItemInfo, killPrivData);
		END;

		{$S Init}

		PROCEDURE InitSSU;

			VAR
				theWorld: SysEnvRec; BEGIN

			gCompanyFStruct := NIL;
			IF GetVol(NIL, gAppWDRefNum) <> noErr THEN
{This should never happen, but you	
may want to be defensive anyway}
			;
			IF SysEnvirons(2, theWorld) = noErr THEN
				gSysWDRefNum := theWorld.sysVRefNum
			ELSE
				gSysWDRefNum := gAppWDRefNum;
		END;

END.
