{ --------------------------------------------------------------------------- }
{ Apple Macintosh Developer Technical Support                                 }
{                                                                             }
{ Some of "Skippy White's Famous High Level Off-Screen Map Routines"          }
{                                                                             }
{ "OffscreenSubs.p"                                                           }
{                                                                             }
{ These routines provide a high-level interface to the QuickDraw & Color      }
{ Manager routines which allow the creation and manipulation of off-screen    }
{ bitmaps and pixmaps. They are designed to run on any machine with 128K or   }
{ later ROMs.                                                                 }
{                                                                             }
{ NOTE that I've modified some of Skippy's routines and that, therefore, any  }
{      resultant errors in syntax or logic belong solely to me.               }
{ --------------------------------------------------------------------------- }

UNIT OffscreenSubs;


INTERFACE

	USES
		Quickdraw, wBMInterface, wBMGlobals, wBMMiscSubs;


	FUNCTION GetMaxAreaDevice (globalRect: Rect): GDHandle;
	FUNCTION CreateOffScreen (VAR myRect: Rect): OffScreenRecHdl;
	PROCEDURE ToOnScreen (COSHdl: OffScreenRecHdl);
	PROCEDURE DisposOffScreen (VAR COSHdl: OffScreenRecHdl);




IMPLEMENTATION




{ ********** }
	FUNCTION GetMaxAreaDevice (globalRect: Rect): GDHandle;

{ Find the greatest overlap device for the given global rectangle.}

		VAR
			area: LONGINT;
			maxArea: LONGINT;
			device: GDHandle;
			intersection: Rect;

	BEGIN

		GetMaxAreaDevice := NIL;
		;
		maxArea := 0;
		;
		device := GetDeviceList;

		WHILE device <> NIL DO
		BEGIN
			IF TestDeviceAttribute(device, screenDevice) THEN
				IF TestDeviceAttribute(device, screenActive) THEN

					IF SectRect(globalRect, device^^.gdRect, intersection) THEN
					BEGIN

						WITH intersection DO
							area := LONGINT(right - left) * LONGINT(bottom - top);

						IF area > maxArea THEN
						BEGIN
							GetMaxAreaDevice := device;
							maxArea := area;
						END;   { IF area > maxArea }

					END;   { IF SectRect ... }

			device := GetNextDevice(device);
		END;   { WHILE device <> NIL }

	END;   { GetMaxAreaDevice }



{ ************************** }
{ Just plain nifty stuff ... }
{ ************************** }


	FUNCTION CreateOffScreen (VAR myRect: Rect): OffScreenRecHdl;
{ Reference: Tech Note #120                             }
{ with special thanks to:                               }
{                      Jon Zap and                      }
{                      Forrest Tanaka of MacDTS         }
{                                                       }
{ NOTE: Local window coordinates are input, but local   }
{ device coordinates are returned for drawing purposes. }

		VAR
			offRowBytes, sizeOfOff, tempSeed: LONGINT;
			localRect, globRect: Rect;
			i, maxDepth: INTEGER;
			err: OSErr;
			COSHdl: OffScreenRecHdl;


		PROCEDURE ErrorOut (error: OSErr);

		BEGIN
			IF error = NewCOSHdlError THEN
				CreateOffScreen := NIL
			ELSE
			BEGIN
				COSHdl^^.CreateOffScreenError := error;
				CreateOffScreen := COSHdl;
			END;

			EXIT(CreateOffScreen);
		END;   { ErrorOut }


	BEGIN   { CreateOffScreen }

		COSHdl := OffScreenRecHdl(NewClearHandle(SizeOf(OffScreenRec)));
		;
		IF MemError <> noErr THEN
			ErrorOut(NewCOSHdlError);
		;
		MoveHHi(Handle(COSHdl));
		HLock(Handle(COSHdl));       { Lock this sucker down !! }

		WITH COSHdl^^ DO
		BEGIN

    { CreateOffScreenError := noErr;   --   We hope !! }
			GetPort(origPort);         { Used by ToOnScreen. }
			drawingRect := myRect;     { Saved for use after call to ToOnScreen. }

			globRect := myRect;        { We're about to switch the Port to off-screen: }
			LocalGlobal(globRect);

			IF NOT aMac2 THEN
			BEGIN
				offGrafPtr := @offGrafPort;
				OpenPort(offGrafPtr);
				maxDepth := 1;
			END   { ... a low-life machine }
			ELSE
			BEGIN

      { Figure out how much space we need for our pixel image.  We call    }
      { mommy ROM's GetMaxDevice to get the pixel map from that.           }
      { We do this to cover the case where the pixel image that we wish to }
      { CopyBits to spans multiple devices (of possibly different depths). }

				myMaxDevice := GetMaxDevice(globRect);
				IF myMaxDevice = NIL THEN
					ErrorOut(MaxDevError);

      { We need to set theGDevice to the device with the maximum pixel }
      { depth (the one we just found), so the pixel map of the new     }
      { CGrafPort will be copied from one of the proper depth.  Save   }
      { the current GDevice for retrieval later and, then, open the    }
      { new CGrafPort to use for our off-screen drawing.               }

				oldDevice := GetGDevice;
				SetGDevice(myMaxDevice);
				;
				offCGrafPtr := @offCGrafPort;               { Initialize this guy. }
				OpenCPort(offCGrafPtr);
				MoveHHi(Handle(offCGrafPtr^.portPixMap));   { Arrgh !! }
				HLock(Handle(offCGrafPtr^.portPixMap));
				maxDepth := offCGrafPtr^.portPixMap^^.pixelSize;

			END;   { ELSE: aMac2 }

    { CanNOT use my GlobalLocal PROC because we may have dragged our           }
    { window to a secondary screen.  "Global" here is with respect to          }
    { the main screen and "Local" is with respect to the secondary screen.     }
    {                                                                          }
    { From Forrest Tanaka:                                                     }
    { When GetMaxDevice returns the secondary screen's GDevice & we set that   }
    { to the current GDevice, then OpenCPort creates a CGrafPort which has a   }
    { portRect = GetMainDevice^^.gdPMap^^.bounds which is in the global        }
    { coordinates for all the screens' pixel images with a topLeft = (0,0).    }
    { The new CGrafPort.portPixMap^^.bounds is in the local coordinates of     }
    { the secondary screen with a topLeft = (0,-640), for example.             }
    {                                                                          }
    { In effect, the port that OpenCPort gives you is NOT a valid port because }
    { the portRect pertains to the wrong screen.  This means that calling      }
    { GlobalLocal shifts the localRect waaaaay over somewhere ... BECAUSE      }
    { the difference between the above portRect and PixMap bounds is SO large: }

			localRect := globRect;
			WITH screenBits.bounds DO
				OffsetRect(localRect, left, top);

    { However, before we do ANYthing more, we should set the         }
    { off-screen's visRgn to the FULL size of the input rect so the  }
    { image stays whole even when the window has been dragged partly }
    { beyond the physical edge(s) of the screen.  Otherwise, the     }
    { visRgn^^.rgnBBox in local coordinates remains equal to         }
    { screenBits.bounds as initialized when _Open(C)Port was called: }

			IF aMac2 THEN
			BEGIN
				RectRgn(offCGrafPort.visRgn, localRect);
				offCGrafPort.portRect := localRect;
			END
			ELSE
			BEGIN
				RectRgn(offGrafPort.visRgn, localRect);
				offGrafPort.portRect := localRect;
			END;

    { We are now ready to calculate the size of the pixel image we will need.  }
    { Then we can set the location-specific and size-specific information of   }
    { the pixel map.  Since Color QuickDraw distinguishes between a bitmap and }
    { a pixel map by checking the high bit of rowBytes, we need to set it.     }

			WITH localRect DO
			BEGIN
				offRowBytes := (maxDepth * (right - left) + 15) DIV 16;   { # of words.    }
				IF ODD(offRowBytes) THEN                                 { ... made even. }
					offRowBytes := offRowBytes + 1;
				offRowBytes := offRowBytes * 2;                           { Back to bytes. }
				sizeOfOff := LONGINT(bottom - top) * offRowBytes;
			END;   { WITH }

			myBits := NewClearHandle(sizeOfOff);    { Allocate space for the pixel image.}
			IF MemError <> noErr THEN
				ErrorOut(NewBaseAddrPtrError);
			MoveHHi(myBits);
			HLock(myBits);

    { NOTE that we're filling in the BitMap/PixMap fields of the new Port }
    { directly, so we do NOT call _SetPortBits or _SetCPortPix later:     }

			IF aMac2 THEN
			BEGIN
				WITH offCGrafPtr^.portPixMap^^ DO
				BEGIN
					baseAddr := myBits^;
					rowBytes := offRowBytes + $8000;    { Remember to be a PixMap.}
					bounds := localRect;
				END;   { WITH }

				offBitMapPtr := BitMapPtr(offCGrafPtr^.portPixMap^);
			END   { IF aMac2 }

			ELSE   { definitely ... "YUCKY" black-and-white. }

			BEGIN
				WITH offGrafPtr^.portBits DO
				BEGIN
					baseAddr := myBits^;
					rowBytes := offRowBytes;
					bounds := localRect;
				END;

				offBitMapPtr := @offGrafPtr^.portBits;
			END;

			IF aMac2 THEN
			BEGIN
      { Next, we clone the color table of the maxDevice }
      { and put it into our off-screen pixel map.       }

				ourCTHandle := myMaxDevice^^.gdPMap^^.pmTable;
				err := HandToHand(Handle(ourCTHandle));    { Clone it. }
				IF err <> noErr THEN
					ErrorOut(CloneHdlError);

				WITH ourCTHandle^^ DO
					FOR i := 0 TO ctSize DO
						ctTable[i].value := i;

      { The following is required to convert GDevice cluts to Pixmap cluts:     }
      {                                                                         }
      { NOTE: THINK Pascal <v 2.0> re-named the field "ctFlags" to "transIndex" }

				ourCTHandle^^.ctFlags := BAND(ourCTHandle^^.ctFlags, $7FFF);
				tempSeed := GetCTSeed;                      { Thanks, Scott Knaster !! }
				ourCTHandle^^.ctSeed := tempSeed;
				offCGrafPtr^.portPixMap^^.pmTable := ourCTHandle;    { --> the off-screen map. }
			END;   { IF aMac2 }

			myRect := localRect;          { Return local screen coordinates. }

		END;   { WITH COSHdl^^ DO }

		ErrorOut(noErr);                { Whew !! }

	END;   { CreateOffScreen}



{ ******************* }
{ Back to "Square 1": }
{ ******************* }


	PROCEDURE ToOnScreen (COSHdl: OffScreenRecHdl);

	BEGIN

		WITH COSHdl^^ DO                { COSHdl is locked coming in. }
		BEGIN
			IF aMac2 THEN
				SetGDevice(oldDevice);
			SetPort(origPort);
		END;   { WITH }

	END;   { ToOnScreen}



{ **************************** }
{ Out with the new.            }
{                              }
{ Whoops -- I meant the old !! }
{ **************************** }


	PROCEDURE DisposOffScreen (VAR COSHdl: OffScreenRecHdl);

		LABEL
			100;

	BEGIN

		IF COSHdl = NIL THEN
			EXIT(DisposOffScreen);

		WITH COSHdl^^ DO
		BEGIN

			IF CreateOffScreenError = MaxDevError THEN
				GOTO 100;

    { NewBaseAddrPtrError or CloneHdlError or noErr ... }

			IF aMac2 THEN
			BEGIN
				IF CreateOffScreenError = noErr THEN
					DisposHandle(Handle(ourCTHandle));
				HUnlock(Handle(offCGrafPtr^.portPixMap));
				CloseCPort(offCGrafPtr);
			END
			ELSE
				ClosePort(offGrafPtr);

			IF CreateOffScreenError <> NewBaseAddrPtrError THEN
			BEGIN
				HUnlock(myBits);
				DisposHandle(myBits);
			END;

		END;   { WITH }
100:
		HUnlock(Handle(COSHdl));
		DisposHandle(Handle(COSHdl));
		COSHdl := NIL;                                            { Mark as gone ... }

	END;   { DisposOffScreen }




END. { UNIT = OffscreenSubs }