{ ******************************************************** }
{ "wBMScrollSubs.p"                                        }
{                                                          }
{ derived from Dan Weston's Assembly Code                  }
{                                                          }
{ by John A. Love, III [ Washington Apple Pi Users' Group] }
{                                                          }
{ using Symantec's "THINK Lightspeed Pascal", v 3.02       }
{                                                          }
{ ******************************************************** }


UNIT wBMScrollSubs;

INTERFACE

	USES
		Quickdraw, wBMGlobals, wBMMiscSubs;


	FUNCTION ScrollHoriz (wp: WindowPtr): ControlHandle;
	FUNCTION ScrollVert (wp: WindowPtr): ControlHandle;
	PROCEDURE ScrollShow (wp: WindowPtr);
	PROCEDURE ScrollHide (wp: WindowPtr);
	PROCEDURE InvalidScroll (wp: WindowPtr);
	PROCEDURE ValidScroll (wp: WindowPtr);
	PROCEDURE ScrollResize (wp: WindowPtr);
	FUNCTION GetWindowType (window: WindowPtr): INTEGER;   { ... another odd-ball. }




IMPLEMENTATION




{ -------------------------------- }
{ Retrieve Control Handle, if any: }
{ -------------------------------- }

	FUNCTION ScrollHoriz (wp: WindowPtr): ControlHandle;


	BEGIN

		ourControl := WindowPeek(wp)^.controlList;

		WHILE ourControl <> NIL DO
		BEGIN
			IF ourControl^^.contrlRFcon = horizScrollID THEN
				LEAVE;
			ourControl := ourControl^^.nextControl;
		END;   { WHILE ourControl <> NIL }

		ScrollHoriz := ourControl;

	END;   { ScrollHoriz }



	FUNCTION ScrollVert (wp: WindowPtr): ControlHandle;


	BEGIN

		ourControl := WindowPeek(wp)^.controlList;

		WHILE ourControl <> NIL DO
		BEGIN
			IF ourControl^^.contrlRFcon = vertScrollID THEN
				LEAVE;
			ourControl := ourControl^^.nextControl;
		END;   { WHILE }

		ScrollVert := ourControl;

	END;   { ScrollVert }



{ ------------------ }
{ Hello, or GoodBye: }
{ ------------------ }

	PROCEDURE ScrollShow (wp: WindowPtr);


	BEGIN

		ourControl := ScrollVert(wp);
		;
		IF ourControl <> NIL THEN
			ShowControl(ourControl);
  { ----- }
		ourControl := ScrollHoriz(wp);
		;
		IF ourControl <> NIL THEN
			ShowControl(ourControl);

	END;   { ScrollShow }



	PROCEDURE ScrollHide (wp: WindowPtr);


	BEGIN

		ourControl := ScrollVert(wp);
		;
		IF ourControl <> NIL THEN
			HideControl(ourControl);
  { ----- }
		ourControl := ScrollHoriz(wp);
		;
		IF ourControl <> NIL THEN
			HideControl(ourControl);

	END;   { ScrollHide }



{ ----------------------------------------------------------------- }
{ Explicitly include the Scroll Bars in the window's Update region. }
{ This Update region will purposely overlap the Grow Box.           }
{ ----------------------------------------------------------------- }

	PROCEDURE InvalidScroll (wp: WindowPtr);


	BEGIN

		WITH updateRect DO
		BEGIN
			IF ScrollVert(wp) <> NIL THEN
			BEGIN
				updateRect := wp^.portRect;
				left := right + frame - scrollWidth;
				InvalRect(updateRect);
			END;   { IF a vertical Scroll Bar }
			;
			IF ScrollHoriz(wp) <> NIL THEN
			BEGIN
				updateRect := wp^.portRect;
				top := bottom + frame - scrollHeight;
				InvalRect(updateRect);
			END;   { IF }
		END;   { WITH }

	END;   { InvalidScroll }



	PROCEDURE ValidScroll (wp: WindowPtr);


	BEGIN

		WITH updateRect DO
		BEGIN
			IF ScrollVert(wp) <> NIL THEN
			BEGIN
				updateRect := wp^.portRect;
				left := right + frame - scrollWidth;
				ValidRect(updateRect);
			END;   { IF a vertical Scroll Bar }
			;
			IF ScrollHoriz(wp) <> NIL THEN
			BEGIN
				updateRect := wp^.portRect;
				top := bottom + frame - scrollHeight;
				ValidRect(updateRect);
			END;   { IF }
		END;   { WITH }

	END;   { ValidScroll }



	PROCEDURE ScrollResize (wp: WindowPtr);
{ Remember, the portRect does NOT include the window frame, }
{ whereas the Scroll Bar and Grow Box sizes do:             }

		VAR
			contentRect: Rect;
			ctlWidth, ctlHeight: INTEGER;
			ctlTop, ctlLeft, ctlBottom, ctlRight: INTEGER;

	BEGIN

		contentRect := wp^.portRect;

  { Need to clip to new portRect for growing or zooming because window }
  { is still clipped to OLD portRect.  Otherwise, upon enlarging the   }
  { window, the ScrollShow PROC would NOT display the Scroll Bars:     }

		ClipRect(contentRect);

		ScrollHide(wp);                    { Hide-and-Go Seek !! }

		ourControl := ScrollVert(wp);
		IF ourControl <> NIL THEN
		BEGIN
			WITH contentRect DO
			BEGIN
				ctlTop := top - frame;
				ctlLeft := (right + frame) - scrollWidth;
				ctlBottom := (bottom + frame) - growBoxSize;
				ctlHeight := ctlBottom - ctlTop;
			END;   { WITH contentRect }

			SizeControl(ourControl, scrollWidth, ctlHeight);
			MoveControl(ourControl, ctlLeft, ctlTop);
		END;   { IF ourControl <> NIL }
  { ----- }
		ourControl := ScrollHoriz(wp);
		IF ourControl <> NIL THEN
		BEGIN
			WITH contentRect DO
			BEGIN
				ctlTop := (bottom + frame) - scrollHeight;
				ctlLeft := left - frame;
				ctlRight := (right + frame) - growBoxSize;
				ctlWidth := ctlRight - ctlLeft;
			END;   { WITH }

			SizeControl(ourControl, ctlWidth, scrollHeight);
			MoveControl(ourControl, ctlLeft, ctlTop);
		END;   { IF }

		ScrollShow(wp);                    { Peek-a-Boo !! }

	END;   { ScrollResize }



	FUNCTION GetWindowType (window: WindowPtr): INTEGER;
{ Another circularity problem for this routine. }
{ Here, window type = Window Definition ID.     }

		CONST
			RomMapInsertLoc = $B9E;
			mapTrue = $FFFF;

		VAR
			varCode, WDEFRsrcID, wType: INTEGER;
			WDEFHandle: Handle;
			WDEFType: ResType;
			WDEFName: Str255;


	BEGIN

		varCode := GetWVariant(window);

  { Now, what about rDocProc types since their Variation  }
  { Codes duplicate those of some standard types such as  }
  { documentProc & dBoxProc.  I could call:               }
  {                                                       }
  { regionSize := WindowPeek(window)^.strucRgn^^.rgnSize; }
  {                                                       }
  { If regionSize = 10, then rgnBBox is rectangular; so   }
  { if <> 10, we've got an rDocProc.  HOWEVER,  if the    }
  { window is invisible because I've not yet called       }
  { _ShowWindow, rgnBBox is empty and regionSize STILL    }
  { equals 10.  The solution is simple ... call           }
  { GetWindowType when the window is being activated.     }
  { Better yet ... the solution presented below, thanks   }
  { to MacDTS, avoids this workaround.  In addition,      }
  { MacDTS' solution avoids the pitfalls of "Murphy"      }
  { inventing a totally new window type with a regionSize }
  { that dupes that of rDocProc.                          }

		WDEFHandle := Handle(QuickStrip(Ptr(WindowPeek(window)^.windowDefProc)));
		;
		LoadResource(WDEFHandle);                 { May have been purged ... }
  { Debugger;                      }
  { IF ResError = resNotFound THEN }
  {   EXIT(GetWindowType);         }
		wordPtr(RomMapInsertLoc)^ := mapTrue;     { Thanks, Ben Cranston !! }
		GetResInfo(WDEFHandle, WDEFRsrcID, WDEFType, WDEFName);

		wType := 16 * WDEFRsrcID + varCode;

		IF (wType = documentProc) | (wType = zoomDocProc) THEN
			hasGrowIcon := TRUE
		ELSE
			hasGrowIcon := FALSE;
		;
		hasGrowIcon := hasGrowIcon | (ScrollHoriz(window) <> NIL) | (ScrollVert(window) <> NIL);

		GetWindowType := wType;

	END;   { GetWindowType }




END.   { UNIT = wBMScrollSubs }