unit iterators;
interface
uses
	ObjIntf;

type
	iterator = object (TObject)
		subject,	{ the subject data structure }
		i			{ the loop variable - updated by method 'another' }
			: TObject;
		procedure init (theSubject : TObject);
		procedure loopReset;
		function another (var anObject : univ TObject) : boolean;
		end;

implementation

{ the iterator must be associated with a data structure that
  will be the loop subject }
procedure iterator.init (theSubject : TObject);
begin
	subject := theSubject;
	i := nil end;

{ generic loop reset just clears out the iteration variable }	
procedure iterator.loopReset;
begin
	i := nil end;

{ method 'another' serves up each element of the subject data structure
  exactly once, one at a time.
  override this function with the following behavior:
  1) if iterator.i is nil, start a loop by setting i to the first operand
  2) if iterator.i is not nil, advance i to next operand
  3) return i in anObject
  4) return true if anObject is non-nil
  5) when there are no more operands:
  	a) set i to nil (exit in reset condition)
	b) return nil in anObject
	c) return false }
function iterator.another (var anObject : univ TObject) : boolean;
begin end;

end.