{$SETC compileForMPW := true } { if false, then compile for application }
{ How to use:
	1) create a memCheckObj (or a descendant) with new(aMemCheckObj)
		(this can be 'new(memCheck)' if you don't override any methods)
	2) initialize before making any more 'new(someObject)' or 'newHandle'
		calls
	3) loop on gotObject until new(someObject) succeeds, as in:
		repeat
			new (obj)
			until memCheck.gotObject (obj);

	Advanced features:
	1) override collectGarbage with application dependant code to
		free up space (optional)
	2) override outOfMemExit to do any clean up before exiting through
		inherited outOfMemExit (optional) 
		
	Note:
	The outOfMemAlertId parameter to initMemCheck is the id of an
	alert that will be posted before the program is halted.  It is
	ignored in MPW mode. }
	
unit memErr;
interface
uses
	{$IFC compileForMPW}
	PasLibIntf,
	IntEnv,
	{$ELSEC}
	Dialogs,
	{$ENDC}
	ObjIntf;

type
	memCheckObj = object (TObject) 
		memRetryCount : integer;
		procedure init;
		function gotObject (o : TObject) : boolean;
		function collectGarbage : boolean;
		procedure outOfMemExit;
		end;
		
procedure initMemCheck(mc : memCheckObj; outOfMemAlertId : integer);

var
	memCheck : memCheckObj; { reference this object in your code }


implementation { ------------------------------------------------------- }

var
	theAlertId : integer;

procedure memCheckObj.init;
begin
	memRetryCount := 0 end;

function memCheckObj.gotObject (o : TObject) : boolean;
begin
	if o <> nil { was newHandle successful? }
	then begin
		gotObject := true;
		memRetryCount := 0 end
	else begin
		gotObject := false;
		if collectGarbage { could we make any space? }
		then memRetryCount := succ (memRetryCount)
		else outOfMemExit end end;

{ If your data structures can be garbage collected, override this method.
	1) return true if any space was made
	2) use memRetryCount to detect multiple collectGarbage calls for 
		the same 'new()'.  memRetryCount=0 on first call. }
function memCheckObj.collectGarbage : boolean;
begin
	collectGarbage := false end;

{ override with clean-up routine, if necessary }
procedure memCheckObj.outOfMemExit;
{$IFC compileForMPW}
begin
	writeln (diagnostic, 'out of memory');
	halt end;
{$ELSEC}
var
	dummy : integer;
begin
	dummy := StopAlert (theAlertId);
	halt end;
{$ENDC}

procedure initMemCheck(mc : memCheckObj; outOfMemAlertId : integer);
begin
	if mc <> nil
	then begin
		memCheck := mc;
		theAlertId := outOfMemAlertId;
		{$IFC not compileForMPW}
		CouldAlert(theAlertId); { do this now, while we have the memory! }
		{$ENDC}
		memCheck.init end
	else halt end;

end.