{ Arbitrarily large stacks of any kind of objects }

unit stacks;
interface
uses
	ObjIntf, iterators, lists, memCheck;

const
	stackFrameDepth = 15; { one more cell per stack frame than this number }
	stackFrameFull = stackFrameDepth + 1;
	
type
	stackObj = object (listObj)
		procedure push (o : TObject);
		procedure pop (var o : univ TObject);
		procedure pick (n : integer; var o : univ TObject);
		function okToPush (o : TObject) : boolean;
		procedure rejectPush (o : TObject);
		end;

	framePointerType = 0..stackFrameFull;
	
	stackFrameObj = object (listElementObj)
		framePointer : framePointerType; { points to next empty cell }
		cell : array [0..stackFrameDepth] of TObject;
		procedure init; override;
		end;
				
	stackSweepObj = object (iterator)
		subjectFrame : stackFrameObj;
		subjectPointer : framePointerType;
		function another (var anObject : univ TObject) : boolean; override;
		end;
	
implementation

{ A stack is stored in a list of stack frames.  push and pop only
	add/delete a frame when absolutely necessary, in order to avoid
	duplicating work when successive push/pop or pop/push operations
	occur at stack frame boundaries. }

procedure stackObj.push (o : TObject);
	
	procedure addAFrame;
	var
		f : stackFrameObj;
	begin
		repeat
			new (f)
			until memCheck.gotObject (f);
		f.init;
		self.insert (f) end;

begin
	if okToPush (o)
	then begin
		{ don't combine these next two tests! MUST guard second test with first! }
		if first = nil
		then addAFrame; { stack is empty }
		if stackFrameObj(first).framePointer = stackFrameFull
		then addAFrame; { last frame is full -- start a new one }
		stackFrameObj(first).cell[stackFrameObj(first).framePointer] := o;
		stackFrameObj(first).framePointer
			:= succ (stackFrameObj(first).framePointer) end
	else rejectPush (o) end;

procedure stackObj.pop (var o : univ TObject);
var
	f : stackFrameObj;
begin
	if first = nil
	then { stack is empty }
		o := nil
	else begin
		if stackFrameObj(first).framePointer = 0
		then begin { roll back a frame }
			self.head(f);
			if f.release then dispose (f) end;
		if first = nil
		then { stack is empty }
			o := nil
		else begin { pluck out an object }
			stackFrameObj(first).framePointer
				:= pred (stackFrameObj(first).framePointer);
			o := stackFrameObj(first).cell
				[stackFrameObj(first).framePointer] end end end;

{ pick 0 = top of stack, nil returned in o if we run off the end.
  pick makes no modification to the stack's contents. }
procedure stackObj.pick (n : integer; var o : univ TObject);
var
	frameNum : integer; { counts down to frame that contains the cell we want }
	frame : stackFrameObj;
begin
	if first = nil
	then
		o := nil
	else begin
		if n < stackFrameObj(first).framePointer
		then { cell is in first frame }
			o := stackFrameObj(first).cell
				[stackFrameObj(first).framePointer - (n+1)]
		else begin { chase links to correct frame; start with second frame }
			frameNum := ((n - stackFrameObj(first).framePointer)
				div stackFrameFull) - 1;
			frame := stackFrameObj(first.next);
			while frameNum > 0 do begin
				if frame <> nil { did we go off the end? }
				then frame := stackFrameObj(frame.next)
				else frameNum := 0; { off end - bail out early }
				frameNum := pred(frameNum) end;
			if frame <> nil
			then o := stackFrameObj(frame).cell
				[stackFrameDepth - ((n - stackFrameObj(first).framePointer)
					mod stackFrameFull)]
			else o := nil end end end;

{ default screen passes anything }
function stackObj.okToPush (o : TObject) : boolean;
begin
	okToPush := true end;

{ default exception trap is an ostrich -- it ignores the problem. }
procedure stackObj.rejectPush (o : TObject);
begin end;

procedure stackFrameObj.init; override;
begin
	inherited init;
	framePointer := 0 end;
	
function stackSweepObj.another(var anObject : univ TObject) : boolean; override;
begin
	{ test for loop reset condition }
	if i = nil
	then begin
		{ start at the top }
		subjectFrame := stackFrameObj(stackObj(subject).first);
		if subjectFrame <> nil
		then subjectPointer := subjectFrame.framePointer end;

	{ test for end of frame condition }
	if subjectFrame <> nil
	then begin
		if subjectPointer = 0
		then begin
			{ access next frame }
			subjectFrame := stackFrameObj(subjectFrame.next);
			if subjectFrame <> nil
			then subjectPointer := subjectFrame.framePointer end end;

	{ return an element }
	if subjectFrame <> nil
	then begin
		subjectPointer := pred (subjectPointer);
		i := subjectFrame.cell[subjectPointer] end
	else
		{ stack is empty }
		i := nil;
		
	{ exit }
	anObject := i;
	another := anObject <> nil end;

end.