{*******************************************************************************
UMenuItemCommand.inc1.p
*******************************************************************************}

USES
	{  Implementation Use }
	Resources,
	Script,
	Dialogs;				{ TToolboxDialogCmd }
	

{###############################################################################
TMenuItemCommand
	This is an abstract class, from which all commands created by
	DoMenuCommand() should (must?) descend.
###############################################################################}
	
{}
{$S ADoCommand}

PROCEDURE TMenuItemCommand.IMenuItemCommand(
						itsCmdNumber:		CmdNumber;
						itsDocument:		TDocument;
						itsView:			TView;
						itsScroller:		TScroller);
	VAR
		menuNumber:		INTEGER;
		itemNumber:		INTEGER;
	
	BEGIN
	ICommand(
		itsCmdNumber,
		itsDocument,
		itsView,
		itsScroller);

	CmdToMenuItem(itsCmdNumber, menuNumber, itemNumber);
	SetMenuNumber(menuNumber);
	SetItemNumber(itemNumber);
	END;  { IMenuItemCommand }

{}
{$S ADoCommand}

PROCEDURE TMenuItemCommand.SetMenuNumber(
						menuNumber:			INTEGER);
	BEGIN
	fMenuNumber := menuNumber;
	END;  { SetMenuNumber }
	
{}
{$S ADoCommand}

PROCEDURE TMenuItemCommand.SetItemNumber(
						itemNumber:			INTEGER);
	BEGIN
	fItemNumber := itemNumber;
	END;  { SetItemNumber }
	
{}
{$S ADoCommand}

FUNCTION TMenuItemCommand.GetMenuNumber
						: INTEGER;
	BEGIN
	GetMenuNumber := fMenuNumber;
	END;  { GetMenuNumber }
	
{}
{$S ADoCommand}

FUNCTION TMenuItemCommand.GetItemNumber
						: INTEGER;
	BEGIN
	GetItemNumber := fItemNumber;
	END;  { GetItemNumber }
	
{}
{$S AFields}

{$IFC qInspector}
PROCEDURE TMenuItemCommand.Fields(
						PROCEDURE DoToField(
							fieldName:		Str255;
							fieldAddr:		Ptr;
							fieldType:		INTEGER));
						OVERRIDE;
	BEGIN
	DoToField('TMenuItemCommand', NIL, bClass);
	DoToField('fMenuNumber', @fMenuNumber, bInteger);
	DoToField('fItemNumber', @fItemNumber, bInteger);
	
	INHERITED Fields(DoToField);
	END;  { Fields }
{$ENDC qDebug}
	
	
	
{###############################################################################
TPoseDialogCmd
	This is an abstract class, from which all commands which are intended
	to display dialogs should descend.
	
	It has two main subclasses:  TMacAppDialogCmd and TToolboxDialogCmd.
	TMacAppDialogCmd poses a MacApp dialog, while TToolboxDialogCmd poses
	a Toolbox dialog.
###############################################################################}
	
{}
{$S ADoCommand}

PROCEDURE TPoseDialogCmd.IPoseDialogCmd(
						itsCmdNumber:		CmdNumber;
						itsDocument:		TDocument;
						itsView:			TView;
						itsScroller:		TScroller);
	BEGIN
	{ 1: Calling parent class' initialization method }
	IMenuItemCommand(
		itsCmdNumber,
		itsDocument,
		itsView,
		itsScroller);

	{ 2: 'overriding' parent class' default initializations }
	{ this command doesn't change the document; the command it creates does }
	fCanUndo	  := FALSE;		{ when FALSE, object is freed after DoIt() }
								{ if fFreeOnCompletion is TRUE (the default) }
	fCausesChange := FALSE;		{ when FALSE, doesn't mark document as changed }
	
	{ 3: Performing local initialization }
	SetCancelled(FALSE);
	END;  { IPoseDialogCmd }
	
{}
{$S ADoCommand}

PROCEDURE TPoseDialogCmd.SetCancelled(
						cancelled:			BOOLEAN);
	BEGIN
	fCancelled := cancelled;
	END;  { SetCancelled }
	
{}
{$S ADoCommand}

FUNCTION  TPoseDialogCmd.GetCancelled
						: BOOLEAN;
	BEGIN
	GetCancelled := fCancelled;
	END;  { GetCancelled }
	
{}
{$S ADoCommand}

PROCEDURE TPoseDialogCmd.CreateTheDialog;
	{ This routine creates the dialog to be posed, and assigns a
		reference to it to an instance variable (which is added
		in TMacAppDialogCmd and TToolboxDialogCmd). }
	BEGIN
	{ does nothing; implemented in TMacAppDialogCmd and TToolboxDialogCmd }
	END;  { CreateTheDialog }
	
{}
{$S ADoCommand}

PROCEDURE TPoseDialogCmd.InitTheDialog;
	{ This routine initializes the dialog to be posed. }
	BEGIN
	{ does nothing }
	END;  { InitTheDialog }
	
{}
{$S ADoCommand}
				
FUNCTION  TPoseDialogCmd.CreateTheCommand
						: TCommand;
	{ This routine creates the command that is to be posted as a result
		of having the user accept the dialog. }
	BEGIN
	CreateTheCommand := NIL;
	END;  { CreateTheCommand }
	
{}
{$S ADoCommand}

PROCEDURE TPoseDialogCmd.PoseTheDialog;
	{ Actually poses the dialog. }
	BEGIN
	{ does nothing; implemented in TMacAppDialogCmd and TToolboxDialogCmd }
	END;  { PoseTheDialog }
	
{}
{$S ADoCommand}

PROCEDURE TPoseDialogCmd.DropTheDialog;
	{ Cleans up after exiting the dialog. }
	BEGIN
	{ does nothing; implemented in TMacAppDialogCmd and TToolboxDialogCmd }
	END;  { DropTheDialog }
	
{}
{$S ADoCommand}

PROCEDURE TPoseDialogCmd.DoIt;
						OVERRIDE;
	{ This routine calls InitTheDialog() to initialize the dialog associated
		with this command, PoseTheDialog() to pose it, and if it is accepted
		by the user, CreateTheCommand() to create the command that is to be
		posted as a result.  The  resulting command is then posted with
		PostCommand(). }
	VAR
		theCommand:		TCommand;
	
	BEGIN
	{ initialize the dialog as needed }
	InitTheDialog;
	
	{ pose it }
	PoseTheDialog;
	
	{ create the resulting command }
	IF fCancelled
	THEN
		theCommand := NIL				{ the user cancelled the dialog }
	ELSE
		theCommand := CreateTheCommand;	{ the user accepted the dialog }
	
	{ post the resulting command }
	IF (theCommand <> NIL)
	THEN
		gTarget.PostCommand(theCommand);
	
	{ clean up }
	DropTheDialog;
	END;  { DoIt }
	
{}
{$S AFields}
				
{$IFC qInspector}
PROCEDURE TPoseDialogCmd.Fields(
						PROCEDURE DoToField(
							fieldName:		Str255;
							fieldAddr:		Ptr;
							fieldType:		INTEGER));
						OVERRIDE;
	BEGIN
	DoToField('TPoseDialogCmd', NIL, bClass);
	DoToField('fCancelled', @fCancelled, bBoolean);
	
	INHERITED Fields(DoToField);
	END;  { Fields }
{$ENDC qDebug}
	
	
	
{###############################################################################
TMacAppDialogCmd
	This is an abstract class, from which all command which are intended to
	display dialogs should descend.
###############################################################################}
	
{}
{$S ADoCommand}
		
PROCEDURE TMacAppDialogCmd.IMacAppDialogCmd(
						itsCmdNumber:		CmdNumber;
						itsDocument:		TDocument;
						itsView:			TView;
						itsScroller:		TScroller;
						itsDialogWindResID:	INTEGER;
						itsDialogViewSig:	IDType);
	BEGIN
	{ 1: Calling parent class' initialization method }
	IPoseDialogCmd(
		itsCmdNumber,
		itsDocument,
		itsView,
		itsScroller);

	{ 2: 'overriding' parent class' default initializations }
	{ none }
	
	{ 3: Performing local initialization }
	SetDialogWindResID(itsDialogWindResID);
	SetDialogViewSig(itsDialogViewSig);
	
	fTheDialog := NIL;  { for now }
	fTheWindow := NIL;  { for now }
	
	{ initialize fTheDialog and fTheWindow }
	CreateTheDialog;
	END;  { IMacAppDialogCmd }
	
{}
{$S ADoCommand}
		
PROCEDURE TMacAppDialogCmd.Free;
	VAR
		whereItsAt:		INTEGER;
	
	BEGIN
	IF (fTheWindow <> NIL)
	THEN
		BEGIN
		IF fTheWindow.fFreeOnClosing
		THEN
			fTheWindow.Close
		ELSE
			BEGIN
			fTheWindow.Close;
			fTheWindow.Free;
			END;  { else }
		
		fTheWindow := NIL;
		fTheDialog := NIL;
		END;  { then }

(*
	{ needed for recurring commands  not yet implemented !!! }
	whereItsAt := gApplication.fCommandQueue.GetSameItemNo(SELF);
	
	IF (whereItsAt <> kEmptyIndex)
	THEN
		gApplication.fCommandQueue.AtDelete(whereItsAt);
*)
	
	INHERITED Free;
	END;  { Free }

{}
{$S ADoCommand}

PROCEDURE TMacAppDialogCmd.SetDialogWindResID(
						dialogWindResID:	INTEGER);
	BEGIN
	fDialogWindResID := dialogWindResID;
	END;  { SetDialogWindResID }
	
{}
{$S ADoCommand}
						
PROCEDURE TMacAppDialogCmd.SetDialogViewSig(
						dialogViewSig:		IDType);
	BEGIN
	fDialogViewSig := dialogViewSig;
	END;  { SetDialogViewSig }
	
{}
{$S ADoCommand}
						
PROCEDURE TMacAppDialogCmd.SetTheDialog(
						theDialog:			TDialogView);
	BEGIN
	fTheDialog := theDialog;
	END;  { SetTheDialog }
	
{}
{$S ADoCommand}
								
PROCEDURE TMacAppDialogCmd.SetTheWindow(
						theWindow:			TWindow);
	BEGIN
	fTheWindow := theWindow;
	END;  { SetTheWindow }
	
{}
{$S ADoCommand}
								
PROCEDURE TMacAppDialogCmd.SetDismisser(
						dismisser:			IDType);
	BEGIN
	fDismisser := dismisser;
	END;  { SetDismisser }
	
{}
{$S ADoCommand}

FUNCTION  TMacAppDialogCmd.GetDialogWindResID
						: INTEGER;
	BEGIN
	GetDialogWindResID := fDialogWindResID;
	END;  { GetDialogWindResID }
	
{}
{$S ADoCommand}

FUNCTION  TMacAppDialogCmd.GetDialogViewSig
						: IDType;
	BEGIN
	GetDialogViewSig := fDialogViewSig;
	END;  { GetDialogViewSig }
	
{}
{$S ADoCommand}

FUNCTION  TMacAppDialogCmd.GetTheDialog
						: TDialogView;
	BEGIN
	GetTheDialog := fTheDialog;
	END;  { GetTheDialog }
	
{}
{$S ADoCommand}

FUNCTION  TMacAppDialogCmd.GetTheWindow
						: TWindow;
	BEGIN
	GetTheWindow := fTheWindow;
	END;  { GetTheWindow }
	
{}
{$S ADoCommand}

FUNCTION  TMacAppDialogCmd.GetDismisser
						: IDType;
	BEGIN
	GetDismisser := fDismisser;
	END;  { GetDismisser }
	
{}
{$S ADoCommand}

PROCEDURE TMacAppDialogCmd.CreateTheDialog;
						OVERRIDE;
	{ This routine creates the dialog to be posed, assigns it to
		fTheDialog, and assigns a reference to the window containing
		the dialog to fTheWindow. }
	VAR
		theWindow:		TWindow;
		theDialog:		TDialogView;
		diffDoc:		BOOLEAN;
	
	BEGIN
	theWindow := GetTheWindow;
	IF (theWindow = NIL)
	THEN
		theWindow := NewTemplateWindow(fDialogWindResID, fChangedDocument)
	ELSE
		BEGIN
		IF (theWindow.fDocument <> fChangedDocument)
		THEN
			theWindow.Free;
		
		theWindow := NewTemplateWindow(fDialogWindResID, fChangedDocument);
		END;  { else }
	FailNil(theWindow);
	
	theDialog := TDialogView(theWindow.FindSubView(fDialogViewSig));
	FailNil(theDialog);
	
	SetTheWindow(theWindow);
	SetTheDialog(theDialog);
	END;  { CreateTheDialog }
	
{}
{$S ADoCommand}

PROCEDURE TMacAppDialogCmd.PoseTheDialog;
	{ Actually poses the dialog. }
	VAR
		dismisser:		IDType;
	
	BEGIN
	{ pose the dialog }
	fTheWindow.Select;		{ bring the window to the front before showing it }
	dismisser := fTheDialog.PoseModally;
	
	{ take note of the user's response }
	SetDismisser(dismisser);
	SetCancelled(dismisser = fTheDialog.fCancelItem);
	END;  { PoseTheDialog }
	
{}
{$S ADoCommand}

PROCEDURE TMacAppDialogCmd.DropTheDialog;
	{ Cleans up after exiting the dialog. }
	VAR
		freeOnClosing:	BOOLEAN;
	
	BEGIN
	freeOnClosing := fTheWindow.fFreeOnClosing;
	fTheWindow.Close;
	
	IF (freeOnClosing)
	THEN
		BEGIN
		SetTheDialog(NIL);
		SetTheWindow(NIL);
		END;  { then }
	END;  { DropTheDialog }
	
{}
{$S AFields}
				
{$IFC qInspector}
PROCEDURE TMacAppDialogCmd.Fields(
						PROCEDURE DoToField(
							fieldName:		Str255;
							fieldAddr:		Ptr;
							fieldType:		INTEGER));
						OVERRIDE;
	BEGIN
	DoToField('TMacAppDialogCmd', NIL, bClass);
	
	DoToField('fDialogWindResID', @fDialogWindResID, bInteger);
	DoToField('fDialogViewSig', @fDialogViewSig, bIDType);
	DoToField('fTheDialog', @fTheDialog, bObject);
	DoToField('fTheWindow', @fTheWindow, bObject);
	DoToField('fDismisser', @fDismisser, bIDType);
	
	INHERITED Fields(DoToField);
	END;  { Fields }
{$ENDC qDebug}
	
	
	
{###############################################################################
TToolboxDialogCmd
	Commands of this class display a Toolbox modal dialog, posting a
	command when and if the dialog is accepted by the user.  It is intended
	to be an abstract class.  Each dialog should have a command class that
	poses it.  Such classes can be quite small  overrides of Fields(),
	InitMenuItemCommand(), CreateTheCommand(), and a new IClassName()
	routine  or quite complex, but most will be small.
###############################################################################}

{}
{$S ADoCommand}	
		
PROCEDURE TToolboxDialogCmd.IToolboxDialogCmd(
						itsCmdNumber:		CmdNumber;
						itsDocument:		TDocument;
						itsView:			TView;
						itsScroller:		TScroller;
						itsDialogResID:		INTEGER;
						itsDismisserSet:	ItemSet;
						itsFilter:			ProcPtr);
	BEGIN
	{ 1: Calling parent class' initialization method }
	IPoseDialogCmd(
		itsCmdNumber,
		itsDocument,
		itsView,
		itsScroller);

	{ 2: 'overriding' parent class' default initializations }
	{ none }
	
	{ 3: Performing local initialization }
	SetDialogResID(itsDialogResID);
	SetTheDialog(NIL);
	SetDismisser(-1);
	SetDismisserSet(itsDismisserSet);
	SetFilter(itsFilter);
	
	CreateTheDialog;
	END;  { IToolboxDialogCmd }

{}
{$S ADoCommand}	

PROCEDURE TToolboxDialogCmd.SetDialogResID(
						dialogResID:		INTEGER);
	BEGIN
	fDialogResID := dialogResID;
	END;  { SetDialogResID }

{}
{$S ADoCommand}	
						
PROCEDURE TToolboxDialogCmd.SetTheDialog(
						theDialog:			DialogPtr);
	BEGIN
	fTheDialog := theDialog;
	END;  { SetTheDialog }

{}
{$S ADoCommand}	
						
PROCEDURE TToolboxDialogCmd.SetDismisser(
						dismisser:			INTEGER);
	BEGIN
	fDismisser := dismisser;
	END;  { SetDismisser }

{}
{$S ADoCommand}	
						
PROCEDURE TToolboxDialogCmd.SetDismisserSet(
						dismisserSet:		ItemSet);
	BEGIN
	fDismisserSet := dismisserSet;
	END;  { SetDismisserSet }

{}
{$S ADoCommand}	
								
PROCEDURE TToolboxDialogCmd.SetFilter(
						filter:				ProcPtr);
	BEGIN
	fFilter := filter;
	END;  { SetFilter }

{}
{$S ADoCommand}	

FUNCTION  TToolboxDialogCmd.GetDialogResID
						: INTEGER;
	BEGIN
	GetDialogResID := fDialogResID;
	END;  { GetDialogResID }

{}
{$S ADoCommand}	

FUNCTION  TToolboxDialogCmd.GetTheDialog
						: DialogPtr;
	BEGIN
	GetTheDialog := fTheDialog;
	END;  { GetTheDialog }

{}
{$S ADoCommand}	

FUNCTION  TToolboxDialogCmd.GetDismisser
						: INTEGER;
	BEGIN
	GetDismisser := fDismisser;
	END;  { GetDismisser }

{}
{$S ADoCommand}	

FUNCTION  TToolboxDialogCmd.GetDismisserSet
						: ItemSet;
	BEGIN
	GetDismisserSet := fDismisserSet;
	END;  { GetDismisserSet }

{}
{$S ADoCommand}	
		
FUNCTION  TToolboxDialogCmd.GetFilter
						: ProcPtr;
	BEGIN
	GetFilter := fFilter;
	END;  { GetFilter }

{}
{$S ADoCommand}	

PROCEDURE TToolboxDialogCmd.CreateTheDialog;
						OVERRIDE;
	{ This routine creates the dialog to be posed, assigns it to
		fTheDialog. }
	BEGIN
	fTheDialog := GetNewCenteredDialog(fDialogResID, NIL, WindowPtr(-1));
	END;  { CreateTheDialog }

{}
{$S ADoCommand}	
		
PROCEDURE TToolboxDialogCmd.PostProcess(
					VAR	itemHit:			INTEGER);
	{ This routine is called after ModalDialog() in the REPEAT loop of
		PoseTheDialog(), to give the class a shot at post-processing
		events. }
	BEGIN
	{ does nothing }
	END;  { PostProcess }
	
{}
{$S ADoCommand}

PROCEDURE TToolboxDialogCmd.PoseTheDialog;
	{ Actually poses the dialog. }
	VAR
		itemHit:		INTEGER;
	
	BEGIN
	{ pose the dialog }
	REPEAT
		ModalDialog(fFilter, itemHit);
		PostProcess(itemHit);
	UNTIL (itemHit IN fDismisserSet);
	
	{ take note of the user's response }
	SetDismisser(itemHit);
	SetCancelled((itemHit = cancel));
	END;  { PoseTheDialog }
	
{}
{$S ADoCommand}

PROCEDURE TToolboxDialogCmd.DropTheDialog;
	BEGIN
	DisposDialog(fTheDialog);
	SetTheDialog(NIL);			{ paranoia }
	END;  { DropTheDialog }
	
{}
{$S AFields}
				
{$IFC qInspector}
PROCEDURE TToolboxDialogCmd.Fields(
						PROCEDURE DoToField(
							fieldName:		Str255;
							fieldAddr:		Ptr;
							fieldType:		INTEGER));
						OVERRIDE;
	BEGIN
	DoToField('TToolboxDialogCmd', NIL, bClass);
	
	DoToField('fDialogResID', 	@fDialogResID, 	bInteger);
	DoToField('fTheDialog', 	@fTheDialog, 	bGrafPtr);
	DoToField('fDismisser', 	@fDismisser, 	bInteger);
	DoToField('fDismisserSet', 	@fDismisser, 	bHexLongint);
	DoToField('fFilter', 		@fFilter, 		bHexLongint);
	
	INHERITED Fields(DoToField);
	END;  { Fields }
{$ENDC qDebug}
	
{}

