{*******************************************************************************
UMenuItemCommand.p
*******************************************************************************}
	
UNIT UMenuItemCommand;

INTERFACE
USES
	{  MacApp }
	UMacApp,

	{  Building Blocks }
	UDialog;

CONST
	kIgnoreDismisser	=	';-) ';		{ a smiley face! (anything will do) }
	
	kNotAMacAppDialog	=	'!!!!';		{ command displays a Toolbox dialog }
	kInvalidRsrcID		=	-32768;		{ command displays a Toolbox dialog }

TYPE
	{###########################################################################
	TMenuItemCommand
		This is an abstract class, from which all commands created by
		DoMenuCommand() should (must?) descend.
	###########################################################################}
	
	TMenuItemCommand		= OBJECT(TCommand)
		fMenuNumber:		INTEGER;
		fItemNumber:		INTEGER;
		
		PROCEDURE TMenuItemCommand.IMenuItemCommand(
								itsCmdNumber:		CmdNumber;
								itsDocument:		TDocument;
								itsView:			TView;
								itsScroller:		TScroller);
		
		PROCEDURE TMenuItemCommand.SetMenuNumber(
								menuNumber:			INTEGER);
								
		PROCEDURE TMenuItemCommand.SetItemNumber(
								itemNumber:			INTEGER);
		
		FUNCTION  TMenuItemCommand.GetMenuNumber
								: INTEGER;
		
		FUNCTION  TMenuItemCommand.GetItemNumber
								: INTEGER;
						
		{$IFC qInspector}
		PROCEDURE TMenuItemCommand.Fields(
								PROCEDURE DoToField(
									fieldName:		Str255;
									fieldAddr:		Ptr;
									fieldType:		INTEGER));
								OVERRIDE;
		{$ENDC qInspector}
		END;  { TMenuItemCommand }
	
	
	
	{###########################################################################
	TPoseDialogCmd
		This is an abstract class, from which all commands which are intended
		to display dialogs should descend.
		
		It has two main subclasses:  TMacAppDialogCmd and TToolboxDialogCmd.
		TMacAppDialogCmd poses a MacApp dialog, while TToolboxDialogCmd poses
		a Toolbox dialog.
	###########################################################################}
	
	TPoseDialogCmd		= OBJECT(TMenuItemCommand)
		fCancelled:			BOOLEAN;
		
		PROCEDURE TPoseDialogCmd.IPoseDialogCmd(
								itsCmdNumber:		CmdNumber;
								itsDocument:		TDocument;
								itsView:			TView;
								itsScroller:		TScroller);
		
		PROCEDURE TPoseDialogCmd.SetCancelled(
								cancelled:			BOOLEAN);
		
		FUNCTION  TPoseDialogCmd.GetCancelled
								: BOOLEAN;
		
		PROCEDURE TPoseDialogCmd.CreateTheDialog;
			{ This routine creates the dialog to be posed, and assigns a
				reference to it to an instance variable (which is added
				in TMacAppDialogCmd and TToolboxDialogCmd). }
		
		PROCEDURE TPoseDialogCmd.InitTheDialog;
			{ This routine initializes the dialog to be posed. }
						
		FUNCTION  TPoseDialogCmd.CreateTheCommand
								: TCommand;
			{ This routine creates the command that is to be posted as a result
				of having the user accept the dialog. }
		
		PROCEDURE TPoseDialogCmd.PoseTheDialog;
			{ Actually poses the dialog. }
		
		PROCEDURE TPoseDialogCmd.DropTheDialog;
			{ Cleans up after exiting the dialog. }
		
		PROCEDURE TPoseDialogCmd.DoIt;
								OVERRIDE;
			{ This routine calls CreateTheDialog() to create the dialog associated
				with this command, InitTheDialog() to initialize it, PoseTheDialog()
				to pose it, and if it is accepted by the user, CreateTheCommand()
				to create the command that is to be posted as a result.  The 
				resulting command is then posted with PostCommand(). }
						
		{$IFC qInspector}
		PROCEDURE TPoseDialogCmd.Fields(
								PROCEDURE DoToField(
									fieldName:		Str255;
									fieldAddr:		Ptr;
									fieldType:		INTEGER));
								OVERRIDE;
		{$ENDC qInspector}
		END;  { TPoseDialogCmd }
	
	
	
	{###########################################################################
	TMacAppDialogCmd
		Commands of this class display a MacApp modal dialog, posting a
		command when and if the dialog is accepted by the user.  It is intended
		to be an abstract class.  Each dialog should have a command class that
		poses it.  Such classes can be quite small  overrides of Fields(),
		InitMenuItemCommand(), and a new IClassName() routine  or quite
		complex, but most will be small.
	###########################################################################}
	
	TMacAppDialogCmd		= OBJECT(TPoseDialogCmd)
		fDialogWindResID:	INTEGER;   { resID of the dialog's window's 'view' }
		fDialogViewSig:		IDType;	   { signature of the dialog view }
		fTheDialog:			TDialogView;	{ the dialog to be posed }
		fTheWindow:			TWindow;		{ the window containing the dialog }
		fDismisser:			IDType;			{ the dialog's dismisser }
		
		PROCEDURE TMacAppDialogCmd.IMacAppDialogCmd(
								itsCmdNumber:		CmdNumber;
								itsDocument:		TDocument;
								itsView:			TView;
								itsScroller:		TScroller;
								itsDialogWindResID:	INTEGER;
								itsDialogViewSig:	IDType);
		
		PROCEDURE TMacAppDialogCmd.Free;
								OVERRIDE;
		
		PROCEDURE TMacAppDialogCmd.SetDialogWindResID(
								dialogWindResID:	INTEGER);
								
		PROCEDURE TMacAppDialogCmd.SetDialogViewSig(
								dialogViewSig:		IDType);
								
		PROCEDURE TMacAppDialogCmd.SetTheDialog(
								theDialog:			TDialogView);
								
		PROCEDURE TMacAppDialogCmd.SetTheWindow(
								theWindow:			TWindow);
								
		PROCEDURE TMacAppDialogCmd.SetDismisser(
								dismisser:			IDType);
		
		FUNCTION  TMacAppDialogCmd.GetDialogWindResID
								: INTEGER;
		
		FUNCTION  TMacAppDialogCmd.GetDialogViewSig
								: IDType;
		
		FUNCTION  TMacAppDialogCmd.GetTheDialog
								: TDialogView;
		
		FUNCTION  TMacAppDialogCmd.GetTheWindow
								: TWindow;
		
		FUNCTION  TMacAppDialogCmd.GetDismisser
								: IDType;
		
		PROCEDURE TMacAppDialogCmd.CreateTheDialog;
								OVERRIDE;
			{ This routine creates the dialog to be posed, assigns it to
				fTheDialog, and assigns a reference to the window containing
				the dialog to fTheWindow. }
		
		PROCEDURE TMacAppDialogCmd.PoseTheDialog;
								OVERRIDE;
			{ Actually poses the dialog. }
		
		PROCEDURE TMacAppDialogCmd.DropTheDialog;
								OVERRIDE;
			{ Cleans up after exiting the dialog. }
						
		{$IFC qInspector}
		PROCEDURE TMacAppDialogCmd.Fields(
								PROCEDURE DoToField(
									fieldName:		Str255;
									fieldAddr:		Ptr;
									fieldType:		INTEGER));
								OVERRIDE;
		{$ENDC qInspector}
		END;  { TMacAppDialogCmd }
	
	
	
	{###########################################################################
	TToolboxDialogCmd
		Commands of this class display a Toolbox modal dialog, posting a
		command when and if the dialog is accepted by the user.  It is intended
		to be an abstract class.  Each dialog should have a command class that
		poses it.  Such classes can be quite small  overrides of Fields(),
		InitMenuItemCommand(), CreateTheCommand(), and a new IClassName()
		routine  or quite complex, but most will be small.
	###########################################################################}
	
	itemRange = 1..30;				{ only items 1..30 may be dismissers }
	ItemSet = SET OF ItemRange;		{ set of items that can dismiss the dialog }
	
	TToolboxDialogCmd		= OBJECT(TPoseDialogCmd)
		fDialogResID:		INTEGER;	{ resID of the dialog's 'DLOG' resource }
		fTheDialog:			DialogPtr;	{ pointer to the dialog to be posed }
		fDismisser:			INTEGER;	{ item that dismissed the dialog }
		fDismisserSet:		ItemSet;	{ items that can dismiss the dialog }
		fFilter:			ProcPtr;	{ filter passed to ModalDialog() }
		
		PROCEDURE TToolboxDialogCmd.IToolboxDialogCmd(
								itsCmdNumber:		CmdNumber;
								itsDocument:		TDocument;
								itsView:			TView;
								itsScroller:		TScroller;
								itsDialogResID:		INTEGER;
								itsDismisserSet:	ItemSet;
								itsFilter:			ProcPtr);
		
		PROCEDURE TToolboxDialogCmd.SetDialogResID(
								dialogResID:		INTEGER);
								
		PROCEDURE TToolboxDialogCmd.SetTheDialog(
								theDialog:			DialogPtr);
								
		PROCEDURE TToolboxDialogCmd.SetDismisser(
								dismisser:			INTEGER);
								
		PROCEDURE TToolboxDialogCmd.SetDismisserSet(
								dismisserSEt:		ItemSet);
								
		PROCEDURE TToolboxDialogCmd.SetFilter(
								filter:				ProcPtr);
		
		FUNCTION  TToolboxDialogCmd.GetDialogResID
								: INTEGER;
		
		FUNCTION  TToolboxDialogCmd.GetTheDialog
								: DialogPtr;
		
		FUNCTION  TToolboxDialogCmd.GetDismisser
								: INTEGER;
		
		FUNCTION  TToolboxDialogCmd.GetDismisserSet
								: ItemSet;
		
		FUNCTION  TToolboxDialogCmd.GetFilter
								: ProcPtr;
		
		PROCEDURE TToolboxDialogCmd.CreateTheDialog;
								OVERRIDE;
			{ This routine creates the dialog to be posed, assigns it to
				fTheDialog. }
		
		PROCEDURE TToolboxDialogCmd.PostProcess(
							VAR	itemHit:			INTEGER);
			{ This routine is called after ModalDialog() in the REPEAT loop of
				PoseTheDialog(), to give the class a shot at post-processing
				events. }
		
		PROCEDURE TToolboxDialogCmd.PoseTheDialog;
								OVERRIDE;
		
		PROCEDURE TToolboxDialogCmd.DropTheDialog;
								OVERRIDE;
						
		{$IFC qInspector}
		PROCEDURE TToolboxDialogCmd.Fields(
								PROCEDURE DoToField(
									fieldName:		Str255;
									fieldAddr:		Ptr;
									fieldType:		INTEGER));
								OVERRIDE;
		{$ENDC qInspector}
		END;  { TToolboxDialogCmd }
		
	{}


IMPLEMENTATION
	{$I UMenuItemCommand.inc1.p}
END.  { UMenuItemCommand.p }
