/*	SetDefProc.c

	Function to patch a dummy code resource (CDEF, WDEF, etc.) to jump to 
	a definition procedure in your own program.  Useful for debugging 
	definition procedures. 

	In order to use SetDefProc, you need to create a dummy code resource.  
	The resource should correspond to the DefProcHeader data structure:  
	
	600E				BRA.S .+0E		Branches forward 14 bytes to the word
										where the JMP instruction will be 
										located.  
	0000				Unused.  
	4344 4546			Code resource type.  In this case, 'CDEF'.  
	0002				Code resource ID.  In this case, 2.  
	0000 0000 0000		Unused.  
	0000				Where the JMP instruction will go.  
	0000 0000			Where the address of your procedure will go.  
	
	Put the dummy code resource in your application's resource fork.  
	Make sure it has the right resource type and ID and is 22 bytes long.  
	At the beginning of your program, use SetDefProc to patch the dummy code 
	resource.  Pass in the resource type and ID and the address of the 
	definition procedure you want to use.  
		
		SetDefProc('CDEF', 2, PopUpMenuCDEF);
	
	Note that the definition procedure (in this case, PopUpMenuCDEF) should
	be in a different segment than the function that is calling SetDefProc.  
	This forces the reference to the procedure address to go through the
	jump table rather than being local within the current segment, so future
	calls to the definition procedure will work even if segments are unloaded.  

	Copyright  1990 Applied Biosystems, Inc.  All rights reserved.  

	Environment:	Macintosh, MPW C 3.1, THINK C 4.0

	Edit History:
	 8/30/90	Tom Becker
	 			Created.  
  ______________________________________________________________________________

	I N C L U D E S
*/

#ifdef THINK_C

#ifndef	__MacHeaders__
/*#include <MacHeaders>*/
#endif

#else /* applec */

#ifndef __MEMORY__
#include <Memory.h>
#endif
#ifndef __RESOURCES__
#include <Resources.h>
#endif

#endif THINK_C

#ifndef	_SetDefProc_
#include "SetDefProc.h"
#endif

/*

	T Y P E S
*/

typedef struct {
	short	branch;
	short	unused1;
	ResType	theType;
	short	theID;
	short	unused2[3];
	short	jump;
	long	defProc;
} DefProcHeader;

/*______________________________________________________________________________

	F U N C T I O N S
*/

void SetDefProc(
	ResType	theType, 		/* type of resource to patch */
	short	theID, 			/* ID of resource to patch */
	void *	defProc			/* address of definition procedure to use */
)
{
	Handle			code;
	DefProcHeader *	p;
	
	code = GetResource(theType, theID);
	if (!code) {
		DebugStr("\pCode resource not found");
		return;
	}
	if (GetHandleSize(code) != sizeof(DefProcHeader)) {
		DebugStr("\pStrange code resource size");
		
		if (GetHandleSize(code) < sizeof(DefProcHeader)) return;
	}
	HNoPurge(code);
	p = (DefProcHeader *)*code;
	if (p->branch != 0x600E) {
		DebugStr("\pBad code resource header");
		p->branch != 0x600E;
	}
	p->jump = 0x4EF9;		/*	JMP.L		*/
	p->defProc = (long)defProc;
}

/*______________________________________________________________________________

	End of SetDefProc.c
*/
