/* VCR Button CDEF.c													*/
/*																		*/
/*	By:																	*/
/*		Chris Faigle													*/
/*																		*/
/*	Developed for the MacWorld Expo New Adventures in Programming		*/
/*		panel MacTutor LIVE											*/
/*																		*/
/*																		*/
/*	Version 1.0															*/
/*	January, 1991														*/
/*																		*/
/*																		*/
/*	This software is in the Public Domain, providing the following:		*/
/*		No fee other than normal copying or bulletin-board time fees	*/
/*		may be charged for this software.  The CDEF may be included in	*/
/*		commercial software without fees or notices.  Portions of the	*/
/*		CDEF are copyright Symantec Corp.								*/
/*																		*/
/*	If you make enhancements to this software, PLEASE send me a copy!	*/
/*																		*/
/*	Address any correspondence/bug reports/suggestions to:				*/
/*	Chris Faigle														*/
/*	c/o Performance Software											*/
/*	575 SouthLake Blvd													*/
/*	Richmond, VA  23236													*/
/*	AppleLink:  CHRIS.FAIGLE											*/
/*																		*/
/*	Please Read VCR Button CDEF Instructions for more info			*/
/*																		*/

#include "VCR Button CDEF.h"

#include "ControlMgr.h"
#include "SetUpA4.h"

#define	INACTIVE					255
#define	LOW_31_BITS_MASK			0x7FFFFFFF
#define	CODE_MASK					0x000F
#define	HI_BYTE						0xFF00
#define	LO_BYTE						0x00FF

#define	IN_BUTTON					1
#define	IN_SHADOW					2


typedef struct
{
	short		save_size;
	short		save_font;
	Style		save_face;
	PenState	save_pen;
} pen_font_struct,**pen_font_handle;


#ifndef	calcCntlRgn
#define	calcCntlRgn					10
#endif

#ifndef	calcThumbRgn
#define	calcThumbRgn				11
#endif

#ifndef NULL
#define	NULL						0l
#endif

/* Declare function return values:			*/
long		draw_control();
long		test_control();
long		calculate_control_region();
long		initialize_control();
long		dispose_control();
long		position_control();
long		thumb_control();
long		drag_control();
long		auto_track_control();
short		save_pen_font();
short		set_pen_font();
short		check_color();
void		calculate_rectangles();
short		colors_match(RGBColor *color1,RGBColor *color2);


/* Remove/Add Comment Markers below to Build CDEF/Source Debug:	*/

/*#define _Making_Code_Resource_		1*/

#if		_Making_Code_Resource_
#define VCRButtonCDEF main
#endif


pascal long VCRButtonCDEF(short code,ControlHandle the_control,short message,long parameter)
{
	long			return_value;
	short			control_type;
	pen_font_struct	save_struct;
	GrafPtr			save_port;

#if _Making_Code_Resource_
	RememberA0();
	SetUpA4();
#endif
	HLock(the_control);									/* Lock the control handle				*/
	GetPort(&save_port);
	SetPort((**the_control).contrlOwner);
	save_pen_font(the_control,&save_struct);			/* Save off the pen, font, style		*/
	control_type=BitAnd(code,CODE_MASK);				/* Find out type 0=No Title 1=Title		*/
	switch(message)										/* Which Message were we sent??			*/
	{
		case drawCntl:
					return_value=draw_control(control_type,the_control,parameter);
					break;
		case testCntl:
					return_value=test_control(control_type,the_control,parameter);
					break;
		case calcCRgns:
					parameter&=LOW_31_BITS_MASK;		/* Strip off High Bit					*/
		case calcCntlRgn:
		case calcThumbRgn:
					return_value=calculate_control_region(control_type,the_control,parameter);
					break;
		case initCntl:
					return_value=initialize_control(control_type,the_control,parameter);
					break;
		case dispCntl:
					return_value=dispose_control(control_type,the_control,parameter);
					break;
		case posCntl:
					return_value=position_control(control_type,the_control,parameter);
					break;
		case thumbCntl:
					return_value=thumb_control(control_type,the_control,parameter);
					break;
		case dragCntl:
					return_value=drag_control(control_type,the_control,parameter);
					break;
		case autoTrack:
					return_value=auto_track_control(control_type,the_control,parameter);
					break;
	}
	set_pen_font(&save_struct);							/* Restore the pen, font, style			*/
	SetPort(save_port);
	HUnlock(the_control);								/* Unlock the control handle			*/
#if _Making_Code_Resource_
	RestoreA4();
#endif
	return(return_value);
}


long draw_control(short control_type,ControlHandle the_control,long parameter)
{
	short			loop;
	short			active;
	short			visible;
	short			hilite;
	Rect			shadow_rect;
	Rect			button_rect;
	Pattern			shadow_pattern;
	Pattern			button_pattern;
	Pattern			white_pattern;
	short			number_pixels;
	short			h_title;
	short			v_title;
	FontInfo		my_info;
	RgnHandle		control_rgn;
	RgnHandle		button_rgn;
	RgnHandle		shadow_rgn;
	RgnHandle		erase_rgn;
	RgnHandle		temp_rgn;
	long			save_color;
	long			save_fore_color1;
	long			save_back_color1;
	short			oval_width;
	short			oval_height;
	short			push_down;
	short			invert_hilite;
	short			draw_inverted;
	short			foreground_on_background;
	short			outline;
	short			color_qd_implemented;
	AuxCtlHndl		aux_control;
	CCTabHandle		the_color;
	ColorSpec		*part_color;
	RGBColor		rgb_save_fore_color1;
	RGBColor		rgb_save_back_color1;
	RGBColor		item_color;
	RGBColor		outline_color;
	RGBColor		text_color;
	short			alternate_background;
	RGBColor		background_color;
	RGBColor		rgb_save_color;
	RGBColor		rgb_save_color2;
	short			save_mode;
	Pattern			save_pen_pattern;
	short			shadow_pattern_id;
	short			font;
	short			face;
	short			size;
	short			outline_pen_size;
	short			draw_if_window_invisible;
	pen_font_struct	save_struct2;

	if((**the_control).contrlHilite==INACTIVE)		/* Is the control inactive??									*/
	{
		active=FALSE;								/* Yes - set active flag to FALSE								*/
	}
	else											/* No - set active flag to TRUE									*/
	{
		active=TRUE;
	}
	visible=(**the_control).contrlVis;				/* Store visibility state										*/
	hilite=(**the_control).contrlHilite;


	if((**the_control).contrlData!=NULL)
	{
		oval_width=(**(vcr_button_data_handle)(**the_control).contrlData).oval_width;
		oval_height=(**(vcr_button_data_handle)(**the_control).contrlData).oval_height;
		invert_hilite=(**(vcr_button_data_handle)(**the_control).contrlData).invert_hilite;
		foreground_on_background=(**(vcr_button_data_handle)(**the_control).contrlData).foreground_on_background;
		outline=(**(vcr_button_data_handle)(**the_control).contrlData).outline;
		alternate_background=(**(vcr_button_data_handle)(**the_control).contrlData).use_alternate_background_color;
		background_color.red=(**(vcr_button_data_handle)(**the_control).contrlData).background_color.red;
		background_color.green=(**(vcr_button_data_handle)(**the_control).contrlData).background_color.green;
		background_color.blue=(**(vcr_button_data_handle)(**the_control).contrlData).background_color.blue;
		shadow_pattern_id=(**(vcr_button_data_handle)(**the_control).contrlData).shadow_pattern_id;
		font=(**(vcr_button_data_handle)(**the_control).contrlData).font;
		face=(**(vcr_button_data_handle)(**the_control).contrlData).face;
		size=(**(vcr_button_data_handle)(**the_control).contrlData).size;
		outline_pen_size=(**(vcr_button_data_handle)(**the_control).contrlData).outline_pen_size;
		if(outline_pen_size<1)
		{
			outline_pen_size=1;
		}
		draw_if_window_invisible=(**(vcr_button_data_handle)(**the_control).contrlData).draw_if_window_invisible;
	}
	else
	{
		/* These are the defaults:					*/
		oval_width=16;
		oval_height=16;
		invert_hilite=TRUE;	/* Changed from FALSE	*/
		foreground_on_background=TRUE;
		outline=TRUE;
		alternate_background=FALSE;
		shadow_pattern_id=3;
		font=geneva;
		face=0;
		size=12;
		outline_pen_size=1;
		draw_if_window_invisible=TRUE;
	}
	/* The following let's us decide not to draw the control if the window is invisible	*/
	/*	This is a good idea, but a bad default.											*/
	if(!draw_if_window_invisible)
	{
		if(!(*(WindowPeek)(**the_control).contrlOwner).visible)
		{
			return(0l);
		}
	}
	if(visible)										/* if Invisible DO NOT Draw										*/
	{
		PenNormal();
		color_qd_implemented=check_color();
		if(color_qd_implemented)
		{
			GetForeColor(&rgb_save_fore_color1);
			GetBackColor(&rgb_save_back_color1);
			GetAuxCtl(the_control,&aux_control);
			if(aux_control!=NULL)
			{
				the_color=(**aux_control).acCTable;
				for(loop=0;loop<=(**the_color).ctSize;++loop)
				{
					part_color=(ColorSpec *)&(**the_color).ctTable[loop];
					switch((*part_color).value)
					{
						case	cFrameColor:
								outline_color.red=(*part_color).rgb.red;
								outline_color.green=(*part_color).rgb.green;
								outline_color.blue=(*part_color).rgb.blue;
								break;
						case	cBodyColor:
								item_color.red=(*part_color).rgb.red;
								item_color.green=(*part_color).rgb.green;
								item_color.blue=(*part_color).rgb.blue;
								break;
						case	cTextColor:
								text_color.red=(*part_color).rgb.red;
								text_color.green=(*part_color).rgb.green;
								text_color.blue=(*part_color).rgb.blue;
								break;
						case	cThumbColor:
								/* Not Used!	*/
								break;
					}
				}
			}
			else
			{
				SysBeep(1);
			}
			RGBForeColor(&outline_color);
			if(alternate_background)
			{
				RGBBackColor(&background_color);
			}
		}
		else
		{
			save_fore_color1=(*(**the_control).contrlOwner).fgColor;
			save_back_color1=(*(**the_control).contrlOwner).bkColor;
		}
		if(hilite&&active)
		{
			push_down=TRUE;
			if(invert_hilite)
			{
				if(foreground_on_background)
				{
					draw_inverted=TRUE;
				}
				else
				{
					draw_inverted=FALSE;
				}
			}
			else
			{
				if(foreground_on_background)
				{
					draw_inverted=FALSE;
				}
				else
				{
					draw_inverted=TRUE;
				}
			}
		}
		else
		{
			push_down=FALSE;
			if(foreground_on_background)
			{
				draw_inverted=FALSE;
			}
			else
			{
				draw_inverted=TRUE;
			}
		}


		GetIndPattern(&shadow_pattern,sysPatListID,shadow_pattern_id);
		GetIndPattern(&button_pattern,sysPatListID,1);	/* 1	*/
		GetIndPattern(&white_pattern,sysPatListID,20);

		calculate_rectangles(the_control,&shadow_rect,&button_rect,push_down);

		/* Let's calculate some Regions (Oh Boy!!)		*/
		control_rgn=NewRgn();
		button_rgn=NewRgn();
		shadow_rgn=NewRgn();
		erase_rgn=NewRgn();
		temp_rgn=NewRgn();

		RectRgn(control_rgn,&(**the_control).contrlRect);

		OpenRgn();							/* Create the region for the button	*/
		FrameRoundRect(&button_rect,oval_width,oval_height);
		CloseRgn(button_rgn);

		if(active)
		{
			OpenRgn();							/* Create the region for the shadow	*/
			FrameRoundRect(&shadow_rect,oval_width,oval_height);
			CloseRgn(shadow_rgn);
			DiffRgn(shadow_rgn,button_rgn,shadow_rgn);
		}
		else
		{
			EmptyRgn(shadow_rgn);				/* No drop shadow if inactive!		*/
		}

		UnionRgn(button_rgn,shadow_rgn,temp_rgn);
		DiffRgn(control_rgn,temp_rgn,erase_rgn);

		EraseRgn(erase_rgn);
		if(!draw_inverted)
		{
			EraseRgn(button_rgn);
		}

		/* Now draw the rects:						*/
		RGBBackColor(&item_color);
		if(draw_inverted)
		{
			FillRoundRect(&button_rect,oval_width,oval_height,button_pattern);
			if(outline)
			{
				if(color_qd_implemented)
				{
					GetForeColor(&rgb_save_color);
					GetBackColor(&rgb_save_color2);
					RGBBackColor(&outline_color);
					RGBForeColor(&rgb_save_color2);
					save_pen_font(the_control,&save_struct2);
					PenSize(outline_pen_size,outline_pen_size);
					FrameRoundRect(&button_rect,oval_width,oval_height);
					set_pen_font(&save_struct2);
					RGBForeColor(&rgb_save_color);
					RGBBackColor(&rgb_save_color2);
				}
				else
				{
					save_color=(*thePort).fgColor;
					ForeColor((*thePort).bkColor);
					FrameRoundRect(&button_rect,oval_width,oval_height);
					ForeColor(save_color);
				}
			}
		}
		else
		{
			FillRoundRect(&button_rect,oval_width,oval_height,white_pattern);
			if(outline)
			{
				save_pen_font(the_control,&save_struct2);
				PenSize(outline_pen_size,outline_pen_size);
				FrameRoundRect(&button_rect,oval_width,oval_height);
				set_pen_font(&save_struct2);
			}
		}

		/* Now let's draw the drop shadow:				*/
		GetClip(temp_rgn);
		SetClip(shadow_rgn);
		if(alternate_background)
		{
			RGBBackColor(&background_color);
		}
		else
		{
			RGBBackColor(&rgb_save_back_color1);
		}
		FillRoundRect(&shadow_rect,oval_width,oval_height,shadow_pattern);
		SetClip(temp_rgn);
		RGBBackColor(&item_color);


		/* Now draw the item name					*/
		TextFont(font);
		TextFace(face);
		TextSize(size);
		if(draw_inverted)
		{
			if(color_qd_implemented)
			{
				GetForeColor(&rgb_save_color);
				GetBackColor(&rgb_save_color2);
				if(colors_match(&outline_color,&text_color))
				{
					/* We cannot have matching colors or the title will not show!!	*/
					if(colors_match(&item_color,&text_color))
					{
						/* Our best bet is to simply reverse the current colors!	*/
						RGBForeColor(&rgb_save_color2);
						RGBBackColor(&rgb_save_color);
					}
					else
					{
						RGBForeColor(&item_color);
						RGBBackColor(&text_color);
					}
				}
				else
				{
					RGBForeColor(&text_color);
					RGBBackColor(&outline_color);
				}
			}
			else
			{
				save_color=(*thePort).fgColor;
				ForeColor((*thePort).bkColor);
			}
		}
		else
		{
			if(color_qd_implemented)
			{
				GetForeColor(&rgb_save_color);
				GetBackColor(&rgb_save_color2);
				RGBForeColor(&text_color);
			}
		}
		number_pixels=StringWidth((**the_control).contrlTitle);
		h_title=(button_rect.right+button_rect.left-number_pixels)/2;
		GetFontInfo(&my_info);
		v_title=(button_rect.top+button_rect.bottom+my_info.ascent-my_info.descent)/2-1;
		MoveTo(h_title,v_title);
		if(!active)
		{
			save_mode=(*(**the_control).contrlOwner).txMode;
			TextMode(srcXor);
		}
		else
		{
			TextMode(srcCopy);
		}
		DrawString((**the_control).contrlTitle);
		if(!active)
		{
			TextMode(save_mode);
		}

		if(color_qd_implemented)
		{
			RGBForeColor(&rgb_save_color);
			RGBBackColor(&rgb_save_color2);
		}
		else
		{
			if(draw_inverted)
			{
				ForeColor(save_color);
			}
		}
		DisposeRgn(control_rgn);
		DisposeRgn(button_rgn);
		DisposeRgn(shadow_rgn);
		ValidRgn(erase_rgn);
		SetClip(temp_rgn);
		DisposeRgn(erase_rgn);
		DisposeRgn(temp_rgn);
		if(color_qd_implemented)
		{
			RGBForeColor(&rgb_save_fore_color1);
			RGBBackColor(&rgb_save_back_color1);
		}
		else
		{
			ForeColor(&save_fore_color1);
			BackColor(&save_back_color1);
		}
	}
	return(0l);
}


long test_control(short control_type,ControlHandle the_control,long parameter)
{
	Point			hit_point;
	Rect			shadow_rect;
	Rect			button_rect;


	hit_point.v=HiWord(parameter);										/* Get the Hit point from the parameter	*/
	hit_point.h=LoWord(parameter);
	if((**the_control).contrlHilite==INACTIVE)							/* Is the control inactive?				*/
	{
		return(0l);														/* Yes -return 0						*/
	}
	else
	{
		calculate_rectangles(the_control,&shadow_rect,&button_rect,FALSE);
		if(PtInRect(hit_point,&button_rect))						/* Is the point in our control?			*/
		{
			return(IN_BUTTON);
		}
	}
	return(0l);															/* Point is not in our control!			*/
}


long calculate_control_region(short control_type,ControlHandle the_control,long parameter)
{
	short				part;
	Rect				indicator_rect;

	RectRgn((RgnHandle)parameter,&(**the_control).contrlRect);			/* Set the Region						*/
	return(0l);
}


long initialize_control(short control_type,ControlHandle the_control,long parameter)
{
	vcr_button_data_handle			data_handle;

	data_handle=(vcr_button_data_handle)GetResource('CFvb',(**the_control).contrlRfCon);
	if(data_handle==NULL)
	{
		(**the_control).contrlData=NewHandle(sizeof(vcr_button_data_struct));	/* Allocate space for out struct						*/
		if((**the_control).contrlData!=NULL)								/* Were we were able to allocate it?					*/
		{
			HLock((**the_control).contrlData);								/* Lock the handle										*/
			/* Set in our Defaults:			*/
			(**(vcr_button_data_handle)(**the_control).contrlData).drop_shadow=TRUE;
			(**(vcr_button_data_handle)(**the_control).contrlData).drop_horizontal=3;
			(**(vcr_button_data_handle)(**the_control).contrlData).drop_vertical=3;
			(**(vcr_button_data_handle)(**the_control).contrlData).outline=TRUE;
			(**(vcr_button_data_handle)(**the_control).contrlData).foreground_on_background=TRUE;
			(**(vcr_button_data_handle)(**the_control).contrlData).oval_width=16;
			(**(vcr_button_data_handle)(**the_control).contrlData).oval_height=16;
			(**(vcr_button_data_handle)(**the_control).contrlData).invert_hilite=FALSE;
			(**(vcr_button_data_handle)(**the_control).contrlData).use_alternate_background_color=FALSE;
			(**(vcr_button_data_handle)(**the_control).contrlData).shadow_pattern_id=3;
			(**(vcr_button_data_handle)(**the_control).contrlData).font=geneva;
			(**(vcr_button_data_handle)(**the_control).contrlData).face=0;
			(**(vcr_button_data_handle)(**the_control).contrlData).size=12;
			(**(vcr_button_data_handle)(**the_control).contrlData).outline_pen_size=1;
			(**(vcr_button_data_handle)(**the_control).contrlData).draw_if_window_invisible=TRUE;
			HUnlock((**the_control).contrlData);							/* Unlock the Handle									*/
		}	
	}
	else
	{
		HandToHand(&data_handle);	/* We cannot DetachResource because this screws up ResEdit if the 'CFvb' resource has not been saved	*/
		(**the_control).contrlData=(Handle)data_handle;
	}
	return(0l);
}


long dispose_control(short control_type,ControlHandle the_control,long parameter)
{
	if((**the_control).contrlData!=NULL)														/* We allocated a Handle											*/
	{
		DisposHandle((**the_control).contrlData);
	}
	return(0l);
}


long position_control(short control_type,ControlHandle the_control,long parameter)
{
	return(0l);
}


long thumb_control(short control_type,ControlHandle the_control,long parameter)
{
	return(0l);
}


long drag_control(short control_type,ControlHandle the_control,long parameter)
{
	return(1l);
}


long auto_track_control(short control_type,ControlHandle the_control,long parameter)
{
	return(0l);														/* User did not make a choice										*/
}


save_pen_font(ControlHandle the_control,pen_font_struct *save_struct)
{
	GetPenState(&(*save_struct).save_pen);
	(*save_struct).save_font=(*(**the_control).contrlOwner).txFont;				/* Save off the font					*/
	(*save_struct).save_size=(*(**the_control).contrlOwner).txSize;				/* Save off the size					*/
	(*save_struct).save_face=(*(**the_control).contrlOwner).txFace;				/* Save off the style					*/
}


set_pen_font(pen_font_struct *save_struct)
{
	SetPenState(&(*save_struct).save_pen);
	TextFont((*save_struct).save_font);											/* Restore the font						*/
	TextSize((*save_struct).save_size);											/* Restore the size						*/
	TextFace((*save_struct).save_face);											/* Restore the style					*/
}


check_color()
{
	SysEnvRec		the_environment;				/* Environment record to determine if Color QD is implemented	*/

	SysEnvirons(1,&the_environment);				/* Have machine tell us the environment							*/
	if(the_environment.hasColorQD)					/* Color QD available?											*/
	{
		return(TRUE);								/* Yup.															*/
	}
	else
	{
		return(FALSE);								/* Nope.														*/
	}
}


void calculate_rectangles(ControlHandle the_control,Rect *shadow_rect,Rect *button_rect,short push_down)
{
	short						height;
	short						width;
	Rect						control_rect;
	vcr_button_data_handle		button_data_handle;
	vcr_button_data_struct		*button_data_ptr;
	short						drop_vertical;
	short						drop_horizontal;
	short						drop_shadow;

	button_data_handle=(vcr_button_data_handle)(**the_control).contrlData;
	if((button_data_handle==NULL))
	{
		button_data_ptr=(vcr_button_data_struct *)NewPtr(sizeof(vcr_button_data_struct));
		fill_in_default_struct(button_data_ptr);
	}
	else
	{
		HLock(button_data_handle);
		button_data_ptr=(*button_data_handle);
	}
	control_rect=(**the_control).contrlRect;
	drop_vertical=(*button_data_ptr).drop_vertical;
	drop_horizontal=(*button_data_ptr).drop_horizontal;
	drop_shadow=(*button_data_ptr).drop_shadow;
	/* Calculate the rectangle of the shadow:	*/
	if(drop_vertical>0)
	{
		(*button_rect).bottom=control_rect.bottom-drop_vertical;
		(*button_rect).top=control_rect.top;
		(*shadow_rect).bottom=control_rect.bottom;
		(*shadow_rect).top=control_rect.top+drop_vertical;
	}
	else
	{
		(*button_rect).bottom=control_rect.bottom;
		(*button_rect).top=control_rect.top-drop_vertical;
		(*shadow_rect).bottom=control_rect.bottom+drop_vertical;
		(*shadow_rect).top=control_rect.top;
	}
	if(drop_horizontal>0)
	{
		(*button_rect).left=control_rect.left+drop_horizontal;
		(*button_rect).right=control_rect.right;
		(*shadow_rect).left=control_rect.left;
		(*shadow_rect).right=control_rect.right-drop_horizontal;
	}
	else
	{
		(*button_rect).left=control_rect.left;
		(*button_rect).right=control_rect.right+drop_horizontal;
		(*shadow_rect).left=control_rect.left-drop_horizontal;
		(*shadow_rect).right=control_rect.right;
	}

	/* Calculate the rectangle of the button:	*/
	if(push_down)
	{
		*button_rect=*shadow_rect;
	}
	if(button_data_handle==NULL)
	{
		DisposPtr(button_data_ptr);
	}
	else
	{
		HUnlock(button_data_handle);
	}
}


fill_in_default_struct(vcr_button_data_struct *button_data_ptr)
{
	(*button_data_ptr).drop_shadow=TRUE;
	(*button_data_ptr).drop_horizontal=3;
	(*button_data_ptr).drop_vertical=3;
	(*button_data_ptr).outline=TRUE;
	(*button_data_ptr).outline_pen_size=1;
	(*button_data_ptr).foreground_on_background=TRUE;
	(*button_data_ptr).oval_width=16;
	(*button_data_ptr).oval_height=16;
	(*button_data_ptr).invert_hilite=FALSE;
	(*button_data_ptr).use_alternate_background_color=FALSE;
}


short colors_match(RGBColor *color1,RGBColor *color2)
{
	long		color_index1;
	long		color_index2;

	color_index1=Color2Index(color1);
	color_index2=Color2Index(color2);
	if(color_index1==color_index2)
	{
		return(TRUE);		/* The index of the colors match, so the colors match	*/
	}
	else
	{
		return(FALSE);		/* The index of the colors does not match, so the colors don't match	*/
	}
}