{---------------------------------------------------}
{																										}
{	Airbrush masker for SuperPaint 2.0a								}
{													    											}
{	Written in THINK Pascal version 3.0.2					    }
{												    												}
{	Allen Stenger		July 1991					    						}
{													    											}
{---------------------------------------------------}

unit Masker;

interface
	uses
		BWtc;

	const

	{ returnCode values for errors (we return any }
	{ Memory Manager error codes as themselves)   }
		BadSelector = 1; 				{ bad selector value }
		InvalidSubMenu = 2; 		{ unimplemented submenu	}
													{ selection }
		MaskNotInSelection = 3;	{ attempt to perform mask }
 													{ operation with mask not }
 													{ totally in selection }
	{ equates for submenu selections }
		OutlineAMask = 1;				{ make current selection the }
													{ mask }
		RemoveFromMask = 2;			{ remove current selection }
													{ from mask }
	{ separator line = 3 }
		CaptureUnderMask = 4;		{ save portion of document under mask }
		RestoreUnderMask = 5;		{ restore saved data to }
													{ document }
	{ separator line = 6 }
		TintMask = 7;						{ tint mask area so we can }
													{ see it }
		ForgetMask = 8;					{ discard all mask and saved }
													{ data }

	type
		SaveArea = record			{ area saved by SuperPaint }
													{ for us as globalData^^ }
				MaskBitMap: BitMap;		{ the mask, a BitMap, always }
													{ maintained in document }
													{ coordinates }
				SavedBitMap: BitMap;	{ saved part of original }
													{ image, always maintained }
													{ in document coordinates }
				Tinted: Boolean;			{ is mask currently tinted? }
			end;
		SaveAreaPtr = ^SaveArea;
		SaveAreaHandle = ^SaveAreaPtr;

	procedure Main (selector: integer; mDataPtr: MenuDataPtr; var globalData: SaveAreaHandle; var returnCode: integer);

implementation

	procedure Main;

	{ look up the save area preserved by SuperPaint, or }
	{ create a new one if none }
		function GetSaveArea: OSErr;
			var
				ourReturnCode: OSErr;	{holds value for GetSaveArea}
		begin
			ourReturnCode := noErr;
			if globalData = nil then
				begin
					globalData := SaveAreaHandle(NewHandle(sizeof(SaveArea)));
					ourReturnCode := MemError;
					if ourReturnCode = noErr then
						with globalData^^, globalData^^.MaskBitMap do
							begin
								baseAddr := nil;
								rowBytes := 0;
								SetRect(bounds, 0, 0, 0, 0);
								SavedBitMap := MaskBitMap;
								Tinted := false;
							end;
				end; {if globalData}
			GetSaveArea := ourReturnCode;
		end; {GetSaveArea}

	{ release save area and storage it references }
		function DisposSaveArea: OSErr;
			var
				ourReturnCode: OSErr;	{ holds function value }
		begin
			ourReturnCode := noErr;
			if globalData <> nil then
				begin
					with globalData^^.MaskBitMap do
						if baseAddr <> nil then
							begin
								DisposPtr(baseAddr);
								ourReturnCode := MemError;
							end;
					with globalData^^.SavedBitMap do
						if (ourReturnCode = noErr) and (baseAddr <> nil) then
							begin
								DisposPtr(baseAddr);
								ourReturnCode := MemError
							end;
					if ourReturnCode = noErr then
						begin
							DisposHandle(Handle(globalData));
							ourReturnCode := MemError
						end;
				end;
			globalData := nil;
			DisposSaveArea := ourReturnCode;
		end; {DisposSaveArea}

	{ Create a bit mask in outRect within the new  }
	{ BitMap outBitMap by copying the mask in inRect }
	{ within inBitMap; or by filling outRect if inBitMap }
	{ is null.}
		function CopyBitMask (inBitMap: BitMap; var outBitMap: BitMap; inRect, outRect: Rect): OSErr;
			var
				ourReturnCode: OSErr;	{ holds value of CopyBitMask}
				il: longint;					{ loop control }
				bitsSize: longint;		{ size of bit array in bytes }
				byteP: PTR;						{ temporary byte pointer }

		begin
	{ make a new BitMap the right size to hold the mask }
			outBitMap.bounds := outRect;
			with outBitMap, outBitMap.bounds do
				begin
					rowBytes := ((right - left + 7) div 8 + 1) div 2 * 2;
					bitsSize := rowBytes * LONGINT(bottom - top);
					baseAddr := NewPtr(bitsSize);
					ourReturnCode := MemError;
				end;

		{ blit the mask into the new BitMap }
			if ourReturnCode = noErr then
				if inBitMap.baseAddr <> nil then
					with outBitMap, outBitMap.bounds do
						CopyBits(inBitMap, outBitMap, inRect, bounds, srcCopy, nil)
				else
					with outBitMap do
						for il := 0 to bitsSize - 1 do
							begin
								byteP := PTR(ORD(baseAddr) + il);
								byteP^ := 255;
							end;	{for il}

			CopyBitMask := ourReturnCode;
		end;	{CopyBitMask}

	{ convert r from Document to GrafPort (local) }
	{ coordinates }
		procedure DocToGraf (var r: Rect; gPtr: GrafPtr);
		begin
			with mDataPtr^, gPtr^ do
				OffsetRect(r, -(selectRect.left - portRect.left), -(selectRect.top - portRect.top));
		end;	{DocToGraf}

	{ Flip (i.e. XOR) the tint of the mask }
		procedure FlipTint (gPtr: GrafPtr);
			var
				workRect: Rect;	{ temp Rectangle }
		begin
			workRect := globalData^^.MaskBitMap.bounds;
			DocToGraf(workRect, gPtr);
			CopyBits(globalData^^.MaskBitMap, gPtr^.portBits, globalData^^.MaskBitMap.bounds, workRect, srcXor, nil);
		end;	{FlipTint}

	{ execute the desired action from the menu }
		procedure TakeAction;
			var
				i: integer;					{ loop control }
				gPtr: GrafPtr;			{ points to current GrafPort }
				workRect: Rect;			{ temporary working rectangle }
				bitsSize: longint;	{ size of bit array in bytes }
				workBitMap: BitMap;	{ temporary bit map }
				trashB: Boolean;		{ temporary Boolean }

		begin
			GetPort(gPtr);

			case mDataPtr^.menuItem of
				OutlineAMask, ForgetMask: 
					begin
						returnCode := DisposSaveArea;
						if returnCode = noErr then
							returnCode := GetSaveArea
					end;
				otherwise
					returnCode := GetSaveArea;
			end;	{case mDataPtr^.menuItem}

			if returnCode = noErr then
				case mDataPtr^.menuItem of

					OutlineAMask: 
						begin
							returnCode := CopyBitMask(mDataPtr^.lassoMaskBits, globalData^^.MaskBitMap, gPtr^.portRect, mDataPtr^.selectRect);
						end;	{OutlineAMask}

					RemoveFromMask: 
						begin
							returnCode := CopyBitMask(mDataPtr^.lassoMaskBits, workBitMap, gPtr^.portRect, mDataPtr^.selectRect);
							if returnCode = noErr then
								begin
									workRect := workBitMap.bounds;
									if globalData^^.Tinted then
										FlipTint(gPtr);	{ remove Tint where mask will be }
											{ removed}
									CopyBits(workBitMap, globalData^^.MaskBitMap, workRect, workRect, srcBic, nil);
									DisposPtr(workBitMap.baseAddr);
									returnCode := MemError;
									workBitMap.baseAddr := nil;
								end	{if returnCode}
						end;	{RemoveFromMask}

					CaptureUnderMask: 
						begin
		{ Quick and dirty check to see }
		{if user's selection includes the mask area. }
		{ A better check would check that all }
		{ bits in MaskBitMap are in lassoMaskBits. }
							trashB := SectRect(mDataPtr^.selectRect, globalData^^.MaskBitMap.bounds, workRect);
							if not EqualRect(workRect, globalData^^.MaskBitMap.bounds) then
								returnCode := MaskNotInSelection
							else
								begin
									with globalData^^, globalData^^.SavedBitMap do
										begin
											if baseAddr <> nil then
												DisposPtr(baseAddr);	{ discard any previous }
														{ capture }
											SavedBitMap := MaskBitMap;
											baseAddr := NewPtr(GetPtrSize(MaskBitMap.baseAddr));
											returnCode := MemError;
											if returnCode = noErr then
												begin
													workRect := bounds;
													DocToGraf(workRect, gPtr);
													CopyBits(gPtr^.portBits, SavedBitMap, workRect, bounds, srcCopy, nil);
												end;	{if returnCode}
										end;	{with globalData^^}
								end;	{if not EqualRect}
						end;	{CaptureUnderMask}

					RestoreUnderMask: 
						begin
							workRect := globalData^^.SavedBitMap.bounds;
							DocToGraf(workRect, gPtr);
							with globalData^^, gPtr^ do
								CopyMask(SavedBitMap, MaskBitMap, portBits, SavedBitMap.bounds, MaskBitMap.bounds, workRect);
							globalData^^.Tinted := false;	{we obliterated tint}
							returnCode := noErr;
						end;	{RestoreUnderMask}

					TintMask: 
						begin
							FlipTint(gPtr);
							with globalData^^ do
								Tinted := not Tinted;
							returnCode := noErr;
						end;	{TintMask}

					ForgetMask: 
						begin
							returnCode := noErr;
						end;	{ForgetMask}

					otherwise
						returnCode := InvalidSubMenu;

				end; {case mDataPtr^.subMenu}

		end; {TakeAction}

	{ update valid menu selections }
		procedure UpdateMenu;
		begin
			with globalData^^, mDataPtr^ do
				begin
					if MaskBitMap.baseAddr = nil then
						begin
							DisableItem(subMenu, RemoveFromMask);
							DisableItem(subMenu, CaptureUnderMask);
							DisableItem(subMenu, TintMask);
							DisableItem(subMenu, ForgetMask)
						end
					else
						begin
							EnableItem(subMenu, RemoveFromMask);
							EnableItem(subMenu, CaptureUnderMask);
							EnableItem(subMenu, TintMask);
							EnableItem(subMenu, ForgetMask)
						end;
					if SavedBitMap.baseAddr = nil then
						DisableItem(subMenu, RestoreUnderMask)
					else
						EnableItem(subMenu, RestoreUnderMask);
					if Tinted then
						DisableItem(subMenu, CaptureUnderMask);
					CheckItem(subMenu, TintMask, Tinted);
				end;	{with globalData^^}
		end;	{UpdateMenu}

	begin {Main}
		case selector of

			menuAbout: 
				begin
					returnCode := textAbout;
		{our About is in TEXT resource}
				end;

			menuOptions: 
				begin
					MenuOptionsPtr(mDataPtr)^.usesScratch := false;
		{we don't use scratch area}
					returnCode := noErr;
				end;

			menuSelected: 
				begin
					TakeAction;
					UpdateMenu;
					if returnCode <> noErr then
						SysBeep(1);
			{ SuperPaint doesn't notify user of error, }
			{ so we will}
				end;

			otherwise
				begin
					returnCode := BadSelector
				end;

		end; {case selector of}

	end; {Main}

end.