;;;;;;;;;;;;;;;;;;;;;;;;;;
; File:  bignums-big.sch ;
;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(define print
  (lambda (n)
    (display n)
    (newline)))
;
(define base-10??
  (lambda (base)
    (if (null? base)
        10
        (car base))))
;
(define to-base??
  (lambda (bases)
    (if (or (null? bases)
            (null? (cdr bases)))
        10
        (cadr bases))))
;
(define normalize reverse)
;
(define unnormalize reverse)
;
(define bigify-digits list)
;
(define no-digits? null?)
;
(define first-digit car)
;
(define rest-digits cdr)
;
(define last-digit 
  (lambda (n)
    (first-digit (last-pair n))))
;
(define push-digit cons)
;
(define big-zero ())
;
(define big-zero?
  (lambda (n)
    (cond ((no-digits? n)
           #t)
          ((and (number? (first-digit n))
                (zero? (first-digit n)))
           (big-zero? (rest-digits n)))
          (else
           #f))))
;
(define big-one '(1))
;
; detokenize and normalize could be combined to save one 
; trip through the argument list.  I prefer the 
; generality of not having them combined--tokenization 
; is not a time critical routine.
;
(define detokenize
  (lambda (x)
    (if (no-digits? x)
        big-zero
        (push-digit (if (number? (first-digit x))
                        (first-digit x)
                        (- (char->integer 
                            (string-ref 
                             (symbol->string 
                              (first-digit x)) 
                             0))
                           87))
                    (detokenize (rest-digits x))))))
;  
(define make-big 
  (lambda (user-big . bases)
    (let 
      ((big (normalize (detokenize user-big)))
       (from-base (base-10?? bases))
       (to-base (to-base?? bases)))
      (if (= from-base to-base 10)
          big
          (big-base-n->big-base-m big from-base 
                                      to-base)))))
;
(define show-big 
  (lambda (big)
    (if (big-zero? big)
        (print "0")
        (let ((first-thing (first-digit big)))
          (if (or (null? first-thing)
                  (pair? first-thing))
              (begin
               (show-big first-thing)
               (display "remainder ")
               (show-big (rest-digits big)))
              (let ((start-value (- (char->integer #\a) 
                                    10)))
                (begin 
                 (for-each 
                  (lambda (x)
                    (cond ((<? x 10) ; a
                           (display x))
                          ((<? x 36) ; z
                           (display 
                            (make-string 
                             1
                             (integer->char 
                              (+ start-value 
                                 x)))))
                          (else ; out of tokens
                           (display "<")
                           (display x)
                           (display ">"))))
                  (unnormalize big))
                 (newline))))))))
;
(define find-biggest-base
  (lambda ()
    (letrec
      ((iter
        (lambda (base bits-per-digit)
          (let ((base-minus-one
                 (- base 1)))
            (if (not (fixnum? (* base-minus-one
                                 base-minus-one)))
                (if (odd? bits-per-digit)
                    (quotient base 2)
                    (quotient base 4))
                (iter (* 2 base) (+ bits-per-digit 
                                    1)))))))
      (iter 2 1))))
;
(define *biggest-base* (find-biggest-base))
;
(define fix-base-10->big-base-n
  (lambda (n . base)
    (let ((base (base-10?? base)))
      (letrec 
        ((recur
          (lambda (n)
            (if (<? n base)
                (bigify-digits n)
                (push-digit (remainder n base)
                            (recur (quotient n 
                                             base)))))))
        (recur n)))))
; 
(define big-base-n->big-base-m
  (lambda (big from-base to-base)
    (let ((to-base
           (fix-base-10->big-base-n to-base 
                                    from-base)))
      (letrec
        ((sum-digits
          (lambda (big base)
            (letrec 
              ((iter 
                (lambda (big power)
                  (if (no-digits? big)
                      0
                      (+ (* (first-digit big)
                            power)
                         (iter (rest-digits big) 
                               (* power base)))))))
              (iter big 1))))
         (recur
          (lambda (big)
            (if (big-<? big to-base)
                (bigify-digits (sum-digits big 
                                           from-base))
                (let ((next-digit (big-div big 
                                           to-base 
                                           from-base)))
                  (push-digit 
                   (sum-digits (rest-digits next-digit) 
                               from-base)
                   (recur (first-digit 
                           next-digit))))))))
        (recur big)))))
;
(define compare-digits
  (lambda (first second)
    (if (no-digits? first)
        (if (no-digits? second)
            '=
            '<)
        (if (no-digits? second)
            '>
            (compare-digits (rest-digits first)
                            (rest-digits second))))))
;
(define less-digits?
  (lambda (first second)
    (eq? (compare-digits first second) '<)))
;
(define more-digits?
  (lambda (first second)
    (eq? (compare-digits first second) '>)))
;
(define same-number-of-digits?
  (lambda (first second)
    (eq? (compare-digits first second) '=)))
;
(define big-add
  (lambda (addend augend . base)
    (let ((base (base-10?? base)))
      (letrec
        ((recur
          (lambda (addend augend rem)
            (if (no-digits? addend)
                (if (no-digits? augend)
                    rem
                    (recur augend addend rem))
                (if (no-digits? augend)
                    (if (no-digits? rem)
                        addend
                        (recur addend rem ()))
                    (let
                      ((new-rem
                        (fix-base-10->big-base-n 
                         (+ (first-digit addend)
                            (first-digit augend)
                            (if (no-digits? rem)
                                0
                                (first-digit rem)))
                         base)))
                      (push-digit 
                       (first-digit new-rem)
                       (recur 
                        (rest-digits addend)
                        (rest-digits augend)
                        (rest-digits new-rem)))))))))
        (recur addend augend ())))))
;
(define big-sub
  (lambda (minuend subtrahend . base)
    (let ((base (base-10?? base)))
      (letrec 
        ((recur
          (lambda (minuend subtrahend borrow)
            (if (no-digits? minuend)
                (if (no-digits? subtrahend)
                    (if (=? borrow 1)
                        (error 
                     "needed to borrow more than I had")
                        ())
                    (error 
                     "(>? subtrahend minuend) => #t" 
                     minuend
                     subtrahend))
                (if (no-digits? subtrahend)
                    (if (zero? borrow)
                        minuend
                        (recur minuend 
                               (bigify-digits borrow) 
                               0))
                    (let ((dig1 (first-digit minuend))
                          (dig2 (+ (first-digit 
                                    subtrahend) 
                                   borrow)))
                      (if (<? dig1 dig2)
                          (push-digit 
                           (+ (- base dig2)
                              dig1)
                           (recur (rest-digits minuend)
                                  (rest-digits 
                                   subtrahend)
                                  1))
                          (push-digit 
                           (- dig1 dig2)
                           (recur (rest-digits minuend)
                                  (rest-digits 
                                   subtrahend)
                                  0)))))))))
        (remove-leading-zeros 
         (recur minuend subtrahend 0))))))
;
(define remove-leading-zeros
  (lambda (n)
    (if (or (no-digits? n)
            (not (zero? (last-digit n))))
        n
        (letrec
          ((peel-off-zeros
            (lambda (n)
              (cond ((no-digits? n)
                     big-zero)
                    ((zero? (first-digit n))
                     (peel-off-zeros (rest-digits n)))
                    (else
                     n))))
           (recur
            (lambda (n user-n)
              (if (no-digits? user-n)
                  big-zero
                  (push-digit 
                   (first-digit n)
                   (recur (rest-digits n) 
                          (rest-digits 
                           user-n)))))))
          (recur n
                 (peel-off-zeros (unnormalize n)))))))
;
(define big-*-digit
  (lambda (big digit . base)
    (let ((base (base-10?? base)))
      (cond
       ((zero? digit)
        big-zero)
       ((=? digit 1)
        big)
       (else
        (letrec
          ((recur
            (lambda (big rem)
              (if (no-digits? big)
                  rem
                  (let 
                    ((new-rem
                      (fix-base-10->big-base-n 
                       (+ (* digit 
                             (first-digit big))
                          (if (no-digits? rem)
                              0
                              (first-digit rem))) 
                       base)))
                    (push-digit 
                     (if (no-digits? new-rem)
                         0
                         (first-digit new-rem))
                     (recur (rest-digits big) 
                            (rest-digits new-rem))))))))
          (recur big '(0))))))))
;
(define big-mul
  (lambda (multiplicand multiplier . base)
    (let ((base (base-10?? base)))
      (if (less-digits? multiplier multiplicand)
          (big-mul multiplier multiplicand base)
          (letrec
            ((recur
              (lambda (multiplicand multiplier 
                                    shift-amount)
                (if (no-digits? multiplicand)
                    big-zero
                    (big-add (big-right-shift 
                              (big-*-digit 
                               multiplier 
                               (first-digit 
                                multiplicand) 
                               base) 
                              shift-amount)
                             (recur (rest-digits 
                                     multiplicand)
                                    multiplier
                                    (1+ shift-amount))
                             base)))))
            (recur multiplicand multiplier 0))))))
;
(define big-right-shift
  (lambda (big n)
    (if (zero? n)
        big
        (push-digit 0 (big-right-shift big 
                                       (1- n))))))
;
(define big-fact
  (lambda (n . base)
    (let ((base (base-10?? base)))
      (letrec
        ((recur
          (lambda (n)
            (if (big-zero? n)
                big-one
                (big-mul n 
                         (recur (big-sub n 
                                         big-one 
                                         base)) 
                         base)))))
        (recur n)))))
;
(define last-digit? 
  (lambda (big)
    (no-digits? (rest-digits big))))
;
(define big-compare?
  (lambda (first second predicate?)
    (letrec
      ((iter
        (lambda (first second)
          (if (no-digits? first)
              (if (no-digits? second)
                  #f
                  #t)
              (if (no-digits? second)
                  #f
                  (if (last-digit? first)
                      (if (last-digit? second)
                          (predicate? 
                           (first-digit first)
                           (first-digit second))
                          #t)
                      (iter (rest-digits first)
                            (rest-digits second))))))))
      (iter first second))))
;
(define equal-digit-big-compare?
  (lambda (first second predicate?)
    (letrec
      ((iter
        (lambda (first second)
          (if (no-digits? first)
              #f
              (if (= (first-digit first)
                     (first-digit second))
                  (iter (rest-digits first)
                        (rest-digits second))
                  (if (predicate? (first-digit first)
                                  (first-digit second))
                      #t
                      #f))))))
      (iter (unnormalize first) (unnormalize second)))))
;
(define big-<?
  (lambda (first second)
    (let ((first (remove-leading-zeros first))
          (second (remove-leading-zeros second)))
      (case (compare-digits first second)
        ((<) #t)
        ((>) #f)
        ((=) 
         (equal-digit-big-compare? first second <?))))))
;      
(define big->? 
  (lambda (first second)
    (let ((first (remove-leading-zeros first))
          (second (remove-leading-zeros second)))
      (case (compare-digits first second)
        ((>) #t)
        ((<) #f)
        ((=) 
         (equal-digit-big-compare? first second >?))))))
;
(define big-=?
  (lambda (first second)
    (let ((first (remove-leading-zeros first))
          (second (remove-leading-zeros second)))
      (case (compare-digits first second)
        ((>) #f)
        ((<) #f)
        ((=) 
         (equal-digit-big-compare? first second =?))))))
;
(define big-<=?
  (lambda (first second)
    (not (big->? first second))))
;
(define big->=?
  (lambda (first second)
    (not (big-<? first second))))
;
(define big-div
  (lambda (dividend divisor . base)
    (let ((base (base-10?? base))
          (dividend (unnormalize dividend))
          (divisor (remove-leading-zeros 
                    divisor)))
      (let ((user-divisor 
             (unnormalize divisor)))
        (letrec
          ((find-next-quotient-digit
            (lambda (dividend)
              (let ((dividend 
                     (remove-leading-zeros 
                      dividend)))
                (if 
                 (big->? divisor dividend)
                 (push-digit 0 dividend)
                 (letrec
                   ((guess-next-quotient-digit
                     (lambda ()
                       (let ((dividend 
                              (unnormalize 
                               dividend)))
                         (let 
                           ((dig1-divisor 
                             (first-digit 
                              user-divisor))
                            (dig1-dividend 
                             (first-digit dividend)))
                           (if 
                            (and 
                             (same-number-of-digits? 
                              dividend
                              user-divisor)
                             (<=? 
                              dig1-divisor
                              dig1-dividend))
                            (quotient 
                             dig1-dividend
                             dig1-divisor)
                            (let
                              ((guess
                                (quotient 
                                 (+ 
                                  (* 
                                   base
                                   dig1-dividend)
                                  (first-digit 
                                   (rest-digits 
                                    dividend)))
                                 dig1-divisor)))
                              (if (>=? guess base)
                                  (- base 1)
                                  guess)))))))
                    (iter
                     (lambda (guess)
                       (let ((subtrahend 
                              (big-*-digit 
                               divisor 
                               guess base)))
                         (if 
                          (big->? subtrahend 
                                  dividend)
                          (iter (- guess 1))
                          (push-digit 
                           guess 
                           (big-sub 
                            dividend 
                            subtrahend base)))))))                    
                   (iter 
                    (guess-next-quotient-digit)))))))            
           (iter 
            (lambda (dividend rem result)
              (if 
               (eq? dividend 'done)
               (push-digit 
                (remove-leading-zeros 
                 result)
                (remove-leading-zeros rem))
               (let* ((new-quotient-digit
                       (find-next-quotient-digit 
                        rem))
                      (new-rem 
                       (cdr new-quotient-digit)))
                 (if 
                  (no-digits? dividend)
                  (iter 
                   'done
                   new-rem
                   (push-digit 
                    (first-digit 
                     new-quotient-digit) 
                    result))
                  (iter
                   (rest-digits dividend)
                   (push-digit 
                    (first-digit dividend) 
                    new-rem)
                   (push-digit 
                    (first-digit 
                     new-quotient-digit) 
                    result))))))))
          (iter (rest-digits dividend) 
                (bigify-digits 
                 (first-digit dividend)) 
                ()))))))
;
'done