;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; File:  bignums-bignum.sch ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(define get-sign car)
;
(define get-numerical-part cdr)
;
(define bignum-neg?
  (lambda (number)
    (if (and (not (no-digits? number))
             (eq? (get-sign number) '-))
        #t
        #f)))
;
(define bignum-change-sign
  (lambda (bignum)
    (if (bignum-neg? bignum)
        (get-numerical-part bignum)
        (attach-sign '- bignum))))
;
(define bignum-zero?
  (lambda (bignum)
    (if (bignum-neg? bignum)
        #f
        (big-zero? bignum))))
;
(define bignum-zero
  big-zero)
;
(define bignum-one
  big-one)
;
(define bignum-abs
  (lambda (big)
    (if (no-digits? big)
        big-zero
        (if (not (number? (get-sign big)))
            (get-numerical-part big)
            big))))
;
(define bignum-add
  (lambda (addend augend . base)
    (let ((base (base-10?? base))
          (negative-addend? (bignum-neg? addend))
          (negative-augend? (bignum-neg? augend))
          (abs-addend (bignum-abs addend))
          (abs-augend (bignum-abs augend)))
      (cond
       ((eq? negative-addend? (bignum-neg? augend))
        (attach-sign 
         (if negative-addend?
             '-
             '+)
         (big-add abs-addend abs-augend base)))
       ((equal? abs-addend abs-augend)
        bignum-zero)
       ((big->? abs-addend abs-augend)
        (attach-sign (if negative-addend?
                         '-
                         '+)                       
                     (big-sub abs-addend
                                abs-augend
                                base)))
       (else
        (attach-sign (if negative-augend?
                         '-
                         '+)
                     (big-sub abs-augend
                                abs-addend
                                base)))))))
;
(define bignum-sub
  (lambda (addend augend . base)
    (let ((base (base-10?? base))
          (abs-augend (bignum-abs augend)))
      (if (big-zero? abs-augend)
          addend
          (bignum-add addend
                      (change-sign augend)
                      base)))))
;
(define bignum-mul
  (lambda (multiplicand multiplier . base)
    (let ((base (base-10?? base)))
      (if (or (bignum-zero? multiplicand)
              (bignum-zero? multiplier))
          bignum-zero
          (attach-sign 
           (if (eq? (bignum-neg? multiplicand)
                    (bignum-neg? multiplier))
               '+
               '-)
           (big-mul (bignum-abs multiplicand)
                    (bignum-abs multiplier)
                    base))))))
;
(define bignum-div
  (lambda (dividend divisor . base)
    (let ((base (base-10?? base)))
      (cond ((bignum-zero? dividend)
             bignum-zero)
            ((bignum-zero? divisor)
             (error "bignum-div :  divide by zero"))
            (else
             (attach-sign 
              (if (eq? (bignum-neg? dividend)
                       (bignum-neg? divisor))
                  '+
                  '-)
              (big-div (bignum-abs dividend)
                       (bignum-abs divisor)
                       base)))))))
;
(define bignum-base-n->bignum-base-m
  (lambda (bignum from-base to-base)
    (let ((sign (get-sign bignum))
          (big (bignum-abs bignum)))
      (attach-sign sign 
                   (big-base-n->big-base-m 
                    big
                    from-base
                    to-base)))))
;
(define bignum-fact
  (lambda (n . base)
    (let ((base (base-10?? base)))
      (letrec
        ((recur
          (lambda (n)
            (if (bignum-zero? n)
                bignum-one
                (bignum-mul 
                 n 
                 (recur (bignum-sub n bignum-one base)) 
                 base)))))
        (recur n)))))
;
(define change-sign
  (lambda (number)
    (if (bignum-neg? number)
        (bignum-abs number)
        (attach-sign '- (bignum-abs number)))))
;
(define attach-sign
  (lambda (sign number)
    (if (eq? sign '-)
        (cons '- number)
        number)))
;
(define trampoline
  (lambda (function args)
    (apply function args)))
;
(define make-bignum 
  (lambda (user-bignum . bases)
    (if (bignum-zero? user-bignum)
        bignum-zero
        (attach-sign 
         (if (bignum-neg? user-bignum)
             '-
             '+)
         (trampoline make-big 
                     (cons (bignum-abs user-bignum)
                           bases))))))
;
(define show-bignum
  (lambda (user-bignum . bases)
    (begin 
     (if (bignum-neg? user-bignum)
         (display "-"))
     (apply show-big 
            (cons (bignum-abs user-bignum) 
                  bases)))))
;
'done