/*********************************************************
 "CmyApp.c"

 by John A. Love, III [Washington Apple Pi Users' Group]

 using Symantec's "THINK C", v 5.0.1
 ... as derived from their "TCL Starter" files
 *********************************************************/




#include <CBartender.h>
#include <CCluster.h>
#include <Commands.h>
#include <Constants.h>
#include <CDataFile.h>
#include <CDesktop.h>
#include <CFWDesktop.h>
#include <CList.h>
#include <CWindow.h>

#include "CmyGlobals.h"
#include "CmyApp.h"
#include "CmyBar.h"
#include "CmyDoc.h"
#include "CmyMisc.h"
#include "CmyTear.h"
#include "help.h"
#include "CSpinner.h"
#include "CMeter.h"


#define			kExtraMasters		4
#define			kRainyDayFund		20480
#define			kCriticalBalance	20480
#define			kToolboxBalance		20480

extern OSType			gSignature;
extern CursHandle		gWatchCursor;
extern CBartender		*gBartender;
extern Boolean			gInBackground;
extern CDesktop			*gDesktop;
extern CBureaucrat		*gGopher;

CFile			*currentFile;
Boolean			printable;



void	CStarterApp::IStarterApp (void)		{


	CApplication::IApplication(kExtraMasters, kRainyDayFund, 
							   kCriticalBalance, kToolboxBalance);
    
}	/* IStarterApp */



void	CStarterApp::SetUpFileParameters (void)		{


	inherited::SetUpFileParameters();	// Be sure to call the default method.

	sfNumTypes = 1;
	sfFileTypes[0] = 'TEXT';

	gSignature = 'JAL3';
	
}	/* SetUpFileParameters */



void	CStarterApp::StartUpAction (short numPreloads)	{

		CSpiffyGraphics		*spiffy = nil;
		

	SetCursor(*gWatchCursor);	// Various CURSor handling routines will reset this.

	gStripAddressMask = GetStripAddressMask();
	;
	gStartBalloons = BalloonsOn();
	gLastBalloon = -1;							/* = notMine. */

	if (gSystem.hasHelpMgr)		/* It's System 7 allright !!! */	{
	
		TRY
		{
			spiffy = new (CSpiffyGraphics);
			spiffy->ISystem7Braggart(this);
			spiffy->DisplaySystem7Braggart();
		}
		CATCH
		{
			SysBeep(10);
			spiffy->DisposeSystem7Braggart();	// Calls ForgetObject(spiffy).
			goto bye;
		}
		ENDTRY;
		spiffy->DisposeSystem7Braggart();
	
	}	/* System 7 */
bye:
	inherited::StartUpAction(numPreloads);
	
	SysBeep(10);

}	/* StartUpAction */



void	CStarterApp::Idle (EventRecord *macEvent)	{


	inherited::Idle(macEvent);
	
	gBalloonsUp = BalloonsOn();

}	/* Idle */



void	CStarterApp::MakeBartender (void)	{


	gBartender = new (CmyBar);
	gBartender->IBartender(MBARapp);

}	/* MakeBartender */



void	CStarterApp::SetUpMenus (void)	{

		MenuHandle	kApple;
		short		kAboutItem = 1, none = 0, whoCares = 0;
		
		
	itsTearMenu = new (CTearMenuDir);	/* Director for a tear-off Menu. */
	itsTearMenu->ITearMenuDir(this);
	;
	inherited::SetUpMenus();	/*	Superclass takes care of adding menus
                      				specified in a MBAR id = 1 resource.	*/                          

	/* My attached Apple MENU resource has a SICN plugged into
	** the "About ..." item for System 7.  Therefore, when
	** operating under System 6, it must be removed:			*/

	if (!gSystem.hasHelpMgr)	{
		kApple = GetMHandle(MENUapple);
		SetItemIcon(kApple, kAboutItem, none);
		SetItemCmd(kApple, kAboutItem, (char)none);
	}

	/* Add item to Help Menu "on-the-fly".  This means that my
	** app's "DoCommand" will be passed a negative command #.
	** See comments within my "CmyBar::FindCmdNumber" routine. */
	if (HelpManagerActive())	{
		if (HMGetHelpMenuHandle(&gHelpMenu) == noErr)
			if (gHelpMenu != nil)	{
				/* Original count = 4, NOT 3 because the Help Manager adds
				** a disabled dotted line before any items we may add: */
				gOrigNumHelpItems = CountMItems(gHelpMenu);
				AppendMenu(gHelpMenu, "\pAbout my application ...");
			}
	}
	
}	/* SetUpMenus */



void	CStarterApp::UpdateMenus (void)		{
/* If there is a doc staring at us, our apps UpdateMenus gets called
** by CDocument::UpdateMenus when it calls CBureaucrat::UpdateMenus.
** The latter calls itsSupervisor->UpdateMenus where itsSupervisor is
** our app.  If there is NO doc, then we get called directly.		  */
 
		#define cmdProgress		1024
		WindowPeek				top;
		
		
	inherited::UpdateMenus();     // Enable standard commands.
        
    if (!gInBackground)		{
    
    	top = (WindowPeek) FrontWindow();

		if ( aDAisUp(top) )			// For System 6.
				gBartender->DisableCmd(cmdAbout);
		else	{
			/* For the "About..." item, we must specifically re-enable once
			** we disable because TCL assumes dimNONE for the Apple Menu. */
			gBartender->EnableCmd(cmdAbout);
			gBartender->EnableCmd(cmdProgress);
		}
		
    	/* Switch-A-Roo problem: it indiscriminantly
    	** removes all user-added Help MENU items:	 */
		if (HelpManagerActive())	{
			if (HMGetHelpMenuHandle(&gHelpMenu) == noErr)
				if (gHelpMenu != nil)
					if (CountMItems(gHelpMenu) == gOrigNumHelpItems)
						AppendMenu(gHelpMenu, "\pAbout my application ...");
		}

		if (IsSystemWindow(top))	{
			/* ... because in our Switch To/From DA routines
			**     we may no longer be in the background:	 */
			gBartender->EnableCmd(cmdClose);
			gBartender->EnableCmd(cmdUndo);
			gBartender->EnableCmd(cmdCut);
			gBartender->EnableCmd(cmdCopy);
			gBartender->EnableCmd(cmdPaste);
			gBartender->EnableCmd(cmdClear);
		}
		else	{		/* zip windows */

			/* CBartender::UpdateAllMenus() has already disabled all items.
			** If we were called by CAbstractText or CDocument, the above
			** Edit commands may have been re-enabled.  In any case, keep
			** them in the state they were in prior to coming here.		  */
			
		}
		
	}	/* NOT in background */
				
}	/* UpdateMenus */



void	CStarterApp::SwitchToDA (void)		{
		
		#define kTearMenu		1000
		#define cmdProgress		1024
		
		
	/* This is wrong because CBartender::UpdateAllMenus
	** disables all MENU items anyway.  The place to disable any
	** of my individual items is in CStarterApp::UpdateMenus().	  */

 // gBartender->DisableCmd(cmdProgress);

	/* These paired Switch routines are the place for
	** disabling and enabling WHOLE Menus only:		  */
	
	gBartender->DisableMenu(kTearMenu);
	
	/* The inherited versions call Suspend/Resume AND toggle
	** gInBackground accordingly.  This is patently wrong
	** because the main Event loop addresses these Multifinder
	** app4Evts anyway and the latter are totally separate from
	** going back and forth to DAs.  This is especially true
	** when using Unifinder with System 6:						*/
	
 // inherited::SwitchToDA();
	
}	/* SwitchToDA */
 


void	CStarterApp::SwitchFromDA (void)		{

		#define kTearMenu		1000


	gBartender->EnableMenu(kTearMenu);

 // inherited::SwitchFromDA();
	
}	/* SwitchFromDA */



static long		GetPercent (void)		{

		long	total, sofar;
		
		
	total = ((CDataFile*)currentFile)->GetLength();
	sofar = ((CDataFile*)currentFile)->GetMark();
	
	return (sofar/total);

}	/* GetPercent */



static long		calcProgress (void)		{
/* Here's where you determine the % progress you've made, say in
** spooling a file to the printer.  For now, however, let's force it,
** so much force as a matter of fact it's really kludgy:			 */

		static long		percent = 0;
		long			finalTicks;
		
		
	if (percent > 100)		percent = 0;	/* A kludgy way to re-cycle */
	percent++;
	Delay(5, &finalTicks);
	return (percent);			// return (GetPercent());
	
}	/* calcProgress */
 


void	CStarterApp::DoCommand (long theCommand)	{

		#define 			kTearMenu		1000
		#define				cmdProgress		1024
		GrafPtr				savePort;
		Str255				theDAName;
		CSpiffyGraphics		*spiffy = nil;
		CSpinner			*roundAndRound = nil;
		short				kMeter = 111;
		CMeter				*meterDoc = nil;


	if (theCommand < 0)		{
	
		if (HiShort(-theCommand) == MENUapple)		{
			/* BIG-time TCL boo-boo for not saving & restoring the Port: */
			GetPort(&savePort);
			GetItem(GetMHandle(MENUapple), LoShort(-theCommand), theDAName);
			OpenDeskAcc(theDAName);
			SetPort(savePort);
		}
		
		else if (HiShort(theCommand) == kHMHelpMenuID)		{
		/* For the Help MENU, theCommand is negative because
		** the menu id in its High word is negative (= -16490). */
		
			/* Although the Help Manager adds not only your item
			** but a disabled dotted line before your addition,
			** we still only add 1 ?????						*/
			if (LoShort(theCommand) == gOrigNumHelpItems + 1)		{
				roundAndRound = new(CSpinner);
				if (roundAndRound == nil)
					SetCursor(*gWatchCursor);
				else
					roundAndRound->ISpinner(acurWorld, 10);
				;
				TRY
				{
					spiffy = new (CSpiffyGraphics);
					spiffy->ISpiffyScroll(this);
					spiffy->DisplaySpiffyScroll();
				}
				CATCH
				{
					SysBeep(10);
					spiffy->DisposeSpiffyScroll();
					goto bye1;
				}
				ENDTRY;
				spiffy->DisposeSpiffyScroll();
		bye1:	;
				if (roundAndRound == nil)		InitCursor();
				else							roundAndRound->Dispose();
			}
			else	{
				/* etc. for other Help Menu items */
			}
							
		}	/* end: kHMHelpMenuID */
		
		else if (HiShort(-theCommand) == kTearMenu)		{
		
				enum	{OFF, ONApple = MENUapple};
			
			HiliteMenu(ONApple);
			DoCommand(cmdAbout);
			HiliteMenu(OFF);
		}
		
		else	inherited::DoCommand(theCommand);
		
	}	/* end: theCommand < 0 */
	
	else	{							/* theCommand > 0 */
		switch (theCommand) {
	
			case cmdAbout:
				roundAndRound = new(CSpinner);
				if (roundAndRound == nil)
					SetCursor(*gWatchCursor);
				else
					roundAndRound->ISpinner(acurDogCow, 10);
				;
				TRY
				{
					spiffy = new (CSpiffyGraphics);
					spiffy->ISpiffyIris(this);
					spiffy->DisplaySpiffyIris();
				}
				CATCH
				{
					SysBeep(10);
					spiffy->DisposeSpiffyIris();
					goto bye2;
				}
				ENDTRY;
				spiffy->DisposeSpiffyIris();
	bye2:		;
				if (roundAndRound == nil)		InitCursor();
				else							roundAndRound->Dispose();
				break;
			
			case cmdProgress:
				currentFile = ((CDocument*)gGopher)->itsFile;
				printable = ((CDocument*)gGopher)->itsPrinter != nil;
			 // if ( !printable || (currentFile == nil) )		return;
				
				SetCursor(*gWatchCursor);
				;
				TRY
				{
					meterDoc = new (CMeter);
					meterDoc->IMeter(kMeter,					// WIND ID
									 (LongFunc) &calcProgress,	// Points to function
									 "\pWatch the spiffy fireworks !!",		// title
									 100,						// maxSteps
									 this);						// Supervisor = app
					meterDoc->itsWindow->Select();
					meterDoc->StartProgress();
					meterDoc->itsWindow->Update();
					meterDoc->UpdateProgress();
				}
				CATCH
				{
					SysBeep(10);
					meterDoc->Dispose();
				}
				ENDTRY;
				;
				InitCursor();
				break;

			default:
				inherited::DoCommand(theCommand);
				
		}	/* end: switch */
	}	/* end: theCommand > 0 */
			
}	/* DoCommand */



void	CStarterApp::DoKeyDown (char theChar, Byte keyCode, EventRecord *macEvent)	{

		enum	{OFF, ONApple = MENUapple, ONHelp = kHMHelpMenuID};
		long	finalTicks, myHelpCommand;
	
	
	if ( (macEvent->modifiers & cmdKey) && theChar == '/' )	{
		HiliteMenu(ONApple);
		DoCommand(cmdAbout);
		HiliteMenu(OFF);
	}
	
	else if (keyCode == KeyHelp)	{
		if (HelpManagerActive())		{
		 // For some reason, System 7 does NOT keep title highlighted:
			HiliteMenu(ONHelp);
			Delay(20, &finalTicks);
			HiliteMenu(OFF);
			myHelpCommand = ((long)ONHelp << 16) + gOrigNumHelpItems + 1;
			DoCommand(myHelpCommand);
		}
		
		else	{
			HiliteMenu(ONApple);
			DoCommand(cmdAbout);
			HiliteMenu(OFF);
		}
	}	/* Help Key */
	
	else	inherited::DoKeyDown(theChar, keyCode, macEvent);
	
}	/* DoKeyDown */



void	CStarterApp::Exit (void)	{

	/* your exit handler here */
	
	ResetBalloons(gStartBalloons);

	PlaySound("\pMoof");
	
}	/* Exit */



/* CreateDocument
**
**		The user chose New from the File menu.
**		In this method, you create a document and send it a NewFile() message.
*/

void	CStarterApp::CreateDocument (void)		{

		CStarterDoc	*theDocument = NULL;
		
	
	TRY
	{
		theDocument = new(CStarterDoc);
			
			/* Send your document an initialization
			** message. The first argument is the
			** supervisor (the application). The second
			** argument is TRUE if the document is printable. */
		
		theDocument->IStarterDoc(this, FALSE);
	
			/* Send the document a NewFile() message.
			** The document will open a window, and
			** set up the heart of the application. */
			
		theDocument->NewFile();
	}
	
	CATCH
	{
		/* This exception handler gets executed if a failure occurred 
		** anywhere within the scope of the TRY block above. Since 
		** this indicates that a new doc could not be created, we
		** check if an object had been allocated and if it has, send 
		** it a Dispose message. The exception will propagate up to
		** CSwitchboard's exception handler, which handles displaying
		** an error alert.											  */
		 
		 if (theDocument) theDocument->Dispose();

	}
	
	ENDTRY;
	
}	/* CreateDocument */



/* OpenDocument
**
**		The user chose Open from the File menu.
**		In this method you need to create a document
**		and send it an OpenFile() message.
**
**		The macSFReply is a good SFReply record that contains
**		the name and vRefNum of the file the user chose to open.
*/
 
void	CStarterApp::OpenDocument (SFReply *macSFReply)		{

		CStarterDoc	*theDocument = NULL;
	
	
	TRY
	{
	
		theDocument = new(CStarterDoc);
			
			/* Send your document an initialization
			** message. The first argument is the
			** supervisor (the application). The second
			** argument is TRUE if the document is printable.	*/
		
		theDocument->IStarterDoc(this, FALSE);
	
			/* Send the document an OpenFile() message.
			** The document will open a window, open
			** the file specified in the macSFReply record,
			** and display it in its window.				*/
			
		theDocument->OpenFile(macSFReply);
	}
	
	CATCH
	{
		/* This exception handler gets executed if a failure occurred 
		** anywhere within the scope of the TRY block above. Since 
		** this indicates that the document could not be opened, we
		** send it a Dispose message. The exception will propagate up to
		** CSwitchboard's exception handler, which handles displaying
		** an error alert.												*/
		 
		 if (theDocument) theDocument->Dispose();
	}
	
	ENDTRY;
	
}	/* OpenDocument */



void	CStarterApp::MakeDesktop (void)		{

	
	gDesktop = new (CFWDesktop);
	((CFWDesktop*)gDesktop)->IFWDesktop(this);
	
}	/* MakeDesktop */



	static void		Window_ShowHide (CWindow *theWindow, long showFlag)		{


		if (theWindow->active)		{
			if (showFlag)	theWindow->ShowResume();
			else			theWindow->HideSuspend();
		}
		
	}	/* Window_ShowHide */



void	CStarterApp::Suspend (void)		{
		
		
	inherited::Suspend();

	gSaveBalloons = gBalloonsUp;		// Where we're at !!!
	ResetBalloons(gStartBalloons);		// On Day Numero Uno !!!

	((CFWDesktop*)gDesktop)->CalcTopFloat();
	if ( ((CFWDesktop*)gDesktop)->topFloat )
    	((CFWDesktop*)gDesktop)->itsFloats->
    							DoForEach1((EachFunc1) Window_ShowHide, (long)false);
	
}	/* Suspend */



void	CStarterApp::Resume (void)		{


	inherited::Resume();

	ResetBalloons(gSaveBalloons);
	gBalloonsUp = gSaveBalloons;

	if ( ((CFWDesktop*)gDesktop)->topFloat )
    	((CFWDesktop*)gDesktop)->itsFloats->
    							DoForEach1((EachFunc1) Window_ShowHide, (long)true);
	((CFWDesktop*)gDesktop)->CalcTopFloat();
	
}	/* Resume */




/* end: "CmyApp.c" */