/***************************************************************************
 Apple Macintosh Developer Technical Support

 Some of "Skippy White's Famous High Level Off-Screen Map Routines"

 "CmyBitMap.c"

 These routines provide a high-level interface to the QuickDraw & Color
 Manager routines which allow the creation and manipulation of off-screen
 bitmaps and pixmaps. They are designed to run on any machine with 128K or
 later ROMs.

 NOTE that I've modified some of Skippy's routines and that, therefore, any
      resultant errors in syntax or logic belong solely to me.

 using Symantec's "THINK C", v 5.0.1
 ... as derived from their "TCL Starter" files
****************************************************************************/




#include <Exceptions.h>			/* Has declaration of "kSilentErr" */
#include <TCLUtilities.h>		/* ... and SetAllocation().		   */

#include "CmyGlobals.h"
#include "CmyBitMap.h"


extern tSystem		gSystem;
extern short		gLastError;



	static void		ErrorOut (OSErr error)	{


		if (error != noErr)   /* ARRRRGH !!! */   Failure(error, kSilentErr);

	}	/* ErrorOut */



/**************************
 Just plain nifty stuff ...
 **************************/

void	CmyBitMap::IBitMap (short width, short height, Boolean makePort)	{
/* Reference: Tech Note #120
   with special thanks to:
                        Jon Zap and
                        Forrest Tanaka of MacDTS

   NOTE: I'm going to totally ignore "makePort" because
   		 my BitMap is intricately tied to the port.		*/

		long			offRowBytes, sizeOfOff, tempSeed;
		Rect			bmRect, localRect, globRect;
		short			maxDepth;
		register short	i;
		OSErr			err;
		Boolean			savedAlloc;


	MoveHHi(this);
	HLock(this);						/* Lock this sucker down !! */

	xferMode = srcCopy;
	;
	SetRect(&bmRect, 0, 0, width, height);
	
 /* ErrorOut(noErr);	--    We hope !! */
	GetPort(&savePort);					/* Used by superclass' EndDrawing. */

	globRect = bmRect;					/* We're about to switch the */
	LocalToGlobal(&topLeft(globRect));	/*   Port to off-screen:	 */
	LocalToGlobal(&botRight(globRect));

	if (!gSystem.hasColorQD)
	{
		macPort = &offGrafPort;
		OpenPort(macPort);
		maxDepth = 1;
	}   /* ... a low-life machine */
	else
	{

	 /* Figure out how much space we need for our pixel image.  We call
		mommy ROM's GetMaxDevice to get the pixel map from that.
		We do this to cover the case where the pixel image that we wish to
		CopyBits to spans multiple devices (of possibly different depths). */

		myMaxDevice = GetMaxDevice(&globRect);
		if (myMaxDevice == nil)		{
			ErrorOut(MaxDevError);
			return;
		}

	 /* We need to set theGDevice to the device with the maximum pixel
		depth (the one we just found), so the pixel map of the new
		CGrafPort will be copied from one of the proper depth.  Save
		the current GDevice for retrieval later and, then, open the
		new CGrafPort to use for our off-screen drawing.			  */

		saveDevice = GetGDevice();
		SetGDevice(myMaxDevice);

		/* Initialize this guy: */
		macCPort = &offCGrafPort;
		macPort = (GrafPtr)macCPort;
		OpenCPort(macCPort);
		/* Arrgh !! Batten down the hatches: */
		MoveHHi(macCPort->portPixMap);
		HLock(macCPort->portPixMap);
		maxDepth = (*macCPort->portPixMap)->pixelSize;

	};   /* else: hasColorQD */

 /* CanNOT use GlobalToLocal(s) here because we may have dragged our
	window to a secondary screen.  "Global" here is with respect to
	the main screen and "Local" is with respect to the secondary screen.

	From Forrest Tanaka:
	When GetMaxDevice returns the secondary screen's GDevice & we set that
	to the current GDevice, then OpenCPort creates a CGrafPort which has a
	portRect = (**((**GetMainDevice).gdPMap)).bounds which is in the global
	coordinates for all the screens' pixel images with a topLeft = (0,0).
	The new (**CGrafPort.portPixMap).bounds is in the local coordinates of
	the secondary screen with a topLeft = (0,-640), for example.

	In effect, the port that OpenCPort gives you is NOT a valid port because
	the portRect pertains to the wrong screen.  This means that calling
	GlobalLocal shifts the localRect waaaaay over somewhere ... BECAUSE
	the difference between the above portRect and PixMap bounds is SO large: */

	localRect = globRect;
	OffsetRect(&localRect, screenBits.bounds.left, screenBits.bounds.top);

 /* However, before we do ANYthing more, we should set the
	off-screen's visRgn to the FULL size of the input rect so the
	image stays whole even when the window has been dragged partly
	beyond the physical edge(s) of the screen.  Otherwise, the
	(**visRgn).rgnBBox in local coordinates remains equal to
	screenBits.bounds as initialized when _Open(C)Port was called:	*/

	if (gSystem.hasColorQD)
	{
		RectRgn(macCPort->visRgn, &localRect);
		macCPort->portRect = localRect;
	}
	else
	{
		RectRgn(macPort->visRgn, &localRect );
		macPort->portRect = localRect;
	};

 /* We are now ready to calculate the size of the pixel image we will need.
	Then we can set the location-specific and size-specific information of
	the pixel map.  Since Color QuickDraw distinguishes between a bitmap and
	a pixel map by checking the high bit of rowBytes, we need to set it.	 */

	/* # of words: */
	offRowBytes = (maxDepth * (localRect.right - localRect.left) + 15) / 16;
	if (offRowBytes % 2)	offRowBytes++;		/* ... made even. */
	offRowBytes *= 2;							/* Back to bytes. */
	sizeOfOff = (long)(localRect.bottom - localRect.top) * offRowBytes;

	/* Allocate space for the pixel image: */
	savedAlloc = SetAllocation(kAllocCanFail);
	myBits = NewHandleClear(sizeOfOff);
	SetAllocation(savedAlloc);
	if (MemError() != noErr)	{
		ErrorOut(NewBaseAddrPtrError);
		return;
	}

	MoveHHi(myBits);
	HLock(myBits);

 /* NOTE that we're filling in the BitMap/PixMap fields of the new Port
	directly, so we do NOT call _SetPortBits or _SetCPortPix later:		*/

	if (gSystem.hasColorQD)
	{
		(**macCPort->portPixMap).baseAddr = *myBits;
		/* Remember to be a PixMap: */
		(**macCPort->portPixMap).rowBytes = offRowBytes | 0x8000;
		(**macCPort->portPixMap).bounds	  = localRect;

		macBitMap = (BitMapPtr) *(macCPort->portPixMap);
	}   /* if aMac2 */

	else   /* definitely ... "YUCKY" black-and-white. */
	{
		macPort->portBits.baseAddr = *myBits;
		macPort->portBits.rowBytes = offRowBytes;
		macPort->portBits.bounds   = localRect;

		macBitMap = &macPort->portBits;
	};

	if (gSystem.hasColorQD)
	{
	 /* Next, we clone the color table of the maxDevice
		and put it into our off-screen pixel map.		*/

		ourCTHandle = ( **(( **myMaxDevice ).gdPMap) ).pmTable;
		savedAlloc = SetAllocation(kAllocCanFail);
		err = HandToHand(&ourCTHandle);			/* Clone it. */
		SetAllocation(savedAlloc);
		if (err != noErr)	{
			ErrorOut(CloneHdlError);
			return;
		}

		for (i = 0; i <= (**ourCTHandle).ctSize; i++)
			(**ourCTHandle).ctTable[i].value = i;

	 /* The following is required to convert GDevice cluts to Pixmap cluts:	*/

		(**ourCTHandle).ctFlags &= 0x7FFF;
		tempSeed = GetCTSeed();					/* Thanks, Scott Knaster !! */
		(**ourCTHandle).ctSeed = tempSeed;
		/* --> the off-screen map: */
		(**( macCPort->portPixMap )).pmTable = ourCTHandle;
	};   /* if hasColorQD */

	if (gSystem.hasColorQD)		SetGDevice(saveDevice);
	SetPort(savePort);
	
	CView::ForceNextPrepare();

	ErrorOut(noErr);					/* Whew !! */

}	/* IBitMap */



/****************************
 Out with the new.

 Whoops -- I meant the old !!
 **************************** */

void	CmyBitMap::Dispose (void)	{


	if (gLastError == MaxDevError)	goto bye;

    /* NewBaseAddrPtrError or CloneHdlError or noErr ... */

	if (gSystem.hasColorQD)
	{
		if (gLastError == noErr)
			DisposHandle(ourCTHandle);
		HUnlock(macCPort->portPixMap);
		CloseCPort(macCPort);
	}
	else
		ClosePort(macPort);
	macPort = NULL;				/* So Dispose() below doesn't do it again. */

	if (gLastError != NewBaseAddrPtrError)
	{
		HUnlock(myBits);
		DisposHandle(myBits);
	}
	macBitMap = NULL;			/* No need to repeat yourself !! */

bye:
	HUnlock(this);

	inherited::Dispose();		/* Kiss him good-bye !! */
	
}	/* Dispose */



void	CmyBitMap::BeginDrawing (void)	{
/* Object is still locked coming in. */

	
	inherited::BeginDrawing();
	
	if (gSystem.hasColorQD)		{
		saveDevice = GetGDevice();
		SetGDevice(myMaxDevice);
	}

}	/* BeginDrawing*/



/*******************
 Back to "Square 1":
 *******************/

void	CmyBitMap::EndDrawing (void)		{
/* Object is locked coming in. */


	if (gSystem.hasColorQD)		SetGDevice(saveDevice);
	inherited::EndDrawing();

}	/* EndDrawing*/




/* end: "CmyBitMap.c" */