{	IconTester.p	-	Steve Sheets, 1991}
{	Sample program to display Icon}
{	Family resources.  This program will}
{	display 3 different Icon Families in all}
{	the various modes.  If any of the icons}
{	are clicked, that icon will become hilited.}

program IconTester;
	const
{	Icon position constants}

		kRow1 = 50;
		kRow2 = 120;
		kRow3 = 190;

		kColumn1 = 10;
		kColumn2 = 60;
		kColumn3 = 110;
		kColumn4 = 160;
		kColumn5 = 210;
		kColumn6 = 260;
		kColumn7 = 310;
		kColumn8 = 360;

{	resource ID of Icon Families}

		kApplicationIcon = 128;
		kCheckerDocIcon = 129;
		kMoofIcon = 130;

{	Window Size Constants}

		kHSize = 300;

		kVColorSize = 230;
		kVBWSize = 90;

{	Number of Selection Flags needed}

		kNumFlags = 18;

		HiliteMode = $938;
		pHiliteBit = 0;

	type
		IconStyle = (BW32, BW16, EightBit32, EightBit16, FourBit32, FourBit16);

{	Globals}
	var
		gIconWindow: WindowPtr;
		gHaveColor, gDone: BOOLEAN;
		gFlags: array[1..kNumFlags] of BOOLEAN;

{	Checks if Color Quickdraw Exists on this machine.}

	function ColorQDExists: boolean;
		const
			ROM85Loc = $28E;
			TwoHighMask = $C000;
		type
			WordPtr = ^INTEGER;
	begin
		ColorQDExists := (BitAnd(WordPtr(ROM85Loc)^, TwoHighMask) = 0);
	end;

{	Prepares the program by setting up variables}
{and creating a window (possibly Color window).}
{No icons are loaded here.}

	procedure SetUp;
		var
			myRect: Rect;
			myStr: Str255;
			myCount: INTEGER;
	begin
		gHaveColor := ColorQDExists;

		myStr := 'Icon Family Demo';
		with myRect, screenbits do
			begin
				top := 40;
				left := (bounds.right - bounds.left - kHSize) div 2;
				right := left + kHSize;
			end;
		if gHaveColor then
			begin
				myRect.bottom := myRect.top + kVColorSize;
				gIconWindow := NewCWindow(nil, myRect, myStr, TRUE, noGrowDocProc, POINTER(-1), TRUE, 0)
			end
		else
			begin
				myRect.bottom := myRect.top + kVBWSize;
				gIconWindow := NewWindow(nil, myRect, myStr, TRUE, noGrowDocProc, POINTER(-1), TRUE, 0);
			end;

		gDone := FALSE;
		for myCount := 1 to kNumFlags do
			gFlags[myCount] := FALSE;
		SetCursor(Arrow);
	end;

	procedure DrawIconFamily (theIconStyle: IconStyle; theBWIcon: Handle; theColorIcon: Handle; thePoint: Point; theFlag: BOOLEAN);
		var
			myDim, myBWRowbytes, myBWSize: INTEGER;
			myRect: Rect;
			myIconBitMap, myMaskBitMap: BitMap;
			myPort: GrafPtr;
			myCLUT: CTabHandle;
			myColorIcon: CIconHandle;
			myPtr1, myPtr2: Ptr;
	begin
{	Check to see if Handle exists}

		if theBWIcon <> nil then
			if theBWIcon^ <> nil then
				begin
					Hlock(theBWIcon);

{    Setup variables based on type of Icon }

					if theIconStyle in [bw16, Eightbit16, Fourbit16] then
						begin
							myBWRowbytes := 2;
							myDim := 16;
						end
					else
						begin
							myBWRowbytes := 4;
							myDim := 32;
						end;

{	Setup Mask Bitmap}

					with myMaskBitMap do
						begin
							myBWSize := myBWRowbytes * myDim;
							baseaddr := Ptr(ord4(StripAddress(theBWIcon^)) + myBWSize);
							rowbytes := myBWRowbytes;
							SetRect(bounds, 0, 0, myDim, myDim);
						end;

{	Setup bounding rectangle}

					with myRect do
						begin
							topleft := thePoint;
							right := left + myDim;
							bottom := top + myDim;
						end;

{	Grab current grafport (for bitmap}

					GetPort(myPort);

{	Check type of mode}

					if theIconStyle in [bw16, bw32] then
						begin
{	If mode is BW, set Icon bitmap}

							with myIconBitMap do
								begin
									baseaddr := StripAddress(theBWIcon^);
									rowbytes := myBWRowbytes;
									SetRect(bounds, 0, 0, myDim, myDim);
								end;

				{	White out Mask Bitmap}
							CopyBits(myMaskBitMap, myPort^.portBits, myMaskBitMap.bounds, myRect, srcBic, nil);
				{	Set Icon bitmap}
							CopyBits(myIconBitMap, myPort^.portBits, myIconBitMap.bounds, myRect, srcOr, nil);
						end
{	If mode is color, make sure color}
{handle is not nil.}
					else if theColorIcon <> nil then
						begin
{	Get CLUT.}

							if theIconStyle in [EightBit16, EightBit32] then
								myCLUT := GetCTable(8)
							else
								myCLUT := GetCTable(4);

							if myCLUT <> nil then
								begin

{	Allocate memory for Color Icon}
									myColorIcon := CIconHandle(NewHandle(SIZEOF(CIcon) + 2 * myBWSize));

									if myColorIcon <> nil then
										begin
{	Load in fields}
											with myColorIcon^^ do
												begin
													with iconPMap do
														begin
															baseAddr := nil;
															SetRect(bounds, 0, 0, myDim, myDim);
															case theIconStyle of
																EightBit16: 
																	begin
																		pixelSize := 8;
																		rowbytes := $8010;
																	end;

																EightBit32: 
																	begin
																		pixelSize := 8;
																		rowbytes := $8020;
																	end;

																FourBit16: 
																	begin
																		pixelSize := 4;
																		rowbytes := $8008;
																	end;

																FourBit32: 
																	begin
																		pixelSize := 4;
																		rowbytes := $8010;
																	end;
																otherwise
															end;
															pmVersion := 0;
															packType := 0;
															packSize := 0;
															hRes := 72;
															vRes := 72;
															pixelType := 0;
															cmpCount := 1;
															cmpSize := pixelSize;
															planebytes := 0;
															pmReserved := 0;
														end;
													with iconMask do
														begin
															rowbytes := myBWRowbytes;
															baseaddr := nil;
															SetRect(bounds, 0, 0, myDim, myDim);
														end;
													iconBMap := iconMask;
												end;

{	Load in CLUT & pixel data}

											myColorIcon^^.iconData := theColorIcon;
											myColorIcon^^.iconPMap.pmTable := myCLUT;

{Copy in BW image and mask}
											HLock(Handle(myColorIcon));
											myPtr1 := POINTER(ORD4(StripAddress(theBWIcon^)) + myBWSize);
											myPtr2 := @myColorIcon^^.iconMaskData[0];
											BlockMove(myPtr1, myPtr2, myBWSize);
											myPtr1 := StripAddress(theBWIcon^);
											myPtr2 := POINTER(ORD4(StripAddress(@myColorIcon^^.iconMaskData[0])) + myBWSize);
											BlockMove(myPtr1, myPtr2, myBWSize);
											HUnlock(Handle(myColorIcon));
{	Plot Icon}
											PlotCIcon(myRect, myColorIcon);

											DisposHandle(Handle(myColorIcon));
										end;
									DisposCTable(myCLUT);
								end;
						end;
					Hunlock(theBWIcon);
				end;
	end;

	procedure SelectIconFamily (theIconStyle: IconStyle; theBWIcon: Handle; thePoint: Point);
		var
			myDim, myBWRowbytes, myBWSize: INTEGER;
			myRect: Rect;
			myMaskBitMap: BitMap;
			myPort: GrafPtr;
	begin
{	Check to see if Handle exists}

		if theBWIcon <> nil then
			if theBWIcon^ <> nil then
				begin
					Hlock(theBWIcon);

{	Setup variables based on type of Icon}

					if theIconStyle in [bw16, Eightbit16, Fourbit16] then
						begin
							myBWRowbytes := 2;
							myDim := 16;
						end
					else
						begin
							myBWRowbytes := 4;
							myDim := 32;
						end;

{	Setup Mask Bitmap}

					with myMaskBitMap do
						begin
							myBWSize := myBWRowbytes * myDim;
							baseaddr := Ptr(ord4(StripAddress(theBWIcon^)) + myBWSize);
							rowbytes := myBWRowbytes;
							SetRect(bounds, 0, 0, myDim, myDim);
						end;

{	Setup bounding rectangle}

					with myRect do
						begin
							topleft := thePoint;
							right := left + myDim;
							bottom := top + myDim;
						end;

{	Grab current grafport (for bitmap}

					GetPort(myPort);

{	Invert the mask bitmap}

					BitClr(Ptr(HiliteMode), pHiliteBit);
					CopyBits(myMaskBitMap, myPort^.portBits, myMaskBitMap.bounds, myRect, srcXor, nil);

					Hunlock(theBWIcon);
				end;
	end;

	function PtInIconFamily (theIconStyle: IconStyle; theBWIcon: Handle; thePoint, theSpot: Point): BOOLEAN;
		var
			myDim, myBWRowbytes, myBWSize: INTEGER;
			myRect: Rect;
			myFlag: BOOLEAN;
			myPtr: Ptr;
	begin
		myFlag := FALSE;

{	Check to see if Handle exists}

		if theBWIcon <> nil then
			if theBWIcon^ <> nil then
				begin
					Hlock(theBWIcon);

{	Setup variables based on type of Icon}

					if theIconStyle in [bw16, Eightbit16, Fourbit16] then
						begin
							myBWRowbytes := 2;
							myDim := 16;
						end
					else
						begin
							myBWRowbytes := 4;
							myDim := 32;
						end;
					myBWSize := myBWRowbytes * myDim;

{	Setup bounding rectangle}

					with myRect do
						begin
							topleft := thePoint;
							right := left + myDim;
							bottom := top + myDim;
						end;

{	Check if spot in bounding rect}

					if PtInRect(theSpot, myRect) then
						begin
{	If so, setup math to look at specific}
{	row of pixels.}
							Hlock(theBWIcon);
							theSpot.v := theSpot.v - thePoint.v;
							theSpot.h := theSpot.h - thePoint.h;
							myPtr := POINTER(ORD4(StripAddress(theBWIcon^)) + myBWSize + (theSpot.v * myBWRowbytes));

{	Check if spot touching set pixel (bit)}

							myFlag := BitTst(myPtr, theSpot.h);
							HUnlock(theBWIcon);
						end;
				end;

		PtInIconFamily := myFlag;
	end;

{Updates the Icon image window.  Draws}
{some text information, then draws all}
{the icons.}

	procedure UpdateIconWindow;

{	Given an icon style, position, resource ID}
{number and selection flag number (number of}
{item in array), draws that icon.  It does this}
{by loading in the correct resources, and calling}
{DrawIconFamily.  When it is done, the resources}
{are set to unpurgeable (allow them to be freed}
{if memory space is an issue).}

		procedure DrawMyIcon (theIconStyle: IconStyle; theH, theV: INTEGER; theIconID: INTEGER; theFlagNum: INTEGER);
			var
				myIconMask, myColorIcon: Handle;
				myFlag: BOOLEAN;
				myPoint: Point;
		begin
			if theIconStyle in [bw16, Eightbit16, Fourbit16] then
				myIconMask := GetResource('ics#', theIconID)
			else
				myIconMask := GetResource('ICN#', theIconID);
			if myIconMask <> nil then
				begin
					HNoPurge(myIconMask);

					myFlag := FALSE;
					myColorIcon := nil;
					case theIconStyle of
						Eightbit16: 
							myColorIcon := GetResource('ics8', theIconID);

						Fourbit16: 
							myColorIcon := GetResource('ics4', theIconID);

						Eightbit32: 
							myColorIcon := GetResource('icl8', theIconID);

						Fourbit32: 
							myColorIcon := GetResource('icl4', theIconID);
						otherwise
							myFlag := TRUE;
					end;

					if myColorIcon <> nil then
						begin
							HNoPurge(myColorIcon);
							myFlag := TRUE;
						end;

					if myFlag then
						begin
							myPoint.h := theH;
							myPoint.v := theV;
							DrawIconFamily(theIconStyle, myIconMask, myColorIcon, myPoint, gFlags[theFlagNum]);
						end;

					if myColorIcon <> nil then
						HPurge(myColorIcon);
					HPurge(myIconMask);
				end;
		end;

	begin
		MoveTo(10, 20);
		DrawString('Select an Icon to hilite it.');

		MoveTo(10, 40);
		DrawString('Black and White Icons:');
		DrawMyIcon(bw32, kColumn1, kRow1, kApplicationIcon, 1);
		DrawMyIcon(bw16, kColumn2, kRow1, kApplicationIcon, 2);
		DrawMyIcon(bw32, kColumn3, kRow1, kCheckerDocIcon, 3);
		DrawMyIcon(bw16, kColumn4, kRow1, kCheckerDocIcon, 4);
		DrawMyIcon(bw32, kColumn5, kRow1, kMoofIcon, 5);
		DrawMyIcon(bw16, kColumn6, kRow1, kMoofIcon, 6);

		if gHaveColor then
			begin
				MoveTo(10, 110);
				DrawString('Four Bit Icons:');
				DrawMyIcon(Fourbit32, kColumn1, kRow2, kApplicationIcon, 7);
				DrawMyIcon(Fourbit16, kColumn2, kRow2, kApplicationIcon, 8);
				DrawMyIcon(Fourbit32, kColumn3, kRow2, kCheckerDocIcon, 9);
				DrawMyIcon(Fourbit16, kColumn4, kRow2, kCheckerDocIcon, 10);
				DrawMyIcon(Fourbit32, kColumn5, kRow2, kMoofIcon, 11);
				DrawMyIcon(Fourbit16, kColumn6, kRow2, kMoofIcon, 12);

				MoveTo(10, 180);
				DrawString('Eight Bit Icons:');
				DrawMyIcon(Eightbit32, kColumn1, kRow3, kApplicationIcon, 13);
				DrawMyIcon(Eightbit16, kColumn2, kRow3, kApplicationIcon, 14);
				DrawMyIcon(Eightbit32, kColumn3, kRow3, kCheckerDocIcon, 15);
				DrawMyIcon(Eightbit16, kColumn4, kRow3, kCheckerDocIcon, 16);
				DrawMyIcon(Eightbit32, kColumn5, kRow3, kMoofIcon, 17);
				DrawMyIcon(Eightbit16, kColumn6, kRow3, kMoofIcon, 18);
			end;
	end;

{	Handles a mousedown in the window}
{(local cooridinates).  Checks for each}
{of the icons if click was on top of it.}
{If so, it changes the hiliting of that icon}
{(and the flag associated with it).}

	procedure ClickIconWindow (theSpot: Point);
{	Given an iconstyle, position, resource ID and}
{flag number, check if theSpot is touching it}
{(using PtInIconFamily).  If so, flips the flag and changes}
{the hiliting (using SelectIconFamily).}

		procedure CheckMyIcon (theIconStyle: IconStyle; theH, theV: INTEGER; theIconID: INTEGER; theFlagNum: INTEGER);
			var
				myIconMask: Handle;
				myPoint: Point;
		begin
			if theIconStyle in [bw16, Eightbit16, Fourbit16] then
				myIconMask := GetResource('ics#', theIconID)
			else
				myIconMask := GetResource('ICN#', theIconID);
			if myIconMask <> nil then
				begin
					HNoPurge(myIconMask);

					myPoint.h := theH;
					myPoint.v := theV;
					if PtInIconFamily(theIconStyle, myIconMask, myPoint, theSpot) then
						begin
							gFlags[theFlagNum] := not gFlags[theFlagNum];
							SelectIconFamily(theIconStyle, myIconMask, myPoint);
						end;

					HPurge(myIconMask);
				end;
		end;

	begin
		CheckMyIcon(bw32, kColumn1, kRow1, kApplicationIcon, 1);
		CheckMyIcon(bw16, kColumn2, kRow1, kApplicationIcon, 2);
		CheckMyIcon(bw32, kColumn3, kRow1, kCheckerDocIcon, 3);
		CheckMyIcon(bw16, kColumn4, kRow1, kCheckerDocIcon, 4);
		CheckMyIcon(bw32, kColumn5, kRow1, kMoofIcon, 5);
		CheckMyIcon(bw16, kColumn6, kRow1, kMoofIcon, 6);

		if gHaveColor then
			begin
				CheckMyIcon(Fourbit32, kColumn1, kRow2, kApplicationIcon, 7);
				CheckMyIcon(Fourbit16, kColumn2, kRow2, kApplicationIcon, 8);
				CheckMyIcon(Fourbit32, kColumn3, kRow2, kCheckerDocIcon, 9);
				CheckMyIcon(Fourbit16, kColumn4, kRow2, kCheckerDocIcon, 10);
				CheckMyIcon(Fourbit32, kColumn5, kRow2, kMoofIcon, 11);
				CheckMyIcon(Fourbit16, kColumn6, kRow2, kMoofIcon, 12);

				CheckMyIcon(Eightbit32, kColumn1, kRow3, kApplicationIcon, 13);
				CheckMyIcon(Eightbit16, kColumn2, kRow3, kApplicationIcon, 14);
				CheckMyIcon(Eightbit32, kColumn3, kRow3, kCheckerDocIcon, 15);
				CheckMyIcon(Eightbit16, kColumn4, kRow3, kCheckerDocIcon, 16);
				CheckMyIcon(Eightbit32, kColumn5, kRow3, kMoofIcon, 17);
				CheckMyIcon(Eightbit16, kColumn6, kRow3, kMoofIcon, 18);
			end;
	end;

{	Main event loop of application.  Handles}
{all normal events, and passes updates &}
{mouse downs to UpdateIconWindow and}
{ClickIconWindow.  This routine loops until}
{user selects close box.}

	procedure MainLoop;
		var
			myWindow: WindowPtr;
			myPort: GrafPtr;
			myEvent: EventRecord;
	begin
		repeat
			SystemTask;

			if GetNextEvent(everyEvent, myEvent) then
				case myEvent.what of

					mouseDown: 
						case FindWindow(myEvent.where, myWindow) of
							inSysWindow: 
								SystemClick(myEvent, myWindow);
							inGoAway: 
								if myWindow = gIconWindow then
									if TrackGoAway(myWindow, myEvent.where) then
										gDone := TRUE;
							inDrag: 
								if myWindow <> frontWindow then
									SelectWindow(myWindow)
								else
									DragWindow(myWindow, myEvent.where, screenbits.bounds);
							inContent: 
								if myWindow <> frontWindow then
									SelectWindow(myWindow)
								else if myWindow = gIconWindow then
									begin
										GetPort(myPort);
										SetPort(myWindow);
										GlobalToLocal(myEvent.where);
										ClickIconWindow(myEvent.where);
										SetPort(myPort);
									end;
						end;

					updateEvt: 
						begin
							GetPort(myPort);
							myWindow := WindowPtr(myEvent.message);
							BeginUpdate(myWindow);
							SetPort(myWindow);
							if myWindow = gIconWindow then
								UpdateIconWindow;
							SetPort(myPort);
						end;

					otherwise
				end;
		until gDone;
	end;

{	Main program}

begin
	SetUp;
	MainLoop;
end.