//------------------------------------------------------------------------------
//	UIconEdit.cp - Class implementations for IconEdit
//
//	Written by K.L. Colclasure for MacTutor, Dec 3, 1989
//	Copyright  1989 MacTutor, all rights reserved.
//------------------------------------------------------------------------------

// These standard types are missing from MPW 3.0 Types.h:
typedef signed char	SignedByte;
typedef unsigned char	Byte;
typedef enum { v, h }	VHSelect;

#include <UMacApp.h>
#include <UIconEdit.h>

#include <ToolUtils.h>

const short			cZoomIn = 1000;
const short			cZoomOut = 1001;
const short			cInvert = 1002;
const short			cDraw = 2000;

const short			kSeedIconID = 1000;
const short			kIconWindowID = 1000;

const short			kIconHBits = 32, kIconVBits = 32;
const long			kIconSizeInBytes =
						kIconHBits * kIconVBits / 8;
const long			kIconSizeInLongs =
						kIconSizeInBytes / 4;
const short			kMaxLong =
						(short) kIconSizeInLongs - 1;

typedef long			LongArray[kIconSizeInLongs];
typedef LongArray		**LongArrayHandle;

//------------------------------------------------------------------------------
pascal void TIconBitMap::IIconBitMap (void) {
//------------------------------------------------------------------------------
	fDataHandle = NewPermHandle (kIconSizeInBytes);
	FailNIL (fDataHandle);
}

//------------------------------------------------------------------------------
pascal void TIconBitMap::Free (void) {
//------------------------------------------------------------------------------
	DisposIfHandle (fDataHandle);
	TObject::Free ();
}

//------------------------------------------------------------------------------
pascal void TIconBitMap::SetIconBitMap (Handle theBitMap) {
//------------------------------------------------------------------------------
	BlockMove (*theBitMap,
		*fDataHandle, kIconSizeInBytes);
}

//------------------------------------------------------------------------------
pascal void TIconBitMap::Clear (void) {
//------------------------------------------------------------------------------
	LongArrayHandle	iconData;
	
	iconData = (LongArrayHandle) fDataHandle;
	for (int i = 0; i <= kMaxLong; i++) {
		(**iconData)[i] = 0;
	}
}

//------------------------------------------------------------------------------
pascal void TIconBitMap::Invert (void) {
//------------------------------------------------------------------------------
	LongArrayHandle	iconData;
	
	iconData = (LongArrayHandle) fDataHandle;
	for (int i = 0; i <= kMaxLong; i++) {
		(**iconData)[i] ^= 0xffffffff;
	}
}

//------------------------------------------------------------------------------
pascal Boolean TIconBitMap::GetBit (Point iconBit) {
//------------------------------------------------------------------------------
	long			bit;
	
	bit = iconBit.v * kIconVBits + iconBit.h;
	return (BitTst (*fDataHandle, bit));
}

//------------------------------------------------------------------------------
pascal void TIconBitMap::SetBit (Point iconBit,
	Boolean turnBitOn) {
//------------------------------------------------------------------------------
	long			bit;
	
	bit = iconBit.v * kIconVBits + iconBit.h;
	if (turnBitOn) BitSet (*fDataHandle, bit);
	else BitClr (*fDataHandle, bit);
}

//------------------------------------------------------------------------------
pascal TIconBitMap *TIconBitMap::Copy (void) {
//------------------------------------------------------------------------------
	TIconBitMap		*copyOfIcon;
	
	copyOfIcon = new TIconBitMap;
	FailNIL (copyOfIcon);
	copyOfIcon->IIconBitMap ();
	copyOfIcon->SetIconBitMap (fDataHandle);
	return (copyOfIcon);
}

//------------------------------------------------------------------------------
pascal void TIconBitMap::CopyDataTo (
	TIconBitMap *anIcon) {
//------------------------------------------------------------------------------
	anIcon->SetIconBitMap (fDataHandle);
}

//------------------------------------------------------------------------------
pascal void TIconBitMap::Draw (Rect *area) {
//------------------------------------------------------------------------------
	PlotIcon (area, fDataHandle);
}

#if qInspector
//------------------------------------------------------------------------------
pascal void TIconBitMap::Fields (
	FieldProcPtr DTF, void *link) {
//------------------------------------------------------------------------------
	(*DTF) ("\pTIconBitMap", nil, bClass, link);
	(*DTF) ("\pfDataHandle", (Ptr) &fDataHandle, bHandle,
		link);
	inherited::Fields (DTF, link);
}
#endif

//------------------------------------------------------------------------------
pascal void TIconView::IRes (TDocument *itsDocument,
	TView *itsSuperView, Ptr *itsParams) {
//------------------------------------------------------------------------------
	TView::IRes (itsDocument, itsSuperView, itsParams);
	fIconDocument = (TIconDocument *) itsDocument;
	fMagnification = kDefaultMagnification;
}

//------------------------------------------------------------------------------
pascal void TIconView::CalcMinSize (VPoint *minSize) {
//------------------------------------------------------------------------------
	minSize->h = (32 * fMagnification) + (2 * kBorder);
	minSize->v = minSize->h;
}

//------------------------------------------------------------------------------
pascal void TIconView::Draw (Rect *area) {
//------------------------------------------------------------------------------
	Rect			destRect;
	
	SetRect (&destRect, kBorder, kBorder,
		kBorder + (32 * fMagnification),
		kBorder + (32 * fMagnification));
	fIconDocument->fIconBitMap->Draw (&destRect);
}

//------------------------------------------------------------------------------
pascal void TIconView::DoSetupMenus (void) {
//------------------------------------------------------------------------------
	inherited::DoSetupMenus ();
	Enable (cZoomIn, fMagnification < 32);
	Enable (cZoomOut, fMagnification > 1);
}

//------------------------------------------------------------------------------
pascal TCommand *TIconView::DoMenuCommand (
	CmdNumber whichCmd) {
//------------------------------------------------------------------------------
	switch (whichCmd) {
		case cZoomIn:
			SetMagnification (fMagnification + 1);
			return (gNoChanges);
		case cZoomOut:
			SetMagnification (fMagnification - 1);
			return (gNoChanges);
		default:
			return (inherited::DoMenuCommand (
				whichCmd));
	}
}

//------------------------------------------------------------------------------
pascal void TIconView::SetMagnification (
	short newMagnification) {
//------------------------------------------------------------------------------
	fMagnification = newMagnification;
	AdjustSize ();
	ForceRedraw ();
}

//------------------------------------------------------------------------------
pascal TCommand *TIconView::DoMouseCommand (
	Point *theMouse, EventInfo *info, Point *hysteresis) {
//------------------------------------------------------------------------------
	TDrawCommand	*theCommand;
	Point			unusedPoint;
	
	if (PointToBit (*theMouse, &unusedPoint)) {
		theCommand = new TDrawCommand;
		FailNIL (theCommand);
		theCommand->IDrawCommand (this);
		return (theCommand);
	} else return (gNoChanges);
}

//------------------------------------------------------------------------------
pascal Boolean TIconView::PointToBit (Point thePoint,
	Point *iconBit) {
//------------------------------------------------------------------------------
	thePoint.h -= kBorder;
	thePoint.v -= kBorder;
	if ((thePoint.h >= 0) &&
		(thePoint.h < 32 * fMagnification) &&
		(thePoint.v >= 0) &&
		(thePoint.v < 32 * fMagnification)) {
		iconBit->h = thePoint.h / fMagnification;
		iconBit->v = thePoint.v / fMagnification;
		return (true);
	} else return (false);
}

//------------------------------------------------------------------------------
pascal void TIconView::DrawBit (Point theBit,
	Boolean turnItOn) {
//------------------------------------------------------------------------------
	Rect			bitRect;
	
	bitRect.top = theBit.v * fMagnification + kBorder;
	bitRect.left = theBit.h * fMagnification + kBorder;
	bitRect.bottom = (theBit.v + 1) * fMagnification + kBorder;
	bitRect.right = (theBit.h + 1) * fMagnification + kBorder;
	FillRect (&bitRect, turnItOn ? qd.black : qd.white);
}

#if qInspector
//------------------------------------------------------------------------------
pascal void TIconView::Fields (FieldProcPtr DTF, void *link) {
//------------------------------------------------------------------------------
	(*DTF) ("\pTIconView", nil, bClass, link);
	(*DTF) ("\pfIconDocument", (Ptr) &fIconDocument,
		bObject, link);
	(*DTF) ("\pfMagnification", (Ptr) &fMagnification,
		bInteger, link);
	inherited::Fields (DTF, link);
}
#endif

//------------------------------------------------------------------------------
pascal void TDrawCommand::IDrawCommand (
	TIconView *itsIconView) {
//------------------------------------------------------------------------------
	ICommand (cDraw, itsIconView->fIconDocument,
		itsIconView, itsIconView->GetScroller (true));
	fIconDocument = itsIconView->fIconDocument;
	fIconView = itsIconView;
	fIconBitMap = fIconDocument->fIconBitMap;
	fConstrainsMouse = true;
	fCanUndo = false;
}

//------------------------------------------------------------------------------
pascal void TDrawCommand::TrackConstrain (
	VPoint *anchorPoint, VPoint *prevPoint,
	VPoint *nextPoint) {
//------------------------------------------------------------------------------
	nextPoint->h = Max (kBorder, Min (nextPoint->h,
		fIconView->fSize.h - kBorder - 1));
	nextPoint->v = Max (kBorder, Min (nextPoint->v,
		fIconView->fSize.v - kBorder - 1));
}

//------------------------------------------------------------------------------
pascal void TDrawCommand::TrackFeedback (
	VPoint *anchorPoint, VPoint *nextPoint,
	Boolean turnItOn, Boolean mouseMoved) {
//------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------
pascal TCommand *TDrawCommand::TrackMouse (
	TrackPhase aTrackPhase, VPoint *anchorPoint,
	VPoint *prevPoint, VPoint *nextPoint,
	Boolean mouseMoved) {
//------------------------------------------------------------------------------
	Point			mousePoint;
	Point			iconBit;
	Boolean			notUsed;
	
	if (mouseMoved) {
		mousePoint = fIconView->ViewToQDPt (nextPoint);
		notUsed = fIconView->PointToBit (mousePoint,
			&iconBit);
		if (aTrackPhase == trackPress) {
			fTurnBitsOn = !fIconBitMap->GetBit (iconBit);
		}
		fIconBitMap->SetBit (iconBit, fTurnBitsOn);
		fIconView->DrawBit (iconBit, fTurnBitsOn);
	}
	return (this);
}

#if qInspector
//------------------------------------------------------------------------------
pascal void TDrawCommand::Fields (FieldProcPtr DTF,
	void *link) {
//------------------------------------------------------------------------------
	(*DTF) ("\pTDrawCommand", nil, bClass, link);
	(*DTF) ("\pfIconDocument", (Ptr) &fIconDocument,
		bObject, link);
	(*DTF) ("\pfIconView", (Ptr) &fIconView, bObject, link);
	(*DTF) ("\pfIconBitMap", (Ptr) &fIconBitMap, bObject,
		link);
	(*DTF) ("\pfTurnBitsOn", (Ptr) &fTurnBitsOn, bBoolean,
		link);
	inherited::Fields (DTF, link);
}
#endif

//------------------------------------------------------------------------------
pascal void TInvertCommand::IInvertCommand (
	TIconDocument *itsIconDocument) {
//------------------------------------------------------------------------------
	ICommand (cInvert, itsIconDocument, nil, nil);
	fIconDocument = itsIconDocument;
}

//------------------------------------------------------------------------------
pascal void TInvertCommand::DoIt (void) {
//------------------------------------------------------------------------------
	fIconDocument->InvertIcon ();
}

//------------------------------------------------------------------------------
pascal void TInvertCommand::UndoIt (void) {
//------------------------------------------------------------------------------
	fIconDocument->InvertIcon ();
}

//------------------------------------------------------------------------------
pascal void TInvertCommand::RedoIt (void) {
//------------------------------------------------------------------------------
	fIconDocument->InvertIcon ();
}

#if qInspector
//------------------------------------------------------------------------------
pascal void TInvertCommand::Fields (FieldProcPtr DTF,
	void *link) {
//------------------------------------------------------------------------------
	(*DTF) ("\pTInvertCommand", nil, bClass, link);
	(*DTF) ("\pfIconDocument", (Ptr) &fIconDocument,
		bObject, link);
	inherited::Fields (DTF, link);
}
#endif

//------------------------------------------------------------------------------
pascal void TIconDocument::IIconDocument (void) {
//------------------------------------------------------------------------------
	IDocument (kFileType, kSignature, kUsesDataFork,
		!kUsesRsrcFork, !kDataOpen, !kRsrcOpen);
	
	fIconBitMap = new TIconBitMap;
	FailNIL (fIconBitMap);
	fIconBitMap->IIconBitMap ();
}

//------------------------------------------------------------------------------
pascal void TIconDocument::DoInitialState (void) {
//------------------------------------------------------------------------------
	Handle			seedIcon;
	
	seedIcon = GetIcon (kSeedIconID);
	FailNILResource (seedIcon);
	fIconBitMap->SetIconBitMap (seedIcon);
}

//------------------------------------------------------------------------------
pascal void TIconDocument::Free (void) {
//------------------------------------------------------------------------------
	FreeIfObject (fIconBitMap);
	TDocument::Free ();
}

//------------------------------------------------------------------------------
pascal void TIconDocument::DoMakeViews (
	Boolean forPrinting) {
//------------------------------------------------------------------------------
	TWindow		*aWindow;
	
	aWindow = NewTemplateWindow (kIconWindowID, this);
}

//------------------------------------------------------------------------------
pascal void TIconDocument::DoSetupMenus (void) {
//------------------------------------------------------------------------------
	inherited::DoSetupMenus ();
	Enable (cInvert, true);
}

//------------------------------------------------------------------------------
pascal TCommand *TIconDocument::DoMenuCommand (
	CmdNumber whichCmd) {
//------------------------------------------------------------------------------
	TInvertCommand	*theCommand;
	
	switch (whichCmd) {
		case cInvert:
			theCommand = new TInvertCommand;
			FailNIL (theCommand);
			theCommand->IInvertCommand (this);
			return (theCommand);
		default:
			return (inherited::DoMenuCommand (
				whichCmd));
	}
}

//------------------------------------------------------------------------------
pascal void RedrawView (TView *aView, void *link) {
//------------------------------------------------------------------------------
	aView->ForceRedraw ();
}

//------------------------------------------------------------------------------
pascal void TIconDocument::InvertIcon (void) {
//------------------------------------------------------------------------------
	short			notUsed;
	
	fIconBitMap->Invert ();
	ForAllViewsDo (RedrawView, &notUsed);
}

#if qInspector
//------------------------------------------------------------------------------
pascal void TIconDocument::Fields (FieldProcPtr DTF,
	void *link) {
//------------------------------------------------------------------------------
	(*DTF) ("\pTIconDocument", nil, bClass, link);
	(*DTF) ("\pfIconBitMap", (Ptr) &fIconBitMap, bObject,
		link);
	inherited::Fields (DTF, link);
}
#endif

//------------------------------------------------------------------------------
pascal void TIconApplication::IIconApplication (
	OSType iconFileType) {
//------------------------------------------------------------------------------
	IApplication (iconFileType);
	
	if (gDeadStripSuppression) {
		TIconView *anIconView = new TIconView;
	}
}

//------------------------------------------------------------------------------
pascal TDocument *TIconApplication::DoMakeDocument (
	CmdNumber itsCmdNumber) {
//------------------------------------------------------------------------------
	TIconDocument	*anIconDocument;
	
	anIconDocument = new TIconDocument;
	FailNIL (anIconDocument);
	anIconDocument->IIconDocument ();
	return (anIconDocument);
}

