{ TCLResources		1991, Joseph R. L. Simkins }

{ USE:  Interface for TCL Resources Classes }
{ include a 'uses' clause listing TCLResources. }

{ DESCRIPTION:  Provides the data type CRes, }
{ which is an object class. CRes has instance }
{ variables of the resource name, a string, and }
{ its ID number, an integer, the resource Type, a }
{ long int interperted as four characters, and }
{ the handle for the resource. }

unit TCLResources;    			{ File is TCLResources.p }

interface

	uses
		ObjIntf;						{ Defines TObject }

{ Here we declare the CRes class. This class }
{ does not support manipulations on partial }
{ resources. (IM6) }

	type
		CRes = object(TObject)		{ Note heritage of this class }

		{ Instance variables }
				fHandle: Handle;			{ Res's Handle }
				fType: ResType;			{ Res's Type }
				fIDNum: Integer;			{ Res's ID number }
				fName: Str255;			{ Res's name }

		{ Methods }
				procedure CRes.IRes;
		  { Initializes a CRes object with empty data }
				function CRes.GetName: Str255;
		  { Returns the current name of a Res object }
		  {  Note: this method only acceses the object }
				{        data , not the actual resource }
				function CRes.GetIDNum: Integer;
		  { Returns the current ID number of a Res object }
		  {  Note: this method only acceses the object }
				{        data , not the actual resource }
				function CRes.GetType: ResType;
		  { Returns the current type of a Res object }
		  {  Note: this method only acceses the object }
				{        data , not the actual resource }
				function CRes.GetHandle: Handle;
		  { Returns the current handle of a Res object }
		  {  Note: this method only acceses the object }
				{        data , not the actual resource }
				procedure CRes.SetName (n: Str255);
		  { Set the current name of a Res object }
		  {  Note: this method only acceses the object }
				{        data , not the actual resource }
				procedure CRes.SetIDNum (id: Integer);
		  { Set the current ID number of a Res object }
		  {  Note: this method only acceses the object }
				{        data , not the actual resource }
				procedure CRes.SetType (t: ResType);
		  { Set the current type of a Res object }
		  {  Note: this method only acceses the object }
				{        data , not the actual resource }
				procedure CRes.SetHandle (h: Handle);
		  { Set the current handle of a Res object, }
				{		useful with AddResource }
		  {  Note: this method only acceses the object }
				{        data , not the actual resource }
				procedure CRes.SetResInfo;
		  { Sets the resource ID, name and type pointed }
				{		to by the handle of a Res object }
				procedure CRes.SetResInfoRetainName;
		  { Sets the resource ID and type pointed to by }
				{		the handle of a Res object the name is }
				{		unchanged }
				procedure CRes.SetResAttrs (a: Integer);
		  { Sets the resource attributes pointed to by }
				{		the handle of a Res object }
				procedure CRes.GetIndResource (i: Integer);
		  { Loads the indexed resource specified by the }
				{		type of a Res object }
				procedure CRes.Get1IndResource (i: Integer);
		  { Loads the indexed resource specified by the }
				{		type of a Res object from current resource file }
				procedure CRes.GetResource;
		  { Loads the resource specified by the type and }
				{		ID of a Res object }
				procedure CRes.Get1Resource;
		  { Loads the resource specified by the type and }
				{		ID of a Res object from current resource file }
				procedure CRes.GetNamedResource;
		  { Loads the resource specified by the type and }
				{		name of a Res object }
				procedure CRes.Get1NamedResource;
		  { Loads the resource specified by the type and }
				{		name of a Res object current resource file }
				procedure CRes.GetResInfo;
		  { Gets the resource ID, name and type pointed to }
				{		by the handle of a Res object }
				procedure CRes.LoadResource;
		  { Loads the resource pointed to by the handle of }
				{		a Res object into memory }
				procedure CRes.ReleaseResource;
		  { Releases the memory from the resource pointed }
				{		to by the handle of a Res object }
				procedure CRes.DetachResource;
		  { Removes the resource pointed to by the handle }
				{		of a Res object from resource map }
				procedure CRes.ChangedResource;
		  { Updates the resource pointed to by the handle }
				{		of a Res object on disk }
				procedure CRes.AddResource;
		  { Add the resource pointed to by the handle of a }
				{		Res object to resource map }
				procedure CRes.RmveResource;
		  { Remove the resource pointed to by the handle of }
				{		a Res object from the resource map }
				procedure CRes.WriteResource;
		  { Writes the resource pointed to by the handle of }
				{		a Res object to disk }
				function CRes.HomeResFile: Integer;
		  { Returns the reference number for the resource }
				{		pointed to by the handle of a Res object }
				function CRes.GetResAttrs: Integer;
		  { Returns the resource attributes pointed to by }
				{		the handle of a Res object }
				function CRes.SizeResource: Longint;
		  { Returns the size pointed to by the handle of }
				{		a Res object }
				function CRes.MaxSizeResrc: Longint;
		  { Returns the maximum (in memory) size pointed }
				{		to by the handle of a Res object }
				function CRes.ResError: Integer;
		  { Returns the error from the last operation on }
				{		a Res object }

				procedure CRes.Free;
				Override;
		  { Use local version for debugging possible TP 4.0 problem }

			end; { Class CRes }

{ DESCRIPTION:  Provides the data type CResFile, }
{ which is an object class.  CResFile has instance }
{ variables of the file name, a string, and its }
{ reference number, an integer. }

{ Here we declare the CResFile class.   This class }
{ does not support manipulations on partial }
{ resources. (IM6) }

	type
		CResFile = object(TObject)				{ Note heritage of this class }

		{ Instance variables }
				fFileName: Str255;					{ ResFile's name }
				fRefNum: Integer;					{ ResFile's reference number }

		{ Methods }
				procedure CResFile.IResFile (n: Str255);
		  { Initializes a CResFile object with a Name }
				function CResFile.GetName: Str255;
		  { Returns the current name of a ResFile object }
				function CResFile.GetRefNum: Integer;
		  { Returns the current reference number of a ResFile object }
				procedure CResFile.SetRefNum (n: Integer);
		  { Set the current resource file ref number for a ResFile object }
				procedure CResFile.CreateResFile;
		  { Creates a the resource fork for a ResFile object }
				procedure CResFile.UseResFile;
		  { Set the current resource file to a ResFile object }
				procedure CResFile.UpdateResFile;
		  { Update a ResFile object without closing it }
				procedure CResFile.SetResFileAttrs (a: Integer);
		  { Set the resource file attributes for a ResFile object }
				procedure CResFile.CloseResFile;
		  { Closes a ResFile object }
				function CResFile.OpenResFile: Integer;
		  { Returns the results of opening a ResFile object }
				function CResFile.OpenRFPerm (v: Integer;
											p: Byte): Integer;
		  { Returns the results of opening a ResFile object }
				function CResFile.GetResFileAttrs: Integer;
		  { Returns the resource file attributes of a ResFile object }
				function CResFile.ResError: Integer;
		  { Returns the error from the last operation on a ResFile object }

			end; { Class CResFile }

{ DESCRIPTION:  Provides the data type CAllRes, }
{ which is an object class.  CAllRes has no instance }
{ variables. All apparant variables are returns }
{ from traps. }

{ Here we declare the CAllRes class.   This class }
{ does not support manipulations on partial }
{ resources. (IM6) }

	type
		CAllRes = object(TObject)				{ Note heritage of this class }

		{ Instance variables - none }

		{ Methods }
				procedure CAllRes.IAllRes;
		  { Initializes a CAllRes object (does noting, }
				{  provided for constinancy) }
				function CAllRes.CurResFile: Integer;
		  { Returns the reference number of current }
				{  resource file }
				function CAllRes.CountTypes: Integer;
		  { Returns the number of types from all open }
				{  resource files }
				function CAllRes.Count1Types: Integer;
		  { Returns the number of types from the current }
				{  resource file }
				function CAllRes.GetIndType (i: Integer): ResType;
		  { Returns the type indexed by i from all open }
				{  resource files }
				function CAllRes.Get1IndType (i: Integer): ResType;
		  { Returns the type indexed by i from the }
				{  current resource file }
				function CAllRes.CountResources (r: ResType): Integer;
		  { Counts the number of resources of the type in }
				{  all open resource files }
				function CAllRes.Count1Resources (r: ResType): Integer;
		  { Counts the number of resources of the type in }
				{  the current resource file }
				function CAllRes.UniqueID (r: ResType): Integer;
		  { Returns the type indexed by i from all open }
				{  resource files }
				function CAllRes.Unique1ID (r: ResType): Integer;
		  { Returns the type indexed by i from the current }
				{  resource file }
				procedure CAllRes.SetResLoad (l: Boolean);
		  { Controls if resources are to be loaded }
		  {  This is a good routine to stay away from }
				{  unless realy needed }

			end; { Class CAllRes }

implementation

{ Here we implement CRes's method bodies. }
{ Note the trap definition to avoid the name }
{		collision with the traps. }

	procedure CRes.IRes;
	{ Assign parameters to instance variables }
	{ Note the name IRes instead of Init }
	begin
		self.fName := '';
		self.fIDNum := 0;
		self.fType := ResType(0);
		self.fHandle := nil;
	end; { CRes.IRes }

	function CRes.GetName: Str255;
	{ return current name of the CRes object }
	begin
		GetName := self.fName;
	end; { CRes.GetName }

	function CRes.GetIDNum: Integer;
	{ return current ID number of the CRes object }
	begin
		GetIDNum := self.fIDNum;
	end; { CRes.GetIDNum }

	function CRes.GetType: ResType;
	{ return the current type of the CRes object }
	begin
		GetType := self.fType;
	end; { CRes.GetType }

	function CRes.GetHandle: Handle;
	{ return the current handle of the Tres object }
	begin
		GetHandle := self.fHandle;
	end; { CRes.GetHandle }

	procedure CRes.SetName (n: Str255);
	{ set the name of the CRes object }
	begin
		self.fName := n;
	end; { CRes.GetName }

	procedure CRes.SetIDNum (id: Integer);
	{ set the ID number of the CRes object }
	begin
		self.fIDNum := id;
	end; { CRes.GetIDNum }

	procedure CRes.SetType (t: ResType);
	{ set the current type of the CRes object }
	begin
		self.fType := t;
	end; { CRes.GetType }

	procedure CRes.SetHandle (h: Handle);
	{ set the current handle of the Tres object }
	begin
		self.fHandle := h;
	end; { CRes.GetHandle }

	procedure CRes.SetResInfo;
	{ sets ID and name of a resource }
		procedure TrapSetResInfo (h: Handle;
										id: Integer;
										n: Str255);
		inline
			$A9A9;
	begin
		TrapSetResInfo(self.fHandle, self.fIDNum, self.fName);
	end; { CRes.SetResInfo }

	procedure CRes.SetResInfoRetainName;
	{ sets ID and name of a resource }
		procedure TrapSetResInfo (h: Handle;
										id: Integer;
										n: Longint);
		inline
			$A9A9;
	begin
		TrapSetResInfo(self.fHandle, self.fIDNum, 0);
	end; { CRes.SetResInfoRetainName }

	procedure CRes.SetResAttrs (a: Integer);
	{ sets attributes of a resource }
		procedure TrapSetResAttrs (h: Handle;
										a: Integer);
		inline
			$A9A7;
	begin
		TrapSetResAttrs(self.fHandle, a);
	end; { CRes.SetResAttrs }

	procedure CRes.GetIndResource (i: Integer);
	{ load resource specified by type and index }
		function TrapGetIndResource (t: ResType;
										i: Integer): Handle;
		inline
			$A99D;
	begin
		self.fHandle := TrapGetIndResource(self.fType, i);
	end; { CRes.GetIndResource }

	procedure CRes.Get1IndResource (i: Integer);
	{ load resource specified by type and index }
	{		from current resource file }
		function TrapGet1IndResource (t: ResType;
										i: Integer): Handle;
		inline
			$A80E;
	begin
		self.fHandle := TrapGet1IndResource(self.fType, i);
	end; { CRes.Get1IndResource }

	procedure CRes.GetResource;
	{ load resource specified by type and ID }
		function TrapGetResource (t: ResType;
										id: Integer): Handle;
		inline
			$A9A0;
	begin
		self.fHandle := TrapGetResource(self.fType, self.fIDNum);
	end; { CRes.GetResource }

	procedure CRes.Get1Resource;
	{ load resource specified by type and ID }
		function TrapGet1Resource (t: ResType;
										id: Integer): Handle;
		inline
			$A81F;
	begin
		self.fHandle := TrapGet1Resource(self.fType, self.fIDNum);
	end; { CRes.Get1Resource }

	procedure CRes.GetNamedResource;
		function TrapGetNamedResource (t: ResType;
										n: Str255): Handle;
		inline
			$A9A1;
	begin
		self.fHandle := TrapGetNamedResource(self.fType, self.fName);
	end; { CRes.GetNamedResource }

	procedure CRes.Get1NamedResource;
	{ load resource specified by type and name }
	{		from current resource file }
		function TrapGet1NamedResource (t: ResType;
										n: Str255): Handle;
		inline
			$A820;
	begin
		self.fHandle := TrapGet1NamedResource(self.fType, self.fName);
	end; { CRes.Get1NamedResource }

	procedure CRes.GetResInfo;
	{ gets name, ID, and type of resource }
		procedure TrapGetResInfo (h: Handle;
										var id: Integer;
										var t: ResType;
										var n: Str255);
		inline
			$A9A8;
	begin
		TrapGetResInfo(self.fHandle, self.fIDNum, self.fType, self.fName);
	end; { CRes.GetResInfo }

	procedure CRes.LoadResource;
	{ load resource into memory }
		procedure TrapLoadResource (h: Handle);
		inline
			$A9A2;
	begin
		TrapLoadResource(self.fHandle);
	end; { CRes.LoadResource }

	procedure CRes.ReleaseResource;
	{ release memory used by resource }
		procedure TrapReleaseResource (h: Handle);
		inline
			$A9A3;
	begin
		TrapReleaseResource(self.fHandle);
	end; { CRes.ReleaseResource }

	procedure CRes.DetachResource;
	{ detach the resource from the resource list }
		procedure TrapDetachResource (h: Handle);
		inline
			$A992;
	begin
		TrapDetachResource(self.fHandle);
	end; { CRes.DetachResource }

	procedure CRes.ChangedResource;
	{ mark the resource as changed to force update }
		procedure TrapChangedResource (h: Handle);
		inline
			$A9AA;
	begin
		TrapChangedResource(self.fHandle);
	end; { CRes.ChangedResource }

	procedure CRes.AddResource;
	{ add the data pointed to by the handle as a resource }
		procedure TrapAddResource (h: Handle;
										t: ResType;
										id: Integer;
										n: Str255);
		inline
			$A9AB;
	begin
		TrapAddResource(self.fHandle, self.fType, self.fIDNum, self.fName);
	end; { CRes.AddResource }

	procedure CRes.RmveResource;
	{ remove the resource from the resource file }
		procedure TrapRmveResource (h: Handle);
		inline
			$A9AD;
	begin
		TrapRmveResource(self.fHandle);
	end; { CRes.RmveResource }

	procedure CRes.WriteResource;
	{ update the resource in the resource file }
		procedure TrapWriteResource (h: Handle);
		inline
			$A9B0;
	begin
		TrapWriteResource(self.fHandle);
	end; { CRes.WriteResource }

	function CRes.HomeResFile: Integer;
	{ return the file reference number for the CRes object }
		function TrapHomeResFile (h: Handle): Integer;
		inline
			$A9A4;
	begin
		HomeResFile := TrapHomeResFile(self.fHandle);
	end; { CRes.HomeResFile }

	function CRes.GetResAttrs: Integer;
	{ return the resource attributes for the CRes object }
		function TrapGetResAttrs (h: Handle): Integer;
		inline
			$A9A6;
	begin
		GetResAttrs := TrapGetResAttrs(self.fHandle);
	end; { CRes.GetResAttrs }

	function CRes.SizeResource: Longint;
	{ return the size in bytes of the resource for the }
	{		CRes object }
		function TrapSizeResource (h: Handle): Longint;
		inline
			$A9A5;
	begin
		SizeResource := TrapSizeResource(self.fHandle);
	end; { CRes.SizeResource }

	function CRes.MaxSizeResrc: Longint;
	{ return the maximum size in bytes of the resource }
	{		for the CRes object }
		function TrapMaxSizeResrc (h: Handle): Longint;
		inline
			$A821;
	begin
		MaxSizeResrc := TrapMaxSizeResrc(self.fHandle);
	end; { CRes.MaxSizeResrc }

	function CRes.ResError: Integer;
	{ return error from last resource operation }
		function TrapResError: Integer;
		inline
			$A9AF;
	begin
		ResError := TrapResError;
	end; { CRes.ResError }

	procedure CRes.Free;
		Override;
	begin
		if (self.fHandle <> nil) then
			DisposHandle(Handle(self.fHandle));
		inherited Free;					{ dispose Cres data }
	end;

{ Here we implement CResFile's method bodies. }
{ Note the trap definition to avoid the name }
{		collision with the traps. }

	procedure CResFile.IResFile (n: Str255);
	{ Assign parameters to instance variables }
	{ Note the name IResFile instead of Init }
	begin
		self.ffileName := n;
		self.frefNum := -1;
	end; { CResFile.IResFile }

	function CResFile.GetName: Str255;
	{ return current file name of the CResFile object }
	begin
		GetName := self.ffileName;
	end; { CResFile.GetName }

	function CResFile.GetRefNum: Integer;
	{ return current refnum of the CResFile object }
	begin
		GetRefNum := self.frefNum;
	end; { CResFile.GetRefNum }

	procedure CResFile.SetRefNum (n: Integer);
		  { Set the resource file ref number of the CResFile object }
	begin
		self.frefNum := n;
	end; { CResFile.SetRefNum }

	procedure CResFile.CreateResFile;
	{ Create Resource Fork of file }
		procedure TrapCreateResFile (fileName: Str255);
		inline
			$A9B1;
	begin
		TrapCreateResFile(self.ffileName);
	end; { CResFile.CreateResFile }

	procedure CResFile.UseResFile;
	{ Use self as current resource file }
		procedure TrapUseResFile (refNum: Integer);
		inline
			$A998;
	begin
		TrapUseResFile(self.frefNum);
	end; { CResFile.UseResFile }

	procedure CResFile.UpdateResFile;
	{ Update the resource file on disk without closing }
	{  the file }
		procedure TrapUpdateResFile (refNum: Integer);
		inline
			$A999;
	begin
		TrapUpdateResFile(self.frefNum);
	end; { CResFile.UpdateResFile }

	procedure CResFile.SetResFileAttrs (a: Integer);
	{ Update the resource file resource file attributes }
		procedure TrapSetResFileAttrs (refNum, attrs: Integer);
		inline
			$A9F7;
	begin
		TrapSetResFileAttrs(self.frefNum, a);
	end; { CResFile.SetResFileAttrs }

	procedure CResFile.CloseResFile;
	{ Update then close the resource file on disk }
		procedure TrapCloseResFile (refNum: Integer);
		inline
			$A99A;
	begin
		TrapCloseResFile(self.frefNum);
	end; { CResFile.CloseResFile }

	function CResFile.OpenResFile: Integer;
	{ open the resource file and return the results }
		function TrapOpenResFile (fileName: Str255): Integer;
		inline
			$A997;
	begin
		self.frefNum := TrapOpenResFile(self.ffileName);
		OpenResFile := self.frefNum;
	end; { CResFile.OpenResFile }

	function CResFile.OpenRFPerm (v: Integer;
									p: Byte): Integer;
	{ open the resource file. specifying permission }
	{  and return the results }
		function TrapOpenRFPerm (fileName: Str255;
										vRefNum: Integer;
										permission: Byte): Integer;
		inline
			$A9C4;
	begin
		self.frefNum := TrapOpenRFPerm(self.ffileName, v, p);
		OpenRFPerm := self.frefNum;
	end; { CResFile.OpenResFile }

	function CResFile.GetResFileAttrs: Integer;
	{ return resource file attributes of the CResFile object }
		function TrapGetResFileAttrs (refNum: Integer): Integer;
		inline
			$A9A6;
	begin
		GetResFileAttrs := TrapGetResFileAttrs(self.frefNum);
	end; { CResFile.GetResFileAttrs }

	function CResFile.ResError: Integer;
	{ return error from last resource file operation }
		function TrapResError: Integer;
		inline
			$A9AF;
	begin
		ResError := TrapResError;
	end; { CResFile.ResError }

{ Here we implement CAllRes's method bodies. }
{ Note the trap definition to avoid the name }
{		collision with the traps. }

	procedure CAllRes.IAllRes;
	{ Note the name IAllRes instead of Init }
	begin
	end; { CAllRes.IAllRes }

	function CAllRes.CurResFile: Integer;
	{ return reference number for current res file }
		function TrapCurResFile: Integer;
		inline
			$A994;
	begin
		CurResFile := TrapCurResFile;
	end; { CAllRes.CurResFile }

	function CAllRes.CountTypes: Integer;
	{ return number of resource types for all }
	{  res files }
		function TrapCountTypes: Integer;
		inline
			$A99E;
	begin
		CountTypes := TrapCountTypes;
	end; { CAllRes.CountTypes }

	function CAllRes.Count1Types: Integer;
	{ return number of resource types for all }
	{  res files }
		function TrapCount1Types: Integer;
		inline
			$A81C;
	begin
		Count1Types := TrapCount1Types;
	end; { CAllRes.Count1Types }

	function CAllRes.GetIndType (i: Integer): ResType;
	{ return Type selected by index }
		procedure TrapGetIndType (var r: ResType;
										i: Integer);
		inline
			$A99F;
		var
			tempType: ResType;
	begin
		TrapGetIndType(tempType, i);
		GetIndType := tempType;
	end; { CAllRes.GetIndType }

	function CAllRes.Get1IndType (i: Integer): ResType;
	{ return Type selected by index from current }
	{  resource file }
		procedure TrapGet1IndType (var r: ResType;
										i: Integer);
		inline
			$A80F;
		var
			tempType: ResType;
	begin
		TrapGet1IndType(tempType, i);
		Get1IndType := tempType;
	end; { CAllRes.Get1IndType }

	function CAllRes.CountResources (r: ResType): Integer;
	{ return number of resources for type }
		function TrapCountResources (r: ResType): Integer;
		inline
			$A99C;
	begin
		CountResources := TrapCountResources(r);
	end; { CAllRes.CountResources }

	function CAllRes.Count1Resources (r: ResType): Integer;
	{ return number of resources for type from }
	{  current reource file }
		function TrapCount1Resources (r: ResType): Integer;
		inline
			$A80D;
	begin
		Count1Resources := TrapCount1Resources(r);
	end; { CAllRes.Count1Resources }

	function CAllRes.UniqueID (r: ResType): Integer;
	{ return unused ID for type }
		function TrapUniqueID (r: ResType): Integer;
		inline
			$A9C1;
	begin
		UniqueID := TrapUniqueID(r);
	end; { CAllRes.UniqueID }

	function CAllRes.Unique1ID (r: ResType): Integer;
	{ return unused ID for type from current }
	{  resource file }
		function TrapUnique1ID (r: ResType): Integer;
		inline
			$A810;
	begin
		Unique1ID := TrapUnique1ID(r);
	end; { CAllRes.Unique1ID }

	procedure CAllRes.SetResLoad (l: Boolean);
	{ Set the load flag }
		procedure TrapSetResLoad (l: Boolean);
		inline
			$A99B;
	begin
		TrapSetResLoad(l);
	end; { CAllRes.SetResLoad }

end.  { Unit TCLResources }