{ UFlags			1991, Joseph R. L. Simkins }

{ DESCRIPTION:  Provides the data type UFlags, }
{ which is an object class.  UFlags has instance  }
{ variables of the show icon flag, startup flag }
{ and the desk flag. }

{ USE:  Add this file to your THINK Pascal project }
{ below TCLResources.p and above your main program file }
{ or unit which uses this unit.  In your using code, }
{ include a 'uses' clause listing UFlags. }

unit UFlags;    		{ File is UFlags.p }

interface

	uses
		ObjIntf, TCLResources;

{ Here we declare the CResFile class; its method }
{  bodies are postponed until the implementation }
{  part below. This class does not support }
{  manipulations on partial resources. (IM6) }

	type
		cFlags = object(TObject)

		{ Instance variables }
				fFlagRes: CRes;					{ cFlags's resources }
				fFlagResFile: CResFile;	{ cFlags's resources }

			{ Methods }
				procedure cFlags.IFlags;
		  { Initializes a cFlags object }
				procedure cFlags.TSetShowFlag (f: Boolean);
		  { Sets the display icon on startup flag }
				procedure cFlags.TSetStartupFlag (f: Boolean);
		  { Sets the randomize StartupScreen flag }
				procedure cFlags.TSetDeskFlag (f: Boolean);
		  { Sets the randomize DeskPicture flag }
				function cFlags.TShowFlag: Boolean;
		  { Returns the display icon on startup flag }
				function cFlags.TStartupFlag: Boolean;
		  { Returns the randomize StartupScreen flag }
				function cFlags.TDeskFlag: Boolean;
		  { Returns the randomize DeskPicture flag }

				procedure cFlags.Free;
				Override;
			{ dispose of handles }

	 	  { Note:  These method names include the optional class name prefix, }
	   	{  as in 'procedure cFlags.IFlags;' }

			end; { Class cFlags }

implementation

	type
		flagsIndex = (ShowIndex, StartupIndex, DeskIndex);
		flagsString = packed array[flagsIndex] of byte;
		flagsStringPtr = ^flagsString;
		flagsStringHdl = ^flagsStringPtr;

{ Here we implement cFlags's method bodies. }

	procedure cFlags.IFlags;
	{ Get the flags from the resource }
	{ Note the name IFlags instead of Init }
	begin
		{ set up resource with control flags, one per byte }
		new(self.fFlagRes);
		self.fFlagRes.SetType('JRS0');
		self.fFlagRes.SetIDNum(-4033);
		self.fFlagRes.GetResource;
		{ set up for update of ScreenPicker Resources }
		new(fFlagResFile);
		self.fFlagResFile.IResFile('');
		self.fFlagResFile.SetRefNum(self.fFlagRes.HomeResFile);
	end; { cFlags.IFlags }

	procedure cFlags.TSetShowFlag (f: Boolean);
		  { Sets the display icon on startup flag }
	begin
		{ update both copies of the flags in memory }
		flagsStringHdl(self.fFlagRes.GetHandle)^^[ShowIndex] := ord(f);
		self.fFlagRes.ChangedResource;
		self.fFlagResFile.UpdateResFile;
	end; { cFlags.TSetShowFlag }

	procedure cFlags.TSetStartupFlag (f: Boolean);
		  { Sets the randomize StartupScreen flag }
	begin
		flagsStringHdl(self.fFlagRes.GetHandle)^^[StartupIndex] := ord(f);
		self.fFlagRes.ChangedResource;
		self.fFlagResFile.UpdateResFile;
	end; { cFlags.TSetStartupFlag }

	procedure cFlags.TSetDeskFlag (f: Boolean);
		  { Sets the randomize DeskPicture flag }
	begin
		flagsStringHdl(self.fFlagRes.GetHandle)^^[DeskIndex] := ord(f);
		self.fFlagRes.ChangedResource;
		self.fFlagResFile.UpdateResFile;
	end; { cFlags.TSetDeskFlag }

	function cFlags.TShowFlag: Boolean;
		  { Returns the display icon on startup flag }
	begin
		TShowFlag := Boolean(flagsStringHdl(self.fFlagRes.GetHandle)^^[ShowIndex]);
	end; { cFlags.TShowFlag }

	function cFlags.TStartupFlag: Boolean;
		  { Returns the randomize StartupScreen flag }
	begin
		TStartupFlag := Boolean(flagsStringHdl(self.fFlagRes.GetHandle)^^[StartupIndex]);
	end; { cFlags.TStartupFlag }

	function cFlags.TDeskFlag: Boolean;
		  { Returns the randomize DeskPicture flag }
	begin
		TDeskFlag := Boolean(flagsStringHdl(self.fFlagRes.GetHandle)^^[DeskIndex]);
	end; { cFlags.TDeskFlag }

	procedure cFlags.Free;
		Override;
	begin
		fFlagRes.Free;						{ dispose of the handle to the flags resource }
		fFlagResFile.Free;				{ dispose of the ScreenPicker file reference number }
		inherited Free;					{ dispose flags data }
	end;

end.  { Unit UFlags }