{ UScreenSet		1991, Joseph R. L. Simkins }

{ DESCRIPTION:  Provides the data type UScreenSet, }
{ which is an object class.  UScreenSet inherits }
{ its variables from CResFile. }

{ USE:  Add this file to your THINK Pascal project }
{ below ObjIntf.p and above your main program file }
{ or unit which uses this unit.  In your using code, }
{ include a 'uses' clause listing UScreenSet. }

unit UScreenSet;    		{ File is UScreenSet.p }

interface

	uses
		ObjIntf, TCLResources;

{ Here we declare the CResFile class; its method }
{  bodies are postponed until the implementation }
{  part below. This class does not support }
{  manipulations on partial resources. (IM6) }

	type
		cScreenSet = object(CResFile)

			{ No New Instance variables }

			{ Methods }
				procedure cScreenSet.IScreenSet (n: Str255);
		  { Initializes a cScreenSet object with a Name }
				procedure cScreenSet.RandomizeScreenSet;
		  { Selects random PICT, ID = 0, for a ScreenSet object }

	 	  { Note:  These method names include the optional class name prefix, }
	   	{  as in 'procedure cScreenSet.IScreenSet;' }

	   	{ Note:  The class also inherits all of class CResFile's methods }
			end; { Class cScreenSet }

implementation

{ Here we implement cScreenSet's method bodies. }

	procedure cScreenSet.IScreenSet (n: Str255);
	{ Assign parameters to instance variables }
	{ Note the name IScreenSet instead of Init }
	begin
		self.IResFile(n);
	end; { cScreenSet.IScreenSet }

	procedure cScreenSet.RandomizeScreenSet;
		var
			oldPict, newPict: CRes;		{ ScreenSet's resources }
			pictCount: Integer;				{ ScreenSet's number of pictures }
			pictIndex: Integer;				{ ScreenSet's number of pictures }
			resources: CAllRes;				{ Global Resources Object }
	begin
		if self.OpenResFile <> -1 then begin
				{ set up for resource information }
				new(resources);
				resources.IAllRes;
				{ get number of PICT resources in file }
				pictCount := resources.Count1Resources('PICT');
				if pictCount > 1 then begin
						{ get information on active (ID = 0) PICT }
						resources.SetResLoad(False);
						new(oldPict);
						oldPict.SetType('PICT');
						oldPict.SetIDNum(0);
						oldPict.Get1Resource;
						{ continue only on no error }
						if oldPict.ResError = 0 then begin
								{ work on next active PICT }
								new(newPict);
								newPict.SetType('PICT');
								{ compute random index from tick count }
								pictIndex := abs(TickCount mod pictCount) + 1;
								newPict.Get1IndResource(pictIndex);
								{ continue only on no error }
								if newPict.ResError = 0 then begin
										newPict.GetResInfo;
										{ only update if different resource chosen }
										if newPict.GetIDNum <> 0 then begin
												{ set old ID = 0 to ID from new }
												oldPict.SetIDNum(newPict.GetIDNum);
												oldPict.SetResInfoRetainName;
												oldPict.ChangedResource;
												{ set new ID to 0 }
												newPict.SetIDNum(0);
												newPict.SetResInfoRetainName;
												newPict.ChangedResource;
											end;
									end;
								newPict.Free;
							end;
						oldPict.Free;
						resources.SetResLoad(True);
					end;
				resources.Free;
				self.CloseResFile;
			end;
	end; { cScreenSet.RandomizeScreenSet }

end.  { Unit UScreenSet }