// globals.c -- low level functions to manipulate globals pointer
// Copyright 1992, David T. Roach, All Rights Reserved

#include <Types.h>
#include <SetUpA4.h>
#include <OSUtils.h>
#include <Traps.h>

#ifndef 	__globals__
#include 	"globals.h"
#endif 		__globals__

void dummy(void)
{	
// let's not use a LINK/UNLK pair here

#pragma options(!force_frame)

//////////////////////////////////////////////////////////////////////////////////////////
// this dummy routine actually contains three 
// assembly language entry points
// one where the globals pointer is  stored
// and two to get or set the globals pointer
// this entry point at dummy() is unused
	
	asm {
// RTS fools Macsbug that this is end of routine
		RTS

// 8 char Macsbug name for the dummy routine
 		DC.B	0x80+'d','u','m','m','y',' ',' ',' '

//////////////////////////////////////////////////////////////////////////////////////////
// void  GlobPtr(void)
//
// the fake function where we store the pointer

	extern GlobPtr:
	
// our storage location
	@theGlobPtr				// force relative addressing
		DC.L	0

// RTS fools Macsbug that this is end of routine
		RTS					

// 8 char Macsbug name for the globals pointer
		DC.B	0x80+'G','l','o','b','P','t','r',' '		

//////////////////////////////////////////////////////////////////////////////////////////
// OSErr SetGlobPtr(Ptr globalsPtr)
//
// see prototype for arguments & return type
// routine to setup the global pointer
// called only once at init time
// modifies A0 and returns w/ OSErr in D0
//		
// unlike using SetUpA5, it shouldn't make any 
// difference if you're at interrupt level
// or application level when you call this 
// function to set up your global pointer
// as long as you're sure the handle or pointer 
// you're passing in is valid 
		
	extern SetGlobPtr: 		
		
// get globals ptr off stack

		MOVE.L	4(A7),D0		
		
// branch if not NIL, error if NIL
	
		BNE.S	@TestForOdd

// note: we can use RTS instead of return 
// because there is no LINK/UNLK pair
// normally ThinkC doesn't like this
			
		MOVE.L	#BadPointer,D0		
		RTS	

@TestForOdd

		BTST	#1,D0

// branch if even address, error if odd

		BEQ.S	@DoStripAddress

		MOVE.L	#BadPointer,D0		
		RTS	

@DoStripAddress
// we should perform a strip address to zero out 
// the most significant byte of our globalsPtr
// in a 24-bit addressing world, so that 
// we are not caught with our pants down 
// if we're called during an interrupt when
// a Nubus card's DRVR has switched the call 
// SwapMMUMode in order to 
// address more than 1 meg of Nubus space, .
//
// IM Vols V & IV both document StripAddress 
// incorrectly. See TechNote #213 for correct 
// documentation.
//
// _StripAddress was first implemented on MacII
// System 4.x. You can probably count on the 
// fact that it always exists, but those of us
// that are really safety conscious should 
// check it w/ TrapAvailable()

		DC.W	_StripAddress		
		
// save A0 on the stack so we don't trash it

		MOVE.L	A0,-(SP)			

// now get the address of our globals ptr storage
// (which is the address of the fake function)
// this forces ThinkC to use PC relative addressing 
// because GlobPtr is within the same C routine

		LEA		@theGlobPtr,A0			
		MOVE.L	D0,(A0)

// now restore A0

		MOVE.L	(SP)+,A0
				
// we only set 'GlobPtr' once during initialization,
// but Tech Note #261 (updated April 1992) 
// is very clear that you should flush the cache 
// anytime you change a value that's stored in 
// a code segment
// 
// safety note from TN261: Before you call any of 
// these [cache] routines, make sure that the
// _HWPriv ($A198) trap is implemented, or 
// your program will crash. _HWPriv
// is implemented in the Macintosh IIx ROMs 
// and later, as well as System 6.0.3 
// and later. The correct way to check for the 
// trap is using the TrapAvailable 
// function documented in Inside Macintosh 
// Volume VI (pages 3-7 to 3-9)

		_FlushInstructionCache
		
// return noErr
		MOVEQ	#0,D0				
		RTS
			
// 8 char Macsbug name format		
		DC.B	0x80+'S','e','t','G','l','o','b','P'		
		
//////////////////////////////////////////////////////////////////////////////////////////
// OSErr GetGlobPtr(Ptr * globalsPtr);
//
// routine to access the global pointer
// may be called at any time

	extern GetGlobPtr:
	
// get the address in which we want to
// return the globals pointer

		MOVE.L	4(A7),D0

// save A0 on the stack so we don't trash it

		MOVE.L	A0,-(SP)			
		MOVE.L	D1,-(SP)			

// now get the address of our globals ptr storage
// (which is the address of the fake function)
// this forces ThinkC to use PC relative addressing 
// because GlobPtr is within the same C routine

		LEA		@theGlobPtr,A0			
		MOVE.L	(A0),D1
		BNE.S	@TestEven

// restore all registers and return
// with BadPointer error in D0

		MOVE.L	(SP)+,D1		
		MOVE.L	(SP)+,A0	
		MOVE.L	#BadPointer,D0		
		RTS	
		
	@TestEven
	
		BTST	#0,D1
		BEQ.S	@DoGlobals

// restore all registers and return
// with BadPointer error in D0

		MOVE.L	(SP)+,D1		
		MOVE.L	(SP)+,A0		
		MOVE.L	#BadPointer,D0		
		RTS	

	@DoGlobals
	
		MOVEA.L	D0,A0
		MOVE.L	D1,(A0)
		
// restore registers

		MOVE.L	(SP)+,D1		
		MOVE.L	(SP)+,A0	
		
// return noErr

		MOVEQ	#0,D0
		RTS

// 8 char Macsbug name format		
		DC.B	0x80+'G','e','t','G','l','o','b','P'

	}	
}

