/*--------------------------------
#
#	DoIdle.c
#	Version 1.0
#
#	Copyright  1992 Tim Swihart
#	All rights reserved.
#
#	October 10, 1992 (v.1.0)
#
----------------------------------*/

#pragma load "AliasMeisterDump"

#pragma segment IdleStuff

// Function prototypes:
Boolean DoFindParentFSSpec(FSSpecPtr theFSSpecPtr);
void DoRenameMe(FSSpecPtr destFSSpecPtr);
void DoRenameButton(DialogPtr theDlogPtr, Str63 btnName);
void DoIdle(void);
Boolean DoGetNewDestination(FSSpecPtr destFSSpecPtr);
Boolean CustomFileOK(void);
static pascal short myDlgHook(short theItem, DialogPtr theDlogPtr, myGetHookRecPtr myHookDataPtr);
static pascal Boolean myFileFilter(CInfoPBPtr thePBPtr, myGetHookRecPtr myHookDataPtr);

// External function prototypes:
extern Boolean DoGetCurDestination(FSSpecPtr curDestFSSPecPtr);
extern void FlushAVolume(FSSpecPtr theFSSpecPtr);
extern OSErr TouchDirectory(short vRefNum, long dirID);

// External globals:
extern short	gMyResFileRefNum;
extern Boolean	gQuitting;
extern Boolean	gLetUserConfigureMe;


// The following routine is responsible for converting an incoming FSSpec into one
// the describes the parent folder holding the file described by the incoming FSSpec
Boolean DoFindParentFSSpec(FSSpecPtr theFSSpecPtr)
{
	
	CInfoPBRec	paramBlock;
	Boolean		didItWork = false;
	
	paramBlock.dirInfo.ioCompletion = nil;
	paramBlock.dirInfo.ioNamePtr = theFSSpecPtr->name;
	paramBlock.dirInfo.ioVRefNum = theFSSpecPtr->vRefNum;
	paramBlock.dirInfo.ioFDirIndex = -1; // find by ID
	paramBlock.dirInfo.ioDrDirID = theFSSpecPtr->parID;
	PBGetCatInfoSync(&paramBlock);
	if (paramBlock.dirInfo.ioResult == 0) {
		theFSSpecPtr->parID = paramBlock.dirInfo.ioDrParID; // grab real parent ID
		didItWork = true;
	}
	return(didItWork);
}


// The following routine is responsible for renaming the application to reflect its new
// destination after it's been configured.  The name should be "" plus the destination 
// name (truncate new name so it stays under 32 characters).  
void DoRenameMe(FSSpecPtr destFSSpecPtr)
{
	Str63					newName;
	ProcessInfoRec			myProcessInfo;
	ProcessSerialNumber		myPSNum;
	FSSpec					myFSSpec;
	
	myPSNum.highLongOfPSN = 0L;
	myPSNum.lowLongOfPSN = kCurrentProcess;
	
	// In order for an app to change it's name on the fly, it has to know who it is, 
	// where it lives, and which volume it lives on.  We can get all of this information
	// from the Process Manger by asking it who the current process is (we're the current
	// process at the time we're allowed to ask).  But first, we have to set up some fields
	myProcessInfo.processInfoLength = sizeof(ProcessInfoRec);
	myProcessInfo.processName = nil; // we get name from FSSpec
	myProcessInfo.processAppSpec = &myFSSpec;
	if (!GetProcessInformation(&myPSNum, &myProcessInfo)) {
		// We also need to munge the incoming name to tack the "" at the front of the
		// name.  Currently, destFSSpecPtr contains destination's file name, so move it 
		// into newName, but move it down one char to make room for ""
		BlockMove(destFSSpecPtr->name, newName + (char) 1, destFSSpecPtr->name[0] + 1);
		newName[1] = ''; // stuff in the "arrow" character
		newName[0] = destFSSpecPtr->name[0] + 1; // update length byte
		
		// make sure new file name isn't more than 31 characters long
		if (newName[0] > 31) { newName[0] = 31; }
		
		// rename the file
		FSpRename(myProcessInfo.processAppSpec, newName);
		
		// hide Finder glitch - renamed files on desktop don't show up with new name until
		// you've rebooted.  Old name is cached and you can't open the app until you've rebooted.
		TouchDirectory(myFSSpec.vRefNum, myFSSpec.parID);
		
		// flush changes to disk
		FlushAVolume(&myFSSpec);
	}
}


// The following routine renames the 'Use ""' button to the name given in btnName (where
// "" represents the name of the current folder being browsed). theDlogPtr refers to the 
// dialog in which the button to be renamed lives.
void DoRenameButton(DialogPtr theDlogPtr, Str63 btnName)
{
	ControlHandle	itemHandle;
	Rect			itemRect;
	short			itemType, howWide;
	Str63			rootName = "\pUse  "; // the static part of the button's name
	
	// get a handle on the button we want to rename
	GetDItem(theDlogPtr, kUseBtnID, &itemType, &((Handle)itemHandle), &itemRect);
	
	// figure out how much space their is for the folder's name
	howWide = itemRect.right - itemRect.left - (StringWidth(rootName));
	
	// make name fit in available space
	TruncString(howWide, btnName, smTruncEnd);

	// concat the folder's name right behind the first quote mark
	BlockMove(btnName + (char) 1, rootName + rootName[0] - 1, btnName[0]);
	
	// adjust the length byte to include the concat'd characters
	// "-1" takes out the white space which helped leave a little room around the ends
	rootName[0] = rootName[0] + btnName[0] - 1;
	
	// stuff a quote mark on the end of the new name
	rootName[rootName[0]] = '';
	
	// rename button and force it to be redrawn
	SetCTitle(itemHandle, rootName);
	ValidRect(&itemRect);
}

// The following routine is responsible for filtering out files (so only folders and 
// volumes appear in my CustomGetFile dialog).
static pascal Boolean myFileFilter(CInfoPBPtr thePBPtr, myGetHookRecPtr myHookDataPtr)
{
	return(!((thePBPtr->dirInfo.ioFlAttrib) & 8));
}


// The following routine is the hook function for our CustomGetFile dialog. It conveys the
// FSSpec for the current destination into the dialog and the new selection's FSSpec out of
// the dialog.
static pascal short myDlgHook(short theItem, DialogPtr theDlogPtr, myGetHookRecPtr myHookDataPtr)
{
	short			myVRefNum;
	short			returnCode = theItem;
	short			tempBtnHilite1, tempBtnHilite2, itemType;
	OSErr			myErr;
	long			myDirID;
	ControlHandle	itemHandle;
	Rect			itemRect;
	Str63			desktopName = "\pDesktop Folder";

	if (GetWRefCon((WindowPtr)theDlogPtr) == sfMainDialogRefCon) { // avoid nested dialogs
		switch(theItem) {
			case sfHookFirstCall: // initialize stuff here
				myHookDataPtr->fixBtnName = true; // button's name will be correct initially
				myHookDataPtr->theFSSpecIsValid = false; // in case user clicks Cancel button
				
				// To force StandardFile to use our default directory initially, move the default 
				// directory's FSSpec from the custom data block pointed to by myHookDataPtr into 
				// the reply record used by StandardFile (a pointer to that record is also part of 
				// the custom data block).  To complete the forced change to the default directory, 
				// return 'sfHookChangeSelection' as the result.
				BlockMove(&(myHookDataPtr->curDestFSSpecPtr->vRefNum), 
							&(myHookDataPtr->replyPtr->sfFile.vRefNum), sizeof(FSSpec));
				DoRenameButton(theDlogPtr, myHookDataPtr->replyPtr->sfFile.name);
				myHookDataPtr->replyPtr->sfFile.name[0] = ''; // name is invalid right now,so zap it
				returnCode = sfHookChangeSelection;
				break;
			
			case sfHookOpenFolder: // force rename of bottom button as folder is opened
				DoRenameButton(theDlogPtr, myHookDataPtr->replyPtr->sfFile.name);
				break;
		
			case sfHookGoToDesktop: // force rename of bottom button
				// the reply record does NOT indicate "Desktop Folder", so force that 
				// name into the bottom button
				DoRenameButton(theDlogPtr, desktopName);
				break;
		
			case kSelectBtnID:
				// Validate FSSpec for hilited folder & fake a hit on the Cancel button 
				// to dismiss dialog
				myHookDataPtr->theFSSpecIsValid = true;
				returnCode = sfItemCancelButton;
				break;
		
			case kUseBtnID:
				// Get FSSpec for current directory  reply record's FSSpec includes the ID for the
				// parent folder (the one we really want), so convert that parID into complete 
				// FSSpec.  This doesn't work for the case where the desktop is selected as the
				// "current" directory.  So, we have to special case this based on whether or not 
				// the Desktop button is dimmed.  If that button is dimmed, user wants the desktop.

				GetDItem(theDlogPtr, sfItemDesktopButton, &itemType, &((Handle)itemHandle), &itemRect);
				if ((**itemHandle).contrlHilite) {  // if current directory IS the desktop
					myErr = FindFolder(kOnSystemDisk, kDesktopFolderType, 
										kDontCreateFolder, &myVRefNum, &myDirID);
					if (!myErr) { // if desired folder exists, create FSSpec for it
						myErr = FSMakeFSSpec(myVRefNum, myDirID, nil, 
												&(myHookDataPtr->replyPtr->sfFile));
						if (!myErr) { myHookDataPtr->theFSSpecIsValid = true; } // let caller know we succeeded
					}
				} else { // current directory is NOT the desktop
					myHookDataPtr->theFSSpecIsValid = 
												DoFindParentFSSpec(&(myHookDataPtr->replyPtr->sfFile));
				}
				returnCode = sfItemCancelButton; // fake a hit on Cancel button to dismiss dialog
				break;

			case sfHookGoToNextDrive:
			case sfHookGoToPrevDrive:
				// For these two cases, we need to rename the "Use " button, but at this
				// exact moment in time, the incoming reply record still holds info for the
				// folder we WERE viewing (i.e.: we can't use the information in that record).
				// We could fumble around here and find out where we're supposed to be, but
				// if we just make a note of the fact that the reply record is junk right now, 
				// we can wait for the next available null event, by which time the reply
				// record will be up to date and we can easily rename the button.
				myHookDataPtr->fixBtnName = false;
				break;
				
			case sfItemVolumeUser:
			case sfHookGoToParent:
				// For these two cases, we need to rename the "Use " button. Unlike the prior
				// two cases, we can build off of the current reply record.
				// We have to special case the desktop folder
				if (myHookDataPtr->replyPtr->sfFile.parID == fsRtDirID) {  
					// Current directory IS the desktop, so set button's name to "Desktop Folder"
					DoRenameButton(theDlogPtr, desktopName);
					myHookDataPtr->fixBtnName = true; // btn name is OK now
				} else { // current directory is NOT the desktop
										DoFindParentFSSpec(&(myHookDataPtr->replyPtr->sfFile));
					myHookDataPtr->fixBtnName = 
										DoFindParentFSSpec(&(myHookDataPtr->replyPtr->sfFile));
					// rename the button
					DoRenameButton(theDlogPtr, myHookDataPtr->replyPtr->sfFile.name);
				}
				break;

			case sfHookNullEvent: // use spare time to dim/undim button as needed
				// if 'theFSSpecIsValid' is false, "Use " button has wrong name
				if (!(myHookDataPtr->fixBtnName)) {
					// we have to special case the desktop folder
					GetDItem(theDlogPtr, sfItemDesktopButton, &itemType, &((Handle)itemHandle), &itemRect);
					if ((**itemHandle).contrlHilite) {  // if current directory IS the desktop
						// set button's name to "Desktop Folder"
						DoRenameButton(theDlogPtr, desktopName);
					} else { // current directory is NOT the desktop
					myHookDataPtr->fixBtnName = 
													DoFindParentFSSpec(&(myHookDataPtr->replyPtr->sfFile));
					}

					// rename the button
					DoRenameButton(theDlogPtr, myHookDataPtr->replyPtr->sfFile.name);
					myHookDataPtr->fixBtnName = true; // btn name is OK now
				}
				
				// make Select button's hilite match Open button
				GetDItem(theDlogPtr, sfItemOpenButton, &itemType, &((Handle)itemHandle), &itemRect);
				tempBtnHilite1 = (**itemHandle).contrlHilite;
				GetDItem(theDlogPtr, kSelectBtnID, &itemType, &((Handle)itemHandle), &itemRect);
				tempBtnHilite2 =  (**itemHandle).contrlHilite;
				if (tempBtnHilite1 != tempBtnHilite2) {
					HiliteControl(itemHandle, tempBtnHilite1);
				}
				break;
		} // end of: switch(theItem)
	}
	return (returnCode);
}


// The following routine is responsible for letting the user pick a new destination for
// aliases created by this application.  If the user cancels this operation, "FALSE" is  
// returned and destFSSpecPtr is undefined.  Otherwise, "TRUE" is returned and destFSSpecPtr  
// contains the new destination's FSSpec record.
Boolean DoGetNewDestination(FSSpecPtr destFSSpecPtr)
{
	Point				centerDlog = {-1, -1};
	StandardFileReply	myReply;
	myGetHookRec		myHookData;

	myHookData.replyPtr = &myReply;
	myHookData.curDestFSSpecPtr = destFSSpecPtr;
	CustomGetFile((FileFilterYDProcPtr) myFileFilter, 0, nil, &myReply, kMyCustomDlogID, 
						centerDlog, (DlgHookYDProcPtr) myDlgHook, nil, nil, nil, &myHookData);
	BlockMove(&(myReply.sfFile), &(destFSSpecPtr->vRefNum), sizeof(FSSpec));
	return(myHookData.theFSSpecIsValid);
}


// The following routine handles idle-time activities.  For this app,
// we'll use this "spare time" as a place to put the configuration code.
void DoIdle(void)
{
	FSSpec		destFSSpec;
	AliasHandle	myAliasHndl, curDestAliasHndl;

	if ((gLetUserConfigureMe) && (CustomFileOK())) {
		if (DoGetCurDestination(&destFSSpec)) { // get current destination to use as default
			if (DoGetNewDestination(&destFSSpec)) { // if user picks new destination, record it
				// convert the destFSSpec to an alias
				// store it as an 'alis' resource in this app
				if (!NewAlias(nil, &destFSSpec, &myAliasHndl)){
						// rip out current 'alis' resource and stuff this one in
						curDestAliasHndl = (AliasHandle) Get1Resource(rAliasType, kMyAlisID);
						if (curDestAliasHndl) { // delete current 'alis' first
							RmveResource((Handle)curDestAliasHndl);
							DisposHandle((Handle)curDestAliasHndl);
						}
						AddResource((Handle)myAliasHndl, rAliasType, kMyAlisID, nil);
						UpdateResFile(gMyResFileRefNum); // flush changes
						
						// rename this application to reflect its new destination.
						DoRenameMe(&destFSSpec);
				}
			}
		}
		gQuitting = true; // set quit flag
	}
}


// The following routine verifies whether or not we can call CustomGetFile().
// If we can, "TRUE" is returned.  Otherwise, "FALSE" is returned.
Boolean CustomFileOK(void)
{
	OSErr	myErr;
	long	gestaltResponse;
	
	myErr = Gestalt(gestaltStandardFileAttr, &gestaltResponse);
	return ((myErr == noErr) && ((gestaltResponse) & (1<<gestaltStandardFile58)));
}
