/*
	Alias Meister.c
	Version 1.0
	 
	Copyright  1992 Tim Swihart
	All Rights Reserved

	October 8, 1992 (v.1.0)
*/

// Global variables:
short		gMyResFileRefNum;
Boolean		gQuitting;
Boolean		gInBackground;
Boolean		gLetUserConfigureMe;
RgnHandle	gCursorRgn;

// Function prototypes:
void main(void);
void MainEventLoop(void);
void DoEvent(EventRecord *event);

// External references:
extern void _DataInit(void);
extern void DoIdle(void);
extern short Initialize(void);
extern void AdjustMenus(void);
extern void DoMenuCommand(long menuResult);


// The following routine is the top-level view of our app.  It initializes the toolbox
// managers, unloads some code that's need just once and turns control over to our 
// main event loop (where the real action lurks).
void main(void)
{
	MaxApplZone();

	gMyResFileRefNum = Initialize();
	UnloadSeg((Ptr)Initialize);	// jettison initialization code since we no longer need it
	PurgeMem(FreeMem());	// clean up a little from unloaded segments

	MainEventLoop();

	TEToScrap();	// export our scrap for others to enjoy
}


// The following routine is the main event loop.  It gives up time to other apps and
// figures out whether or not there's time for doing idle-time tasks.
void MainEventLoop()
{
	EventRecord		event;
	long			sleepTime;
	Boolean			gotEvent;
	short			stayOutCounter = 5; // it's a fudge factor
	
	while (!gQuitting) {

		sleepTime = GetDblTime(); // user adjusts by changing dbl-click time in ctrl panel
		if (gInBackground) sleepTime = -1L;

		gotEvent = WaitNextEvent(everyEvent, &event, sleepTime, gCursorRgn);

		if (gotEvent) {
			DoEvent(&event);
		} else {
			 // To try to avoid loading IdleStuff segment when we're just creating aliases,
			 // we need to ignore the first couple of null events while we wait for the
			 // 'ODOC' Apple event to come through
			if (!--stayOutCounter) { DoIdle(); }
		}
	};
}


// The following routine handles the normal set of events that we're likely to receive.  Apple
// events are handled by different routines (see Events.d for those routines).
void DoEvent(EventRecord *event)
{
	short			myError;
	short			windowPart;
	WindowPtr		window;
	char			key;
	Point			mountPoint;

	switch (event->what) {
		case mouseDown:
			windowPart = FindWindow(event->where, &window);
			switch (windowPart) {
				case inMenuBar:
					AdjustMenus();	// prepare menu items first
					DoMenuCommand(MenuSelect(event->where));
					break;
				case inContent:
					break;
				case inDrag:
					break;
				case inGoAway:
					break;
			}
			break;
		case keyDown:
		case autoKey:
			key = event->message & charCodeMask;
			if (event->modifiers & cmdKey) {	// is command key down?
				if (event->what == keyDown) {
					AdjustMenus();				// prepare menu items first
					DoMenuCommand(MenuKey(key));
					break;
				}
			}
			break;
		case activateEvt:
			break;
		case updateEvt:
			break;
		case diskEvt:
			if ((event->message >> 16) != noErr) {
				mountPoint.h = kDILeft;
				mountPoint.v = kDITop;
				myError = DIBadMount(mountPoint, event->message);
			}
			break;
		case osEvt:
			switch ((event->message >> 24) & 0x0ff) {
				case suspendResumeMessage:
					if ((event->message & resumeFlag) == 0) { // suspending
						gInBackground = true;
						TEToScrap();	// export our scrap for others to enjoy
					} else { // resuming
						gInBackground = false;
						if (event->message & convertClipboardFlag) TEFromScrap();
						InitCursor();	// make sure we have pointer for cursor
					}
					break;
				case mouseMovedMessage:
					DisposeRgn(gCursorRgn);	// get rid of old region
					gCursorRgn = NewRgn();
					SetRectRgn(gCursorRgn, -32768, -32768, 32766, 32766); // entire space
					break;
			}
			break;
		case kHighLevelEvent:	// It's an Apple event
			AEProcessAppleEvent(event);
			break;   
	}
}
