/*--------------------------------
#
#	EventHandlers.c
#	Version 1.0
#
#	Copyright  1992 Tim Swihart
#	All rights reserved.
#
#	October 10, 1992 (v.1.0)
#
----------------------------------*/

// Function prototypes:
pascal OSErr HandleODOC(AppleEvent *theAppleEvent, AppleEvent *reply, 
							long myRefCon);									
pascal OSErr HandleQUIT(AppleEvent *theAppleEvent, AppleEvent *reply, 
							long myRefCon);									
pascal OSErr HandleOAPP(AppleEvent *theAppleEvent, AppleEvent *reply, 
							long myRefCon);
pascal OSErr HandlePDOC(AppleEvent *theAppleEvent, AppleEvent *reply, 
							long myRefCon);
static OSErr RequiredCheck(AppleEvent *theAppleEvent);
void FlushAVolume(FSSpecPtr theFSSpecPtr);
Boolean DoGetCurDestination(FSSpecPtr curDestFSSPecPtr);
Boolean AliasMgrCallsOK(void);
Boolean FindFolderOK(void);

// External globals:
extern Boolean	gQuitting;
extern Boolean	gLetUserConfigureMe;
									
// ------------------------------------- Apple  Event  Handlers -------------------------------------

//	The following routine is the "required" Apple event handler for "odoc" events
//	This is the routine that creates the aliases for us, then forces the app to quit.
pascal OSErr HandleODOC(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon)
{
	short 		markChar, fileRef, howManyChars;
	OSErr		myErr;
	long		itemsInList, i, tempCount;
	Size		actualSize;
	MenuHandle	myMenuHndl;
	AliasHandle	newAliasHndl;
	AEKeyword	theKeyword;
	AEDescList	docList;
	FSSpec		myFSS, destFSSpec;
	FInfo		theFInfo;
	DescType	typeCode;
	
	// After processing the incoming files, auto-quit this app (makes it behave like a "grinder")
	gQuitting = true;
	
	gLetUserConfigureMe = false;	// supresses configuration dialog

	myErr = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &docList);
	if (myErr) {
		AEDisposeDesc(&docList);	// release memory used by our copy of the event descriptor
		return(myErr);
	}
	
	myErr = RequiredCheck(theAppleEvent);
	if (myErr) {
		AEDisposeDesc(&docList);	// release memory used by our copy of the event descriptor
		return(myErr);
	}
	
	myErr = AECountItems(&docList, &itemsInList);
	if (myErr) {
		AEDisposeDesc(&docList);	// release memory used by our copy of the event descriptor
		return(myErr);
	}
	
	for (i = 1; i <= itemsInList; i++) {
		myErr = AEGetNthPtr(&docList, i, typeFSS, &theKeyword, &typeCode,
							 (Ptr)&myFSS, sizeof(FSSpec), &actualSize);
		if (myErr) {
			AEDisposeDesc(&docList);	// release memory used by our copy of the event descriptor
			return(myErr);
		}
			
		// get destFSSpec, convert incoming FSSpec to alias, create new file at dest,
		// attach 'alis' to new file, close file, attach custom icon if needed
		if (DoGetCurDestination(&destFSSpec)) {
			// name is invalid, but parID is correct.  Use name from incoming file
			tempCount = sizeof(Str63);
			BlockMove(myFSS.name, destFSSpec.name, sizeof(Str63));
			howManyChars = destFSSpec.name[0];
			if (howManyChars < 31) { // there's room in the buffer
				destFSSpec.name[++howManyChars] = ' '; // pad blank space
				destFSSpec.name[0] = howManyChars; // adjust length byte for new character
			} else { // no room in the buffer, overwrite last character
				destFSSpec.name[howManyChars] = ' '; // pad blank space
			}
			
			// create & open new file in destination folder
			FSpCreateResFile(&destFSSpec, ApplCreator, docsFileType, smSystemScript);
			fileRef = FSpOpenResFile(&destFSSpec, fsWrPerm); // = -1 for error
			if (fileRef != -1) {
				// check for existing resource in case file already existed
				newAliasHndl = (AliasHandle) Get1Resource(rAliasType, kFndrAlisID);
				if (newAliasHndl) { // delete current 'alis'
					RmveResource((Handle)newAliasHndl);
					DisposHandle((Handle)newAliasHndl);
				}
				
				// convert incoming FSSpec into alias
				NewAlias(nil, &myFSS, &newAliasHndl);
				
				// add new 'alis' to file we opened
				AddResource((Handle)newAliasHndl, rAliasType, kFndrAlisID, nil);
				
				CloseResFile(fileRef);
				FlushAVolume(&destFSSpec); // force changes to disk
				
				// get Finder info for doc we just created
				FSpGetFInfo(&destFSSpec, &theFInfo);
				
				theFInfo.fdFlags |= kIsAlias; // set 'isAlias' bit in Finder info
				FSpSetFInfo(&destFSSpec, &theFInfo); // write it to disk
				FlushAVolume(&destFSSpec); // force changes to disk
			}
		}
	}
	InitCursor();	// Finder puts up watch when document is dropped
					// on an application, so set cursor back to an arrow
	AEDisposeDesc(&docList);	// release memory used by our copy of the event descriptor
	return(noErr);
}


//	The following routine is the "required" Apple event handler for "quit" events
//	Don't ExitToShell from here, just set a global flag so the main event loop knows
//	we need to quit.  Bad things will happen to you if you ExitToShell here!!!
pascal OSErr HandleQUIT(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon)
{
	OSErr		myErr;
	
	myErr = RequiredCheck(theAppleEvent);
	if (myErr) return(myErr);

	gQuitting = true;
	return(noErr);
}


//	The following routine is the "required" Apple event handler for "oapp" events
//	This routine just sets a flag for the main event loop so it knows where we
//	were launched by the user double-clicking this app or by something being dropped
//	onto this app.  For "normal" launces, allow the user to configure the app.  For
//	launching with incoming documents, we'll need to auto-quit (which is set by the ODOC
//	handler).
pascal OSErr HandleOAPP(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon)
{
	gLetUserConfigureMe = true;
	return(RequiredCheck(theAppleEvent));
}


//	The following routine is the "required" Apple event handler for "pdoc" events
//	This app doesn't support printing, so reject this event unconditionally.
pascal OSErr HandlePDOC(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon)
{
	return(errAEEventNotHandled); 
}


//	The following routine verifies that all required parameters have been retrieved
//	properly.  If the
static OSErr RequiredCheck(AppleEvent *theAppleEvent)
{
	OSErr		myErr;
	DescType	typeCode;
	Size		actualSize;
	
	myErr = AEGetAttributePtr(theAppleEvent, keyMissedKeywordAttr, 
							   typeWildCard, &typeCode, 0L, 0, &actualSize);
							   
	if (myErr == errAEDescNotFound) return(noErr);	// no parms skipped, this is good.
	if (myErr == noErr) return(errAEEventNotHandled);	// we missed something, this is bad.
	return(myErr);	// something else went wrong
}


// The following routine is responsible for flushing the volume that the incoming file is on.
// We can find the right volume's ref number from the FSSpecPtr bring passed in 
void FlushAVolume(FSSpecPtr theFSSpecPtr)
{
	ParamBlockRec		myPBlockRec;

	// flush boot volume to make sure change is written to disk
	myPBlockRec.ioParam.ioCompletion = 0L;
	myPBlockRec.ioParam.ioNamePtr = 0L;
	myPBlockRec.ioParam.ioVRefNum = theFSSpecPtr->vRefNum;
	PBFlushVolSync(&myPBlockRec);
}


// The following routine is responsible for finding the current destination that aliases
// created by this app will be sent to.  The current destination is stored as an 'alis'
// resource in the app's resource fork.  If no 'alis' exists (this happens when the app is
// new), then the user's "Apple Menu Items" folder inside their boot volume's System folder
// will be used.  If the destination was found, "TRUE" is returned and the FSSpecPtr points
// to the current destination folder.  If no destination can be found, 'FALSE' is returned 
// and the FSSpecPtr is undefined.
Boolean DoGetCurDestination(FSSpecPtr curDestFSSpecPtr)
{
	short		myVRefNum;
	OSErr		myErr;
	Boolean 	wasChanged;
	Boolean		didItWork = false;	// assume the worst
	Boolean		foundIt = true;		// assume the best
	long		myDirID;
	AliasHandle	curDestAliasHndl=nil;
	CInfoPBRec 	paramBlock;
	
	if ((FindFolderOK()) && (AliasMgrCallsOK())) {
		// try to load 'alis' resource from app's rFork
		curDestAliasHndl = (AliasHandle) Get1Resource(rAliasType, kMyAlisID);
		if (curDestAliasHndl) { // convert alias into FSSpec
			myErr = ResolveAlias(nil, curDestAliasHndl, curDestFSSpecPtr, &wasChanged);
			if (!myErr) { didItWork = true; }
		} 
		if (!didItWork) { // if no 'alis' resource, or alias didn't resolve, use Apple Menu Items
			myErr = FindFolder(kOnSystemDisk, kAppleMenuFolderType, 
								kDontCreateFolder, &myVRefNum,&myDirID);
			if (!myErr) { // if desired folder exists, create FSSpec for it
				myErr = FSMakeFSSpec(myVRefNum, myDirID, nil, curDestFSSpecPtr);
				if (!myErr) { didItWork = true; } // let caller know we succeeded
			}
		}
		if (didItWork) {
			// munge FSSpec a tad.  Move the target's dirID into the "parID" field.  
			// That way, when CustomGetFile jumps to the "current destination", it lands 
			// correctly and not up one level.
			paramBlock.dirInfo.ioCompletion = nil;
			paramBlock.dirInfo.ioNamePtr = curDestFSSpecPtr->name;
			paramBlock.dirInfo.ioVRefNum = curDestFSSpecPtr->vRefNum;
			paramBlock.dirInfo.ioFDirIndex = 0; // find by name
			paramBlock.dirInfo.ioDrDirID = curDestFSSpecPtr->parID;
			PBGetCatInfoSync(&paramBlock);
			if (paramBlock.dirInfo.ioResult == 0) {
				curDestFSSpecPtr->parID = paramBlock.dirInfo.ioDrDirID;
			}
		}
	}
	return(didItWork);
}


// The following routine verifies whether or not the Alias Manager is present so
// we can make calls to it.
Boolean AliasMgrCallsOK(void)
{
	OSErr	myErr;
	long	gestaltResponse;
		
	myErr = Gestalt(gestaltAliasMgrAttr, &gestaltResponse);
	return ((myErr == noErr) && ((gestaltResponse) & (1<<gestaltAliasMgrPresent)));
}


// The following routine verifies whether or not we can call Findfolder().
// If we can, "TRUE" is returned.  Otherwise, "FALSE" us returned.
Boolean FindFolderOK(void)
{
	OSErr	myErr;
	long	gestaltResponse;
		
	myErr = Gestalt(gestaltFindFolderAttr, &gestaltResponse);
	return ((myErr == noErr) && ((gestaltResponse) & (1<<gestaltFindFolderPresent)));
}
