/*--------------------------------
#
#	Initialize.c
#	Version 1.0
#
#	Copyright  1992 Tim Swihart
#	All rights reserved.
#
#	October 10, 1992 (v.1.0)
#
#	The code in this file initializes the various
#	toolbox managers we'll need & installs our
#	Apple event handlers.  All of the routines in
#	this file are called only once & that's 
#	while the application is starting up.  So, 
#	I put it in its own segment so it can be 
#	unloaded after executing, freeing up its memory
#	& reducing the amount of memory needed for this app.
#
----------------------------------*/


// External gloabls:
extern	Boolean		gQuitting;
extern	Boolean		gInBackground;
extern	Boolean		gLetUserConfigureMe;
extern	RgnHandle	gCursorRgn;

// External references:
extern pascal OSErr HandleOAPP(AppleEvent *theAppleEvent, AppleEvent *reply, 
									long myRefCon);
extern pascal OSErr HandleODOC(AppleEvent *theAppleEvent, AppleEvent *reply, 
									long myRefCon);
extern pascal OSErr HandlePDOC(AppleEvent *theAppleEvent, AppleEvent *reply, 
									long myRefCon);
extern pascal OSErr HandleQUIT(AppleEvent *theAppleEvent, AppleEvent *reply, 
									long myRefCon);

// Function prototypes:
void 	DoAEInstallation(void);
short 	Initialize(void);
Boolean	System7Available(void);

// The following routine initializes the toolbox managers, sets up our
// menu bar, and does a few other housekeeping things.
short Initialize(void)
{
	Handle				menuBar;

	FlushEvents (everyEvent, 0);

	InitGraf(&(qd.thePort));
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(0L);
	InitCursor();

	gInBackground = false;
	gQuitting = false;
	gLetUserConfigureMe = false;	// config dialog suppressed by default
	gCursorRgn = NewRgn();	// force cursor-moved event immediately

	if (!System7Available()) {	// Not running System 7?  PUNT!
		Alert(rNotSystem7, 0L);
		gQuitting = true;
		return(CurResFile());
	}

	TEFromScrap();
	ZeroScrap();

	menuBar = GetNewMBar(rMenuBar);	// Create the menu bar
	if (menuBar) {
		SetMenuBar(menuBar);
		DisposHandle(menuBar);
		AddResMenu(GetMHandle(mApple), 'DRVR');	// add Apple Menu items
		DrawMenuBar();
	} else { // no menu bar, so punt
		gQuitting = true;
		return(CurResFile());
	}
	DoAEInstallation();	// install AppleEvent handlers
	return(CurResFile());
}



// The following routine installs our AppleEvent handlers.
void DoAEInstallation(void)
{
	AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, 
						   (EventHandlerProcPtr)HandleODOC, 
						   0, false);
	AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, 
						   (EventHandlerProcPtr)HandleQUIT, 
						   0, false);
	AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments, 
						   (EventHandlerProcPtr)HandlePDOC, 
						   0, false);
	AEInstallEventHandler(kCoreEventClass, kAEOpenApplication, 
						   (EventHandlerProcPtr)HandleOAPP, 
						   0, false);							   
}


// The following routine tells us whether or not we're running under at least System 7.0.
// I'm skipping the check for whether or not Gestalt is present because MPW provides glue
// that handles the case where Gestalt wasn't present.
Boolean System7Available(void)
{
	long whichVers;

	Gestalt(gestaltSystemVersion, &whichVers); // ex: 7.0.1 comes back as $0701
	whichVers = whichVers & 0xF00; // mask off minor version number
	if (whichVers < 0x700) {
		return(false);	// didn't get what we wanted
	} else {
		return(true);	// we're running under at least 7.0.0
	}
}
