UNIT GestaltStorage;

INTERFACE

USES Memtypes,QuickDraw,OSIntf,ToolIntf,GestaltEqu,
	GestaltStorage_glue;

CONST	gsFromStart=0;
		gsStorageSize=8;
TYPE	
		(* Key concept: through the use of Gestalt and our private selector	*)
		(* we store an address to a pointer in system heap, and thus access	*)
		(* it later, even during interrupts tasks							*)
		(* We could have stored the address of the 'raw' pointer to datas	*)
		(* but we decided to use the following structure for our datas:		*)
		
		gsStoragePtr=^gsStorageData;
		gsStorageData=RECORD
						{kind of 'private' fields}
						gsSize:Size;		{size of the 'raw' datas	}
						gsFuncAddr:LongInt;	{address of the GestaltFunction}
						{kind of 'public' field}
						{raw datas goes here for gsSize bytes long}
					  END;
					  
		(* For Pascal uses,we defined that structure to mimic the asm 	*)
		(* structure of the GestaltFunction. Forget it otherwise		*)
		gsFuncPtr=^gsFuncData;
		gsFuncData=RECORD
						BRAinstruction:integer;	{entrypoint of asm routine}
						gsStorageAddr:LongInt;	{our DC.L storage}
						{func code goes here as defined in the .a code}
					END;
					

FUNCTION GSNew(selector:OSType;storageSize:Size;globalData:Ptr):OSErr;
FUNCTION GSGet(selector:OSType;bucketPtr:Ptr):OSErr;
FUNCTION GSRead(selector:OSType;offset,length:LongInt;bucketPtr:Ptr):OSErr;
FUNCTION GSSet(selector:OSType;globalData:Ptr):OSErr;
FUNCTION GSWrite(selector:OSType;offset,length:LongInt;globalData:Ptr):OSErr;
FUNCTION GSDispose(selector:OSType):OSErr;

IMPLEMENTATION


FUNCTION GSNew(selector:OSType;storageSize:Size;globalData:Ptr):OSErr;
VAR		theErr:OSErr;
		globalDataAddr:LongInt;
		theStorage:Ptr;
		theGestaltFunc:Ptr;
		theGestaltFuncAddr:LongInt;
		oldStorageSize:Size;
BEGIN
	theErr:=__Gestalt(selector,globalDataAddr);
	IF theErr=gestaltUndefSelectorErr THEN 
	{ An undefined selector was passed to Gestalt : it's OK for us to allocate a brand new	}
	{ storage component.																	}
	BEGIN
		theGestaltFunc:=GSNewFunction;
		IF theGestaltFunc<>NIL THEN
		BEGIN
			theStorage:=GSNewStorage(storageSize,globalData);
			IF theStorage<>NIL THEN
			BEGIN
				gsFuncPtr(theGestaltFunc)^.gsStorageAddr:=ORD(theStorage);
				gsStoragePtr(theStorage)^.gsFuncAddr:=ORD(theGestaltFunc);
				theErr:=__NewGestalt(selector,ProcPtr(theGestaltFunc));
			END
			ELSE
			BEGIN
				gsStoragePtr(theStorage)^.gsFuncAddr:=0;
				theErr:=memFullErr;
			END;
		END
		ELSE theErr:=memFullErr;
	END
	ELSE IF theErr=$0100 THEN 
	{ value returned by the _Gestalt function if storage address is NIL (header)}
	{ the selector is still valid, so we can re-use it							}
	{ in this case, the globalDataAddr longint holds the address of the 		}
	{ gestalt function.															}
	BEGIN
		theGestaltFuncAddr:=globalDataAddr;			
		IF theGestaltFuncAddr<>0 THEN						 {probably paranoid}
		BEGIN
			theGestaltFunc:=Ptr(theGestaltFuncAddr);
			IF gsFuncPtr(theGestaltFunc)^.BRAinstruction=$6004 THEN
			BEGIN
				theStorage:=GSNewStorage(storageSize,globalData);
				IF theStorage<>NIL THEN
				BEGIN
					gsFuncPtr(theGestaltFunc)^.gsStorageAddr:=ORD(theStorage);
					gsStoragePtr(theStorage)^.gsFuncAddr:=theGestaltFuncAddr;
					theErr:=noErr;
				END
				ELSE theErr:=memFullErr;
			END
			ELSE theErr:=gestaltDupSelectorErr;				{really paranoid}
		END
		ELSE theErr:=gestaltDupSelectorErr;
	END
	ELSE IF theErr=noErr THEN 
	{an entry already exist: we OVERWRITE it with same Size}
	BEGIN
		oldStorageSize:=gsStoragePtr(Ptr(globalDataAddr))^.gsSize;
		BlockMove(globalData,Ptr(globalDataAddr+SizeOf(gsStorageData)),oldStorageSize);
	END
	ELSE theErr:=gestaltDupSelectorErr;
	{an unexpected error occurs: we safetely do nothing}
	GSNew:=theErr;
END;


FUNCTION GSGet(selector:OSType;bucketPtr:Ptr):OSErr;
VAR theErr:OSErr;
	globalDataAddr:LongInt;
BEGIN
	theErr:=__Gestalt(selector,globalDataAddr);
	IF theErr=noErr THEN
	WITH gsStoragePtr(Ptr(globalDataAddr))^ DO
		BlockMove(Ptr(globalDataAddr+SizeOf(gsStorageData)),bucketPtr,gsSize);
	GSGet:=theErr;
END;

FUNCTION GSRead(selector:OSType;offset,length:LongInt;bucketPtr:Ptr):OSErr;
VAR theErr:OSErr;
	globalDataAddr:LongInt;
BEGIN
	theErr:=__Gestalt(selector,globalDataAddr);
	IF theErr=noErr THEN
		BlockMove(Ptr(globalDataAddr+SizeOf(gsStorageData)+offSet),bucketPtr,length);
	GSRead:=theErr;
END;

FUNCTION GSSet(selector:OSType;globalData:Ptr):OSErr;
VAR	theErr:OSErr;
	globalDataAddr:LongInt;
BEGIN
	theErr:=__Gestalt(selector,globalDataAddr);
	IF theErr=noErr THEN
	WITH gsStoragePtr(Ptr(globalDataAddr))^ DO
		BlockMove(globalData,Ptr(globalDataAddr+SizeOf(gsStorageData)),gsSize);
	GSSet:=theErr;
END;

FUNCTION GSWrite(selector:OSType;offset,length:LongInt;globalData:Ptr):OSErr;
VAR theErr:OSErr;
	globalDataAddr:LongInt;
BEGIN
	theErr:=__Gestalt(selector,globalDataAddr);
	IF theErr=noErr THEN
		BlockMove(globalData,Ptr(globalDataAddr+SizeOf(gsStorageData)+offSet),length);
	GSWrite:=theErr;
END;

FUNCTION GSDispose(selector:OSType):OSErr;
VAR theErr:OSErr;
	globalDataAddr:LongInt;
	globalData:Ptr;
BEGIN
	theErr:=__Gestalt(selector,globalDataAddr);
	IF theErr=noErr THEN
	BEGIN
		globalData:=Ptr(globalDataAddr);
		IF globalData<>NIL THEN
		BEGIN
			gsFuncPtr(Ptr(gsStoragePtr(globalData)^.gsFuncAddr))^.gsStorageAddr:=0;
			DisposPtr(globalData);
			globalData:=NIL;
		END
		ELSE theErr:=nilHandleErr;
	END;
	GSDispose:=theErr;
END;

END.
