/************************************************************************************

Sound101_recordplay.c

 1992 Praxitel, Inc. and Iggi Monahelis

This file has all the necessary routines for recording 'snd ' resource sounds or 
Audio Interchange File Format (AIFF) sounds, and play them back.


*************************************************************************************/
#include "Sound101.h"


int 			gDestroyChannel = FALSE;
SndChannelPtr	gSndChan;
short 			gFRefNum;
long			gQuality = 'good';

/************************************************************************************

Record_snd_resource

Records to memory an 'snd ' recource sound, and then saves it to a file.  It first 
prompts the user for the filename of the file to save the sound resource to.

It takes one parameter, quality, which is the desired sound quality. (good, better, or
best).
 
*************************************************************************************/
void Record_snd_resource(long quality)
{
	OSErr myErr;
	Point myPoint;
	Handle mySndH;
	FInfo	finder;
	short  resRef;
	SFReply sf;
	int		resID;
	
	myPoint.h = myPoint.v = 50;
	sf.fName[0] = 0; 
	mySndH = NULL;
	
	/* let user select a file to save sound into */
	SFPutFile(myPoint,"\pSave sound as:",sf.fName,NULL,&sf);
	
	if ((sf.good)) {
 		SetVol(0L,sf.vRefNum);
 		
 		/* create a file for the sound and open it */
 		CreateResFile(sf.fName);
 		resRef = OpenResFile(sf.fName);
 		
 		/* get the file's Finder info, we are going to change the creator and filetype */
 		GetFInfo( sf.fName, sf.vRefNum, &finder );
		/* set the file type to 'sfil' and the creator to 'movr'. that way
		under System 7, when the file gets double-clicked in the Finder
		it will be played by the System
		*/
		finder.fdType = 'sfil';
		finder.fdCreator = 'movr';
		/* now change the file's Finder info */
		SetFInfo( sf.fName, sf.vRefNum, &finder );
	
		/* now record the sound resource into the handle mySndH */
		myErr = SndRecord( NULL, myPoint, quality, &mySndH);
		if (myErr) { /* if there were problems, clean up */
			CloseResFile(resRef);
			FSDelete(sf.fName, sf.vRefNum);
			if (mySndH)
				DisposHandle(mySndH);
		}
		else { /* otherwise save the sound resource into the file */
			resID = UniqueID('snd ');
			AddResource(mySndH, 'snd ', resID, "\pIggi sound");
			UpdateResFile(resRef);
			DetachResource(mySndH);
			CloseResFile(resRef);
			DisposeHandle(mySndH);
		}
	}
}

/************************************************************************************

Record_AIFF_sound

Records to directly to disk an AIFF sound.  It first prompts the user for the filename
of the file to save the sound into.

It takes one parameter, quality, which is the desired sound quality. (good, better, or
best).
 
*************************************************************************************/
void Record_AIFF_sound(long quality)
{
	OSErr myErr, err = FALSE;
	Point myPoint;
	short	fRefNum;
	short  resRef;
	FInfo	finder;
	Str255	tempStr;
	short	oldVol;
	SFReply sf;

	myPoint.h = myPoint.v = 50;
	
	sf.fName[0] = 0; 
	
	/* let user select a file to save sound into */
	SFPutFile(myPoint,"\pSave sound as:",sf.fName,NULL,&sf);

	if ((sf.good)) {
 		SetVol(0L,sf.vRefNum);

		/* create a file and open it */
 		CreateResFile(sf.fName);
 		resRef = OpenResFile(sf.fName);
 		
 		/* get the file's Finder info, we are going to change the creator and filetype */
 		GetFInfo( sf.fName, sf.vRefNum, &finder );

		/* set the filetype to AIFF, creator to RMLP.  The sound is then
		recognized as a "Read My Lips" sound
		*/
		
		finder.fdType = 'AIFF';
		finder.fdCreator = 'RMLP';
		/* now change the file's Finder info */
		SetFInfo( sf.fName, sf.vRefNum, &finder );
	
 		if (!ResError() && resRef)
 			CloseResFile(resRef);
		/* open the data fork for recording directly to disk */
		err = FSOpen(sf.fName, sf.vRefNum, &fRefNum);
		if (!err) {
			/* note here we pass in the fRefNum of the file we are recording 
			directly into.  for the sound resource (above) we passed a sound handle */
			err = SndRecordToFile(NULL, myPoint, quality, fRefNum);
			if (!err) /* everything OK, close the file */
				err = FSClose(fRefNum); 
			else { /* error, close the file and delete it */
				FSClose(fRefNum); 
				FSDelete(sf.fName, sf.vRefNum);
			}
		}
	}
}

/************************************************************************************

PlayASound

Prompts the user to select a sound (either 'snd ' or AIFF) thru the standard file
dialog, and then plays it.
 
*************************************************************************************/
void PlayASound(void)
{

SFReply			reply;
OSErr 			myErr;
Handle		tempHandle;
int 		numResources;
int			tempId;
OSType		tempType;
int			index;
Str255		tempStr;
CursHandle	hCurs;
Cursor		waitCursor;

	gSndChan = 0;
	gFRefNum = 0;
	
	/* let user select the sound file to play */
	if ( DisplayGetFile( &reply ) ) {
		/* allocate a new sound channel */
		myErr = SndNewChannel(&gSndChan, sampledSynth, initMono, NULL);
		
		hCurs = GetCursor(1300); /* set the cursor to the command-period cursor */
		waitCursor = **hCurs;
		SetCursor(&waitCursor);
		
		SetVol(0L, reply.vRefNum);
		if ( reply.fType == 'AIFF' ) { /* if the file is an AIFF one, and open it */
			myErr = FSOpen(reply.fName, reply.vRefNum, &gFRefNum);
			/* play the sound */
			PlayAIFF();
			}
		else {
			/* it is a sound resource file, open it, and get the resource
			id of the first sound resource */
			gFRefNum = OpenResFile(reply.fName);
			SetResLoad( FALSE );
			tempHandle = Get1IndResource('snd ',1);
			GetResInfo(tempHandle,&tempId,&tempType,tempStr);
			ReleaseResource(tempHandle);
			SetResLoad( TRUE );
			/* play the sound */
			PlaySND(tempId);
		}
	}
}

/************************************************************************************

PlaySND

Plays an 'snd ' resource sound using the SndStartFilePlay toolbox call.  The only 
parameter is the resource id of the sound to be played.  The sound file has allready 
been opened by the routine that called PlaySND.  We pass the id of the sound resource
to be played to the SndStartFilePlay routine.

The sound is played asynchronously.  For this reason, we set a global variable
(gDestroyChannel) to FALSE.  When the sound is done playing our callback routine is 
called (MyCallBack), which sets gDestroyChannel to TRUE.  This is our indication that
the sound is done playing and we can go ahead and close the file, and free the sound
channel.

NOTE:
The userInfo field of the sound channel structure is a convenient place to store the
current A5.  This way in the callback routine we can have access to our A5, and thus have 
access to our globals.  See the TechNotes for more info about SetCurrentA5 and SetA5.

*************************************************************************************/
void PlaySND(int tempId)
{
	gDestroyChannel = 0x0;
	gSndChan->userInfo = SetCurrentA5();
	/* since this is a sound resource, we pass its id to SndStartFilePlay */
 	SndStartFilePlay(gSndChan, 0, tempId, 20L*1024L, 0x0L,
                             0x0L, (ProcPtr) MyCallBack, TRUE);
}

/************************************************************************************

PlayAIFF

Plays an AIFF sound using the SndStartFilePlay toolbox call.  The sound file has 
allready been opened by the routine that called PlayAIFF.  The FrefNum of the file
to be played is contained in the global variable "gFRefNum", which get passed in 
to SndStartFilePlay.

The sound is played asynchronously.  For this reason, we set a global variable
(gDestroyChannel) to FALSE.  When the sound is done playing our callback routine is 
called (MyCallBack), which sets gDestroyChannel to TRUE.  This is our indication that
the sound is done playing and we can go ahead and close the file, and free the sound
channel.

NOTE:
The userInfo field of the sound channel structure is a convenient place to store the
current A5.  This way in the callback routine we can have access to our A5, and thus have 
access to our globals.  See the TechNotes for more info about SetCurrentA5 and SetA5.

*************************************************************************************/
void PlayAIFF(void /*SFReply *sf*/)
{
	OSErr myErr;

	gDestroyChannel = 0x0;
	gSndChan->userInfo = SetCurrentA5();
	/* since this is an AIFF sound, we pass the frefnum of the file.  The sound
	is played directly from disk */
	myErr = SndStartFilePlay (gSndChan, gFRefNum, 0, 20L*1024L, 0x0L, 0x0L, 
								(ProcPtr) MyCallBack, TRUE);

}

/************************************************************************************

MyCallBack

When the sound is done playing, this callback routine is called.  We indicated this
routine to be called in the SndStartFilePlay call above.

This routine basically sets our global flag (gDestroyChannel) to TRUE.  This is 
our indication that the sound is done playing and we can go ahead and dispose of 
the sound channel we allocated, and close the file that contained the sound.
 
*************************************************************************************/
pascal void MyCallBack (SndChannelPtr chan)
{
	OSErr myErr;
	long  myA5;
	
	myA5 = SetA5( chan->userInfo ); 	
	gDestroyChannel = TRUE;
	myA5 = SetA5( myA5 ); 	

}

/************************************************************************************

DisplayGetFile

Prompts the user to select a file thru the standard file
dialog.  
 
*************************************************************************************/
Boolean DisplayGetFile(SFReply *reply)
{
	Point		where = {100, 100};

	SFGetFile(where, (ConstStr255Param) "\pSelect a sound to play.",
						 nil, -1, 0, nil, reply);
	return(reply->good);
}


/************************************************************************************

GetQuality

Simple routine to let the user select the sound quality to be used for recording.

 
*************************************************************************************/
void GetQuality(void)
{

	DialogPtr	myDialog;
	short		itemHit = 0;
	short		qualityPick;
	Boolean		done = FALSE;
	GrafPtr 	oldPort;
	
	GetPort(&oldPort);

	myDialog = GetNewDialog( optionsDlgID, 0, (WindowPtr)-1);
	
	if (myDialog)
	{
		ShowWindow(myDialog);
		SelectWindow(myDialog);
		SetPort(myDialog);
		
		/* do initialization for the dialog */

		switch(gQuality)
		{
			case siGoodQuality:
				SetRadioButton( myDialog, dGood);
				qualityPick = dGood;
				break;
			case siBetterQuality:
				SetRadioButton( myDialog, dBetter);
				qualityPick = dBetter;
				break;
			case siBestQuality:
				SetRadioButton( myDialog, dBest);
				qualityPick = dBest;
				break;
		}
				
		while( !done) {
			ModalDialog( NULL, &itemHit);
			
			switch( itemHit) {
				case dCancel:
					done = TRUE;
					break;
				
				case dDone:
					done = TRUE;
					
					switch(qualityPick) {
						case dBest:
							gQuality = siBestQuality;
							break;
						
						case dBetter:
							gQuality = siBetterQuality;
							break;
						
						case dGood:
							gQuality = siGoodQuality;
							break;
					}
					break;
					
				case dBest:
				case dBetter:
				case dGood:
					ToggleRadioButtons( myDialog, qualityPick, itemHit);
					qualityPick = itemHit;
					break;
				
			}
		}
			/* clean-up */
		DisposDialog( myDialog);
		SetPort(oldPort);
	}
}

/************************************************************************************

ToggleRadioButtons
SetRadioButton

Couple of utility routines to set and toggle radio buttons.
 
*************************************************************************************/
void	ToggleRadioButtons( DialogPtr theDialog, short oldItem, short newItem)
{
	ControlHandle	control;
	short			itemType;
	Rect			box;
	
	GetDItem(theDialog, oldItem,  &itemType, &control, &box);
	SetCtlValue(control, 0);
	GetDItem(theDialog, newItem, &itemType, &control, &box);
	SetCtlValue(control, 1);
}

void	SetRadioButton( DialogPtr theDialog, short theItem)
{
	ControlHandle	control;
	short			itemType;
	Rect			box;
	
	GetDItem(theDialog, theItem, &itemType, &control, &box);
	SetCtlValue( control, 1);
}
