#include <MacApp.h>
#include <Fonts.h>
#include "TReportPrinter.h"

//TReportPrinter

#pragma segment PrintInit
pascal void TReportPrinter::IReportPrinter(CStr255 theFont, 
											short theFace, 
											short theSize, 
											short leftMargin, 
											short bottomOfPage,
											Boolean autoFormFeed)
{
	short	theFontNumber;
	
	fPrintRecord = nil;
	fPrintingPort = nil;
	fMyStatusWindow = nil;
	
	fAutoFormFeed = autoFormFeed;		// true if we handle formfeeds
	
	// initialize the font/face/size fields
	
	fCurrentFontName = theFont;
	GetFNum(theFont, theFontNumber);	
	
	fCurrentFontNumber = theFontNumber;
	fCurrentFace = theFace;
	fCurrentSize = theSize;
	
	fBottomOfPage = bottomOfPage;
	fLeftMargin = leftMargin;
	fUserAborted = false;
	fMyStatusWindow = nil;
	fPrintedLines = nil;
	fCurrentPage = nil;
	fPrintManagerIsOpen = false;
	
	// GetFreeSpaceOnDisk returns a value in multiples of 1024. We can only use
	// half the space since the print manager spool file and a background print
	// file of the same size might exist at the same time.
	// Then, If we have 80 character lines, we'll get about 12 of them per 1024 bytes. 
	
	short	theVol;
	OSErr err = GetVol(0,theVol);			// get default volume ref. number
	long maxSpoolLines = (GetFreeSpaceOnDisk(theVol) / 2) * 12;
	fMaxSpoolLines = maxSpoolLines;
}

#pragma segment PrintOpen
pascal void TReportPrinter::DoPrOpen(void)
{
	FailInfo	fi;
	THPrint		aPrintRecord = nil;
	
	VOLATILE(aPrintRecord);
	
	if(fi.Try()) {
		if(!fPrintManagerIsOpen) {		// open only if necessary
			PrOpen();
			
			// Allocate and initialize a print record if necessary
			
			if(fPrintRecord == nil) {
				aPrintRecord = (THPrint)NewHandle(sizeof(TPrint));
				PrintDefault(aPrintRecord);		// Set "standard" values
				fPrintRecord = aPrintRecord;
			}
			fPrintManagerIsOpen = true;
		}
		fi.Success();
	}
	else {
		fPrintManagerIsOpen = false;
		if(fPrintRecord) {
			DisposeHandle((Handle)fPrintRecord);
			fPrintRecord = nil;
		}
		PrClose();
		fi.ReSignal();
	}
}

#pragma segment PrintOpen
pascal void TReportPrinter::SetDocumentName(CStr255 theDoc)
{
	fReportTitle = theDoc;
}

#pragma segment PrintMain
pascal void TReportPrinter::Free(void)
{
	if(fPrintRecord) {
 		DisposeHandle((Handle)fPrintRecord);
		fPrintRecord = nil;
	}
	
	// fPrintingPort should have been freed and set to 0 by the PrCloseDoc
	// call in EndReport(). But just to be on the safe side...
	
	if(fPrintingPort) {	
		PrCloseDoc(fPrintingPort);
		fPrintingPort = nil;
	}
	
	if(IsObject(fMyStatusWindow))
		fMyStatusWindow->Close();
		
	inherited::Free();
}

#pragma segment PrintMain
pascal Boolean TReportPrinter::HandlePageSetup(void)
{	
//	Note: The TReportPrinter MUST be initialized before allowing
//		  a call to this method.
	
	Boolean	rc = false;
	DoPrOpen();		// open the print manager if necessary
	if(fPrintRecord) {
		rc = PrStlDialog(fPrintRecord);
	}
	return rc;
}

#pragma segment PrintOpen
pascal Boolean TReportPrinter::StartReport(CommandNumber/* aCmdNumber*/)
{
	// display the Job Dialog and if it returns true, open the print port and
	// then open the first page.
	
	FailInfo	fiWindow, fiPrint;
	Boolean doIt = false;
	TReportStatusWindow *aStatusWindow = nil;
	TPPrPort	aPrintingPort = nil;

	VOLATILE(aStatusWindow);
	
	if(fiPrint.Try()) {
		DoPrOpen();						// open the print manager if necessary
		
		fCurrentPage = 0;
		fPrintedLines = 0;
		fUserAborted = false;
		
		doIt = PrJobDialog(fPrintRecord);	
		
		if(doIt && PrError() == noErr) {
		
			// first we try to open the print document
			
			aPrintingPort = PrOpenDoc(fPrintRecord, 0, 0);
			fPrintingPort = aPrintingPort;
			
			if(PrError() == noErr) {
				if(fiWindow.Try()) {
					aStatusWindow = (TReportStatusWindow *) gViewServer->NewTemplateWindow(kStatusWindowView, 0);
					aStatusWindow->IReportStatusWindow(this);
					aStatusWindow->Open();
					fMyStatusWindow = aStatusWindow;
					fiWindow.Success();
				}
				else {
					fMyStatusWindow = nil;
					fiWindow.ReSignal();
				}
				gApplication->PollEvent(false);		// let everybody else update, etc.
				if(DoOpenPage())	 {				// try to start a page
					doIt = true;					// everything worked!
					
					// verify that the bottom margin does not excede the capabilities
					// of the printer
				
					if(fBottomOfPage > ((GrafPtr)fPrintingPort)->portRect.bottom)
						fBottomOfPage = ((GrafPtr)fPrintingPort)->portRect.bottom;
				}
				else {					// PrOpenPage failed
					doIt = false;
				}
			}					
			else {								// PrOpenDoc failed...
				PrCloseDoc(fPrintingPort);		// frees fPrintingPort
				fPrintingPort = nil;			
				doIt = false;
			}
		}
		else {
			doIt = false;	// PrJobDialog() failed or the user hit cancel
		}
		if(!doIt)			// finish cleaning up after any failures
			DoPrClose();
		fiPrint.Success();
	}
	else {
		fiPrint.ReSignal();
	}
	return doIt;
}
#pragma segment PrintMain
pascal void TReportPrinter::DoPrClose(void)
{
		PrClose();						// close the print manager (TN 161)
		fPrintManagerIsOpen = false;
		if(fPrintRecord) {			// and dispose of the print record
			DisposeHandle((Handle)fPrintRecord);
			fPrintRecord = nil;
		}
		if(IsObject(fMyStatusWindow))
			fMyStatusWindow->Close();					// has free on closing checked in viewedit
		fMyStatusWindow = nil;
}

#pragma segment PrintMain
pascal void TReportPrinter::EndReport(void)
{
	// Note: This is the simplest possible way to wrap up a print job.
	
	TPrStatus	status;
	
	PrClosePage(fPrintingPort);	// ensure current page is closed
	PrCloseDoc(fPrintingPort);		// frees fPrintingPort
	fPrintingPort = nil;			
	
	if((**fPrintRecord).prJob.bJDocLoop == bSpoolLoop && PrError() == noErr) {		
		gApplication->PollEvent(false);					// let everybody else update, etc.
		PrPicFile(fPrintRecord,0,0,0,status);
	}
	DoPrClose();	
}

#pragma segment PrintImage
pascal Boolean TReportPrinter::DoOpenPage(void)
{
	// Notes: PrOpenPage completely resets the printing grafport so it is necessary
	//        to set the font, face, and size every time we do this. 
	
	fCurrentPage++;
	
	fMyStatusWindow->SetMessage(fReportTitle);
	gApplication->PollEvent(false);					// let everybody else update, etc.

	PrOpenPage(fPrintingPort, 0);
	if(PrError() == noErr) {
		TextFont(fCurrentFontNumber);
		TextFace(fCurrentFace);
		TextSize(fCurrentSize);
		return true;
	}
	else {
		PrClosePage(fPrintingPort);	// match the cPrOpenPage
		PrCloseDoc(fPrintingPort);		// frees fPrintingPort
		fPrintingPort = nil;		
		return false;
	}
}

#pragma segment PrintImage
pascal void TReportPrinter::DoClosePage(void)
{
	TPPrPort	aPrintingPort = nil;
	FailInfo	fi;
	
	VOLATILE(aPrintingPort);
	
	PrClosePage(fPrintingPort);			// end the page
	
	if(fPrintedLines > fMaxSpoolLines) {
		EndReport();					// print what we have
		if(fi.Try()) {
			aPrintingPort = PrOpenDoc(fPrintRecord, 0, 0);	// start a new one
			fPrintingPort = aPrintingPort;
			fi.Success();
		}
		else {
			fPrintingPort = nil;
			fi.ReSignal();
		}
	
	}
	gApplication->PollEvent(false);	// let everybody else update, etc.
}

// Note: The next three methods must be called AFTER DoOpenPage() to have the desired
//       effect.

#pragma segment PrintImage
pascal void TReportPrinter::SetFont(CStr255 theFont)
{

	short	theFontNumber;
	
	fCurrentFontName = theFont;
	GetFNum(theFont, theFontNumber);	
	
	fCurrentFontNumber = theFontNumber;
	
	TextFont(fCurrentFontNumber);	// actually set the font in the grafport

}
#pragma segment PrintImage
pascal void TReportPrinter::SetFace(short theFace)
{
	fCurrentFace = theFace;
	TextFace(fCurrentFace);
}
#pragma segment PrintImage
pascal void TReportPrinter::SetSize(short theSize)
{
	fCurrentSize = theSize;
	TextSize(fCurrentSize);
}

#pragma segment PrintImage
pascal void TReportPrinter::DoFormFeed(void)
{
	PrintFooter();
	DoClosePage();
	DoOpenPage();
	PrintHeader();
}

#pragma segment PrintImage
pascal void TReportPrinter::PrintHeader(void)
{
	FontInfo	theInfo;

// basic operation only - override for custom header

	GetFontInfo(theInfo);
	MoveTo(fLeftMargin, theInfo.ascent+theInfo.descent+theInfo.leading);
	
	// don't forget to increment fPrintedLines

}

#pragma segment PrintImage
pascal void TReportPrinter::PrintFooter(void)
{
	// Override for custom Footer
	// don't forget to increment fPrintedLines
}

#pragma segment PrintImage
pascal Boolean TReportPrinter::PrintALine(const CStr255 theLine, Boolean doLineFeed)
{
	// Note: LineFeeds are done based on the font metrics. For other size linefeeds
	//       use VerticalTab();
	 
	CPoint		theSpot;
	FontInfo	theInfo;
	GrafPtr	savedPort;
	
	if(fUserAborted) {	
		PrSetError(iPrAbort);		// let the print manager know
		EndReport();
		return false;
	}
		
	DrawString(theLine);
	if(doLineFeed) {
		fPrintedLines++;
		GetPen(theSpot);
		GetFontInfo(theInfo);
		short verticalPosition = theSpot.v + theInfo.ascent+theInfo.descent+theInfo.leading;
		MoveTo(fLeftMargin, verticalPosition);
		
		if(verticalPosition >= fBottomOfPage && fAutoFormFeed) {
			DoFormFeed();
		}
		GetPort(savedPort);
		gApplication->PollEvent(false);	// let everybody else update, etc.
		SetPort(savedPort);						// make sure we're in the right place
	}
	return true;
}

#pragma segment PrintImage
pascal void TReportPrinter::VerticalTab(short pixels)
{
	CPoint	theSpot;
		
	GetPen(theSpot);
	theSpot.v+=pixels;
	if(theSpot.v >= fBottomOfPage) {
		DoFormFeed();
	}
	else
		MoveTo(theSpot.h, theSpot.v);
}

#pragma segment PrintInit
pascal unsigned long TReportPrinter::GetFreeSpaceOnDisk(int theVolume)
{
    HVolumeParam HPB;
    OSErr err;
	unsigned long pfreeBytes;
	
    HPB.ioNamePtr = 0L;       
    HPB.ioVRefNum = theVolume;      
    HPB.ioVolIndex = 0;       
    err = PBHGetVInfo((HParamBlockRec *)&HPB,false);
    if (err == noErr)
        pfreeBytes = (unsigned long int)HPB.ioVFrBlk * HPB.ioVAlBlkSiz / 1024;
    else
        pfreeBytes = 0L;    
    return(pfreeBytes);            

}
#pragma segment PrintImage
pascal void TReportPrinter::UserAbort(void) 
{
	fUserAborted = true; 		// so we'll fall out of the print loop
	fMyStatusWindow = nil;		// window's been freed so forget about it	
}
//TReportStatusWindow

#pragma segment PrintImage
pascal void TReportStatusWindow::IReportStatusWindow(TReportPrinter *aPrinter)
{
	fMyPrinter = aPrinter;
}

#pragma segment PrintImage
pascal void TReportStatusWindow::DoCommandKeyEvent(TToolboxEvent* event)
{	
	if(event->fCharacter == '.') {
		fMyPrinter->UserAbort();	// tell the ReportPrinter to bail out
		this->Close();	
	}
	else
		inherited::DoCommandKeyEvent(event);
}

#pragma segment PrintImage
pascal void TReportStatusWindow::SetMessage(CStr255 theMessage)
{
	FailInfo	fi;
	if(fi.Try()) {
		TStaticText *theText = (TStaticText *) FindSubView(kDocumentNameID);
		theText->SetText(theMessage, true);
		fi.Success();
	}
	else {
		fi.ReSignal();
	}
	
}

