/*********************************************************
 "CSpinner.c"

 by John A. Love, III [Washington Apple Pi Users' Group]

 using Symantec's "THINK C", v 5.0.2
 ... as derived from Gregory H. Dow's CSpinner Class
 ... [THINKin' CaP, Winter 1991]
 *********************************************************/




#include <Global.h>
#include "CSpinner.h"


CSpinner			*A5Spinner = nil;
extern CursHandle	gWatchCursor;



/* =======================
** My spinning CURSor Task
** ======================= */

static pascal void		SpinTheBottle (void)	{

		CursHandle		globe;
		short			globeID;


	asm	{								// PushA5()
		move.l	a5, -(sp)
	}

	asm	{								// GetMyA5()
		// Access my Globals by fetching the value of A5 we've stored
		// before the VBLTask and then put it into A5.
		//
		// We know that when the VBL task is called, A0 will point to it.
		// Therefore, the value of A5 that we stored will be at - 4(A0),
		// which is immediately below the VBL Task in our data block.
		move.l	-4(a0), a5
	}

	globeID = HiWord( (**A5Spinner->acurHdl).cursorIDs[A5Spinner->whichCursor - 1] );
	globe = GetCursor(globeID);
	if (globe != nil)	SetCursor(*globe);

	/* Reset stuff for next time: */
	A5Spinner->whichCursor--;
	if (A5Spinner->whichCursor == 0)
		A5Spinner->whichCursor = (*A5Spinner->acurHdl)->nbrCursors;
	;
	A5Spinner->myVBLRec.MyVBLTask.vblCount = A5Spinner->interval;

	asm	{								// PopA5()
		move.l	(sp)+, a5
	}

}	/* SpinTheBottle */



/* =============================
** Round-and-around she goes ...
** ============================= */

void	CSpinner::ISpinner (short acurID, short delay)		{

		short		vErr;


	acurHdl = (acurHandle) GetResource('acur', acurID);
	if (acurHdl == nil)
	{
		interval = 0;
		SetCursor(*gWatchCursor);
	}
	else
	{
		/* Lock and then stuff me in A5 global space so the
		** spinner function can access the instance variables. */
		MoveHHi(this);
		HLock(this);
		;
		A5Spinner = this;
		
		interval = delay;
		whichCursor = (*acurHdl)->nbrCursors;
		;
		myVBLRec.CurrA5 = (long)CurrentA5;
		myVBLRec.MyVBLTask.vblAddr = &SpinTheBottle;
		myVBLRec.MyVBLTask.vblCount = interval;		/* # times every second. */
		myVBLRec.MyVBLTask.qType = vType;
		myVBLRec.MyVBLTask.vblPhase = 0;
		;
		vErr = VInstall( (QElemPtr)&myVBLRec.MyVBLTask );
		if (vErr != noErr)		SetCursor(*gWatchCursor);

	}	/* else */

}	/* ISpinner */



void	CSpinner::Dispose()		{

		short			ignore;


	if (acurHdl != nil)		{
		ignore = VRemove( (QElemPtr)&myVBLRec.MyVBLTask );
		;
		/* Vice _DisposHandle, so that acurHdl will no longer
		** be recognized as a valid Handle when Dispose()
		** below calls ForgetObject():						  */
		ReleaseResource(acurHdl);
		HUnlock(this);
	}
	
	InitCursor();

	inherited::Dispose();
	A5Spinner = nil;
	
}	/* Dispose */




/* end: "CSpinner.c" */
