/*********************************************************
 "CmyApp.c"

 by John A. Love, III [Washington Apple Pi Users' Group]

 using Symantec's "THINK C", v 5.0.2
 ... as derived from their "TCL Starter" files
 *********************************************************/




#include <CBartender.h>
#include <CCluster.h>
#include <Commands.h>
#include <Constants.h>
#include <CDataFile.h>
#include <CDesktop.h>
#include <CFWDesktop.h>
#include <CList.h>
#include <CWindow.h>

#include "CmyGlobals.h"
#include "CmyApp.h"
#include "CmyBar.h"
#include "CmyMisc.h"
#include "CmyTear.h"
#include "help.h"
#include "CSpinner.h"
#include "CMeter.h"
#include "CQuickTime.h"


#define kExtraMasters		10			// By the book ...
#define kRainyDayFund		60000
#define kCriticalBalance	40000
#define kToolboxBalance		20000
//
#define kHandCursor			3004
#define kCheckCursor		3005
#define kCameraCursor		3006
//
#define kTearMenu			1000
#define cmdProgress			1024
#define cmdNewMovie			cmdNew
#define cmdOpenMovie		cmdOpen
#define cmdDeleteMovie		1025


extern CBartender		*gBartender;
extern CBureaucrat		*gGopher;
extern CClipboard		*gClipboard;
extern CDesktop			*gDesktop;
extern Boolean			gInBackground;
extern OSType			gSignature;
extern CursHandle		gWatchCursor;

CFile					*currentFile;
Boolean					printable;
SFReply					gMacSFReply;
short					movieAppRes;


short CMovie::cFirstMovieEditTaskIndex = 1;		// Initialize the Class Variable.



void	CMovieApp::IMovieApp (void)		{

		Boolean				savedAlloc;
		
		
	CApplication::IApplication(kExtraMasters, kRainyDayFund, 
							   kCriticalBalance, kToolboxBalance);
							   
	if (!gSystem.hasColorQD)
	{
		savedAlloc = SetAllocation(kAllocCanFail);
		gHandCursor = GetCursor(kHandCursor);
		SetAllocation(savedAlloc);
		FailNILRes(gHandCursor);
		HNoPurge(gHandCursor);
		;
		savedAlloc = SetAllocation(kAllocCanFail);
		gCheckCursor = GetCursor(kCheckCursor);
		SetAllocation(savedAlloc);
		FailNILRes(gCheckCursor);
		HNoPurge(gCheckCursor);
		;
		savedAlloc = SetAllocation(kAllocCanFail);
		gCameraCursor = GetCursor(kCameraCursor);
		SetAllocation(savedAlloc);
		FailNILRes(gCameraCursor);
		HNoPurge(gCameraCursor);
		;
		niftyHandCRSR = nil;
		niftyCheckCRSR = nil;
		niftyCameraCRSR = nil;
	}
	
	else
	{
		savedAlloc = SetAllocation(kAllocCanFail);
		niftyHandCRSR = GetCCursor(kHandCursor);
		SetAllocation(savedAlloc);
		FailNILRes(niftyHandCRSR);
		HPurge(niftyHandCRSR);						// See IM 5.
		;
		savedAlloc = SetAllocation(kAllocCanFail);
		niftyCheckCRSR = GetCCursor(kCheckCursor);
		SetAllocation(savedAlloc);
		FailNILRes(niftyCheckCRSR);
		HPurge(niftyCheckCRSR);
		;
		savedAlloc = SetAllocation(kAllocCanFail);
		niftyCameraCRSR = GetCCursor(kCameraCursor);
		SetAllocation(savedAlloc);
		FailNILRes(niftyCameraCRSR);
		HPurge(niftyCameraCRSR);
		
	}	/* color QuickDraw */
	    
}	/* IMovieApp */



void	CMovieApp::InitToolbox (void)		{

		OSErr	err = noErr;


	inherited::InitToolbox();
	
	if (QuickTimeIsInstalled())		err = EnterMovies();
	if (err != noErr)	FailMemError();
	
}	/* InitToolbox */



void	CMovieApp::MakeClipboard (void)	{


	gClipboard = new (CMovieClip);
	gClipboard->IClipboard(this, TRUE);

}	/* MakeClipboard */



static pascal Boolean	myFileFilter (ParmBlkPtr PB)	{
/* File Filter so we don't display Movies that are already open. */

		Boolean		displayIt = false, dontDisplayIt = true;
		Byte		attribute = PB->fileParam.ioFlAttrib;
		
	
	if ( BitTst((Ptr) &attribute, 0) /* File is open */ )
			return	(dontDisplayIt);
	else	return	(displayIt);
	
}	/* myFileFilter */



void	CMovieApp::SetUpFileParameters (void)		{

		#define useMyFilter		true
		FileFilterProcPtr		stdFileFilter = nil;


	inherited::SetUpFileParameters();	// Be sure to call the default method.

	sfNumTypes	   = 1;
	sfFileTypes[0] = 'MooV';
	sfFileFilter   = (useMyFilter) ?
							(FileFilterProcPtr) &myFileFilter : stdFileFilter;

	gSignature = 'J3';
	
}	/* SetUpFileParameters */



void	CMovieApp::MemoryShortage (Size bytesNeeded)	{


	if ( QuickTimeIsInstalled() )		FailOSErr(kGeneralQuickTimeMemoryError);
	else								inherited::MemoryShortage(bytesNeeded);

}	/* MemoryShortage */



void	CMovieApp::StartUpAction (short numPreloads)	{

		CSpiffyGraphics		*spiffy = nil;
		CRemote				*remote = nil;
		

	SetCursor(*gWatchCursor);	// Various CURSor handling routines will reset this.

	gStripAddressMask = GetStripAddressMask();
	;
	gStartBalloons = BalloonsOn();
	gLastBalloon = -1;							/* = notMine. */
	;
	/* TCL's Method, DispatchCursor, eventually calls
	** _HMGetIndHelpMsg to display the appropriate
	** Balloons for each pane based on the app's attached
	** 'hrct' resource ... however, the latter Trap calls
	** _Get1Resource and if there's a Movie file open,
	** we'll end up searching just this Movie file and
	** nothing else. As a direct result, a "resNotFound"
	** error will be generated because said Movie file
	** doesn't have an attached 'hrct' resource, naturally.
	** To overcome this BIG-time boo-boo, I call
	** UseResFile(movieAppRes) within CMovieDesktop's method
	** = DispatchCursor.  Luckily, QuickTime accesses its
	** own Balloon Help and doesn't call Get1Resource.  In
	** short, once I call UseResFile for any of my panes, the
	** same call sufficies for all ... otherwise, of course,
	** I'd need to override DispatchCursor for all panes.
	**
	**    Why-Oh-Why did Symantec & Apple do this to me ?!*!?    */
	
	movieAppRes = CurResFile();

	if (gSystem.hasHelpMgr)		/* It's System 7 allright !!! */
	{
		TRY
		{
			spiffy = new (CSpiffyGraphics);
			spiffy->ISystem7Braggart(this);
			spiffy->DisplaySystem7Braggart();
		}
		CATCH
		{
			SysBeep(10);
			if (spiffy)
				spiffy->DisposeSystem7Braggart();		// Calls ForgetObject(spiffy).
		}
		ENDTRY;
		
		if (spiffy)		spiffy->DisposeSystem7Braggart();
	
	}	/* System 7 */
	
	if ( QuickTimeIsInstalled () )
	{
		TRY
		{
			remote = new (CRemote);
			remote->IRemote(this);
			remote->itsWindow->Select();
			remote->itsWindow->Prepare();
		}
		CATCH
		{
			SysBeep(10);
			if (remote)		remote->Dispose();			// Calls ForgetObject(remote).
		}
		ENDTRY;
	
	}	/* QuickTimeIsInstalled */
	
	inherited::StartUpAction(numPreloads);
	
	SysBeep(10);

}	/* StartUpAction */



void	CMovieApp::Idle (EventRecord *macEvent)	{
		
		
	gBalloonsUp = BalloonsOn();
	
	/* As much as I love "Switch-A-Roo", using it with
	** Balloons showing drives me positively (more) crazy !!!
	**
	** By setting the two class variables below = NULL,
	** CView::DispatchCursor will re-show the daggum Balloons: */
	
	if ( EventAvail(updateMask, macEvent) )
	{
		if (gBalloonsUp)
		{
			CView::cCurrHelpView = NULL;
			CView::cLastHelpView = NULL;
		}
	}

	// Sends Dawdle messages to each Gopher in the Chain of Command:
	inherited::Idle(macEvent);
	
}	/* Idle */



void	CMovieApp::MakeBartender (void)	{


	gBartender = new (CmyBar);
	gBartender->IBartender(MBARapp);

}	/* MakeBartender */



void	CMovieApp::SetUpMenus (void)	{

		CTearMenuDir	*cryBaby = nil;
		MenuHandle		kApple;
		short			kAboutItem = 1, none = 0, whoCares = 0;
		
		
	cryBaby = new (CTearMenuDir);	/* Director for a tear-off Menu. */
	cryBaby->ITearMenuDir(this);
	itsTearMenu = cryBaby;
	;
	inherited::SetUpMenus();		/* Superclass takes care of adding menus
                      				** specified in a MBAR id = 1 resource.	*/                          

	/* My attached Apple MENU resource has a SICN plugged into
	** the "About ..." item for System 7.  Therefore, when
	** operating under System 6, it must be removed:			*/

	if (!gSystem.hasHelpMgr)	{
		kApple = GetMHandle(MENUapple);
		SetItemIcon(kApple, kAboutItem, none);
		SetItemCmd(kApple, kAboutItem, (char)none);
	}

	/* Add item to Help Menu "on-the-fly".  This means that my
	** app's "DoCommand" will be passed a negative command #.
	** See comments within my "CmyBar::FindCmdNumber" routine. */
	if (HelpManagerActive())	{
		if (HMGetHelpMenuHandle(&gHelpMenu) == noErr)
			if (gHelpMenu != nil)	{
				/* Original count = 4, NOT 3 because the Help Manager adds
				** a disabled dotted line before any items we may add: */
				gOrigNumHelpItems = CountMItems(gHelpMenu);
				AppendMenu(gHelpMenu, "\pAbout my application ...");
			}
	}
	
}	/* SetUpMenus */



void	CMovieApp::UpdateMenus (void)		{
/* If there is a doc staring at us, our apps UpdateMenus gets called
** by CDocument::UpdateMenus when it calls CBureaucrat::UpdateMenus.
** The latter calls itsSupervisor->UpdateMenus where itsSupervisor is
** our app.  If there is NO doc, i.e. window, then we get called directly. */
 
		WindowPeek				top;
		
		
	inherited::UpdateMenus();     // Enable standard commands.
        
    if (!gInBackground)		{
    
    	top = (WindowPeek) FrontWindow();

		if ( aDAisUp(top) )		{
			/* For System 6, even if a DA is on top, our application may still be
			** in the foreground.  See comments within my SwitchFrom/ToDA routines. */
			gBartender->DisableCmd(cmdAbout);
			;
		 /* The DA may NOT do its own thing ... */
			gBartender->SetCmdText(cmdUndo,  "\pUndo");
			gBartender->SetCmdText(cmdCut,   "\pCut");
			gBartender->SetCmdText(cmdCopy,  "\pCopy");
			gBartender->SetCmdText(cmdPaste, "\pPaste");
			gBartender->SetCmdText(cmdClear, "\pClear");
		}
		
		else	{
			/* Zip windows or the Clipboard window or a Movie window on
			** top.  For the latter case, my CQuickTime::UpdateMenus will
			** subsequently change the appropriate Menu item text to what
			** it should be:											  */
			
			/* For the "About..." item, we must specifically re-enable once
			** we disable because TCL assumes dimNONE for the Apple Menu. */
			gBartender->EnableCmd(cmdAbout);
			gBartender->EnableCmd(cmdProgress);
			
			if ( !QuickTimeIsInstalled() )		{
				/* This UNdoes what CApplication::UpdateMenus may
				** have done since cmdNewMovie = TCL's cmdNew:	  */
				gBartender->SetCmdText(cmdNewMovie,
									   "\pSorry ... QuickTime not installed !!!");
				gBartender->DisableCmd(cmdNewMovie);
			}
			else
				gBartender->EnableCmd(cmdDeleteMovie);
				
			// Can open something, QuickTime or not:
			gBartender->EnableCmd(cmdOpenMovie);
			
			// A spiffy addition:
			if (top != nil)		{
				/* Either the Clipboard window or my dialogs for
				** going to/selecting a frame or a Movie window.
				** See note above for the latter case:			*/
				gBartender->SetCmdText(cmdClose, "\pClose");
				;
				gBartender->SetCmdText(cmdUndo, "\pUndo");
				gBartender->SetCmdText(cmdCut, "\pCut");
				gBartender->SetCmdText(cmdCopy, "\pCopy");
				gBartender->SetCmdText(cmdPaste, "\pPaste");
				gBartender->SetCmdText(cmdClear, "\pClear");
			}
			else	{
				gBartender->SetCmdText(cmdClose, "\pNo window to close");
				;
				gBartender->SetCmdText(cmdUndo,  "\pNo window to edit");
				gBartender->SetCmdText(cmdCut,   "\pNo window to edit");
				gBartender->SetCmdText(cmdCopy,  "\pNo window to edit");
				gBartender->SetCmdText(cmdPaste, "\pNo window to edit");
				gBartender->SetCmdText(cmdClear, "\pNo window to edit");
			}
			
		}	/* No DA is showing on top */
		
    	/* Switch-A-Roo problem: it indiscriminantly
    	** removes all user-added Help MENU items:	 */
		if (HelpManagerActive())	{
			if (HMGetHelpMenuHandle(&gHelpMenu) == noErr)
				if (gHelpMenu != nil)
					if (CountMItems(gHelpMenu) == gOrigNumHelpItems)
						AppendMenu(gHelpMenu, "\pAbout my application ...");
		}

	}	/* NOT in background */
				
}	/* UpdateMenus */



void	CMovieApp::SwitchToDA (void)	{
		
		
	/* This is wrong because CBartender::UpdateAllMenus
	** disables all MENU items anyway.  The place to disable any
	** of my individual items is in CMovieApp::UpdateMenus().	  */

 // gBartender->DisableCmd(cmdProgress);

	/* These paired Switch routines are the place for
	** disabling and enabling WHOLE Menus only:		  */
	
	gBartender->DisableMenu(kTearMenu);
	
	/* The inherited versions call Suspend/Resume AND toggle
	** gInBackground accordingly.  This is patently wrong
	** because the main Event loop addresses these Multifinder
	** app4Evts anyway and the latter are totally separate from
	** going back and forth to DAs.  This is especially true
	** when using Unifinder with System 6:						*/
	
 // inherited::SwitchToDA();
	
}	/* SwitchToDA */
 


void	CMovieApp::SwitchFromDA (void)		{


	gBartender->EnableMenu(kTearMenu);

 // inherited::SwitchFromDA();
	
}	/* SwitchFromDA */



static long		GetPercent (void)		{

		long	total, sofar;
		
		
	total = ((CDataFile*)currentFile)->GetLength();
	sofar = ((CDataFile*)currentFile)->GetMark();
	
	return (sofar/total);

}	/* GetPercent */



static long		calcProgress (void)		{
/* Here's where you determine the % progress you've made, say in
** spooling a file to the printer.  For now, however, let's force it,
** so much force as a matter of fact it's really kludgy:			 */

		static long		percent = 0;
		long			finalTicks;
		
		
	if (percent > 100)		percent = 0;	/* A kludgy way to re-cycle */
	percent++;
	Delay(5, &finalTicks);
	return (percent);			// return (GetPercent());
	
}	/* calcProgress */
 


static pascal short		DeleteDlgHook (short theItem, DialogPtr theDialog)		{
/* A temporary Dialog Hook just to change "Open" to "Delete": */

		short		OpenButtonItem = 1, itemType;
		Handle		itemHandle;
		Rect		itemRect;
		Boolean		fileSelected, folderSelected;
		Str255		buttonName;
		
	
	switch	(theItem)	{
	
		case	sfHookNullEvent:
		
			fileSelected = Length(gMacSFReply.fName) != 0;
			folderSelected = gMacSFReply.fType != 0;
			;
			GetDItem(theDialog, OpenButtonItem, &itemType, &itemHandle, &itemRect);
			GetCTitle((ControlHandle)itemHandle, &buttonName);
			
			if (!folderSelected && !fileSelected)	{
				if (buttonName[1] != 'N')
					SetCTitle((ControlHandle)itemHandle, "\pNada !!");
			}
			
			else if (fileSelected)	{
				if (buttonName[1] != 'D')
					SetCTitle((ControlHandle)itemHandle, "\pDelete");
			}
			
			else if (folderSelected)	{
				if (buttonName[1] != 'O')
					SetCTitle((ControlHandle)itemHandle, "\pOpen");
			}
			
			break;
		
		default:
			break;
			
	}
	
	return	(theItem);
	
}	/* DeleteDlgHook */



void	CMovieApp::DoCommand (long theCommand)		{

	 // #define cmdNewMovie		cmdNew		--		Handled by CreateDocument.
	 // #define cmdOpenMovie	cmdOpen		--		Handled by OpenDocument.
		
		GrafPtr					savePort;
		Str255					theDAName;
		CSpiffyGraphics			*spiffy		   = nil;
		CSpinner				*roundAndRound = nil;
		short					kMeter		   = 111;
		CMeter					*meterDoc	   = nil;
		CMovieFile				*movieFile	   = nil;
		StandardFileReply		newReply;
		DlgHookProcPtr			saveHook;


	if (theCommand < 0)		{
	
		if (HiShort(-theCommand) == MENUapple)		{
			/* BIG-time TCL boo-boo for not saving & restoring the Port: */
			GetPort(&savePort);
			GetItem(GetMHandle(MENUapple), LoShort(-theCommand), theDAName);
			OpenDeskAcc(theDAName);
			SetPort(savePort);
		}
		
		else if (HiShort(theCommand) == kHMHelpMenuID)		{
		/* For the Help MENU, theCommand is negative because
		** the menu id in its High word is negative (= -16490). */
		
			/* Although the Help Manager adds not only your item
			** but a disabled dotted line before your addition,
			** we still only add 1 ?????						*/
			if (LoShort(theCommand) == gOrigNumHelpItems + 1)		{
				TRY
				{
					roundAndRound = new(CSpinner);
					roundAndRound->ISpinner(acurWorld, 10);
				}
				CATCH
				{
					if (roundAndRound)		roundAndRound->Dispose();
					SetCursor(*gWatchCursor);
				}
				ENDTRY;
				;
				TRY
				{
					spiffy = new (CSpiffyGraphics);
					spiffy->ISpiffyScroll(this);
					spiffy->DisplaySpiffyScroll();
				}
				CATCH
				{
					SysBeep(10);
					if (spiffy)		spiffy->DisposeSpiffyScroll();
				}
				ENDTRY;
				;
				if (spiffy)		spiffy->DisposeSpiffyScroll();
				if (roundAndRound == nil)		InitCursor();
				else							roundAndRound->Dispose();
			}
			else	{
				/* etc. for other Help Menu items */
			}
							
		}	/* end: kHMHelpMenuID */
		
		else if (HiShort(-theCommand) == kTearMenu)		{
		
				enum	{OFF, ONApple = MENUapple};
		
			gDesktop->Prepare();		// ??????????
			;
			HiliteMenu(ONApple);
			DoCommand(cmdAbout);
			HiliteMenu(OFF);
		}
		
		else	inherited::DoCommand(theCommand);
		
	}	/* end: theCommand < 0 */
	
	else	{							/* theCommand > 0 */
		switch (theCommand) {
	
			case cmdAbout:
				TRY
				{
					roundAndRound = new(CSpinner);
					roundAndRound->ISpinner(acurDogCow, 10);
				}
				CATCH
				{
					if (roundAndRound)		roundAndRound->Dispose();
					SetCursor(*gWatchCursor);
				}
				ENDTRY;
				;
				TRY
				{
					spiffy = new (CSpiffyGraphics);
					spiffy->ISpiffyIris(this);
					spiffy->DisplaySpiffyIris();
				}
				CATCH
				{
					SysBeep(10);
					if (spiffy)		spiffy->DisposeSpiffyIris();
				}
				ENDTRY;
				;
				if (spiffy)		spiffy->DisposeSpiffyIris();
				if (roundAndRound == nil)		InitCursor();
				else							roundAndRound->Dispose();
				break;	/* cmdAbout */
			
			case cmdProgress:
				currentFile = ((CDocument*)gGopher)->itsFile;
				printable = ((CDocument*)gGopher)->itsPrinter != nil;
			 // if ( !printable || (currentFile == nil) )		return;
				
				SetCursor(*gWatchCursor);
				;
				TRY
				{
					meterDoc = new (CMeter);
					meterDoc->IMeter(kMeter,					// WIND ID
									 (LongFunc) &calcProgress,	// Points to function
									 "\pWatch the spiffy fireworks !!",		// title
									 100,						// maxSteps
									 this);						// Supervisor = app
					meterDoc->itsWindow->Select();
					meterDoc->StartProgress();
					meterDoc->itsWindow->Update();
					meterDoc->UpdateProgress();
				}
				CATCH
				{
					SysBeep(10);
					if (meterDoc)		meterDoc->Dispose();
				}
				ENDTRY;
				;
				InitCursor();
				break;	/* cmdProgress */
			
			case cmdDeleteMovie:
				movieFile = new (CMovieFile);
				movieFile->IMovieFile();
				
				saveHook = sfGetDLOGHook;	// Restore below for "Open Movie".
				sfGetDLOGHook = (DlgHookProcPtr) &DeleteDlgHook;
				;
				ChooseFile(&gMacSFReply);
				if (gMacSFReply.good)	{
					ConvertOldToNew(&gMacSFReply, &newReply);
					movieFile->SpecifyFSSpec(&newReply.sfFile);
					movieFile->ThrowOut();
				}
				movieFile->Dispose();		// Dump temporary object.
				;
				sfGetDLOGHook = saveHook;
				break;	/* cmdDeleteMovie */

			default:
				inherited::DoCommand(theCommand);
				
		}	/* end: switch */
	}	/* end: theCommand > 0 */
			
}	/* DoCommand */



void	CMovieApp::DoKeyDown (char theChar, Byte keyCode, EventRecord *macEvent)	{

		enum			{OFF, ONApple = MENUapple, ONHelp = kHMHelpMenuID};
		long			finalTicks, myHelpCommand;
		WindowPeek		tearPeek;
		Point			tlWind, defaultPt = {100, 100};
		Rect			desk;
		CWindow			*theWindow;
	
	
	if (macEvent->modifiers & cmdKey)
	{
		if (theChar == '/')
		{
			HiliteMenu(ONApple);
			DoCommand(cmdAbout);
			HiliteMenu(OFF);
		}	/* CMD-? */
		
		else if (keyCode == 0x0D /* = 'w', regardless of modifier key */)
		{
			if (macEvent->modifiers & optionKey)	// = <CMD-Option> w
			{
				theWindow = gDesktop->GetTopWindow();
				while (theWindow != nil)
				{
					theWindow->Close();
					theWindow = gDesktop->GetTopWindow();
				}
			}
		}
		
		else if (keyCode == 0x30 || keyCode == 0x48)	/* <Tab> */
		{
			if (itsTearMenu->corner.h == 0 && itsTearMenu->corner.v == 0)
				// NEVER torn-off before:
				itsTearMenu->TornOff(defaultPt);
			
			else
			{
				// Toggle between showing & hiding:
				tearPeek = (WindowPeek) itsTearMenu->GetMacWindow();
				tlWind = topLeft( (**(tearPeek->contRgn)).rgnBBox );
				((CFWDesktop*)gDesktop)->GetBounds(&desk);
				
				if (tlWind.v > desk.bottom && tlWind.h > desk.right)
					/* Torn-off before, BUT subsequently closed because
					** CTearOffMenu::CloseWind calls CWindow::MoveOffScreen.
					** Put window back where it was BEFORE you closed it: */
					itsTearMenu->TornOff(itsTearMenu->corner);
				else
					itsTearMenu->CloseWind(itsTearMenu->itsWindow);
			}
			
		}	/* CMD-Tab */
		
	}	/* CMD-key pressed */
	
	else if (keyCode == KeyHelp)
	{
		if (HelpManagerActive())
		{
		 // For some reason, System 7 does NOT keep title highlighted:
			HiliteMenu(ONHelp);
			Delay(20, &finalTicks);
			HiliteMenu(OFF);
			myHelpCommand = ((long)ONHelp << 16) + gOrigNumHelpItems + 1;
			DoCommand(myHelpCommand);
		}
		
		else
		{
			HiliteMenu(ONApple);
			DoCommand(cmdAbout);
			HiliteMenu(OFF);
		}
		
	}	/* Help Key */
	
	else	inherited::DoKeyDown(theChar, keyCode, macEvent);
	
}	/* DoKeyDown */



void	CMovieApp::Exit (void)	{

	/* your exit handler here */
	
	if ( QuickTimeIsInstalled() )
	{	
		ExitMovies();
	}
	;
	if (niftyHandCRSR)
	{
		LoadResource(niftyHandCRSR);
		DisposCCursor(niftyHandCRSR);
	}
	;
	if (niftyCheckCRSR)
	{
		LoadResource(niftyCheckCRSR);
		DisposCCursor(niftyCheckCRSR);
	}
	;
	if (niftyCameraCRSR)
	{
		LoadResource(niftyCameraCRSR);
		DisposCCursor(niftyCameraCRSR);
	}

	ResetBalloons(gStartBalloons);

	PlaySound("\pMoof");
	
}	/* Exit */



/*
**	CreateDocument
**
**		The user chose New from the File menu.
*/

void	CMovieApp::CreateDocument (void)		{

		CQuickTime		*newStuff = NULL;
						
	
	/* Need this here because upon launching my application, the TCL sends
	** a DoCommand(cmdNew) message to CmyApp to open up a blank document:	*/
	
	if ( !QuickTimeIsInstalled() )		return;
	
	TRY
	{
		newStuff = new (CQuickTime);
		newStuff->INewQuickTime(this);
		newStuff->DisplayQuickTime();
	}
	CATCH
	{
		SysBeep(10);
		if (newStuff)		newStuff->Dispose();
	}
	ENDTRY;
				
}	/* CreateDocument */



void	CMovieApp::ChooseFile (SFReply *macSFReply)	{
/* Prompt the user to select a file.  The TCL passes the standard SFReply
** to CAplication's ChooseFile and OpenDocument, so I canNOT take advantage
** of System 7's StandardFileReply.
**
**			... at least I haven't figured how to ... YET !!!
**
** When opening an existing document, CApplication::DoCommand calls:
**
**		ChooseFile(&macSFReply);
**		if (macSFReply.good)	{
**			SetCursor(*gWatchCursor);
**			OpenDocument(&macSFReply);
**		}
**
** If QuickTime is NOT installed, I still need access to my
** OpenDocument method to display a special ALeRT.							*/

		Point		centerDLOG	  = {-1, -1};
		Str255		prompt		  = "\pPlease select a Movie file:";
		Boolean		wasLocked;


	if ( !QuickTimeIsInstalled() )		{
		macSFReply->good = true;
	}
	else	{
		wasLocked = Lock(TRUE);
		SFGetFilePreview(centerDLOG, prompt, sfFileFilter, sfNumTypes,
						 sfFileTypes, sfGetDLOGHook, macSFReply);
		Lock(wasLocked);
		
		if ( !macSFReply->good )
			SysBeep(10);

		CView::cCurrHelpView = NULL;
		CView::cLastHelpView = NULL;
	}
	
}	/* ChooseFile */



/*
**	OpenDocument
**
**		The user chose Open from the File menu.
**
**		The macSFReply is a good SFReply record that contains
**		the name and vRefNum of the file the user chose to open.
*/
 
void	CMovieApp::OpenDocument (SFReply *macSFReply)		{


		#define kNotQuickTime	3003
		CQuickTime				*forShame = NULL, *greatStuff = NULL;


	if (!QuickTimeIsInstalled())	{
				
		TRY
		{
			forShame = new (CQuickTime);
			forShame->INoQuickTime(kNotQuickTime, this);
			forShame->DisplayNoQuickTime();
		}
		CATCH
		{
			SysBeep(10);
			if (forShame)	forShame->Dispose();	// Calls ForgetObject(forShame):
		}
		ENDTRY;
					
		if (forShame)	forShame->Dispose();
	}
			
	else	{
						
		TRY
		{
			greatStuff = new (CQuickTime);
			greatStuff->IQuickTime(this);
			greatStuff->itsMovie->OpenMovie(macSFReply);
			greatStuff->DisplayQuickTime();
			greatStuff->itsMovie->PlayMovie();
		}
		CATCH
		{
			SysBeep(10);
			if (greatStuff)		greatStuff->Dispose();
		}
		ENDTRY;
	}
	
}	/* OpenDocument */



void	CMovieApp::MakeDesktop (void)	{

	
	gDesktop = new (CMovieDesktop);
	((CMovieDesktop*)gDesktop)->IMovieDesktop(this);
	
}	/* MakeDesktop */



	static void		Window_ShowHide (CWindow *theWindow, long showFlag)		{


		if (theWindow->active)		{
			if (showFlag)	theWindow->ShowResume();
			else			theWindow->HideSuspend();
		}
		
	}	/* Window_ShowHide */



void	CMovieApp::Suspend (void)	{
		
		
	inherited::Suspend();

	gSaveBalloons = gBalloonsUp;		// Where we're at !!!
	ResetBalloons(gStartBalloons);		// On Day Numero Uno !!!

	((CMovieDesktop*)gDesktop)->CalcTopFloat();
	if ( ((CMovieDesktop*)gDesktop)->topFloat )
    	((CMovieDesktop*)gDesktop)->itsFloats->
    							DoForEach1((EachFunc1) Window_ShowHide, (long)false);
	
}	/* Suspend */



void	CMovieApp::Resume (void)	{


	inherited::Resume();

	ResetBalloons(gSaveBalloons);
	gBalloonsUp = gSaveBalloons;

	if ( ((CMovieDesktop*)gDesktop)->topFloat )
    	((CMovieDesktop*)gDesktop)->itsFloats->
    							DoForEach1((EachFunc1) Window_ShowHide, (long)true);
	((CMovieDesktop*)gDesktop)->CalcTopFloat();
	
}	/* Resume */




/* end: "CmyApp.c" */