/**************************************************
 "dissMask.c"

 by:	Michael S. Morton [MacTutor, December 1990]
 **************************************************/




#include "dissMask.h"



static void		round2 (register short *i)		{
/* round-up to the next power of 2: */

		register short		result = 1;
		
		
	while (result < *i)		result <<= 1;
	
	*i = result;

}	/* round2 */



static unsigned long	seqMasks [ /* 0...32 */ ] =
{
	/* Masks to generate the pseudo-random sequence.
	** The table runs 0...32, but 0...1 are invalid: */
	
	0x0,		0x0,		0x03,		0x06,
	0x0C,		0x14,		0x30,		0x60,
	0xB8,		0x0110,		0x0240,		0x0500,
	0x0CA0,		0x1B00,		0x3500,		0x6000,
	0xB400,		0x00012000, 0x00020400, 0x00072000,
	0x00090000, 0x00140000, 0x00300000, 0x00400000,
	0x00D80000, 0x01200000, 0x03880000, 0x07200000,
	0x09000000, 0x14000000, 0x32800000, 0x48000000,
	0xA3000000
};	/* seqMask */



Boolean		dissMaskInit (Rect *srcRect, register dissMaskInfo *info)	{
/* INPUT:	bounds of source rectangle
** OUTPUT:	"state-of-the-world" for mask */

		register short				log2 = 0;
		register unsigned long		ct;
		Rect						offRect;
		short						offRowBytes;
		Size						sizeOfOff;
		Handle						myBits;
		
	
	// Copy source rect and then rormalize it to {0,0}:
	offRect = *srcRect;
	OffsetRect(&offRect, -offRect.left, -offRect.top);
	
	/* Round bounds rect up to next power of 2.  This speeds up
	** the dissolve considerably by removing bounds-checking.
	** Also, ensure that the width of the BitMap is a multiple
	** of 4 bytes per "Inside Macintosh":						*/
	
	round2(&offRect.bottom);
	round2(&offRect.right);
	;
	offRowBytes = (offRect.right + 15) / 16;	/* # of words.	  */
	if (offRowBytes % 2)	offRowBytes++;		/* ... made even. */
	offRowBytes *= 2;							/* Back to bytes. */
	sizeOfOff = (long) offRect.bottom * offRowBytes;
	;
	offRect.right = offRowBytes * 8;			/* -> Pixels.	  */
	info->maskRect = offRect;
	info->maskMap.bounds = offRect;
	
	// Finish filling-in pixMap:
	
	info->maskMap.rowBytes = offRowBytes;
	myBits = NewHandleClear(sizeOfOff);
	if (MemError() != noErr)	return	(false);
	else						info->maskMap.baseAddr = *myBits;
	
	/* Compute total number of pixels in the mask BitMap
	** and initialize the countdown counter:			 */
	
	info->pixLeft = (long) offRect.right * offRect.bottom;
	ct = info->pixLeft;
	while (ct > 1)
		// until log2(ct) == 0:
		{ ct >>= 1;  ++log2; }		/* ... shift down one, bump log2 */
	;
	if (log2 < 2 || log2 > 32) /* outside of table bounds */	return	(false);
	;
	// Set up mask which generates cycle of length = 2**log2:
	info->seqMask = seqMasks [log2];
	
	/* Because we count iterations, we need not watch
	** the sequence element -- it can start anywhere: */
	
	info->seqElement = 1;	// Any non-zero value.
	
	--info->pixLeft;		// Kludge: one element done outside loop.
	
	return	(true);
	
}	/* dissMaskInit */



void	dissMaskNext (register dissMaskInfo *info, register unsigned long steps)	{
/* INPUT:	# of steps to take
** OUTPUT:	"state-of-the-world" for mask */

		register unsigned long		element, mask;		// For use in asm code ...
		register void				*baseAddr;
		
		
	if (steps == 0)		steps = 1;		// Natch !!!
	if (steps > info->pixLeft) /* more steps than we need */	steps = info->pixLeft;
	info->pixLeft -= steps;
	
	element = info->seqElement;
	mask = info->seqMask;
	baseAddr = info->maskMap.baseAddr;
	
	// If all the arithmetic can be done in 16 bits, do so:
	
	steps--;	// For "dbra".
	
	if ( (info->seqMask & 0xFFFF) == info->seqMask )
		asm
		{
			@loopStart16:	/* Set bit for current sequence element: */
							move.w	element, d0			// Copy bit #.
							move.b	d0, d1
							lsr.w	#3, d0				// Bit -> byte.
							bset	d1, 0(baseAddr, d0.w)
							
							/* Advance to next sequence element.
							** If element = 1, we're finished:	*/
							lsr.w	#1, element			// Throw out a bit.
							bcc.s	@skipXOR16
							eor.w	mask, element		// Flip mask bits.
							
			@skipXOR16:		dbra	steps, @loopStart16
		}
		
	else
		asm
		{
			@loopStart32:	move.l	element, d0
							move.b	d0, d1
							lsr.l	#3, d0
							bset	d1, 0(baseAddr, d0.l)
							
							lsr.l	#1, element
							bcc.s	@skipXOR32
							eor.l	mask, element
							
			@skipXOR32:		dbra	steps, @loopStart32
			
							sub.l	#0x00010000, steps
							bpl.s	@loopStart32
		}
	
	info->seqElement = element;		// Update from asm-loop.
	
	if (!info->pixLeft)		/* All general pixels are copied. */
		asm
		{
			// Special case: element #0 never comes up, so set bit #0 in byte #0:
			bset	#0, (baseAddr)
		}
		
}	/* dissMaskNext */



void	dissMaskFinish (register dissMaskInfo *info)	{
/* Clean up: */

		Handle		h;
		
		
	h = RecoverHandle(info->maskMap.baseAddr);
	DisposHandle(h);
	;
	info->maskMap.baseAddr = nil;		// Mark as gone.
	
}	/* dissMaskFinish */



void	dissolve (BitMap *srcBits, BitMap *dstBits,
				  Rect *srcRect, Rect *dstRect, unsigned short steps)	{
				  
			dissMaskInfo	info;
			unsigned long	pixPerStep;
			
			
	if (!dissMaskInit(srcRect, &info) )
		CopyBits(srcBits, dstBits, srcRect, dstRect, srcCopy, nil);
		
	else	{
		if (steps == 0)		steps = 1;
		pixPerStep = (info.pixLeft / steps) + 1;
		
		HideCursor();
		;
		while (info.pixLeft)	{
			dissMaskNext(&info, pixPerStep);
			CopyMask(srcBits, &info.maskMap, dstBits,
					 srcRect, &info.maskRect, dstRect);
		}
		dissMaskFinish(&info);
		;
		ShowCursor();
	}	/* else */
	
}	/* dissolve */




/*	{ end file "dissMask.c" }  */
