/********************************/
/*								*/
/*  DLOGFilter Code				*/
/*  							*/
/*	Copyright 1993, Dave Mark 	*/
/*	Do not duplicate, All 	 	*/
/*	rights reserved.	 	 	*/
/*								*/	
/********************************/

#define kDialogResID		128
#define kMBARid				128
#define kMessageAlertID		129

#define kSleep				60L
#define kMoveToFront		(WindowPtr)-1L
#define kNULLFilterProc		(ProcPtr)0L

#define kOn					1
#define kOff				0

#define kEditItemExists		true
#define kEventNotHandledYet	false
#define kEventHandled		true

#define kMaxFieldLength		10

#define kEnterKey			3
#define kBackSpaceKey		8
#define kTabKey				9
#define kReturnKey			13
#define kEscapeKey			27
#define kLeftArrow			28
#define kRightArrow			29
#define kUpArrow			30
#define kDownArrow			31
#define kPeriodKey			46
#define kDeleteKey			127

#define iTenCharMaxText		4
#define iNumberText			6

#define mApple				128
#define iAbout				1

#define mFile				129
#define iDialog				1
#define iQuit				3


/*************/
/*  Globals  */
/*************/

Boolean		gDone;


/***************/
/*  Functions  */
/***************/

void			ToolboxInit( void );
void			MenuBarInit( void );
void			EventLoop( void );
void			DoEvent( EventRecord *eventPtr );
void			HandleMouseDown( EventRecord *eventPtr );
void			HandleMenuChoice( long menuChoice );
void			HandleAppleChoice( short item );
void			HandleFileChoice( short item );
void			DoDialog( void );
pascal	Boolean	DLOGFilter( DialogPtr dialog, EventRecord *eventPtr, short *itemHitPtr );
Boolean			ScrapIsOnlyDigits( void );
Boolean			CallFilterProc( DialogPtr dialog, EventRecord *eventPtr, short *itemHitPtr );
short			CurEditField( DialogPtr dialog );
short			SelectionLength( DialogPtr dialog );
void			Message( Str255 messageStr );

/* see tech note 304 */
pascal OSErr SetDialogDefaultItem( DialogPtr theDialog, short newItem ) 
    = { 0x303C, 0x0304, 0xAA68 };        
pascal OSErr SetDialogCancelItem( DialogPtr theDialog, short newItem )
    = { 0x303C, 0x0305, 0xAA68 };    
pascal OSErr SetDialogTrackCursor( DialogPtr theDialog, Boolean tracks )
    = { 0x303C, 0x0306, 0xAA68 };
pascal OSErr GetStdFilterProc( ModalFilterProcPtr *theProc )
	= { 0x303C, 0x0203, 0xAA68 };


/************************************* main */

void	main( void )
{
	ToolboxInit();
	MenuBarInit();
	
	EventLoop();
}


/************************************* ToolboxInit */

void	ToolboxInit( void )
{
	InitGraf( &thePort );
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs( NULL );
	InitCursor();
}


/************************************* MenuBarInit */

void	MenuBarInit( void )
{
	Handle			menuBar;
	MenuHandle		menu;
	
	menuBar = GetNewMBar( kMBARid );
	SetMenuBar( menuBar );

	menu = GetMHandle( mApple );
	AddResMenu( menu, 'DRVR' );
	
	DrawMenuBar();
}


/************************************* EventLoop */

void	EventLoop( void )
{		
	EventRecord		event;
	
	gDone = false;
	while ( gDone == false )
	{
		if ( WaitNextEvent( everyEvent, &event, kSleep, NULL ) )
			DoEvent( &event );
	}
}


/************************************* DoEvent */

void	DoEvent( EventRecord *eventPtr )
{
	char	theChar;
	
	switch ( eventPtr->what )
	{
		case mouseDown: 
			HandleMouseDown( eventPtr );
			break;
		case keyDown:
		case autoKey:
			theChar = eventPtr->message & charCodeMask;
			if ( (eventPtr->modifiers & cmdKey) != 0 ) 
				HandleMenuChoice( MenuKey( theChar ) );
			break;
	}
}


/************************************* HandleMouseDown */

void	HandleMouseDown( EventRecord *eventPtr )
{
	WindowPtr		window;
	short			thePart;
	long			menuChoice;
	GrafPtr			oldPort;
	long			windSize;
	Rect			growRect;
	
	thePart = FindWindow( eventPtr->where, &window );
	
	switch ( thePart )
	{
		case inMenuBar:
			menuChoice = MenuSelect( eventPtr->where );
			HandleMenuChoice( menuChoice );
			break;
		case inSysWindow : 
			SystemClick( eventPtr, window );
			break;
		case inDrag : 
			DragWindow( window, eventPtr->where, &screenBits.bounds );
			break;
	}
}


/************************************* HandleMenuChoice */

void	HandleMenuChoice( long menuChoice )
{
	short	menu;
	short	item;
	
	if ( menuChoice != 0 )
	{
		menu = HiWord( menuChoice );
		item = LoWord( menuChoice );
		
		switch ( menu )
		{
			case mApple:
				HandleAppleChoice( item );
				break;
			case mFile:
				HandleFileChoice( item );
				break;
		}
		HiliteMenu( 0 );
	}
}


/************************************* HandleAppleChoice */

void	HandleAppleChoice( short item )
{
	MenuHandle	appleMenu;
	Str255		accName;
	short		accNumber;
	
	switch ( item )
	{
		case iAbout:
			SysBeep( 20 );
			break;
		default:
			appleMenu = GetMHandle( mApple );
			GetItem( appleMenu, item, accName );
			accNumber = OpenDeskAcc( accName );
			break;
	}
}


/************************************* HandleFileChoice */

void	HandleFileChoice( short item )
{
	short	newPICTid;
	
	switch ( item )
	{
		case iDialog:
			DoDialog();
			break;
		case iQuit:
			gDone = true;
			break;
	}
}


/************************************* DoDialog */

void	DoDialog( void )
{
	DialogPtr		dialog;
	Boolean			dialogDone = false;
	short			itemHit, iType;
	Handle			iHandle;
	Rect			iRect;
	Str255			numberStr;
	long			number;
	
	dialog = GetNewDialog( kDialogResID, NULL, kMoveToFront );

	ShowWindow( dialog );
	SetPort( dialog );
	
	SetDialogDefaultItem( dialog, ok );
	SetDialogCancelItem( dialog, cancel );
	SetDialogTrackCursor( dialog, kEditItemExists );
	
// Be sure to read tech note #304 which covers these three routines

	while ( ! dialogDone )
	{
		ModalDialog( DLOGFilter, &itemHit );
		
		switch( itemHit )
		{
			case ok:
				GetDItem( dialog, iNumberText, &iType, &iHandle, &iRect );
				GetIText( iHandle, numberStr);
				
				if ( numberStr[ 0 ] == 0 )
				{
					Message( "\pYou must enter a number in the number field!" );
				}
				else
				{
					StringToNum( numberStr, &number );
					
					if ( (number >= 1) && (number <= 100) )
						dialogDone = true;
					else
					{
						Message( "\pPlease enter a number between 1 and 100..." );
						SelIText( dialog, iNumberText, 0, 32767 );
					}
				}
				break;
			case cancel:
				dialogDone = true;
				break;
		}
	}
		
	DisposDialog( dialog );
	
	if ( itemHit == ok )
		Message( "\pYour number was valid!!!" );
	else
		Message( "\pDialog cancelled..." );
}


/************************************* FilterProc */

pascal	Boolean	DLOGFilter( DialogPtr dialog, EventRecord *eventPtr, short *itemHitPtr )
{
	char				c;
	short				iType;
	Handle				iHandle;
	Rect				iRect;
	Str255				textStr;
	long				scrapLength, scrapOffset;
	short				selecLength;

	switch ( eventPtr->what )
	{
		case keyDown:
		case autoKey:
			c = (eventPtr->message & charCodeMask);
			
			if ( (c == kReturnKey) || (c == kEnterKey) ||
				(c == kTabKey) || (c == kBackSpaceKey) ||
				(c == kEscapeKey) || (c == kLeftArrow) ||
				(c == kRightArrow) || (c == kUpArrow) ||
				(c == kDownArrow) || (c == kDeleteKey) )
			{
				return( CallFilterProc( dialog, eventPtr, itemHitPtr ) );
			}
			else if ( CurEditField( dialog ) == iTenCharMaxText )
			{
				GetDItem( dialog, iTenCharMaxText, &iType, &iHandle, &iRect );
				GetIText( iHandle, textStr);
				
				selecLength = SelectionLength( dialog );
				
				if ( ( (eventPtr->modifiers & cmdKey) != 0) && (c == 'v') )
				{
					scrapLength = GetScrap( NULL, 'TEXT', &scrapOffset );
					
					if ( textStr[ 0 ] + scrapLength - selecLength > kMaxFieldLength )
					{
						SysBeep( 20 );
						*itemHitPtr = iTenCharMaxText;
						return( kEventHandled );
					}
					else
						return( CallFilterProc( dialog, eventPtr, itemHitPtr ) );
				}
				if ( (textStr[ 0 ] == kMaxFieldLength) && (selecLength == 0) )
				{
					SysBeep( 20 );
					return( kEventHandled );
				}
				else
					return( CallFilterProc( dialog, eventPtr, itemHitPtr ) );
			}
			else if ( CurEditField( dialog ) == iNumberText )
			{
				GetDItem( dialog, iNumberText, &iType, &iHandle, &iRect );
				
				selecLength = SelectionLength( dialog );
				
				if ( ( (eventPtr->modifiers & cmdKey) != 0) && (c == 'v') )
				{
					if ( ScrapIsOnlyDigits() )
					{
						return( CallFilterProc( dialog, eventPtr, itemHitPtr ) );
					}
					else
					{
						SysBeep( 20 );
						*itemHitPtr = iNumberText;
						return( kEventHandled );
					}
				}
				else if ( ((c < '0') || (c > '9')) && ( (eventPtr->modifiers & cmdKey) == 0) )
				{
					SysBeep( 20 );
					*itemHitPtr = iNumberText;
					return( kEventHandled );
				}
				else
				{
					return( CallFilterProc( dialog, eventPtr, itemHitPtr ) );
				}
			}
			break;
	}
	
	return( CallFilterProc( dialog, eventPtr, itemHitPtr ) );
}


/************************************* ScrapIsOnlyDigits */

Boolean	ScrapIsOnlyDigits( void )
{
	Handle			textHandle;
	long			scrapLength, scrapOffset;
	Boolean			onlyDigits = true;
	unsigned short	i;
	
	textHandle = NewHandle( 0 );
	
	scrapLength = GetScrap( textHandle, 'TEXT', &scrapOffset );
	
	if ( scrapLength <= 0 )
		return( false );
	
	HLock( textHandle );
	
	for ( i=0; i<scrapLength; i++ )
	{
		if ( ((*textHandle)[ i ] < '0') || ((*textHandle)[ i ] > '9') )
			onlyDigits = false;
	}
	
	HUnlock( textHandle );
	
	DisposHandle( textHandle );
	
	return( onlyDigits );
}


/************************************* CallFilterProc */

Boolean	CallFilterProc( DialogPtr dialog, EventRecord *eventPtr, short *itemHitPtr )
{
	ModalFilterProcPtr	theModalProc;
	OSErr				myErr;
	
	myErr = GetStdFilterProc(&theModalProc);

	if (myErr == noErr)
		return( theModalProc( dialog, eventPtr, itemHitPtr ) );
	else
		return( kEventNotHandledYet );
}


/************************************* CurEditField */

short	CurEditField( DialogPtr dialog )
{
	return( ((DialogPeek)dialog)->editField + 1 );
}


/************************************* SelectionLength */

short	SelectionLength( DialogPtr dialog )
{
	TEHandle	teH;
	
	teH = ((DialogPeek)dialog)->textH;
	
	return( (**teH).selEnd - (**teH).selStart );
}


/************************************* Message */

void	Message( Str255 messageStr )
{
	short unused;

	ParamText( messageStr, "\p", "\p", "\p" );
	
	unused = NoteAlert( kMessageAlertID, kNULLFilterProc );
}