program ExperimentThree;
	{ Add the ability to make 'callbacks' to the host application. }

	{ ============= common types and constants ============== }
	{ These must be the same in the parent app and external modules. }

	type
		xmPtr = ^xmBlock;
		xmBlock = record
				request: integer;
				result: integer;
				param: array[1..8] of longint;
				callbackAddr: ProcPtr;
			end;

	const
		xmSuccess = 0;
		xmFail = -1;
		xreqNewXMWindow = 5001;
		xreqCloseXMWindow = 5002;
		xreqSysBeep = 5003;

	{ ============ procedures for the code module ============= }

	{ --------------- glue routines ---------------- }

	procedure DoCallback (theXmPtr: xmPtr; addr: ProcPtr);
	inline
		$205F, $4E90;

	{ ------------------------------- }

	function NewXMWindow (theXmPtr: xmPtr; wBounds: rect; wTitle: str255; wType: integer): WindowPtr;
	begin
		with theXmPtr^ do
			begin
				request := xreqNewXMWindow;
				param[1] := longint(wBounds.topLeft);
				param[2] := longint(wBounds.botRight);
				param[3] := longint(@wTitle);
				param[4] := longint(wType);
				DoCallback(theXmPtr, callbackAddr);
				NewXMWindow := WindowPtr(param[5]);
				if param[5] = 0 then
					result := xmFail
				else
					result := xmSuccess;
			end;
	end;

	{ ------------------------------- }

	procedure CloseXMWindow (theXmPtr: xmPtr; window: WindowPtr);
	begin
		with theXmPtr^ do
			begin
				request := xreqCloseXMWindow;
				param[1] := longint(window);
				DoCallback(theXmPtr, callbackAddr);
			end;
	end;

	{ ------------------------------- }

	procedure DoBeep (theXmPtr: xmPtr; numBeeps: integer);
	begin
		with theXmPtr^ do
			begin
				request := xreqSysBeep;
				param[1] := longint(numBeeps);
				DoCallback(theXmPtr, callbackAddr);
			end;
	end;

	{ --------- external module code that 'does something' --------- }

	procedure xmMain (theXmPtr: xmPtr);
			{ Get a window, draw some circles, wait a second, and close it. }
		var
			tempRect: rect;
			tempStr: str255;
			tempLong: longint;
			theWindow: WindowPtr;
			i: integer;
			oldPort: GrafPtr;
	begin
		if theXmPtr <> nil then
			begin
				tempStr := 'External Module Window';
				SetRect(tempRect, 100, 100, 300, 300);

				{ first callback: get a window from the parent app }
				theWindow := NewXMWindow(theXmPtr, tempRect, tempStr, noGrowDocProc);
				if theXmPtr^.result = xmSuccess then
					begin
						GetPort(oldPort);
						SetPort(theWindow);
						SetRect(tempRect, 99, 99, 100, 100);
						for i := 1 to 100 do	{ draw a pattern }
							begin
								InsetRect(tempRect, -2, -2);
								FrameOval(tempRect);
							end;

				{ second callback: make a noise }
						DoBeep(theXmPtr, 3);
						Delay(60, tempLong);

				{ third callback: ask the app to close the window }
						CloseXMWindow(theXmPtr, theWindow);
						SetPort(oldPort);
					end;
			end;
	end;

	{ ============ procedures in the parent application ============= }

	procedure CallCodeRes (theXmPtr: xmPtr; addr: ProcPtr);
	inline
		$205F, $4E90;

	{ ------------------------------- }

	procedure CallbackDispatcher (theXmPtr: xmPtr);
	{ Don't let this move or be purged! }
		var
			tempRect: rect;
			tempStr: str255;
			tempInt: integer;
	begin
		if theXmPtr <> nil then
			with theXmPtr^ do
				case request of

					xreqNewXMWindow: 
						begin
							tempRect.topLeft := point(param[1]);
							tempRect.botRight := point(param[2]);
							tempStr := StringPtr(param[3])^;
							tempInt := param[4];
							param[5] := longint(NewWindow(nil, tempRect, tempStr, true, tempInt, windowPtr(-1), false, 0));
						end;

					xreqCloseXMWindow: 
						begin
							if param[1] <> 0 then
								DisposeWindow(windowPtr(param[1]));
						end;

					xreqSysBeep: 
						begin
							if not (param[1] in [1..20]) then
								param[1] := 1;
							for tempInt := 1 to param[1] do
								SysBeep(0);
						end;

					otherwise
					{ unknown callback request }
				end;
	end;

	{ ============ main ============= }

	var
		myXmPtr: xmPtr;
begin
	myXmPtr := xmPtr(NewPtrClear(sizeOf(xmBlock)));
	if myXmPtr <> nil then
		begin
			myXmPtr^.callbackAddr := @CallbackDispatcher;
			CallCodeRes(myXmPtr, @xmMain);
			DisposPtr(ptr(myXmPtr));
		end;
end.