program ExperimentFour;
	{ Add 'areq' calls from the host app to the module. }

	{ ============= common types and constants ============== }
	{ These must be the same in the parent app and external modules. }

	type
		xmPtr = ^xmBlock;
		xmBlock = record
				request: integer;
				result: integer;
				param: array[1..8] of longint;
				callbackAddr: ProcPtr;
			end;

	const
		xmSuccess = 0;
		xmFail = -1;
		xreqNewXMWindow = 5001;
		xreqCloseXMWindow = 5002;
		xreqSysBeep = 5003;

		{ new in Experiment 4 }

		xmUnknownAppRequest = -2;

		areqOpen = 7000;					{ required commands }
		areqClose = 7001;
		areqDoEvent = 7002;
		areqGetInfo = 7003;

		areqDrawMoire = 8001;			{ our module-specific commands }
		areqCalcFactorial = 8002;

	{ ============ procedures for the code module ============= }

	{ --------------- glue routines ---------------- }

	procedure DoCallback (theXmPtr: xmPtr; addr: ProcPtr);
	inline
		$205F, $4E90;

	{ ------------------------------- }

	function NewXMWindow (theXmPtr: xmPtr; wBounds: rect; wTitle: str255; wType: integer): WindowPtr;
	begin
		with theXmPtr^ do
			begin
				request := xreqNewXMWindow;
				param[1] := longint(wBounds.topLeft);
				param[2] := longint(wBounds.botRight);
				param[3] := longint(@wTitle);
				param[4] := longint(wType);
				DoCallback(theXmPtr, callbackAddr);
				NewXMWindow := WindowPtr(param[5]);
				if param[5] = 0 then
					result := xmFail
				else
					result := xmSuccess;
			end;
	end;

	{ ------------------------------- }

	procedure CloseXMWindow (theXmPtr: xmPtr; window: WindowPtr);
	begin
		with theXmPtr^ do
			begin
				request := xreqCloseXMWindow;
				param[1] := longint(window);
				DoCallback(theXmPtr, callbackAddr);
			end;
	end;

	{ ------------------------------- }

	procedure DoBeep (theXmPtr: xmPtr; numBeeps: integer);
	begin
		with theXmPtr^ do
			begin
				request := xreqSysBeep;
				param[1] := longint(numBeeps);
				DoCallback(theXmPtr, callbackAddr);
			end;
	end;

	{ --------- our external module code that 'does something' --------- }

	procedure xmMain (theXmPtr: xmPtr);
	{ Respond to different messages from the parent application. }
	{ Some messages are required and common to all modules; some }
	{ are unique to this module. }

		{ ---------- required ----------- }

		procedure OpenXModule;
		begin
			{ One-time call.  Create any menus }
			{ and custom data structures here. }
		end;

		{ --------------------- }

		procedure CloseXModule;
		begin
			{ Release any memory and clean up. }
		end;

		{ --------------------- }

		procedure DoXmEvent;
		begin
			{ Respond to Mac events }
		end;

		{ --------------------- }

		procedure GetXmInfo;
			var
				nameStr, authorStr: str255;
		begin
			nameStr := 'Experiment Four Module';
			authorStr := 'Rob Spencer  1993';
			theXmPtr^.param[1] := longint(NewString(nameStr));
			theXmPtr^.param[2] := longint(NewString(authorStr));
		end;

		{ ----------- unique ---------- }

		procedure DrawMoire;
		{ This was the entire external procedure in Experiment 3 }
			var
				tempRect: rect;
				tempStr: str255;
				tempLong: longint;
				theWindow: WindowPtr;
				i: integer;
				oldPort: GrafPtr;
		begin
			tempStr := 'External Module Window';
			SetRect(tempRect, 100, 100, 300, 300);
			theWindow := NewXMWindow(theXmPtr, tempRect, tempStr, noGrowDocProc);
			if theXmPtr^.result = xmSuccess then
				begin
					GetPort(oldPort);
					SetPort(theWindow);
					SetRect(tempRect, 99, 99, 100, 100);
					for i := 1 to 100 do
						begin
							InsetRect(tempRect, -2, -2);
							FrameOval(tempRect);
						end;
					DoBeep(theXmPtr, 3);
					Delay(60, tempLong);
					CloseXMWindow(theXmPtr, theWindow);
					SetPort(oldPort);
				end;
		end;

		{ --------------------- }

		procedure CalculateFactorial;
		{ Calculate the factorial of param[1], put the result in param[2]. }
			var
				n: integer;
				factorial: longint;
				tempStr: str255;
				tempStrHandle: StringHandle;
		begin
			n := theXmPtr^.param[1];
			if (n < 0) or (n > 12) then
				begin
					{ Error! return a readable message }
					{ so the app may inform the user.  }
					theXmPtr^.result := xmFail;
					tempStr := 'factorial input out-of-range';
					tempStrHandle := NewString(tempStr);
					Hlock(handle(tempStrHandle));
					theXmPtr^.param[1] := longint(handle(tempStrHandle));
				end
			else
				begin
					{ Ok, do the calculation. }
					factorial := 1;
					while n > 1 do
						begin
							factorial := factorial * n;
							n := n - 1;
						end;
					theXmPtr^.param[2] := factorial;
				end;
		end;

		{ ---------- xmMain ---------- }

	begin
		if theXmPtr <> nil then				{ Dispatch the app's request }
			begin
				theXmPtr^.result := xmSuccess;	{ be optimistic }
				case theXmPtr^.request of
					areqOpen: 
						OpenXModule;
					areqClose: 
						CloseXModule;
					areqDoEvent: 
						DoXmEvent;
					areqGetInfo: 
						GetXmInfo;
					areqDrawMoire: 
						DrawMoire;
					areqCalcFactorial: 
						CalculateFactorial;
					otherwise
						theXmPtr^.result := xmUnknownAppRequest;
				end;
			end;
	end;

	{ ============ procedures in the parent application ============= }

	procedure CallCodeRes (theXmPtr: xmPtr; addr: ProcPtr);
	inline
		$205F, $4E90;

	{ ------------------------------- }

	procedure CallbackDispatcher (theXmPtr: xmPtr);
	{ Don't let this move or be purged! }
		var
			tempRect: rect;
			tempStr: str255;
			tempInt: integer;
	begin
		if theXmPtr <> nil then
			with theXmPtr^ do
				case request of

					xreqNewXMWindow: 
						begin
							tempRect.topLeft := point(param[1]);
							tempRect.botRight := point(param[2]);
							tempStr := StringPtr(param[3])^;
							tempInt := param[4];
							param[5] := longint(NewWindow(nil, tempRect, tempStr, true, tempInt, windowPtr(-1), false, 0));
						end;

					xreqCloseXMWindow: 
						begin
							if param[1] <> 0 then
								DisposeWindow(windowPtr(param[1]));
						end;

					xreqSysBeep: 
						begin
							if not (param[1] in [1..20]) then
								param[1] := 1;
							for tempInt := 1 to param[1] do
								SysBeep(0);
						end;

					otherwise
					{ unknown callback request }
				end;
	end;

	{ ============ main ============= }

	var
		myXmPtr: xmPtr;
		tempStr: str255;
begin
	myXmPtr := xmPtr(NewPtrClear(sizeOf(xmBlock)));
	if myXmPtr <> nil then
		begin
			myXmPtr^.callbackAddr := @CallbackDispatcher;

			{ repeat Experiment 4; draw a pattern }
			myXmPtr^.request := areqDrawMoire;
			CallCodeRes(myXmPtr, @xmMain);

			{ get info about the external code }
			with myXmPtr^ do
				begin
					request := areqGetInfo;
					CallCodeRes(myXmPtr, @xmMain);
					ShowText;
					tempStr := StringHandle(param[1])^^;
					DisposHandle(handle(param[1]));
					writeLn(tempStr);
					tempStr := StringHandle(param[2])^^;
					DisposHandle(handle(param[2]));
					writeLn(tempStr);
				end;

			{ calculate a factorial and check for errors }
			with myXmPtr^ do
				begin
					{ Use Pascal lazy I/O for this demo. }
					ShowText;
					write('Enter a number : ');
					readLn(param[1]);
					request := areqCalcFactorial;
					CallCodeRes(myXmPtr, @xmMain);
					if result = xmSuccess then
						writeLn(param[1], ' ! =', param[2])
					else
						begin
							tempStr := StringHandle(param[1])^^;
							DisposHandle(handle(param[1]));
							writeLn(tempStr);
						end;
				end;

		end;
	DisposPtr(ptr(myXmPtr));
end.