program ExperimentFiveMain;
	{ A minimal but 'real' application to demonstate the }
	{ use of external code modules with callbacks.       }
	{ Application shell based on Apple's "Sample"        }
	{ Rob Spencer  1993 }

	uses
		Globals, AppUtils, XModuleGlue, XmUtils;


{ === hard-wired code to manage our external === }

	procedure InitMoire;
	{ Add a custom menu }
		var
			xmMenu: menuHandle;
			itemStr: str255;
	begin
		xmMenu := NewMenu(mExternal, 'Externals');
		itemStr := 'Moir';
		AppendMenu(xmMenu, itemStr);
		InsertMenu(xmMenu, 0);
		DrawMenuBar;
	end;

{ ------------------------------------------- }

	procedure RunMoire;
		var
			codeHandle: handle;
			codePtr: ProcPtr;
			myXmPtr: xmPtr;
			tempStr: str255;
	begin

		{ Get the address of the code; }
		{ it's a precompiled resource  }
		{ in the app 's res fork.      }

		codeHandle := GetResource(kXCodeType, rMoireXMODid);
		if codeHandle = nil then
			begin
				AlertUser(kMissingCode, false);
				exit(RunMoire);
			end;
		MoveHHi(codeHandle);
		Hlock(codeHandle);
		codePtr := ProcPtr(codeHandle^);

		{ Jump to the code, same as Experiment 4 }

		myXmPtr := xmPtr(NewPtrClear(sizeOf(xmBlock)));
		if myXmPtr <> nil then
			begin
				myXmPtr^.callbackAddr := @CallbackDispatcher;

			{ draw a pattern }
				myXmPtr^.request := areqDrawMoire;
				CallCodeRes(myXmPtr, codePtr);

			{ get info about the external code }
			{ but now display it in an alert   }
				with myXmPtr^ do
					begin
						request := areqGetInfo;
						CallCodeRes(myXmPtr, codePtr);
						tempStr := StringHandle(param[1])^^;
						DisposHandle(handle(param[1]));
						tempStr := concat(tempStr, chr(13), StringHandle(param[2])^^);
						DisposHandle(handle(param[2]));
						AlertUser(tempStr, false);
					end;
			end;

		DisposPtr(ptr(myXmPtr));		{ Dispose of the parameter block, }
		Hunlock(codeHandle);				{ and release the code too.       }
		DetachResource(codeHandle);
		DisposHandle(codeHandle);
	end;

{ == minimal Mac application code === }

	procedure Initialize;
		var
			ignoreError: OSErr;
			theWorld: SysEnvRec;
			menuBar: handle;
	begin
		MaxApplZone;
		MoreMasters;
		InitGraf(@thePort);
		InitFonts;
		InitWindows;
		InitMenus;
		TEInit;
		InitDialogs(nil);
		InitCursor;

		ignoreError := SysEnvirons(kSysEnvironsVersion, theWorld);
		if (theWorld.machineType < 0) or (theWorld.systemVersion < $604) then
			AlertUser(kEarlyMac, true);

		menuBar := GetNewMBar(rMenuBar);
		if menuBar = nil then
			AlertUser(kOutOfMemory, true);
		SetMenuBar(menuBar);
		DisposHandle(menuBar);
		AddResMenu(GetMHandle(mApple), 'DRVR');
		DrawMenuBar;

		gAllDone := false;
		gInBackground := false;
	end;

{ ------------------------------------------- }

	procedure DoMenuCommand (menuResult: LONGINT);
		var
			menuID, menuItem, itemHit: integer;
			daName: str255;
			daRefNum: integer;
			handledByDA: boolean;
	begin
		menuID := HiWrd(menuResult);
		menuItem := LoWrd(menuResult);
		case menuID of
			mApple: 
				case menuItem of
					iAbout: 
						itemHit := Alert(rAboutAlert, nil);
					otherwise
						begin
							GetItem(GetMHandle(mApple), menuItem, daName);
							daRefNum := OpenDeskAcc(daName);
						end;
				end;
			mFile: 
				case menuItem of
					iQuit: 
						gAllDone := true;
					otherwise
				end;
			mEdit: 
				handledByDA := SystemEdit(menuItem - 1);

		{ Note hard-wired call to external }
			mExternal: 
				case menuItem of
					iMoire: 
						RunMoire;
					otherwise
				end;

			otherwise
		end;
		HiliteMenu(0);
	end;

{ ------------------------------------------- }

	procedure DrawWindow (window: WindowPtr);
	begin
		{ dispatch to XMOD update routines }
	end;

{ ------------------------------------------- }

	procedure DoContentClick (window: WindowPtr; event: EventRecord);
	begin
	end;

{ ------------------------------------------- }

	procedure DoUpdate (window: WindowPtr);
	begin
		if IsAppWindow(window) then
			begin
				BeginUpdate(window);
				if not EmptyRgn(window^.visRgn) then
					DrawWindow(window);
				EndUpdate(window);
			end;
	end;

{ ------------------------------------------- }

	procedure DoActivate (window: WindowPtr; becomingActive: boolean);
	begin
		if IsAppWindow(window) then
			if becomingActive then
			else
	end;

{ ------------------------------------------- }

	procedure DoEvent (event: EventRecord);
		var
			part, err: integer;
			window: WindowPtr;
			key: char;
	begin
		case event.what of
			mouseDown: 
				begin
					part := FindWindow(event.where, window);
					case part of
						inMenuBar: 
							DoMenuCommand(MenuSelect(event.where));
						inSysWindow: 
							SystemClick(event, window);
						inContent: 
							if window <> FrontWindow then
								SelectWindow(window)
							else
								DoContentClick(window, event);
						inDrag: 
							DragWindow(window, event.where, screenBits.bounds);
						inGrow: 
							;
						inZoomIn, inZoomOut: 
							;
						otherwise
					end;
				end;
			keyDown, autoKey: 
				begin
					key := chr(BAnd(event.message, charCodeMask));
					if (BAnd(event.modifiers, cmdKey) <> 0) and (event.what = keyDown) then
						DoMenuCommand(MenuKey(key));
				end;
			activateEvt: 
				DoActivate(WindowPtr(event.message), BAnd(event.modifiers, activeFlag) <> 0);
			updateEvt: 
				DoUpdate(WindowPtr(event.message));
			diskEvt: 
				if HiWrd(event.message) <> noErr then
					err := DIBadMount(point($00500070), event.message);
			kOSEvent: 
				case BAnd(BRotL(event.message, 8), $FF) of
					kSuspendResumeMessage: 
						begin
							gInBackground := BAnd(event.message, kResumeMask) = 0;
							DoActivate(FrontWindow, not gInBackground);
						end;
				end;
			otherwise
		end;
	end;

{ ================= main ================== }

	var
		event: EventRecord;
begin
	Initialize;
	InitMoire;
	repeat
		if WaitNextEvent(everyEvent, event, 40, nil) then
			DoEvent(event);
	until gAllDone;
end.