unit MoireXMOD;

	{ Externals are responsible for all memory that they allocate. }
	{ If the app should dispose of something, then the app must    }
	{ supply its own handle to the external. }

interface
	uses
		XMGlobals, XMGlue;
	procedure main (theXmPtr: xmPtr);

implementation

	const
		kXMODResName = 'Moir';	{ ** This must match this XMOD's res name ** }
		kNumCommands = 3;
		kMenuName = 'Moirs';
		kMenuStr = 'circles/1;ellipses/2;with lines/3;-);About Moir...';

	procedure main (theXmPtr: xmPtr);

		procedure DoXMMenuCommand (menuResult: longint);
		forward;
		procedure DrawMoire (window: WindowPtr);
		forward;

		{ ---------- required ----------- }

		procedure OpenXModule;
			{ One-time call.  Create any menus, windows, }
			{ and custom data structures here.           }
			var
				myMenu: MenuHandle;
				tempRect: rect;
				tempStr: str255;
				window: WindowPtr;
		begin
			myMenu := GetXMMenu(theXmPtr, kMenuName);
			if myMenu <> nil then
				begin
					AppendMenu(myMenu, kMenuStr);
					DrawMenuBar;
				end;

			{ Any number of additional menus could be added, e.g. : }

			{    myMenu := GetXMMenu(theXmPtr, 'Second Menu'); }
			{    if myMenu <> nil then }
			{     begin }
			{      AppendMenu(myMenu, 'first item/4;second item/5;-);third item/6'); }
			{      DrawMenuBar; }
			{     end; }

			{ Now create our window -- one time only }

			tempStr := 'Moir Window';
			SetRect(tempRect, 100, 100, 300, 300);
			window := NewXMWindow(theXmPtr, tempRect, tempStr, noGrowDocProc);
			if theXmPtr^.result = xmSuccess then
				begin
					SetPort(window);
					ShowWindow(window);
					{ The update event will get it drawn... }
				end;

		end;

		{ --------------------- }

		procedure CloseXModule;
		{ Release any memory and clean up. }
			var
				myMenu: MenuHandle;
				i, numMyWindows: integer;
				owner: OSType;
				window: windowPtr;
		begin
			myMenu := GetXMMenu(theXmPtr, kMenuName);	{ a callback }
			if myMenu <> nil then
				DisposeMenu(myMenu);

				{ Find and close all XMOD's windows }

			numMyWindows := CountXMWindows(theXmPtr, true);
			for i := 1 to numMyWindows do
				begin
					window := GetIndXMWindow(theXmPtr, i, true, owner);
					if window <> nil then
						CloseXMWindow(theXmPtr, window);
				end;
		end;

		{ --------------------- }

		procedure DoXmEvent;
			{ Respond to Mac events. }
			type
				EventPtr = ^EventRecord;
			var
				event: EventRecord;
				part, err: integer;
				window: WindowPtr;
				key: char;
				didHandle: boolean;
				tempRect: rect;
				tempLong: longint;
		begin
			{ Get data out of the xmBlock now, since  }
			{ any callbacks we make may overwrite it. }
			event := EventPtr(theXmPtr^.param[1])^;
			window := WindowPtr(theXmPtr^.param[2]);
			didHandle := false;

			case event.what of
				mouseDown: 
					begin
						part := FindWindow(event.where, window);
						case part of
							inMenuBar: 
								begin
									{ For this event only, param[2] is the menuResult. }
									{ For all others, it's the WindowPtr.              }
									DoXMMenuCommand(theXmPtr^.param[2]);
									didHandle := true;
								end;
							inContent: 
								if window = FrontWindow then
									begin
										tempRect := window^.portRect;
										SetPort(window);
										InvertRect(tempRect);
										Delay(20, tempLong);
										InvertRect(tempRect);
									end;
							otherwise
						end;
					end;
				keyDown, autoKey: 
					begin
						key := chr(BAnd(event.message, charCodeMask));
						if (BAnd(event.modifiers, cmdKey) <> 0) and (event.what = keyDown) then
							DoXMMenuCommand(MenuKey(key))
						else
							begin
								{ Handle any typing in the XMOD's window. }
							end;
					end;
				activateEvt: 		{ *** update any menus, incl. app 'Print...' menu later. *** }
					SetPort(window);
				updateEvt: 
					begin
						BeginUpdate(window);
						DrawMoire(window);
						EndUpdate(window);
					end;
				otherwise
			end;

			{ respond to the app }
			if didHandle then
				theXmPtr^.param[3] := 1
			else
				theXmPtr^.param[3] := 0;
		end;

		{ ----------- specialized ---------- }

		procedure DoXMMenuCommand (menuResult: longint);
			var
				menuID, menuItem, prevStyle: integer;
				owner: OSType;
				window: windowPtr;
		begin
			menuID := HiWrd(menuResult);
			menuItem := LoWrd(menuResult);

			{ Custom for this XMOD. }

			if menuItem = 5 then
				AlertUser(theXmPtr, concat('Moir Module', chr(13), 'Rob Spencer 1993'))
			else
				begin
					if CountXMWindows(theXmPtr, true) > 0 then
						window := GetIndXMWindow(theXmPtr, 1, true, owner);
					if window <> nil then
						begin
							ShowWindow(window);
							SelectWindow(window);
							prevStyle := GetWRefCon(window);
							if prevStyle <> menuItem then
								begin
									SetWRefCon(window, menuItem);	{ cache the 'style' request here }
									EraseRect(window^.portRect);
									InvalRect(window^.portRect);
								end;
						end;
				end;
			HiliteMenu(0);
		end;

	{ --------------------- }

		procedure DrawMoire (window: WindowPtr);
		{ 'Style' changes the graphic pattern.  It's stored in the refCon. }
			var
				tempRect: rect;
				tempStr: str255;
				tempLong: longint;
				i, style: integer;
				oldPort: GrafPtr;
		begin
			SetRect(tempRect, 100, 100, 300, 300);
			GetPort(oldPort);
			SetPort(window);
			ShowWindow(window);
			style := GetWRefCon(window);
			if not (style in [1..3]) then
				style := 1;
			SetWRefCon(window, style);
			SetRect(tempRect, 99, 99, 100, 100);

			case style of
				1: 
					for i := 1 to 100 do						{ circles }
						begin
							InsetRect(tempRect, -2, -2);
							FrameOval(tempRect);
						end;
				2: 
					for i := 1 to 100 do						{ ellipses }
						begin
							InsetRect(tempRect, -2, -3);
							FrameOval(tempRect);
						end;
				3: 
					for i := 0 to 100 do						{ circles & lines }
						begin
							InsetRect(tempRect, -2, -2);
							FrameOval(tempRect);
							MoveTo(0, 2 * i);
							LineTo(202, 2 * i);
						end;
				otherwise
			end;
			SetPort(oldPort);
		end;

		{ ---------- main procedure of the XMOD ---------- }

	begin
		if theXmPtr <> nil then				{ Dispatch the app's request }
			begin
				theXmPtr^.result := xmSuccess;					{ be optimistic }
				theXmPtr^.XMODResName := kXMODResName;	{ some callbacks need this }
				case theXmPtr^.request of
					areqOpen: 
						OpenXModule;
					areqClose: 
						CloseXModule;
					areqDoEvent: 
						DoXmEvent;
					areqGetInfo: 
						DoGetXmInfo(theXmPtr, kXMODResName, 'Rob Spencer 1993');
					areqCountCommands: 
						DoCountCommands(theXmPtr, kNumCommands);
					areqGetIndCommand: 
						;
					areqGetErrorInfo: 
						;
					areqDoCommand: 
						SysBeep(0);
					otherwise
						theXmPtr^.result := xmUnknownRequest;
				end;
			end;
	end;
end.