unit SoundsXMOD;

	{ Externals are responsible for all memory that they allocate. }
	{ If the app should dispose of something, then the app must    }
	{ supply its own handle to the external. }

interface
	uses
		XMGlobals, XMGlue, Sound;
	procedure main (theXmPtr: xmPtr);

implementation

	const
		kXMODResName = 'Snd1';	{ ** This must match this XMOD's res name ** }
		kNumCommands = 0;
		kMenuName = 'Sounds';

	procedure main (theXmPtr: xmPtr);

		procedure DoXMMenuCommand (menuResult: longint);
		forward;

		{ ---------- required ----------- }

		procedure OpenXModule;
			{ One-time call.  Create any menus, windows, }
			{ and custom data structures here.           }
			var
				myMenu: MenuHandle;
				tempRect: rect;
				window: WindowPtr;
				numPicts, i, tempId: integer;
				tempHandle: handle;
				tempType: ResType;
				tempStr, menuStr: str255;
		begin
			myMenu := GetXMMenu(theXmPtr, kMenuName);
			if myMenu <> nil then
				begin
					{ assemble the menu from 'snd ' names }
					menuStr := '';
					numPicts := Count1Resources('snd ');
					if numPicts > 0 then
						for i := 1 to numPicts do
							begin
								tempHandle := Get1IndResource('snd ', i);
								GetResInfo(tempHandle, tempId, tempType, tempStr);
								DetachResource(tempHandle);
								DisposeHandle(tempHandle);
								menuStr := concat(menuStr, tempStr, ';');
							end;
					menuStr := concat(menuStr, '-);About Sounds...');
					AppendMenu(myMenu, menuStr);
					DrawMenuBar;
				end;

			{ Any number of additional menus could be added. }

			{ No window in this module }

		end;

		{ --------------------- }

		procedure CloseXModule;
		{ Release any memory and clean up. }
			var
				myMenu: MenuHandle;
				i, numMyWindows: integer;
				owner: OSType;
				window: windowPtr;
		begin
			myMenu := GetXMMenu(theXmPtr, kMenuName);
			if myMenu <> nil then
				DisposeMenu(myMenu);
		end;

		{ --------------------- }

		procedure DoXmEvent;
			{ Respond to Mac events. }
			type
				EventPtr = ^EventRecord;
			var
				event: EventRecord;
				part, err: integer;
				window: WindowPtr;
				key: char;
				didHandle: boolean;
				tempRect: rect;
		begin
			{ Get data out of the xmBlock now, since  }
			{ any callbacks we make may overwrite it. }
			event := EventPtr(theXmPtr^.param[1])^;
			window := WindowPtr(theXmPtr^.param[2]);
			didHandle := false;

			case event.what of
				mouseDown: 
					begin
						part := FindWindow(event.where, window);
						case part of
							inMenuBar: 
								begin
									{ For this event only, param[2] is the menuResult. }
									{ For all others, it's the WindowPtr.              }
									DoXMMenuCommand(theXmPtr^.param[2]);
									didHandle := true;
								end;
							otherwise
						end;
					end;
				otherwise
			end;

			{ respond to the app }
			if didHandle then
				theXmPtr^.param[3] := 1
			else
				theXmPtr^.param[3] := 0;
		end;

		{ ----------- specialized ---------- }

		procedure DoXMMenuCommand (menuResult: longint);
			var
				menuID, menuItem: integer;
				tempMenu: MenuHandle;
				sndHandle: handle;
		begin
			menuID := HiWrd(menuResult);
			menuItem := LoWrd(menuResult);

			{ Custom for this XMOD. }

			tempMenu := GetMHandle(menuID);
			if tempMenu = nil then
				exit(DoXMMenuCommand);

			if menuItem = CountMItems(tempMenu) then
				AlertUser(theXmPtr, concat('Sounds Module', chr(13), 'Rob Spencer 1992'))
			else
				begin
					sndHandle := Get1IndResource('snd ', menuItem);
					if sndHandle <> nil then
						begin
							if SndPlay(nil, sndHandle, false) = noErr then
								repeat
								until SoundDone;
							DetachResource(sndHandle);
							DisposeHandle(sndHandle);
						end;
				end;
			HiliteMenu(0);
		end;

		{ ---------- main procedure of the XMOD ---------- }

	begin
		if theXmPtr <> nil then				{ Dispatch the app's request }
			begin
				theXmPtr^.result := xmSuccess;					{ be optimistic }
				theXmPtr^.XMODResName := kXMODResName;	{ some callbacks need this }
				case theXmPtr^.request of
					areqOpen: 
						OpenXModule;
					areqClose: 
						CloseXModule;
					areqDoEvent: 
						DoXmEvent;
					areqGetInfo: 
						DoGetXmInfo(theXmPtr, kXMODResName, 'Rob Spencer 1992');
					areqCountCommands: 
						DoCountCommands(theXmPtr, kNumCommands);
					areqGetIndCommand: 
						;
					areqGetErrorInfo: 
						;
					areqDoCommand: 
						SysBeep(0);
					otherwise
						theXmPtr^.result := xmUnknownRequest;
				end;
			end;
	end;
end.