unit PongXMOD;
	{ An external that receives 'idle' messages. }
interface
	uses
		XMGlobals, XMGlue;
	procedure main (theXmPtr: xmPtr);

implementation

	type
		pongDataHandle = ^pongDataPtr;
		pongDataPtr = ^pongDataRec;
		pongDataRec = record
				where: point;
				radius, speed, dx, dy: integer;
			end;

	const
		kXMODResName = 'Pong';	{ ** This must match this XMOD's res name ** }
		kNumCommands = 0;
		kMenuName = 'Pong';
		kMenuStr = 'big;small;-);fast;slow;-);About Pong...';
		iBig = 1;
		iSmall = 2;
		iFast = 4;
		iSlow = 5;
		iAbout = 7;

	procedure main (theXmPtr: xmPtr);

	{ --------------------- }

		procedure BounceTheBall (window: windowPtr);
			var
				ball, frame: rect;
				where: point;
				pongData: pongDataHandle;
				oldPort: GrafPtr;
		begin
			GetPort(oldPort);
			SetPort(window);

			frame := window^.portRect;
			pongData := pongDataHandle(GetWRefCon(window));
			Hlock(handle(pongData));
			with pongData^^ do
				begin
					InsetRect(frame, radius, radius);
					with where do
						SetRect(ball, h - radius, v - radius, h + radius, v + radius);
					EraseRect(window^.portRect);
					PaintOval(ball);
					with where, frame do
						begin
							h := h + speed * dx;
							v := v + speed * dy;

									{ edge detection and reversal }

							if (h >= right) or (h <= left) then
								dx := -dx;
							if (v >= bottom) or (v <= top) then
								dy := -dy;
						end;
				end;
			Hunlock(handle(pongData));
			SetPort(oldPort);
		end;

		{ --------------------- }

		procedure DoXMMenuCommand (menuResult: longint);
		forward;

		{ ---------- required ----------- }

		procedure OpenXModule;
			{ One-time call.  Create any menus, windows, }
			{ and custom data structures here.           }
			var
				theMenu: MenuHandle;
				tempRect: rect;
				tempStr: str255;
				window: WindowPtr;
				pongData: pongDataHandle;
		begin
			theMenu := GetXMMenu(theXmPtr, kMenuName);
			if theMenu <> nil then
				begin
					AppendMenu(theMenu, kMenuStr);
					DrawMenuBar;
				end;

			{ Now create our window -- one time only }

			tempStr := 'pong';
			SetRect(tempRect, 100, 100, 300, 200);
			window := NewXMWindow(theXmPtr, tempRect, tempStr, noGrowDocProc);
			if theXmPtr^.result = xmSuccess then
				begin
					pongData := pongDataHandle(NewHandleClear(sizeOf(pongDataRec)));
					with pongData^^ do
						begin
							radius := 6;
							speed := 4;
							where.h := radius + 2;
							where.v := where.h;
							dx := 1;
							dy := 1;
						end;
					CheckItem(theMenu, iSmall, true);
					CheckItem(theMenu, iFast, true);
					SetWRefCon(window, longint(pongData));
					SetPort(window);
					ShowWindow(window);
				end;
			ReceiveIdleEvents(theXmPtr, true);
		end;

		{ --------------------- }

		procedure CloseXModule;
		{ Release any memory and clean up. }
			var
				myMenu: MenuHandle;
				i, numMyWindows: integer;
				owner: OSType;
				window: windowPtr;
				tempH: handle;
		begin
			myMenu := GetXMMenu(theXmPtr, kMenuName);	{ a callback }
			if myMenu <> nil then
				DisposeMenu(myMenu);

				{ Find and close all XMOD's windows }

			numMyWindows := CountXMWindows(theXmPtr, true);
			for i := 1 to numMyWindows do
				begin
					window := GetIndXMWindow(theXmPtr, i, true, owner);
					if window <> nil then
						begin
							tempH := handle(GetWRefCon(window));
							if tempH <> nil then
								DisposHandle(tempH);
							CloseXMWindow(theXmPtr, window);
						end;
				end;
		end;

		{ --------------------- }

		procedure DoXmEvent;
			{ Respond to Mac events. }
			type
				EventPtr = ^EventRecord;
			var
				event: EventRecord;
				part, err: integer;
				window: WindowPtr;
				key: char;
				didHandle: boolean;
				tempRect: rect;
				tempLong: longint;
		begin
			{ Get data out of the xmBlock now, since  }
			{ any callbacks we make may overwrite it. }
			event := EventPtr(theXmPtr^.param[1])^;
			window := WindowPtr(theXmPtr^.param[2]);
			didHandle := false;

			case event.what of
				mouseDown: 
					begin
						part := FindWindow(event.where, window);
						case part of
							inMenuBar: 
								begin
									{ For this event only, param[2] is the menuResult. }
									{ For all others, it's the WindowPtr.              }
									DoXMMenuCommand(theXmPtr^.param[2]);
									didHandle := true;
								end;
							inContent: 
								if window = FrontWindow then
									begin
										{ Run pong at full speed. }
										HideCursor;
										while Button do
											BounceTheBall(window);
										ShowCursor;
										didHandle := true;
									end;
							otherwise
						end;
					end;
				keyDown, autoKey: 
					begin
						key := chr(BAnd(event.message, charCodeMask));
						if (BAnd(event.modifiers, cmdKey) <> 0) and (event.what = keyDown) then
							begin
								DoXMMenuCommand(MenuKey(key));
								didHandle := true;
							end
						else
							begin
								{ Handle any typing in the XMOD's window. }
							end;
					end;
				activateEvt: 		{ *** update any menus, incl. app 'Print...' menu later. *** }
					begin
						SetPort(window);
						didHandle := true;
					end;
				updateEvt: 
					begin
						BeginUpdate(window);
						{ do nothing... }
						EndUpdate(window);
						didHandle := true;
					end;
				otherwise
			end;

			{ respond to the app }
			if didHandle then
				theXmPtr^.param[3] := 1
			else
				theXmPtr^.param[3] := 0;
		end;

		{ --------------------- }

		procedure DoXMMenuCommand (menuResult: longint);
			var
				menuID, menuItem: integer;
				owner: OSType;
				window: windowPtr;
				pongData: pongDataHandle;
				theMenu: menuHandle;
		begin
			menuID := HiWrd(menuResult);
			menuItem := LoWrd(menuResult);
			theMenu := GetMHandle(menuID);

			{ Custom for this XMOD.  Use the window's refCon to hold fast/slow/big/small info. }

			if menuItem = iAbout then
				AlertUser(theXmPtr, concat('Pong Module', chr(13), 'Rob Spencer 1993'))
			else
				begin
					if CountXMWindows(theXmPtr, true) > 0 then
						window := GetIndXMWindow(theXmPtr, 1, true, owner);
					if window <> nil then
						begin
							pongData := pongDataHandle(GetWRefCon(window));
							Hlock(handle(pongData));
							with pongData^^ do
								case menuItem of
									iBig: 
										begin
											radius := 12;
											CheckItem(theMenu, iBig, true);
											CheckItem(theMenu, iSmall, false);
										end;
									iSmall: 
										begin
											radius := 6;
											CheckItem(theMenu, iSmall, true);
											CheckItem(theMenu, iBig, false);
										end;
									iFast: 
										begin
											speed := 6;
											CheckItem(theMenu, iFast, true);
											CheckItem(theMenu, iSlow, false);
										end;
									iSlow: 
										begin
											speed := 2;
											CheckItem(theMenu, iSlow, true);
											CheckItem(theMenu, iFast, false);
										end;
									otherwise
								end;
							Hunlock(handle(pongData));
							ShowWindow(window);
						end;
				end;
			HiliteMenu(0);
		end;

	{ --------------------- }

		procedure DoIdle;
			var
				window: windowPtr;
				owner: OSType;
		begin
			window := GetIndXMWindow(theXmPtr, 1, true, owner);
			if window <> nil then
				BounceTheBall(window);
		end;

		{ ---------- main procedure of the XMOD ---------- }

	begin
		if theXmPtr <> nil then				{ Dispatch the app's request }
			begin
				theXmPtr^.result := xmSuccess;					{ be optimistic }
				theXmPtr^.XMODResName := kXMODResName;	{ some callbacks need this }
				case theXmPtr^.request of
					areqOpen: 
						OpenXModule;
					areqClose: 
						CloseXModule;
					areqDoEvent: 
						DoXmEvent;
					areqGetInfo: 
						DoGetXmInfo(theXmPtr, kXMODResName, 'Rob Spencer 1993');
					areqCountCommands: 
						DoCountCommands(theXmPtr, kNumCommands);
					areqGetIndCommand: 
						;
					areqGetErrorInfo: 
						;
					areqDoCommand: 
						SysBeep(0);
					areqDoIdle: 
						DoIdle;
					otherwise
						theXmPtr^.result := xmUnknownRequest;
				end;
			end;
	end;
end.