unit PostcardXMOD;
	{ After this module is compiled as a stand-alone document of   }
	{ type XMOD and creator xmAp, use ResEdit to add as many named }
	{ PICT resources as you want. }

interface
	uses
		XMGlobals, XMGlue;
	procedure main (theXmPtr: xmPtr);

implementation

	const
		kXMODResName = 'Card';	{ ** This must match this XMOD's res name ** }
		kNumCommands = 0;
		kMenuName = 'Postcards';

	procedure main (theXmPtr: xmPtr);

		procedure DoXMMenuCommand (menuResult: longint);
		forward;
		procedure ShowPostcard (window: WindowPtr);
		forward;

		{ ---------- required ----------- }

		procedure OpenXModule;
			{ One-time call.  Create any menus, windows, }
			{ and custom data structures here.           }
			var
				myMenu: MenuHandle;
				tempRect: rect;
				window: WindowPtr;
				numPicts, i, tempId: integer;
				tempHandle: handle;
				tempType: ResType;
				tempStr, menuStr: str255;
		begin
			myMenu := GetXMMenu(theXmPtr, kMenuName);
			if myMenu <> nil then
				begin
					{ assemble the menu from PICT names }
					menuStr := '';
					numPicts := Count1Resources('PICT');
					if numPicts > 0 then
						for i := 1 to numPicts do
							begin
								tempHandle := Get1IndResource('PICT', i);
								GetResInfo(tempHandle, tempId, tempType, tempStr);
								DetachResource(tempHandle);
								DisposeHandle(tempHandle);
								menuStr := concat(menuStr, tempStr, ';');
							end;
					menuStr := concat(menuStr, '-);About Postcards...');
					AppendMenu(myMenu, menuStr);
					DrawMenuBar;
				end;

			{ Any number of additional menus could be added. }

			{ Now create our window -- one time only }

			tempStr := 'postcards';
			SetRect(tempRect, 100, 100, 300, 300);
			window := NewXMWindow(theXmPtr, tempRect, tempStr, noGrowDocProc);
			ShowPostcard(window);
		end;

		{ --------------------- }

		procedure CloseXModule;
		{ Release any memory and clean up. }
			var
				myMenu: MenuHandle;
				i, numMyWindows: integer;
				owner: OSType;
				window: windowPtr;
		begin
			myMenu := GetXMMenu(theXmPtr, kMenuName);	{ a callback }
			if myMenu <> nil then
				DisposeMenu(myMenu);

				{ Find and close all XMOD's windows }

			numMyWindows := CountXMWindows(theXmPtr, true);
			for i := 1 to numMyWindows do
				begin
					window := GetIndXMWindow(theXmPtr, i, true, owner);
					if window <> nil then
						CloseXMWindow(theXmPtr, window);
				end;
		end;

		{ --------------------- }

		procedure DoXmEvent;
			{ Respond to Mac events. }
			type
				EventPtr = ^EventRecord;
			var
				event: EventRecord;
				part, err: integer;
				window: WindowPtr;
				key: char;
				didHandle: boolean;
				tempRect: rect;
		begin
			{ Get data out of the xmBlock now, since  }
			{ any callbacks we make may overwrite it. }
			event := EventPtr(theXmPtr^.param[1])^;
			window := WindowPtr(theXmPtr^.param[2]);
			didHandle := false;

			case event.what of
				mouseDown: 
					begin
						part := FindWindow(event.where, window);
						case part of
							inMenuBar: 
								begin
									{ For this event only, param[2] is the menuResult. }
									{ For all others, it's the WindowPtr.              }
									DoXMMenuCommand(theXmPtr^.param[2]);
									didHandle := true;
								end;
							inContent: 
								if window = FrontWindow then
									begin
										SetPort(window);
										tempRect := window^.portRect;
										InvertRect(tempRect);
										while Button do
											;
										InvertRect(tempRect);
									end;
							otherwise
						end;
					end;
				keyDown, autoKey: 
					begin
						key := chr(BAnd(event.message, charCodeMask));
						if (BAnd(event.modifiers, cmdKey) <> 0) and (event.what = keyDown) then
							DoXMMenuCommand(MenuKey(key))
						else
							begin
								{ Handle any typing in the XMOD's window. }
							end;
					end;
				activateEvt: 		{ update any menus, e.g. an app 'Print...' menu later. }
					SetPort(window);
				updateEvt: 
					begin
						BeginUpdate(window);
						ShowPostcard(window);
						EndUpdate(window);
					end;
				otherwise
			end;

			{ respond to the app }
			if didHandle then
				theXmPtr^.param[3] := 1
			else
				theXmPtr^.param[3] := 0;
		end;

		{ ----------- specialized ---------- }

		procedure DoXMMenuCommand (menuResult: longint);
			var
				menuID, menuItem, prevPictNum: integer;
				owner: OSType;
				window: windowPtr;
				tempMenu: MenuHandle;
		begin
			menuID := HiWrd(menuResult);
			menuItem := LoWrd(menuResult);

			{ Custom for this XMOD. }

			tempMenu := GetMHandle(menuID);
			if tempMenu = nil then
				exit(DoXMMenuCommand);

			if menuItem = CountMItems(tempMenu) then
				AlertUser(theXmPtr, concat('Postcard Module', chr(13), 'Rob Spencer 1992'))
			else
				begin
					if CountXMWindows(theXmPtr, true) > 0 then
						window := GetIndXMWindow(theXmPtr, 1, true, owner);
					if window <> nil then
						begin
							ShowWindow(window);
							prevPictNum := GetWRefCon(window);
							if prevPictNum <> menuItem then
								begin
									ShowWindow(window);
									SetWRefCon(window, menuItem);	{ cache the request here }
									EraseRect(window^.portRect);
									InvalRect(window^.portRect);
								end;
						end;
				end;
			HiliteMenu(0);
		end;

	{ --------------------- }

		procedure ShowPostcard (window: WindowPtr);
		{ The item number of the current PICT is stored in the refCon. }
			var
				tempStr: str255;
				numPicts, pictNumber: integer;
				oldPort: GrafPtr;
				thePict: PicHandle;
				pictRect: rect;
		begin

			{ get the PICT based on the menuId in the refCon }

			numPicts := Count1Resources('PICT');

			GetPort(oldPort);
			pictNumber := GetWRefCon(window);
			if not (pictNumber in [1..numPicts]) then
				pictNumber := 1;
			SetWRefCon(window, pictNumber);

			thePict := PicHandle(Get1IndResource('PICT', pictNumber));
			if thePict <> nil then
				begin
					pictRect := thePict^^.picFrame;
					with pictRect do
						begin
							right := right - left;
							bottom := bottom - top;
							left := 0;
							top := 0;
							SizeWindow(window, right, bottom, false);
						end;
					SetPort(window);
					ShowWindow(window);
					DrawPicture(thePict, pictRect);
					DetachResource(handle(thePict));
					DisposeHandle(handle(thePict));
				end;
			SetPort(oldPort);
		end;

		{ ---------- main procedure of the XMOD ---------- }

	begin
		if theXmPtr <> nil then				{ Dispatch the app's request }
			begin
				theXmPtr^.result := xmSuccess;					{ be optimistic }
				theXmPtr^.XMODResName := kXMODResName;	{ some callbacks need this }
				case theXmPtr^.request of
					areqOpen: 
						OpenXModule;
					areqClose: 
						CloseXModule;
					areqDoEvent: 
						DoXmEvent;
					areqGetInfo: 
						DoGetXmInfo(theXmPtr, kXMODResName, 'Rob Spencer 1993');
					areqCountCommands: 
						DoCountCommands(theXmPtr, kNumCommands);
					areqGetIndCommand: 
						;
					areqGetErrorInfo: 
						;
					areqDoCommand: 
						SysBeep(0);
					otherwise
						theXmPtr^.result := xmUnknownRequest;
				end;
			end;
	end;
end.