unit XMGlue;
	{ Glue routines to be included only in XMOD projects }
	{ or in the app while debugging. }
interface
	uses
		XMGlobals;

	{ glue for 'xreq's, i.e. callbacks }

	function NewXMWindow (theXmPtr: xmPtr; wBounds: rect; wTitle: str255; wType: integer): WindowPtr;
	procedure CloseXMWindow (theXmPtr: xmPtr; window: WindowPtr);
	function GetXMMenu (theXmPtr: xmPtr; menuName: str15): MenuHandle;
	function CountXMWindows (theXmPtr: xmPtr; justMine: boolean): integer;
	function GetIndXMWindow (theXmPtr: xmPtr; i: integer; justMine: boolean; var owner: OSType): WindowPtr;
	procedure AlertUser (theXmPtr: xmPtr; message: str255);
	procedure ReceiveIdleEvents (theXmPtr: xmPtr; wantIdle: boolean);

	{ glue for 'areq's, i.e. calls from the app that the XMOD must answer }

	procedure DoGetXmInfo (theXmPtr: xmPtr; XmName, XmAuthor: str255);
	procedure DoCountCommands (theXmPtr: xmPtr; numCmds: integer);

implementation

	{ ====== xreq glue routines ====== }

	procedure DoCallback (theXmPtr: xmPtr; addr: ProcPtr);
	inline
		$205F, $4E90;

	{ ------------------------------- }

	function NewXMWindow (theXmPtr: xmPtr; wBounds: rect; wTitle: str255; wType: integer): WindowPtr;
	begin
		with theXmPtr^ do
			begin
				request := xreqNewXMWindow;
				param[1] := longint(wBounds.topLeft);
				param[2] := longint(wBounds.botRight);
				param[3] := longint(@wTitle);
				param[4] := longint(wType);
				DoCallback(theXmPtr, callbackAddr);
				NewXMWindow := WindowPtr(param[5]);
				if param[5] = 0 then
					result := xmFail
				else
					result := xmSuccess;
			end;
	end;

	{ ------------------------------- }

	procedure CloseXMWindow (theXmPtr: xmPtr; window: WindowPtr);
	begin
		with theXmPtr^ do
			begin
				request := xreqCloseXMWindow;
				param[1] := longint(window);
				DoCallback(theXmPtr, callbackAddr);
			end;
	end;

	{ ------------------------------- }

	procedure SetXMGlobal (theXmPtr: xmPtr; globalName: OSType; value: longint);
	begin
		with theXmPtr^ do
			begin
				request := xreqSetGlobal;
				param[1] := longint(globalName);
				param[2] := value;
				DoCallback(theXmPtr, callbackAddr);
			end;
	end;

	{ ------------------------------- }

	function GetXMGlobal (theXmPtr: xmPtr; globalName: OSType): longint;
	begin
		with theXmPtr^ do
			begin
				request := xreqGetGlobal;
				param[1] := longint(globalName);
				DoCallback(theXmPtr, callbackAddr);
				GetXMGlobal := param[2];
			end;
	end;

	{ ------------------------------- }

	function GetXMMenu (theXmPtr: xmPtr; menuName: str15): MenuHandle;
	begin
		with theXmPtr^ do
			begin
				request := xreqGetMenu;
				param[1] := longint(@menuName);
				DoCallback(theXmPtr, callbackAddr);
				GetXMMenu := MenuHandle(param[2]);
				if param[2] = 0 then
					result := xmFail
				else
					result := xmSuccess;
			end;
	end;

	{ ------------------------------- }

	function CountXMWindows (theXmPtr: xmPtr; justMine: boolean): integer;
	{ NB: This counts all windows, but some may not be visible! }
	begin
		with theXmPtr^ do
			begin
				request := xreqCountWindows;
				if justMine then
					param[1] := 1
				else
					param[1] := 0;
				DoCallback(theXmPtr, callbackAddr);
				CountXMWindows := param[2];
				result := xmSuccess;
			end;
	end;

	{ ------------------------------- }

	function GetIndXMWindow (theXmPtr: xmPtr; i: integer; justMine: boolean; var owner: OSType): WindowPtr;
	{ NB: The window returned may not be visible! }
	begin
		with theXmPtr^ do
			begin
				request := xreqGetIndWindow;
				param[1] := i;
				if justMine then
					param[2] := 1
				else
					param[2] := 0;
				DoCallback(theXmPtr, callbackAddr);
				GetIndXMWindow := WindowPtr(param[3]);
				owner := OSType(param[4]);
				if param[3] = 0 then
					result := xmFail
				else
					result := xmSuccess;
			end;
	end;

	{ ------------------------------- }

	procedure AlertUser (theXmPtr: xmPtr; message: str255);
	begin
		with theXmPtr^ do
			begin
				request := xreqSimpleAlert;
				param[1] := longint(@message);
				DoCallback(theXmPtr, callbackAddr);
				result := xmSuccess;
			end;
	end;

	{ ------------------------------- }

	procedure ReceiveIdleEvents (theXmPtr: xmPtr; wantIdle: boolean);
	begin
		with theXmPtr^ do
			begin
				request := xreqReceiveIdle;
				if wantIdle then
					param[1] := 1
				else
					param[1] := 0;
				DoCallback(theXmPtr, callbackAddr);
				result := xmSuccess;
			end;
	end;

	{ ====== areq glue routines ====== }

	procedure DoGetXmInfo (theXmPtr: xmPtr; XmName, XmAuthor: str255);
	begin
			{ The app must have given us empty handles in p[1] and p[2] }
		with theXmPtr^ do
			if (param[1] <> 0) and (param[2] <> 0) then
				begin
					SetString(StringHandle(param[1]), XmName);
					SetString(StringHandle(param[2]), XmAuthor);
				end;
	end;

	{ ------------------------------- }

	procedure DoCountCommands (theXmPtr: xmPtr; numCmds: integer);
	begin
		theXmPtr^.param[1] := numCmds;
	end;

	{ ------------------------------- }

	{ ------------------------------- }


end.