unit XMUtils;
	{ This unit stays in the parent application. }
interface
	uses
		Globals, XMGlobals, AppUtils;	{ **** Add XMOD unit to 'uses' list for debugging. **** }

	type
		typeCode = (tNil, tResFileID, tLongint, tPointer, tHandle, tWindowPtr, tMenuId, tGlobal, tIdleData);
		XmDataRecord = record
				xmResName: OSType;			{ if tGlobal, this is global's code }
				xmDataType: typeCode;
				xmData: longint;				{ cast to any of the above types    }
			end;
	const
		kDebugXMODName = 'Dbug';
		kMaxXMODs = 32;
	var
		gCallbackAddress: ProcPtr;
		gXmDatabase: handle;
		gXmIdleFlags: longint;

		{ Startup and shutdown routines }

	procedure InitXModules;
	procedure CloseXModules;

		{ XMOD database routines }

	function BlankXDRecord: XmDataRecord;
	function AddXmRecord (theXmRec: XmDataRecord): OSErr;
	function DeleteXmRecord (theXmRec: XmDataRecord): OSErr;
	function FindXmRecByName (theXmDataType: typeCode; theXmResName: OSType): XmDataRecord;
	function FindXmRecByData (theXmDataType: typeCode; theData: longint): XmDataRecord;
	function FindIndXmRec (which: integer): XmDataRecord;

		{ Idle event routines }

	procedure SetXmIdleFlag (theXmResName: OSType; setFlag: boolean);
	procedure GiveXMODsIdleTime;

		{ Event loop routines }

	function DoExternalEvent (event: EventRecord; var menuResult: longint): boolean;

		{ 'areq' glue routines }

		{ *** Not finished: must support the following: *** }
		{  areqGetIndCommand }
		{  areqGetErrorInfo  }

	function SendXmOpen (xmResName: OSType): OSErr;
	function SendXmClose (xmResName: OSType): OSErr;
	function SendXmEvent (xmResName: OSType; theEvent: EventRecord; evtData: longint; var didHandle: boolean): OSErr;
	function SendXmCommand (xmResName: OSType; commandName: str255): OSErr;
	function GetXmInfo (xmResName: OSType; var resName, authorStr: str255): OSErr;
	function CountXmCommands (xmResName: OSType; var numCommands: integer): OSErr;
	function SendXmIdle (xmResName: OSType): OSErr;

implementation

	{ ====== private to this unit ====== }

	function CallExternal (xmResName: OSType; myXmPtr: xmPtr): OSErr;
	{ This handles the details of an 'areq' call to the external. }
	{ Note the variations for debugging vs. true external code.   }

	{ **** Changed in Exp 8; must copy to Exp 7 **** }
		var
			codeHandle: handle;
			codePtr: ProcPtr;
			tempXDRec: XmDataRecord;
			resFileId: integer;

		{ ---------------------- }

		procedure DoJsr (theXmPtr: xmPtr; addr: ProcPtr);
		inline
			$205F, $4E90;

		{ ---------------------- }

	begin

		if gDebugMode then
			begin
{ codePtr := @main;       **** uncomment for debug mode **** }
				DoJsr(myXmPtr, codePtr);
				CallExternal := myXmPtr^.result;
			end
		else
			begin

				{ set the current res file as necessary }

				tempXDRec := FindXmRecByName(tResFileId, xmResName);
				resFileId := tempXDRec.xmData;
				if resFileId <> 0 then
					begin
						UseResFile(resFileId);

				{ get the code }

						codeHandle := Get1NamedResource(kXCodeType, xmResName);
						if codeHandle = nil then
							begin
								CallExternal := xmCantLoadCode;
								exit(CallExternal);
							end;
						MoveHHi(codeHandle);
						Hlock(codeHandle);
						codePtr := ProcPtr(codeHandle^);
					end;

				{ Jump to the external code. }

				DoJsr(myXmPtr, codePtr);

				{ Our info is in the Xm block, so  }
				{ we can release the code itself.  }
				if not gDebugMode then
					begin
						Hunlock(codeHandle);
						DetachResource(codeHandle);
						DisposHandle(codeHandle);
					end;
				CallExternal := myXmPtr^.result;
				UseResFile(gAppResId);
			end;
	end;

	{ ===========  XMOD database routines =========== }
	{ These use the global handle gXmDatabase         }
	{ which is just a concatenation of XmDataRecords. }

	function BlankXDRecord: XmDataRecord;
	{ Return a blank record. }
		var
			tempXDRec: XmDataRecord;
	begin
		with tempXDRec do
			begin
				xmResName := OSType(0);
				xmDataType := tNil;
				xmData := 0;
			end;
		BlankXDRecord := tempXDRec;
	end;

	{ ------------------------------- }

	function AddXmRecord (theXmRec: XmDataRecord): OSErr;
		var
			newBytes: integer;
			offset: longint;
	begin
		if gXmDatabase = nil then
			gXmDatabase := NewHandle(0);
		offset := GetHandleSize(gXmDatabase);
		newBytes := sizeOf(XmDataRecord);
		Hunlock(gXmDatabase);
		SetHandleSize(gXmDatabase, offset + newBytes);
		if MemError <> noErr then
			AddXmRecord := MemError
		else
			begin
				BlockMove(@theXmRec, ptr(ord4(gXmDatabase^) + offset), newBytes);
				AddXmRecord := noErr;
			end;
		MoveHHi(gXmDatabase);
		Hlock(gXmDatabase);
	end;

	{ ------------------------------- }

	function DeleteXmRecord (theXmRec: XmDataRecord): OSErr;
		{ Find the record by data type and XMOD name; }
		{ if it exists, delete it.                    }
		var
			lenRec, numRecs: integer;
			offset: longint;
			tempXDRec: XmDataRecord;
			found: boolean;
	begin
		DeleteXmRecord := noErr;
		tempXDRec := BlankXDRecord;
		if gXmDatabase <> nil then
			begin
				lenRec := sizeOf(XmDataRecord);
				numRecs := GetHandleSize(gXmDatabase) div lenRec;
				found := false;
				offset := 0;
				repeat
					BlockMove(ptr(ord4(gXmDatabase^) + offset), @tempXDRec, lenRec);
					if (theXmRec.xmDataType = tempXDRec.xmDataType) and (theXmRec.xmResName = tempXDRec.xmResName) then
						found := true
					else
						offset := offset + lenRec;
				until found or (offset >= numRecs * lenRec);
			end;

		if found then 	{ delete it and shrink the database accordingly }
			begin
				Hunlock(gXmDatabase);
				BlockMove(ptr(ord4(gXmDatabase^) + offset + lenRec), ptr(ord4(gXmDatabase^) + offset), numRecs * lenRec - offset);
				SetHandleSize(gXmDatabase, lenRec * (numRecs - 1));
				if MemError <> noErr then
					DeleteXmRecord := MemError;
				MoveHHi(gXmDatabase);
				Hlock(gXmDatabase);
			end;
	end;

	{ ======= startup routines ======= }

	procedure AddXMODsForCurrFile;
		var
			i, numXMODs, tempId: integer;
			codeHandle: handle;
			tempXDRec: XmDataRecord;
			tempType: ResType;
			tempStr: str255;
	begin
		numXMODs := Count1Resources(kXCodeType);
		if numXMODs > 0 then
			for i := 1 to numXMODs do
				begin
					codeHandle := Get1IndResource(kXCodeType, i);
					tempXDRec := BlankXDRecord;
					GetResInfo(codeHandle, tempId, tempType, tempStr);
					tempXDRec.xmResName := copy(tempStr, 1, 4);
					tempXDRec.xmData := HomeResFile(codeHandle);
					tempXDRec.xmDataType := tResFileId;
					if AddXmRecord(tempXDRec) <> noErr then
						SimpleAlert(kOutOfMemory, true);
				end;
	end;

	{ ------------------------------- }

	procedure FindAllXModules;
		{ Search the app's folder for all XMOD-type files. }
		var
			fileName: str255;
			fileIndex, appVRefNum, resFileNum: integer;
			err: OSErr;
			myPBPtr: CInfoPBPtr;
			isXMODFile: boolean;
			myFInfo: FInfo;
	begin
		err := GetVol(nil, appVRefNum);
		fileIndex := 1;
		repeat
			myPBPtr := CInfoPBPtr(NewPtrClear(sizeOf(CInfoPBRec)));
			with myPBPtr^ do
				begin
					ioNamePtr := @fileName;
					ioVRefNum := appVRefNum;
					ioFDirIndex := fileIndex;
				end;
			err := PBGetCatInfo(myPBPtr, false);
			if err = noErr then
				if not Btst(myPBPtr^.ioFlAttrib, 4) then				{ got a file, not a folder }
					if GetFInfo(fileName, 0, myFInfo) = noErr then
						begin
							isXMODFile := (myFInfo.fdType = kXMFileType) and (myFInfo.fdCreator = kAppType);
							if isXMODFile then					{ load and add to database }
								begin
									resFileNum := OpenResFile(fileName);
									if resFileNum <> -1 then
										AddXMODsForCurrFile;
								end;
						end;
			fileIndex := fileIndex + 1;
			DisposPtr(ptr(myPBPtr));
		until err <> noErr;
	end;

	{ ------------------------------- }

	procedure InitXModules;
	{ Call just once at app startup time. }
	{ Scans for XMOD resources and builds }
	{ the database for later dispatching. }
		var
			oldResFile, i, numXMODs: integer;
			tempXDRec: XmDataRecord;
			err: OSErr;
			realResName, authorStr: str255;
	begin
		oldResFile := CurResFile;
		gXmDatabase := nil;
		gXmIdleFlags := 0;
		if gDebugMode then
			begin
				tempXDRec := BlankXDRecord;
				tempXDRec.xmResName := kDebugXMODName;
				tempXDRec.xmData := oldResFile;
				tempXDRec.xmDataType := tResFileId;
					{ Get the real res name of our XMOD, then }
					{ make the correct entry in the database. }
				if GetXmInfo(tempXDRec.xmResName, realResName, authorStr) = noErr then
					begin
						tempXDRec.xmResName := realResName;
						if AddXmRecord(tempXDRec) <> noErr then
							SimpleAlert(kOutOfMemory, true);
					end;
			end
		else
			begin
				AddXMODsForCurrFile;	{ add XMOD's embedded in the app's res fork }
				FindAllXModules;			{ add XMOD's in other files in app's folder }
			end;
		UseResFile(oldResFile);

		{ Now send each XMOD its 'Open' call }

		numXMODs := GetHandleSize(gXmDatabase) div sizeOf(XmDataRecord);
		if numXMODs > 0 then
			for i := 1 to numXMODs do
				begin
					tempXDRec := FindIndXmRec(i);
					if tempXDRec.xmResName <> '' then
						err := SendXmOpen(tempXDRec.xmResName);
				end;
	end;

	{ ------------------------------- }

	procedure CloseXModules;
		{ Call only when the app is about to terminate. }
		var
			i, numXMODs: integer;
			tempXDRec: XmDataRecord;
			err: OSErr;
	begin

		{ Send each XMOD its 'Close' call  }
		{ The 'tResFileID' type is allocated just once and so }
		{ assures that each XMOD gets this message only once. }

		numXMODs := GetHandleSize(gXmDatabase) div sizeOf(XmDataRecord);
		if numXMODs > 0 then
			for i := 1 to numXMODs do
				begin
					tempXDRec := FindIndXmRec(i);
					if (tempXDRec.xmResName <> '') and (tempXDRec.xmDataType = tResFileID) then
						err := SendXmClose(tempXDRec.xmResName);
				end;

		if gXmDatabase <> nil then
			DisposHandle(gXmDatabase);
	end;

	{ ------------------------------- }

	function FindXmRecByName (theXmDataType: typeCode; theXmResName: OSType): XmDataRecord;
		var
			lenRec, numRecs: integer;
			offset: longint;
			tempXDRec: XmDataRecord;
			found: boolean;
	begin
		tempXDRec := BlankXDRecord;
		if gXmDatabase <> nil then
			begin
				lenRec := sizeOf(XmDataRecord);
				numRecs := GetHandleSize(gXmDatabase) div lenRec;
				found := false;
				offset := 0;
				repeat
					BlockMove(ptr(ord4(gXmDatabase^) + offset), @tempXDRec, lenRec);
					if (theXmDataType = tempXDRec.xmDataType) and EqualString(theXmResName, tempXDRec.xmResName, false, false) then
						found := true
					else
						offset := offset + lenRec;
				until found or (offset >= numRecs * lenRec);
			end;
		FindXmRecByName := tempXDRec;
	end;

	{ ------------------------------- }

	function FindXmRecByData (theXmDataType: typeCode; theData: longint): XmDataRecord;
		{ Match event data, e.g. menu id or windowPtr, to info in the database. }
		var
			lenRec, numRecs: integer;
			offset: longint;
			tempXDRec: XmDataRecord;
			found: boolean;
	begin
		tempXDRec := BlankXDRecord;
		if gXmDatabase <> nil then
			begin
				lenRec := sizeOf(XmDataRecord);
				numRecs := GetHandleSize(gXmDatabase) div lenRec;
				found := false;
				offset := 0;
				repeat
					BlockMove(ptr(ord4(gXmDatabase^) + offset), @tempXDRec, lenRec);
					if (theXmDataType = tempXDRec.xmDataType) and (theData = tempXDRec.xmData) then
						found := true
					else
						offset := offset + lenRec;
				until found or (offset >= numRecs * lenRec);
			end;
		if found then
			FindXmRecByData := tempXDRec
		else
			FindXmRecByData := BlankXDRecord;
	end;

	{ ------------------------------- }

	function FindIndXmRec (which: integer): XmDataRecord;
		{ Find the ith record; used at startup and quit time }
		{ to send 'Open' and 'Close' messages to all XMODs.  }
		var
			lenRec, numRecs: integer;
			offset: longint;
			tempXDRec: XmDataRecord;
			found: boolean;
	begin
		tempXDRec := BlankXDRecord;
		if gXmDatabase <> nil then
			begin
				lenRec := sizeOf(XmDataRecord);
				numRecs := GetHandleSize(gXmDatabase) div lenRec;
				if which in [1..numRecs] then
					begin
						offset := (which - 1) * lenRec;
						BlockMove(ptr(ord4(gXmDatabase^) + offset), @tempXDRec, lenRec);
					end;
			end;
		FindIndXmRec := tempXDRec;
	end;

	{ ====== procedures to manage idle event sending to XMODs ====== }

	procedure SetXmIdleFlag (theXmResName: OSType; setFlag: boolean);
		{ The longint gXmIdleFlags is a set of 32 bits, for up to  }
		{ 32 XMODs.  Setting a bit implies that it should receive  }
		{ idle events.  Getting the correct name for the ith bit   }
		{ relies on the gXmDatabase loading all tResFileID records }
		{ first at launch time. }
		var
			i: integer;
			tempXDRec: XmDataRecord;
			found: boolean;
	begin
		i := 1;
		found := false;
		repeat
			tempXDRec := FindIndXmRec(i);
			if (tempXDRec.xmDataType = tResFileID) and (tempXDRec.xmResName = theXmResName) then
				found := true
			else
				i := i + 1;
		until found or (tempXDRec.xmDataType <> tResFileID);
		if found then
			if setFlag then
				BitSet(@gXmIdleFlags, i - 1)		{ i-1 since bit numbering starts at zero }
			else
				BitClr(@gXmIdleFlags, i - 1)
		else
	end;

	{ ------------------------------- }

	procedure GiveXMODsIdleTime;
		{ Called from the main event loop. }
		var
			i: integer;
			tempXDRec: XmDataRecord;
			err: OSErr;
	begin
		if (gXmDatabase <> nil) and (gXmIdleFlags <> 0) then
			for i := 1 to kMaxXMODs do
				if BitTst(@gXmIdleFlags, i - 1) then	{ the ith XMOD wants idle events }
					begin
						tempXDRec := FindIndXmRec(i);
						err := SendXmIdle(tempXDRec.xmResName);
					end;
	end;

	{ ====== event dispatcher called from the app's main event loop ====== }

	function DoExternalEvent (event: EventRecord; var menuResult: longint): boolean;
		{ Returns true if an XMOD 'owns' and will get sent this event. }
		var
			part, err, menuId: integer;
			window: WindowPtr;
			evtData: longint;	{ cast as WindowPtr or MenuResult, depending on event type }
			key: char;
			handledByXMOD: boolean;
			tempXDRec: XmDataRecord;
	begin
		handledByXMOD := false;
		tempXDRec := BlankXDRecord;
		menuResult := 0;

		{ Dispatch the menu event to the appropriate external.  }
		{ Note that several events are ignored here so that the }
		{ app event system will have to deal with them.         }

		case event.what of

			mouseDown: 
				begin
					part := FindWindow(event.where, window);
					case part of
						inMenuBar: 
							begin
							{ Can't call MenuSelect twice; it eats the menu action, so }
							{ it's called here only.  If the menu is an XMOD menu, we  }
							{ pass the event along, but if it was an app menu, the var }
							{ menuResult is used to pass the info back to the main     }
							{ event loop. }

								menuResult := MenuSelect(event.where);
								menuID := HiWrd(menuResult);
								tempXDRec := FindXmRecByData(tMenuId, menuID);
							end;
						inContent: 
							if window = FrontWindow then
								tempXDRec := FindXmRecByData(tWindowPtr, longint(window));
						inGoAway: 
							begin
								if TrackGoAway(window, event.where) then
									HideWindow(window);
								DoExternalEvent := true;
								exit(DoExternalEvent);
							end;
						otherwise
					end;
				end;

			keyDown, autoKey: 
				begin
					key := chr(BAnd(event.message, charCodeMask));
					if (BAnd(event.modifiers, cmdKey) <> 0) then		{ process as menu event }
						begin
							menuID := HiWrd(MenuKey(key));
							tempXDRec := FindXmRecByData(tMenuId, menuID);
						end
					else
						begin
						{ Dispatch as event for the front window; the XMOD }
						{ will test to see if it should process it.        }
							window := FrontWindow;
							tempXDRec := FindXmRecByData(tWindowPtr, longint(window));
						end;
				end;

			activateEvt: 
				begin
					window := WindowPtr(event.message);
					if IsAppWindow(window) then
						tempXDRec := FindXmRecByData(tWindowPtr, longint(window));
				end;

			updateEvt: 
				begin
					window := WindowPtr(event.message);
					if IsAppWindow(window) and not EmptyRgn(window^.visRgn) then
						tempXDRec := FindXmRecByData(tWindowPtr, longint(window));
				end;

			kOSEvent: 
				case BAnd(BRotL(event.message, 8), $FF) of
					kSuspendResumeMessage: 
						begin
							{ *** NB: We ignore this for now, to see later if }
							{    update & activate evts will get sent anyway. }
							{      gInBackground := BAnd(event.message, kResumeMask) = 0; }
							{      DoActivate(FrontWindow, not gInBackground); }
						end;
				end;
			otherwise
		end;

		{ If the menu or window matched any known XMOD, }
		{ send that XMOD the event now.                 }

		if menuResult <> 0 then
			evtData := menuResult
		else
			evtData := longint(window);

		if tempXDRec.xmDataType <> tNil then		{ found it }
			if SendXmEvent(tempXDRec.xmResName, event, evtData, handledByXMOD) = noErr then
				;
		DoExternalEvent := handledByXMOD;
	end;

	{ ====== headers for 'areq' calls to the external ====== }

	function SendXmOpen (xmResName: OSType): OSErr;
		var
			myXmPtr: xmPtr;
	begin
		myXmPtr := xmPtr(NewPtrClear(sizeOf(xmBlock)));
		if myXmPtr <> nil then
			with myXmPtr^ do
				begin
					callbackAddr := gCallbackAddress;
					result := xmSuccess;
					request := areqOpen;
					SendXmOpen := CallExternal(xmResName, myXmPtr);
				end;
		DisposPtr(ptr(myXmPtr));
	end;

	{ ------------------------------- }

	function SendXmClose (xmResName: OSType): OSErr;
		var
			myXmPtr: xmPtr;
	begin
		myXmPtr := xmPtr(NewPtrClear(sizeOf(xmBlock)));
		if myXmPtr <> nil then
			with myXmPtr^ do
				begin
					callbackAddr := gCallbackAddress;
					result := xmSuccess;
					request := areqClose;
					SendXmClose := CallExternal(xmResName, myXmPtr);
				end;
		DisposPtr(ptr(myXmPtr));
	end;

	{ ------------------------------- }

	function SendXmCommand (xmResName: OSType; commandName: str255): OSErr;
	{ This routine is normally entered in }
	{ response to a script language call. }
	{ param[1] is a StringHandle to the command name }
		var
			myXmPtr: xmPtr;
			tempH: handle;
	begin
		myXmPtr := xmPtr(NewPtrClear(sizeOf(xmBlock)));
		if myXmPtr <> nil then
			with myXmPtr^ do
				begin
					callbackAddr := gCallbackAddress;
					result := xmSuccess;

					request := areqDoCommand;
					tempH := handle(NewString(commandName));
					if tempH = nil then
						begin
							DisposPtr(ptr(myXmPtr));
							SendXmCommand := xmFail;
							exit(SendXmCommand);
						end;
					MoveHHi(tempH);
					Hlock(tempH);
					param[1] := longint(tempH);

					SendXmCommand := CallExternal(xmResName, myXmPtr);

					Hunlock(tempH);
					DisposHandle(tempH);
				end;
		DisposPtr(ptr(myXmPtr));
	end;

	{ ------------------------------- }

	function SendXmEvent (xmResName: OSType; theEvent: EventRecord; evtData: longint; var didHandle: boolean): OSErr;
		var
			myXmPtr: xmPtr;
	begin
		myXmPtr := xmPtr(NewPtrClear(sizeOf(xmBlock)));
		if myXmPtr <> nil then
			with myXmPtr^ do
				begin
					callbackAddr := gCallbackAddress;
					result := xmSuccess;
					request := areqDoEvent;
					param[1] := longint(@theEvent);
					param[2] := evtData;		{ cast as WindowPtr or MenuResult, depending on event type }
					SendXmEvent := CallExternal(xmResName, myXmPtr);
					didHandle := param[3] <> 0;
				end;
		DisposPtr(ptr(myXmPtr));
	end;

	{ ------------------------------- }

	function GetXmInfo (xmResName: OSType; var resName, authorStr: str255): OSErr;
		var
			myXmPtr: xmPtr;
	begin
		myXmPtr := xmPtr(NewPtrClear(sizeOf(xmBlock)));
		if myXmPtr <> nil then
			with myXmPtr^ do
				begin
					callbackAddr := gCallbackAddress;
					result := xmSuccess;

					request := areqGetInfo;
					{ We provide empty handles to the XMOD... }
					param[1] := longint(NewHandle(0));
					param[2] := longint(NewHandle(0));

					GetXmInfo := CallExternal(xmResName, myXmPtr);

					{ so we dispose of those handles, too.    }
					resName := StringHandle(param[1])^^;
					DisposHandle(handle(param[1]));
					authorStr := StringHandle(param[2])^^;
					DisposHandle(handle(param[2]));
				end;
		DisposPtr(ptr(myXmPtr));
	end;

	{ ------------------------------- }

	function CountXmCommands (xmResName: OSType; var numCommands: integer): OSErr;
	{ param[1] is numCommands }
		var
			myXmPtr: xmPtr;
	begin
		myXmPtr := xmPtr(NewPtrClear(sizeOf(xmBlock)));
		if myXmPtr <> nil then
			with myXmPtr^ do
				begin
					callbackAddr := gCallbackAddress;
					result := xmSuccess;

					request := areqCountCommands;
					CountXmCommands := CallExternal(xmResName, myXmPtr);
					numCommands := param[1];
				end;
		DisposPtr(ptr(myXmPtr));
	end;

	{ ------------------------------- }

	function SendXmIdle (xmResName: OSType): OSErr;
		{ This call does not send or receive any parameters. }
		var
			myXmPtr: xmPtr;
	begin
		myXmPtr := xmPtr(NewPtrClear(sizeOf(xmBlock)));
		if myXmPtr <> nil then
			with myXmPtr^ do
				begin
					callbackAddr := gCallbackAddress;
					result := xmSuccess;
					request := areqDoIdle;
					SendXmIdle := CallExternal(xmResName, myXmPtr);
				end;
		DisposPtr(ptr(myXmPtr));
	end;

	{ ------------------------------- }

		{ *** Not finished: must support the following: *** }
		{  areqGetIndCommand }
		{  areqGetErrorInfo  }

	{ ------------------------------- }

end.