/*
 	AboutBoxDirectorC
 	Written and copyright by Malcolm H. Teas 1991. All rights reserved.
 
 	This file is a standalone class group that handles an about box made of a PICT
 	resource.  It's resource fork contains the sample resources for a window, picture
 	pane, and a PICT.  The AboutBoxPictureC is defined here to add the DoClick method
 	to the CPicture class.
 	
 	Notes:
 		 Probably ought to add some error catchers.
*/


#include <CPicture.h>
#include <CWindow.h>
#include <CDecorator.h>
#include <Events.h>

#include "CAboutBoxDirector.h"


/*	External global references. */
extern CDesktop			*gDesktop;			/* The visible Desktop. */
extern CDecorator		*gDecorator;		/* The window arranger. */

VersRecHndl	vh;								//	For the version information.


/*	The picture subclass that passes the DoClick message to the about box director. */
class CAboutBoxPicture : public CPicture  {
	public:
		void DoClick (Point hitPt, short modifiers, long when);
		void Draw (Rect *area);
};


/*	Initialize the about box by calling the inherited method.  Then create
	the about box window and it's pane from the resource fork.  Don't show them yet. */
void CAboutBoxDirector::IAboutBoxDirector (CDirectorOwner *aSupervisor, short resourceID)
{
	LongRect	LRect;
	Rect		ignore;
	short		width, height;
	
	itsDefaultPane = nil;
	
	inherited::IDirector (aSupervisor);
	
	itsWindow = new CWindow;				//	Make a modal window.
	FailNIL (itsWindow);
	itsWindow->IWindow (resourceID, false, gDesktop, this);
	itsWindow->SetModal (true);
	
	itsDefaultPane = new CAboutBoxPicture;	//	Make it's picture pane.
	FailNIL (itsDefaultPane);
	itsDefaultPane->IViewRes ('PctP', resourceID, itsWindow, this);
	FailResError ();
	FailMemError ();

	/*	Make the window the same size as the pane which in turn is the same
		size as the PICT. */
	((CPicture *)itsDefaultPane)->FrameToBounds ();
	itsDefaultPane->GetFrame (&LRect);
	width = LRect.right - LRect.left;
	height = LRect.bottom - LRect.top;
	itsWindow->ChangeSize (width, height);
	
	gDecorator->CenterWindow (itsWindow);	//	And center window.
		
	itsGopher = this;						//	Make this director the gopher.
	
	//	Get the version info if any, this'll leave a nil if nothing's found.
	vh = (VersRecHndl) GetResource ('vers', 1);
}


/*	Free the allocated memory and restore the state of the world. */ 
void CAboutBoxDirector::Dispose (void)
{
	itsWindow->Deactivate ();
	itsDefaultPane->Dispose ();
	inherited::Dispose ();
}


/*	Close the about box. */
void CAboutBoxDirector::DoKeyDown (char c, Byte code, EventRecord *event)
{
	this->Dispose ();
}


/*	Show the about box for "ticks" ticks.  Then hide it again. If the user
	pressed a key or the mouse button, the hide the about box early.  */
void CAboutBoxDirector::Flash (long ticks)
{
	long	startTicks;
	KeyMap	keys;
	
	itsWindow->Select ();
	itsWindow->Activate ();
	itsWindow->Update ();
	for (startTicks = TickCount (); startTicks + ticks >= TickCount (); )  {
		GetKeys (keys);
		if (Button() || keys[0] || keys [1] || keys[2] || keys[3])
			break;
	}
	
	this->Dispose ();
}


/*	Show the about box until the user presses either a key or the mouse button. */
void CAboutBoxDirector::About (void)
{
	itsWindow->Select ();
}


/*	This is the method that handles a click in the about box picture. */
void CAboutBoxPicture::DoClick (Point hitPt, short modifiers, long when)
{
	itsSupervisor->Dispose ();
}


/*	Draw the picture and overlay the long version string from the vers resource (if
	there's one) in the upper right corner.  Use Geneva 10 font.  */
void CAboutBoxPicture::Draw(Rect *area)
{
	short		font;
	StringPtr	string;
	short		size;
	LongRect	r;
	
	inherited::Draw (area);
	
	if (vh == nil)
		return;
	
	LoadResource ((Handle) vh);
		
	GetFNum ("\pGeneva", &font);
	TextFont (font);
	TextSize (10);
	
	string = (*vh)->shortVersion[0] + 1 + (*vh)->shortVersion;
	size = StringWidth (string);
	GetAperture (&r);
	MoveTo (r.right - size - 15, 15);
	DrawString (string);
}
