/* Tree.h - data struct and protos for Tree.c */

#include "Hiccup.h"

typedef short (*TreeCompProc)(void *newDataPtr,
	void *dataHolder, long dataIndex);
typedef long (*TreeAddProc)(void *dataHolder,
	void *newDataPtr);
typedef long (*TreeAppendProc)(void *dataHolder,
	long oldDataIndex, void *newDataPtr);
typedef Boolean (*TreeProc)(void *dataHolder,
	long dataIndex);

#ifndef ACCESS_TREE_DATA
typedef struct HIDDEN_Tree
	{
	HH				nodeHH;
	void 			*dataHolder; // typ HHPtr or Handle
	TreeCompProc	compProc;
	TreeAddProc		addProc;
	TreeAppendProc	appendProc;
	} HIDDEN_Tree;

typedef struct Tree
	{
	char	contents[sizeof(HIDDEN_Tree)];
	} Tree, *TreePtr;

typedef void *TreeNodePtr;

#else

typedef struct Tree
	{
	HH				nodeHH;
	void 			*dataHolder;// typ HHPtr or Handle
	TreeCompProc	compProc;
	TreeAddProc		addProc;
	TreeAppendProc	appendProc;
	} Tree, *TreePtr;

typedef struct TreeNode
	{
	short		left,	// child is > 0,
				right;	// thread is <= 0
	/* data */
	long	dataIndex;//into "dataHolder"
	} TreeNode, *TreeNodePtr;

#endif

Boolean TreeNew(TreePtr tP, short numToStart, short numInc,
	TreeCompProc compProc, TreeAddProc addProc,
	TreeAppendProc appendProc, void *dataHolder);
short AddToTree(TreePtr tP, void *dataPtr);
Handle TreeClose(TreePtr tP);
void TreeDispose(TreePtr tP);
short NextNode(TreePtr tP, short oldIndex);
short PreviousNode(TreePtr tP, short oldIndex);
short FirstNodeMatching(TreePtr tP,
	TreeCompProc tcp, void *dataPtr);
Boolean ForEachNodeInOrderDo(Boolean forwards, 
	TreePtr tP, TreeProc theDoer);
