/* TreeExample.c : build tree of C strings in  ASCII  order,
with "strcmp" as the basic  comparison  procedure.  Data  is
handled two ways; tracked in an array of structs,  and  left
in original location (allPurposeReplies[] array).  Functions
for this second way  are  prefixed  with  "ND_",  for
"No Data allocated". */

#include <stdio.h>
#include <string.h>
//full access to tree internals not needed here.
//#define ACCESS_TREE_DATA
#include "Tree.h"
//#undef ACCESS_TREE_DATA

//...does your CPU byte?
static char *allPurposeReplies[] = 
	{
"You're a person of few characters, aren't you?",
"Abbreviated responses only annoy, please stop that.",
"Abrv rsps nly any, pls stp tht.",
"Couldn't fail not to disagree with you more...",
"Sorry, now in IBM mode, no intelligent responses \
available...",
"Maybe you'd do better if you tried dropping things \
at random on the keyboard?",
"Do you smell something funny? A bit like wood \
burning\n\tin a vacuum, or maybe porridge trying to go\n\t\
where no porridge has gone before?",
"Have you considered leaving your brain to science?",
"Have you considered leaving your brain to \
Campbell's Soup?",
"Blow it out your ear, lowlife wetware.",
""
	};

/* The above strings will first be tracked in  an  array  of
structs recording  the  address  of  each  string  (and  the
length).  This  array  of  C_StringRecord  structs  will  be
contained in the Handle replyHH.h */
typedef struct C_StringRecord
	{
	short	len;
	Ptr		str;
	} C_StringRecord, *CSRPtr;

void main(void);
Boolean TreeUpReplies(TreePtr tP, HHPtr replyHP);
void PrintAlphaOrder(TreePtr tP);
short C_StringCompProc(void *newDataPtr,
	void *dataHolder, long dataIndex);
long C_StringAddProc(void *dataHolder, void *newDataPtr);
Boolean PrintOne(void *dataHolder, long dataIndex);
Boolean ND_TreeUpReplies(TreePtr tP);
void ND_PrintAlphaOrder(TreePtr tP);
short ND_C_StringCompProc(void *newDataPtr,
	void *dataHolder, long dataIndex);
long ND_C_StringAddProc(void *dataHolder, void *newDataPtr);
Boolean ND_PrintOne(void *dataHolder, long dataIndex);

/* First approach: as tree "replyTree" is  built  also  copy
string addresses from allPurposeReplies[] into a new  Handle
replyHH.h. The "dataIndex" in each node is the  array  index
of  the  appropriate  "C_StringRecord"  entry  in  this  new
Handle.
However, there is no need to  build  a  Handle  holding  the
strings - can use the array allPurposeReplies as-is. In this
case, the dataIndex is the string index ("whichString")  and
no new data structure (replyHH) is needed. For variety, this
second tree "replyTreeND" is printed in reverse order. */
void main(void)
	{
	Tree	replyTree, replyTreeND;
	HH		replyHH;
	
	if (TreeUpReplies(&replyTree, &replyHH)
		&& ND_TreeUpReplies(&replyTreeND))
		{
		PrintAlphaOrder(&replyTree);
		HicDispose(&replyHH);
		TreeDispose(&replyTree);
		ND_PrintAlphaOrder(&replyTreeND);
		TreeDispose(&replyTreeND);
		}
	}


Boolean TreeUpReplies(TreePtr tP, HHPtr replyHP)
	{
	short	whichString = 0, treeIndex;
	Boolean	result;
	
	result = HicNew(replyHP, 40, 10);
	if (!result)
		return FALSE;
	result = TreeNew(tP, 6, 3, C_StringCompProc,
				C_StringAddProc, NULL, replyHP);
	if (!result)
		{
		HicDispose(replyHP);
		return FALSE;
		}
	while (allPurposeReplies[whichString][0])
		{
		treeIndex =
			AddToTree(tP, allPurposeReplies[whichString++]);
		if (!treeIndex)
			{
			HicDispose(replyHP);
			TreeDispose(tP);
			return FALSE;
			}
		}
	(void)TreeClose(tP);
	HicClose(replyHP);
	return TRUE;
	}

void PrintAlphaOrder(TreePtr tP)
	{
	(void)ForEachNodeInOrderDo(TRUE, tP, PrintOne);
	}

/* "dataHolder" points to an HH whose handle holds an  array
of C_StringRecord structs, each of which records the address
and length of a string from "allPurposeReplies". "dataIndex"
is in units of C_StringRecord. */
short C_StringCompProc(void *newDataPtr,
	void *dataHolder, long dataIndex)
	{
	HHPtr	hP = (HHPtr)dataHolder;
	CSRPtr	basePtr = (CSRPtr)(*(hP->h)),
			oldCSRPtr = basePtr + dataIndex;
	Ptr		oldStringPtr = oldCSRPtr->str,
			newStringPtr = (Ptr)newDataPtr;
	short		cmp;
	
	if (dataIndex < 0) // root
		return -1;
	cmp = strcmp(newStringPtr, oldStringPtr);
	return cmp;
	}

long C_StringAddProc(void *dataHolder, void *newDataPtr)
	{
	
	HHPtr	hP = (HHPtr)dataHolder;
	C_StringRecord	csr;
	CSRPtr	basePtr = (CSRPtr)(*(hP->h)),
			nextPtr = (CSRPtr)(*(hP->h) + hP->freePos);
	long	dataIndex = nextPtr - basePtr;
	Boolean	result;
	
	csr.str = (Ptr)newDataPtr;
	csr.len = strlen(csr.str);
	result = HicAddPtrSize(hP, &csr, sizeof(csr));
	if (!result)
		return -1L;
	return dataIndex;
	}

Boolean PrintOne(void *dataHolder, long dataIndex)
	{
	HHPtr	hP = (HHPtr)dataHolder;
	CSRPtr	basePtr = (CSRPtr)(*(hP->h)),
			oldCSRPtr = basePtr + dataIndex;
	Ptr		oldStringPtr = oldCSRPtr->str;
	short	len = oldCSRPtr->len;
	
	printf("%d %s\n", len, oldStringPtr);
	return TRUE;
	}

Boolean ND_TreeUpReplies(TreePtr tP)
	{
	short	whichString = 0, treeIndex;
	Boolean	result;
	
	result = TreeNew(tP, 10, 4, ND_C_StringCompProc,
		ND_C_StringAddProc, NULL, allPurposeReplies);
	if (!result)
		{
		return FALSE;
		}
	while (allPurposeReplies[whichString][0])
		{
		treeIndex =
			AddToTree(tP, (void *)(whichString++));
		if (!treeIndex)
			{
			TreeDispose(tP);
			return FALSE;
			}
		}
	(void)TreeClose(tP);
	return TRUE;
	}

void ND_PrintAlphaOrder(TreePtr tP)
	{
	(void)ForEachNodeInOrderDo(FALSE, tP, ND_PrintOne);
	}

short ND_C_StringCompProc(void *newDataPtr,
	void *dataHolder, long dataIndex)
	{
	char	**base = (char **)dataHolder;
	Ptr		newStringPtr = base[(long)newDataPtr],
			oldStringPtr = base[dataIndex];
	short	cmp;
	
	if (dataIndex < 0) // root
		return -1;
	cmp = strcmp(newStringPtr, oldStringPtr);
	// or cmp = strcmp(allPurposeReplies[(long)newDataPtr],
	//			allPurposeReplies[dataIndex]);
	return cmp;
	}

long ND_C_StringAddProc(void *dataHolder, void *newDataPtr)
	{
		
	return((long)newDataPtr);
	}

Boolean ND_PrintOne(void *dataHolder, long dataIndex)
	{
	char	**base = (char **)dataHolder;
	Ptr		oldStringPtr = base[dataIndex];
	
	printf("%s\n", oldStringPtr);
	//or printf("%s\n", allPurposeReplies[dataIndex]);
	return TRUE;
	}
