/********************************************************/
/*														*/
/*  This code is										*/
/*	Copyright 1993, Dave Mark, All Rights Reserved  	*/
/*														*/
/********************************************************/

#define kMBARResID			128
#define kSleep				60L
#define kMoveToFront		(WindowPtr)-1L
#define kNilFilterProc		(ProcPtr)0L
#define kEmptyString		"\p"
#define kHasGoAway			true
#define kInvisible			false

#define kListDefProc		0
#define kDontDrawYet		false
#define kHasGrow			true
#define kHasHScroll			true
#define kHasVScroll			true
#define kFindNext			true

#define kListWindow			0
#define kDAWindow			1
#define kUnknownWindow		2
#define kPictWindow			3
#define kNilWindow			4

#define kMinWindowWidth		210
#define kMinWindowHeight	63
#define kWindowHeight		255
#define kMinPictWinHeight	50
#define kMinPictWinWidth	150

#define mApple				128
#define iAbout				1

#define mFile				129
#define iNewList			1
#define iClose				2
#define iQuit				4

#define kErrorAlertID		128


/************************/
/*		Typedefs		*/
/************************/

typedef	struct
{
	WindowRecord	w;
	short			wType;
	ListHandle		list;
} ListRecord, *ListPeek;

typedef	struct
{
	WindowRecord	w;
	short			wType;
	short			PictResID;
} PictRecord, *PictPeek;


/*************/
/*  Globals  */
/*************/

Boolean		gDone;
short		gNewWindowX = 20, gNewWindowY = 50;


/***************/
/*  Functions  */
/***************/

void	ToolboxInit( void );
void	MenuBarInit( void );
void	CreateListWindow( void );
void	DestroyWindow( WindowPtr w );
void	EventLoop( void );
void	DoEvent( EventRecord *eventPtr );
void	DoUpdate( EventRecord *eventPtr );
void	DoActivate( EventRecord *eventPtr );
void	HandleMouseDown( EventRecord *eventPtr );
void	DoContentClick( EventRecord *eventPtr, WindowPtr w );
void	CreatePictWindow( ListHandle list );
void	BumpGlobalXandY( void );
void	DoGrow( EventRecord *eventPtr, WindowPtr w );
void	HandleMenuChoice( long menuChoice );
void	HandleAppleChoice( short item );
void	HandleFileChoice( short item );
void	CenterWindow( WindowPtr w );
void	CenterPict( PicHandle picture, Rect *destRectPtr );
short	WindowType( WindowPtr window );
void	DoError( Str255 errorString );


/**************************** main **********************/

void	main( void )
{
	ToolboxInit();
	MenuBarInit();
	
	EventLoop();
}


/****************** ToolboxInit *********************/

void	ToolboxInit( void )
{
	InitGraf( &thePort );
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs( nil );
	InitCursor();
}


/****************** MenuBarInit ***********************/

void	MenuBarInit( void )
{
	Handle			menuBar;
	MenuHandle		menu;
	
	menuBar = GetNewMBar( kMBARResID );
	SetMenuBar( menuBar );

	menu = GetMHandle( mApple );
	AddResMenu( menu, 'DRVR' );
	
	DrawMenuBar();
}


/********************************	CreateListWindow	*******/

void	CreateListWindow( void )
{
	Rect			r, dataBounds;
	WindowPtr		w;
	Point			cSize, cIndex;
	ListHandle		list;
	short			i, dummy, numPicts;
	Handle			rHandle;
	short			resID;
	ResType			theResType;
	Str255			rName;
	Ptr				wStorage;
	ListPeek		l;
	
	SetRect( &r, gNewWindowX, gNewWindowY,
		gNewWindowX + kMinWindowWidth, gNewWindowY + kWindowHeight);
	
	BumpGlobalXandY();
	
	wStorage = NewPtr( sizeof( ListRecord ) );
	
	w = NewWindow( wStorage, &r, "\pPicture List", kInvisible,
		documentProc, kMoveToFront, kHasGoAway, 0L );
	
	SetPort( w );
	TextFont( systemFont );
	
	SetRect( &dataBounds, 0, 0, 1, 0 );
	SetPt( &cSize, 0, 0 );
	SetRect (&r, 0, 0, kMinWindowWidth -15, kWindowHeight -15);
	
	list = LNew( &r, &dataBounds, cSize, kListDefProc,
		w, kDontDrawYet, kHasGrow, kHasHScroll, kHasVScroll );
	
	(**list).selFlags = lOnlyOne;
	
	l = (ListPeek)w;
	
	l->wType = kListWindow;
	l->list = list;
	
	numPicts = CountResources( 'PICT' );
			
	for ( i = 0; i<numPicts; i++ )
	{
		rHandle = GetIndResource( 'PICT', i + 1 );
		GetResInfo( rHandle, &resID, &theResType, rName );
		
		dummy = LAddRow( 1, i, list );
		SetPt( &cIndex, 0, i );
		
		if ( rName[ 0 ] > 0 )
			LAddToCell( &(rName[1]), rName[0], cIndex, list );
		else
			LAddToCell( "<Unnamed>", 10, cIndex, list );
	}
	
	ShowWindow( w );
	LDoDraw( true, list );
}


/********************************	DestroyWindow	*******/

void	DestroyWindow( WindowPtr w )
{
	ListPeek		l;
	
	if ( WindowType( w ) == kListWindow )
	{
		HideWindow( w );
		l = (ListPeek)w;
		
		LDispose( l->list );
		
		CloseWindow( w );
		
		DisposePtr( (Ptr)w );
	}
	else if ( WindowType( w ) == kPictWindow )
	{
		CloseWindow( w );
		DisposePtr( (Ptr)w );
	}
}


/************************************* EventLoop */

void	EventLoop( void )
{		
	EventRecord		event;
	
	gDone = false;
	while ( gDone == false )
	{
		if ( WaitNextEvent( everyEvent, &event, kSleep, NULL ) )
			DoEvent( &event );
	}
}


/************************************* DoEvent */

void	DoEvent( EventRecord *eventPtr )
{
	char	theChar;
	
	switch ( eventPtr->what )
	{
		case mouseDown: 
			HandleMouseDown( eventPtr );
			break;
		case keyDown:
		case autoKey:
			theChar = eventPtr->message & charCodeMask;
			if ( (eventPtr->modifiers & cmdKey) != 0 ) 
				HandleMenuChoice( MenuKey( theChar ) );
			break;
		case updateEvt:
			DoUpdate( eventPtr );
			break;
		case activateEvt:
			DoActivate( eventPtr );
			break;
	}
}


/************************************* DoUpdate	 */

void	DoUpdate( EventRecord *eventPtr )
{
	WindowPtr		w;
	short			numPicts, i;
	ListPeek		l;
	ListHandle		list;
	GrafPtr			oldPort;
	Rect			r;
	Point			cellIndex;
	PicHandle		pic;
	PictPeek		p;
	
	w = (WindowPtr)(eventPtr->message);
	BeginUpdate( w );
	
	if ( WindowType( w ) == kListWindow )
	{
		GetPort( &oldPort );
		SetPort( w );
		
		l = (ListPeek)w;
		list = l->list;
		
		DrawGrowIcon( w );

		LUpdate( (**list).port->visRgn, list );
		
		SetPort( oldPort );
	}
	else if ( WindowType( w ) == kPictWindow )
	{
		GetPort( &oldPort );
		SetPort( w );
		
		r = w->portRect;
		
		p = (PictPeek)w;
		
		pic = GetPicture( p->PictResID );
		
		CenterPict( pic, &r );
		DrawPicture( pic, &r );
		
		SetPort( oldPort );
	}
	EndUpdate( w );
}


/************************************* DoActivate	 */

void	DoActivate( EventRecord *eventPtr )
{
	WindowPtr		w;
	ListPeek		l;
	ListHandle		list;
	
	w = (WindowPtr)(eventPtr->message);
	
	if ( WindowType( w ) == kListWindow )
	{
		l = (ListPeek)w;
		list = l->list;
		
		if ( eventPtr->modifiers & activeFlag )
			LActivate( true, list );
		else
			LActivate( false, list );
		
		DrawGrowIcon( w );
	}
}


/************************************* HandleMouseDown */

void	HandleMouseDown( EventRecord *eventPtr )
{
	WindowPtr		window;
	short			thePart;
	long			menuChoice;
	GrafPtr			oldPort;
	long			windSize;
	Rect			growRect;
	
	thePart = FindWindow( eventPtr->where, &window );
	
	switch ( thePart )
	{
		case inMenuBar:
			menuChoice = MenuSelect( eventPtr->where );
			HandleMenuChoice( menuChoice );
			break;
		case inSysWindow : 
			SystemClick( eventPtr, window );
			break;
		case inContent:
			DoContentClick( eventPtr, window );
			break;
		case inGrow:
			DoGrow( eventPtr, window );
			break;
		case inDrag : 
			DragWindow( window, eventPtr->where, &screenBits.bounds );
			break;
		case inGoAway:
			if ( TrackGoAway( window, eventPtr->where ) )
				DestroyWindow( window );
			break;
	}
}


/************************************* DoContentClick	 */

void	DoContentClick( EventRecord *eventPtr, WindowPtr w )
{
	GrafPtr			oldPort;
	ListHandle		list;
	ListPeek		l;
	
	if ( w != FrontWindow() )
	{
		SelectWindow( w );
	}
	else if ( WindowType( w ) == kListWindow )
	{
		GetPort( &oldPort );
		SetPort( w );
		
		GlobalToLocal( &(eventPtr->where) );
		
		l = (ListPeek)w;
		list = l->list;
		
		if ( LClick( eventPtr->where, eventPtr->modifiers, list ) )
			CreatePictWindow( list );
		SetPort( oldPort );
	}
}


/***************************** CreatePictWindow	 */

void	CreatePictWindow( ListHandle list )
{
	Cell			cell;
	PicHandle		pic;
	Handle			rHandle;
	Rect			r;
	short			resID;
	ResType			theResType;
	Str255			rName;
	PictPeek		p;
	Ptr				wStorage;
	WindowPtr		w;
	
	SetPt( &cell, 0, 0 );
	
	if ( LGetSelect( kFindNext, &cell, list ) )
	{
		rHandle = GetIndResource( 'PICT', cell.v + 1 );
		pic = (PicHandle)rHandle;
		
		r = (**pic).picFrame;
		
		if ( r.right - r.left < kMinPictWinWidth )
			r.right = r.left + kMinPictWinWidth;
			
		if ( r.bottom - r.top < kMinPictWinHeight )
			r.bottom = r.top + kMinPictWinHeight;
		
		OffsetRect( &r, gNewWindowX - r.left,
						gNewWindowY - r.top );
		
		BumpGlobalXandY();
		
		wStorage = NewPtr( sizeof( PictRecord ) );
		
		GetResInfo( rHandle, &resID, &theResType, rName );
		
		if ( rName[ 0 ] > 0 )
		{
			w = NewWindow( wStorage, &r, rName, kInvisible,
				noGrowDocProc, kMoveToFront, kHasGoAway, 0L );
		}
		else
		{
			w = NewWindow( wStorage, &r, "\p<Unnamed>", kInvisible,
				noGrowDocProc, kMoveToFront, kHasGoAway, 0L );
		}
		
		ShowWindow( w );
		SetPort( w );
		
		p = (PictPeek)w;
		p->wType = kPictWindow;
		p->PictResID = resID;
	}
}


/************************************* BumpGlobalXandY	 */

void	BumpGlobalXandY( void )
{
	gNewWindowX += 20;
	gNewWindowY += 20;
	
	if ( (gNewWindowX > screenBits.bounds.right - 100) ||
		(gNewWindowY > screenBits.bounds.bottom - 100) )
	{
		gNewWindowX = 20;
		gNewWindowY = 50;
	}
}


/************************************* DoGrow	 */

void	DoGrow( EventRecord *eventPtr, WindowPtr w )
{
	Rect		r;
	GrafPtr		oldPort;
	Cell		cSize;
	long		windSize;
	ListHandle	list;
	
	if ( WindowType( w ) == kListWindow )
	{
		r.top = kMinWindowHeight;
		r.bottom = 32767;
		r.left = kMinWindowWidth;
		r.right = 32767;
		
		windSize = GrowWindow( w, eventPtr->where, &r );
		if ( windSize )
		{
			GetPort( &oldPort );
			SetPort( w );
			EraseRect( &w->portRect );

			SizeWindow( w,
					LoWord (windSize),
					HiWord(windSize), true );
			
			list = ((ListPeek)w)->list;
			LSize( LoWord(windSize)-15,
					HiWord(windSize)-15, list );
					
			HLock( (Handle)list );
			cSize = (*list)->cellSize;
			HUnlock( (Handle)list );
			
			cSize.h = LoWord( windSize ) - 15;
			LCellSize( cSize, list );
			InvalRect( &w->portRect );
			SetPort( oldPort );
		}
	}
}


/************************************* HandleMenuChoice */

void	HandleMenuChoice( long menuChoice )
{
	short	menu;
	short	item;
	
	if ( menuChoice != 0 )
	{
		menu = HiWord( menuChoice );
		item = LoWord( menuChoice );
		
		switch ( menu )
		{
			case mApple:
				HandleAppleChoice( item );
				break;
			case mFile:
				HandleFileChoice( item );
				break;
		}
		HiliteMenu( 0 );
	}
}


/************************************* HandleAppleChoice */

void	HandleAppleChoice( short item )
{
	MenuHandle	appleMenu;
	Str255		accName;
	short		accNumber;
	
	switch ( item )
	{
		case iAbout:
			SysBeep( 20 );
			break;
		default:
			appleMenu = GetMHandle( mApple );
			GetItem( appleMenu, item, accName );
			accNumber = OpenDeskAcc( accName );
			break;
	}
}


/************************************* HandleFileChoice */

void	HandleFileChoice( short item )
{
	switch ( item )
	{
		case iNewList:
			CreateListWindow();
			break;
		case iClose:
			DestroyWindow( FrontWindow() );
			break;
		case iQuit:
			gDone = true;
			break;
	}
}


/****************** CenterPict ********************/

void	CenterPict( PicHandle picture, Rect *destRectPtr )
{
	Rect	windRect, pictRect;
	
	windRect = *destRectPtr;
	pictRect = (**( picture )).picFrame;
	OffsetRect( &pictRect, windRect.left - pictRect.left,
						   windRect.top	 - pictRect.top);
	OffsetRect( &pictRect,(windRect.right - pictRect.right)/2,
						  (windRect.bottom - pictRect.bottom)/2);
	*destRectPtr = pictRect;
}


/************************************* WindowType */

short	WindowType( WindowPtr window )
{
	if ( window == nil )
		return( kNilWindow );

	if ( ((WindowPeek)window)->windowKind < 0 )
		return( kDAWindow );
	
	if ( ((ListPeek)window)->wType == kListWindow )
		return( kListWindow );
	
	if ( ((ListPeek)window)->wType == kPictWindow )
		return( kPictWindow );
	
	return( kUnknownWindow );
}


/***************** DoError ********************/

void	DoError( Str255 errorString )
{
	ParamText( errorString, kEmptyString, kEmptyString, kEmptyString );
	
	StopAlert( kErrorAlertID, kNilFilterProc );
	
	ExitToShell();
}