/*****
 * 
 * Projection.c
 * Projection formulas
 * Abridged version for MacTech
 *
 * Alexei Lebedev, October 4, 1992
 *
 *****/

#include "Projection.h"
#include "FixedPointMath.h"
#include "SineTable.h"

Point2D center;				/* x- and y- centers of the 2D rectangle */
Point2D spaceSize = {1, 1};	/* x- and y- centers of the 3D rectangle */ 
Point2D theta = {15, 15};  	/* half the view angle of camera in degrees */
Vector2D cot;				/* cotangent of viewAngle */
Vector2D eyez;				/* distance from camera to the screen */
Vector2D ratio;				/* special ratio for perspective projection */

/* ------------------- */

static void AdjVars()
{
	eyez.x = cot.x * spaceSize.x;
	eyez.y = cot.y * spaceSize.y;
	
	ratio.x = cot.x * center.x;
	ratio.y = cot.y * center.y;
}

/* ------------------- */

void SetSpaceSize(short sizex, short sizey)
{
	spaceSize.x = sizex;
	spaceSize.y = sizey;
	
	AdjVars();
}

/* ------------------- */

void SetViewAngle(short x, short y)
{
	Fixed sin, cos;	

	GetTrigValues(theta.x = x, &sin, &cos);
	cot.x = Divide(cos, sin);
	
	GetTrigValues(theta.y = y, &sin, &cos);
	cot.y = Divide(cos, sin);

	AdjVars();
}

/* ------------------- */

void ViewPort(Rect *r)
{			
	center.y = (r->bottom - r->top) / 2;
	center.x = (r->right - r->left) / 2;
	
	AdjVars();
}

/* ------------------- */

void D3toD2(Vector3D *p3D, Point *p2D)
{				
   p2D->v = center.y - FixRnd(Divide(Multiply(p3D->y, ratio.y), eyez.y - p3D->z));  
   p2D->h = center.x + FixRnd(Divide(Multiply(p3D->x, ratio.x), eyez.x - p3D->z));
}

/* ------------------- */

void D3toD2par(Vector3D *p3D, Point *p2D)
{				
   p2D->v = center.y - FixRnd(p3D->y * center.y / spaceSize.y);   
   p2D->h = center.x + FixRnd(p3D->x * center.x / spaceSize.x);
}

/* ------------------- */

void D2toD3(Point p2D, Vector3D *p3D)
{
	p3D->x = Int2Fix((p2D.h - center.x) * spaceSize.x) / center.x;
	p3D->y = Int2Fix((center.y - p2D.v) * spaceSize.y) / center.y;
	p3D->z = 0;
}