/*****
 * 
 * SineTable.c
 * SineTable: Values of sin() and cos() for -360 <= a <= 720
 * Full version
 *
 * Alexei Lebedev, October 6, 1992
 *
 *****/
     
#include "SineTable.h"
#include "FixMath.h"

#define SineTableRES	'sint'
#define SineTableID		128

Fixed	gSineTable[90+1];

void InitSineTable()
{
	Fixed rad, pi;
	short i;
	Handle h;
	
	pi = X2Fix(3.14159);
	h = Get1Resource(SineTableRES, SineTableID);
	if (!h) { 
		for (i = 0; i <= 90; i++) {
			rad = i * pi / 180;
			gSineTable[i] = Frac2Fix(FracSin(rad));
		}	
		h = NewHandle(sizeof(gSineTable));
		if (!h) return;
		BlockMove((void*)&gSineTable, *h, sizeof(gSineTable));
		AddResource(h, SineTableRES, SineTableID, "\p");
		if (ResError()) { DisposHandle(h); return; }
		ReleaseResource(h);
	} else {
		BlockMove(*h, (void*)&gSineTable, sizeof(gSineTable));
		ReleaseResource(h);
	}
}

/* ------------------------- */

void GetTrigValues(short angle, Fixed *sin, Fixed *cos)
{
	if (angle < 0) angle += 360; else 
	if (angle > 360) angle -= 360;
	
	if (angle <= 90) {
		*sin = gSineTable[angle];
		*cos = gSineTable[90 - angle];
	} else if (angle <= 180) {
		*sin = gSineTable[180 - angle];
		*cos = -gSineTable[angle - 90];
	} else if (angle <= 270) {
		*sin = -gSineTable[angle -180];
		*cos = -gSineTable[270 - angle];
	} else {	
		*sin = -gSineTable[360 - angle];
		*cos = gSineTable[angle - 270];
	}
}