program PictLister;

{/****************************************************/}
{/*																													  */}
{/*  This code is																								  */}
{/*																													  */}
{/*	Copyright 1993, Dave Mark, All rights reserved 										  */}
{/*																													  */}
{/****************************************************/}

	const
		kMBARResID = 128;
		kSleep = 60;
		kMoveToFront = -1;
		kNilFilterProc = 0;
		kEmptyString = '';
		kHasGoAway = true;
		kInvisible = false;

		kListDefProc = 0;
		kDontDrawYet = false;
		kHasGrow = true;
		kHasHScroll = true;
		kHasVScroll = true;
		kFindNext = true;

		kListWindow = 0;
		kDAWindow = 1;
		kUnknownWindow = 2;
		kPictWindow = 3;
		kNilWindow = 4;

		kMinWindowWidth = 210;
		kMinWindowHeight = 63;
		kWindowHeight = 255;
		kMinPictWinHeight = 50;
		kMinPictWinWidth = 150;

		mApple = 128;
		iAbout = 1;

		mFile = 129;
		iNewList = 1;
		iClose = 2;
		iQuit = 4;

		kErrorAlertID = 128;


{    * * * * * * * * * * * * * / / * Typedefs * / / * * * * * * * * * * * * * /}
	type

		ListPeek = ^ListRecord;
		ListRecord = record
				w: WindowRecord;
				wType: INTEGER;
				list: ListHandle;
			end;

		PictPeek = ^PictRecord;
		PictRecord = record
				w: WindowRecord;
				wType: INTEGER;
				PictResID: INTEGER;
			end;


{    * * * * * * * * * * * * * / / * Globals * / / * * * * * * * * * * * * * /}
	var
		gDone: BOOLEAN;
		gNewWindowX, gNewWindowY: INTEGER;


{    / * * * * * * * * * * * * * * * / / * Functions * / / * * * * * * * * * * * * * * * /}

{ The procedures CenterWindow and DoError are not called anywhere in the original c code but }
{ do show up in the list of functions. Also, the DoError procedure code was provided whereas the }
{ the CenterWindow code was not }

{  procedure ToolboxInit;}
{  procedure MenuBarInit;}
{  procedure CreateListWindow;}
{  procedure DestroyWindow (w: WindowPtr );}
{  procedure EventLoop;}
{  procedure DoEvent  (var eventPtr: EventRecord);}
{  procedure DoUpdate  (var eventPtr: EventRecord);}
{  procedure DoActivate  (var eventPtr: EventRecord);}
{  procedure HandleMouseDown  (var eventPtr: EventRecord);}
{  procedure DoContentClick ( var eventPtr: EventRecord; w: WindowPtr);}
{  procedure CreatePictWindow (list: ListHandle );}
{  procedure BumpGlobalXandY;}
{  procedure DoGrow ( var eventPtr: EventRecord; w: WindowPtr);}
{  procedure HandleMenuChoice ( menuChoice: LONGINT);}
{  procedure HandleAppleChoice ( item: INTEGER );}
{  procedure HandleFileChoice ( item: INTEGER );}
{  procedure CenterWindow ( w: WindowPtr );}
{  procedure CenterPict ( picture: PicHandle ; var destRectPtr:  Rect );}
{  function WindowType ( window: WindowPtr ): INTEGER;}
{  procedure DoError ( errorString: Str255 );}


{ --------------------------------------- DoError --------------------------}
	procedure DoError (errorString: Str255);
		var
			dummy: INTEGER;
	begin
		ParamText(errorString, kEmptyString, kEmptyString, kEmptyString);

		dummy := StopAlert(kErrorAlertID, ProcPtr(kNilFilterProc));

		ExitToShell;
	end; {DoError}


{ --------------------------------------- BumpGlobalXandY --------------------------}
	procedure BumpGlobalXandY;
	begin
		gNewWindowX := gNewWindowX + 20;
		gNewWindowY := gNewWindowY + 20;

		if ((gNewWindowX > screenBits.bounds.right - 100) or (gNewWindowY > screenBits.bounds.bottom - 100)) then
			begin
				gNewWindowX := 20;
				gNewWindowY := 50;
			end;

	end; {BumpGlobalXandY}


{ --------------------------------------- CreateListWindow --------------------------}
	procedure CreateListWindow;
		var
			r, dataBounds: Rect;
			w: WindowPtr;
			cSize, cIndex: Point;
			list: ListHandle;
			i, dummy, numPicts: INTEGER;
			rHandle: Handle;
			resID, stringLength: INTEGER;
			theResType: ResType;
			rName: Str255;
			wStorage: Ptr;
			l: ListPeek;
	begin

		SetRect(r, gNewWindowX, gNewWindowY, gNewWindowX + kMinWindowWidth, gNewWindowY + kWindowHeight);

		BumpGlobalXandY;

		wStorage := NewPtr(sizeof(ListRecord));

		w := NewWindow(wStorage, r, 'Picture List', kInvisible, documentProc, WindowPtr(kMoveToFront), kHasGoAway, 0);

		SetPort(w);
		TextFont(systemFont);

		SetRect(dataBounds, 0, 0, 1, 0);
		SetPt(cSize, 0, 0);
		SetRect(r, 0, 0, kMinWindowWidth - 15, kWindowHeight - 15);

		list := LNew(r, dataBounds, cSize, kListDefProc, w, kDontDrawYet, kHasGrow, kHasHScroll, kHasVScroll);

		list^^.selFlags := lOnlyOne;

		l := ListPeek(w);

		l^.wType := kListWindow;
		l^.list := list;

		numPicts := CountResources('PICT');

		for i := 0 to (numPicts - 1) do
			begin
				rHandle := GetIndResource('PICT', i + 1);
				GetResInfo(rHandle, resID, theResType, rName);

{ the following was coded differently than Dave's version }
				if (rName = '') then
					rName := '<Unnamed>';

				dummy := LAddRow(1, i, list);
				SetPt(cIndex, 0, i);

				LAddToCell(Pointer(Ord(@rName) + 1), Length(rName), cIndex, list);
			end;

		ShowWindow(w);
		LDoDraw(true, list);
	end; {CreateListWindow}


{ --------------------------------------- CenterPict --------------------------}
	procedure CenterPict (picture: PicHandle; var destRectPtr: Rect);
		var
			windRect, pictRect: Rect;
	begin
		windRect := destRectPtr;
		pictRect := picture^^.picFrame;
		OffsetRect(pictRect, windRect.left - pictRect.left, windRect.top - pictRect.top);
		OffsetRect(pictRect, (windRect.right - pictRect.right) div 2, (windRect.bottom - pictRect.bottom) div 2);
		destRectPtr := pictRect;
	end; {CenterPict}


{ ------------------------------------- WindowType ---------------}
	function WindowType (window: WindowPtr): INTEGER;
	begin
		if window = nil then
			begin
				WindowType := kNilWindow;
				exit(WindowType);
			end;

		if WindowPeek(window)^.windowKind < 0 then
			begin
				WindowType := kDAWindow;
				exit(WindowType);
			end;

		if ListPeek(window)^.wType = kListWindow then
			begin
				WindowType := kListWindow;
				exit(WindowType);
			end;

		if ListPeek(window)^.wType = kPictWindow then
			begin
				WindowType := kPictWindow;
				exit(WindowType);
			end;

		WindowType := kUnknownWindow;
	end; {WindowType}


{ ------------------------------------ CreatePictWindow ----------------------}
	procedure CreatePictWindow (list: ListHandle);
		var
			myCell: Cell;
			pic: PicHandle;
			rHandle: Handle;
			r: Rect;
			resID: INTEGER;
			theResType: ResType;
			rName: Str255;
			p: PictPeek;
			wStorage: Ptr;
			w: WindowPtr;
			strSize: LONGINT;
	begin
		SetPt(myCell, 0, 0);

		if (LGetSelect(kFindNext, myCell, list)) then
			begin
				rHandle := GetIndResource('PICT', myCell.v + 1);
				pic := PicHandle(rHandle);

				r := pic^^.picFrame;

				if (r.right - r.left < kMinPictWinWidth) then
					r.right := r.left + kMinPictWinWidth;

				if (r.bottom - r.top < kMinPictWinHeight) then
					r.bottom := r.top + kMinPictWinHeight;

				OffsetRect(r, gNewWindowX - r.left, gNewWindowY - r.top);

				BumpGlobalXandY;

				wStorage := NewPtr(sizeof(PictRecord));

				GetResInfo(rHandle, resID, theResType, rName);

				StringToNum(rName[0], strSize);
				if (strSize > 0) then
					w := NewWindow(wStorage, r, rName, kInvisible, noGrowDocProc, WindowPtr(kMoveToFront), kHasGoAway, 0)
				else
					w := NewWindow(wStorage, r, '< Unnamed > ', kInvisible, noGrowDocProc, WindowPtr(kMoveToFront), kHasGoAway, 0);

				ShowWindow(w);
				SetPort(w);

				p := PictPeek(w);
				p^.wType := kPictWindow;
				p^.PictResID := resID;
			end;

	end;	{CreatePictWindow}


{ --------------------------------------- DoContentClick --------------------------}
	procedure DoContentClick (var eventPtr: EventRecord; w: WindowPtr);
		var
			oldPort: GrafPtr;
			list: ListHandle;
			l: ListPeek;
	begin
		if (w <> FrontWindow) then
			SelectWindow(w)
		else if (WindowType(w) = kListWindow) then
			begin
				GetPort(oldPort);

				SetPort(w);

				GlobalToLocal(eventPtr.where);

				l := ListPeek(w);
				list := l^.list;

				if (LClick(eventPtr.where, eventPtr.modifiers, list)) then
					CreatePictWindow(list);
				SetPort(oldPort);
			end;
	end; {DoContentClick}


{ ------------------------------------- DoGrow ---------------}
	procedure DoGrow (var eventPtr: EventRecord; w: WindowPtr);
		var
			r: Rect;
			oldPort: GrafPtr;
			cSize: Cell;
			windSize: LONGINT;
			list: ListHandle;
	begin
		if (WindowType(w) = kListWindow) then
			begin
				r.top := kMinWindowHeight;
				r.bottom := 32767;
				r.left := kMinWindowWidth;
				r.right := 32767;

				windSize := GrowWindow(w, eventPtr.where, r);
				if (windSize <> 0) then
					begin
						GetPort(oldPort);
						SetPort(w);
						EraseRect(w^.portRect);

						SizeWindow(w, LoWord(windSize), HiWord(windSize), true);

						list := ListPeek(w)^.list;
						LSize(LoWord(windSize) - 15, HiWord(windSize) - 15, list);

						HLock(Handle(list));
						cSize := list^^.cellSize;
						HUnlock(Handle(list));

						cSize.h := LoWord(windSize) - 15;
						LCellSize(cSize, list);
						InvalRect(w^.portRect);
						SetPort(oldPort);
					end;
			end;
	end;	{DoGrow}


{ ------------------------------------ DoUpdate ------------------------}
	procedure DoUpdate (var eventPtr: EventRecord);
		var
			w: WindowPtr;
			numPicts, i: INTEGER;
			l: ListPeek;
			list: ListHandle;
			oldPort: GrafPtr;
			r: Rect;
			cellIndex: Point;
			pic: PicHandle;
			p: PictPeek;
	begin
		w := WindowPtr(eventPtr.message);
		BeginUpdate(w);

		if (WindowType(w) = kListWindow) then
			begin
				GetPort(oldPort);
				SetPort(w);

				l := ListPeek(w);
				list := l^.list;

				DrawGrowIcon(w);

				LUpdate(list^^.port^.visRgn, list);

				SetPort(oldPort);
			end
		else if (WindowType(w) = kPictWindow) then
			begin
				GetPort(oldPort);
				SetPort(w);

				r := w^.portRect;

				p := PictPeek(w);

				pic := GetPicture(p^.PictResID);

				CenterPict(pic, r);
				DrawPicture(pic, r);

				SetPort(oldPort);
			end;
		EndUpdate(w);
	end; {DoUpdate}


{ --------------------------------------- DoActivate --------------------------}
	procedure DoActivate (var eventPtr: EventRecord);
		var
			w: WindowPtr;
			l: ListPeek;
			list: ListHandle;
	begin
		w := WindowPtr(eventPtr.message);

		if (WindowType(w) = kListWindow) then
			begin
				l := ListPeek(w);
				list := l^.list;

				if BitAnd(eventPtr.modifiers, activeFlag) <> 0 then
					LActivate(true, list)
				else
					LActivate(false, list);

				DrawGrowIcon(w);
			end;

	end;	{DoActivate}


{ ------------------------------------ DestroyWindow ------------------------}
	procedure DestroyWindow (w: WindowPtr);
		var
			l: ListPeek;
	begin
		if (WindowType(w) = kListWindow) then
			begin
				HideWindow(w);
				l := ListPeek(w);

				LDispose(l^.list);

				CloseWindow(w);

				DisposePtr(Ptr(w));
			end
		else if (WindowType(w) = kPictWindow) then
			begin
				CloseWindow(w);
				DisposePtr(Ptr(w));
			end;
	end; {DestroyWindow}


{ ------------------------------------ HandleFileChoice ------------------------}
	procedure HandleFileChoice (item: INTEGER);
	begin
		case item of
			iNewList: 
				CreateListWindow;
			iClose: 
				DestroyWindow(FrontWindow);
			iQuit: 
				gDone := true;
		end;		{case}

	end;	{HandleFileChoice}


{ ------------------------------------ HandleAppleChoice ----------------------}
	procedure HandleAppleChoice (item: INTEGER);
		var
			appleMenu: MenuHandle;
			accName: Str255;
			accNumber, unused: INTEGER;
	begin
		case item of
			iAbout: 
				SysBeep(20);
			otherwise
				begin
					appleMenu := GetMHandle(mApple);
					GetItem(appleMenu, item, accName);
					accNumber := OpenDeskAcc(accName);
				end;
		end;	{case}

	end;	{HandleAppleChoice}


{ ------------------------------------ HandleMenuChoice ----------------------}
	procedure HandleMenuChoice (menuChoice: LONGINT);
		var
			menu: INTEGER;
			item: INTEGER;
	begin
		if menuChoice <> 0 then
			begin
				menu := HiWord(menuChoice);
				item := LoWord(menuChoice);

				case menu of
					mApple: 
						HandleAppleChoice(item);
					mFile: 
						HandleFileChoice(item);
				end; {case}
				HiliteMenu(0);
			end;

	end;	{HandleMenuChoice}


{ ------------------------------------ HandleMouseDown ----------------------}
	procedure HandleMouseDown (var eventPtr: EventRecord);
		var
			window: WindowPtr;
			thePart: INTEGER;
			menuChoice: LONGINT;
			oldPort: GrafPtr;
			windSize: LONGINT;
			growRect: Rect;
	begin
		thePart := FindWindow(eventPtr.where, window);

		case thePart of
			inMenuBar: 
				begin
					menuChoice := MenuSelect(eventPtr.where);
					HandleMenuChoice(menuChoice);
				end;
			inSysWindow: 
				SystemClick(eventPtr, window);
			inContent: 
				DoContentClick(eventPtr, window);
			inGrow: 
				DoGrow(eventPtr, window);
			inDrag: 
				DragWindow(window, eventPtr.where, screenBits.bounds);
			inGoAway: 
				if (TrackGoAway(window, eventPtr.where)) then
					DestroyWindow(window);
		end;	{case}

	end;	{HandleMouseDown}


{ ------------------------------------ DoEvent ----------------------}
	procedure DoEvent (var eventPtr: EventRecord);
		var
			theChar: Char;
	begin
		case eventPtr.what of
			mouseDown: 
				HandleMouseDown(eventPtr);
			keyDown, autoKey: 
				begin
					theChar := CHR(BitAnd(eventPtr.message, charCodeMask));
					if (BitAnd(eventPtr.modifiers, cmdKey) <> 0) then
						HandleMenuChoice(MenuKey(theChar));
				end;
			updateEvt: 
				DoUpdate(eventPtr);
			activateEvt: 
				DoActivate(eventPtr);
		end;	{case}

	end;	{DoEvent}


{ ------------------------------------- EventLoop -----------------------------}
	procedure EventLoop;
		var
			event: EventRecord;
	begin
		gDone := FALSE;

		while (gDone = FALSE) do
			if WaitNextEvent(everyEvent, event, kSleep, nil) then
				DoEvent(event);

	end; {EventLoop}


{ ------------------------------------ MenuBarInit ----------------------}
	procedure MenuBarInit;
		var
			menuBar: Handle;
			menu: MenuHandle;
	begin
		menuBar := GetNewMBar(kMBARResID);
		SetMenuBar(menuBar);

		menu := GetMHandle(mApple);
		AddResMenu(menu, 'DRVR');

		DrawMenuBar;
	end; {MenuBarInit}


{ ------------------------------------ ToolboxInit ----------------------}
	procedure ToolBoxInit;
	begin
		InitGraf(@thePort);
		InitFonts;
		InitWindows;
		InitMenus;
		TEInit;
		InitDialogs(nil);
		InitCursor;
	end;


{/*																																	*/}
{/*	The compiler directive $I- turns automatic initialization off (THINK Pascal 4.0.2).		*/}
{/*	This was done so that ToolboxInit could be incorporated into the code, just as the		*/}
{/*	original C code does.																									*/}
{/*																																	*/}
{ ------------------------------------ Main Program----------------------}
{$PUSH}
{$I-}
begin

	gNewWindowX := 20;
	gNewWindowY := 50;

	ToolboxInit;
	MenuBarInit;

	EventLoop;

end.
{$POP}