
// Code to set up a UniversalProcPtr for the callback routine we pass to the library

#include <Quickdraw.h>
#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Windows.h>
#include <Dialogs.h>
#include <OSUtils.h>
#include <SegLoad.h>
#include <Events.h>
#include <TextUtils.h>

#include <Memory.h>
#include <Resources.h>
#include <MixedMode.h>
#include "LibraryInterface.h"

enum {
	uppCallbackProcInfo = kCStackBased // no result, no parameters
};

#if USESROUTINEDESCRIPTORS

// Use these definitions if we're compiling for Power Macintosh or another
// Macintosh system which supports Mixed Mode
typedef UniversalProcPtr CallbackUPP;

#define CallCallbackProc(userRoutine)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine),\
			uppCallbackProcInfo)
#define NewCallbackProc(userRoutine)		\
		(CallbackUPP) NewRoutineDescriptor((ProcPtr)userRoutine, \
					   uppCallbackProcInfo, GetCurrentISA())
#else
// Use these definitions if we're compiling for a 68K-based Macintosh
typedef CallbackPtr CallbackUPP;

#define CallCallbackProc(userRoutine)		\
		(*(userRoutine))()
#define NewCallbackProc(userRoutine)		\
		(CallbackUPP)(userRoutine)
#endif

#if defined(powerc) && !defined(_MWERKS_)
	QDGlobals	qd;
#endif

JumpTable			gJumpTable;
WindowPtr			gMyWindow;

OSErr LoadLibrary()
{
	// This is the code that loads our 68K library and calls its main entry point
	// to set up the table of function entry points.
	Handle	theResource;
	
	theResource= Get1IndResource('STUB', 1);
	if (theResource) {
		MoveHHi(theResource); HLock(theResource);	 // Lock it -- and NEVER unlock it
		// Call the entry point, no parameters
		return CallSetupProc(*theResource, &gJumpTable);
		// we know it's 68K, so we didn't bother with creating a Routine Descriptor
	} else
		return ResError();
}

void MyCallbackRoutine()
{
	// We'll pass a UniversalProcPtr for this routine to the library
	MoveTo(2, 14);
	DrawString((StringPtr)"\pCallback called...");
}

main()
{
	OSErr		err;
	Rect	bounds = { 40, 10, 190, 260 };
	
	// Set up the managers
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(0);
	InitCursor();

	// Let's create a window 
	gMyWindow = NewWindow(NULL, &bounds, (StringPtr)"\pLibrary demo", true,
												noGrowDocProc, (WindowPtr)-1, false, 0);
	SetPort(gMyWindow);
	TextFace(0); TextFont(1); TextSize(12);

	// Load the stand-alone code
	err = LoadLibrary();
	if (err == noErr) {
		OSErr	result;
		// Call the library routine. but first...
		// Create a UniversalProcPtr for our callback routine
		CallbackUPP		ourCallback;
		ourCallback = NewCallbackProc(MyCallbackRoutine);
		result = CallRoutine1Proc(gJumpTable.routine1, ourCallback);
		DisposeRoutineDescriptor(ourCallback); // free up the memory for the
																					 // routine descriptor
	} else {
		// Since we have a window, we can use it to report the error
		Str255	errNumString;
		
		MoveTo(2, 14);
		DrawString((StringPtr)"\pError ");
		NumToString(err, errNumString);
		DrawString(errNumString);
	}
	
	// Tell the user how to get out
	MoveTo(2, gMyWindow->portRect.bottom - 2);
	TextFace(bold);
	DrawString((StringPtr)"\pClick the mouse to exit.");
	while (Button());
	while (!Button());
}
