UNIT Music;
INTERFACE
	USES
		Components;

	CONST
		kMusicComponentType = 'musi';

	TYPE

		MusicDescription = RECORD
				size: LongInt; { including header }
				musicType: LongInt; { 'musi' }
				resvd1: LongInt; { 0 }
				resvd2: Integer;{ 0 }
				dataRefIndex: Integer; { 1 }
				musicFlags: LongInt; { 0 }
				headerData: ARRAY[1..1] OF LongInt; { actually, some sort of tone descriptions }
			END;



		MusicDescriptionPtr = ^MusicDescription;

		MusicDescriptionHandle = ^MusicDescriptionPtr;


		TunePlayer = ComponentInstance;

	CONST
		kTuneQueueDepth = 8;

	TYPE

		TuneStatus = RECORD
				tune, tunePtr: ^LongInt;
				time: longint;
				queueCount, queueSpots: Integer;
				queueTime: LongInt;
				reserved: ARRAY[1..3] OF LongInt;
			END;


	CONST
		kStopTuneFade = 1;
		kStopTuneSustain = 2;
		kStopTuneInstant = 4;
		kStopTuneReleaseChannels = 8;


	TYPE
		MusicOpWord = LongInt;
		MusicOpWordPtr = ^MusicOpWord;


	CONST
		kMaxTunePlayerParts = 32;
		tunePlayerRunning = -2100;
		kTunePlayerType = 'tune';

	FUNCTION TuneSetHeader (tp: TunePlayer;
									header: Ptr): ComponentResult;
	INLINE
		$2F3C, $4, 4, $7000, $A82A;

	FUNCTION TuneSetTimeScale (tp: TunePlayer;
									scale: LongInt): ComponentResult;
	INLINE
		$2F3C, $4, 6, $7000, $A82A;

	FUNCTION TuneGetTimeScale (tp: TunePlayer;
									VAR scale: LongInt): ComponentResult;
	INLINE
		$2F3C, $4, 7, $7000, $A82A;

	CONST
		kTuneStartNow = 1;
		kTuneDontClipNotes = 2;
		kTuneExcludeEdgeNotes = 4;
		kTuneStartNewMaster = 16384;

	FUNCTION TuneQueue (tp: TunePlayer;
									tune: MusicOpWordPtr;
									tuneRate: Fixed;
									tuneStartPosition: LongInt;
									tuneStopPosition: LongInt;
									queueFlags: LongInt;
									callBackProc: ProcPtr;
									refCon: LongInt): ComponentResult;
	INLINE
		$2F3C, $1C, 10, $7000, $A82A;

	FUNCTION TuneGetStatus (tp: TunePlayer;
									VAR status: TuneStatus): ComponentResult;
	INLINE
		$2F3C, $4, 12, $7000, $A82A;

	CONST
		kStopSustain = 1;
		kStopFadeout = 2;

	FUNCTION TuneStop (tp: TunePlayer;
									stopFlags: LongInt): ComponentResult;
	INLINE
		$2F3C, $4, 13, $7000, $A82A;

	FUNCTION TuneResume (tp: TunePlayer): ComponentResult;
	INLINE
		$2F3C, 0, 14, $7000, $A82A;

	FUNCTION TuneFlush (tp: TunePlayer): ComponentResult;
	INLINE
		$2F3C, 0, 15, $7000, $A82A;

	FUNCTION TuneSetVolume (tp: TunePlayer;
									volume: Fixed): ComponentResult;
	INLINE
		$2F3C, $4, 16, $7000, $A82A;

	FUNCTION TuneGetVolume (tp: TunePlayer): ComponentResult;
	INLINE
		$2F3C, 0, 17, $7000, $A82A;

	FUNCTION TunePreroll (tp: TunePlayer): ComponentResult;
	INLINE
		$2F3C, 0, 18, $7000, $A82A;

	FUNCTION TuneUnroll (tp: TunePlayer): ComponentResult;
	INLINE
		$2F3C, 0, 19, $7000, $A82A;


IMPLEMENTATION
END.