//	AEThreads.h
//
//	Copyright  1993-94 Steve Sisak
//	
//	License is granted to use, modify, make derivative works, and 
//	duplicate this code at will, so long as this notice remains intact.
//

#ifndef __AETHREADS__
#include "AEThreads.h"
#endif
#ifndef __ERRORS__
#include <Errors.h>
#endif
#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif
#if __MWERKS__
#if defined(powerc) || defined(__powerc)
#include <CPlusLibPPC.h>
#else
#include <stdlib.h>
#include <CPlusLib68k.h>
#endif
#endif

typedef struct AEThreadDesc  AEThreadDesc;
typedef struct AEThreadParam AEThreadParam;
typedef struct AESwapData    AESwapData;

struct AEThreadDesc					// Kept in the OS refcon
{
	AEEventHandlerUPP	handler;	// The real handler
	long				refcon;		// The real refcon
	Size				stackSize;	// Stack size for handling event
	ThreadOptions		options;	// Thread options for event
	ThreadID			holder;		// used as a semaphore
};

struct AEThreadParam				// Used in spawning
{
	const AppleEvent*	event;
	AppleEvent*			reply;
	AEThreadDesc*		desc;
	ThreadID			thread;
	OSErr				result;
};

struct AESwapData
{
	void*				fTopHandler;	// Top failure handler

#ifdef __MWERKS__
	DestructorChain*	fStaticChain;	//
#endif
};

pascal void		SwitchInHandler(ThreadID threadBeingSwitched, void *switchProcParam);
pascal void		SwitchOutHandler(ThreadID threadBeingSwitched, void *switchProcParam);
pascal OSErr	SpawnAEThread(const AppleEvent *theAppleEvent, AppleEvent *reply, long handlerRefcon);
pascal long		AEThread(AEThreadParam* parms);

AEEventHandlerUPP gSpawnAEThreadUPP = nil;

#pragma segment foobar

pascal OSErr AEInstallThreadedEventHandler(
	AEEventClass		theAEEventClass,
	AEEventID			theAEEventID,
	AEEventHandlerUPP	proc,
	long				handlerRefcon,
	ThreadOptions		options,
	Size				stacksize)
{
	AEThreadDesc* desc = (AEThreadDesc*) NewPtr(sizeof(AEThreadDesc));
	OSErr		  err  = MemError();
	
	if (gSpawnAEThreadUPP == nil)
	{
		gSpawnAEThreadUPP = NewAEEventHandlerProc(SpawnAEThread);
	}
	
	if (err == noErr)
	{
		desc->handler	= proc;
		desc->refcon	= handlerRefcon;
		desc->stackSize	= stacksize;
		desc->options	= options;
		desc->holder	= kNoThreadID;

		err = AEInstallEventHandler(theAEEventClass, theAEEventID, gSpawnAEThreadUPP, (long) desc, false);
	}
	
	return err;
}

#pragma segment AEThread

pascal void SwitchInHandler(ThreadID threadBeingSwitched, void *switchProcParam)
{
	AESwapData* swap = (AESwapData*) switchProcParam;

#ifdef __MWERKS__
	swap->fStaticChain = __local_destructor_chain;
#endif
}

pascal void SwitchOutHandler(ThreadID threadBeingSwitched, void *switchProcParam)
{
	AESwapData* swap = (AESwapData*) switchProcParam;

#ifdef __MWERKS__
	__local_destructor_chain = swap->fStaticChain;
#endif
}


#define ErrCheck(label, result)  if ((result) != noErr) goto label;

pascal long AEThread(AEThreadParam* parms)
{
	AppleEvent		event;			// Original parameters we care about
	AppleEvent		reply;
	AEThreadDesc*	desc;
	OSErr			err;
	OSErr			procErr;
	AESwapData		swap;

	event = *parms->event;			// copy these into our own stack frame
	reply = *parms->reply;
	desc  =  parms->desc;

#if 0	
	myTopHandler = gTopHandler;		// Save global failure handler
	gTopHandler  = nil;				// don't let failures propagate outside
#endif

	ErrCheck(punt, err = SetThreadSwitcher(kCurrentThreadID, SwitchInHandler,  &swap, true));
	ErrCheck(punt, err = SetThreadSwitcher(kCurrentThreadID, SwitchOutHandler, &swap, false));
	ErrCheck(punt, err = AESuspendTheCurrentEvent(&event));

	parms->result = noErr;		// Let caller know we're ready

	// At this point, we need to let our caller return

	while (desc->holder != kNoThreadID)
	{
		YieldToThread(desc->holder);
	}

	// We are now on our own
	
	procErr = err = CallAEEventHandlerProc(desc->handler, &event, &reply, desc->refcon);

		// Since the event was suspended, we need to stuff the error code ourselves	
		// note that there's not much we can do about reporting errors beyond here
		
	err = AEPutAttributePtr(&reply, keyErrorNumber, typeShortInteger, &procErr, sizeof(procErr));

#if qDebug
	if (err)
		ProgramBreak("\pAEPutAttributePtr failed installing error code - very bad");
#endif

	err = AEResumeTheCurrentEvent(&event, &reply, kAENoDispatch, 0);	// This had better work

#if qDebug
	if (err)
		DebugStr("\pAEResumeTheCurrentEvent failed - very bad");
#endif

#if 0
	gTopHandler	 = myTopHandler;	// Restore global failure handler
	myTopHandler = nil;				// Keep terminator from firing handlers
#endif

punt:
	parms->result = err;
	return nil;
}

#pragma segment Spawn

pascal OSErr SpawnAEThread(const AppleEvent *event, AppleEvent *reply, long handlerRefcon)
{
	AEThreadParam param;
	
	param.event  = event;
	param.reply  = reply;
	param.desc	 = (AEThreadDesc*) handlerRefcon;
	param.thread = kNoThreadID;

	if (!param.desc)
	{
		param.result = paramErr;
	}
	else
	{
		while (param.desc->holder != kNoThreadID)	// make sure no-one else is trying to start a handler
		{
			YieldToAnyThread();
		}
		
		if ((param.result = GetCurrentThread(&param.desc->holder)) == noErr)	// Grab the semaphore
		{
			param.result = NewThread(kCooperativeThread,
							(ThreadEntryProcPtr) &AEThread,
							&param,
							param.desc->stackSize,
							param.desc->options,
							nil,
							&param.thread);
			
			if (param.result == noErr)
			{
				param.result = 1;
			
				do
				{
					YieldToThread(param.thread);
				}
				while (param.result == 1);	// Wait for thread to pick up parameters
			}
		}
		
		param.desc->holder = kNoThreadID;	// release any claims we have
	}
	
	return param.result;
}




