/********************************************************* DEFINITION
	DATE:	9/17/93
	AUTHOR: Eric R. Ros
	
	CLASS:  CPPMaBell
	
	SUPERCLASS: CPPObject
	
		This C++ class handles general AppleTalk maintenance
	
********************************************************************/

#include <CPPMaBell.h>
#include <GestaltEqu.h>
#include <OSUtils.h>
#include <Appletalk.h>
#include <string.h>
#include <PPCToolBox.h>
#include <script.h>
#include <StringTools.h>

/*-----------------------------------------------------------------*/
/*----------------------- INTERNAL CONSTANTS ----------------------*/
/*-----------------------------------------------------------------*/

#define 	kHasSelfSend	48		/* version 48 or later has sss */
#define		kPhaseTwo		53		/* version 53 or later has phase 2 Atalk */
#define		kSync			FALSE	
#define		kAsync			TRUE

#define	kNeedsPPC				-1313
#define	kNeedsProgramLinking	-1314
#define kNeedsAppletalkOn		-1315

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPMaBell::CPPMaBell (Boolean AllowSelfSend)
	/* figure out what version of appletalk we are using, and initialize */
	/* our local variables */
	{
		long 		gestaltAnswer;
		OSErr 		ErrCode;
		SysEnvRec	MacSetup;

		XPPRefNum = MPPRefNum = ADSPRefNum = kDriverNotOpen;

		/* get the version of appletalk we are currently using */
		ErrCode = Gestalt (gestaltAppleTalkVersion, &gestaltAnswer);
		if (! ErrCode) 
		  {
			this->AtalkVersionNumber = gestaltAnswer;
		  } 
		else	/* if we can't use Gestalt, get the driver # the old way */
		  {
			ErrCode = SysEnvirons(1, &MacSetup);
			this->AtalkVersionNumber = MacSetup.atDrvrVersNum;
		  }
			
		/* Enable or disable selfsend */
		if (AllowSelfSend)
		  EnableSelfSend();
		else
		  DisableSelfSend();
		  
		// automatically open MPP and XPP
		this->OpenATalkDriver(kSelectMPP | kSelectXPP);
	}
	
/*-----------------------------------------------------------------*/

	CPPMaBell::~CPPMaBell (void)
	/* nothing to be done in the destructor */
	{
		
	}

	
/*-----------------------------------------------------------------*/

	char 	*CPPMaBell::ClassName (void)
	{
		return "CPPMaBell";
	}

/*-----------------------------------------------------------------*/

	OSErr	CPPMaBell::OpenATalkDriver (short whichDrivers)
	/* open the specified driver(s); MPP/XPP/ADSP */
	{
		OSErr	ErrCode = noErr;
		short	RefNum = 0;
		
		/* asked to open MPP */
		if (whichDrivers & kSelectMPP)
		  {
		  	ErrCode = OpenDriver ("\p.MPP", &RefNum);
		  	if (ErrCode == noErr)
		  	  this->MPPRefNum = RefNum;
		  }
		
		/* asked to open XPP */
		if (whichDrivers & kSelectXPP)
		  {
		  	ErrCode = OpenDriver ("\p.XPP", &RefNum);
		  	if (ErrCode == noErr)
		  	  this->XPPRefNum = RefNum;
		  }
		
		/* asked to open ADSP */
		if (whichDrivers & kSelectADSP)
		  {
		  	ErrCode = OpenDriver ("\p.DSP", &RefNum);
		  	if (ErrCode == noErr)
		  	  this->ADSPRefNum = RefNum;
		  }
	
		return ErrCode;
	}

/*-----------------------------------------------------------------*/

	short	CPPMaBell::GetDriverNumber(short whichDriver)
	/* get the version number for the specified driver; MPP/XPP/ADSP */
	{
		OSErr	ErrCode = noErr;
		
		switch (whichDriver) {
		
			case kSelectMPP :
				if (MPPRefNum == kDriverNotOpen)
				  ErrCode = this->OpenATalkDriver(kSelectMPP);	
				return (ErrCode == noErr) ? this->MPPRefNum : kDriverNotOpen;
				break;
			
			case kSelectXPP :	
				if (XPPRefNum == kDriverNotOpen)
				  ErrCode = this->OpenATalkDriver(kSelectXPP);	
				return (ErrCode == noErr) ? this->XPPRefNum : kDriverNotOpen;
				break;
			
			case kSelectADSP:	
				if (ADSPRefNum == kDriverNotOpen)
				  ErrCode = this->OpenATalkDriver(kSelectADSP);	
				return (ErrCode == noErr) ? this->ADSPRefNum : kDriverNotOpen;
				break;
		}
	}

/*-----------------------------------------------------------------*/

	short	CPPMaBell::GetAtalkVersion (void)
	/* return the version of Appletalk we are using */
	{
		return this->AtalkVersionNumber;
	}

/*-----------------------------------------------------------------*/
	
	Boolean	CPPMaBell::EnableSelfSend (void)
	/* Set up Appletalk so that we can send to our own node */
	/* Return TRUE if the call succeeds, and set SelfSendState */
	/* appropriately */
	{
		MPPParamBlock	LookupRec;
		OSErr			ErrCode;
		
		if (this->AtalkVersionNumber < kHasSelfSend)
		  return this->SelfSendState = FALSE;
		
		LookupRec.SETSELF.newSelfFlag = (Byte)1;
		ErrCode = PSetSelfSend (&LookupRec, kSync);
		if (ErrCode == noErr)
		  return this->SelfSendState = TRUE;
		else
		  return this->SelfSendState = FALSE;
		  
	}

/*-----------------------------------------------------------------*/

	Boolean CPPMaBell::DisableSelfSend (void)
	/* Set up Appletalk so that we cannot send to our own node */
	/* Return TRUE if the call succeeds, and set SelfSendState */
	/* appropriately */
	{
		MPPParamBlock	LookupRec;
		OSErr			ErrCode;
		
		if (this->AtalkVersionNumber < kHasSelfSend)
		  return this->SelfSendState = FALSE;
		
		LookupRec.SETSELF.newSelfFlag = (Byte)0;
		ErrCode = PSetSelfSend (&LookupRec, kSync);
		if (ErrCode == noErr)
		  return !(this->SelfSendState = FALSE);
		else
		  return this->SelfSendState = FALSE;
		  
	}

/*-----------------------------------------------------------------*/

	OSErr CPPMaBell::InitPPCToolbox (void)
	// return noErr if the PPCToolbox is present, initialized, and
	// suports incoming and outgoing messages 
	{
		long	PPCAttributes;
		OSErr	ErrCode;
		OSErr	InitResult;
		
		if (Gestalt(gestaltPPCToolboxAttr, &PPCAttributes) == noErr)
		  {
		  	if ((PPCAttributes & gestaltPPCSupportsRealTime) != 0)
		  	  {
		  	  	InitResult = PPCInit();
		  	  	ErrCode = Gestalt(gestaltPPCToolboxAttr, &PPCAttributes);
		  	  }
		  	  
		  	if ((PPCAttributes & gestaltPPCSupportsOutGoing) == 0)
		  	  {
		  	  	// can't open appletalk
		  	  	return kNeedsAppletalkOn;
		  	  }
		  	  
		  	if ((PPCAttributes & gestaltPPCSupportsIncoming) == 0)
		  	  {
		  	  	// program linking is disabled
		  	  	return kNeedsProgramLinking;
		  	  }
		  	  
		  	return ErrCode;
		  }
		else
		  return noErr;
	}

/*-----------------------------------------------------------------*/

	OSErr	CPPMaBell::OpenCommunicationPort (StringPtr OurName,
											  StringPtr PortType,
											  PPCPortRefNum *newRefNum)
	/* given the name of the application and the special name given to */
	/* the application's communication ports, this method tries to */
	/* open a new communications port.  If it fails,If so, it returnts TRUE and */
	/* the port Reference Number is returned in newRefNum */
	/* If it fails, it returns FALSE */
	{
		PPCOpenPBRec	OpenRec;
		PPCPortRec		PortRec;
		LocationNameRec	LocationRec;
		EntityName		TheName;
		OSErr			ErrCode;
			
		// set up the port specification record
		PortRec.nameScript = smRoman;
		PortRec.portKindSelector = ppcByString;
		CopyString (OurName, PortRec.name);
		CopyString ("\pPPCCommPort", PortRec.u.portTypeStr);
				
		// set up the record which describes who we are
		LocationRec.locationKindSelector = ppcNBPTypeLocation;
		PStrCat(32, &LocationRec.u.nbpType[0], 3, OurName, "\p", PortType);
		
		// set up the 'open port' record
		OpenRec.serviceType = ppcServiceRealTime;
		OpenRec.resFlag = 0;
		OpenRec.portName = &PortRec;
		OpenRec.locationName = &LocationRec;
		OpenRec.networkVisible = TRUE;
		
		// make the call and, if no errors, get the port number
		if ((ErrCode = PPCOpen (&OpenRec, FALSE)) == noErr)
		  *newRefNum = OpenRec.portRefNum;
		else
		  *newRefNum = -1;
		return ErrCode;
	}
	
/*-----------------------------------------------------------------*/

	OSErr	CPPMaBell::CloseCommunicationPort (PPCPortRefNum portRefNum)
	/* given the reference number of an open port, this method tries */
	/* to close the port; returns TRUE if the port is successfully */
	/* closed */
	{
		PPCClosePBRec	CloseRec;
		
		CloseRec.portRefNum = portRefNum;
		return PPCClose(&CloseRec, FALSE);
	}
	
/*-----------------------------------------------------------------*/

	OSErr	CPPMaBell::EndSession (PPCSessRefNum	sessionID)
	/* Close an open communication session, given its ID */
	{
		PPCEndPBRec			EndRec;
		
		// Synchronously close the connection from our end
		EndRec.sessRefNum = sessionID;
		return PPCEnd (&EndRec, FALSE);
	}

/*-----------------------------------------------------------------*/

	Boolean	CPPMaBell::SelfSendEnabled (void)
	/* return TRUE if we can send to our own node */
	{
		return this->SelfSendState;
	}

/*-----------------------------------------------------------------*/

	Boolean	CPPMaBell::HasPhase2ATalk (void)
	/* return TRUE if the appletalk version indicates that we have */
	/* phase 2 capability */
	{
		if (this->AtalkVersionNumber <= kPhaseTwo)
		  return FALSE;
		else
		  return TRUE;
	}

/*-----------------------------------------------------------------*/
