/********************************************************* DEFINITION
	DATE:	9/17/93
	AUTHOR: Eric R. Ros
	
	CLASS:  CPPNode411
	
	SUPERCLASS: CPPPeriodicTask
	
		This C++ class lets you look up nodes on the network.
	
********************************************************************/

#include	<CPPNode411.h>
#include	<CPPTaskManager.h>
#include	<CPPNodeInfo.h>
#include	<CPPObjectList.h>
#include	<MemoryTools.h>
#include	<ConvertTools.h>

/*-----------------------------------------------------------------*/
/*----------------------- INTERNAL CONSTANTS ----------------------*/
/*-----------------------------------------------------------------*/

#define		kSync			FALSE	
#define		kAsync			TRUE

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPNode411::CPPNode411 (CPPTaskManager *TaskManager, 
							long minPeriod, 
							Boolean deleteWhenDone) :
						  	CPPPeriodicTask (TaskManager, minPeriod,
						  				   deleteWhenDone)
	{
		this->FoundList = new CPPObjectList();
		this->lookupRec = NULL;
		this->returnBuffer = NULL;
		this->NTE = NULL;
	}
	
/*-----------------------------------------------------------------*/

	CPPNode411::~CPPNode411 (void)
	{
		MPPParamBlock 	killPB;
		OSErr 			ErrCode;
	
		// cancel the lookup call, if there is one pending
		if (this->lookupRec->MPPioResult == 1) 
		  {
			killPB.NBPnKillQEl = (Ptr)&lookupRec->MPP.qLink;
			ErrCode = PKillNBP(&killPB, false);
		  }
		
		// clear the list of nodes
		this->FoundList->DeleteItems(TRUE);
		delete this->FoundList;
		
		// release all memory used in doing lookups
		NukePtr(lookupRec);
		NukePtr(returnBuffer);
		NukePtr(NTE);	
	}
	
/*-----------------------------------------------------------------*/

	char *CPPNode411::ClassName (void)
	{
		return "CPPNode411";
	}

/*-----------------------------------------------------------------*/

	CPPNodeInfo *CPPNode411::NthNode (long whichItem, Boolean getCopy)
	/* return (a copy of) the specified node from the list of found */
	/* nodes; return FALSE if they ask for a node which is greater  */
	/* than the number of nodes retrieved so far */
	{		
		if (this->FoundList->GetNumItems() < whichItem)
		  return NULL;
		else
		  {
		  	if (getCopy)
		  	  return (CPPNodeInfo *)((CPPNodeInfo *)((*FoundList)[whichItem]))->Clone();
			else
			  return (CPPNodeInfo *)(((*FoundList)[whichItem]));
		  }
	}

/*-----------------------------------------------------------------*/

	CPPObjectList	*CPPNode411::GetFoundList (void)
	/* return the list where the data is stored */
	{
		return this->FoundList;
	}

/*-----------------------------------------------------------------*/

	long CPPNode411::NumNodesFound (Boolean *isDone)
	/* return the number of nodes found so far; return whether */
	/* the task has completed in the 'hasCompleted' variable */
	{
		*isDone = this->hasCompleted;
		return this->FoundList->GetNumItems();
	}
	
/*-----------------------------------------------------------------*/
	
	void CPPNode411::StartNodeLookup (StringPtr ObjectName,
									 StringPtr TypeName, 
									 StringPtr ZoneName,
									 short maxNumResponses,
									 CompletionProc DoProc)
	{
		// exit if this task is still involved in a lookup
		// or the manager is bogus
		if ((!this->hasCompleted) || (!this->ourManager))
		  return;
				
		// otherwise, set up the call
		this->SetCompletionProc(DoProc);
		this->hasCompleted = FALSE;			// note that we are not done
		this->FoundList->DeleteItems(TRUE);	// get rid of all known nodes
		
		SetupNBPCall(ObjectName, TypeName, ZoneName, maxNumResponses);
		
		if (this->callResult == noErr)
		  this->ourManager->AddPeriodicTask(this);	// now add the task
	}

/*-----------------------------------------------------------------*/
	
	void CPPNode411::DoPeriodicAction (void)
	/* if the lookup call has completed, extract the nodes and */
	/* execute another lookup call, unless we have gotten all the nodes */
	/* in which case, set the hasCompleted flag */
	{
		// call the inherited method to update frequency count
		CPPPeriodicTask::DoPeriodicAction();
		
		// do our periodic task
		switch (this->lookupRec->MPPioResult) {
			case noErr :	// the call has completed
				this->ProcessNodes(lookupRec->NBPnumGotten);
				this->hasCompleted = TRUE;
				this->callResult = noErr;
				break;
				
			case 1  :		// still busy getting names
				break;
				
			default	:		// an error occurred
				this->callResult = lookupRec->MPPioResult;
				hasCompleted = TRUE;
				break;
		}
		
	}
	
/*-----------------------------------------------------------------*/

	void CPPNode411::DoCompletedAction (void)
	{
	    NukePtr(this->returnBuffer);
		NukePtr(this->lookupRec);
		
		CPPPeriodicTask::DoCompletedAction();
	}
		
/*-----------------------------------------------------------------*/
/*----------------------- PRIVATE METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	void CPPNode411::SetupNBPCall (StringPtr ObjectName, StringPtr TypeName, 
								  StringPtr ZoneName, short maxNumBack)
	{
		this->returnBuffer = NewPtrClear(100 * maxNumBack);
		if ((this->callResult = MemError()) == noErr)
		  {
		  	this->lookupRec = (MPPPBPtr)NewPtrClear(sizeof(MPPParamBlock));
			if ((this->callResult = MemError()) == noErr)
			  {
			  	this->NTE = (NTEPtr)NewPtrClear(sizeof(NamesTableEntry));
				if ((this->callResult = MemError()) == noErr)
				  {
					// set up the lookup call  
					NBPSetEntity((Ptr)NTE->nt.entityData, (Ptr)ObjectName, 
								 (Ptr)TypeName, (Ptr)ZoneName);
					lookupRec->MPPioCompletion = NULL;
					lookupRec->NBPinterval = 5;
					lookupRec->NBPcount = 3;
					lookupRec->NBPentityPtr = (Ptr)NTE->nt.entityData;
					lookupRec->NBPretBuffSize =  100 * maxNumBack;
					lookupRec->NBPretBuffPtr = this->returnBuffer;
					lookupRec->NBPmaxToGet =  maxNumBack;
			
					// issue the lookup call asynchronously
					this->callResult = PLookupName(lookupRec, kAsync);		
				 	return;
				  }
			  }
		  }

		// we will only get here if a memory allocation fails
	  	hasCompleted = TRUE;
	  	return;
	}
	
/*-----------------------------------------------------------------*/

	void CPPNode411::ProcessNodes (short NumGotten)
	{
		short	i;
		EntityName	NodeName;
		AddrBlock	NodeAddress;
		CPPNodeInfo	*TempNA = NULL;

		for (i = 1;i<= NumGotten; i++)
		  {
		  	// get the data out of the buffer
			NBPExtract(this->returnBuffer, NumGotten, i, &NodeName, &NodeAddress);
		 	
		 	// create and initialize a NameAndAddress object
		 	TempNA = new CPPNodeInfo ();
		 	TempNA->SetNodeAddress (NodeAddress.aSocket & 0x0ff, NodeAddress.aNode & 0x0ff,
		 							NodeAddress.aNet);
		 	TempNA->SetNodeName ((StringPtr)Data2PString(&NodeName.objStr[1], NodeName.objStr[0]), 
		 						 (StringPtr)Data2PString(&NodeName.typeStr[1], NodeName.typeStr[0]), 
		 						 (StringPtr)Data2PString(&NodeName.zoneStr[1], NodeName.zoneStr[0]));
		  	
		  	// add it to the list of 'found' nodes
		  	this->FoundList->AppendItem((CPPObject *)TempNA);
		  }
		
	}
