/********************************************************* DEFINITION
	DATE:	9/17/93
	AUTHOR: Eric R. Ros
	
	CLASS:  CPPNodeInfo
	
	SUPERCLASS: CPPObject
	
		This class stores a network name and address and allows
		lookups, confirms, etc. on that address.
	
********************************************************************/

#include <CPPNodeInfo.h>
#include <MemoryTools.h>
#include <StringTools.h>

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPNodeInfo::CPPNodeInfo (void)
	{
		this->isRegistered = FALSE;
		objectString = typeString = zoneString = NULL;
		socketNumber = nodeNumber = zoneNumber = 0;
		this->NTE = NULL;
	}

/*-----------------------------------------------------------------*/

	CPPNodeInfo::~CPPNodeInfo (void)
	/* deregister our node and dispose of all memory we allocated */
	{
		OSErr	ErrCode;
		
		if (this->isRegistered)
		  DeregisterNodeAddress (&ErrCode);
	
		NukePtr(this->objectString);
		NukePtr(this->typeString);
		NukePtr(this->zoneString);
		NukePtr(this->NTE);
	}

/*-----------------------------------------------------------------*/

	char *CPPNodeInfo::ClassName (void)
	{
		return "CPPNodeInfo";
	}

/*-----------------------------------------------------------------*/

	CPPObject *CPPNodeInfo::Clone (void)
	/* make a copy of all the data in the current NodeInfo and return it */
	{
		CPPNodeInfo *NewInfo = new CPPNodeInfo();
				
		NewInfo->SetNodeName(this->objectString, this->typeString, this->zoneString);
		NewInfo->SetNodeAddress (this->socketNumber, this->nodeNumber, this->zoneNumber);
		NewInfo->isRegistered = this->isRegistered;
				
		return (CPPObject *)NewInfo;
	}

/*-----------------------------------------------------------------*/

	void CPPNodeInfo::SetNodeName (StringPtr ObjectStr, 
								   StringPtr TypeStr, 
								   StringPtr ZoneStr)
	/* determine what this object's name will be */
	{		
		NukePtr(this->objectString);
		NukePtr(this->typeString);
		NukePtr(this->zoneString);
		
		this->objectString = String2String(ObjectStr);
		this->typeString = String2String(TypeStr);
		this->zoneString = String2String(ZoneStr);
	}

/*-----------------------------------------------------------------*/

	void CPPNodeInfo::SetNodeAddress (short SocketNum, 
									  short NodeNum, 
									  short ZoneNum)
	/* set the components of this node's address */
	{
		this->zoneNumber = ZoneNum;
		this->nodeNumber = NodeNum;
		this->socketNumber = SocketNum;
	}
	
/*-----------------------------------------------------------------*/
	
	void CPPNodeInfo::GetNodeName (StringPtr *ObjectStr, 
								   StringPtr *TypeStr, 
						 		   StringPtr *ZoneStr)
	/* find out what this node's name is */
	{
		*ObjectStr = this->objectString;
		*TypeStr = this->typeString;
		*ZoneStr = this->zoneString;
	}

/*-----------------------------------------------------------------*/

	void CPPNodeInfo::GetNodeAddress (short *SocketNum, 
									  short *NodeNum, 
									  short *ZoneNum)
	/* find out what this node's address is */
	{
		*ZoneNum = this->zoneNumber;
		*NodeNum = this->nodeNumber;
		*SocketNum = this->socketNumber;
	}

/*-----------------------------------------------------------------*/

	StringPtr CPPNodeInfo::ReturnShortNameString (void)
	/* return a string in the form object@zone */
	{
		Str255		STemp;
		
		// construct the concatenated string
		PStrCat (255, STemp, 3, this->objectString, "\p@",
								this->zoneString);
		// allocate space for the string and make a copy of it
		if (STemp[0] > 0)
		  return String2String(STemp);
		else
		  return NULL;
	}

/*-----------------------------------------------------------------*/

	StringPtr CPPNodeInfo::ReturnNameString (void)
	/* return a string in the form object:type@zone */
	{
		Str255		STemp;
		
		// construct the concatenated string
		PStrCat (255, STemp, 5, this->objectString, "\p:",
								this->typeString, "\p@",
								this->zoneString);
								
		// allocate space for the string and make a copy of it
		if (STemp[0] > 0)
		  return String2String(STemp);
		else
		  return NULL;
	}

/*-----------------------------------------------------------------*/

	Boolean CPPNodeInfo::ConfirmNodeAddress (OSErr *ErrCode)
	/* Do a synchronous confirmation of the node's address */
	{
		MPPParamBlock 		LookupRec;
		AddrBlock			ConfirmAddr;
		NamesTableEntry		NTE;
		
		// construct the address which we want to confirm
		ConfirmAddr.aNet = this->zoneNumber;
		ConfirmAddr.aNode = this->nodeNumber;
		ConfirmAddr.aSocket = this->socketNumber;
		
		// construct the name we want to confirm
		NBPSetEntity((Ptr)NTE.nt.entityData, (Ptr)this->objectString, 
					 (Ptr)this->typeString, (Ptr)this->zoneString);
		
		// set up the call record
		LookupRec.MPPioCompletion = NULL;
		LookupRec.NBPentityPtr = (Ptr)&NTE.nt.entityData;
		LookupRec.NBPinterval = 2;
		LookupRec.NBPcount = 3;
		LookupRec.NBPconfirmAddr = ConfirmAddr;
		
		// make the call synchronously
		if ((*ErrCode = PConfirmName(&LookupRec,FALSE)) != noErr)
		  return FALSE;
		else
		  {	// save the returned socket number of the object
		  	this->socketNumber = LookupRec.NBPnewSocket;
		    return TRUE;
		  }
	}

/*-----------------------------------------------------------------*/

	Boolean CPPNodeInfo::RegisterNodeAddress (OSErr *ErrCode)
	/* take the Address data in the node and register ourselves on the network */
	{
		MPPParamBlock 		LookupRec;
		
		// the Names Table entry must be fixed in memory as long as the
		// node is registered on the network; allocate it as a pointer here 
		this->NTE = (NamesTableEntry *)NewPtrClear(sizeof(NamesTableEntry));
		if ((*ErrCode = MemError()) != noErr)
		  return FALSE;
		
		NBPSetNTE((Ptr)this->NTE, (Ptr)this->objectString, (Ptr)this->typeString, 
								(Ptr)this->zoneString, this->socketNumber);
		
		/* try three times - pause for 2 seconds between tries */
		LookupRec.NBPinterval = 2;
		LookupRec.NBPcount = 3;
		/* give it the namestableentry we constructed */
		LookupRec.NBP.NBPPtrs.ntQElPtr = (Ptr)this->NTE;
		/* verify that we be the only node with this exact address */
		LookupRec.NBPverifyFlag = 1; 
	
		*ErrCode = PRegisterName(&LookupRec, false);
		
		if (*ErrCode == noErr)
		  return this->isRegistered = TRUE;
		else
		  return FALSE;
	}

/*-----------------------------------------------------------------*/

	Boolean CPPNodeInfo::DeregisterNodeAddress (OSErr *ErrCode)
	{
		MPPParamBlock	LookupRec;
		
		if (!this->isRegistered)		// if we haven't registered yet,
		  {
		  	*ErrCode = nbpNotFound;		// return an error and fail
		  	return FALSE;
		  }
		
		// construct a names table entry for the object, and call PRemoveName
		LookupRec.NBPentityPtr = (Ptr)this->NTE->nt.entityData;
		
		// remove the name from the network list, and de-allocate the
		// memory used for the entity name
		if ((*ErrCode = PRemoveName(&LookupRec, FALSE)) == noErr)
		  {
		  	DisposPtr((Ptr)this->NTE);
		  	this->NTE = NULL;
		  }
	}

/*-----------------------------------------------------------------*/
	
	Ptr	CPPNodeInfo::InfoToStream (void)
	{
		Ptr	Buffer, TempPtr;
		AddrBlock	TempAddr;
		
		TempAddr.aSocket = this->socketNumber;
		TempAddr.aNode = this->nodeNumber;
		TempAddr.aNet = this->zoneNumber;
		
		TempPtr = Buffer = NewPtrClear(*this->objectString + *this->typeString +
									   *this->zoneString + 3 + sizeof(AddrBlock));
		if (Buffer)
		  {
		  	BlockMove (this->objectString, Buffer, *this->objectString+1);
		  	TempPtr += *this->objectString+1;
		  	BlockMove (this->typeString, TempPtr, *this->typeString+1);
		   	TempPtr += *this->typeString+1;
			BlockMove (this->zoneString, TempPtr, *this->zoneString+1);
			TempPtr += *this->zoneString + 1;
			BlockMove(&TempAddr, TempPtr, sizeof(TempAddr));
		  }
		return Buffer;
	}
	
/*-----------------------------------------------------------------*/

	CPPNodeInfo *StreamToInfo (Ptr Buffer, Ptr *BufferEnd)
	/* construct a NameAndAddress object from the data in the passed-in */
	/* buffer; set BufferEnd to point to the spot in the buffer which */
	/* directly follows the name and address */
	{
		CPPNodeInfo		*TempNA = new CPPNodeInfo();
		StringPtr		ObjStr, TypeStr, ZoneStr;
		AddrBlock		TempAddr;
		
		ObjStr = (StringPtr)Buffer;
		TypeStr = (StringPtr)(Buffer + *Buffer + 1);
		ZoneStr = (StringPtr)(TypeStr + *TypeStr + 1);
		BlockMove (ZoneStr + *ZoneStr + 1, &TempAddr, sizeof(AddrBlock));
		
		TempNA->SetNodeName (ObjStr, TypeStr, ZoneStr);
		TempNA->SetNodeAddress (TempAddr.aSocket, TempAddr.aNode, TempAddr.aNet);
		*BufferEnd = (Ptr)(ZoneStr + *ZoneStr + 1 + sizeof(AddrBlock));
		return TempNA;
	}

/*-----------------------------------------------------------------*/

	Boolean		CPPNodeInfo::Equals (CPPNodeInfo *TestNA)
	{
		if ((this->socketNumber == TestNA->socketNumber) &&
		    (this->nodeNumber == TestNA->nodeNumber) &&
		    (this->zoneNumber == TestNA->zoneNumber))
		  {
		  	// if the network address is the same, check the
		  	// 'types' of the nodes
		  	if (PStrCmp(this->typeString, TestNA->typeString) == 0)
		  	  return TRUE;
		  }

		return FALSE;
	}

