/***************************************************** IMPLEMENTATION
	DATE:	9/26/93
	AUTHOR: Eric R. Ros

	CLASS:  CPPApplication
	
	SUPERCLASS: CPPObject
	
		This C++ class manages a very basic application
	
********************************************************************/

#include <CPPApplication.h>

#include <CPPMenuBar.h>
#include <CPPApplication.h>
#include <CPPWindowManager.h>
#include <Commands.h>

/*-----------------------------------------------------------------*/
/*--------------------- APPLICATION GLOBALS -----------------------*/
/*-----------------------------------------------------------------*/

	CPPWindowManager	*gWindowManager = NULL;
	CPPMenuBar			*gMenuBar = NULL;
	Boolean				gQuitApp = FALSE;
	Boolean				gInBackground = FALSE;
	CPPApplication		*gApplication = NULL;
	
/*-----------------------------------------------------------------*/
/*---------------------- INTERNAL CONSTANTS -----------------------*/
/*-----------------------------------------------------------------*/

#define	kOSEvent				app4Evt	/* event used by MultiFinder */
#define	kSuspendResumeMessage	1		/* high byte of suspend/resume event message */
#define	kResumeMask				1		/* bit of message field for resume vs. suspend */
#define	kMouseMovedMessage		0xFA	/* high byte of mouse-moved event message */
#define	kNoEvents				0	   	/* no events mask */
#define	kHighLevelEvent			23


/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPApplication::CPPApplication (void)
	{
		// all the real work is done in the initialize method
	}

/*-----------------------------------------------------------------*/

	CPPApplication::~CPPApplication (void)
	{
		// all the real work is done in the shutdown method
	}

/*-----------------------------------------------------------------*/

	char	*CPPApplication::ClassName (void)
	{
		return "CPPApplication";
	}

/*-----------------------------------------------------------------*/

	void CPPApplication::Initialize (void)
	/* initialize the toolbox and all application specific stuff */
	{
		EventRecord event;
		
		// first initialize the toolbox
		InitGraf((Ptr) &thePort);
		InitFonts();
		InitWindows();
		InitMenus();
		TEInit();
		InitDialogs(nil);
		InitCursor();
	
		AllocateMemory();
		
		// throw out the first three events in the queue
		for (short count = 1; count <= 3; count++)
			EventAvail(everyEvent, &event);
	
		BuildMenuBar ();
			
		BuildWindowManager ();
		
		DoApplicationSetup ();
	}

/*-----------------------------------------------------------------*/

	void	CPPApplication::ShutDown(void)
	/* do final cleanup of application stuff */
	{		
		delete gWindowManager;
		DoApplicationShutDown();
	}
	
/*-----------------------------------------------------------------*/

	void CPPApplication::AllocateMemory (void)
	/* make sure our app has enough memory to run */
	/* SUBCLASS MAY OVERRIDE */
	{
		MoreMasters ();
	}

/*-----------------------------------------------------------------*/

	void CPPApplication::BuildMenuBar (void)
	/* create a menu bar with only the apple menu in it */
	/* SUBCLASS SHOULD OVERRIDE */
	{
		gMenuBar = new CPPMenuBar ();
	}

/*-----------------------------------------------------------------*/

	void CPPApplication::BuildWindowManager (void)
	/* create and initialize the window manager */
	/* SUBCLASS MAY OVERRIDE */
	{
		gWindowManager = new CPPWindowManager();
	}

/*-----------------------------------------------------------------*/

	void	CPPApplication::DoApplicationSetup(void)
	/* user specific window creation, etc. goes here */
	/* SUBCLASS SHOULD OVERRIDE */
	{
		this->idleTime = 15;
	
	}

/*-----------------------------------------------------------------*/

	void CPPApplication::DoAppPeriodicTask (void)
	/* perform any tasks which should be done on every cycle through */
	/* the event loop */
	/* SUBCLASS SHOULD OVERRIDE */
	{
	
	}

/*-----------------------------------------------------------------*/

	void	CPPApplication::DoApplicationShutDown(void)
	/* user specific object destruction, etc. goes here */
	/* SUBCLASS SHOULD OVERRIDE */
	{
	
	}

/*-----------------------------------------------------------------*/

	Boolean	CPPApplication::DoCommand (short commandID)
	/* handle a command associated with the menu bar */
	/* SUBCLASS SHOULD OVERRIDE */
	{
		switch (commandID) {
		
			case kCmdQuit :
				gQuitApp = TRUE;
				break;
			default :
				return FALSE;
				break;
		}
	
		return TRUE;
	}

/*-----------------------------------------------------------------*/

	void CPPApplication::Go (void)
	{
		EventRecord		theEvent;
		CPPWindow		*theWindowObject;
	
		while (!gQuitApp)
		  {
		  	DoAppPeriodicTask();
		  
		  	if (WaitNextEvent(everyEvent, &theEvent, idleTime, NULL))
			  DoEvent(&theEvent);
			else	
			  {		// idle tasks; send a DoIdle message to the front window
			  	theWindowObject = gWindowManager->FrontWindowObject();
			  	if (theWindowObject)
			  	  theWindowObject->DoIdle();
			  }
		  }
	}

/*-----------------------------------------------------------------*/
/*---------------------- PROTECTED METHODS ------------------------*/
/*-----------------------------------------------------------------*/

	void	CPPApplication::DoEvent(EventRecord *theEvent)
	{
		short			part, err;
		WindowPtr		window;
		char			key;
		Point			pt;
		Rect			contentRct, oldRct, growLimits;
		long			size;
		CPPWindow		*TempWindow = NULL,
						*FWindow = gWindowManager->FrontWindowObject();
		
		// Send command keys to the menu bar object to handle
		if ((theEvent->what == keyDown) || (theEvent->what == autoKey))
		  {
			key = theEvent->message & charCodeMask;
			if (theEvent->modifiers & cmdKey) 
			  {		/* Command key down. */
				if (theEvent->what == keyDown) 
				  {
					gMenuBar->DoMenu(MenuKey(key));
					return;
				  }
			  }
		  }
		  
		// determine which window the event occurred in so that we can
		// send it's owner the 'do' message
		if ((theEvent->what == updateEvt) || (theEvent->what == activateEvt))
		  window = (WindowPtr)theEvent->message;
		else
		  if (theEvent->what == keyDown || theEvent->what == autoKey)
		    window = FrontWindow();
		  else
		    part = FindWindow(theEvent->where, &window);
	
		// find the owner of the window which the event occurred in
		TempWindow = gWindowManager->FindWindowObject(window);
	
		/* do not allow clicks in different windows if the front window */
		/* is modal or movable modal */
		if (FWindow->IsModalWindow())
		  {
		  	if ((theEvent->what == mouseDown) && (TempWindow != FWindow))
		  	  {
		  	  	SysBeep(1);
		  	  	return;
		  	  }
		  }
	
		// ask the window owner to handle the event
		if ((TempWindow && !TempWindow->Do(theEvent)) || !TempWindow)
			switch(theEvent->what) {
			
				case mouseDown:
					switch(part) {
						case inMenuBar:		
							gMenuBar->DoMenu(MenuSelect(theEvent->where));
							break;
				  	  	
						case inSysWindow:	/* Let the system handle the mouseDown. */
							SystemClick(theEvent, window);
							break;
					}
					break;
									
				case diskEvt:
					if (HiWord(theEvent->message) != noErr) {
						SetPt(&pt, 100, 100);
						err = DIBadMount(pt, theEvent->message);
					}
					break;
	
				case kOSEvent:
					switch ((theEvent->message >> 24) & 0x0FF) {
							/* Must logical and with 0x0FF to get only low byte. */
							/* High byte of message. */
		
						case kMouseMovedMessage:
							break;
		
						case kSuspendResumeMessage:
								/* Suspend/resume is also an activate/deactivate. */
							gInBackground = !(theEvent->message & kResumeMask);
							if (TempWindow)
							  if (gInBackground)
							    {
							      gWindowManager->DeactivateWindowObject(TempWindow);
							      TEToScrap();
							    }
							  else
							    gWindowManager->ActivateWindowObject(TempWindow);
							break;
					}
					break;
	
				case kHighLevelEvent:
					SysBeep(1);		// don't handle these
					break;
	
				case updateEvt:
					if (TempWindow)
					  TempWindow->Update(theEvent);
					break;
		
			}	//switch
	
	}
