/***************************************************** IMPLEMENTATION
	DATE:	10/13/93
	AUTHOR: Eric R. Ros

	CLASS:  CPPGossipMonger
	
	SUPERCLASS: CPPObject
	
		This C++ class lets objects talk to each other.
		Its architecture is based on that of the TCL CCollaborator class.
	
********************************************************************/

#include <CPPGossipMonger.h>
#include <CPPObjectList.h>

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPGossipMonger::CPPGossipMonger (void)
	{
		Talkers = NULL;
		Listeners = NULL;
	}
	
/*-----------------------------------------------------------------*/

	CPPGossipMonger::~CPPGossipMonger (void)
	/* tell all of the objects you listen to and all of those who */
	/* listen to you that you are going away now */
	{
		CPPGossipMonger	*Temp;
		long			i;
		
		// stop listening to the objects we listen to
		if (Talkers)
		for (i = Talkers->GetNumItems(); i > 0 ; i--)
		  {
		  	if ((Temp = (CPPGossipMonger *)((*Talkers)[i])) != NULL)
		  	  this->StopListeningTo(Temp);
		  }
		
		// tell the objects that listen to us to stop doing so
		if (Listeners)
		for (i = Listeners->GetNumItems(); i > 0 ; i--)
		  {
		  	if ((Temp = (CPPGossipMonger *)((*Listeners)[i])) != NULL)
		  	  Temp->StopListeningTo(this);
		  }
		  
	}
	
/*-----------------------------------------------------------------*/

	char	*CPPGossipMonger::ClassName (void)
	{
		return "CPPGossipMonger";
	}
	
/*-----------------------------------------------------------------*/

	void CPPGossipMonger::ListenTo( CPPGossipMonger *newTalker)
	/* Add newTalker to the list of objects we listen to */
	{
		AddTalker (newTalker);
		newTalker->AddListener(this);
	}
	
/*-----------------------------------------------------------------*/

	void CPPGossipMonger::StopListeningTo( CPPGossipMonger *theTalker)
	/* remove theTalker from the list of objects we listen to */
	{
		RemoveTalker (theTalker);
		theTalker->RemoveListener(this);
	}
	
/*-----------------------------------------------------------------*/
/*---------------------- PROTECTED METHODS ------------------------*/
/*-----------------------------------------------------------------*/

	void CPPGossipMonger::BroadcastMessage( short reason, void* info)
	/* send a message containing the reason and info to each */
	/* object which listens to us */
	{
		CPPGossipMonger	*Temp;
		
		if (Listeners)
		for (long i = 1; i <= Listeners->GetNumItems(); i++)
		  {
		  	Temp = (CPPGossipMonger *)((*Listeners)[i]);
		  	if (Temp)
		  	  Temp->ReceiveMessage (this, reason, info);
		  }
		  
	}
	
/*-----------------------------------------------------------------*/

	void CPPGossipMonger::ReceiveMessage(CPPGossipMonger *toldBy, 
										 short reason, void* info)
	/* Receive a message from another object */
	/* SUBCLASS SHOULD OVERRIDE */
	{
	
	}
	
/*-----------------------------------------------------------------*/
/*----------------------- PRIVATE METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	void CPPGossipMonger::AddListener( CPPGossipMonger *newListener)
	/* Add a new object to the list of objects who listen to us */
	{
		if (!Listeners)
		  Listeners = new CPPObjectList();
		Listeners->AppendItem(newListener);
	}
	
/*-----------------------------------------------------------------*/

	void CPPGossipMonger::RemoveListener( CPPGossipMonger *theListener)
	/* Remove an object from the list of objects who listen to us */
	{
		if (Listeners)
		  Listeners->DeleteItem(theListener, FALSE);
	}
	
/*-----------------------------------------------------------------*/

	void CPPGossipMonger::AddTalker( CPPGossipMonger *newTalker)
	{
		if (!Talkers)
		  Talkers = new CPPObjectList();
		Talkers->AppendItem(newTalker);
	}
	
/*-----------------------------------------------------------------*/

	void CPPGossipMonger::RemoveTalker( CPPGossipMonger *theTalker)
	{
		Talkers->DeleteItem (theTalker, FALSE);
	}
	
/*-----------------------------------------------------------------*/
