/********************************************************* DEFINITION
	DATE:	9/24/93
	AUTHOR: Eric R. Ros
	
	CLASS:  CPPFilterText
	
	SUPERCLASS: CPPText
	
		This C++ class manages a CPPText area, with the additional
		benefit of being able to filter out classes of input
		characters (ie. only numbers, no punctuation, etc.)
		You may also block out specific characters using 
		SetFilterString.
	
********************************************************************/

#include <CPPFilterText.h>
#include <ctype.h>
#include <Commands.h>
#include <StringTools.h>
#include <MathTools.h>
	
/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPFilterText::CPPFilterText (CPPWindow *OurWindow,
								  Rect *ViewArea,
								  Rect *DestArea,
								  int	maxLength,
								  FilterSense	passBlock,
								  int	filterType,
								  Boolean UseHScroll,
								  Boolean UseVScroll,
								  int Font, 
								  int FSize) :
					CPPText (OurWindow, ViewArea, DestArea,
							 maxLength, UseHScroll, UseVScroll,
							 Font, FSize)
	{
		this->filterString = NULL;
		this->filterKind = filterType;
		this->fSense = passBlock;
		this->useBlankOut = FALSE;
	}
	
/*-----------------------------------------------------------------*/

	CPPFilterText::CPPFilterText (CPPWindow *OurWindow,
								  int	maxLength,
								  Boolean UseHScroll,
								  Boolean UseVScroll,
								  FilterSense	passBlock,
								  int	filterType,
								  int Font, 
								  int FSize) :
					CPPText (OurWindow, maxLength, UseHScroll, 
							 UseVScroll, Font, FSize)
	{
		this->filterString = NULL;
		this->filterKind = filterType;
		this->fSense = passBlock;
		this->useBlankOut = FALSE;
	}
	
/*-----------------------------------------------------------------*/

	CPPFilterText::~CPPFilterText (void)
	/* dispose of the filter string; it's the only data we own */
	{
		if (this->filterString)
		  DisposPtr((Ptr)this->filterString);
	}

/*-----------------------------------------------------------------*/

	char *CPPFilterText::ClassName (void)
	{
		return "CPPFilterText";
	}

/*-----------------------------------------------------------------*/

	void CPPFilterText::DoUseBlankOut (Boolean useIt, unsigned char blankChar)
	{
		if (this->useBlankOut = useIt)
		  this->blankChar = blankChar;
	}

/*-----------------------------------------------------------------*/

	Boolean	CPPFilterText::DoCommand (short commandID)
	{
		return CPPText::DoCommand(commandID);
	}
	
/*-----------------------------------------------------------------*/

	void CPPFilterText::SetFilterSense (FilterSense newSense)
	/* use this routine to specify the sense of the filter; if */
	/* blockIt is TRUE, block the characters specified by the */
	/* filters; if blockIt is FALSE, pass only the characters */
	/* specified by the filter */
	{
		this->fSense = newSense;
	}

/*-----------------------------------------------------------------*/

	void CPPFilterText::SetFilterString (StringPtr theString)
	/* let the user specify a particular set of characters to block */
	/* or pass, depending on the value of 'blockIt'*/
	{
		if (this->filterString)
		  DisposPtr((Ptr)this->filterString);
		if (theString)
		  this->filterString = String2String(theString);
	}

/*-----------------------------------------------------------------*/

	StringPtr CPPFilterText::GetFilterString (void)
	/* return the set of characters which are being blocked */
	{
		return this->filterString;
	}

/*-----------------------------------------------------------------*/

	short CPPFilterText::GetFilterKind (void)
	/* return the classes of characters which are being blocked */
	{
		return this->filterKind;
	}

/*-----------------------------------------------------------------*/

	void	CPPFilterText::SetFilterKind (short theKind)
	/* set the classes of characters which should be blocked */
	{
		this->filterKind = theKind;
	}

/*-----------------------------------------------------------------*/

	void	CPPFilterText::AddFilterKind (short theKind)
	/* add a class to the classes of chars which should be blocked */
	{
		this->filterKind |= theKind;
	}

/*-----------------------------------------------------------------*/

	void	CPPFilterText::RemoveFilterKind (short theKind)
	/* remove a class from the classes of chars which should be blocked */
	{
		this->filterKind &= ~theKind;
	}

/*-----------------------------------------------------------------*/

	void CPPFilterText::TypeChar (unsigned char TheKey)
	/* call the inherited method to type the key if it is allowed */
	{
		Boolean	canType = TRUE;
		
		if (SCharAllowed(&TheKey))
		  CPPText::TypeChar (TheKey);
	}

/*-----------------------------------------------------------------*/

	void CPPFilterText::DoPaste (void)
	/* paste the contents of the scrap into the TE area; filter the */
	/* text before pasting it in */
	{
		long	dummy;
		Handle	ScrapText;
		Ptr		NewText;
		
		if (this->TextBlock)
		  {
		  	if (TEFromScrap() != noErr)
		  	  SysBeep(5);
		  	else
		  	  {
		  	  	// get a handle to the text in the scrap
		  	  	if ((TEGetScrapLen() > 0) && 
		  	  		(ScrapText = TEScrapHandle()))
				
				// filter the text in the scrap
				HLock(ScrapText);
				NewText = DoFilter (*ScrapText, TEGetScrapLen());
				HUnlock(ScrapText);
				
				// take the filtered text, store it back into the
				// desk scrap, then call the inherited method to insert 
				if (NewText)
				  {
				  	ZeroScrap();
				    PutScrap(GetPtrSize(NewText), 'TEXT', NewText);
				    DisposPtr(NewText);
				    CPPText::DoPaste();
				  }
		  	  }
		  }
	}

/*-----------------------------------------------------------------*/

	void CPPFilterText::InsertTextPtr (Ptr theText, long textLen, 
									   long insertWhere, 
									   Boolean ScrollToInsertion)
	/* insert the string passed to us at the current selection point */
	/* NOTE we must filter this text to be sure it has none of the */
	/* restricted characters in it first */
	{
		Ptr	tempText = DoFilter (theText, textLen);
		
		if (tempText)
		  {
		    CPPText::InsertTextPtr (tempText, GetPtrSize(tempText),
		  						    insertWhere, ScrollToInsertion);
		  	DisposPtr(tempText);
		  }
	}

/*-----------------------------------------------------------------*/

	void CPPFilterText::SetTextPtr (Ptr theText, long textLen,
									Boolean ScrollToInsertion)
	/* replace the contents of the textedit area with the contents */
	/* of the string passed to us; NOTE we must filter this text to */
	/* be sure it has none of the restricted characters in it first */
	{
		Ptr	tempText = DoFilter (theText, textLen);
		
		if (tempText)
		  {
		    CPPText::SetTextPtr (tempText, GetPtrSize(tempText),
		  						 ScrollToInsertion);
		  	DisposPtr(tempText);
		  }
	}

/*-----------------------------------------------------------------*/

	Boolean CPPFilterText::CharAllowed (unsigned char theKey)
	/* return TRUE if theKey is allowed to be typed into the */
	/* text area */
	{
		short				classFilter = this->filterKind;
		register StringPtr	specificFilter = this->filterString;
		
		// Now pass the key through the user-defined filter
		if (specificFilter)
		  {
		  	register unsigned char *traverse = specificFilter+1;
		  	for (long i = 1; i <= *specificFilter; i++, traverse++)
		  	  if (theKey == *traverse)
		  	    return this->fSense;
		  }

		if (classFilter != all)
		  {
			// otherwise,
			// first pass the key through the general filters
		  	if (classFilter & alpha)
		  	  if (isalpha(theKey))
		  	    return this->fSense;
		  	if (classFilter & number)
		  	  if (isdigit(theKey))
		  	    return this->fSense;
		  	if (classFilter & punct)
		  	  if (ispunct(theKey))
		  	    return this->fSense;
		  	if (classFilter & spaces)
		  	  if (isspace(theKey))
		  	    return this->fSense;
		  }
		else  
		  return this->fSense;
		
		// if the filter specifies what to pass, allow all control
		// characters through by default
		if (this->fSense == pass)
		  if (iscntrl(theKey))
		    return pass;
		
		// if we haven't found the character in any of the filters,
		// block or pass it accordingly
		return !this->fSense;
	}

/*-----------------------------------------------------------------*/

	Boolean CPPFilterText::SCharAllowed (unsigned char *theKey)
	/* this lets you modify the key you are checking if the character */
	/* it represents is not permitted in this field.  In this way, if */
	/* 'useBlankOut' is active, you can replace the key with a dummy */
	/* key which is acceptable instead of just blowing it away */
	{
		if (!CharAllowed (*theKey))
		  if (this->useBlankOut)
		    *theKey = this->blankChar;
		  else
		    return FALSE;
	
		return TRUE;
	}	

/*-----------------------------------------------------------------*/

	Ptr	CPPFilterText::DoFilter (Ptr theText, short textLen)
	/* take a pointer to a block of text as an input and return */
	/* a pointer to a block of text with all the 'illegal' */
	/* characters filtered out */
	{
		Ptr	TempText = NewPtrClear(textLen);
		register	unsigned char *source = (unsigned char *)theText,
								  *dest = (unsigned char *)TempText;
		
		if (theText && TempText)
		  {
		  	for (long i = 1; i <= textLen; i++, source++)
		  	  {
		  	    if (SCharAllowed(source))
		  	      {
		  	      	*dest = *source;
		  	      	dest++;
		  	      }
		  	  }
		  	SetPtrSize(TempText, ((Ptr)dest - TempText));
		  	return TempText;
		  }
		else
		  return NULL;
	}

/*-----------------------------------------------------------------*/

