/********************************************************* DEFINITION
	DATE:	9/24/93
	AUTHOR: Eric R. Ros
	
	CLASS:  CPPFilterText
	
	SUPERCLASS: CPPText
	
		This C++ class manages a CPPText area, with the additional
		benefit of being able to filter out classes of input
		characters (ie. only numbers, no punctuation, etc.)
		You may also block out specific characters using 
		SetFilterString.
	
********************************************************************/

#include <CPPText.h>

#ifdef __cplusplus
extern "C" {
#endif

enum {
	none 	= 0x0,
	alpha 	= 0x1,
	number 	= 0x2,
	punct 	= 0x4,
	spaces	= 0x8,
	all		= 0xF
};

typedef enum { 
	block = 0,
	pass = 1
}	FilterSense;

#ifdef __cplusplus
}
#endif

class CPPFilterText : public CPPText	{
public:
					CPPFilterText (CPPWindow *OurWindow,
								  Rect *ViewArea,
								  Rect *DestArea,
								  int	maxLength = 32000,
								  FilterSense passBlock = pass,
								  int	filterType = all,
								  Boolean UseHScroll = TRUE,
								  Boolean UseVScroll = TRUE,
								  int Font = geneva, 
								  int FSize = 9);
					CPPFilterText (CPPWindow *OurWindow,
								  int	maxLength = 32000,
								  Boolean UseHScroll = TRUE,
								  Boolean UseVScroll = TRUE,
								  FilterSense passBlock = pass,
								  int	filterType = all,
								  int Font = geneva, 
								  int FSize = 9);
					~CPPFilterText (void);

	virtual	char	*ClassName (void);

	void			SetFilterSense (FilterSense newSense);

	void			SetFilterString (StringPtr theString);
	StringPtr		GetFilterString (void);
	void			SetFilterKind (short theKind);
	short 			GetFilterKind (void);
	void			AddFilterKind (short theKind);
	void			RemoveFilterKind (short theKind);

	void			DoUseBlankOut (Boolean useIt, unsigned char blankChar);

	virtual	Boolean	DoCommand (short commandID);

	virtual	void	DoPaste (void);

	virtual	void	TypeChar (unsigned char TheKey);

	virtual	void 	InsertTextPtr (Ptr theText, long textLen, 
								   long insertWhere, 
								   Boolean ScrollToInsertion);

	virtual	void 	SetTextPtr (Ptr theText, long textLen,
								Boolean ScrollToInsertion);
private:
	StringPtr	filterString;
	Boolean		fSense;
	short		filterKind;
	
	unsigned char	blankChar;
	Boolean		useBlankOut;
	
	Boolean 		SCharAllowed (unsigned char *theKey);
	Boolean 		CharAllowed (unsigned char theKey);
	Ptr				DoFilter (Ptr theText, short textLen);
	
};
