/********************************************************* DEFINITION
	DATE:	9/22/93
	AUTHOR: Eric R. Ros
	
	CLASS:  CPPVisualList
	
	SUPERCLASS: CPPVisualObject
	
		This C++ class lets you manage a scrolling list of data
	
********************************************************************/

#include <CPPVisualList.h>
#include <CPPWindow.h>
#include <CPPList.h>
#include <ToolboxTools.h>
#include <MathTools.h>
#include <CPPDArray.h>
#include <InstallDefProc.h>
#include <CPPStringList.h>

/*-----------------------------------------------------------------*/
/*--------------------------- GLOBALS -----------------------------*/
/*-----------------------------------------------------------------*/

	char	defaultEntry[3] = "\1";
	extern	Rect		kEmptyRect;
	extern	RGBColor	RGBBlack;
	extern	RGBColor	RGBWhite;
	
/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPVisualList::CPPVisualList (CPPWindow *theWindow, Rect *boundsRect,
								  CPPDArray *coreData, Boolean ownsCoreData,
								  Boolean canBeTarget, 
								  short selectionFlags, short cellHeight) : 
					CPPVisualObject(theWindow, boundsRect, canBeTarget)
	{
		Rect	dataRect = {0, 0, 0, 1};
		Point	cellSize;
		short	cellWidth;
				
		this->coreData = NULL;
		this->ownsData = FALSE;

		if (!theWindow)
		  return;
		  		
	  	// Create the LDEF to be used by this list
		this->LDEFStub = NewHandleClear(8);
		this->resID = -1;
	  	// 1. Get a valid, unique ID for an LDEF resource
	  	while (this->resID < 128)
	  	  this->resID = Unique1ID ('LDEF');
	  	// 2. Create the LDEF resource in the current resource fork
	  	AddResource (this->LDEFStub, 'LDEF', this->resID, "\pLDEF Stub");
	  	WriteResource(this->LDEFStub);
	  	// 3. Install the defproc
	  	InstallDefProc (CurResFile(), 'LDEF', this->resID, (Ptr)&CallLDEF);
				
		// figure out how large everything should be
		this->listRect = *boundsRect;
		InsetRect (&this->listRect, 1, 1);
	  	listRect.right = (listRect.right - 15 <= listRect.left + 15) ?
	  		listRect.left + 15 : listRect.right - 15;
	  	cellWidth = this->listRect.right - this->listRect.left;
		
		SetPt (&cellSize, cellWidth, cellHeight);
		this->vList = LNew (&this->listRect, &dataRect, cellSize, 
							this->resID, theWindow->GetWindow(), 
							TRUE, FALSE, FALSE, TRUE);

		AttachList (coreData, ownsCoreData);

		// set up the desired clicking behavior, and store
		// a handle to this object in the list's UserHandle field 
	  	if (this->vList)
	  	  {
	  	    (*this->vList)->selFlags = selectionFlags;
	  	    (*this->vList)->userHandle = (Handle)this;
	  	  }
	}

/*-----------------------------------------------------------------*/

	CPPVisualList::~CPPVisualList (void)
	{
		if (this->coreData && Talkers->FindIndex(this->coreData))
		  StopListeningTo (this->coreData);
		  
		if (this->vList)
		  LDispose(vList);

		if (this->LDEFStub)
		  {
		    RmveResource(this->LDEFStub);
		   	DisposHandle(this->LDEFStub);
		   	this->LDEFStub = NULL;
		  }

		if (this->ownsData && this->coreData)
		  {
		  	this->coreData->DeleteItems(TRUE);
		  	delete this->coreData;
		  }
	}

/*-----------------------------------------------------------------*/

	char *CPPVisualList::ClassName (void)
	{
		return "CPPVisualList";
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::AttachList (CPPDArray *coreData, 
									   Boolean ownsCoreData)
	{
		// delete the old core list, if any
		if (this->coreData && this->ownsData)
	  	  {
	  		this->coreData->DeleteItems(TRUE);
	  		delete this->coreData;  
		  }
		
		// attach the new list
		if ((this->coreData = coreData) != NULL)
		  this->ListenTo (coreData);
		this->ownsData = ownsCoreData;
	  	
	  	// call 'add' for each item in the list
	  	for (long i = 1; i <= GetNumItems(); i++)
	  	  ReceiveMessage (this->coreData, kItemAdded, (void *)i);
	}

/*-----------------------------------------------------------------*/

	CPPDArray	*CPPVisualList::GetItsData (void)
	{
		return this->coreData;
	}

/*-----------------------------------------------------------------*/

	void CPPVisualList::ReceiveMessage( CPPGossipMonger *toldBy, 
										short reason, void* info)
	/* respond to messages from the core data list */
	{
		Rect	tempRect;
		Point	tempPt = {(long)info - 1, 0};
		StringPtr		TempPtr = NULL;
		
		switch (reason) {
		
			case kItemAdded 	:
				LAddRow (1, (short)((long)info - 1), this->vList);
				LSetCell (&defaultEntry, 2, tempPt, this->vList);
				break;
				
			case kItemDeleted	:
				LDelRow (1, (short)((long)info - 1), this->vList);
				break;
				
			case kItemChanged	:
				SetPt (&tempPt, 0, (short)((long)info - 1));
				LDraw (tempPt, this->vList);
				break;
				
			case kAllChanged	:
				LUpdate ((*this->vList)->port->visRgn, this->vList);
				break;
		
			default	:
				CPPGossipMonger::ReceiveMessage (toldBy, reason, info);
				break;
		}
		
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::SetSelect (long whichCell, Boolean isSelected,
									 Boolean doUpdate)
	/* select or de-select a cell */
	/* NOTE:  Assumes whichcell is a '1-based' index */
	{
		Point	tempPoint = {whichCell-1, 0};
		
		if (IN_RANGE (whichCell, 1, GetNumItems()))
		  LSetSelect (isSelected, tempPoint, this->vList);
	}

/*-----------------------------------------------------------------*/

	Boolean	CPPVisualList::CellSelected (long whichCell)
	/* return whether or not this cell is selected */
	/* NOTE:  Assumes whichcell is a '1-based' index */
	{
		Point	tempPoint = {whichCell-1, 0};
		
		if (IN_RANGE (whichCell, 1, GetNumItems()))
		  return LGetSelect (FALSE, &tempPoint, this->vList);
		else
		  return FALSE;
	}

/*-----------------------------------------------------------------*/

	Boolean	CPPVisualList::DoKey (char theKey, short modifiers, short what)
	{
		long	selectedCell;
		
		if (!IsVisible()) return FALSE;

		if (modifiers & cmdKey) // Command key down
		  return FALSE;
		else
		  switch (theKey) {
		  	case 0x1E :	// up arrow
		  		this->MoveUp1 ();
		  		break;
		  		
		  	case 0x1F :	// down arrow
		  		this->MoveDown1 ();
		  		break;
		  		
		  	case 0x0D :	// return
		  		if (FirstSelectedCell (&selectedCell))
		  		  this->DoDoubleClick (selectedCell);
		  		break;
		  }
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::SelectAll (Boolean doSelect)
	/* select or deselect all items in the list */
	{
		LDoDraw (FALSE, this->vList);
		for (long i = 1; i <= GetNumItems(); i++)
		  this->SetSelect (i, TRUE, FALSE);
		LDoDraw (TRUE, this->vList);
		Draw();
	}

/*-----------------------------------------------------------------*/

	Boolean CPPVisualList::DoClick (EventRecord *theEvent)
	/* is Override */
	{
		WindowPtr	theWindow;
		Point		myPt = theEvent->where;
		TEPtr		TE;
		GrafPtr		SavePort;
		Boolean		isDouble;
		Point		tempPoint;
		long		hitCell;
		short		code;
		
		if (!IsVisible()) return FALSE;

	  	code = FindWindow(theEvent->where, &theWindow);
	  	if (theWindow == this->owningWindow)
	  	  {
			GetPort (&SavePort);
			SetPort (this->owningWindow);
		  	Global2Local (&myPt);
	  	  
	  	  	if (PtInRect (myPt, GetBounds()))
	  	  	  {
	  	  		isDouble = LClick (myPt, theEvent->modifiers, this->vList);
	  	  		  
	  	  		if (FirstSelectedCell (&hitCell))
	  	  		  {
	  	  		    if (isDouble)
		  	  		  DoDoubleClick(hitCell);
		  	  		else
		  	  		  DoSingleClick(hitCell);	  	  		  
	  	  		  }
	  	  	  }
			  
			SetPort (SavePort);
		  }
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::DrawCellContents (long whichCell, Rect *itsFrame, 
											 Boolean selected)
	/* Draw the actual contents of the specified cell in the */
	/* specified frame */
	/* SUBCLASS MUST OVERRIDE */
	{

	}
	
/*-----------------------------------------------------------------*/

	void	CPPVisualList::DrawCell (long whichCell, Rect *itsFrame, Boolean selected)
	/* Draw the specified cell inside the given rectangle */
	{
		RgnHandle	OldClip = NewRgn();
		Rect		tempRect = *GetBounds();
		GrafPtr		SavePort;
		
		GetPort(&SavePort);
		SetPort(this->owningWindow);
		// clip so that the cell can't draw outside the list
		GetClip (OldClip);
		InsetRect(&tempRect, 1, 1);
		ClipRect (&tempRect);
		
		// use the user's method to draw the cell
		DrawCellContents(whichCell, itsFrame, selected);
		
		// restore the old clipping region
		SetClip (OldClip);
		DisposeRgn(OldClip);
		SetPort(SavePort);
	}

/*-----------------------------------------------------------------*/
	
	void CPPVisualList::Draw (void)
	/* update the list's visual display */
	/* Is Override */
	{
		Rect	tempRect = *GetBounds();
		PenState	oldState;
		RgnHandle	OldClip = NewRgn();
		RgnHandle	NewClip = NewRgn();
		long		numItems = GetNumItems();
		GrafPtr		SavePort;
		
		if (this->owningWindow && this->IsVisible())
		  {
		  	GetPort(&SavePort);
		  	SetPort (this->owningWindow);
		  	
			FrameRect (&tempRect);	// draw the frame

			LUpdate ((*this->vList)->port->visRgn, this->vList);

			if (this->isTarget)
			  DrawHilite();

			SetPort(SavePort);
		  }
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::MakeVisible (Boolean nowVisible)
	/* Hide or show the list, depending on 'nowVisible' */
	{
		Rect	boundsRect = *GetBounds();
		GrafPtr	SavePort;
		
		// exit if it is already in the desired state
		if (nowVisible == IsVisible())
		  return;
		
		GetPort (&SavePort);
		SetPort (this->owningWindow);
		
		CPPVisualObject::MakeVisible (nowVisible);
		
		if (nowVisible)
		  {	// show the control(s) and restore the
		  	// list content rectangle
		  	if ((**this->vList).vScroll)
		  	  ShowControl ((**this->vList).vScroll);
		  	if ((**this->vList).hScroll)
		  	  ShowControl ((**this->vList).hScroll);
		  	(**this->vList).rView = this->saveListRect;
		  	Draw();
		  }
		else
		  {	// hide the control(s) and replace the
		  	// list content rectangle with -1,-1,-1,-1
		  	EraseRect (&boundsRect);
		  	if (this->isTarget)
		  	  EraseHilite();
		  	if ((**this->vList).vScroll)
		  	  HideControl ((**this->vList).vScroll);
		  	if ((**this->vList).hScroll)
		  	  HideControl ((**this->vList).hScroll);
		  	this->saveListRect = (**this->vList).rView;
		  	(**this->vList).rView = kEmptyRect;
		  }
		  
		SetPort(SavePort);
	}
	
/*-----------------------------------------------------------------*/

	void	CPPVisualList::Activate (Boolean nowActive)
	/* respond to an activate/deactivate message */
	{
		GrafPtr	SavePort;
				
		GetPort (&SavePort);
		SetPort (this->owningWindow);
		LActivate (nowActive, this->vList);
		  
		// let superclass know our state
		CPPVisualObject::Activate(nowActive);

		SetPort(SavePort);
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::MoveContent (short newH, short newV)
	/* move the entire list to the new position */
	{
		Rect	oldList = (**this->vList).rView;
		short	deltaH, deltaV;
		Rect	vRect, hRect;
		
		deltaH = newH - oldList.left,
		deltaV = newV - oldList.top;
		
		OffsetRect (&((**this->vList).rView), 
					  deltaH, deltaV);
	  	if ((**this->vList).vScroll)
	  	  {
	  	  	vRect = (**(**this->vList).vScroll).contrlRect;
		  	MoveControl ((**this->vList).vScroll, 
		  				 vRect.left + deltaH, 
		  				 vRect.top + deltaV);
	  	  }
	  	if ((**this->vList).hScroll)
	  	  {
	  	  	hRect = (**(**this->vList).hScroll).contrlRect;
		  	MoveControl ((**this->vList).hScroll, 
		  				 hRect.left + deltaH, 
		  				 hRect.top + deltaV);
	  	  }
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::ResizeContent (short newWidth, short newHeight)
	/* change the size of the list */
	{
		Rect	oldList = (**this->vList).rView;
		Rect	oldvScroll, oldhScroll;
		Rect	newList, newvScroll, newhScroll;
		Point	cellSize;
		ControlHandle	hScroll = (**this->vList).hScroll,
						vScroll = (**this->vList).vScroll;
						
		// resize the body of the list
		SetRect (&newList, oldList.left, oldList.top,
				 oldList.left+newWidth-15, oldList.top + newHeight - 15);
		(**this->vList).rView = newList;
		(**this->vList).cellSize.h = (newList.right - newList.left);
		
		if (vScroll)
		  {
		  	SetRect (&newvScroll, newList.right, newList.top-1,
		  			 newList.right+16, newList.bottom);
		  	HideControl(vScroll);
		  	MoveControl(vScroll, newvScroll.left, newvScroll.top);
		  	SizeControl(vScroll, 16, newList.bottom - newList.top + 2);
		  	ShowControl(vScroll);
		  }

		if (hScroll)
		  {
		  	SetRect (&newhScroll, newList.right-1, newList.bottom,
		  			 newList.left, newList.bottom+16);
		  	HideControl(hScroll);
		  	MoveControl(hScroll, newhScroll.left, newhScroll.top);
		  	SizeControl(hScroll, newList.right - newList.left + 2, 16);
		  	ShowControl(hScroll);
		  }
		
		Draw();
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::DrawHilite (void)
	{
		Rect	tempRect = *GetBounds();
		PenState	oldState;
		
		GetPenState(&oldState);	
		
  	  	InsetRect(&tempRect, -3, -3);
  	  	PenSize (2,2);
  	  	PenPat (black);
  	  	FrameRect (&tempRect);
  	  	
		SetPenState (&oldState);
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::EraseHilite (void)
	{
		Rect	tempRect = *GetBounds();
		PenState	oldState;
		
		GetPenState(&oldState);	
		
  	  	InsetRect(&tempRect, -3, -3);
  	  	PenSize (2,2);
  	  	PenPat (white);
  	  	FrameRect (&tempRect);
  	  	
		SetPenState (&oldState);
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::TargetHilite (Boolean makeTarget)
	/* draw/erase the 2 pixel wide border around the active list */
	{		
		if (this->isTarget != makeTarget)
		  {
			if (makeTarget)	// draw the hilite
		  	  DrawHilite();
		  	else			// erase the hilite
		  	  EraseHilite();
		  	
			CPPVisualObject::TargetHilite(makeTarget);
		  }
	}

/*-----------------------------------------------------------------*/

	long	CPPVisualList::TopCell (void)
	/* return the index of the topmost visible cell */
	/* NOTE:  the value returned assumes a '1-based' index */
	{
		Point	topPoint = {listRect.top + 5, listRect.left + 5};
		
		return Point2Cell (topPoint);
	}

/*-----------------------------------------------------------------*/

	long	CPPVisualList::BottomCell (void)
	/* return the index of the bottommost visible cell */
	/* NOTE:  the value returned assumes a '1-based' index */
	{
		Point	bottomPoint = {listRect.bottom - 5, listRect.left + 5};
		short	Temp;
		
		if ((Temp = Point2Cell (bottomPoint)) == 0)
		  return this->GetNumItems();
		else
		  return Temp;
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::GetCellRect (long whichCell, Rect *cRect)
	/* calculate the bounds of the specified cell depending on */
	/* the setting of the scrollbar, and return it in cRect */
	/* NOTE:  Assumes whichcell is a '1-based' index */
	{
		Point	tempPoint = {whichCell - 1, 0};
		
		LRect (cRect, tempPoint, this->vList);
	}

/*-----------------------------------------------------------------*/

	void  CPPVisualList::DoLDEF (short message, Boolean isSelect, 
						  Rect *cRect, Cell theCell, short lDataOffset, 
						  short lDataLen, ListHandle aList)
	/* This is the body of the LDEF; it simply calls a method */
	/* which does the drawing/hiliting of cells in the list */
	{	
		Ptr	theData = NULL;
		
		switch (message) {

			case lDrawMsg :
			case lHiliteMsg:
				DrawCell (theCell.v+1, cRect, isSelect);
				break;
		}
	}

/*-----------------------------------------------------------------*/
/*---------------------- PROTECTED METHODS ------------------------*/
/*-----------------------------------------------------------------*/

	Rect	*CPPVisualList::GetBounds (void)
	/* return the boundsrect of the object */
	{
		this->bounds = (**this->vList).rView;
		InsetRect (&this->bounds, -1, -1);
		this->bounds.right += 15;
		return &this->bounds;
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::DoSingleClick (long hitCell)
	/* Handle a single click in the specified cell */
	{
	
	}
	
/*-----------------------------------------------------------------*/

	void	CPPVisualList::DoDoubleClick (long hitCell)
	/* Handle a double click in the specified cell */
	/* SUBCLASS SHOULD OVERRIDE */
	{
		if (this->doClickProc)	// call the user-specified click handler
		 (doClickProc)(this->owningWObject);
		else
		if (this->commandEquivalent != kNoCommand)
		  this->owningWObject->DoCommand (this->commandEquivalent);
	}
	
/*-----------------------------------------------------------------*/

	Boolean	CPPVisualList::PrevSelectedCell (long *whichCell)
	/* return TRUE if there is a selected cells before the */
	/* passed in cell and return its coordinates */
	{
		long	current;
		
		if (FirstSelectedCell (&current))
		  {
		  	for (long i = current-1; i > 0; i--)
		  	  if (CellSelected(i))
		  	    {
		  	      *whichCell = i;
		  	      return TRUE;
		  	    }
		  }
		  
		return FALSE;
	}
	
/*-----------------------------------------------------------------*/

	Boolean	CPPVisualList::NextSelectedCell (long *whichCell)
	/* return TRUE if there is a selected cells after the */
	/* passed in cell and return its coordinates */
	{
		Point		tempPoint = {*whichCell-1, 0};
		
		if (*whichCell <= 0)
		  return FirstSelectedCell (whichCell);
				
		while (LNextCell (FALSE, TRUE, &tempPoint, this->vList))
		  if (LGetSelect (FALSE, &tempPoint, this->vList))
		    {
		      *whichCell = tempPoint.v+1;
		      return TRUE;
		    }
		
		return FALSE;
	}
	
/*-----------------------------------------------------------------*/

	Boolean	CPPVisualList::FirstSelectedCell (long *whichCell)
	/* return TRUE if there are any selected cells and return the */
	/* coordinates of the first selected cell */
	{
		Point	tempPoint = {0, 0};

		do {
		  if (LGetSelect (FALSE, &tempPoint, this->vList))
		    {
		      *whichCell = tempPoint.v+1;
		      return TRUE;
		    }
		} while (LNextCell (FALSE, TRUE, &tempPoint, this->vList));
		
		return FALSE;
	}

/*-----------------------------------------------------------------*/

	long	CPPVisualList::NumSelectedCells (void)
	/* return the total number of selected cells in the list */
	{
		Point	tempPoint = {0, 0};
		long	count	= 0;

		do {
		  if (LGetSelect (FALSE, &tempPoint, this->vList))
		    count++;
		} while (LNextCell (FALSE, TRUE, &tempPoint, this->vList));
		
		return count;
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::MoveUp1 (void)
	/* select the cell above the currently selected one */
	{
		long	selectedCell;
		
		if (this->owningWindow)
		  {
	  	  	if (FirstSelectedCell (&selectedCell))
	  	  	  {
	  	  	  	if (selectedCell > 1)
	  	  	  	  {
	  	  	  	    SetSelect (selectedCell, FALSE, FALSE);
	  	  	  	    SetSelect (selectedCell-1, TRUE, FALSE);
	  	  	  	    AdjustList (selectedCell-1);
		  	  	  	DoSingleClick(selectedCell);
		  	  	  }
	  	  	  }
		  }
	}

/*-----------------------------------------------------------------*/

	void	CPPVisualList::MoveDown1 (void)
	/* select the cell below the currently selected one */
	{
		long	selectedCell;
		
		if (this->owningWindow)
		  {
	  	  	if (FirstSelectedCell (&selectedCell))
	  	  	  {
	  	  	  	if (selectedCell < GetNumItems())
	  	  	  	  {
	  	  	  	    SetSelect (selectedCell, FALSE, FALSE);
	  	  	  	    SetSelect (selectedCell+1, TRUE, FALSE);
	  	  	  	    AdjustList (selectedCell+1);
		  	  	  	DoSingleClick(selectedCell);
		  	  	  }
	  	  	  }
		  }
	}
	
/*-----------------------------------------------------------------*/

	void	CPPVisualList::AdjustList (long whichCell)
	/* if the specified cell is not visible, scroll the list so that */
	/* it will be visible */
	{
		Point	currentCell = {whichCell - 1, 0};
		Rect	CellRect;
		
		LRect(&CellRect, currentCell, this->vList);
		if ((CellRect.bottom > (**this->vList).rView.bottom) || 
		   (CellRect.top < (**this->vList).rView.top))
			LAutoScroll(this->vList);
	}

/*-----------------------------------------------------------------*/
	
	long	CPPVisualList::Point2Cell (Point thePoint)
	/* given a point in local coordinates, return which cell it is in; */
	/* return 0 if it is not in any cell */
	/* the value returned assumes a '1-based' index */
	{
		Point	tempPoint = {0, 0};
		Rect	cellRect, listRect = *GetBounds();
		Point	topPoint = {listRect.top + 5, listRect.left + 5};
		
		for (tempPoint.v = 0; tempPoint.v < GetNumItems(); tempPoint.v++)
		  {
		  	LRect (&cellRect, tempPoint, this->vList);
		  	if (PtInRect (thePoint, &cellRect))
		  	  return tempPoint.v+1;
		  }

		return 0;
	}

/*-----------------------------------------------------------------*/

	long	CPPVisualList::GetNumItems (void)
	{
		if (this->coreData)
		  return this->coreData->GetNumItems();
		else
		  return 0;
	}

/*-----------------------------------------------------------------*/
/*----------------------- PRIVATE METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	 pascal void CPPVisualList::CallLDEF (short message, Boolean isSelect, 
						  Rect *cRect, Cell theCell, short lDataOffset, 
						  short lDataLen, ListHandle aList)
	{
		CPPVisualList	*ListObj = NULL;
		
		if (aList)
		  ListObj = (CPPVisualList *)((**aList).userHandle);
		
		if (ListObj)
		  ListObj->DoLDEF(message, isSelect, cRect, theCell, 
		  				  lDataOffset, lDataLen, aList);
	}
