/********************************************************* DEFINITION
	DATE:	9/17/93
	AUTHOR: Eric R. Ros

	CLASS:  CPPListenTask
	
	SUPERCLASS: CPPPeriodicTask
	
		This C++ class lets you listen for and accept connection
		requests received on a particular PPC Port
	
********************************************************************/

#include <CPPListenTask.h>
#include <CPPTaskManager.h>
#include <MemoryTools.h>

/*-----------------------------------------------------------------*/
/*----------------------- INTERNAL CONSTANTS ----------------------*/
/*-----------------------------------------------------------------*/

#define		kSync			FALSE	
#define		kAsync			TRUE

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPListenTask::CPPListenTask (CPPTaskManager *TaskManager, 
									long minPeriod, 
									Boolean deleteWhenDone) :
								  	CPPPeriodicTask (TaskManager, minPeriod,
								  				   deleteWhenDone)
	{
		this->sessionID = 0;
		this->listenRecPtr = NULL;
		this->portRecPtr = NULL;
		this->locationRecPtr = NULL;
	}

/*-----------------------------------------------------------------*/

	CPPListenTask::~CPPListenTask (void)
	{
		NukePtr(this->listenRecPtr);
		NukePtr(this->locationRecPtr);
		NukePtr(this->portRecPtr);
	}

/*-----------------------------------------------------------------*/

	char *CPPListenTask::ClassName (void)
	{
		return "CPPListenTask";
	}

/*-----------------------------------------------------------------*/

	void CPPListenTask::DoPeriodicAction (void)
	/* if the connection has completed or failed, store the result */
	/* code and note that we are done */
	{
		CPPPeriodicTask::DoPeriodicAction();
		
		if (this->listenRecPtr->informParam.ioResult != 1)
		  {
		  	this->callResult = this->listenRecPtr->informParam.ioResult;
		  	this->hasCompleted = TRUE;
		  	if (this->callResult == noErr)
		  	  this->sessionID = this->listenRecPtr->informParam.sessRefNum;
		  }
	}

/*-----------------------------------------------------------------*/

	void CPPListenTask::StartListenTask (PPCPortRefNum PortID,
											   CompletionProc DoProc)
	/* Set up the write parameter block and make the call */
	{		
		if (!this->hasCompleted)
		  return;

		this->sessionID = 0;
		this->SetCompletionProc(DoProc);
		
		this->listenRecPtr = (PPCParamBlockPtr)NewPtrClear(sizeof(PPCParamBlockRec));
	  	if ((this->callResult = MemError()) == noErr)
	  	  {
	  	  	NukePtr(this->locationRecPtr);
			this->locationRecPtr = (LocationNamePtr)NewPtrClear(sizeof(LocationNameRec));
		  	if ((this->callResult = MemError()) == noErr)
	  	  	  {
		  	  	NukePtr(this->portRecPtr);
				this->portRecPtr = (PPCPortPtr)NewPtrClear(sizeof(PPCPortRec));
			  	if ((this->callResult = MemError()) == noErr)
			  	  {
					// set up the listen record's fields
					this->listenRecPtr->informParam.ioCompletion = NULL;
					this->listenRecPtr->informParam.portRefNum = PortID;
					this->listenRecPtr->informParam.autoAccept = TRUE;
					this->listenRecPtr->informParam.portName = this->portRecPtr;
					this->listenRecPtr->informParam.locationName = this->locationRecPtr;
					this->listenRecPtr->informParam.userName = NULL;
					
					// make the call to handle connections asynchronously
					if ((this->callResult = 
					 	 PPCInform((PPCInformPBPtr)this->listenRecPtr, kAsync)) == noErr)
					  {
						this->hasCompleted = FALSE;			// note that we are not done
						this->ourManager->AddPeriodicTask(this);	// now add the task
						return;
					  }
			  	  }
	  	  	  }
	  	  }
	  	
	  	// these lines will be executed if memory allocation fails
	 	NukePtr(this->listenRecPtr);
	 	NukePtr(this->portRecPtr);
	 	NukePtr(this->locationRecPtr);
	}

/*-----------------------------------------------------------------*/

	void CPPListenTask::DoCompletedAction (void)
	/* SUBCLASS MUST OVERRIDE */
	{
		CPPPeriodicTask::DoCompletedAction();
	}
	
/*-----------------------------------------------------------------*/

	PPCSessRefNum	CPPListenTask::GetNewSessionID (void)
	/* return the session number which resulted from the connection */
	{
		return this->sessionID;
	}

/*-----------------------------------------------------------------*/

	void CPPListenTask::GetConnectedToInfo (Boolean *hasConnected,
											 LocationNamePtr *Location,
											 PPCPortPtr *PortRec)
	/* Return the location and the port record which describe the */
	/* port we are connected to */
	{
		if (this->sessionID != 0)
		  *hasConnected = TRUE;
		*Location = this->locationRecPtr;
		*PortRec = this->portRecPtr;
	}
