/********************************************************* DEFINITION
	DATE:	9/17/93
	AUTHOR: Eric R. Ros

	CLASS:  CReadTask
	
	SUPERCLASS: CPeriodicTask
	
		This C++ class lets you read data asynchronously through a 
		PPC connection.
	
********************************************************************/

#include <CPPReadTask.h>
#include <CPPTaskManager.h>
#include <MemoryTools.h>

/*-----------------------------------------------------------------*/
/*----------------------- INTERNAL CONSTANTS ----------------------*/
/*-----------------------------------------------------------------*/

#define		kSync			FALSE	
#define		kAsync			TRUE

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPReadTask::CPPReadTask (CPPTaskManager *TaskManager, 
							  long minPeriod, 
							  Boolean deleteWhenDone) :
							   CPPPeriodicTask (TaskManager, minPeriod,
							  				    deleteWhenDone)
	{
		this->returnData = NULL;
		this->readBuffer = NULL;
		this->totalSize = 0;
		this->ownsData = TRUE;
		this->sessionID = 0;
		this->readPBPtr = NULL;
		this->blockCreator = (OSType)"????";
		this->blockType = (OSType)"????";
	}
	
/*-----------------------------------------------------------------*/

	CPPReadTask::~CPPReadTask (void)
	{
		PPCClosePBRec	CloseRec;
	
		// if there is a read call outstanding, close the session
		if (this->readPBPtr)
		  if (this->readPBPtr->ioResult == 1)
		    {
			  CloseRec.portRefNum = this->sessionID;
			  PPCClose(&CloseRec, FALSE);
		    }
		
		// dispose of all the data and control blocks
		NukePtr(this->readPBPtr);
		NukePtr(this->readBuffer);
		if (this->ownsData)
		  NukeHandle(this->returnData);
	}

/*-----------------------------------------------------------------*/

	char *CPPReadTask::ClassName (void)
	{
		return "CPPWriteTask";
	}

/*-----------------------------------------------------------------*/

	void CPPReadTask::StartReadTask (PPCSessRefNum ConnectionID, short blockSize,
								   CompletionProc DoProc)
	// Set up the read parameter block and make the call
	{
		if (!this->hasCompleted)
		  return;

		this->SetCompletionProc(DoProc);
		this->blockType = (OSType)"????";
		this->blockCreator = (OSType)"????";
		
		this->readPBPtr = (PPCReadPBPtr)NewPtrClear(sizeof(PPCReadPBRec));
		if ((this->callResult = MemError()) == noErr)
		  {
			// allocate the buffer for data to be returned into
			this->readBuffer = NewPtrClear(blockSize);
			if ((this->callResult = MemError()) == noErr)
			  {
				this->sessionID = ConnectionID;
				this->blockSize = blockSize;
				
				// set up the ReadPB's fields so that when it gets to the
				// DoPeriodicTask call it will cause a READ to be posted
				this->readPBPtr->ioResult = noErr;
				this->readPBPtr->actualLength = 0;
				this->readPBPtr->more = TRUE;
				
				// now add the task to the periodic task manager queue
				this->hasCompleted = FALSE;
				this->ourManager->AddPeriodicTask(this);
				return;
			  }
		  }
	
		// we will only get here if memory allocation failed
		NukePtr(this->readBuffer);
		NukePtr(this->readPBPtr);
	}

/*-----------------------------------------------------------------*/

void CPPReadTask::DoPeriodicAction (void)
// if the read has completed or failed, store the result code
// and note that we are done
{
	// call the inherited method to update frequency count
	CPPPeriodicTask::DoPeriodicAction();

	switch (this->readPBPtr->ioResult) {
	
		case 1 :		// read hasn't completed yet
			break;
			
		case noErr :
			if (this->readPBPtr->actualLength > 0)
			  {				  
				// allocate or enlarge the return buffer to hold the
				// data which was returned
				if (!this->returnData)
				  this->returnData = NewHandleClear(this->readPBPtr->actualLength);
				else
				  SetHandleSize(this->returnData, totalSize + 
				  				  this->readPBPtr->actualLength);
				
				if ((this->callResult = MemError()) != noErr)
				 {
				 	this->hasCompleted = TRUE;
					return;
				 }

				// now, copy the data into to return buffer
				BlockMove (this->readBuffer, (*this->returnData) + this->totalSize, 
						   this->readPBPtr->actualLength);
				this->totalSize += this->readPBPtr->actualLength;
			  }
			
			// if there is more data, issue a read call
			if (this->readPBPtr->more)
			  {
				this->readPBPtr->ioCompletion = NULL;
				this->readPBPtr->sessRefNum = sessionID;
				this->readPBPtr->bufferLength = this->blockSize;
				this->readPBPtr->bufferPtr = this->readBuffer;
				
				// make the call to read the data asynchronously
				this->callResult = PPCRead(this->readPBPtr, kAsync);
			  }
			else
			  {
			  	this->blockCreator = this->readPBPtr->blockCreator;
			  	this->blockType = this->readPBPtr->blockType;
			    this->hasCompleted = TRUE;
			  }
			break;
	
		default :
			this->callResult = this->readPBPtr->ioResult;
	  	  	this->hasCompleted = TRUE;
	  	  	SysBeep(1);
			break;
	}
}

/*-----------------------------------------------------------------*/

	void CPPReadTask::DoCompletedAction (void)
	// when the task has completed, de-allocate the space used for the
	// receiving buffer
	{
		// dispose of the read buffer
		NukePtr(this->readBuffer);
		NukePtr(this->readPBPtr);
		
		CPPPeriodicTask::DoCompletedAction();
	}
	
/*-----------------------------------------------------------------*/

	void CPPReadTask::GetDataInfo (long *dataLen, OSType *type, 
								   OSType *creator)

	/* return the size of the data in the return buffer, */
	/* along with it's type and creator */
	{
		if (this->returnData)
		  {
		    *dataLen = GetHandleSize(this->returnData);
		    *type = this->blockType;
		    *creator = this->blockCreator;
		  }
		else
		  {
		    *dataLen = 0;
		    *type = (OSType)"????";
		    *creator = (OSType)"????";
		  }
	}

/*-----------------------------------------------------------------*/

	Handle CPPReadTask::GetData (Boolean BecomeOwner, Boolean *AmITheOwner)
	/* return the handle containing the data which was read by the task */
	/* If BecomeOwner is TRUE, relinquish responsibility for disposing */
	/* of the handle; if we have already relinquished responsibility */
	/* (ie. someone else has already become the owner of the handle) */
	/* return FALSE in AmITheOwner, otherwise return TRUE */
	{
		if (this->returnData)
		  {
			*AmITheOwner = ownsData && BecomeOwner;
			if (BecomeOwner)
			  this->ownsData = FALSE;
		  }
		else
		  *AmITheOwner = FALSE;
		return this->returnData;
	}

/*-----------------------------------------------------------------*/

