/********************************************************* DEFINITION
	DATE:	9/28/93
	
	CLASS:  CPPTaskList
	
	SUPERCLASS: CPPVisualObjectList
	
		This C++ class manages a list of CPPPeriodicTask objects
	
********************************************************************/

#pragma once

#include <CPPTaskList.h>
#include <CPPPeriodicTask.h>
#include <CPPTaskManager.h>
#include <MemoryTools.h>
#include <CPPObjectList.h>
#include <CPPDArray.h>
#include <string.h>

/*-----------------------------------------------------------------*/
/*--------------------------- GLOBALS -----------------------------*/
/*-----------------------------------------------------------------*/

	extern		RGBColor	RGBWhite;
	extern		RGBColor	RGBBlack;
	extern		RGBColor	RGBRed;

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/


	CPPTaskList::CPPTaskList (CPPWindow *theWindow, Rect *boundsRect,
							  CPPDArray *coreData, Boolean ownsData,
							  Boolean canBeTarget) :
				 CPPVisualObjectList (theWindow, boundsRect, 
				 					  coreData, ownsData, canBeTarget, 0, 14)
	{

	}
	
/*-----------------------------------------------------------------*/

	CPPTaskList::~CPPTaskList (void)
	{
	
	}
			
/*-----------------------------------------------------------------*/

	char	*CPPTaskList::ClassName (void)
	{
		return "CPPTaskList";
	}
				
/*-----------------------------------------------------------------*/

	CPPPeriodicTask	*CPPTaskList::operator[] (long getWhich)
	{
		CPPTaskManager	*theList = (CPPTaskManager *)this->GetItsData();
		
		if (theList)
		  return (CPPPeriodicTask *)((*theList)[getWhich]);
		else
		  return NULL;
	}

/*-----------------------------------------------------------------*/
/*---------------------- PROTECTED METHODS ------------------------*/	
/*-----------------------------------------------------------------*/

	void	CPPTaskList::DrawCellContents (long whichCell, Rect *itsFrame, 
									  	   Boolean selected)
	{		
		Str32	countStr, periodStr;
		CPPPeriodicTask	*theTask = NULL;
		short	countPos = itsFrame->right - 35,
				periodPos = itsFrame->right - 70;
		Rect	tempRect;
		GVarHandle	CG = (GVarHandle)((CWindowPeek)
						  (this->owningWindow))->port.grafVars;
		
		if (this->coreData)
		  theTask = (*this)[whichCell];
	
		if (this->InColorWindow() && selected)
		  RGBBackColor ((&(**CG).rgbHiliteColor));

		if (theTask)
		  {
		  	if (theTask->isRunning)
		  	  RGBForeColor(&RGBRed);
		  	NumToString(theTask->GetPeriod(), periodStr);
		  	NumToString(theTask->GetTimesCalled(), countStr);
		  	// draw the task name
		  	SetRect (&tempRect, itsFrame->left, itsFrame->top,
		  						periodPos, itsFrame->bottom);
		    TextBox (theTask->ClassName(), strlen(theTask->ClassName()), 
		  		   	 &tempRect, teJustLeft);
		  	// draw the period
		  	SetRect (&tempRect, periodPos, itsFrame->top,
		  						countPos, itsFrame->bottom);
		  	TextBox (periodStr+1, *periodStr, &tempRect, teJustRight);
		  	// draw the count
		  	SetRect (&tempRect, countPos, itsFrame->top,
		  						itsFrame->right, itsFrame->bottom);
		  	TextBox (countStr+1, *countStr, &tempRect, teJustRight);
		  	
			RGBForeColor (&RGBBlack);
		  	MoveTo(periodPos, itsFrame->top);
		  	LineTo(periodPos, itsFrame->bottom);

		  	MoveTo(countPos, itsFrame->top);
		  	LineTo(countPos, itsFrame->bottom);
		  }
		  
		if (selected)
		  if (InColorWindow())
		    RGBBackColor (&RGBWhite);
		  else
		    InvertRect (itsFrame);

		RGBForeColor (&RGBBlack);
	}

/*-----------------------------------------------------------------*/

	void CPPTaskList::ReceiveMessage( CPPGossipMonger *toldBy, 
									  short reason, void* info)
	/* respond to messages from the core data list */
	{
		Rect	tempRect;
		Point	tempPt = {(long)info - 1, 0};
		StringPtr		TempPtr = NULL;
		
		switch (reason) {
		
			case kTaskActivated 	:				
			case kTaskDeactivated	:
				GetCellRect ((long)info, &tempRect);
				DrawCell ((long)info, &tempRect, CellSelected((long)info));
				break;
				
			default	:
				CPPVisualObjectList::ReceiveMessage (toldBy, reason, info);
				break;
		}
		
	}

