/********************************************************* DEFINITION
	DATE:	9/16/93
	AUTHOR: Eric R. Ros
	
	CLASS:  CPPTaskManager
	
	SUPERCLASS: CPPObjectList
	
		This C++ class is an abstract class for keeping track of 
		periodic tasks.
	
********************************************************************/

#include <CPPTaskManager.h>
#include <CPPPeriodicTask.h>
#include <string.h>

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPTaskManager::CPPTaskManager (void)
	{
		
	}
	
/*-----------------------------------------------------------------*/

	CPPTaskManager::~CPPTaskManager (void)
	/* dispose of all the tasks in our queue */
	{
		DeleteItems(TRUE);
	}
	
/*-----------------------------------------------------------------*/

	char 	*CPPTaskManager::ClassName (void)
	{
		return "CPPTaskManager";
	}

/*-----------------------------------------------------------------*/
	
	Boolean CPPTaskManager::AddPeriodicTask (CPPPeriodicTask *TheTask)
	/* try to add the task to the list; return FALSE if it could */
	/* not be added successfully, TRUE otherwise */
	{
		long	oldNumItems = this->numItems;
		
		this->AppendItem((CPPObject *)TheTask);
		if (this->numItems != oldNumItems)
		  return TRUE;
		else
		  return FALSE;
	}
	
/*-----------------------------------------------------------------*/

	Boolean CPPTaskManager::RemovePeriodicTask (CPPPeriodicTask *TheTask,
												Boolean disposeOfTask)
	/* remove TheTask from the list of periodic tasks we service; if */
	/* we can't remove it, return FALSE, otherwise return TRUE */
	{
		long	oldNumItems = this->numItems;
		
	  	this->DeleteItem(TheTask, disposeOfTask);
	  	if (oldNumItems != this->numItems)
	  	  return TRUE;
		else
		  return FALSE;
	}
	
/*-----------------------------------------------------------------*/

	long CPPTaskManager::HowManyTasksOfType (char *ClassName)
	/* Given a c-formatted string describing the name of a class */
	/* this routine returns how many objects of that class are in the */
	/* task queue */
	{
		long	i = 1;
		long	Count = 0;
		
		while (i <= numItems)
		  {
		  	if (strcmp (((*this)[i])->ClassName(), ClassName) == 0)
		  	  Count ++;
		  	i++;
		  }
		return Count;
	}
	
/*-----------------------------------------------------------------*/

	void CPPTaskManager::RunPeriodicTasks (void)
	/* Give all of the periodic tasks we manage a chance to run if */
	/* their period has expired */
	{
		long	i = 1;
		CPPPeriodicTask	*TheTask;
		
		while (i <= numItems)
		  {
		  	TheTask = (CPPPeriodicTask *)((*this)[i]);
		  	if (TheTask)
		  	  {
		  	  	// execute the task if it needs to run
		  		if (TheTask->NeedsToRun())
		  		  {
		  		  	TheTask->isRunning = TRUE;
		  		  	BroadcastMessage (kTaskActivated, (void *)i);
		  		    TheTask->DoPeriodicAction();
		  		    TheTask->isRunning = FALSE;
		  		  	BroadcastMessage (kTaskDeactivated, (void *)i);
		  		  }
		  		
		  		// if it has completed remove it from the queue
		  		if (TheTask->hasCompleted)
		  		  {
		  		    CPPList::DeleteItem(i, FALSE);	// dequeue task without deleting
		  		    TheTask->DoCompletedAction();	// perform final operation
		  		    if (TheTask->deleteWhenDone)	// dispose of the task if 
		  		      delete TheTask;				//  requested
		  		  }
		  		else
		  		  i++;	// only 'move' to the next item if we don't delete this one
		  	  }
		  }
	}
	
/*-----------------------------------------------------------------*/

	short CPPTaskManager::HowManyPeriodicTasks (void)
	{
		return GetNumItems();
	}
	
/*-----------------------------------------------------------------*/

