/********************************************************* DEFINITION
	DATE:	9/28/93
	AUTHOR: Eric R. Ros
	
	CLASS:  CPPUserList
	
	SUPERCLASS: CPPVisualObjectList
	
		This C++ class manages a list of CPPNodeInfo objects which
		contain the addresses of all known yenta applications
	
********************************************************************/

#pragma once

#include "CPPUserList.h"
#include <CPPNodeInfo.h>
#include <MemoryTools.h>
#include <CPPObjectList.h>
#include <CPPDArray.h>

/*-----------------------------------------------------------------*/
/*--------------------------- GLOBALS -----------------------------*/
/*-----------------------------------------------------------------*/

	extern		RGBColor	RGBWhite;
	extern		void DoSendButtonClick (CPPWindow *ourWindow);
	extern		CPPNodeInfo 	*gOurIdentity;
	extern		StringPtr	ShortName	 (CPPNodeInfo *theNode);

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/


	CPPUserList::CPPUserList (CPPWindow *theWindow, Rect *boundsRect,
							  CPPDArray *coreData, Boolean ownsData) :
				 CPPVisualObjectList (theWindow, boundsRect, 
				 					  coreData, ownsData, TRUE, 0, 14)
	{
		
	}
	
/*-----------------------------------------------------------------*/

	CPPUserList::~CPPUserList (void)
	{
	
	}
			
/*-----------------------------------------------------------------*/

	char	*CPPUserList::ClassName (void)
	{
		return "CPPUserList";
	}
				
/*-----------------------------------------------------------------*/

	Boolean	CPPUserList::AddNewUser (CPPNodeInfo *UserData)
	/* if the node represented by 'userData' is not already in the */
	/* user list, add it */ 
	{
		long	itsIndex;
		
		if (UserData && this->coreData)
		  {
			if (this->KnownUser(UserData, &itsIndex))
			  return FALSE;
			((CPPObjectList *)this->coreData)->AppendItem(UserData->Clone());
			return TRUE;
		  }
		else
		  return FALSE;
	}
	
/*-----------------------------------------------------------------*/

	void	CPPUserList::DeleteUser (CPPNodeInfo *UserData)
	/* if the node represented by 'UserData' is in the list, delete it */
	{
		long	itsIndex;
		
		if (this->coreData && this->KnownUser(UserData, &itsIndex))
		  this->coreData->DeleteItem (itsIndex, TRUE);
	}
	
/*-----------------------------------------------------------------*/

	Boolean	CPPUserList::KnownUser (CPPNodeInfo *UserData, long *whichItem)
	/* if the node represented by 'UserData' is in the list, return TRUE */
	/* and set 'whichitem' to its index in the list.  Otherwise, return FALSE */
	{		
		if (!this->coreData)
		  return FALSE;
		  
		for (long i = 1; i <= GetNumItems(); i++)
		  if (UserData->Equals((*this)[i]))
		    {
		      *whichItem = i;
		      return TRUE;
		    }
		    
		return FALSE;
	}

/*-----------------------------------------------------------------*/

	CPPNodeInfo	*CPPUserList::operator[] (long getWhich)
	{
		CPPObjectList	*theList = (CPPObjectList *)this->GetItsData();
		
		if (theList)
		  return (CPPNodeInfo *)((*theList)[getWhich]);
		else
		  return NULL;
	}

/*-----------------------------------------------------------------*/
/*---------------------- PROTECTED METHODS ------------------------*/	
/*-----------------------------------------------------------------*/

	void	CPPUserList::DrawCellContents (long whichCell, Rect *itsFrame, 
									  	   Boolean selected)
	{		
		StringPtr	UsersName;
		CPPNodeInfo	*theNode = NULL;
		short		i = 1;
		GVarHandle	CG = (GVarHandle)((CWindowPeek)
						  (this->owningWindow))->port.grafVars;
		
		if (this->coreData)
		  theNode = (*this)[whichCell];
	
		if (this->InColorWindow() && selected)
		  RGBBackColor ((&(**CG).rgbHiliteColor));

		if (theNode)
		  {
		  	UsersName = ShortName(theNode);
			  	
		  	if (theNode->Equals(gOurIdentity))
		  	  TextFace(bold);
		  	TextBox(UsersName+1, *UsersName, itsFrame, teJustLeft);
		  	NukePtr(UsersName);
		  	TextFace(0);
		  }
		  
		if (selected)
		  if (InColorWindow())
		    RGBBackColor (&RGBWhite);
		  else
		    InvertRect (itsFrame);
	}
		
