/***************************************************** IMPLEMENTATION
	DATE:	9/15/93
	AUTHOR: Eric R. Ros

	CLASS:  CPPList
	
	SUPERCLASS: none
	
		This C++ class manages a list of pointers
	
********************************************************************/

#include <CPPList.h>
#include <MathTools.h>

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPList::CPPList (void) : CPPDArray (sizeof(Ptr))
	/* Class constructor */
	{

	}

/*-----------------------------------------------------------------*/

	CPPList::~CPPList (void)
	/* Class destructor */
	{

	}

/*-----------------------------------------------------------------*/

	Boolean	CPPList::Member (char *className)
	{
		if (strcmp(className, CPPList::ClassName()) == 0)
		  return TRUE;
		else
		  return CPPDArray::Member(className);
	}
	
/*-----------------------------------------------------------------*/

	char *CPPList::ClassName (void)
	{
		return "CPPList";
	}
	
/*-----------------------------------------------------------------*/

	void	CPPList::InsertItem (Ptr NewItem, long insertAt)
	/* insert a pointer in the list at the specified position.  if */
	/* insertAt is larger than the number of items, it is appended */
	/* to the end of the list */
	{
		CPPDArray::InsertItem((Ptr)(&NewItem), insertAt);
	}

/*-----------------------------------------------------------------*/

	void	CPPList::AppendItem (Ptr NewItem)
	/* add a pointer to the end of the list */
	{
		CPPDArray::InsertItem((Ptr)(&NewItem), GetNumItems()+1);
	}

/*-----------------------------------------------------------------*/

	void	CPPList::DeleteItems (Boolean doDispose)
	/* remove all pointers from the list; if doDispose is TRUE, dispose */
	/* of the memory each pointer points to */
	{		
		for (long i = GetNumItems(); i > 0; i--)
		  DeleteItem (i, doDispose);
	}

/*-----------------------------------------------------------------*/

	void	CPPList::DeleteItem (long deleteWhich, Boolean doDispose)
	/* remove the pointer from the list; if doDispose is TRUE, dispose */
	/* of the memory the pointer points to */
	{	
		Ptr TempPtr;
		
		if (IN_RANGE(deleteWhich, 1, GetNumItems()))
		  {
		  	if (doDispose)	// clear the pointer if requested
		  	  ClearItem(deleteWhich);
		  	CPPDArray::DeleteItem (deleteWhich, doDispose);
		  }
	}

/*-----------------------------------------------------------------*/

	Ptr CPPList::operator[] (long getWhich)
	/* return the pointer at position getWhich */
	{
		Ptr TempPtr = NULL;
		
		if (IN_RANGE(getWhich, 1, GetNumItems()))
		  {
		  	CPPDArray::Get(&TempPtr, getWhich);
		  	return TempPtr;
		  }
		else
		  return NULL;
	}

/*-----------------------------------------------------------------*/

	void	CPPList::SetItem (Ptr NewItem, long setWhich, Boolean deleteOld)
	/* Replace the pointer at position setWhich; dispose of the memory */
	/* pointed to by the pointer at that position if deleteOld is TRUE */
	{
		if (IN_RANGE(setWhich, 1, this->numItems))
		  {
		  	if (deleteOld)
		  	  ClearItem(setWhich);
		  	  
			CPPDArray::SetItem ((Ptr)(&NewItem), setWhich, deleteOld);
		  }
	}
	
/*-----------------------------------------------------------------*/

	void	CPPList::Swap (long Index1, long Index2)
	/* if both indices are valid, this routine swaps the items */
	/* at those indices */
	{
		Ptr Item1, Item2;
		
		if (IN_RANGE(Index1, 0, this->numItems) &&
			IN_RANGE(Index2, 0, this->numItems))
		  {
		  	Get (&Item1, Index1);
		  	Get (&Item2, Index2);
		  	
		  	SetItem ((Ptr)(&Item1), Index2, FALSE);
		  	SetItem ((Ptr)(&Item2), Index1, FALSE);
		  }
	}
	
/*-----------------------------------------------------------------*/
	
	long	CPPList::GetNumItems (void)
	{
		return this->numItems;
	}

/*-----------------------------------------------------------------*/

	long	CPPList::FindIndex (Ptr ItemToFind)
	/* search for any occurrance of the pointer ItemToFind in the */
	/* list; if it is present, return its index */
	{
		return CPPDArray::FindIndex ((Ptr)(&ItemToFind));
	}


/*-----------------------------------------------------------------*/
/*---------------------- PROTECTED METHODS ------------------------*/
/*-----------------------------------------------------------------*/

	void	CPPList::ClearItem (long clearWhich)
	/* Dispose of the memory pointed to by the item at position */
	/* clearWhich without resizing the list */
	{
		Ptr		TempPtr;
		long	Nada = 0L;
		
		if (IN_RANGE (clearWhich, 1, this->numItems))
		  {
		  	Get (&TempPtr, clearWhich);
	  	    if (TempPtr)
	  	      {
	  	      	DisposePtr(TempPtr);
				CPPDArray::SetItem ((Ptr)(&Nada), clearWhich, FALSE);
	 	      }
		  }
	}

/*-----------------------------------------------------------------*/

	void	CPPList::ClearItems (void)
	/* Dispose of the memory pointed to by all items in the list */
	/* without resizing the list */
	{
		for (long i=1;i<=this->numItems; i++)
		  ClearItem(i);
	}