/***************************************************** IMPLEMENTATION
	DATE:	9/17/93
	AUTHOR: Eric R. Ros

	CLASS:  CPPStringList
	
	SUPERCLASS: CPPList
	
		This C++ class manages a list of Pascal strings
	
********************************************************************/

#include <CPPStringList.h>
#include <stdlib.h>
#include <StringTools.h>

/*-----------------------------------------------------------------*/
/*--------------------- INTERNAL UTILITIES ------------------------*/
/*-----------------------------------------------------------------*/
	
	#ifdef __cplusplus
	extern "C" {
	#endif
		int compare (const void *n1, const void *n2);
	#ifdef __cplusplus
	}
	#endif

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/

	CPPStringList::CPPStringList (void) : CPPList ()
	{
	
	}
	
/*-----------------------------------------------------------------*/

	CPPStringList::~CPPStringList (void)
	{
	
	}
			
/*-----------------------------------------------------------------*/

	Boolean	CPPStringList::Member (char *className)
	{
		if (strcmp(className, CPPStringList::ClassName()) == 0)
		  return TRUE;
		else
		  return CPPList::Member(className);
	}

/*-----------------------------------------------------------------*/

	char *CPPStringList::ClassName (void)
	{
		return "CPPStringList";
	}
	
/*-----------------------------------------------------------------*/

	void CPPStringList::InsertItem (StringPtr NewItem, long insertAt)
	/* use the inherited method to insert a string */
	{
		CPPList::InsertItem((Ptr)NewItem, insertAt);
	}
	
/*-----------------------------------------------------------------*/

	void CPPStringList::AppendItem (StringPtr NewItem)
	/* use the inherited method to add a string */
	{
		CPPList::AppendItem((Ptr)NewItem);
	}
	
/*-----------------------------------------------------------------*/
			
	StringPtr CPPStringList::operator[] (long getWhich)
	/* use the inherited method to return the specified string */
	{
		return (StringPtr)(CPPList::operator[](getWhich));
	}
	
/*-----------------------------------------------------------------*/

	void CPPStringList::SetItem (StringPtr NewItem, long setWhich, Boolean deleteOld)
	/* use the inherited method to set a string */
	{
		CPPList::SetItem((Ptr)NewItem, setWhich, deleteOld);
	}
	
/*-----------------------------------------------------------------*/

	long CPPStringList::FindIndex	(StringPtr ItemToFind)
	/* use the inherited method to find a string */
	{
		return CPPList::FindIndex ((Ptr)ItemToFind);
	}
	
/*-----------------------------------------------------------------*/

	void CPPStringList::InsertInOrder (StringPtr TheString)
	/* the only truly unique method; use a built in string */
	/* comparison function to place TheString in the list */
	/* in sorted (ascending) order */
	{
		long	First = 1, 
				Last = numItems, 
				Middle = 1;
		short	CmpResult;
		Boolean	Found = FALSE;
		
		while ((!Found) && (Last >= First))
		  {
		  	Middle = (First + Last) / 2;
		  	CmpResult = PStrCmp (TheString, (*this)[Middle]);
		  	if (CmpResult < 0)		// TheString comes before Middle
		  	  Last = Middle - 1;
		  	else if (CmpResult > 0)	// TheString comes after Middle
		  	  First = Middle + 1;
		  	else					// TheString goes here
		  	  {
		  	    First = Middle; 
		  	    break;
		  	  }
		  }
		  
		// if we have gotten this far, there was no exact match for
		// TheString, so insert it at Middle
		InsertItem(TheString, First);
	}
	
/*-----------------------------------------------------------------*/	

	void CPPStringList::SortList (void)
	/* Use C's built in QSort routine to sort the array */
	{
		HLock(this->listItems);
		qsort(*this->listItems, this->numItems, sizeof(StringPtr),
			  compare);
		HUnlock(this->listItems);
		
		// let the people who depend on us know that all items may
		// have changed
		BroadcastMessage (kAllChanged, NULL);
	}

/*-----------------------------------------------------------------*/	
	
	#ifdef __cplusplus
	extern "C" {
	#endif
		int compare (const void *n1, const void *n2)
		{
			return PStrCmp (*((StringPtr *) n1), *((StringPtr *) n2));
		}
	#ifdef __cplusplus
	}
	#endif

