/***************************************************** IMPLEMENTATION
	DATE:	10/24/93
	AUTHOR: Eric R. Ros

	CLASS:  CPPTree
	
	SUPERCLASS: CPPGossipMonger
	
		This C++ class manages the common data (root) for a 
		hierarchical tree.  It is meant to be used in conjunction
		with CPPTreeNode
	
********************************************************************/

#include <CPPTree.h>
#include <CPPTreeNode.h>

/*-----------------------------------------------------------------*/
/*------------------------ PUBLIC METHODS -------------------------*/
/*-----------------------------------------------------------------*/
			
	CPPTree::CPPTree (void)
	{
		this->topNode = NULL;
	}

/*-----------------------------------------------------------------*/

	CPPTree::~CPPTree (void)
	/* if there is any data in the tree, destroy it */
	{
		if (this->topNode)
		  topNode->ClearFamily (topNode);
	}

/*-----------------------------------------------------------------*/

	Boolean	CPPTree::Member (char *className)
	{
		if (strcmp(className, CPPTree::ClassName()) == 0)
		  return TRUE;
		else
		  return CPPGossipMonger::Member(className);
	}
	
/*-----------------------------------------------------------------*/
			
	char *CPPTree::ClassName (void)
	{
		return "CPPTree";
	}
		
/*-----------------------------------------------------------------*/

	short	CPPTree::Depth (void)
	/* return the maximum depth of the tree */
	{
		if (this->topNode)
		  return this->topNode->FamilyDepth(0);
		else
		  return 0;
	}

/*-----------------------------------------------------------------*/

	void	CPPTree::SetTopMostNode (CPPTreeNode *theNode)
	/* keep track of who the first node in the tree is */
	/* dependent on */
	{
		// cut the 'listen to' dependency b/w the node and ourselves
		if (this->topNode)
		  this->topNode->StopListeningTo(this);

		this->topNode = theNode;
		theNode->Root = this;
		
		// establish a listen to dependency b/w the node and ourselves
		this->topNode->ListenTo (this);
	}

/*-----------------------------------------------------------------*/

	void	CPPTree::ReceiveMessage (CPPGossipMonger *toldBy, 
									 short reason, void* info)
	{
		CPPGossipMonger::ReceiveMessage(toldBy, reason, info);
	}
