/***************************************************** IMPLEMENTATION
	DATE:	10/24/93
	AUTHOR: Eric R. Ros

	CLASS:  CPPTreeNode
	
	SUPERCLASS: CPPObjectList
	
		This C++ class manages a node in a hierarchical tree
	
********************************************************************/

#pragma once

#include <CPPObjectList.h>
class CPPTreeNode;
class CPPTree;

#define		kPrepareDelete		9
#define		kChildDeleted		10
#define		kNodeAdded			11
#define		kPrepareRemove		12
#define		kChildRemoved		13
#define		kNodeChangedData	14

typedef Boolean (*BApplyProc)(CPPTreeNode *theNode, long param);
typedef void (*ApplyProc)(CPPTreeNode *theNode, long param);


class CPPTreeNode : public CPPObjectList {
public:
	CPPTree			*Root;
	CPPTreeNode		*Parent;
	
			CPPTreeNode (CPPObject *NodeData, CPPTree *BelongsTo, Boolean becomeOwner);
			~CPPTreeNode (void);
			
	virtual	char *ClassName (void);
	virtual	Boolean	Member (char *className);
		
	virtual CPPObject *Clone(void);

	void	TellParent (short Message, void *data);

			CPPTreeNode	*NthChild (long whichChild);

			void	AddNode (CPPTreeNode *NewNode);
			void	InsertNode (CPPTreeNode *NewNode, long insertWhere);

	static	Boolean	DeleteNode (CPPTreeNode *theNode);
	static	void 	DeleteFamily (CPPTreeNode *theNode);
	static	void	ClearFamily (CPPTreeNode *theNode);
	static	long	FamilySize (CPPTreeNode *theNode);
			
			void	RemoveChild (CPPTreeNode *theChild);
			Boolean InsertParent (CPPTreeNode *NewNode);
			void	SwitchParents (CPPTreeNode *OldParent, CPPTreeNode *NewParent);

			short	FamilyDepth (short currentDepth);

			CPPObjectList	*BuildSelectionList (void);

			CPPObject	*GetNodeData (void);
			void		SetNodeData (CPPObject *NewData, Boolean becomeOwner);

	virtual	void	ReceiveMessage (CPPGossipMonger *toldBy, 
									short reason, void* info);
	virtual	void	GetFamilyBounds (Rect *bounds);
	
protected:
	CPPObject	*nodeData;
	Boolean		ownsData;
};

void	ApplyToFamily (CPPTreeNode *familyHead, ApplyProc theProc, 
					   long param);
Boolean	BApplyToFamily (CPPTreeNode *familyHead, BApplyProc theProc, 
						long param);
