/*
	File:		TextWindow.cp

	Contains:	A window designed to hold some TEXT no scroll bars...
	Written by: Dave Mark
	Copyright:	1995 by Dave Mark, all rights reserved.
 */


const short			kTextWindowTemplateID = 1029;
const short			kDefaultSTRResID = 1000;


#include "TextWindow.h"
#include <ToolUtils.h>

MenuHandle			TTextWindow::fgMenu;
unsigned long		TTextWindow::fgWindowTitleCount = 0;


//	methods

TTextWindow::TTextWindow()
{
	fDraggedTextHandle = nil;

	TTextWindow::fgWindowTitleCount++;	//	Starts out as zero but we want Text Window 1
	this->CreateWindow();	// kNormalWindow is default. Method inherited from TWindow
							// TWindow::CreateWindow() calls our MakeNewWindow().
}


TTextWindow::~TTextWindow()
{
//	No particular cleanup in this version, since our real purpose was
//	to demonstrate the Drag Manager and menus, not to build a text handling app.
}


WindowPtr
TTextWindow::MakeNewWindow( WindowPtr behindWindow )
{
	WindowPtr	aWindow;
	Str255		titleString;
	GrafPtr		savedPort;
	
	GetPort(&savedPort);
	
	aWindow = GetNewColorOrBlackAndWhiteWindow( kTextWindowTemplateID,
							nil, behindWindow );
	
	if (aWindow)
	{
		GetWTitle(aWindow,titleString);
		if (StrLength(titleString) != 0)
		{
			Str255 numberString;
			
			NumToString( fgWindowTitleCount, numberString );
			BlockMove(&numberString[1],&titleString[titleString[0]+1],numberString[0]);
			titleString[0] += numberString[0];
		}
		SetWTitle(aWindow,titleString);

		SetPort(aWindow);

		ShowWindow(aWindow);
	}
	SetPort(savedPort);

	return aWindow;
}


void
TTextWindow::Draw(void)
{
	Rect		r;
	char		*textPtr;
	long		textLength;
	Handle		stringH;
	
	r = fWindow->portRect;
	EraseRect( &r );
//	Two cases to deal with...
//		1) If nothing's been dragged in yet, we'll use the default string which is
//			a handle to a pascal format string.
//		2) If there has been a received drag, fDraggedTextHandle is a handle to
//			a block of data (no pascal length byte)
	if ( fDraggedTextHandle == nil )
	{
		stringH = (Handle)GetString( kDefaultSTRResID );
		
		if ( stringH == nil )
			return;
		
		HLock( stringH );
		
		textPtr = &((*stringH)[1]);
		textLength = (long)((*stringH)[0]);
		TETextBox( textPtr, textLength, &r, teFlushLeft );
		
		HUnlock( stringH );
	}
	else
	{
		HLock( fDraggedTextHandle );
		
		TETextBox( *fDraggedTextHandle, (long)GetHandleSize(fDraggedTextHandle), &r, teFlushLeft );
		
		HUnlock( fDraggedTextHandle );
	}
}


void
TTextWindow::Activate( Boolean activating )
{
	if ( activating )
	{
		InsertMenu( fgMenu, 0 );
		gMenuBar->Invalidate();
	}
	else
		DeleteMenu( mText );
}


void
TTextWindow::Click( EventRecord * )
{
	this->Select();
}


void
TTextWindow::ClickAndDrag( EventRecord *eventPtr )
{
	OSErr			err;
	DragReference   dragRef;
	RgnHandle       dragRegion, tempRgn;
	Rect			itemBounds;
	char			*textPtr;
	long			textLength;
	Handle			stringH;
    
    err = NewDrag( &dragRef );
    if ( err != noErr )
		return;	// Should return err, but needs a change to TWindow::ClickAndDrag return type

	if ( fDraggedTextHandle == nil )
	{
		stringH = (Handle)GetString( kDefaultSTRResID );
		if ( stringH == nil )
			return;
			
		HLock( stringH );
		
		textPtr = &((*stringH)[1]);
		textLength = (long)((*stringH)[0]);	
	
		err = AddDragItemFlavor( dragRef,
	                            (ItemReference)fWindow,// no particular reason, could use any number here
	                            (FlavorType) 'TEXT',
	                            textPtr,
	                            textLength,
	                            (FlavorFlags)0 );
	
		HUnlock( stringH );
	}
	else
	{
		HLock( fDraggedTextHandle );
	
		err = AddDragItemFlavor( dragRef,
	                            (ItemReference)fWindow,// no particular reason, could use any number here
	                            (FlavorType) 'TEXT',
	                            *fDraggedTextHandle,
	                            (long)GetHandleSize(fDraggedTextHandle),
	                            (FlavorFlags)0 );
	
		HUnlock( fDraggedTextHandle );
	}
    if ( err != noErr )
	{
		DisposeDrag( dragRef );
		return;	// Should return err, but needs a change to TWindow::ClickAndDrag return type
	}
	
	itemBounds = (**((WindowPeek)fWindow)->contRgn).rgnBBox;
	
	err = SetDragItemBounds( dragRef, (ItemReference)fWindow, &itemBounds );
	if ( err != noErr )
	{
		DisposeDrag( dragRef );
		return;	// Should return err, but needs a change to TWindow::ClickAndDrag return type
	}
	
    dragRegion = NewRgn();
	RectRgn( dragRegion, &itemBounds );
	tempRgn = NewRgn();
	CopyRgn( dragRegion, tempRgn );
	InsetRgn( tempRgn, 1, 1 );
	DiffRgn( dragRegion, tempRgn, dragRegion );
	DisposeRgn( tempRgn );
	
    err = TrackDrag( dragRef, eventPtr, dragRegion );
    DisposeRgn( dragRegion );
    DisposeDrag( dragRef );
    return;	// Should return err, but needs a change to TWindow::ClickAndDrag return type
}


OSErr
TTextWindow::DragEnterWindow( DragReference dragRef )
{
	fCanAcceptDrag = IsTextFlavorAvailable( dragRef );
	fIsWindowHighlighted = false;
	
	if ( fCanAcceptDrag )
		return noErr;
	else
		return dragNotAcceptedErr;
}


OSErr
TTextWindow::DragInWindow( DragReference dragRef )
{
	DragAttributes	attributes;
	RgnHandle		tempRgn;

	GetDragAttributes( dragRef, &attributes );
	
	if ( (! fCanAcceptDrag) || (! (attributes & dragHasLeftSenderWindow)) 
		|| (attributes & dragInsideSenderWindow) )
		return dragNotAcceptedErr;
	
	if ( this->IsMouseInContentRgn( dragRef ) )
	{
		if ( ! fIsWindowHighlighted )
		{
			tempRgn = NewRgn();
			RectRgn( tempRgn, &fWindow->portRect );
			
			if ( ShowDragHilite( dragRef, tempRgn, true ) == noErr )
				fIsWindowHighlighted = true;
				
			DisposeRgn(tempRgn);
		}
	}
		
	return noErr;
}


OSErr
TTextWindow::DragLeaveWindow( DragReference dragRef )
{
	if ( fIsWindowHighlighted )
		HideDragHilite( dragRef );
	
	fIsWindowHighlighted = false;
	fCanAcceptDrag = false;
	
	return noErr;
}


OSErr
TTextWindow::HandleDrop( DragReference dragRef )
{
	OSErr			err;
	Size			dataSize;
	ItemReference	item;
	FlavorFlags		flags;
	DragAttributes	attributes;

	GetDragAttributes( dragRef, &attributes );
	
	if ( attributes & dragInsideSenderWindow )
		return dragNotAcceptedErr;

	err = GetDragItemReferenceNumber( dragRef, 1, &item );
	if ( err == noErr )
		err = GetFlavorFlags( dragRef, item, 'TEXT', &flags );

	if ( err == noErr )
	{
		err = GetFlavorDataSize( dragRef, item, 'TEXT', &dataSize);
		if  (err == noErr )
		{
			fDraggedTextHandle = TempNewHandle( dataSize, &err );
			
			if ( fDraggedTextHandle == nil )
				fDraggedTextHandle = NewHandle( dataSize );

			if ( fDraggedTextHandle == nil )
				err = dragNotAcceptedErr;
			else
			{
				HLock( fDraggedTextHandle );
				err = GetFlavorData( dragRef, item, 'TEXT',
						*fDraggedTextHandle, &dataSize, 0L );
				HUnlock( fDraggedTextHandle );

				if ( err != noErr)
				{
					err = dragNotAcceptedErr;
					DisposeHandle( fDraggedTextHandle );
					fDraggedTextHandle = nil;
				}
				else
				{
					SetPort( fWindow );
					InvalRect( &(fWindow->portRect) );
				}
			}
		}
	}
	
	return( err );
}


void
TTextWindow::SetTextFont( short newFont )
{
	GrafPtr	oldPort;
	
	GetPort( &oldPort );
	SetPort( fWindow );
	
	TextFont( newFont );
	
	SetPort( oldPort );
}


Boolean
TTextWindow::IsTextFlavorAvailable( DragReference dragRef )
{
	unsigned short	numItems;
	FlavorFlags		flags;
	OSErr			err;
	ItemReference	item;
	
	CountDragItems( dragRef, &numItems );
	
	if ( numItems < 1 )
		return( false );
	
	err = GetDragItemReferenceNumber( dragRef, 1, &item );
	if ( err == noErr )
		err = GetFlavorFlags( dragRef, item, 'TEXT', &flags );
	
	return( err == noErr );
}


Boolean
TTextWindow::IsMouseInContentRgn( DragReference dragRef )
{
	Point	globalMouse;
	OSErr	err;
	
	err = GetDragMouse( dragRef, &globalMouse, 0L );
	
	if ( err == noErr )
		return( PtInRgn( globalMouse, ((WindowPeek)fWindow)->contRgn ) );
	else
		return( false );
}


void
TTextWindow::SetUpStaticMenu( void )
{
	TTextWindow::fgMenu = gMenuBar->GetMenuFromCMNU( mText );
}