/*
	File:		TextWindow.cp

	Contains:	A window designed to hold some TEXT no scroll bars...
	Written by: Dave Mark
	Copyright:	1995 by Dave Mark, all rights reserved.
 */


const short			kTextWindowTemplateID = 1029;
const short			kDefaultSTRResID = 1000;
const short			kMinimumFontSize = 6;
const short			kMaximumFontSize = 255;


#include "TextWindow.h"
#include <ToolUtils.h>

MenuHandle			TTextWindow::fgMenu;
MenuHandle			TTextWindow::fgFontSubMenu;
MenuHandle			TTextWindow::fgSizeSubMenu;
MenuHandle			TTextWindow::fgStyleSubMenu;
unsigned long		TTextWindow::fgWindowTitleCount = 0;


//	methods

TTextWindow::TTextWindow()
{
	fDraggedTextHandle = nil;

	TTextWindow::fgWindowTitleCount++;	//	Starts out as zero but we want Text Window 1
	this->CreateWindow();	// kNormalWindow is default. Method inherited from TWindow
							// TWindow::CreateWindow() calls our MakeNewWindow().
}


TTextWindow::~TTextWindow()
{
//	No particular cleanup in this version, since our real purpose was
//	to demonstrate the Drag Manager and menus, not to build a text handling app.
}


WindowPtr
TTextWindow::MakeNewWindow( WindowPtr behindWindow )
{
	WindowPtr	aWindow;
	Str255		titleString;
	GrafPtr		savedPort;
	
	GetPort(&savedPort);
	
	aWindow = GetNewColorOrBlackAndWhiteWindow( kTextWindowTemplateID,
							nil, behindWindow );
	
	if (aWindow)
	{
		GetWTitle(aWindow,titleString);
		if (StrLength(titleString) != 0)
		{
			Str255 numberString;
			
			NumToString( fgWindowTitleCount, numberString );
			BlockMove(&numberString[1],&titleString[titleString[0]+1],numberString[0]);
			titleString[0] += numberString[0];
		}
		SetWTitle(aWindow,titleString);

		SetPort(aWindow);
		
		fCurrentFontSize = 18;
		TextSize( fCurrentFontSize );
		
		fCurrentFontStyle = 0;
		
		ShowWindow(aWindow);
	}
	SetPort(savedPort);

	return aWindow;
}


void
TTextWindow::Draw(void)
{
	Rect		r;
	char		*textPtr;
	long		textLength;
	Handle		stringH;
	
	r = fWindow->portRect;
	EraseRect( &r );
//	Two cases to deal with...
//		1) If nothing's been dragged in yet, we'll use the default string which is
//			a handle to a pascal format string.
//		2) If there has been a received drag, fDraggedTextHandle is a handle to
//			a block of data (no pascal length byte)
	if ( fDraggedTextHandle == nil )
	{
		stringH = (Handle)GetString( kDefaultSTRResID );
		
		if ( stringH == nil )
			return;
		
		HLock( stringH );
		
		textPtr = &((*stringH)[1]);
		textLength = (long)((*stringH)[0]);
		TETextBox( textPtr, textLength, &r, teFlushLeft );
		
		HUnlock( stringH );
	}
	else
	{
		HLock( fDraggedTextHandle );
		
		TETextBox( *fDraggedTextHandle, (long)GetHandleSize(fDraggedTextHandle), &r, teFlushLeft );
		
		HUnlock( fDraggedTextHandle );
	}
}


void
TTextWindow::Activate( Boolean activating )
{
	if ( activating )
	{
		InsertMenu( fgMenu, 0 );
		gMenuBar->Invalidate();
	}
	else
	{
		DeleteMenu( mText );
		gMenuBar->Invalidate();
	}
}


void
TTextWindow::Click( EventRecord * )
{
	this->Select();
}


void
TTextWindow::AdjustMenusBeforeMenuSelection( void )
{
	AdjustFontMenu();
	AdjustSizeMenu();
	AdjustStyleMenu();
}


Boolean
TTextWindow::DoMenuSelection( short menu, short item )
{
	Str255		itemString;
	short		fontNumber;
	
	if ( menu == mTextFont )
	{
		GetMenuItemText( fgFontSubMenu, item, itemString );
		GetFNum( itemString, &fontNumber );
		SetPort( fWindow );
		TextFont( fontNumber );
		InvalRect( &fWindow->portRect );
		return true;
	}
	else
		return false;
}


Boolean
TTextWindow::DoMenuCommand(unsigned long menuCommand)
{
	long	newSize;
	
	switch (menuCommand)
	{
		case cClose:
			if (this->Close() && this->DeleteAfterClose())
				delete this;
			return true;
			break;
			
		case cFontSize9:
			SetNewTextSize( 9 );
			return true;
			break;
		case cFontSize10:
			SetNewTextSize( 10 );
			return true;
			break;
		case cFontSize12:
			SetNewTextSize( 12 );
			return true;
			break;
		case cFontSize18:
			SetNewTextSize( 18 );
			return true;
			break;
		case cFontSize24:
			SetNewTextSize( 24 );
			return true;
			break;
		case cFontSize36:
			SetNewTextSize( 36 );
			return true;
			break;
			
		case cFontSizeLarger:
			SetNewTextSize( fCurrentFontSize + 1 );
			return true;
			break;
		case cFontSizeSmaller:
			SetNewTextSize( fCurrentFontSize - 1 );
			return true;
			break;
		case cFontSizeOther:
			newSize = DoNumberDialog();
			if ( newSize != kCancelButtonPressed )
				SetNewTextSize( newSize );
			return true;
			break;
			
		case cFontStylePlain:
		case cFontStyleBold:
		case cFontStyleItalic:
		case cFontStyleUnderline:
		case cFontStyleOutline:
		case cFontStyleShadow:
			DoStyleCommand( menuCommand );
			return true;
			break;
	}

	return false;
}


void
TTextWindow::ClickAndDrag( EventRecord *eventPtr )
{
	OSErr			err;
	DragReference   dragRef;
	RgnHandle       dragRegion, tempRgn;
	Rect			itemBounds;
	char			*textPtr;
	long			textLength;
	Handle			stringH;
    
    err = NewDrag( &dragRef );
    if ( err != noErr )
		return;	// Should return err, but needs a change to TWindow::ClickAndDrag return type

	if ( fDraggedTextHandle == nil )
	{
		stringH = (Handle)GetString( kDefaultSTRResID );
		if ( stringH == nil )
			return;
			
		HLock( stringH );
		
		textPtr = &((*stringH)[1]);
		textLength = (long)((*stringH)[0]);	
	
		err = AddDragItemFlavor( dragRef,
	                            (ItemReference)fWindow,// no particular reason, could use any number here
	                            (FlavorType) 'TEXT',
	                            textPtr,
	                            textLength,
	                            (FlavorFlags)0 );
	
		HUnlock( stringH );
	}
	else
	{
		HLock( fDraggedTextHandle );
	
		err = AddDragItemFlavor( dragRef,
	                            (ItemReference)fWindow,// no particular reason, could use any number here
	                            (FlavorType) 'TEXT',
	                            *fDraggedTextHandle,
	                            (long)GetHandleSize(fDraggedTextHandle),
	                            (FlavorFlags)0 );
	
		HUnlock( fDraggedTextHandle );
	}
    if ( err != noErr )
	{
		DisposeDrag( dragRef );
		return;	// Should return err, but needs a change to TWindow::ClickAndDrag return type
	}
	
	itemBounds = (**((WindowPeek)fWindow)->contRgn).rgnBBox;
	
	err = SetDragItemBounds( dragRef, (ItemReference)fWindow, &itemBounds );
	if ( err != noErr )
	{
		DisposeDrag( dragRef );
		return;	// Should return err, but needs a change to TWindow::ClickAndDrag return type
	}
	
    dragRegion = NewRgn();
	RectRgn( dragRegion, &itemBounds );
	tempRgn = NewRgn();
	CopyRgn( dragRegion, tempRgn );
	InsetRgn( tempRgn, 1, 1 );
	DiffRgn( dragRegion, tempRgn, dragRegion );
	DisposeRgn( tempRgn );
	
    err = TrackDrag( dragRef, eventPtr, dragRegion );
    DisposeRgn( dragRegion );
    DisposeDrag( dragRef );
    return;	// Should return err, but needs a change to TWindow::ClickAndDrag return type
}


OSErr
TTextWindow::DragEnterWindow( DragReference dragRef )
{
	fCanAcceptDrag = IsTextFlavorAvailable( dragRef );
	fIsWindowHighlighted = false;
	
	if ( fCanAcceptDrag )
		return noErr;
	else
		return dragNotAcceptedErr;
}


OSErr
TTextWindow::DragInWindow( DragReference dragRef )
{
	DragAttributes	attributes;
	RgnHandle		tempRgn;

	GetDragAttributes( dragRef, &attributes );
	
	if ( (! fCanAcceptDrag) || (! (attributes & dragHasLeftSenderWindow)) 
		|| (attributes & dragInsideSenderWindow) )
		return dragNotAcceptedErr;
	
	if ( this->IsMouseInContentRgn( dragRef ) )
	{
		if ( ! fIsWindowHighlighted )
		{
			tempRgn = NewRgn();
			RectRgn( tempRgn, &fWindow->portRect );
			
			if ( ShowDragHilite( dragRef, tempRgn, true ) == noErr )
				fIsWindowHighlighted = true;
				
			DisposeRgn(tempRgn);
		}
	}
		
	return noErr;
}


OSErr
TTextWindow::DragLeaveWindow( DragReference dragRef )
{
	if ( fIsWindowHighlighted )
		HideDragHilite( dragRef );
	
	fIsWindowHighlighted = false;
	fCanAcceptDrag = false;
	
	return noErr;
}


OSErr
TTextWindow::HandleDrop( DragReference dragRef )
{
	OSErr			err;
	Size			dataSize;
	ItemReference	item;
	FlavorFlags		flags;
	DragAttributes	attributes;

	GetDragAttributes( dragRef, &attributes );
	
	if ( attributes & dragInsideSenderWindow )
		return dragNotAcceptedErr;

	err = GetDragItemReferenceNumber( dragRef, 1, &item );
	if ( err == noErr )
		err = GetFlavorFlags( dragRef, item, 'TEXT', &flags );

	if ( err == noErr )
	{
		err = GetFlavorDataSize( dragRef, item, 'TEXT', &dataSize);
		if  (err == noErr )
		{
			fDraggedTextHandle = TempNewHandle( dataSize, &err );
			
			if ( fDraggedTextHandle == nil )
				fDraggedTextHandle = NewHandle( dataSize );

			if ( fDraggedTextHandle == nil )
				err = dragNotAcceptedErr;
			else
			{
				HLock( fDraggedTextHandle );
				err = GetFlavorData( dragRef, item, 'TEXT',
						*fDraggedTextHandle, &dataSize, 0L );
				HUnlock( fDraggedTextHandle );

				if ( err != noErr)
				{
					err = dragNotAcceptedErr;
					DisposeHandle( fDraggedTextHandle );
					fDraggedTextHandle = nil;
				}
				else
				{
					SetPort( fWindow );
					InvalRect( &(fWindow->portRect) );
				}
			}
		}
	}
	
	return( err );
}


void
TTextWindow::AdjustFontMenu( void )
{
	short		fontNumber, numFonts, i;
	Str255		fontName, itemName;
	
	fontNumber = fWindow->txFont;
	GetFontName( fontNumber, fontName );
	
	numFonts = CountMItems( fgFontSubMenu );
	
	for ( i=1; i<=numFonts; i++ )
	{
		GetMenuItemText( fgFontSubMenu, i, itemName );
		
		if ( EqualString( itemName, fontName, true, true ) )
			CheckItem( fgFontSubMenu, i, true);
		else
			CheckItem( fgFontSubMenu, i, false);
	}
}


void
TTextWindow::AdjustSizeMenu( void )
// Use fCurrentFontSize to place check marks, do enabling in size menu
{
	if ( fCurrentFontSize == kMinimumFontSize )
		gMenuBar->EnableCommand( cFontSizeSmaller, false );
	else
		gMenuBar->EnableCommand( cFontSizeSmaller, true );
	
	if ( fCurrentFontSize == kMaximumFontSize )
		gMenuBar->EnableCommand( cFontSizeLarger, false );
	else
		gMenuBar->EnableCommand( cFontSizeLarger, true );
	
	gMenuBar->EnableAndCheckCommand( cFontSize9, true, false );
	gMenuBar->EnableAndCheckCommand( cFontSize10, true, false );
	gMenuBar->EnableAndCheckCommand( cFontSize12, true, false );
	gMenuBar->EnableAndCheckCommand( cFontSize18, true, false );
	gMenuBar->EnableAndCheckCommand( cFontSize24, true, false );
	gMenuBar->EnableAndCheckCommand( cFontSize36, true, false );
	gMenuBar->EnableAndCheckCommand( cFontSizeOther, true, false );
	
	switch ( fCurrentFontSize )
	{
		case 9:
			gMenuBar->EnableAndCheckCommand( cFontSize9, true, true );
			break;
		case 10:
			gMenuBar->EnableAndCheckCommand( cFontSize10, true, true );
			break;
		case 12:
			gMenuBar->EnableAndCheckCommand( cFontSize12, true, true );
			break;
		case 18:
			gMenuBar->EnableAndCheckCommand( cFontSize18, true, true );
			break;
		case 24:
			gMenuBar->EnableAndCheckCommand( cFontSize24, true, true );
			break;
		case 36:
			gMenuBar->EnableAndCheckCommand( cFontSize36, true, true );
			break;
		default:
			gMenuBar->EnableAndCheckCommand( cFontSizeOther, true, true );
			break;
	}
	
	SetOtherMenuItemString();
}


void
TTextWindow::SetOtherMenuItemString( void )
{
	Str255	menuItemStr = "\pOther (";
	Str255	numberStr;
	
	NumToString( (long)fCurrentFontSize, numberStr );
	
	PascalStringCat( menuItemStr, numberStr );
	PascalStringCat( menuItemStr, "\p)..." );
	
	gMenuBar->SetItemString( cFontSizeOther, menuItemStr );
}


void
TTextWindow::PascalStringCat( Str255 dest, Str255 source )
{
	unsigned char	i, destStringLength, sourceStringLength;
	
	destStringLength = dest[0];
	sourceStringLength = source[0];
	
	if ( sourceStringLength <= 0 )
		return;
	
	for ( i=1; i<=sourceStringLength; i++ )
		dest[i+destStringLength] = source[i];
	
	dest[0] += sourceStringLength;
}


void
TTextWindow::SetNewTextSize( short newSize )
{
	GrafPtr		oldPort;
	
	GetPort( &oldPort );
	SetPort( fWindow );
	
	if ( newSize < kMinimumFontSize )
		newSize = kMinimumFontSize;
	
	if ( newSize > kMaximumFontSize )
		newSize = kMaximumFontSize;
	
	TextSize( newSize );
	fCurrentFontSize = newSize;
	
	InvalRect( &(fWindow->portRect) );
	
	SetPort( oldPort );
}


void
TTextWindow::AdjustStyleMenu( void )
// Use fCurrentFontStyle to place check marks in style menu
{
	gMenuBar->EnableAndCheckCommand( cFontStylePlain, true, false );
	gMenuBar->EnableAndCheckCommand( cFontStyleBold, true, false );
	gMenuBar->EnableAndCheckCommand( cFontStyleItalic, true, false );
	gMenuBar->EnableAndCheckCommand( cFontStyleUnderline, true, false );
	gMenuBar->EnableAndCheckCommand( cFontStyleOutline, true, false );
	gMenuBar->EnableAndCheckCommand( cFontStyleShadow, true, false );
	
	if ( fCurrentFontStyle == 0 )
		gMenuBar->EnableAndCheckCommand( cFontStylePlain, true, true );
	else
	{
		if ( fCurrentFontStyle & bold )
			gMenuBar->EnableAndCheckCommand( cFontStyleBold, true, true );
		
		if ( fCurrentFontStyle & italic )
			gMenuBar->EnableAndCheckCommand( cFontStyleItalic, true, true );
		
		if ( fCurrentFontStyle & underline )
			gMenuBar->EnableAndCheckCommand( cFontStyleUnderline, true, true );
		
		if ( fCurrentFontStyle & outline )
			gMenuBar->EnableAndCheckCommand( cFontStyleOutline, true, true );
		
		if ( fCurrentFontStyle & shadow )
			gMenuBar->EnableAndCheckCommand( cFontStyleShadow, true, true );
	}
}


void
TTextWindow::DoStyleCommand( unsigned long styleCommand )
{
	GrafPtr	oldPort;
	
	GetPort( &oldPort );
	SetPort( fWindow );
	
	switch( styleCommand )
	{
		case cFontStylePlain:
			fCurrentFontStyle = 0;
			break;
		case cFontStyleBold:
			if ( fCurrentFontStyle & bold )
				fCurrentFontStyle -= bold;
			else
				fCurrentFontStyle += bold;
			break;
		case cFontStyleItalic:
			if ( fCurrentFontStyle & italic )
				fCurrentFontStyle -= italic;
			else
				fCurrentFontStyle += italic;
			break;
		case cFontStyleUnderline:
			if ( fCurrentFontStyle & underline )
				fCurrentFontStyle -= underline;
			else
				fCurrentFontStyle += underline;
			break;
		case cFontStyleOutline:
			if ( fCurrentFontStyle & outline )
				fCurrentFontStyle -= outline;
			else
				fCurrentFontStyle += outline;
			break;
		case cFontStyleShadow:
			if ( fCurrentFontStyle & shadow )
				fCurrentFontStyle -= shadow;
			else
				fCurrentFontStyle += shadow;
			break;
	}
		
	TextFace( fCurrentFontStyle );
	InvalRect( &(fWindow->portRect) );
	
	SetPort( oldPort );
}


Boolean
TTextWindow::IsTextFlavorAvailable( DragReference dragRef )
{
	unsigned short	numItems;
	FlavorFlags		flags;
	OSErr			err;
	ItemReference	item;
	
	CountDragItems( dragRef, &numItems );
	
	if ( numItems < 1 )
		return( false );
	
	err = GetDragItemReferenceNumber( dragRef, 1, &item );
	if ( err == noErr )
		err = GetFlavorFlags( dragRef, item, 'TEXT', &flags );
	
	return( err == noErr );
}


Boolean
TTextWindow::IsMouseInContentRgn( DragReference dragRef )
{
	Point	globalMouse;
	OSErr	err;
	
	err = GetDragMouse( dragRef, &globalMouse, 0L );
	
	if ( err == noErr )
		return( PtInRgn( globalMouse, ((WindowPeek)fWindow)->contRgn ) );
	else
		return( false );
}


void
TTextWindow::SetUpStaticMenu( void )
{
	TTextWindow::fgMenu = gMenuBar->GetMenuFromCMNU( mText );
	TTextWindow::fgSizeSubMenu = gMenuBar->GetMenuFromCMNU( mTextFontSize );
	InsertMenu( TTextWindow::fgSizeSubMenu, -1 );
	TTextWindow::fgStyleSubMenu = gMenuBar->GetMenuFromCMNU( mTextFontStyle );
	InsertMenu( TTextWindow::fgStyleSubMenu, -1 );
	
	TTextWindow::fgFontSubMenu = GetMenu( mTextFont );
	InsertMenu( TTextWindow::fgFontSubMenu, -1 );
	AppendResMenu( TTextWindow::fgFontSubMenu, 'FONT' );
}


long
TTextWindow::DoNumberDialog( void )
{
	Boolean		dialogDone = false;
	Str255		text;
	Handle		itemHandle;
	short		itemHit, itemType;
	Rect		itemRect;
	long		returnValue;
	DialogPtr	dialog;
	
	dialog = GetNewDialog( kGetNumberDialogResID, NULL, (WindowPtr)-1L );
	
	if ( dialog == NULL )
		return kCancelButtonPressed;
	
	SetDialogDefaultItem( dialog, ok );
	SetDialogCancelItem( dialog, cancel );
	
	NumToString( (long)fCurrentFontSize, text );
	
	GetDialogItem( dialog, iNumberEditTextField, &itemType,
		&itemHandle, &itemRect );
	SetDialogItemText( itemHandle, text );
	
	SelectDialogItemText( dialog, iNumberEditTextField, 0, 32767 );
	
	ShowWindow( dialog );
	
	do
	{
		ModalDialog( NULL, &itemHit );
		
		if ( itemHit == ok )
		{
			GetDialogItem( dialog, iNumberEditTextField, &itemType,
				&itemHandle, &itemRect );
			GetDialogItemText( itemHandle, text );
			
			StringToNum( text, &returnValue );
			
			if ( (returnValue < kMinimumFontSize) ||
				(returnValue > kMaximumFontSize) )
			{
				SysBeep( 20 );
				SelectDialogItemText( dialog, iNumberEditTextField, 0, 32767 );
				itemHit = iNumberEditTextField;
			}
			else
				dialogDone = true;
		}
		else if ( itemHit == cancel )
		{
			returnValue = kCancelButtonPressed;
			dialogDone = true;
		}
	} while ( ! dialogDone );
	
	DisposeDialog( dialog );
	
	return returnValue;
}