#include "CPictScrollerWindow.h"
#include <LStdControl.h>
#include <LPicture.h>
#include <UReanimator.h>


CPictScrollerWindow*
CPictScrollerWindow::CreatePictScrollerWindowStream(
	LStream		*inStream)
//	This function gets called to create a new CPictScrollerWindow object.
{
	return (new CPictScrollerWindow(inStream));
}

		
CPictScrollerWindow::CPictScrollerWindow(
	LStream		*inStream)
		: LWindow(inStream)
//	Here's the constructor. Notice that we just pass the input parameter on
//	to the LWindow constructor. We have added a data member to our LWindow
//	subclass and initialize it here. mChildWindow is a pointer to a Pict
//	Selector window (aka, a child window) if one exists. If you click in
//	the Picture... button, if a child window exists, it is brought to the
//	front. If the child window does not exist, it is created.
{
	mChildWindow = nil;
}


void
CPictScrollerWindow::FinishCreateSelf()
//	FinishCreateSelf() overrides its LWindow counterpart and is called
//	automatically at object construction time. We'll add our two listeners:
//	one for the Beep button and one for the Picture... button.
{
	LStdButton	*theButton = (LStdButton*) FindPaneByID(1000);
	theButton->AddListener(this);
	
	theButton = (LStdButton*) FindPaneByID(1001);
	theButton->AddListener(this);
}


void
CPictScrollerWindow::ListenToMessage(
	MessageT	inMessage,
	void		*ioParam)
{

	switch (inMessage) {

		case 1000:
//	Beep if the Beep button is pressed.
			SysBeep(10);
			break;
			
		case 1001:
// Open the Pict Selector window if the Picture... button is pressed.
			DisplayPictSelector();
			break;
			
		case 2001:
		case 2002:
		case 2003: {
			if ((*(Int32*)ioParam) == Button_On) {
//	When ListenToMessage() gets called in response to a message from a control,
//	it puts the value of the control in the param ioParam. We only take an action
//	when a radio button is being turned on. We don't do anything in response to
//	a button being turned off.
				LPicture *thePicture = (LPicture*) mChildWindow->FindPaneByID(2010);
				
				if (inMessage != thePicture->GetPictureID()) {
					thePicture->SetPictureID(inMessage);

					Rect	pictFrame;
					thePicture->CalcLocalFrameRect(pictFrame);
					thePicture->ResizeImageTo(pictFrame.right - pictFrame.left,

						  pictFrame.bottom - pictFrame.top, false);
					thePicture->Refresh();
					
					thePicture = (LPicture*) FindPaneByID(1003);
					thePicture->SetPictureID(inMessage);
					thePicture->Refresh();
				}
			}
			break;
		}
	}
}


void
CPictScrollerWindow::DisplayPictSelector()
{
//	If the Pict Selector window doesn't exist, we'll create it.
	if (mChildWindow == nil) {
		mChildWindow = LWindow::CreateWindow(2000, this);

//	This call searches the specified RidL resource (in this case, 2000)
//	and calls AddListener for each item in the RidL. A RidL resource
//	is much like a DITL resource. It is a list of broadcasting pane IDs.
//	Each of our radio buttons is a broadcasting pane (broadcasts a
//	message). When we grouped the three radio buttons, Constructor
//	created a RidL resource listing the three radio button pane IDs.
//	Call LinkListenerToControls() if you have a RidL, or AddListener()
//	for each broadcasting control if you don't have a RidL.
		UReanimator::LinkListenerToControls(this, mChildWindow, 2000);
		
//	Gets the pane ID of the picture pane in the scroller window
		LPicture *scrollerPicture = (LPicture*) FindPaneByID(1003);
		
		LStdRadioButton *theRadioButton =
			(LStdRadioButton*)mChildWindow->FindPaneByID(
				scrollerPicture->GetPictureID() );
		
//	Our Pict scaling code is executed when a message is sent to
//	ListenToMessage(), above. This code also calculates which
//	picture should be drawn in the miniframe. To be sure that this
//	code gets executed, we first turn off the appropriate radio button
//	then turn it back on again.
		theRadioButton->SetValue( Button_Off );
		theRadioButton->SetValue( Button_On );
		
		mChildWindow->Show();
	
	} else {
		mChildWindow->Select();
	}
}

Boolean
CPictScrollerWindow::AllowSubRemoval(
	LCommander	*inSub)
//	AllowSubRemoval() is called by PowerPlant (by LWindow::AttemptClose() and
//	LWindow::DoClose()) when an LWindow is closed. This is the hook where you
//	might put up a "Save Changes" dialog and not close the window if Cancel
//	was clicked. In our case, we won't put up this kind of dialog. Instead,
//	we'll just mark the mChildWindow as nil and return true, saying it is OK
//	to delete the child window.
//	It should be noted that AllowSubRemoval() is called when any window that has a
//	super-commander is closed (In general, all windows will have a super-commander
//	-- perhaps the app object). When you close the parent window, the AllowSubRemoval()
//	of the parent window's supercommander (the LApplication object's AllowSubRemoval())
//	gets called. We'll learn about sub and super commanders in a future column.
{
	if (inSub == mChildWindow) {
		mChildWindow = nil;
	}
	
	return true;
}