MODULE DragShape;
(*
	Experiment with real-time dragging of GX shapes.

	Start of development by LDO 1995 August 6.
	Last modified 1996 March 1.
*)

	FROM SYSTEM IMPORT
		ADDRESS,
		ADR,
		CAST;
	FROM Types IMPORT
		noErr,
		Extended,
		Fixed,
		Handle,
		LongCard,
		LongReal,
		OSErr,
		Point,
		Rect,
		ResID,
		Str255,
		DebugStr;
	FROM Memory IMPORT
		HNoPurge,
		NewHandle;
	FROM Resources IMPORT
		Get1Resource,
		ReleaseResource;
	IMPORT
		QuickDraw;
	FROM QuickDraw IMPORT
		PicHandle,
		RgnHandle,
		WindowPtr,
		DisposeRgn,
		EqualPt,
		GlobalToLocal,
		InitCursor;
	IMPORT
		Events;
	FROM Events IMPORT
		EventRecord,
		GetMouse,
		WaitMouseUp;
	FROM Menus IMPORT
		GetMBarHeight;
	IMPORT
		Windows;
	FROM Drag IMPORT
		DragAttributes,
		DragReference,
		DragTrackingMessage,
		GetDragMouse,
		HideDragHilite,
		ShowDragHilite;
	FROM GXMathTypes IMPORT
		gxMapping,
		gxPoint;
	IMPORT
		GXGraphicsTypes;
	FROM GXGraphicsTypes IMPORT
		nogxShape,
		gxHitTestInfo,
		gxRectangle,
		gxShape,
		gxViewPort;
	FROM GXMathRoutines IMPORT
		MoveMapping,
		RotateMapping;
	FROM GXGraphicsRoutines IMPORT
		GXDrawShape,
		GXEnterGraphics,
		GXExitGraphics,
		GXGetShapeLocalBounds,
		GXGetShapeMapping,
		GXHitTestShape,
		GXSetShapeHitTest,
		GXSetShapeMapping,
		GXSetShapeViewPorts;
	FROM Arithmetic IMPORT
		Fixed1,
		FixedHalf,
		ArcSin,
		ArcTan2,
		Cos,
		DLimit,
		Double2Fix,
		Double2X,
		Fix2Double,
		FixedToIntCeil,
		FixedToIntFloor,
		FixedSquared,
		IntToFixed,
		Pi,
		Sin,
		SMax,
		Sqrt,
		Squared;
	IMPORT
		Useful;
	FROM Useful IMPORT
		CurrentA6,
		DisposeHandleVar,
		IgnoreOSErr;
	FROM QDUseful IMPORT
		GetScreenBitsBounds,
		NewRectRgn,
		PositionRect;
	FROM ToolboxUseful IMPORT
		ControlKeyDown,
		FinitToolbox,
		InitToolbox;
	FROM CountedStrings IMPORT
		AppendCounted,
		AppendtoCounted,
		MakeCounted;
	FROM HLUseful IMPORT
		ExtToFixedStr;
	IMPORT
		GXUseful;
	FROM GXUseful IMPORT
		ClipViewPortRect,
		DisposeShapeVar,
		GetWindowViewPort,
		InfiniteShape,
		MoveSprite,
		NewChildViewPort,
		PointToFixedPoint,
		QDPictureToShape,
		SetShapeViewPort;
	FROM GXGraphicsLibraries IMPORT
		HandleToShape;
	FROM DragUseful IMPORT
		NoInboundTransferObject,
		InboundTransferObject,
		DisposeInboundTransfer,
		GetInboundDataHandle,
		HasInboundData,
		NewSingleInboundDragTransfer;
	FROM AppUseful IMPORT
		Err;
	FROM EventHandler IMPORT
		EventLoop,
		FinitEventHandlers,
		InitEventHandlers,
		InstallQuitCheck;
	IMPORT
		WindowHandler;
	FROM WindowHandler IMPORT
		PaneMessageHandler,
		PaneMessages,
		WindowMessages,
		WindowPane,
		FinitWindowHandling,
		GetPaneBounds,
		InitWindowHandling,
		InvalPane,
		NewWindowPane,
		OpenWindow,
		SetPaneMessageHandler,
		SetWindowMessageHandler;

(*
	Useful stuff
*)

	PROCEDURE DebugReal
	  (
		Msg : ARRAY OF CHAR;
		Value : LongReal
	  );
	  (* displays a specified real value preceded by a message. *)

		VAR
			TempExt : Extended;
			MsgStr, TempStr : Str255;

	BEGIN
		MakeCounted(Msg, MsgStr);
		AppendtoCounted(' ', MsgStr);
		Double2X(Value, TempExt);
		ExtToFixedStr(TempExt, TempStr, 5);
		AppendCounted(ADR(TempStr), MsgStr);
		AppendtoCounted('; g', MsgStr);
		DebugStr(ADR(MsgStr))
	END DebugReal;

	PROCEDURE GetShapeBoundsCenter
	  (
		TheShape : gxShape;
		VAR Center : gxPoint
	  );

		VAR
			Bounds : gxRectangle;

	BEGIN
		GXGetShapeLocalBounds(TheShape, Bounds);
		Center.x := (Bounds.right + Bounds.left) DIV 2;
		Center.y := (Bounds.bottom + Bounds.top) DIV 2
	END GetShapeBoundsCenter;

	PROCEDURE SetStandardHitTest
	  (
		TheShape : gxShape
	  );

	BEGIN
		GXSetShapeHitTest
		  (
			(*target :=*) TheShape,
			(*mask :=*)
					GXGraphicsTypes.gxAnyPart
				-
					GXGraphicsTypes.gxShapePart{GXGraphicsTypes.gxBoundsPart},
			(*tolerance :=*) Fixed1
		  )
	END SetStandardHitTest;

	PROCEDURE HitShape
	  (
		TheShape : gxShape;
		ThePoint : Point
	  ) : BOOLEAN;
	  (* simple shape hit-test routine. *)

		VAR
			TheFixedPoint : gxPoint;
			Info : gxHitTestInfo;

	BEGIN
		PointToFixedPoint(ThePoint, TheFixedPoint);
		RETURN
				GXHitTestShape
				  (
					(*target :=*) TheShape,
					(*@test :=*) TheFixedPoint,
					(*@result :=*) Info
				  )
			<>
				GXGraphicsTypes.gxNoPart
	END HitShape;

(*
	Global data
*)

	VAR
		MainWindow : WindowPtr;
		MainPane : WindowPane;
		MainViewPort : gxViewPort;
		DragTransfer : InboundTransferObject;
		ReceiveDrag : BOOLEAN;
		Background, TheShape : gxShape;

(*
	Window handling
*)

	PROCEDURE MainWindowClosed
	  (
		Unused : ADDRESS
	  ) : BOOLEAN;
	  (* installed as a quit-check to make sure event loop doesn't
		terminate until my main window has closed. *)

	BEGIN
		RETURN
			MainWindow = NIL
	END MainWindowClosed;

	PROCEDURE HandleWindowCloseMessage
	  (
		TheWindow : WindowPtr;
		TheMessage : WindowMessages;
		Unused : LongCard
	  );
	  (* called when user closes window. *)

	BEGIN
		MainWindow := NIL (* allows event loop to terminate *)
	END HandleWindowCloseMessage;

	PROCEDURE HandlePaneUpdateMessage
	  (
		TheWindow : WindowPtr;
		ThePane : WindowPane;
		TheMessage : PaneMessages;
		Unused1, Unused2, Unused3 : LongCard
	  );

	BEGIN
		GXDrawShape(Background);
		IF TheShape <> nogxShape THEN
			GXDrawShape(TheShape)
		END (*IF*)
	END HandlePaneUpdateMessage;

	PROCEDURE HandlePaneEventMessage
	  (
		TheWindow : WindowPtr;
		ThePane : WindowPane;
		TheMessage : PaneMessages;
		VAR TheEvent : EventRecord;
		VAR Handled : BOOLEAN;
		Unused : LongCard
	  );
	  (* handles an event to do with the pattern display pane. *)

		PROCEDURE HandlePaneClick;
		  (* handles a mouse click on the main display pane.
			Lets the user drag the foreground shape about. *)

			VAR
				SaveBackgroundViewPorts : GXUseful.ViewPortList;
				DragLimits : Rect;
				DoRotate : BOOLEAN;
				LastMouse, ThisMouse : Point;
				ShapeCenter, ThisDelta : gxPoint;
				ShapeMapping : gxMapping;

			PROCEDURE GetDragLimits;
			  (* works out the limits within which the mouse can
				be moved, based on its initial position (in LastMouse),
				so that TheShape cannot be dragged outside the visible
				part of the window. *)

				VAR
					ShapeBounds : gxRectangle;

			BEGIN
				GetPaneBounds(ThePane, DragLimits);
				GXGetShapeLocalBounds(TheShape, ShapeBounds);
				DragLimits.left :=
						DragLimits.left
					+
						LastMouse.h
					-
						FixedToIntFloor(ShapeBounds.left);
				DragLimits.top :=
						DragLimits.top
					+
						LastMouse.v
					-
						FixedToIntFloor(ShapeBounds.top);
				DragLimits.right :=
						DragLimits.right
					+
						LastMouse.h
					-
						FixedToIntCeil(ShapeBounds.right);
				DragLimits.bottom :=
						DragLimits.bottom
					+
						LastMouse.v
					-
						FixedToIntCeil(ShapeBounds.bottom);
				IF Events.cmdKey IN TheEvent.modifierSet THEN
					DoRotate := FALSE
				ELSE
					DoRotate :=
								FixedSquared
								  (
										IntToFixed(LastMouse.h)
									-
											(ShapeBounds.right + ShapeBounds.left)
										DIV
											2
								  )
							+
								FixedSquared
								  (
										IntToFixed(LastMouse.v)
									-
											(ShapeBounds.bottom + ShapeBounds.top)
										DIV
											2
								  )
						>
								FixedSquared
								  (
										SMax
										  (
											(ShapeBounds.right - ShapeBounds.left),
											(ShapeBounds.bottom - ShapeBounds.top)
										  )
									DIV
										2
								  )
							DIV
								16
				END (*IF*)
			END GetDragLimits;

			PROCEDURE CalculateRotationAngle() : Fixed;
			  (* calculates the angle through which the shape should be
				rotated as a result of the drag. *)

				VAR
					DeltaX, DeltaY, Delta : LongReal;
					RadiusX, RadiusY, Radius : LongReal;
					DragAngle, RotationAngleSin : LongReal;

			BEGIN
				DeltaX := Fix2Double(ThisDelta.x);
				DeltaY := Fix2Double(ThisDelta.y);
				Delta := Sqrt(Squared(DeltaX) + Squared(DeltaY));
				RadiusX := Fix2Double(IntToFixed(LastMouse.h) - ShapeCenter.x);
				RadiusY := Fix2Double(IntToFixed(LastMouse.v) - ShapeCenter.y);
				Radius := Sqrt(Squared(RadiusX) + Squared(RadiusY));
				DragAngle := ArcTan2(DeltaX, DeltaY) - ArcTan2(RadiusX, RadiusY);
				RotationAngleSin := DLimit
				   (
				 	Delta * Sin(DragAngle) / Radius,
					FLOATD(-1),
					FLOATD(1)
				  );
				IF ControlKeyDown() THEN
					DebugReal('DeltaX:', DeltaX);
					DebugReal('DeltaY:', DeltaY);
					DebugReal('Delta angle:', ArcTan2(DeltaX, DeltaY));
					DebugReal('RadiusX:', RadiusX);
					DebugReal('RadiusY:', RadiusY);
					DebugReal('Radius angle:', ArcTan2(RadiusX, RadiusY));
					DebugReal('DragAngle:', DragAngle);
					DebugReal('RotationAngle:', ArcSin(RotationAngleSin));
				END (*IF*);
				RETURN
					Double2Fix(ArcSin(RotationAngleSin) * FLOATD(180) / Pi())
			END CalculateRotationAngle;

			PROCEDURE DrawBackground
			  (
				NrViewPorts : LongCard;
				ViewPorts : (*gxViewPortPtr*) ADDRESS
			  );
			  (* action routine to draw the background into the specified
				viewports. *)

			BEGIN
				GXSetShapeViewPorts(Background, NrViewPorts, ViewPorts);
				GXDrawShape(Background)
			END DrawBackground;

		BEGIN (*HandlePaneClick*)
			LastMouse := TheEvent.where;
			GlobalToLocal(LastMouse);
			IF HitShape(TheShape, LastMouse) THEN
				GXUseful.InitViewPortList(SaveBackgroundViewPorts);
				GXUseful.GetShapeViewPorts(Background, SaveBackgroundViewPorts);
				GXGetShapeMapping(TheShape, ShapeMapping);
				REPEAT
					GetDragLimits;
					GetMouse(ThisMouse);
					IF ThisMouse.h < DragLimits.left THEN
						ThisMouse.h := DragLimits.left
					ELSIF ThisMouse.h > DragLimits.right THEN
						ThisMouse.h := DragLimits.right
					END (*IF*);
					IF ThisMouse.v < DragLimits.top THEN
						ThisMouse.v := DragLimits.top
					ELSIF ThisMouse.v > DragLimits.bottom THEN
						ThisMouse.v := DragLimits.bottom
					END (*IF*);
					IF NOT EqualPt(ThisMouse, LastMouse) THEN
						ThisDelta.x := IntToFixed(ThisMouse.h - LastMouse.h);
						ThisDelta.y := IntToFixed(ThisMouse.v - LastMouse.v);
						MoveMapping
						  (
							(*@target :=*) ShapeMapping,
							(*hOffset :=*) ThisDelta.x,
							(*vOffset :=*) ThisDelta.y
						  );
						IF DoRotate THEN
							GetShapeBoundsCenter(TheShape, ShapeCenter);
							RotateMapping
							  (
								(*@target :=*) ShapeMapping,
								(*angle :=*) CalculateRotationAngle(),
								(*xCenter :=*) IntToFixed(ThisMouse.h),
								(*yCenter :=*) IntToFixed(ThisMouse.v)
							  )
						END (*IF*);
						MoveSprite
						  (
							(*SpriteShape :=*) TheShape,
							(*@NewSpriteMapping :=*) ShapeMapping,
							(*DrawBackground :=*) CAST
							  (
								GXUseful.BackgroundDrawAction,
								ADR(DrawBackground)
							  ),
							(*DrawBackgroundArg :=*) CurrentA6()
						  );
						LastMouse := ThisMouse
					END (*IF*)
				UNTIL
					NOT WaitMouseUp();
				GXUseful.SetShapeViewPorts(Background, SaveBackgroundViewPorts);
				GXUseful.DisposeViewPortList(SaveBackgroundViewPorts)
			END (*IF*)
		END HandlePaneClick;

	BEGIN (*HandlePaneEventMessage*)
		CASE TheEvent.what OF
		| Events.mouseDown:
			HandlePaneClick;
			Handled := TRUE
		ELSE
		  (* ignore *)
		END (*CASE*)
	END HandlePaneEventMessage;

	PROCEDURE HasAcceptableType
	  (
		Transfer : InboundTransferObject
	  ) : BOOLEAN;
	  (* has the specified inbound transfer got data that I can use. *)

	BEGIN
		RETURN
				HasInboundData(Transfer, Useful.GXGraphicsType)
			OR
				HasInboundData(Transfer, Useful.PictureType)
	END HasAcceptableType;

	PROCEDURE HandlePaneDragTrack
	  (
		TheWindow : WindowPtr;
		ThePane : WindowPane;
		TheMessage : PaneMessages;
		TheDrag : DragReference;
		Attributes : DragAttributes;
		Message : (*DragTrackingMessage*) LONGCARD
	  );
	  (* handles tracking of a drag into the pattern display pane.
		Sets up global context for accepting the drag, if appropriate. *)

		VAR
			HiliteRect : Rect;
			HiliteRgn : RgnHandle;

	BEGIN
		CASE VAL(DragTrackingMessage, Message) OF
		| WindowHandler.DragTrackingEnterPane:
			DragTransfer := NoInboundTransferObject;
			ReceiveDrag := FALSE;
			LOOP (*once*)
				NewSingleInboundDragTransfer(TheDrag, DragTransfer, Err);
				IF Err <> noErr THEN
					EXIT
				END (*IF*);
				IF DragTransfer = NoInboundTransferObject THEN
					EXIT
				END (*IF*);
				IF
						NOT (WindowHandler.DragInsideSenderPane IN Attributes)
					AND
						HasAcceptableType(DragTransfer)
				THEN
					GetPaneBounds(ThePane, HiliteRect);
					HiliteRgn := NewRectRgn(HiliteRect);
					IgnoreOSErr(ShowDragHilite(TheDrag, HiliteRgn, TRUE));
					DisposeRgn(HiliteRgn);
					ReceiveDrag := TRUE
				END (*IF*);
			  (* all done *)
				EXIT
			END (*LOOP*)
		| WindowHandler.DragTrackingLeavePane:
			IF ReceiveDrag THEN
				IgnoreOSErr(HideDragHilite(TheDrag))
			END (*IF*);
			DisposeInboundTransfer(DragTransfer)
		ELSE
		  (* ignore *)
		END (*CASE*)
	END HandlePaneDragTrack;

	PROCEDURE HandlePaneDragReceive
	  (
		TheWindow : WindowPtr;
		ThePane : WindowPane;
		TheMessage : PaneMessages;
		TheDrag : DragReference;
		Attributes : DragAttributes;
		VAR Err : OSErr
	  );
	  (* receives a drag into the pattern display pane. *)

		VAR
			TheData : Handle;
			NewShape : gxShape;

		PROCEDURE SetupTheShape;
		  (* positions the shape where the user released it, and sets up
			its viewport for display. *)

			VAR
				MouseWhere : Point;
				ShapeCenter, NewShapeCenter : gxPoint;
				ShapeMapping : gxMapping;

		BEGIN
			IF InfiniteShape(NewShape) THEN
			  (* use as background *)
				DisposeShapeVar(Background);
				Background := NewShape;
				SetShapeViewPort(Background, MainViewPort)
			ELSE
			  (* use as figure *)
				DisposeShapeVar(TheShape);
				TheShape := NewShape;
				IgnoreOSErr(GetDragMouse(TheDrag, NIL, ADR(MouseWhere)));
				GlobalToLocal(MouseWhere);
				GetShapeBoundsCenter(TheShape, ShapeCenter);
				GXGetShapeMapping(TheShape, ShapeMapping);
				PointToFixedPoint(MouseWhere, NewShapeCenter);
				MoveMapping
				  (
					(*@target :=*) ShapeMapping,
					(*hOffset :=*) NewShapeCenter.x - ShapeCenter.x,
					(*vOffset :=*) NewShapeCenter.y - ShapeCenter.y
				  );
				GXSetShapeMapping(TheShape, ADR(ShapeMapping));
				SetShapeViewPort(TheShape, MainViewPort);
				SetStandardHitTest(TheShape)
			END (*IF*);
			InvalPane(ThePane)
		END SetupTheShape;

	BEGIN (*HandlePaneDragReceive*)
		IF ReceiveDrag THEN
			TheData := NewHandle(0);
			LOOP (*once*)
				GetInboundDataHandle
				  (
					(*TheTransfer :=*) DragTransfer,
					(*DataType :=*) Useful.GXGraphicsType,
					(*Dest :=*) TheData,
					(*@Err :=*) Err
				  );
				IF Err = noErr THEN
					NewShape := HandleToShape(TheData, 0, NIL);
					SetupTheShape;
					EXIT
				END (*IF*);
				GetInboundDataHandle
				  (
					(*TheTransfer :=*) DragTransfer,
					(*DataType :=*) Useful.PictureType,
					(*Dest :=*) TheData,
					(*@Err :=*) Err
				  );
				IF Err = noErr THEN
					NewShape := QDPictureToShape(CAST(PicHandle, TheData), TRUE);
					SetupTheShape;
					EXIT
				END (*IF*);
			  (* all done *)
				EXIT
			END (*LOOP*);
			DisposeHandleVar(TheData)
		END (*IF*)
	END HandlePaneDragReceive;

	PROCEDURE OpenMainWindow;

		CONST
			ScreenMargin = 4;
		VAR
			MainScreenBounds, WindowBounds, MainPaneBounds : Rect;
			WindowTitle : Str255;

		PROCEDURE LoadShape
		  (
			ShapeResID : ResID;
			VAR ResultShape : gxShape
		  );

			VAR
				FlattenedShape : ADDRESS;

		BEGIN
			FlattenedShape := Get1Resource(Useful.GXGraphicsType, ShapeResID);
			IF FlattenedShape <> NIL THEN
				HNoPurge(FlattenedShape);
				ResultShape := HandleToShape(FlattenedShape, 1, ADR(MainViewPort));
				ReleaseResource(FlattenedShape)
			END (*IF*)
		END LoadShape;

	BEGIN (*OpenMainWindow*)
		GetScreenBitsBounds(MainScreenBounds);
		MainScreenBounds.top := MainScreenBounds.top + GetMBarHeight();
		WindowBounds.top := 0;
		WindowBounds.left := 0;
		WindowBounds.bottom := 342;
		WindowBounds.right := 512;
		MainPaneBounds := WindowBounds;
		PositionRect(WindowBounds, MainScreenBounds, FixedHalf, FixedHalf);
		MakeCounted("Try Me", WindowTitle);
		OpenWindow
		  (
			(*@BoundsRect :=*) WindowBounds,
			(*Title :=*) ADR(WindowTitle),
			(*Visible :=*) TRUE,
			(*HasGoAway :=*) TRUE,
			(*HasGrow :=*) FALSE,
			(*IsModalDialog :=*) FALSE,
			(*IsPaletteWindow :=*) FALSE,
			(*Color :=*) TRUE,
			(*ProcID :=*) Windows.rDocProc,
			(*Behind :=*) Windows.WindowInFront,
			(*WindowType :=*) 0,
			(*RefCon :=*) NIL,
			(*@Result :=*) MainWindow,
			(*@Err :=*) Err
		  );
		SetWindowMessageHandler
		  (
			(*TheWindow :=*) MainWindow,
			(*TheMessage :=*) WindowHandler.WindowCloseMessage,
			(*TheHandler :=*) HandleWindowCloseMessage
		  );
		NewWindowPane
		  (
			(*InWindow :=*) MainWindow,
			(*RefCon :=*) NIL,
			(*@Bounds :=*) MainPaneBounds,
			(*FixedWidth :=*) FALSE,
			(*FixedHeight :=*) FALSE,
			(*TracksRightEdge :=*) TRUE,
			(*TracksBottomEdge :=*) TRUE,
			(*TakesFocus :=*) FALSE,
			(*ClickThrough :=*) FALSE,
			(*Visible :=*) TRUE,
			(*Enabled :=*) TRUE,
			(*@ThePane :=*) MainPane,
			(*@Err :=*) Err
		  );
		MainViewPort := NewChildViewPort(GetWindowViewPort(MainWindow));
		ClipViewPortRect(MainViewPort, MainPaneBounds);
		LoadShape(128, Background);
		LoadShape(129, TheShape);
		IF TheShape <> nogxShape THEN
			SetStandardHitTest(TheShape)
		END (*IF*);
		SetPaneMessageHandler
		  (
			(*ThePane :=*) MainPane,
			(*TheMessage :=*) WindowHandler.PaneUpdateMessage,
			(*TheHandler :=*) HandlePaneUpdateMessage
		  );
		SetPaneMessageHandler
		  (
			(*ThePane :=*) MainPane,
			(*TheMessage :=*) WindowHandler.PaneEventMessage,
			(*TheHandler :=*) CAST(PaneMessageHandler, HandlePaneEventMessage)
		  );
		SetPaneMessageHandler
		  (
			(*ThePane :=*) MainPane,
			(*TheMessage :=*) WindowHandler.PaneDragTrackMessage,
			(*TheHandler :=*) CAST(PaneMessageHandler, HandlePaneDragTrack)
		  );
		SetPaneMessageHandler
		  (
			(*ThePane :=*) MainPane,
			(*TheMessage :=*) WindowHandler.PaneDragReceiveMessage,
			(*TheHandler :=*) CAST(PaneMessageHandler, HandlePaneDragReceive)
		  );
		InstallQuitCheck(MainWindowClosed, NIL, Err);
	END OpenMainWindow;

(*
	Mainline
*)

	PROCEDURE Initialize;

	BEGIN
		MainWindow := NIL;
		Background := nogxShape;
		TheShape := nogxShape;
		InitToolbox;
		InitCursor;
		GXEnterGraphics;
		InitEventHandlers(TRUE, Err);
		InitWindowHandling(TRUE, Err);
		OpenMainWindow
	END Initialize;

	PROCEDURE Finalize;

	BEGIN
		FinitWindowHandling;
		FinitEventHandlers;
		GXExitGraphics;
		FinitToolbox
	END Finalize;

BEGIN (*DragShape*)
	Initialize;
	EventLoop;
	Finalize
END DragShape.
