/*
	File:		som_DigitalMeter.cpp

	Contains:	Sample Part's SOM based part public interface implementation

	Written by:	Steve Smith
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
	
	----------------------------------------------------------------
	
	Notes:		DigitalMeter is implemented using a "wrapped" class strategy.
				This means that the SOM class is just a smart delegator and
				the real implementation lives in a C++ class. The reason for
				doing this is twofold. First, this strategy all but elimintates
				the need for you to have to modify the IDL file and run the
				SOM compiler/emitter. Secondly, given that all code changes
				will occur in the C++ class, build turn around time is reduced
				to however fast the compiler can crunch code.
				
				This is not to say that implementing pure SOM classes isn't
				possible, nor necessary. There will be objects (ie. Extensions)
				which need to be exported for other parts to use which will need
				to be written in IDL and SOM implementation files.
*/

// Notification that this is a SOM source file
#define PartCo_som_DigitalMeter_Class_Source

// define underscore field names (ie. _fSelf)
#define VARIABLE_MACROS

// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (ie. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with the "ev" parameter set.
#include <Except.h>
#endif

// -- DigitalMeter Includes --

#ifndef SOM_PartCo_som_DigitalMeter_xih
#include "som_DigitalMeter.xih"
#endif

#ifndef _DIGITALMETER_
#include "DigitalMeter.h"
#endif

#ifndef _DIGITALMETERDEF_
#include "DigitalMeterDef.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

//+
#ifndef SOM_EduObjectListener_xh
#include "EduObjectListener.xh"
#endif

#ifndef SOM_ODDragItemIterator_xh
#include <DgItmIt.xh>
#endif

#pragma segment somDigitalMeter

//====================================================================
// som_DigitalMeter
//====================================================================

SOM_Scope void  SOMLINK som_DigitalMeter__somInit(PartCo_som_DigitalMeter *somSelf)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__somInit");

	// somInit and somUninit methods behave like C++ constructors in that the
	// inherited methods are called automatically. Thus, there is no need to
	// call the parent class' somInit or somUninit.
	
	// There is also no need to set instance variables to zero/NULL
	// since SOM guarantees that a newly constructed object is zeroed.
	
	// Lastly, it is not valid to perform any operation here which may throw
	// an exception.
}

SOM_Scope void  SOMLINK som_DigitalMeter__somUninit(PartCo_som_DigitalMeter *somSelf)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__somUninit");

	delete _fPart;
}

SOM_Scope void  SOMLINK som_DigitalMeter__InitPart(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__InitPart");

	SOM_TRY
		// Create the real C++ DigitalMeter.		
		_fPart = new DigitalMeter;

		PartCo_som_DigitalMeter_parent_ODPart_InitPart(somSelf,ev,storageUnit,partWrapper);
		_fPart->InitPart(ev,storageUnit,partWrapper);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__InitPartFromStorage(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__InitPartFromStorage");

	SOM_TRY
		// Create the real C++ DigitalMeter.		
		_fPart = new DigitalMeter;

		PartCo_som_DigitalMeter_parent_ODPart_InitPartFromStorage(somSelf,ev,storageUnit,partWrapper);
		_fPart->InitPartFromStorage(ev,storageUnit,partWrapper);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__Externalize(PartCo_som_DigitalMeter *somSelf, Environment *ev)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__Externalize");

	SOM_TRY
		PartCo_som_DigitalMeter_parent_ODPart_Externalize(somSelf,ev);
		_fPart->Externalize(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODSize  SOMLINK som_DigitalMeter__Purge(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODSize size)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__Purge");

	ODSize result;

	SOM_TRY
		result = _fPart->Purge(ev,size);
	SOM_CATCH_ALL
		result = 0;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_DigitalMeter__Release(PartCo_som_DigitalMeter *somSelf, Environment *ev)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__Release");

	SOM_TRY
		PartCo_som_DigitalMeter_parent_ODPart_Release(somSelf,ev);
		_fPart->Release(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__ReleaseAll(PartCo_som_DigitalMeter *somSelf, Environment *ev)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__ReleaseAll");

	SOM_TRY
		_fPart->ReleaseAll(ev);
		PartCo_som_DigitalMeter_parent_ODPart_ReleaseAll(somSelf,ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODExtension*  SOMLINK som_DigitalMeter__AcquireExtension(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODType extensionName)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__GetExtension");

	ODExtension* result;

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
		result = PartCo_som_DigitalMeter_parent_ODPart_AcquireExtension(somSelf,ev,extensionName);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODBoolean  SOMLINK som_DigitalMeter__HasExtension(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODType extensionName)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__HasExtension");

	ODBoolean result;

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
		result = PartCo_som_DigitalMeter_parent_ODPart_HasExtension(somSelf,ev,extensionName);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_DigitalMeter__ReleaseExtension(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODExtension* extension)
{
    PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__ReleaseExtension");

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
	    PartCo_som_DigitalMeter_parent_ODPart_ReleaseExtension(somSelf,ev,extension);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__FulfillPromise(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODStorageUnitView* /*promiseSUView*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__FulfillPromise");
}

SOM_Scope void  SOMLINK som_DigitalMeter__DropCompleted(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODPart* ,
		ODDropResult dropResult)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__DropCompleted");
}

SOM_Scope ODDragResult  SOMLINK som_DigitalMeter__DragEnter(
		PartCo_som_DigitalMeter *somSelf, 
		Environment *ev,
		ODDragItemIterator* dragInfo,
		ODFacet* facet,
		ODPoint* where)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	
	ODBoolean	result = kODFalse;
	SOM_TRY
		result =  _fPart->IsAcceptable(ev,dragInfo);
	SOM_CATCH_ALL
	SOM_ENDTRY
	return result;
}

SOM_Scope ODDragResult  SOMLINK som_DigitalMeter__DragWithin(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODDragItemIterator* dragInfo,
		ODFacet* facet,
		ODPoint* where)
{
	return somSelf->DragEnter(ev,dragInfo,facet,where);
}

SOM_Scope void  SOMLINK som_DigitalMeter__DragLeave(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFacet* /*facet*/,
		ODPoint* /*where*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__DragLeave");
}

SOM_Scope ODDropResult  SOMLINK som_DigitalMeter__Drop(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODDragItemIterator* dropInfo,
		ODFacet* facet,
		ODPoint* where)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	
	SOM_TRY
		_fPart->ReceiveDrop(ev,dropInfo);
	SOM_CATCH_ALL
	SOM_ENDTRY
		return kODDropCopy;
}

SOM_Scope ODBoolean  SOMLINK som_DigitalMeter__RevealFrame(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/,
		ODShape* /*revealShape*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__RevealFrame");

	return kODFalse;
}

SOM_Scope ODBoolean  SOMLINK som_DigitalMeter__EditInLinkAttempted(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
    PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__EditInLinkAttempted");

    return kODFalse;
}

SOM_Scope void  SOMLINK som_DigitalMeter__EmbeddedFrameUpdated(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame,
		ODUpdateID change)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__EmbeddedFrameChanged");
}

SOM_Scope void  SOMLINK som_DigitalMeter__EmbeddedFrameSpec(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/,
		ODObjectSpec* /*spec*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__EmbeddedFrameSpec");
}

SOM_Scope void  SOMLINK som_DigitalMeter__ContainingPartPropertiesUpdated(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnit* propertyUnit)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__ContainingPartPropertiesChanged");
}

SOM_Scope ODEmbeddedFramesIterator*  SOMLINK som_DigitalMeter__CreateEmbeddedFramesIterator(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__CreateEmbeddedFramesIterator");

	// Scripting know if you part supports embedding by calling this method.
	// Since we dont' support embedding, we let ODPart handle it. This is the
	// only "unimplemented" method that we really have to handle.
	PartCo_som_DigitalMeter_parent_ODPart_CreateEmbeddedFramesIterator(somSelf,ev,frame);
	return kODNULL;
}

SOM_Scope void  SOMLINK som_DigitalMeter__DisplayFrameAdded(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__DisplayFrameAdded");

	SOM_TRY
		_fPart->DisplayFrameAdded(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__DisplayFrameRemoved(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__DisplayFrameRemoved");

	SOM_TRY
		_fPart->DisplayFrameRemoved(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__DisplayFrameConnected(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__DisplayFrameConnected");

	SOM_TRY
		_fPart->DisplayFrameConnected(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__DisplayFrameClosed(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__DisplayFrameClosed");

	SOM_TRY
		_fPart->DisplayFrameClosed(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__AttachSourceFrame(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame,
		ODFrame* sourceFrame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__AttachSourceFrame");

	SOM_TRY
		_fPart->AttachSourceFrame(ev,frame,sourceFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__FrameShapeChanged(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__FrameShapeChanged");

	SOM_TRY
		_fPart->FrameShapeChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__ViewTypeChanged(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__ViewTypeChanged");

	SOM_TRY
		_fPart->ViewTypeChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__PresentationChanged(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__PresentationChanged");
}

SOM_Scope void  SOMLINK som_DigitalMeter__SequenceChanged(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__SequenceChanged");
}

SOM_Scope void  SOMLINK som_DigitalMeter__ClonePartInfo(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODDraftKey key,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView,
		ODFrame* scopeFrame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__ClonePartInfo");

	SOM_TRY
		_fPart->ClonePartInfo(ev,key,partInfo,storageUnitView,scopeFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__WritePartInfo(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__WritePartInfo");

	SOM_TRY
		_fPart->WritePartInfo(ev,partInfo,storageUnitView);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODInfoType  SOMLINK som_DigitalMeter__ReadPartInfo(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnitView* storageUnitView)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__ReadPartInfo");

	ODInfoType result;

	SOM_TRY
		result = _fPart->ReadPartInfo(ev,frame,storageUnitView);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODID  SOMLINK som_DigitalMeter__Open(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__Open");

	ODID result;

	SOM_TRY
		result = _fPart->Open(ev,frame);
	SOM_CATCH_ALL
		result = kODNULLID;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODFrame*  SOMLINK som_DigitalMeter__RequestEmbeddedFrame(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* containingFrame,
		ODFrame* baseFrame,
		ODShape* frameShape,
		ODPart* embeddedPart,
		ODTypeToken viewType,
		ODTypeToken presentation,
		ODBoolean isOverlaid)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__RequestEmbeddedFrame");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_DigitalMeter__RemoveEmbeddedFrame(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__RemoveEmbeddedFrame");
}

SOM_Scope ODShape*  SOMLINK som_DigitalMeter__RequestFrameShape(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/,
		ODShape* /*frameShape*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__RequestFrameShape");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_DigitalMeter__UsedShapeChanged(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__UsedShapeChanged");
}

SOM_Scope ODStorageUnit*  SOMLINK som_DigitalMeter__AcquireContainingPartProperties(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__GetContainingPartProperties");

	return kODNULL;
}

SOM_Scope ODShape*  SOMLINK som_DigitalMeter__AdjustBorderShape(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFacet* /*embeddedFacet*/,
		ODShape* /*shape*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__AdjustBorderShape");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_DigitalMeter__FacetAdded(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFacet* facet)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__FacetAdded");

	SOM_TRY
		_fPart->FacetAdded(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__FacetRemoved(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFacet* facet)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__FacetRemoved");

	SOM_TRY
		_fPart->FacetRemoved(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__CanvasChanged(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFacet* /*facet*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__CanvasChanged");
}

SOM_Scope void  SOMLINK som_DigitalMeter__GeometryChanged(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFacet* facet,
		ODBoolean clipShapeChanged,
		ODBoolean externalTransformChanged)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__GeometryChanged");

	SOM_TRY
		_fPart->GeometryChanged(ev,facet,clipShapeChanged,externalTransformChanged);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__Draw(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFacet* facet,
		ODShape* invalidShape)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__Draw");

	SOM_TRY
		_fPart->Draw(ev,facet,invalidShape);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__CanvasUpdated(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODCanvas* /*canvas*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__CanvasUpdated");
}

SOM_Scope void  SOMLINK som_DigitalMeter__HighlightChanged(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFacet* facet)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__HighlightChanged");

	SOM_TRY
		_fPart->HighlightChanged(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODULong  SOMLINK som_DigitalMeter__GetPrintResolution(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__GetPrintResolution");

	return kMinImagingResolution;
}

SOM_Scope ODLinkSource*  SOMLINK som_DigitalMeter__CreateLink(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODByteArray* /*data*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__CreateLink");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_DigitalMeter__LinkUpdated(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODLink* link,
		ODUpdateID change)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__LinkUpdated");
}

SOM_Scope void  SOMLINK som_DigitalMeter__RevealLink(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODLinkSource* /*linkSource*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__RevealLink");
}

SOM_Scope void  SOMLINK som_DigitalMeter__LinkStatusChanged(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__LinkStatusChanged");
}

SOM_Scope ODBoolean  SOMLINK som_DigitalMeter__BeginRelinquishFocus(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__BeginRelinquishFocus");

	ODBoolean result;

	SOM_TRY
		result = _fPart->BeginRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_DigitalMeter__CommitRelinquishFocus(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__CommitRelinquishFocus");

	SOM_TRY
		_fPart->CommitRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__AbortRelinquishFocus(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__AbortRelinquishFocus");

	SOM_TRY
		_fPart->AbortRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__FocusAcquired(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__FocusAcquired");

	SOM_TRY
		_fPart->FocusAcquired(ev,focus,ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__FocusLost(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__FocusLost");

	SOM_TRY
		_fPart->FocusLost(ev,focus,ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__CloneInto(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODDraftKey key,
		ODStorageUnit* toSU,
		ODFrame* scope)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__CloneInto");

	SOM_TRY
		PartCo_som_DigitalMeter_parent_ODPart_CloneInto(somSelf,ev,key,toSU,scope);
		_fPart->CloneInto(ev,key,toSU,scope);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__ExternalizeKinds(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODTypeList* kindset)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__ExternalizeKinds");

	SOM_TRY
		_fPart->ExternalizeKinds(ev,kindset);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__ChangeKind(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODType kind)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__ChangeKind");

	SOM_TRY
		_fPart->ChangeKind(ev,kind);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODBoolean  SOMLINK som_DigitalMeter__HandleEvent(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODEventData* event,
		ODFrame* frame,
		ODFacet* facet,
		ODEventInfo* eventInfo)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__HandleEvent");

	ODBoolean result;

	SOM_TRY
		result = _fPart->HandleEvent(ev,event,frame,facet,eventInfo);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_DigitalMeter__AdjustMenus(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODFrame* frame)
{
	PartCo_som_DigitalMeterData *somThis = PartCo_som_DigitalMeterGetData(somSelf);
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__AdjustMenus");

	SOM_TRY
		_fPart->AdjustMenus(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DigitalMeter__UndoAction(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODActionData* /*actionState*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__UndoAction");
}

SOM_Scope void  SOMLINK som_DigitalMeter__RedoAction(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODActionData* /*actionState*/)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__RedoAction");
}

SOM_Scope void  SOMLINK som_DigitalMeter__DisposeActionState(PartCo_som_DigitalMeter *somSelf, Environment *ev,
		ODActionData* ,
		ODDoneState doneState)
{
	PartCo_som_DigitalMeterMethodDebug("PartCo_som_DigitalMeter","som_DigitalMeter__DisposeActionState");
}
