
#pragma once

#if ! DEBUG
// debug traps should not compile into a non-DEBUG version
	#undef USE_DEBUG_TRAPS
	#define USE_DEBUG_TRAPS	0
#endif


#if USE_DEBUG_TRAPS	// [

#ifdef __cplusplus
extern "C" {
#endif


/*________________________________________________________________________________
	Note:
	The __FILE__ directive can be used to record the file name in which  a memory
	allocation is made. This information can be used by the DebugLeaks code.
	__FILE__ generates a null-terminated C-string.
________________________________________________________________________________*/


// _____________________________ Memory Manager _________________________________

#define BlockMove				DebugTraps_BlockMove
#define BlockMoveData			DebugTraps_BlockMoveData
#define NewPtr(sz)				DebugTraps_NewPtr( sz, __FILE__)
#define NewPtrClear(sz)			DebugTraps_NewPtrClear( sz, __FILE__)
#define NewPtrSys(sz)			DebugTraps_NewPtrSys( sz, __FILE__)
#define NewPtrSysClear(sz)		DebugTraps_NewPtrSysClear( sz, __FILE__)
#define GetPtrSize				DebugTraps_GetPtrSize
#define SetPtrSize				DebugTraps_SetPtrSize
#define PtrZone					DebugTraps_PtrZone
#define DisposePtr				DebugTraps_DisposePtr
#define PtrToHand(s,d,z)		DebugTraps_PtrToHand( s, d, z, __FILE__)
#define NewHandle(sz)			DebugTraps_NewHandle( sz, __FILE__)
#define NewHandleClear(sz)		DebugTraps_NewHandleClear( sz, __FILE__)
#define NewHandleSys(sz)		DebugTraps_NewHandleSys( sz, __FILE__)
#define NewHandleSysClear(sz)	DebugTraps_NewHandleSysClear( sz, __FILE__)
#define GetHandleSize			DebugTraps_GetHandleSize
#define SetHandleSize			DebugTraps_SetHandleSize
#define ReallocateHandle		DebugTraps_ReallocateHandle
#define DisposeHandle			DebugTraps_DisposeHandle
#define HLock					DebugTraps_HLock
#define HUnlock					DebugTraps_HUnlock
#define HGetState				DebugTraps_HGetState
#define HSetState				DebugTraps_HSetState
#define HandAndHand				DebugTraps_HandAndHand
#define HandToHand(h)			DebugTraps_HandToHand( h, __FILE__)
#define HPurge					DebugTraps_HPurge
#define HNoPurge				DebugTraps_HNoPurge
#define EmptyHandle				DebugTraps_EmptyHandle
#define HandleZone				DebugTraps_HandleZone
#define RecoverHandle			DebugTraps_RecoverHandle
#define RecoverHandleSys		DebugTraps_RecoverHandleSys
#define SetZone					DebugTraps_SetZone
#define GetZone					DebugTraps_GetZone
#define TempNewHandle(sz, err)	DebugTraps_TempNewHandle( sz, err, __FILE__)
#define TempDisposeHandle		DebugTraps_TempDisposeHandle
#define MoveHHi					DebugTraps_MoveHHi
#define PtrAndHand				DebugTraps_PtrAndHand


pascal void 	DebugTraps_BlockMove(const void *srcPtr, void *destPtr, Size byteCount);
pascal void 	DebugTraps_BlockMoveData(const void *srcPtr, void *destPtr, Size byteCount);

pascal Ptr		DebugTraps_NewPtr(Size byteCount, const char *srcFileName);
pascal Ptr		DebugTraps_NewPtrSys(Size byteCount, const char *srcFileName);
pascal Ptr		DebugTraps_NewPtrClear(Size byteCount, const char *srcFileName);
pascal Ptr		DebugTraps_NewPtrSysClear(Size byteCount, const char *srcFileName);
pascal Size		DebugTraps_GetPtrSize(Ptr p);
pascal void		DebugTraps_SetPtrSize(Ptr p, Size newSize);
pascal THz		DebugTraps_PtrZone(Ptr p);
pascal void		DebugTraps_DisposePtr(Ptr p);

pascal OSErr	DebugTraps_PtrToHand(const void *srcPtr, Handle *dstHndl, long size, const char *srcFileName);
pascal Handle	DebugTraps_NewHandle(Size byteCount, const char *srcFileName);
pascal Handle	DebugTraps_NewHandleClear(Size byteCount, const char *srcFileName);
pascal Handle	DebugTraps_NewHandleSys(Size byteCount, const char *srcFileName);
pascal Handle	DebugTraps_NewHandleSysClear(Size byteCount, const char *srcFileName);
pascal Size 	DebugTraps_GetHandleSize(Handle h );
pascal void 	DebugTraps_SetHandleSize(Handle h, Size newSize);
pascal void		DebugTraps_ReallocateHandle(Handle h, Size byteCount);
pascal void 	DebugTraps_DisposeHandle(Handle h);
pascal void 	DebugTraps_HLock(Handle h);
pascal void 	DebugTraps_HUnlock(Handle h);
pascal SInt8	DebugTraps_HGetState(Handle h);
pascal void		DebugTraps_HSetState(Handle h, SInt8 flags);
pascal OSErr 	DebugTraps_HandAndHand(Handle hand1, Handle hand2);
pascal OSErr	DebugTraps_HandToHand(Handle *theHndl, const char *srcFileName);
pascal void		DebugTraps_HPurge(Handle h);
pascal void		DebugTraps_HNoPurge(Handle h);
pascal void		DebugTraps_EmptyHandle(Handle h);
pascal THz		DebugTraps_HandleZone(Handle h);
pascal Handle	DebugTraps_RecoverHandle(Ptr p);
pascal Handle	DebugTraps_RecoverHandleSys(Ptr p);
pascal void		DebugTraps_MoveHHi(Handle h);
pascal OSErr	DebugTraps_PtrAndHand(const void *ptr1, Handle hand2, long size);

pascal void		DebugTraps_SetZone(THz hz);
pascal THz		DebugTraps_GetZone(void);

pascal Handle	DebugTraps_TempNewHandle(Size logicalSize, OSErr *resultCode, const char *srcFileName);
pascal void		DebugTraps_TempDisposeHandle( Handle h, OSErr *err);






// _____________________________ Other Managers _________________________________


#ifdef __cplusplus
}
#endif

#endif	// ]












