/***************************************************************************
	File:		SMTPSenderDoc.h
	Version:	1.0
	Purpose:	A Document representing a piece of mail

	Author: 	Christopher Haupt (cfh)
				Copyright  1996, Christopher Haupt
	
	Revision History:
	960201.cfh 	New file
***************************************************************************/

#pragma once

#include <LSingleDoc.h>
#include <LListener.h>
#include <LAsyncClient.h>
#include <PP_Networking.h>

// this document will be able to create and display existing mail
// messages...

const Int16	kSMTPPort	= 25;

class LEndpoint;
class LNetworkAddress;

class SMTPSenderDoc : 	public LSingleDoc, 
						public LListener,
						protected LAsyncClient
{
public:
						SMTPSenderDoc(LCommander *inSuper);
	virtual				~SMTPSenderDoc();

			
	virtual void		ListenToMessage(MessageT inMessage, void *ioParam);
	
	virtual void		Connect();
	virtual void		Disconnect();
	virtual Boolean		IsIdle();

	virtual Boolean		AllowSubRemoval(LCommander*	inSub);
		
protected:
	virtual void		HandleAsyncMessage(
								const LAsyncMessage& inMessage);
			void		BuildSessionWindow(void);
			void		SendMailMessage(void);

	enum {eConnecting = 0, eGreetingLong, eGreetingShort, 
			eMailSender, eMailDestination, eMailInitiateData, eMailBody, 
			eQuitting, eDisconnecting};

			void		RunMachine(char *inDataBuffer, Uint32 inDataSize);
			void		SendHELO(Boolean inUseShort = false);
			void		SendQUIT(void);
			void		SendRSET(void);
			void		SendNOOP(void);
			void 		SendMAIL(void);
			void 		SendRCPT(LStr255 &inRecipient);
			Boolean		SendNext(void);
			void 		SendDATA(void);
			void 		SendBody(void);
			void 		SendHeader(void);
			void		MakeMailDate(LStr255 &outDate);

			Boolean		ParseReply(char *inBuffer, Uint32 inBufferLen, Uint32& inPos);

	LStr255				mTo;
	LStr255				mSubject;
	LStr255				mSMTPHost;
	LStr255				mReturnAddress;
	Handle				mBody;
	LEndpoint*			mEndpoint;		// our network endpoint object
	LCaption*			mStatusPane;
	Int32				mMachineState;
	
	Int32				mLastCode;
	Int32				mCurPos;
	Int32				mMachineReplyState;
	enum {eStarting = 0, eGetStatusMode, eSkipJunkMode, eEOLMode };
	char				statusBuffer[8];
};
