/***************************************************************************
	File:		SMTPSender.cp
	Version:	1.0
	Purpose:	A Document interface to SMTP send functionality
	Author: 	Christopher Haupt (cfh)
				Copyright  1996, Christopher Haupt
	
	Revision History:
	960201.cfh 	New file, derived from starterapp
***************************************************************************/

#include "SMTPSender.h"

#include <LGrowZone.h>
#include <LThread.h>
#include <PP_Messages.h>
#include <PP_Resources.h>
#include <PPobClasses.h>
#include <UMemoryMgr.h>
#include <URegistrar.h>
#include <UThread.h>

#include "SMTPSenderDoc.h"

// constants
const Uint32	kAppRunQuantum	= 0;

// ===========================================================================
//		 Main Program
// ===========================================================================

void main(void)
{
									// Set Debugging options
	SetDebugThrow_(debugAction_Alert);
	SetDebugSignal_(debugAction_Alert);

	InitializeHeap(3);				// Initialize Memory Manager
									// Parameter is number of Master Pointer
									//   blocks to allocate
	
									// Initialize standard Toolbox managers
	UQDGlobals::InitializeToolbox(&qd);
	
	new LGrowZone(20000);			// Install a GrowZone function to catch
									//    low memory situations.

	SMTPSenderApp	theApp;			
	theApp.Run();
}


// ---------------------------------------------------------------------------
//		 SMTPSenderApp 		
// ---------------------------------------------------------------------------
//	Constructor

SMTPSenderApp::SMTPSenderApp()
{	
	RegisterAllPPClasses();

	// Create the main thread.  This will initialize the thread classes
	new UMainThread;
	
	// Setup our application attachment
	AddAttachment(new LYieldAttachment(kAppRunQuantum));
}


// ---------------------------------------------------------------------------
//		 ~SMTPSenderApp			
// ---------------------------------------------------------------------------
//	Destructor
//

SMTPSenderApp::~SMTPSenderApp()
{
}

// ---------------------------------------------------------------------------
//		 StartUp
// ---------------------------------------------------------------------------
//	This function lets you do something when the application starts up
//	without a document. For example, you could issue your own new command.

void
SMTPSenderApp::StartUp()
{
	ObeyCommand(cmd_New, nil);	
}

// ---------------------------------------------------------------------------
//		 MakeNewDocument
// ---------------------------------------------------------------------------
//	Create a new Document and pass back an AppleEvent Model object
//	representing that Document

LModelObject*
SMTPSenderApp::MakeNewDocument()
{
	return (LModelObject*) new SMTPSenderDoc(this);
}

// ---------------------------------------------------------------------------
//		 FindCommandStatus
// ---------------------------------------------------------------------------
//
void
SMTPSenderApp::FindCommandStatus(
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean		&outUsesMark,
	Char16		&outMark,
	Str255		outName)
{

	switch (inCommand) {
		case cmd_New:				
			outEnabled = true;			
			break;

		default:
			LApplication::FindCommandStatus(inCommand, outEnabled,
												outUsesMark, outMark, outName);
			break;
	}
}
