// CSoundsDemoData.cp -- document data
// Created 20/3/96 12:28 by AppMaker

// The purpose of the Data module is to shield the rest of your application
// from your internal data structures and the representation of your files.
// The Data module provides an abstract interface to the contents of your files.

#include "CSoundsDemoData.h"

#include <LFileStream.h>

//----------
// initialize application data

void	CSoundsDemoData::InitAppData()
{
}

// ---------------------------------------------------------------------------
//		 CSoundsDemoData
// ---------------------------------------------------------------------------

CSoundsDemoData::CSoundsDemoData()
		:LBroadcaster()
{
	mFile = nil;
	mDirty = false;
}

// ---------------------------------------------------------------------------
//		 ~CSoundsDemoData
// ---------------------------------------------------------------------------
//	Destructor
//

CSoundsDemoData::~CSoundsDemoData()
{
	CloseFile();
}

//----------
void	CSoundsDemoData::newData()
{
	CreateData();
	initDocData();
}

//----------
void	CSoundsDemoData::openData	(FSSpec		*inMacFSSpec)
{
	mFile = OpenFile (inMacFSSpec);
	initDocData();
	ReadData();
}

//----------
void	CSoundsDemoData::initDocData()
{
}

//----------
Boolean	CSoundsDemoData::IsDirty()
{
	return mDirty;
}

//----------
void	CSoundsDemoData::DoSave()
{
	WriteData(mFile);
}

//----------
void	CSoundsDemoData::DoSaveAs	(FSSpec		*inMacFSSpec)
{
	LFileStream		*file;

	file = CreateFile(inMacFSSpec);
	if (file != nil) {
		WriteData(file);
		CloseFile();	// old file
		mFile = file;
	}
}

//----------
void	CSoundsDemoData::DoRevert()
{
	DisposeData();
	if (mFile != nil) {
		ReadData();
	}
}

//----------
void	CSoundsDemoData::CloseFile()
{
	if (mFile != nil) {
		delete mFile;
		mFile = nil;
	}
}

//----------
// CreateFile is called in two situations:
// 1. To Save a new untitled document
// 2. To SaveAs to a new file

LFileStream		*CSoundsDemoData::CreateFile	(FSSpec		*inMacFSSpec)
{
	LFileStream		*file = new LFileStream(*inMacFSSpec);

	file->CreateNewDataFile(kSignature, kFileType);
	file->OpenDataFork(fsRdWrPerm);

	return file;
}

//----------
LFileStream		*CSoundsDemoData::OpenFile		(FSSpec		*inMacFSSpec)
{
	LFileStream		*file = new LFileStream(*inMacFSSpec);

	file->OpenDataFork(fsRdWrPerm);

	return file;
}

// The next few methods are for transferring data between the
// data file and your internal data structures, and for disposing
// your data structures. They are called by Open, Close, etc.
// Replace their bodies with whatever is suitable for your application

// define internal data structures to describe the file format:

//----------
typedef struct
{
	int			stuff;

} SoundsDemoData;

//----------
void	CSoundsDemoData::CreateData()
{
	// initialize your data structures
}

//----------
void	CSoundsDemoData::DisposeData()
{
	// delete your data structures

	mDirty = false;
}

//----------
void	CSoundsDemoData::ReadData()
{
	// read all or part of the file into your data structures
}

//----------
// WriteData is called in two situations:
// 1. To Save the current document
// 2. To SaveAs to a new file

void	CSoundsDemoData::WriteData		(LFileStream	*file)
{
	// write from your data structures to the file

	mDirty = false;
}

// The remaining methods are for accessing your data as logical chunks.
// These are just models for your own accessor functions;
// they aren't called by any AppMaker-generated code.
// Replace them with whatever is suitable for your application.

//----------
void	CSoundsDemoData::GetStuff	(void*		stuff)
{
}

//----------
void	CSoundsDemoData::SetStuff	(void*		stuff)
{
}

// SoundsDemoData.cp
