// UNIT BTreeDef

// B-Tree HELPER -  1985-90, 1993-5 (M)agreeable software, inc.

// Version 2.2

// 6 June 1995
 
// BTreeDef - Definitions and variables for use with B_Tree (BT) units

// B_Tree constants
 
#pragma once

#ifndef __ERRORS__
#include <Errors.h>
#endif
#ifndef __MEMORY__
#include <Memory.h>
#endif
#ifndef __PACKAGES__
#include <Packages.h>
#endif
#ifndef __RESOURCES__
#include <Resources.h>
#endif
#ifndef __STANDARDFILE__
#include <StandardFile.h>
#endif
#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef qDebug

#define qDebug 0								// 1 to include debug code, 0 to exclude

#endif

#if qDebug

#include <stdio.h>

#define kDebugItemSize 14				// Assume 8 + keySize (= 6).  Change according to tree
																// Used in debugging function Print_Page
#define kDebugKeyType kPStringCompare
																// Assume Pascal string.  Change according to tree
																// used in debugging function Print_Page
#define kDisplayKeySize 6				// Assume display max of five characters
																// Used in debugging function Print_Page

extern FILE		*gDebugStream;

#endif

#define kBTreeFileFormat		2

#define kInternalErr				4
#define kControlSizeErr			5
#define kNonContigErr				8		// did not find requested contiguous blocks in map
#define kMaxTrees				 1000
#define kMaxBlockSize		16388		// Maximum block size - largest integer such that
																// 8*blockSize*(blockSize-8) < maxlongint

// File Constants
#define noFile						-1
#define prevErr						-1
#define noAddr						-1L
#define kNoFile						-1
#define kPrevErr					-1
#define kNoAddr						-1L

// Map constants
#define kBitMapLength				8		// Number of bits in one bitmap, should be one byte
#define kBitMapEnd					7		// kBitMapLength-1
#define kMaxBitMaps			16380		// Max number of bitmaps in one mapControl record
#define kMapOffset					8		// sizeOf(longInt)+2*sizeOf(integer)

// Page constants
#define kItemOffset					6		// Offset to first item on page, sizeof(integer)+sizeof(FileAddr)
#define kKeyOffset					8		// Offset to key in item, 2*sizeof(FileAddr)
#define kMaxSimBytes		16388

// fileControl constants

#define kFileControlSaveSize	70	// the number of bytes to save in the file as the file control Record


#define kFileControlWorkSize	144 // the number of bytes in the relocatable block first obtained by
																	// Create_BTreeFile or Open_BTreeFile.	This first block holds the
																	// actual file control record plus a working area.	It does not
																	// contain the tree list record.	B-Tree HELPER expands and
																	// contracts the block as necessary for the tree list record

typedef short (*CompareProcPtr) (Ptr searchKeyP, Ptr treeKeyP, short keySize);

// Comparison selection constants
#define kSelectShort				1L
#define kSelectLong					2L
#define kSelectCString			3L
#define	kSelectPString			4L
#define kSelectIUString			5L

#define kShortCompare				(CompareProcPtr)kSelectShort
#define kLongCompare				(CompareProcPtr)kSelectLong
#define kCStringCompare			(CompareProcPtr)kSelectCString
#define	kPStringCompare			(CompareProcPtr)kSelectPString
#define kIUStringCompare		(CompareProcPtr)kSelectIUString

#define kLess			-1				// Comparison results for a compared to b
#define kEqual		0
#define kGreater	1

														// Search type for Search_Page
#define kFirstPass	1				// Accept all found records
#define kAndPass		2				// Mark records found again
#define kOrPass			3				// Add found records if not in list

#define kListIncrement 50		// number of list items for each SetHandleSize
														// increment of foundListH in SearchMgr


//	Naming conventions

//	typedef struct {} xRecord, *xPtr, **xHandle

//	variables
//			xRecord		x;
//			xPtr			xP;
//			xHandle		xH;

enum {kUniqueKeys, kDuplicateKeys};		// For use in Insert_Key and Delete_Key

typedef	enum {fast, reliable, buffered} PerformanceType;

typedef long blockNr;
typedef long FileAddr;

#if GENERATINGPOWERPC					// Ensure file compatibility
															// between 680xx and PowerMac
#pragma options align=mac68k
#endif

typedef struct
	{
	FileAddr	bufferAddr;
	Handle		bufferH;
	char			changed;
	char			fillChar;
	short			fillWord;
	} BufferControlItem;
	
typedef struct
	{
	long			bufferSize;
	short			nrBlocks;
	short			fillWord;
	BufferControlItem	item[1];
	} BufferControlRecord, *BufferControlPtr, **BufferControlHandle;

typedef	struct
	{
	long						freeBlocks;
	short						firstAvailBitMap;
	short						lastAvailBitMap;
	unsigned char		theMap[kMaxBitMaps];
	} MapControlRecord, *MapControlPtr, **MapControlHandle;

typedef	struct
	{
	short			version;				// Version of record format
	short			maxKeysPerPage;
	short			maxKeySize;
	FileAddr	rootAddr;				// Root of this tree
	} TreeControlRecord;

//	List of nrTrees tree control records
//	Number of trees bounded by BlockSize?

typedef	struct
	{
	short							nrTrees;
	TreeControlRecord	trees[kMaxTrees];
	} TreeListRecord, *TreeListPtr, **TreeListHandle;

typedef	struct
	{
	FileAddr	nextAddr;
	FileAddr	itemAddr;
	char			key[kMaxSimBytes];		// Key length is determined by creator
	} ItemRecord, *ItemPtr, **ItemHandle;

typedef	struct
	{
	short			itemCount;
	FileAddr	leftAddr;
	char			itemList;						// Length is determined by BT_Open
	} PageRecord, *PagePtr, **PageHandle;

typedef struct
	{
	long			found;
	FileAddr	foundAddr;
	} FoundItem;
	
typedef struct						// List of record addresses found by walking a tree
	{
	long 			count;
	FoundItem	list[1L];
	} FoundListRecord, *FoundListPtr, **FoundListHandle;

typedef	struct
	{
	short						version; 			// Version of B-Tree HELPER format
	char						FCRChanged;		// Write File Control Record if true
	PerformanceType	performance;
	short						FCRSize;			// Check on length
	short						blockSize; 		// Number of bytes per block, Map_Mgr allocates space
																//	as contiguous blocks
	long						freeBlocks; 	// Number of blocks available in file
	long						spareBlocks; 	// Number of blocks extra to be allocated or left
 																// before collapsing file.	Not to exceed BlockSize
	FileAddr				lastAddr; 		// Highest address allocated by OS to file
																//	LastAddr DIV BlockSize + 1 =
																//		Number of block groups
	FileAddr				firstMap; 		// Address of first Map and block group
																//		Maps are at beginning of block groups
																//		A block group is blocks per map plus Map Block
																//		Maps are at beginning of block groups
	FileAddr				firstFreeMap;	// Address of Map and block group	to begin search
																// Assumed first block groupe with free blocks
	FileAddr				lastMap;			// Address of last Map and block group
	short						nrTrees;			// used to read correct length for TreeList
	FileAddr				treeListAddr;	// Address of TreeArray
	long						reserved1;		// Reserved for future use
	long						reserved2;		// Reserved for future use
	long						reserved3;		// Reserved for future use
	long						reserved4;		// Reserved for future use

	long						application1;	// Reserved for application use
	long						application2;	// Reserved for application use
	long						application3;	// Reserved for application use
	long						application4;	// Reserved for application use

// End of FileControlRecord kept in file, see kFileControlSaveSize above

	short						fNum;					// path number of open file
	Handle					replyH;				// Handle to copy of reply used to open file
	BufferControlHandle		bufferControlH;		// Information for buffered IO
	IOParam						paramBlock;	// Parameter block for low-level IO

// MapWork area for this file

	long							blocksPerMap;	// BlockSize-kMapOffset
	FileAddr					mapAddr;
	MapControlHandle	mapControlH;
	char							mapChanged; 	// Write map if true

// See kFileControlWorkSize above

// Copy of nrTrees tree control records

	TreeListRecord		treeList;

	} FileControlRecord, *FileControlPtr, **FileControlHandle;

#if GENERATINGPOWERPC
#pragma options align=reset
#endif