//global data structures for the actual function of the program
#include "BTreeDef.h"
#include "BTreeProtos.h"
#include "StandardFile.h"


//need the structures for the data

 struct CustomerType{
 	unsigned char 	CustName[32];
 	long			CustNo;
 	long			StringAddr;
 	};
 typedef struct CustomerType CustomerType;
 typedef CustomerType *CustomerTypePtr;
 
 struct PartType {
 	unsigned char	PartName[32];
 	long			PartNo;
 	unsigned char	PartPrice[6];
 	};
 typedef struct PartType PartType;
 typedef PartType *PartTypePtr;
 
 struct LineItemType {
 	long 			LineItemNo;
 	long			PartNo;
 	unsigned short	NumItems;
 	unsigned char	ItemPrice[6];
	};
typedef struct LineItemType LineItemType;
typedef LineItemType *LineItemTypePtr;
 
 struct InvoiceType {
 	long			InvoiceNumber;
 	long			LineItem1;
 	long			LineItem2;
 	long			CustNo;
 	};
 typedef struct InvoiceType InvoiceType;
 typedef InvoiceType *InvoiceTypePtr;
 
  //We need to make a resource that will contain our sequence numbers.  Only 
 //the Line Item and Invoice records will need them assigned.
 
 struct SequenceRecord {
 	long	LineItems;
 	long	Invoices;
 	};
 	
 typedef struct SequenceRecord SequenceRecord;
 
 typedef SequenceRecord *SRPointer, **SRHandle;
 
 #define kSRType	'kSRT'
 #define kSRID		128
 
 //useful to have some constants
 
 
 //first define tree numbers
 
 #define kCustomerNumTree	0
 #define kCustomerNameTree	1
 #define kPartNumTree	2
 #define kPartNameTree	3
 #define kInvoiceTree		4
 #define kLineItemTree		5
 #define kInvoiceCustTree 6
 
 //define record sizes-- we'll use this quite a bit
 
 #define kCustomerSize	sizeof(CustomerType)
 #define kCustStringSize	40
 #define kPartSize		sizeof(PartType)
 #define kInvoiceSize	sizeof(InvoiceType)
 #define kLineItemSize	sizeof(LineItemType)
 
 
 //define key sizes-- also useful
 
 #define kCustomerNumSize	4
 #define kCustomerNameSize	32
 #define kPartNumSize	4
 #define kPartNameSize	32
 #define kInvoiceNumSize	4
 #define kLineItemNumSize	4
 
 /* define allocation block size and indirectly page size.  This is a fairly important decision
 in a large, "serious" database.  If incorrectly defined, you may not be able to accommodate
 your largest objects, or you may have gross inefficiency in file use, or both.  We will use
 a block size of 36 as it will accommodate our Customer and Part structs nicely, will
 allow for a fair size index page, and it really doesn't need to be perfect for this example.
 B-Tree Helper comes with a little program that allows you to play a bit with sizes and see
 the consequences of your decisions before you hard code anything. */
 
 #define kBlockSize		36L
 #define kSpareBlocks	10L
 #define kType			'IExD'
 #define kCreator		'IExA'
 #define kKeysPerPage	20	//this is small for most applications
 
 // constants for the part and  customer actions.
 
#define	kFindByName	1
#define kFindByNumber	2
#define	kNext		3
#define kPrevious	4
#define kInsert		5
#define	kEdit		6
#define kDelete		7 



//Here are the actual variables 

//need a flag that shows if a file is open
 extern Boolean gFileIsOpen;
 
//These are the variables for the data records when they are in memory
// these represent the "current" record

 extern CustomerType 	gCustomer;
 extern unsigned char	gCustString[40];
 extern PartType		gPart;
 extern InvoiceType		gInvoice;
 extern LineItemType	gLineItems[2];
 
// These are variables that will be used by B-Tree Helper 
 extern TreeListHandle 		gInvoiceTrees;
extern  FileControlHandle	gInvoiceFCH;
extern long					gBlockSize;
extern long					gSpareBlocks;
extern StandardFileReply	gFileReply;
extern SFTypeList			gTypeList;
extern short				gNumTypes;
extern short				gPermission;

//Our sequence handle
extern SRHandle	gSequenceHandle;

//resource fork reference number
extern short				gResFileNo;

// action switches

extern short	gCustomerAction;
extern short	gPartAction;
extern short	gInvoiceAction;
 
 
// used both by modeless shell and Invoice Example
extern Boolean gDone;
extern Boolean gInBackground;
extern DialogPtr gCustomerDialog;
extern DialogPtr gPartDialog;
extern DialogPtr gInvoiceDialog;
extern MenuHandle gFileMenu;
extern MenuHandle gEditMenu;
extern MenuHandle gWindowMenu;
 
 
 