// ---------------------------------------------------------
// Implementation of class BitGrpMapper - a class that
// does most of the base 2, 3, 5 and 9 arithmetic
// ---------------------------------------------------------


#define WRITE_LOOKUP_TABLES 		0


#if WRITE_LOOKUP_TABLES == 1
# include <stdlib.h>
# include <fstream.h>
#endif

#include "BitGrpMapper.h"

long BitGrpMapper::ubTable[33] = { 0 };
long BitGrpMapper::lbTable[33] = { 0 };


void BitGrpMapper::Init( 	long				chunkSz,
													long				numBits )
{
	switch (chunkSz) {
		case 1:
			lookupTable = NULL;
			firstMask = 0x01;
			firstMatchBit = 0x01;
			break;
		case 2:
			lookupTable = lookupTable2;
			firstMask = 0x03;
			firstMatchBit = 0x02;
			break;
		case 4:
			lookupTable = lookupTable4;
			firstMask = 0x0f;
			firstMatchBit = 0x08;
			break;
		case 8:
			lookupTable = lookupTable8;
			firstMask = 0xff;
			firstMatchBit = 0x80;
			break;
	}
		
	// Calculate the upper bound and lower bound lookup tables
	// used in RemoveLoop() of RouterRules.cp
	
	chunkSize = chunkSz;
	chunkCount = numBits / chunkSz;
	long excessBits = numBits % chunkSz;
	if (excessBits) chunkCount++;
	firstMask <<= ((chunkCount - 1) * chunkSz);
	firstMatchBit <<= ((chunkCount - 1) * chunkSz);
	
	ulng base = chunkSz + 1;
	ulng curBase = 1;
	long i;
	
	for (i=1; i<chunkCount; i++) {
		lbTable[i] = curBase;
		ubTable[i] = chunkSz * curBase;
		curBase *= base;
	}
	lbTable[i] = curBase;
	if (excessBits) ubTable[i] = excessBits * curBase;
	else            ubTable[i] = chunkSz * curBase;
	numGrpLists = ubTable[i] + curBase;
	

#if WRITE_LOOKUP_TABLES == 1
	{
		IndexEntry	tmpTable[256];
		ofstream		file;
		
		file.open( "BitGrpMapperTables.cp" );
		file << "#include \"BitGrpMapper.h\"" << endl << endl;
		CalcLookupTable( tmpTable, 2 );
		WriteLookupTable( file, tmpTable, "lookupTable2" );
		CalcLookupTable( tmpTable, 4 );
		WriteLookupTable( file, tmpTable, "lookupTable4" );
		CalcLookupTable( tmpTable, 8 );
		WriteLookupTable( file, tmpTable, "lookupTable8" );
		file.close();
	}
#endif
}


#if WRITE_LOOKUP_TABLES == 1

void BitGrpMapper::CalcLookupTable( IndexEntry  lookupTbl[],
																		long				chunkSz )
{
	long	lookupByte; 	// 0 .. 255
	long	byteInLong; 	// 3 .. 0, 0 for the most sig. byte
	long	chunkCount = 8 / chunkSz; // 8, 4, 2 or 1
	long  curChunk; 		// 0 .. chunkCount-1
	long	bitCounts[8]; // bitCounts[0]: right most chunk
	long	curBit; 			//  bit 0 .. bit 7 (right to left)
	long	bitIndex; 		// 0 .. chunkSz-1
	long  base = chunkSz + 1; // base 2, 3, 5, or 9
	long  curBase;			// a power of base
	
	for (lookupByte=0; lookupByte<256; lookupByte++) {
		
		curBit = 1;
		for (curChunk=0; curChunk<chunkCount; curChunk++) {
			bitCounts[curChunk] = 0;
			for (bitIndex=0; bitIndex<chunkSz; bitIndex++) {
				if (lookupByte & curBit) bitCounts[curChunk]++;
				curBit <<= 1;
			}
		}
		
		curBase = 1;
		for (byteInLong=3; byteInLong>=0; byteInLong--) {
			lookupTbl[lookupByte].index[byteInLong] = 0;
			for (curChunk=0; curChunk<chunkCount; curChunk++) {
				lookupTbl[lookupByte].index[byteInLong] +=
					curBase * bitCounts[curChunk];
				curBase *= base;
			}
		}
	}					
}


void BitGrpMapper::WriteLookupTable( ofstream		&file,
																		 IndexEntry table[],
																		 char*			tableName )
{
	long	entryCount;
	long	indexCount;
	
	file << "IndexEntry BitGrpMapper::" 
		<< tableName << "[256] = {" << endl;
	
	for (entryCount=0; entryCount<256; entryCount++) {
		file << "  { ";
		for (indexCount=0; indexCount<4; indexCount++) {
			// Printing as pointer to long writes value as
			// a four byte hex number in the Symantec environ.
			// Not so with my brand new CodeWarrior (at least
			// using the default project settings). Would need
			// to be fixed if the lookup tables had to be 
			// rebuilt.
			file << (long*) table[entryCount].index[indexCount];
			file << ((indexCount == 3) ? " }" : ", ");
		}
		file << ((entryCount == 255) ? ' ' : ',' ) << endl;
	}
			
	file << "};" << endl << endl;
}
#endif	
