//	------------------------------------------------------------------------------------------	
//
//		Copyright 1996, Chris De Salvo
//		Not to be distributed without expressed consent
//
//		Written by Chris De Salvo
//
//	------------------------------------------------------------------------------------------	

//	------------------------------	Includes

#include <AppleEvents.h>
#include <GestaltEqu.h>

#include "AEHandler.h"
#include "ErrorHandler.h"

//	------------------------------	Private Definitions

#define kGestaltMask				1L

//	------------------------------	Private Types
//	------------------------------	Private Variables
//	------------------------------	Private Functions

static void AEInstallHandlers(void);
static OSErr CheckForRequiredParams(AppleEvent *theEvent);

pascal OSErr AEOpenApp(AppleEvent *theEvent, AppleEvent *theReply, long refcon);
pascal OSErr AEOpenDoc(AppleEvent *theEvent, AppleEvent *theReply, long refcon);
pascal OSErr AEPrintDoc(AppleEvent *theEvent, AppleEvent *theReply, long refcon);
pascal OSErr AEQuit(AppleEvent *theEvent, AppleEvent *theReply, long refcon);

//	------------------------------	Public Variables

//	--------------------	AppleEventInit

void
AppleEventInit(void)
{
OSErr	theErr;
SInt32	features;

	theErr = Gestalt(gestaltAppleEventsAttr, &features);
	if (theErr)
	{
		NonFatalError("Error returned on Gestalt for AppleEvents.");
		return;
	}
	
	if (! (features & (kGestaltMask << gestaltAppleEventsPresent)))
		return;
		
	AEInstallHandlers();
}

//	--------------------	AEInstallHandlers

static void
AEInstallHandlers(void)
{
	if (AEInstallEventHandler(kCoreEventClass, kAEOpenApplication, NewAEEventHandlerProc(AEOpenApp), 0L, false))
		FatalError("Could not install 'oapp' AppleEvent handler.");
	
	if (AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, NewAEEventHandlerProc(AEOpenDoc), 0L, false))
		FatalError("Could not install 'odoc' AppleEventHandler.");
	
	if (AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments, NewAEEventHandlerProc(AEPrintDoc), 0L, false))
		FatalError("Could not install 'pdoc' AppleEventHandler.");
	
	if (AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, NewAEEventHandlerProc(AEQuit), 0L, false))
		FatalError("Could not install 'quit' AppleEventHandler.");
}

//	--------------------	AEOpenApp

pascal OSErr
AEOpenApp(AppleEvent *theEvent, AppleEvent *theReply, long refcon)
{
#pragma unused (theEvent, theReply, refcon)

	return (noErr);
}

//	--------------------	AEOpenDoc

pascal OSErr
AEOpenDoc(AppleEvent *theEvent, AppleEvent *theReply, long refcon)
{
#pragma unused (theEvent, theReply, refcon)

	return (noErr);
}

//	--------------------	AEPrintDoc

pascal OSErr
AEPrintDoc(AppleEvent *theEvent, AppleEvent *theReply, long refcon)
{
#pragma unused (theEvent, theReply, refcon)
	return (noErr);
}

//	--------------------	AEQuit

pascal OSErr
AEQuit(AppleEvent *theEvent, AppleEvent *theReply, long refcon)
{
#pragma unused (theEvent, theReply, refcon)

	return (noErr);
}

//	--------------------	CheckForRequiredParams

static OSErr
CheckForRequiredParams(AppleEvent *theEvent)
{
DescType	returnedType;
Size		actualSize;
OSErr	theErr;

	theErr = AEGetAttributePtr(theEvent, keyMissedKeywordAttr, typeWildCard, &returnedType, nil, 0, &actualSize);
	
	if (theErr == errAEDescNotFound)
		return (noErr);
	else
		if (theErr == noErr)
			return (errAEParamMissed);

	return (theErr);
}