//	------------------------------------------------------------------------------------------	
//
//	Copyright  1996 Apple Computer, Inc., All Rights Reserved
//
//
//		You may incorporate this sample code into your applications without
//		restriction, though the sample code has been provided "AS IS" and the
//		responsibility for its operation is 100% yours.  However, what you are
//		not permitted to do is to redistribute the source as "DSC Sample Code"
//		after having made changes. If you're going to re-distribute the source,
//		we require that you make it clear in the source that the code was
//		descended from Apple Sample Code, but that you've made changes.
//
//		Authors:
//			Chris De Salvo
//			Jamie Osborne
//
//	------------------------------------------------------------------------------------------	

//	------------------------------	Includes

#include <Fonts.h>

#include "DrawSprocket.h"
#include "InputSprocket.h"

#include "ErrorHandler.h"
#include "EventHandler.h"
#include "Graphics.h"
#include "MenuHandler.h"
#include "RedbookHandler.h"
#include "SoundHandler.h"
#include "NetSprocketSupport.h"
#include "SprocketInvaders.h"

//	------------------------------	Private Definitions
//	------------------------------	Private Types
//	------------------------------	Private Variables
//	------------------------------	Private Functions

void main(void);
static void ToolboxInit(void);

//	------------------------------	Public Variables

//	--------------------	main

void
main(void)
{
//	When we start up we record the state of the modifier keys on the keyboard
//	and then pass those values to the various initialization routines in the game.
SInt16	startupModifiers;
	
	ToolboxInit();

	//	Make sure that the DrawSprocket and InputSprocket libraries are present
	if (DSpStartup == nil)
		FatalError("This game requires DrawSprocket.");

	if (ISpInit == nil)
		FatalError("This game requires InputSprocket.");

	//	Reseed the random number generator
	qd.randSeed =  TickCount();

	startupModifiers = EventInit();
	gNetSprocketPresent = InitNetworking();

	MenuInit();

	GraphicsInit(startupModifiers);
	SoundHandlerInit(startupModifiers);
	RedbookHandlerInit(startupModifiers);
	
	//	Run the game
	EventLoop();

	//	Shut down the handlers and quit
	SoundHandlerReset();
	GraphicsReset();

	if (gNetSprocketPresent)
		ShutdownNetworking();

	ISpStop();

	FlushEvents(everyEvent, 0);
	ExitToShell();
}

//	--------------------	main

static void
ToolboxInit(void)
{
	MaxApplZone();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	
	FlushEvents(everyEvent, 0);
}