/*	TList Class Methods	*/

/*	TList implements a singly-linked List of pointers
	The TList has pointers to the start and end nodes.
	Each node has a pointer to the next TVector in the list
	
    start --> node1 --> node2 ->....-
                                     |--> nodeN --> 0
                              end --
          
    New nodes are added by inserting them after a specific node
	using insert (TVector *aVector, TVector *afterVector)
	or by adding them after the end node using insert (TVector *aVector).

*/
#include "Tangrams.h"

TList::TList()
{
	start = 0;
	end = 0;
}

void TList::IList(TList& aList)
{
	TVector *v1, *v2;
	v1 = aList.start;
	while (v1) {
		v2 = new TVector (*v1);
		insert(v2);
		v1 = v1->next;
	}
	return;
}

TList::~TList()
{
	TVector	*n1, *n2;
	
	n1 = start;
	while (n1 != 0)
	{
		n2 = n1->next;
		delete n1;
		n1 = n2;
	}
}

void TList::insert (TVector *aVector)
{
/*	Append TVector as the last item in the list */
	if (end)
		insert (aVector, end);
	else
	{
		aVector->next = 0;
		start = end = aVector;
	}
}

void TList::insert (TVector *aVector, TVector *afterVector)
{
/*	Insert TVector after afterVector */
	if (afterVector)
	{
		aVector->next = afterVector->next;
		afterVector->next = aVector;
	}
	else
	{
		aVector->next = start;
		start = aVector;
	}
	if (end == afterVector)
		end = aVector;
}

void TList::remove (TVector *aVector)
{	
	if (start)
	{
		if (aVector == start)
		{
			start = start->next;
			if (end == aVector)
				end = 0;
		}
		else
		{
			TVector*	n = start;
			while (n)
			{
				if (n->next == aVector)
				{
					n->next = aVector->next;
					if (aVector == end)
						end = n;
					break;
				}
				else
					n = n->next;
			}
		}
	}
}

TVector*	TList::first (void)
{
	return start;
}

TVector*	TList::next (TVector *aVector)
{
/*	Return the next node in the list. */
	return aVector->next;
}

TVector*	TList::last (void)
{
	return end;
}

Boolean	TList::valid (TVector *aVector)
{
	TVector	*n;
	n = start;
	if (aVector)
		while (n)
			if (aVector == n)
				return true;
			else
				n = n->next;
	return false;
}