/*	TPolygon Class methods */

#include "Tangrams.h"
#include "Debug.h"

#include <QuickDraw.h>

TPolygon::TPolygon () {;}

TPolygon::TPolygon (TPolygon& p)
{
	*(TVector *)this = p;
	sides.IList(p.sides);
}


void TPolygon::IPolygon (TPolygon *p)
{
	*(TVector *)this = *p;
	sides.IList(p->sides);
}

void TPolygon::IPolygon (MyPolygon *theData)
{
/*	Create a TPolygon using cartesian input data */
	long	i;
	TVector	*aSide, previousSide;
	
/*	The origin defines the vector position
	of the first vertex	relative to the origin and the h=0 axis */
	setHV (theData->vertex[0].h, theData->vertex[0].v);
	
	for (i = 0; i < theData->numVertices ; i++)
	{
		aSide = new TVector;
		if (!aSide)
			MyError("\pIPolygon: Failed to create a new side vector");
	 	sides.insert (aSide);
	/*	Convert each vertex to a vector from the previous vertex	*/
	 	if (i == theData->numVertices -1)
	 	/*	The last vector is relative to the origin	*/
			aSide->setHV (theData->vertex[0].h - theData->vertex[i].h,
	 					theData->vertex[0].v - theData->vertex[i].v);
	 	else
			aSide->setHV (theData->vertex[i+1].h - theData->vertex[i].h,
	 					theData->vertex[i+1].v - theData->vertex[i].v);
	}
}

/*	Access */
TVector	TPolygon::getOrigin (void)
{
	return *this;
}

TVector* TPolygon::firstSide (void)
{
	return sides.first();
}

TVector* TPolygon::nextSide (TVector *aSide)
{
	TVector *v = sides.next(aSide);
	if (v)
		return v;
	else
		return sides.first();
}

TVector* TPolygon::lastSide (void)
{
	return sides.last();
}

float	TPolygon::getRotation (void)
{
	return sides.first()->getAngle();
}

void	TPolygon::rotate (float anAngle)
{
		TVector	*theSide, *firstSide;
	
		firstSide = sides.first();
		if (firstSide)
		{			
			theSide = firstSide;
			do {
				theSide->rotate(anAngle);
				theSide = sides.next(theSide);;
			} while (theSide);
		}
}
	
void	TPolygon::translateTo (TVector *where)
{
	if (where)
	{
		*(TVector *)this = *where;
	}
	else
		set(0,1);
}
	
void	TPolygon::rotateTo (float direction)
{
	rotate (direction - getRotation());
}
	
Boolean TPolygon::fill (TList *unplaced, TList *placed)
{
	Boolean		result = false;
	TShape	*thePiece, *previousPiece = 0;
	TPolygon	*newSpace;

	thePiece = (TShape *)unplaced->first();
	if (thePiece)
	{
		do { /*	Try all unplaced pieces */
		
		/*	Transfer the current piece to the placed list */
			unplaced->remove (thePiece);
			placed->insert(thePiece);
			
		/*	Align thePiece with the first side of the Shape */
			align(thePiece);
			do { /*	Try each flip state */
				do { /*	Try all orientations */
					if (this->contains (thePiece))
					{
					/*	Copy the Shape */
						newSpace = new TPolygon (*this);
					/*	Modify the copy to remove the area of thePiece. */
						newSpace->subtract (thePiece);
					/*	Try to place remaining pieces in this shape. */
						result = newSpace->fill (unplaced, placed);
						
					/*	Discard the modified shape. */
						delete newSpace;
						
						if (result) return true;
					}
				} while (thePiece->turn());
			} while (thePiece->flip());
			
		/*	Move the current piece back to the unplaced list*/
			placed->remove (thePiece);
			unplaced->insert (thePiece, previousPiece);
			
		/*	Next unplaced piece */
			previousPiece = thePiece;
			thePiece = (TShape *)unplaced->next (thePiece);
		} while (!result && thePiece);
	}
	else
	/*	No pieces left to place - job done */
		result = true;
	return result;
}

void TPolygon::align (TPolygon *p)
{
	p->translateTo(this);
	p->rotateTo (getRotation());
}

void TPolygon::subtract (TPolygon *thePiece)
{
	TVector	*firstPieceSide, *pieceSide, 
			*firstShapeSide, *shapeSide,
			*aSide, difference;

/*	The origins of the Shape and thePiece should be congruent */
	Assert(TVector(*this) == thePiece->getOrigin(),
			"\pSubtract: thePiece is not placed at the origin of the Shape");
			
/*	Start at the first sides */
	firstPieceSide = pieceSide = thePiece->firstSide();
	firstShapeSide = shapeSide = firstSide();
	
/*	Calculate the difference */
/*	Delete all consecutive sides of the shape that are
	congruent with sides of aPiece. */
	do {
		difference = *shapeSide;
		difference -= *pieceSide;
		if (difference.getLength() == 0)
		{
		/*	These sides are congruent */
			*this += *shapeSide;
			aSide = shapeSide;
			shapeSide = nextSide(aSide);
			sides.remove (aSide);
			delete aSide;
			pieceSide = thePiece->nextSide(pieceSide);
			if (pieceSide == firstPieceSide)
			{
			/*	We've gone right round thePiece
				All piece sides congruent.
				We have filled an enclosed space in the shape.
				So just return */
				return;
			}
			else if (*shapeSide || *pieceSide)
			/*	The next sides are parallel
				Loop to check if they are equal */
				;
			else
			/*	The next sides are not concurrent */
				break;
		}
		else
		{
		/*	These sides are concurrent but have different lengths
			Reduce the shape side to the length and direction of the residue,
			put the origin here, and move to the next piece side */
			*shapeSide = difference;
			*this += *pieceSide;
			
			pieceSide = thePiece->nextSide(pieceSide);
			break;
		}

	} while (true);
	
/*	pieceSide is the first non-congruent side of thePiece.
	Make a new list of the rest of the pieceSides in reverse order */
	
	TList *newSides = new TList();
	while (pieceSide != thePiece->firstSide())
	{
		aSide = new TVector(*pieceSide);
		newSides->insert(aSide, 0);
		pieceSide = thePiece->nextSide(pieceSide);
	} 
	
/*	Scan the list of newSides
	and compare them with the last sides of the shape.
	Eliminate duplicated or concurrent sides */
	pieceSide = newSides->first();
	while (pieceSide)
	{
		shapeSide = lastSide();
		if (*pieceSide == *shapeSide)
		/*	Sides are congruent so remove from the shape and loop */
			sides.remove (shapeSide);
		else if (*pieceSide || *shapeSide)
		{
		/*	Sides are parallel but different lengths */
			shapeSide->setLength (shapeSide->getLength() - pieceSide->getLength());
			pieceSide = newSides->next(pieceSide);
			break;
		}
		else
			break;
		pieceSide = newSides->next(pieceSide);
	}
/*	Reverse the rest of the pieceSides and append to the shape */
	while (pieceSide)
	{
		newSides->remove (pieceSide);
		shapeSide = (TVector *)newSides->next(pieceSide);
		sides.insert (pieceSide);
		pieceSide->reverse();
		pieceSide = shapeSide;
	}
/*	Destroy the temporary List */
	delete newSides;
}

Boolean TPolygon::contains (TPolygon *p)
{
/*	Test if *this contains (is wholly outside) p  */
	Boolean		result = true;
	TVector		*inside, *firstInside,
				*outside, *firstOutside,
				outStart, inStart, relative;
//				minAngle, maxAngle;
	
	inStart = p->getOrigin();
	inside = firstInside = p->firstSide();
	do { /* for each side of *p */
		outStart = (TVector)*this;
		outside = firstOutside = firstSide();
/*		relative = outStart;
		relative -= inStart;
		minAngle = maxAngle = relative;
*/		do	 /*	for each side of *this */
		{
		/*	Check if sides of *p and *this intersect */
			if (*inside || *outside)
			/*	Parallel lines don't intersect */
				;
			else if (Intercept (&inStart, inside, &outStart, outside))
			{
			/*	These sides intersect */
				result = false;
				break;
			}
			
		/*	update the subtended angles
			relative += *outside;
			if (relative > maxAngle) maxAngle = relative;
			else if (relative < minAngle) minAngle = relative; */
			
		/*	next side of *this */
			outStart += *outside;
			outside = nextSide (outside);
		} while (outside != firstOutside);
		
		if (!result) break;
		
	/*	If max and min subtended angles are not parallel
		the start point of *p side is outside *this
		if (!(maxAngle || minAngle))
		{
			result = false;
			break;
		} */
	/*	next side of *p */
		inStart += *inside;
		inside = p->nextSide (inside);
	} while (inside != firstInside);
	
	return result;
}
