/*=====================================================================

								TANGRAMS
								
				December 1996  Programmers' Challenge
				
							 1996 Alan Hart
						  
======================================================================*/

/*
NOTES

1.	Translate all the Polygons into relative vector sequences
	to do the manipulations and tests.
	Once we have a solution, convert the final positions back into
	transformations for return to the caller.
			
2.	Holes are almost the same as pieces. Exceptions:
		- We don't need to return their transformations
		- They probably can't be flipped,
		- They probably can't lie along the edges of the shape.

3.	The process will be recursive trial and error:

	Where pieces are used, Holes can also be included in the process,
	taking into account that they can't be flipped or deployed at the edge of the shape.

	- Put the first vertex of the first piece at the first vertex of the shape.
	- Align the first side of the piece with the first side of the shape.
	- If the piece is not entirely inside the shape, try the piece's next vertex.
	- If the piece is inside the shape modify the shape to remove the piece
		and recurse to try to place the remaining pieces.
	- If the pieces are placed successfully, return
	- When all vertices of the piece have been tried unsuccesfully, flip it and repeat
	- When both flips of the piece have been tried,
		move it to the next vertex of the shape and repeat
	- When all vertices of the shape have been tried,
		flip the shape and repeat

4.	We need a slick procedure for testing whether one polygon lies entirely inside another.

*/


#include "Tangrams.h"

void SolveTangram(
	MyPolygon	*theShape,
	long		numHoles,
	MyPolygon	*theHoles[],
	long		numPieces,
	MyPolygon	*thePieces[],
	MyTransform	*theXForms[]
)

{
/*	Initialise the Tangram instance	*/
	TTangram theTShape;
	theTShape.ITangram (theShape,
						numHoles, theHoles,
						numPieces, thePieces,
						theXForms);
	
/*	Fill The Shape	*/
	theTShape.solve();

	return;
}

