import java.awt.*;
import java.util.*;

abstract class Shape
{
	boolean		highlighted;
	ShapeCanvas	shapeCanvas;
	int			shapeX, shapeY, shapeWidth, shapeHeight;
	Rectangle	boundsRect;
	
	Shape( ShapeCanvas canv, int x, int y, int width, int height )
	{
		shapeCanvas = canv;
		highlighted = false;
		
		shapeX = x;
		shapeY = y;
		shapeWidth = width;
		shapeHeight = height;
		
		boundsRect = new Rectangle( x, y, width, height );
	}
	
	abstract public void draw( Graphics g );
	
	public void erase( Graphics g ) //Added for Step2
	{
		g.clearRect( shapeX, shapeY, shapeWidth, shapeHeight );
	}
	
	public void setHighlight( boolean newHighlight )
	{
		highlighted = newHighlight;
	}
	
	public boolean isHighlighted()
	{
		return highlighted;
	}
	
	public boolean isPointInShape( int x, int y )
	{
		return boundsRect.inside( x, y );
	}

	public void	moveThisMuch( int x, int y ) //Added for Step2
	{
		boundsRect.move( shapeX + x, shapeY + y );
		shapeX = boundsRect.x;
		shapeY = boundsRect.y;
		shapeWidth = boundsRect.width;
		shapeHeight = boundsRect.height;
	}
}

class RectShape extends Shape
{
	RectShape( ShapeCanvas canv, int x, int y, int width, int height )
	{
		super( canv, x, y, width, height );
	}
	
	public void draw( Graphics g )
	{
		if ( isHighlighted() )
		{
			g.setColor( Color.black );
			g.fillRect( shapeX, shapeY, shapeWidth, shapeHeight );
			g.setColor( Color.red );
			g.fillRect( shapeX+2, shapeY+2, shapeWidth-4, shapeHeight-4 );
		}
		else
		{
			g.setColor( Color.red );
			g.fillRect( shapeX, shapeY, shapeWidth, shapeHeight );
		}
	}
}

//Step 1: Add new Shape subclass
class OvalShape extends Shape
{
	OvalShape( ShapeCanvas canv, int x, int y, int width, int height )
	{
		super( canv, x, y, width, height );
	}
	
	public void draw( Graphics g )
	{
		if ( isHighlighted() )
		{
			g.setColor( Color.black );
			g.fillOval( shapeX, shapeY, shapeWidth, shapeHeight );
			g.setColor( Color.red );
			g.fillOval( shapeX+2, shapeY+2, shapeWidth-4, shapeHeight-4 );
		}
		else
		{
			g.setColor( Color.red );
			g.fillOval( shapeX, shapeY, shapeWidth, shapeHeight );
		}
	}
}
//END Step 1

class ShapeCanvas extends Canvas
{
	Vector		shapes;
	Shape		curShape;
	Point		mousePosition; //Part of Step 2, used for drag tracking
	
	ShapeCanvas( int width, int height )
	{
		shapes = new Vector();
		curShape = null;
		
		setBackground( Color.yellow );
		
		resize( width, height );
	}
	
	public void addShape( Shape newShape )
	{
		shapes.addElement( newShape );
	}
	
	public void paint( Graphics g )
	{
		for ( Enumeration e = shapes.elements(); e.hasMoreElements(); )
		{
			Shape s = (Shape)e.nextElement();
			s.draw( g );
		}
	}
	
	public Shape findInShapeList( int x, int y )
	{
		for ( Enumeration e = shapes.elements(); e.hasMoreElements(); )
		{
			Shape s = (Shape)e.nextElement();
			
			if ( s.isPointInShape( x, y ) )
			{
				s.draw( getGraphics() );
				
				return s;
			}
		}
		
		return null;
	}
	
	public void update (Graphics g)
	{
    		paint(g);
	}
	
	public boolean mouseDown( Event e, int x, int y )
	{
		curShape = findInShapeList( x, y );
		
		if ( curShape != null ) //For Step2
		{
			curShape.setHighlight( true );
			mousePosition = new Point( x, y );
			
			return true;
		}
			
		return false;
	}
	
//Step 2: Add Shape dragging
	public boolean mouseDrag( Event e, int x, int y )
	{
		Graphics	g;
		
		if ( curShape != null )
		{
			g = getGraphics();
			
			curShape.erase( g );
			
			curShape.moveThisMuch( x - mousePosition.x,
				y - mousePosition.y );
				
			mousePosition.x = x;
			mousePosition.y = y;
			
			curShape.draw( g );
			
			return true;
		}
		
		return false;
	}
	
	public boolean mouseUp( Event e, int x, int y )
	{
		if ( curShape != null )
		{
			curShape.setHighlight( false );
			repaint();
			
			return true;	
		}
		
		return false;
	}
//End Step 2
}

public class ShapeWorld extends java.applet.Applet
{
	ShapeCanvas	sCanvas;
	final int	shapeWidth = 20;
	final int	shapeHeight = 20;
	
	public void init()
	{
		int			x, y;
		
		sCanvas = new ShapeCanvas( 440, 290 );
		add( sCanvas );
		
		Random ran = new Random();
		Rectangle b = sCanvas.bounds();
		
		for ( int i=1; i<=10; i++ )
		{
			x = b.x + (int)((float)(b.width) * ran.nextFloat() );
			if ( x > b.x + b.width - shapeWidth )
				x -= shapeWidth;
				
			y = b.y + (int)((float)(b.height) * ran.nextFloat() );
			if ( y > b.y + b.height - shapeHeight )
				y -= shapeHeight;
			
			RectShape r = new RectShape( sCanvas, x, y, shapeWidth, shapeHeight );
			sCanvas.addShape( r );
		}

//Step3: Create some of the new shape		
		for ( int i=1; i<=10; i++ )
		{
			x = b.x + (int)((float)(b.width) * ran.nextFloat() );
			if ( x > b.x + b.width - shapeWidth )
				x -= shapeWidth;
				
			y = b.y + (int)((float)(b.height) * ran.nextFloat() );
			if ( y > b.y + b.height - shapeHeight )
				y -= shapeHeight;
			
			OvalShape r = new OvalShape( sCanvas, x, y, shapeWidth, shapeHeight );
			sCanvas.addShape( r );
		}
//End Step3
	}
}