'From VisualWorks(R) VanGogh beta release 1 of January 1996 on March 5, 1996 at 11:07:50 am'!

MacComponentProxy subclass: #MacTypeInPopUpProxy
	instanceVariableNames: 'control inputField items '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mac PlatPok-Examples'!
MacTypeInPopUpProxy comment:
'This class implements a Type-In Pop-Up Menu, which consists of a TextEdit and a PopUpMenu Control. It is described in detail in Macintosh Human Interface Guidelines, pages 91-92.  
Although it is documented, the Macintosh does not provide a single control which implements this behaviour.  Therefore this class implements it by holding on to a MacTextEdit and a MacPopUpMenuProxy.  
This class is an example, and only implements the behaviour.  A real proxy would provide event notification and data accessing so a client could do something when the value changes.

Instance Variables
	control: a MacPopUpMenuProxy
	inputField: a MacTextEdit
	items: for speed we store the menu items'!


!MacTypeInPopUpProxy methodsFor: 'instance creation'!


createBounds: bounds items: aSet initialSelection: anInteger 

	"Create a Type-In PopUpMenu Control with the given values."

	"Create my components"
	self createMenuControl: (self menuBoundsFrom: bounds);
		createTextControl: (self inputFieldBoundsFrom: bounds).

	"Cache the items, then insert them into the menu"
	items := aSet.
	items do: [:item | 
		control menu append: item asPascalString]. 
	
	"Set the input field to the initial selection"
	inputField setText: (aSet at: anInteger)! !

!MacTypeInPopUpProxy methodsFor: 'component creation'!


createMenuControl: bounds

	"Create my menu and configure it appropriately"

	control := (MacPopUpMenuProxy creatorClass new)
		smalltalkWindowClass: MacPopUpMenuProxy;
		clientBounds: bounds;
		window: self window;
		createWindow!

createTextControl:
Use the appropriate creator object to make a text edit object

createTextControl: bounds

	"Create my input field and configure it appropriately"

	inputField := (MacTextEditProxy creatorClass new)
		clientBounds: bounds;
		styledText: false;
		window: self window;
		createWindow! !

!MacTypeInPopUpProxy methodsFor: 'bounds utilities'!


inputFieldBoundsFrom: aBounds

	"Answer a bounds rectangle for the input field"

	^(aBounds copy) 
		right: aBounds right - Host api kMenuWidth - 1;
		bottom: aBounds top + Host api kMenuHeight!


menuBoundsFrom: aBounds

	"Answer a bounds rectangle for the popup button"

	^(aBounds copy) 
		left: aBounds right - Host api kMenuWidth;
		bottom: aBounds top + Host api kMenuHeight! !


!MacTypeInPopUpProxy methodsFor: 'event processing'!


activateEvt: theEvent

	"Dispatch the events to my components"

	control activateEvt: theEvent.
	inputField activateEvt: theEvent!


mouseDown: theEvent

	"Handle a mouse down event in my window"

	"The standardMouseDown: method handles everything but inContent"
	(self window standardMouseDown: theEvent) ifFalse: [
		self mouseDownInContent: theEvent]!


mouseDownInContent: theEvent

	"Handle a mouse down event in my window's content"

	| thePoint eventDetails |

	"Find the child which was hit.  eventDetails contains the control object and the part code, or nil"
	eventDetails := self window findChild: theEvent local.
	eventDetails isNil ifTrue: [^nil].

	"Send the event to the appropriate component"
	eventDetails proxy = control ifTrue: [
		self mouseDownInMenu: theEvent]
	ifFalse: [
		inputField mouseDown: theEvent details: eventDetails]!


mouseDownInMenu: theEvent

	"Handle a mouse down event in my menu"

	| match thePoint |

	"Add the input field text if it's not in the menu"
	(match := items includes: inputField getText) ifFalse: [
		control menu insert: inputField getText.
		control menu insert: Host api menuSeparator.

	"Track the control.  If a menu item is selected, put its value into the input field"
	(control track: theEvent local) > 0 ifTrue: [
		inputField setText: (control menu getItemText: control getValue)].

	"Remove the extra items, so the menu only has them during mouse processing"
	match ifFalse: [
		control menu deleteItem: 1;
			deleteItem: 1]]! !

!MacTypeInPopUpProxy methodsFor: 'accessing'!


inputField

	^inputField!

items

	^items!

items: aSet

	items := aSet!

control

	^control! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MacTypeInPopUpProxy class
	instanceVariableNames: ''!

!MacTypeInPopUpProxy class methodsFor: 'instance creation'!


newOnWindow: aWindow bounds: aBounds items: aSet initialSelection: anInteger

	"Create and return a new proxy"
	^(self new)
		window: aWindow;
		createBounds: aBounds items: aSet initialSelection: anInteger;
		yourself! !

!MacTypeInPopUpProxy class methodsFor: 'examples'!


example

	"Demonstrate a Type-In PopUpMenu Control"
	"MacTypeInPopUpProxy example"

	| windowProxy proxy |
	"Use a window creator to make a window proxy"
	windowProxy := (MacWindowProxy creatorClass new)
		clientBounds: (50@50 extent: 180@100);
		title: 'PopUp Example';
		createWindow.

	"Create a Type-In Pop-Up proxy on the windowProxy"
	controlProxy := self newOnWindow: windowProxy
		bounds: (10@10 extent: 100@19)
		items: #('Seagoon' 'Eccles' 'Moriarty' 'Henry' 'Min' 'Bluebottle')
		initialSelection: 2.

	"Register events which the proxy will handle"
	windowProxy
		forwardEvents: #(
				mouseDown)
			to: controlProxy.

	"Register events which only the text field needs to handle"
	windowProxy
		forwardEvents: #(
				keyDown
				autoKey
				nullEvent)
			to: controlProxy inputField.

	"Register events which the window can handle"
	windowProxy
		forwardEvents: #(
				activateEvt
				updateEvt)
			to: windowProxy.

windowProxy show.

	^windowProxy! !
