/*------------------------------------------------------------------------------
	File:		TextTransferExt.cpp

	Contains:	Text Transfer Extension class implementation

	Written by:	Andrey Dolgachev
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
------------------------------------------------------------------------------*/

// -- Compiler/Preprocessor Notification --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (ie. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with "ev" parameter set.
#include <Except.h>
#endif

// Notification that this is a SOM source file
#define TextTransferExt_Class_Source

// define underscore field names (ie. _fSelf)
#define VARIABLE_MACROS

// -- TextTransferExt Includes --

#ifndef SOM_TextTransferExt_xih
#include "TextTransferExt.xih"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODExtension_xh
#include <Extensn.xh>
#endif


#pragma segment TextTransferExt


//------------------------------------------------------------------------------
// Method:			Release
// Origin:			ODRefCountedObject
//
// Description:		The Release method decrements an objects reference count
//					by 1, and tells the draft to release the object from 
//					memory if the ref count falls to 0. The parent class
//					must be called at the beginning of this method.
//------------------------------------------------------------------------------
SOM_Scope	void
SOMLINK 	TextTransferExt__Release
			(
				TextTransferExt*	somSelf,
				Environment*		ev
			)
{
    TextTransferExtMethodDebug("TextTransferExt","Release");

	TextTransferExt_parent_KSS_DataTransferExt_Release(somSelf, ev);
}

//------------------------------------------------------------------------------
// Method:		InitTextTransferExt
// Origin:		TextTransferExt
//------------------------------------------------------------------------------
SOM_Scope	void
SOMLINK		TextTransferExt__InitTextTransferExt
			(
				TextTransferExt*	somSelf,
				Environment*		ev,
				ODPart* 			owner
			)
{
    TextTransferExtMethodDebug("TextTransferExt","InitTextTransferExt");

	somSelf->InitExtension(ev, owner);
}

//------------------------------------------------------------------------------
// Method:		GetSelection
// Origin:		TextTransferExt
//------------------------------------------------------------------------------
SOM_Scope	ODBoolean
SOMLINK		TextTransferExt__GetSelection
			(
				TextTransferExt*	somSelf,
				Environment*		ev,
				ODULong* 			start,
				ODULong* 			end
			)
{
    TextTransferExtMethodDebug("TextTransferExt", "GetSelection");
    
	ODBoolean	result = kODTrue;
	
	return result;
}

//------------------------------------------------------------------------------
// Method:		SetSelection
// Origin:		TextTransferExt
//------------------------------------------------------------------------------
SOM_Scope 	ODBoolean
SOMLINK 	TextTransferExt__SetSelection
			(
				TextTransferExt*	somSelf,
				Environment*		ev,
				ODULong 			start,
				ODULong 			end
			)
{
    TextTransferExtMethodDebug("TextTransferExt","SetSelection");

	ODBoolean result = kODTrue;

	return result;
}

//------------------------------------------------------------------------------
// Method:		AppendText
// Origin:		TextTransferExt
//------------------------------------------------------------------------------
SOM_Scope	ODBoolean
SOMLINK		TextTransferExt__AppendText
			(
				TextTransferExt*			somSelf,
				Environment*				ev,
				ODPlatformType				type,
				ODByteArray* 				data
			)
{
    TextTransferExtMethodDebug("TextTransferExt","AppendText");

	return kODFalse;
}

//------------------------------------------------------------------------------
// Method:		GetData
// Origin:		DataTransferExt
//
// Description:	Call the parent class to retrieve the data from the part.
//------------------------------------------------------------------------------
SOM_Scope	ODBoolean
SOMLINK		TextTransferExt__GetData
			(
				TextTransferExt*		somSelf,
				Environment*			ev,
				ODPlatformType		 	type,
				ODByteArray*			data
			)
{
    TextTransferExtMethodDebug("TextTransferExt","TextTransferExt__GetData");
		
	return ( TextTransferExt_parent_KSS_DataTransferExt_GetData(somSelf,ev,type,data) );
}

//------------------------------------------------------------------------------
// Method:		SetData
// Origin:		DataTransferExt
//
// Description:	Call the parent class to set the part's data to that specified.
//------------------------------------------------------------------------------
SOM_Scope	ODBoolean
SOMLINK		TextTransferExt__SetData
			(
				TextTransferExt*		somSelf,
				Environment*			ev,
				ODPlatformType			type,
				ODByteArray* 			data
			)
{
    TextTransferExtMethodDebug("TextTransferExt","TextTransferExt__SetData");

    return ( TextTransferExt_parent_KSS_DataTransferExt_SetData(somSelf,ev,type,data) );
}
