//
//	RadioGroup.cp
//
//	by Edward Voas, Apple Computer, Inc.
//
//	Copyright  1997 Edward Voas, All rights reserved.
//
//	Braindead radio group implementation.
//

#include "RadioGroup.h"

typedef struct
{
	ControlRef		control;
	SInt16			value;
} RadioEntry;

RadioGroup::RadioGroup()
{
	fInfo = NewHandle( 0 );
}

RadioGroup::~RadioGroup()
{
	DisposeHandle( fInfo );
}
		
void
RadioGroup::AddControl( ControlRef control, SInt16 value )
{
	Size		size;
	SInt16		numItems;
	
	size = GetHandleSize( fInfo );
	numItems = size / sizeof( RadioEntry );
	
	SetHandleSize( fInfo, size + sizeof( RadioEntry ) );
	
	((RadioEntry*)*fInfo)[ numItems ].value = value;
	((RadioEntry*)*fInfo)[ numItems ].control = control;
	
	SetValue( value ); 
}

SInt16
RadioGroup::GetValue()
{
	return fValue;
}

void
RadioGroup::SetValue( SInt16 value )
{
	SInt16		numItems;
	SInt16		i;
	ControlRef	oldControl = nil, newControl = nil;
	
	if ( value != fValue )
	{
		numItems = GetHandleSize( fInfo ) / sizeof( RadioEntry );
		
		for ( i = 0; i < numItems; i++ )
		{
			if ( ((RadioEntry*)*fInfo)[ i ].value == value )
				newControl = ((RadioEntry*)*fInfo)[ i ].control;
			
			if ( ((RadioEntry*)*fInfo)[ i ].value == fValue )
				oldControl = ((RadioEntry*)*fInfo)[ i ].control;
		}
		if ( newControl )
		{
			if ( oldControl )
				SetControlValue( oldControl, 0 );
		
			SetControlValue( newControl, 1 );
		}
		fValue = value;
	}
}

void
RadioGroup::SetValueByControl( ControlRef control )
{
	SInt16		numItems;
	SInt16		i;
	ControlRef	oldControl = nil, newControl = nil;
	SInt16		value;
	Boolean		valueFound = false;
		
	numItems = GetHandleSize( fInfo ) / sizeof( RadioEntry );
	
	for ( i = 0; i < numItems; i++ )
	{
		if ( ((RadioEntry*)*fInfo)[ i ].control == control )
		{
			value = ((RadioEntry*)*fInfo)[ i ].value;
			valueFound = true;
		}
		
		if ( ((RadioEntry*)*fInfo)[ i ].value == fValue )
			oldControl = ((RadioEntry*)*fInfo)[ i ].control;
	}
	if ( valueFound )
	{
		if ( oldControl )
			SetControlValue( oldControl, 0 );
	
		SetControlValue( control, 1 );
	
		fValue = value;
	}
}
