
#import "LunchObject.h"
#import "LunchView.h"
#import <remote/NXProxy.h>

@implementation LunchObject

- init
{
	[super init];
	where = [[NSMutableString alloc] init];
	return self;
}

- eat:sender
{
	[where release];
	where = [NSString stringWithCString:[(TextField *)lunchField stringValue]];
	[remoteHub updateEveryone];
	return self;
}

- textDidGetKeys:sender isEmpty:(BOOL)flag
{
	[lunchButton setEnabled:!flag];
	return self;
}

- appDidInit:sender
{
    remoteHub = [NXConnection connectToName:"Lunch" onHost:"*"];
    if (remoteHub)
		{
		//We will be a client
        ourServer 	= [remoteHub connectionForProxy];
		[remoteHub setProtocolForProxy:@protocol(LunchServerMethods)];
		isServer 	= NO;
	    } 
	else
		{
		//We are first to launch, so we are the server
		ourServer 	= [NXConnection registerRoot: self withName:"Lunch"];
		remoteHub 	= self;
		isServer 	= YES;
		clientList	= [[List alloc] init];
	    }

    [ourServer registerForInvalidationNotification:self];
    [ourServer runFromAppKit];
    [remoteHub addClient:self];
	
	[self updateEveryone];

    return self;
}

- appWillTerminate:sender
{
    [remoteHub removeClient:self];
    return self;
}

- senderIsInvalid:sender
{
	if ( sender != self )
		{
		//server died, we all have to go away
		remoteHub = nil;
		[NXApp terminate:self];
		}
	else if ( isServer )
		[self removeClient:sender];
	
	if ( isServer )
		[self updateEveryone];
	
    return self;
}

- updateEveryoneAfterDelay
{
	[clientList makeObjectsPerform:@selector(updateStats) with:nil];	
	return self;
}

- (void)updateEveryone
{
	[self perform:@selector(updateEveryoneAfterDelay) with:nil afterDelay:1 cancelPrevious:YES];
	return;
}

- (void)addClient:client;
{
	[clientList addObjectIfAbsent:client];
	[self updateEveryoneAfterDelay];
}

- (void)removeClient:client;
{
	[clientList removeObject:client];
	[self updateEveryoneAfterDelay];
}

- (List *)clientList
{
	return clientList;
}

- updateStatsAfterDelay
{
	List	*serverClientList;
	serverClientList = [remoteHub clientList];
	[lunchView updateLunchList:serverClientList];
	return self;
}

- (void)updateStats;
{
	[self perform:@selector(updateStatsAfterDelay) with:nil afterDelay:1 cancelPrevious:YES];
}

- (NSString *)wantToEatWhere;
{
	return where;
}

@end
