//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeap_AC__
#include "CTidyHeap_AC.h"
#endif

#if qTidyHeap

#ifndef __CTidyHeapAction_AC__
#include "CTidyHeapAction_AC.h"
#endif
#ifndef __CTidyHeapBlock_AC__
#include "CTidyHeapBlock_AC.h"
#endif

//========================================================================================
// class CTidyHeapBlock_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC constructor
//----------------------------------------------------------------------------------------

CTidyHeapBlock_AC::CTidyHeapBlock_AC() :
fId(0),
fSize(0),
fRefCon(NULL),
fAddress(NULL),
fDeleteAction(NULL),
fLineNum(0),
fHeader(NULL)
{
	*fFile = '\0';
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC destructor
//----------------------------------------------------------------------------------------

CTidyHeapBlock_AC::~CTidyHeapBlock_AC() 
{
	if(fDeleteAction) delete fDeleteAction;
}


//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::GetHeadOfList
//----------------------------------------------------------------------------------------

CTidyHeapBlock_AC* CTidyHeapBlock_AC::GetHeadOfList() 
{ 
	return CTidyHeap_AC::Instance()->fBlockList.GetHead(); 
} 

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::GetID
//----------------------------------------------------------------------------------------

long CTidyHeapBlock_AC::GetID() const 
{ 
	return fId; 
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::GetSize
//----------------------------------------------------------------------------------------

long CTidyHeapBlock_AC::GetSize() const 
{ 
	return fSize; 
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::GetFileName
//----------------------------------------------------------------------------------------

const char* CTidyHeapBlock_AC::GetFileName() const 
{ 
	return fFile; 
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::GetLineNum
//----------------------------------------------------------------------------------------

short CTidyHeapBlock_AC::GetLineNum() const 
{ 	
	return fLineNum; 
}	

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::GetFlags
//----------------------------------------------------------------------------------------

CTidyHeapBitFlags_AC& CTidyHeapBlock_AC::GetFlags() 
{ 	
	return fFlags; 
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::GetRefCon
//----------------------------------------------------------------------------------------

void* CTidyHeapBlock_AC::GetRefCon() const 
{ 
	return fRefCon; 
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::SetRefCon
//----------------------------------------------------------------------------------------

void CTidyHeapBlock_AC::SetRefCon(void* inRefCon)
{
	fRefCon = inRefCon; 
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::AquireDeleteAction
//----------------------------------------------------------------------------------------

void CTidyHeapBlock_AC::AquireDeleteAction( CTidyHeapAction_AC* action )
{
	if(fDeleteAction) 
		delete fDeleteAction;
	fDeleteAction = action;
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::GetDeleteAction
//----------------------------------------------------------------------------------------

CTidyHeapAction_AC* CTidyHeapBlock_AC::GetDeleteAction() const
{
	return fDeleteAction;
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::Verify
//----------------------------------------------------------------------------------------

CTidyHeapError_AC CTidyHeapBlock_AC::Verify()
{
	return CTidyHeap_AC::Instance()->VerifyBlock( fAddress );
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::GetMarker
//----------------------------------------------------------------------------------------

CTidyHeapMarker_AC CTidyHeapBlock_AC::GetMarker( short markerNum )
{
	return CTidyHeap_AC::Instance()->GetMarker( fAddress, markerNum );
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::LogBlockData
//----------------------------------------------------------------------------------------

void CTidyHeapBlock_AC::LogBlockData()
{
	char *fileName = fLineNum > 0 ? fFile : "unknown" ;
	short lineNum = fLineNum;

	CTidyHeap_AC::Instance()->Logf("### BlockInfo: ID:%ld, Address:0x%X, Size:%ld" , fId, fAddress, fSize );
	CTidyHeap_AC::Instance()->Logf("               File '%s'; Line %d",fileName, lineNum );
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::GetAddress
//----------------------------------------------------------------------------------------

void* CTidyHeapBlock_AC::GetAddress() const
{
	return fAddress;
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::IsExempt
//----------------------------------------------------------------------------------------

Boolean CTidyHeapBlock_AC::IsExempt() const
{
	return fFlags.TestFlag( kExempt );
}

//----------------------------------------------------------------------------------------
// CTidyHeapBlock_AC::Exempt
//----------------------------------------------------------------------------------------
	
void CTidyHeapBlock_AC::SetExempt(Boolean inBool)
{
	if(inBool && !IsExempt()) 
	{
		fFlags.SetFlag( kExempt, true );
		CTidyHeap_AC::Instance()->GetBlockList().Remove(this);
		// if not in list, we're ok too
	}
	else
	if(!inBool && IsExempt()) 
	{
		fFlags.SetFlag( kExempt, false );
	
		if(!InList(&CTidyHeap_AC::Instance()->GetBlockList()))
			CTidyHeap_AC::Instance()->GetBlockList().Append(this);
	}
}


#endif