//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeap_AC__
#include "CTidyHeap_AC.h"
#endif

#if qTidyHeap

// Toolbox

#ifndef __LOWMEM__
#include <LowMem.h>
#endif

// TidyHeap

#ifndef __CTidyHeapSettings_AC__
#include "CTidyHeapSettings_AC.h"
#endif
#ifndef __CTidyHeapFile_AC__
#include "CTidyHeapFile_AC.h"
#endif

//========================================================================================
// class CTidyHeapSettings_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC default constructor
//----------------------------------------------------------------------------------------

CTidyHeapSettings_AC::CTidyHeapSettings_AC()
{
	RevertToDefault();
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC copy constructor
//----------------------------------------------------------------------------------------

CTidyHeapSettings_AC::CTidyHeapSettings_AC(const CTidyHeapSettings_AC &inSettings)
{
	*this = inSettings;
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC destructor
//----------------------------------------------------------------------------------------

CTidyHeapSettings_AC::~CTidyHeapSettings_AC()
{
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC assignment operator
//----------------------------------------------------------------------------------------

CTidyHeapSettings_AC &CTidyHeapSettings_AC::operator = (const CTidyHeapSettings_AC &inSettings)
{
	fPrefs = inSettings.fPrefs;
	fActions = inSettings.fActions;
	
	pstrcpy( fSessionName,  inSettings.fSessionName);

	fNewFailCount = inSettings.fNewFailCount;
	return *this;
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::RevertToDefault
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::RevertToDefault()
{
	SetSessionNameToDefault();
	
	fPrefs.SetAllFlags(0xFFFFFFFF);
		// enable all feature prefs
		
	fActions.SetFlag(kDeleteVerifyAction, true);
	fActions.SetFlag(kGarbageIn, true);
	fActions.SetFlag(kGarbageOut, true);
		// default installed actions
		
	fNewFailCount = 0;
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::Read
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::Read( CTidyHeapFile_AC *inFile )
{
	inFile->ReadObject( fPrefs );

	if(!inFile->GetErr())
		inFile->ReadObject( fActions );
	
	if(!inFile->GetErr())
		inFile->Read((char*) fSessionName, sizeof(Str32) );

	if(!inFile->GetErr())
		inFile->Read((char*) &fNewFailCount, sizeof(long) );

	if(inFile->GetErr()) 
	{
		RevertToDefault();
	}
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::Write
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::Write( CTidyHeapFile_AC *inFile )
{
	inFile->WriteObject( fPrefs );

	if(!inFile->GetErr())
		inFile->WriteObject( fActions );
	
	if(!inFile->GetErr())
		inFile->Write((char*) fSessionName, sizeof(Str32) );

	if(!inFile->GetErr())
		inFile->Write((char*) &fNewFailCount, sizeof(long) );
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::GetWantsWarnings
//----------------------------------------------------------------------------------------

Boolean CTidyHeapSettings_AC::GetWantsWarnings() const 
{ 
	return fPrefs.TestFlag(kWarnings); 
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::SetWantsWarnings
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::SetWantsWarnings(Boolean isEnabled) 
{ 
	fPrefs.SetFlag(kWarnings, isEnabled); 
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::IsEnabled
//----------------------------------------------------------------------------------------

Boolean CTidyHeapSettings_AC::IsEnabled() const 
{ 
	return fPrefs.TestFlag(kEnabled); 
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::SetIsEnabled
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::SetIsEnabled(Boolean isEnabled) 
{ 
	fPrefs.SetFlag(kEnabled, isEnabled); 
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::SetSessionName
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::SetSessionName(const Str32 &inStr)
{
	pstrcpy( fSessionName, inStr );
}

//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC::SetSessionNameToDefault
//----------------------------------------------------------------------------------------

void CTidyHeapSettings_AC::SetSessionNameToDefault()
{
	SetSessionName(::LMGetCurApName());
}

#endif