//----------------------------------------------------------------------------------------
// CTidyHeapActions_AC.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeapActions_AC__
#define __CTidyHeapActions_AC__

#ifndef __CTidyHeap_AC__
#include "CTidyHeap_AC.h"
#endif

#ifndef __CTidyHeapAction_AC__
#include "CTidyHeapAction_AC.h"
#endif


//========================================================================================
// 	class CActionBreakOnDelete_AC
//
//	This action is meant to be a CTidyHeapBlock_AC's delete action. This basically calls
// 	DebugStr and logs the break to Logf. You can see when a specific block is being 
// 	deleted this way.
//========================================================================================

class CTidyHeapBlock_AC;
class CActionBreakOnDelete_AC: public CTidyHeapAction_AC
{
public:
	CActionBreakOnDelete_AC(const char* outStr);

	CActionBreakOnDelete_AC() { fStr[0] = '\0'; }

	virtual ~CActionBreakOnDelete_AC() {}
	
	virtual void DoAction(CTidyHeapBlock_AC*);

private:
	char fStr[256];
};

//========================================================================================
// 	class CMakeNewFail
//
// 	This action will cause new to fail when new is called x times. Send the count you
// 	want to fail into the constructor.
//
// 	This needs to be a global new action.
//========================================================================================

class CActionMakeNewFail_AC: public CTidyHeapAction_AC
{
public:
	CActionMakeNewFail_AC(long whenToFail);

	virtual ~CActionMakeNewFail_AC() {}
	
	virtual void DoAction(CTidyHeapBlock_AC*);
	
	virtual void SetFailNum(long inCount) { fFailNum = inCount; }

	long GetFailNum() { return fFailNum; }
	
	virtual void SetNewTally(long inCount) { fNewTally = inCount; }

	long GetNewTally() const { return fNewTally; }
	
protected:
	long 		fNewTally;
	long 		fFailNum;
};

//========================================================================================
// 	class CActionDoubleDeleteWatcher_AC
//
// 	This action causes all affected blocks to never be deleted. 
// 	When the user calls delete CWhateverClass the block is checked for its delete flag, 
// 	if it has already been deleted, and error will be logged. The block will still not
// 	be deleted.
// 
//	This can either be a global new or delete action, or it can be attached to a specific
//  block as a delete action.
//
//========================================================================================

class CActionDoubleDeleteWatcher_AC: public CTidyHeapAction_AC
{
public:
	CActionDoubleDeleteWatcher_AC() {}

	virtual ~CActionDoubleDeleteWatcher_AC() {}
	
	virtual void DoAction(CTidyHeapBlock_AC*);
};

//========================================================================================
// class CActionVerifyAll_AC
//
// This action causes *ALL* blocks to be verified when its DoAction method is called..
//
// This can be either a new or a delete action.
//
//========================================================================================

class CActionVerifyAll_AC: public CTidyHeapAction_AC
{
public:
	CActionVerifyAll_AC(Boolean logToFile = true) : fLog(logToFile) {}

	virtual ~CActionVerifyAll_AC() {}
	
	virtual void DoAction(CTidyHeapBlock_AC*);
	
	Boolean DoLog() const { return fLog; }
	
	void DoLog(Boolean inLog) { fLog = inLog; }
	
private:
	Boolean fLog;
};

//========================================================================================
// class CActionVerify_AC
//
// This action causes *ALL* blocks to be verified when its DoAction method is called..
//
// This can be either a new or a delete action.
//
//========================================================================================

class CActionVerifyBlock_AC: public CTidyHeapAction_AC
{
public:
	CActionVerifyBlock_AC(Boolean logToFile = true) : fLog(logToFile) {}

	virtual ~CActionVerifyBlock_AC() {}
	
	virtual void DoAction(CTidyHeapBlock_AC*);

	Boolean DoLog() const { return fLog; }
	
	void DoLog(Boolean inLog) { fLog = inLog; }

private:
	Boolean fLog;
};

//========================================================================================
// class CActionExemptBlocks_AC
//
// This action causes *ALL* blocks to not be tracked while it is installed.
//
// This needs to be a global new action
//
//========================================================================================

class CActionExemptBlocks_AC: public CTidyHeapAction_AC
{
public:
	CActionExemptBlocks_AC(long inStopper = 0) : fStopper(inStopper) {}
		// if inStopper > 0 this class will only except blocks of
		// id <= fStopper.
	
	virtual ~CActionExemptBlocks_AC() {}
	
	virtual void DoAction(CTidyHeapBlock_AC*);

private:
	long fStopper;
};


//========================================================================================
// class CActionBreakOnNew_AC
//
// This action causes a call to DebugStr when new is called for a block with the preset
// Id.
//
// This needs to be a global new action
//
//========================================================================================

class CActionBreakOnNew_AC: public CTidyHeapAction_AC
{
public:
	CActionBreakOnNew_AC(long inID) : fID(inID) {}

	virtual ~CActionBreakOnNew_AC() {}
	
	virtual void DoAction(CTidyHeapBlock_AC*);
	
private:
	long fID;
};


#endif