//----------------------------------------------------------------------------------------
// CTidyHeapFile_AC.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeapFile_AC__
#define __CTidyHeapFile_AC__

// Toolbox

#ifndef __FILES__
#include <Files.h>
#endif

// TidyHeap

#ifndef __MTidyHeapAllocator_AC__
#include "MTidyHeapAllocator_AC.h"
#endif

//========================================================================================
// class CTidyHeapFile_AC
//========================================================================================

class MTidyHeapPersistantObject_AC;
class CTidyHeapFile_AC : public MTidyHeapAllocator_AC
{
public:
	
	// constructors/destructors
	
	CTidyHeapFile_AC();
	
	CTidyHeapFile_AC(const CTidyHeapFile_AC &);
	
	CTidyHeapFile_AC(const FSSpec &);
	
	virtual ~CTidyHeapFile_AC();

	// Assignment operator
	
	CTidyHeapFile_AC &operator = (const CTidyHeapFile_AC &);

	// Opening and Closing
	
	enum ePerms
	{
		kRead,
		kWrite,
		kAppend
	};
	
	void Open(	ePerms inPerms = kRead );
	
	void Close();

	Boolean IsOpen() const { return fRef != 0; }
	
	// Reading and Writing
	
	long Read( char *, long size);
	
	long Write( const char *, long size);

	void ReadObject(MTidyHeapPersistantObject_AC &);
	
	void WriteObject(MTidyHeapPersistantObject_AC &);

	// Specifying

	void SetSpec( const FSSpec & );
	
	void SetName( const unsigned char* );

	void AppendExtension(const char*);	
	
	void SetVolRefNum( short );
	
	void SetDirID( long );
	
	const FSSpec &GetSpec() const { return fSpec; }
	
	void SetTypeAndCreator( OSType inType, OSType inCreator );
	
	// Misc Info "getters"
	
	long GetSize();
	
	OSErr GetErr() const { return fErr; }
	
protected:

	static void CopySpec( FSSpec *dest, const FSSpec *src ) 
		{ ::BlockMove( src, dest, sizeof(FSSpec)); }
	
protected:
	FSSpec 	fSpec;
	short 	fRef;
	OSErr 	fErr;
	OSType 	fType;
	OSType 	fCreator;
};

#endif
