//----------------------------------------------------------------------------------------
// CTidyHeapSettings_AC.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeapSettings_AC__
#define __CTidyHeapSettings_AC__

#ifndef __CTidyHeapPersistantObject_AC__
#include "CTidyHeapPersistantObject_AC.h"
#endif

#ifndef __CTidyHeap_ACOBJECT__
#include "MTidyHeapAllocator_AC.h"
#endif

#ifndef	__CTidyHeapBitFlags_AC__
#include "CTidyHeapBitFlags_AC.h"
#endif

//----------------------------------------------------------------------------------------
// class CTidyHeapSettings_AC
//----------------------------------------------------------------------------------------

class CTidyHeapSettings_AC : public MTidyHeapAllocator_AC, public MTidyHeapPersistantObject_AC
{
public:

	enum eFeaturePrefs
	{
		kEnabled,
		kWarnings,
		kUndeletedBlockWarning,
		kLogUndeletedBlocks,
		kPointersLivesInAppHeap,
		kNumPrefs
	};
	enum eActionPrefs
	{
		kDoubleDeleteWatcher, 		// Global Delete Action
		kMakeNewFail,				// Global New Action
		kIncrementalFailer,			// Global New Action
		kNewVerifyAction,			// Global New Action
		kDeleteVerifyAction,   	 	// Global Delete Action
		kNewVerifyAllAction,		// Global New Action
		kDeleteVerifyAllAction,		// Global Delete Action
		kExemptUnknownBlocks,		// Global New Action
		kGarbageIn,					// Global New Action
		kGarbageOut,				// Global Delete Action
		kNewBreakOnUnknownBlock,	// Global New Action
		kDeleteBreakOnUnknownBlock, // Global Delete Action
		kNumActions			
	};	

public:

	CTidyHeapSettings_AC();

	CTidyHeapSettings_AC(const CTidyHeapSettings_AC &);

	virtual ~CTidyHeapSettings_AC();

	CTidyHeapSettings_AC &operator = (const CTidyHeapSettings_AC &);
	
	virtual void RevertToDefault();
	
	virtual CTidyHeapBitFlags_AC &GetFeaturePrefs();

	virtual CTidyHeapBitFlags_AC &GetActionPrefs();
		
	virtual const Str32& GetSessionName() const;
	
	virtual void SetSessionName(const Str32 &);
	
	virtual void SetNewFailCount( long inCount );
		// this value is only used for the NewFailerAction	

	virtual long GetNewFailCount() const;
	
	virtual Boolean GetWantsWarnings() const;

	virtual void SetWantsWarnings(Boolean isEnabled);
	
	virtual Boolean IsEnabled() const;

	virtual void SetIsEnabled(Boolean isEnabled);
		
protected:

	virtual void SetSessionNameToDefault();		
		
private:
	
	virtual void Read( CTidyHeapFile_AC * );

	virtual void Write( CTidyHeapFile_AC * );
	
protected:

	CTidyHeapBitFlags_AC 	fPrefs;
	CTidyHeapBitFlags_AC 	fActions;
	Str32 				fSessionName;	
	long 				fNewFailCount;

};

//----------------------------------------------------------------------------------------
// Inlines
//----------------------------------------------------------------------------------------

inline CTidyHeapBitFlags_AC &CTidyHeapSettings_AC::GetFeaturePrefs() { return fPrefs; }
inline CTidyHeapBitFlags_AC &CTidyHeapSettings_AC::GetActionPrefs() { return fActions; }
inline const Str32 &  CTidyHeapSettings_AC::GetSessionName() const { return fSessionName; }
inline void CTidyHeapSettings_AC::SetNewFailCount( long inCount ) { fNewFailCount = inCount; }
inline long CTidyHeapSettings_AC::GetNewFailCount() const { return fNewFailCount; }


#endif
