//----------------------------------------------------------------------------------------
// CTidyHeap_AC.h
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeap_AC__
#define __CTidyHeap_AC__

#ifndef __TidyHeapSwitches_AC__
#include "TidyHeapSwitches_AC.h"
#endif

#if qTidyHeap

//----------------------------------------------------------------------------------------
// includes
//----------------------------------------------------------------------------------------

#ifndef __CTidyHeapActionList_AC__
#include "CTidyHeapActionList_AC.h"
#endif
#ifndef __CTidyHeapBlock_AC__
#include "CTidyHeapBlock_AC.h"
#endif
#ifndef __CTidyHeapBlockList_AC__
#include "CTidyHeapBlockList_AC.h"
#endif
#ifndef __CTidyHeapSettings_AC__
#include "CTidyHeapSettings_AC.h"
#endif
#ifndef __TidyHeapTypes_AC__
#include "TidyHeapTypes_AC.h"
#endif

//----------------------------------------------------------------------------------------
// prototypes
//----------------------------------------------------------------------------------------

void* operator new(size_t size, const char* file, short lineNum); 
void* operator new(size_t size, void* ptr, const char* file, short lineNum); 

//----------------------------------------------------------------------------------------
// forward refs
//----------------------------------------------------------------------------------------

struct BlockHeader;
struct BlockTrailer;
class CTidyHeapAction_AC;
class CTidyHeapBlock_AC;
class CTidyHeapState_AC;
class CTidyHeapLogger_AC;
class CTidyHeapFile_AC;

//----------------------------------------------------------------------------------------
// class CTidyHeap_AC
//----------------------------------------------------------------------------------------

class CTidyHeap_AC : public CTidyHeapSettings_AC
{
	
public:

//----------------------------------------------------------------------------------------
// Constructor/Destructor
// Note: These are called from CreateTidyHeap and from Initialize, so don't call them
// yourself.
//----------------------------------------------------------------------------------------
	
	CTidyHeap_AC();
	
	virtual ~CTidyHeap_AC();

	virtual void ITidyHeap();

//----------------------------------------------------------------------------------------
// Enumerations
//----------------------------------------------------------------------------------------

	enum eMarkers
	{
		kInvalidMarker = 	0xF0F0F0F0,
		kMarker1 = 			0xF1F1F1F1,
		kMarker2 = 			0xF2F2F2F2,
		kMarker3 = 			0xF3F3F3F3,
		kStateData = 		0xF5F5F5F5
	};
	enum eErrors
	{
		kNoErr,
		kBadMarker1,
		kBadMarker2,
		kBadMarker3,
		kNoMarkersFound,
		kPtrCrossRefFailed,
		kPtrNotInAppHeap
	};
	enum
	{
		kSettingsFileType = 'thpf',
		kSettingsFileCreator = 'thap'
	};

//----------------------------------------------------------------------------------------
// Info
//----------------------------------------------------------------------------------------

	static CTidyHeap_AC* Instance();

	virtual CTidyHeapError_AC GetErr();
		// new/delete/verify etc. can cause an error.

	virtual CTidyHeapBlock_AC* GetBlock(void*);
		// This is how you get at your block class for whatever
		// data you've allocated.
		
//-- these return numbers for currently allocated blocks
	
	virtual long GetBlockCount() const;
	
//--  these are tallys for the lifetime of your app.
	
	virtual long GetBlockCountTally() const;
	
	virtual long GetSizeTally() const;
	
	virtual long GetDebugSizeTally() const;

	virtual float GetAveSize() const;

//----------------------------------------------------------------------------------------
// Prefs
//----------------------------------------------------------------------------------------

	virtual Boolean IsEnabled() const;
	
	virtual void SetIsEnabled(Boolean isEnabled);

//----------------------------------------------------------------------------------------
// Utils 
//----------------------------------------------------------------------------------------

	virtual void VerifyAllBlocks(Boolean logBlockData = false);

	static void Terminate();
		// Usually this is called by the shared lib and you don't need to
		// call it, but it's here if you want it. Use at your own risk. 

	virtual CTidyHeapError_AC VerifyBlock(void*, Boolean logBlockData = false);
		// this is here as a convienience, you can also call verify
		// on the specific block you're dealing with by calling
		// GetBlock first, then calling CTidyHeapBlock_AC::Verify.
		
	virtual void MakeNextNewFail();	
		// This does what it says. ;-)
		
//----------------------------------------------------------------------------------------
// Global Actions
//----------------------------------------------------------------------------------------
	
	//-- Actions installed here are called for every block.
	// Note: Actions you install into TidyHeap are deleted by TidyHeap at Termination
	// Time.
	
	virtual void AquireNewAction( CTidyHeapAction_AC*  );
		// A New action is called after allocation immediately before
		// new returns. 
	
	virtual void RemoveNewAction( CTidyHeapAction_AC* );
		// Removes and does not delete.
	
	virtual void AquireDeleteAction( CTidyHeapAction_AC*);
		// A Delete action is called immediately before deletion.
	
	virtual void RemoveDeleteAction( CTidyHeapAction_AC* );
		// Removes and does not delete.
			
	virtual void AquireTerminationAction( CTidyHeapAction_AC*  );
		// A Termination action is called when TidyHeap terminates, after
		// all the logging is completed, just before TidyHeap is deleted.
		// A termination Action doesn't interact with any blocks.
		
	virtual void RemoveTerminationAction( CTidyHeapAction_AC* );
		// Removes and does not delete.	
		
	virtual CTidyHeapActionList_AC& GetTerminationActionList();

	virtual CTidyHeapActionList_AC& GetNewActionsList();

	virtual CTidyHeapActionList_AC& GetDeleteActionList();
	
	virtual void RemoveAllActions();
		// removes all actions from global new and delete lists.
		
//----------------------------------------------------------------------------------------
// Logging
//----------------------------------------------------------------------------------------

	virtual void Logf(const char* format, ...);
		// Logf data strings go to the LogHook, whatever that may be.
		
	virtual void Warnf(const char* format, ...);
		// Warnf goes to DebugStr, only if warnings are enabled.
		
	virtual CTidyHeapLogger_AC *AquireLogger( CTidyHeapLogger_AC *, Boolean deletePrevious = true );
		// Sending in nil will essentially turn off logging.
		
	virtual CTidyHeapLogger_AC *GetLogger();
		
	//-- Available Log hooks: 
	// 1. LogFile
	// 2. DebugStr
	
	// Use these interfaces to install them.
		
	virtual void SetToDebugStrLogger( Boolean doHalt = false );
		// if do halt is false, ";g" will be appended to the logf string,
		// cause macsbug to immediately return and continue processing.	
	
	virtual void SetToLogFileLogger();	
		// Sets logfile hook.
	
	//-- Misc Logging helpers

	virtual void LogError();
		// This will log the current error with a string description.
		
		
//----------------------------------------------------------------------------------------
// MISC
//----------------------------------------------------------------------------------------
	
	void PushInfo(const char* fileName, short lineNum);
		//** probably don't want to call this. Use SaveBlockInfo macro instead.
		// I'd make this private if I could.
		
	virtual void GetSessionFileLoc(short& vol, long &dirId);
		
	virtual void HandleNilAllocation();
	
	virtual CTidyHeapBlock_AC *CreateBlockObject();

	CTidyHeapBlockList_AC& GetBlockList();
	
	static void ExemptBlockFromLeakCheck(void* inBlock);

public:
	
	static GlobalNewProc GlobalNew;
	static GlobalDeleteProc GlobalDelete;

protected:
	virtual void HandleTermination();
			
private:

//----------------------------------------------------------------------------------------
// private methods
//----------------------------------------------------------------------------------------

	void* CreateBlock(size_t size, const char* file, short line);
	
	void RemoveBlock(void* ptr);
	
	BlockHeader* FindHeader(char* ptr);
	
	BlockTrailer* FindTrailer(BlockHeader*);
	
	long VerifyState( CTidyHeapState_AC & );
	
	void DeleteActions( CTidyHeapActionList_AC & );
	
	void DoAction( CTidyHeapAction_AC*, BlockHeader*);
	
	static CTidyHeapBlock_AC* DefaultCreateBlockObject();

	CTidyHeapMarker_AC GetMarker(void*, short markerNum );

	void InstallPrefActions();
	
	Boolean PtrInAppHeap(void*);
		
private:

//----------------------------------------------------------------------------------------
// data members
//----------------------------------------------------------------------------------------

	long 						fCount;
	long 						fDeleteCount;
	long 						fSizeTally;
	long	 					fDebugTally;
	
	CTidyHeapLogger_AC* 			fLogger;
	char						fLogString[256];
	
	char 						fFileName[32];
	short 						fLineNum;
	
	CTidyHeapBlockList_AC 			fBlockList;
	CTidyHeapActionList_AC 		fTerminationActions;
	CTidyHeapActionList_AC 		fNewActions;
	CTidyHeapActionList_AC 		fDeleteActions;

	CTidyHeapError_AC 				fErr;

	Boolean 					fNextNewFails;
	Boolean						fEnabledOnStart;
	
	CTidyHeapFile_AC*				fSettingsFile;
	
	static CTidyHeap_AC*			fgSelf;
	
//----------------------------------------------------------------------------------------
// friends o' tidy heap.
//----------------------------------------------------------------------------------------

	friend void* operator new(size_t size, const char* fileName, short lineNum);
	friend void* operator new(size_t size);
	friend void operator delete(void* ptr );
	friend class CTidyHeapState_AC;
	friend class CTidyHeapBlock_AC;
		
};	
//----------------------------------------------------------------------------------------
// CTidyHeap_AC's Inlines
//----------------------------------------------------------------------------------------

inline long CTidyHeap_AC::GetBlockCountTally() const 
{ 
	return fCount; 
}
inline long CTidyHeap_AC::GetSizeTally() const 
{ 
	return fSizeTally; 
}
inline long CTidyHeap_AC::GetDebugSizeTally() const 
{ 
	return fDebugTally; 
}
inline float CTidyHeap_AC::GetAveSize() const 
{ 
	return (fSizeTally / fCount) + 0.5; 
}
inline CTidyHeapLogger_AC* CTidyHeap_AC::GetLogger() 
{ 
	return fLogger;
}

//--  these are optional macros.
#define TH_new new(__FILE__, __LINE__)
	// Use this new in place of normal new. The benefit here is that the file name
	// and line number is saved into the block of memory that you're creating. So
	// if there's a problem with it, you'll know exactly where the block was created.
	// Sorry about the ugly macro.

#define TH_SaveBlockInfo() CTidyHeap_AC::Instance()->PushInfo(__FILE__, __LINE__)
	// This is an alternate to using the TH_new macro. If you call this immediately before
	// a new, it will also save the pertinant creation info.

	// You are required to define these in your environment.
extern CTidyHeap_AC* CreateTidyHeap();

#define gTidyHeap CTidyHeap_AC::Instance()
	// for compatibility, will be removed sometime in the future
		
		
#else

inline void TH_Stub() {}

#define TH_new new
#define TH_SaveBlockInfo() TH_Stub()

#endif

#endif