//----------------------------------------------------------------------------------------
// CTidyHeapActions_AC.cp
// Copyright  1984-97 by Apple Computer, Inc. All rights reserved.
//----------------------------------------------------------------------------------------

// tidyheap

#ifndef __CTidyHeap_AC__
#include "CTidyHeap_AC.h"
#endif

#if qTidyHeap

#ifndef __CTidyHeapActions_AC__
#include "CTidyHeapActions_AC.h"
#endif
#ifndef __CTidyHeapBlock_AC__
#include "CTidyHeapBlock_AC.h"
#endif

#ifndef _STDIO
#include <stdio.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

// Ansi

#ifndef _STRING
#include <string.h>
#endif

//========================================================================================
// class CActionBreakOnDelete_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CActionBreakOnDelete_AC::DoAction
//----------------------------------------------------------------------------------------

CActionBreakOnDelete_AC::CActionBreakOnDelete_AC(const char* outStr)
{
	strcpy(fStr, outStr);
}

//----------------------------------------------------------------------------------------
// CActionBreakOnDelete_AC::DoAction
//----------------------------------------------------------------------------------------

void CActionBreakOnDelete_AC::DoAction(CTidyHeapBlock_AC* inBlock)
{
	// this needs some work.
	CTidyHeap_AC::Instance()->Warnf("deleting Block #%ld at 0x%X", 
		inBlock->GetID(),
		inBlock->GetAddress());
	
	CTidyHeap_AC::Instance()->Logf("### Deleting the following block:");
	inBlock->LogBlockData();
}

//========================================================================================
// class CActionMakeNewFail_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// CActionMakeNewFail_AC constructor
//----------------------------------------------------------------------------------------

CActionMakeNewFail_AC::CActionMakeNewFail_AC(long whenToFail): 
fNewTally(0), 
fFailNum(whenToFail)
{ 
}

//----------------------------------------------------------------------------------------
// CActionMakeNewFail_AC::DoAction
//----------------------------------------------------------------------------------------

void CActionMakeNewFail_AC::DoAction(CTidyHeapBlock_AC*)
{
	if(++fNewTally == fFailNum)
	{
		CTidyHeap_AC::Instance()->Warnf("Forcing new %ld to fail.", fNewTally);
		CTidyHeap_AC::Instance()->MakeNextNewFail();
	}
}


//========================================================================================
// class CActionDoubleDeleteWatcher_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// 	CActionDoubleDeleteWatcher_AC::DoAction
//----------------------------------------------------------------------------------------

void CActionDoubleDeleteWatcher_AC::DoAction(CTidyHeapBlock_AC* inBlock)
{
	inBlock->GetFlags().SetFlag(CTidyHeapBlock_AC::kDontDelete, true);
}

//========================================================================================
// class CActionVerifyAll_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// 	CActionVerifyAll_AC::DoAction
//----------------------------------------------------------------------------------------

void CActionVerifyAll_AC::DoAction(CTidyHeapBlock_AC* /*inBlock*/)
{
	CTidyHeapLogger_AC* oldPtr = NULL;
	
	if(!fLog)
	{
		oldPtr = CTidyHeap_AC::Instance()->AquireLogger(NULL, false);
	}
	
	CTidyHeap_AC::Instance()->VerifyAllBlocks();
	
	if(!fLog)
	{
		CTidyHeap_AC::Instance()->AquireLogger( oldPtr, false);
	}
}

//========================================================================================
// class CActionVerifyBlock_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// 	CActionVerifyBlock_AC::DoAction
//----------------------------------------------------------------------------------------

void CActionVerifyBlock_AC::DoAction(CTidyHeapBlock_AC* inBlock)
{
	CTidyHeapLogger_AC* oldPtr = NULL;
	
	if(!fLog)
	{
		oldPtr = CTidyHeap_AC::Instance()->AquireLogger(NULL, false);
	}
	
	inBlock->Verify();
	
	if(!fLog)
	{
		CTidyHeap_AC::Instance()->AquireLogger( oldPtr, false);
	}
}


//========================================================================================
// class CActionExemptBlocks_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// 	CActionExemptBlocks_AC::DoAction
//----------------------------------------------------------------------------------------

void CActionExemptBlocks_AC::DoAction(CTidyHeapBlock_AC* inBlock)
{
	if(!fStopper || fStopper >= inBlock->GetID())
		inBlock->SetExempt(true); 
}


//========================================================================================
// class CActionBreakOnNew_AC
//========================================================================================

//----------------------------------------------------------------------------------------
// 	CActionBreakOnNew_AC::DoAction
//----------------------------------------------------------------------------------------

void CActionBreakOnNew_AC::DoAction(CTidyHeapBlock_AC* inBlock)
{
	if(inBlock->GetID() == fID)
	{
		char str[80];
		::sprintf(str, "CActionBreakOnNew_AC: Block ID: %ld", fID);
		::c2pstr(str);
		
		::DebugStr((StringPtr) str);
	}
}





#endif