/* Dynamic.c  Functions for dynamic k-approximate virus infection detection.
	Copyright  1992 by William H. Hsu.
	Thanks to John Norstad and Ephraim Vishniac for help and comments.  Portions of this code are based on [Morton 90], which appears in the May 1990 issue of MacTutor.  Reused with permission.  You may copy, alter, use, and distribute all code listed here if you leave the file unchanged up to this line.
	Think C version.

	Notes:
		A main advantage of this code, as explained in the Methods and History section, is that its effectiveness is not diminished by its availability.  No matter how many potential virus authors read it, the algorithm will remain equally effective against circumvention.
		To use this code in your programs:
	 it will be necessary to obtain non-functional
but significant (larger than 300 bytes) resource segments from the virus you are trying to detect
	- using a resource editor, insert the viral data under an unused type, such as 'VDAT', used in the code below
	- this will render the virus code inactive and most likely invisible to conventional (Class 2 and 3) detection programs; as and added security measure, you may wish to include only code segments above 300 bytes (or a similar threshold length) to ensure that the virus is crippled
	- both these C routines and the Boyer-Moore routines require an expanded 512K Mac or later (specifically, System 3.2 or later); they have been fully tested on the SE, II, and IIcx		
	-	this function should be run upon first launching your application, or, if it is an operating system utility, during a dormant or idle period
	- the code below assumes that the VDAT resource contains all 5 segments of nVIR A; change this accordingly by adding additional virus types (under a name other than the original infected type) */

#include "dec.h"

FILE *my_file;
void dynamic()
{
  char m[MAXSIZE];
  int pattern_length, index;
  MATRIX table;
  register Handle rsrc;
  short resCount;
  ToolBoxInit();
  CurResFile();
  resCount = Count1Resources ('VDAT');	
	/* how many of this type are there? */
  open_file (&my_file, WRITE_MODE);
#ifdef _REPORT	/* developer debug flag */
  printf("Searching for <<virus name>>:\n\n");
  fprintf(my_file, "Searching for <<virus 			name>>:\n\n");
#endif
  while (resCount) /* loop down to 1 */
  {
		if (resCount == 3)
		{
  	  printf("\nSearching for <<virus 					name>>:\n\n");
  	  fprintf(my_file, "\nSearching for <<virus 				name>>:\n\n");
		}	
   rsrc = Get1IndResource ('VDAT', resCount);    
	/* get the resource's handle, but don't load 			it */
	index = SizeResource (rsrc);
	HLock (rsrc);
	/* load next virus segment */
	pattern_length = copy_array (*rsrc, m, 			&index);
#ifdef _REPORT
  printf("Next virus segment loaded (length 		%d).  Resources left to scan: %d\n", 			pattern_length, resCount);
	fprintf(my_file, "Next virus segment loaded 		(length %d).  Resources left to scan: 			%d\n", pattern_length, resCount);
#endif
	HUnlock (rsrc);
	initialize(&table, pattern_length+1);
	vResCheck('nVRB', m, pattern_length, table, NO_REPORT);
#	ifdef _REPORT
			printf("\n");
			fprintf(my_file, "\n");
#	endif
	vResCheck('nVRA', m, pattern_length, table, 		NO_REPORT);
#	ifdef _REPORT
			printf("\n\n");
			fprintf(my_file, "\n\n");
#	endif
    --resCount;
  }   
  fclose(my_file);
}

void initialize(table, length)
MATRIX *table;
int length;
{
  allocate_table(table, length);
  clear_table(*table, length);
}

void allocate_table(table, size)
MATRIX *table;
int size;
{
  int i;
  *table = (MATRIX)calloc(size, (size_t)sizeof(long *));
  for (i = 0; i < size; i++)
    (*table)[i] = (long *)calloc(2, 						(size_t)sizeof(long));
}

void clear_table(table, length)
MATRIX table;
int length;
{
  int i;
  for (i = 0; i <= length; i++)
    table[i][0] = (long)UNIT*i;
}

/* vResCheck - Perform dynamic string search on all resources of a specified type in the current application. */
void vResCheck (type, m, pattern_length, table, report)
	register ResType type;	/* INPUT: type of 		resource to sum */
	char m[MAXSIZE];
	int pattern_length;
	MATRIX table;
	register short report;
/* INPUT: >0 => report errors with debugger */
	{
		register short resCount;	
			/* number of resources of this type */
		register Handle rsrc;	/* resource to check */
		register short oldResFile;
			/* for preserving current resource file */
		register Boolean oldResLoad;
			/* for preserving "ResLoad" flag */
		Boolean found;
		char n[MAXSIZE];
		int text_length, local_count = 1;
		int index;
			
	/* Switch to the application's resource file.  Note that all resource calls from here on are the "one deep" calls from Inside Mac, Volume
IV. */
	oldResFile = CurResFile();
	/* remember initial resource file */
	oldResLoad = ResLoad;	/* remember "ResLoad" state */
	resCount = Count1Resources (type);	
		/* how many of this type are there? */
  if (report)
  {
		fprintf(my_file, "Text string ");
		printf("Text string ");
  }
	while (resCount)	/* loop down to 1 */
	{	/* get the resource's handle, but don't load it */
		rsrc = Get1IndResource (type, resCount);
			/* see if it's already in memory */
		if (!rsrc)	/* not available? */
		{
			if (report > 0)		/* debugging flag */
				DebugStr ("\pResource not available!");
			goto EXIT;
		}
		else
		{
			index = SizeResource (rsrc);
			HLock (rsrc);
			found = FALSE;
			while ((text_length = copy_array(*rsrc, n, 				&index)) && (!found))
			{
				fprintf (my_file, "%d ", local_count);
				printf ("%d ", local_count);
				local_count++;
				clear_table (table, pattern_length);
				if (pattern_length <= text_length)
				{
					if (compare (m, n, pattern_length, 						text_length, table))
					  found = TRUE;
				}
			}
			HUnlock (rsrc);
		}
		--resCount;	/* get next index number */
	}	/* end of loop through resources */
	
	EXIT: /* goto here on tampering or error */
		UseResFile (oldResFile);	/* restore original resource file */
		SetResLoad (oldResLoad);	/* restore original loading state */
	}	/* end of vResCheck() */

/* compare: the actual dynamic programming algorithm, modified to a level of padding tolerance defined by THRESHOLD */
char compare(p, t, pattern_length, text_length, table)
char p[], t[];
int pattern_length, text_length;
MATRIX table;
{ 
  long value1, value2, value3;
  int i, j, flip, beep;
  flip = TRUE;
  for (j = 1; j <= text_length; j++)
  {
    table[0][flip] = 0;
    for (i = 1; i <= pattern_length; i++)
    {
			if (p[i-1] == t[j-1]) /* initialize */
        value1 = table[i-1][!flip];
      else
        value1 = (table[i-1][!flip])+UNIT;
      value2 = (table[i-1][flip])+UNIT;
				/* UNIT: the orginal algorithm uses this 				weight for all variations in the text */
      value3 = (table[i][!flip])+EPSILON;
				/* EPSILON: small weighted "distance" --				as opposed to the single unit */
      table[i][flip] = MIN3(value1, value2, 				value3);	/* see discussion of dynamic */
    }
    if (table[pattern_length][flip] <= 					THRESHOLD)
    {
			if (report)
			{
      printf("%ld-approximate match found.\n", 					table[pattern_length][flip]);
      fprintf(my_file, "%ld-approximate match 					found.\n", table[pattern_length][flip]);
			}
			return (TRUE);
    }
    flip = !(flip);		/* only an O(2m)-sized 			array is needed to simulate a "matrix", 			because only 2 columns are used */
  }
  return (FALSE);
}

